// swad_text.c: text messages on screen, in several languages

/*
    SWAD (Shared Workspace At a Distance in Spanish),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.

    This file is part of SWAD core.
    Copyright (C) 1999-2024 Antonio Caas Vargas

    Catal translation:
	Antonio Caas Vargas
	Joan Llus Daz Rodrguez
    German translation:
	Antonio Caas Vargas
	Rafael Barranco-Droege
    English translation (finished):
	Antonio Caas Vargas
    Spanish translation (finished):
	Antonio Caas Vargas
    French translation:
	Antonio Caas Vargas
    Guarani translation:
	Antonio Caas Vargas
    Italian translation (finished):
	Antonio Caas Vargas
	Nicola Comunale Rizzo
	Francisco Manuel Herrero Prez
	Giuseppe Antonio Pagin
	Antonella Grande
    Polish translation:
	Antonio Caas Vargas
	Wojtek Kieca
	Tomasz Olechowski
	Mateusz Stanko
    Portuguese translation:
	Antonio Caas Vargas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*****************************************************************************/
/********************************* Headers ***********************************/
/*****************************************************************************/

#include <stddef.h>		// For NULL

#include "swad_action.h"
#include "swad_agenda.h"
#include "swad_assignment.h"
#include "swad_browser.h"
#include "swad_building.h"
#include "swad_center.h"
#include "swad_config.h"
#include "swad_country.h"
#include "swad_course.h"
#include "swad_date.h"
#include "swad_degree.h"
#include "swad_degree_type.h"
#include "swad_department.h"
#include "swad_exam_log.h"
#include "swad_figure.h"
#include "swad_forum.h"
#include "swad_hidden_visible.h"
#include "swad_hierarchy.h"
#include "swad_holiday.h"
#include "swad_info.h"
#include "swad_institution.h"
#include "swad_language.h"
#include "swad_mail.h"
#include "swad_menu.h"
#include "swad_notification.h"
#include "swad_photo.h"
#include "swad_place.h"
#include "swad_privacy.h"
#include "swad_program.h"
#include "swad_project.h"
#include "swad_project_config.h"
#include "swad_record.h"
#include "swad_role.h"
#include "swad_room.h"
#include "swad_rubric_type.h"
#include "swad_setting.h"
#include "swad_statistic.h"
#include "swad_survey.h"
#include "swad_syllabus.h"
#include "swad_tab.h"
#include "swad_test.h"
#include "swad_test_visibility.h"
#include "swad_timeline.h"
#include "swad_timeline_note.h"
#include "swad_timetable.h"
#include "swad_user.h"

/*****************************************************************************/
/******************************** Constants **********************************/
/*****************************************************************************/

#ifndef L
#define L 3	// English
#endif

unsigned Txt_Current_CGI_SWAD_Language = ((unsigned) L);

const char *Txt_NEW_LINE = "\r\n";	// End of line in a file. If we put only \n the file does not look good in some Windows text editors

// The HTML entity for "" is &szlig; It stands for "S-Z ligature", because this symbol comes from the ligature of a Gothic S and a Z.
// For polish HTML entities, see https://pl.wikipedia.org/wiki/Alfabet_polski
// Guarani:
// E with tilde: &#7868;
// e with tilde: &#7869;
// I with tilde: &#296;
// i with tilde: &#297;

/***** Languages *****/
const char *Txt_STR_LANG_NAME[1 + Lan_NUM_LANGUAGES] =
	{
	[Lan_LANGUAGE_UNKNOWN] = "",
	[Lan_LANGUAGE_CA     ] = "Catal&agrave;",
	[Lan_LANGUAGE_DE     ] = "Deutsch",
	[Lan_LANGUAGE_EN     ] = "English",
	[Lan_LANGUAGE_ES     ] = "Espa&ntilde;ol",
	[Lan_LANGUAGE_FR     ] = "Fran&ccedil;ais",
	[Lan_LANGUAGE_GN     ] = "Ava&ntilde;e'&#7869;",
	[Lan_LANGUAGE_IT     ] = "Italiano",
	[Lan_LANGUAGE_PL     ] = "Polski",
	[Lan_LANGUAGE_PT     ] = "Portugu&ecirc;s",
	[Lan_LANGUAGE_TR     ] = "T&uuml;rk&ccedil;e",
	};
const char *Txt_Do_you_want_to_change_the_language_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	[Lan_LANGUAGE_UNKNOWN] = "",
	[Lan_LANGUAGE_CA     ] = "Voleu canviar l'idioma a catal&agrave;?",
	[Lan_LANGUAGE_DE     ] = "Wollen Sie die Sprache auf Deutsch umstellen?",
	[Lan_LANGUAGE_EN     ] = "Do you want to change the language to English?",
	[Lan_LANGUAGE_ES     ] = "&iquest;Desea cambiar el idioma a espa&ntilde;ol?",
	[Lan_LANGUAGE_FR     ] = "Voulez-vous changer la langue au fran&ccedil;ais?",
	[Lan_LANGUAGE_GN     ] = "&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",
	[Lan_LANGUAGE_IT     ] = "Vuoi cambiare la lingua in italiano?",
	[Lan_LANGUAGE_PL     ] = "Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",
	[Lan_LANGUAGE_PT     ] = "Voc&ecirc; quer mudar o idioma para portugu&ecirc;s?",
	[Lan_LANGUAGE_TR     ] = "Dili T&uuml;rk&ccedil;e olarak de&gbreve;i&scedil;tirmek istiyor musunuz?",
	};
const char *Txt_Do_you_want_to_change_your_language_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	[Lan_LANGUAGE_UNKNOWN] = "",
	[Lan_LANGUAGE_CA     ] = "Voleu canviar el seu idioma a catal&agrave;?",
	[Lan_LANGUAGE_DE     ] = "Wollen Sie Ihre Sprache auf Deutsch umstellen?",
	[Lan_LANGUAGE_EN     ] = "Do you want to change your language to English?",
	[Lan_LANGUAGE_ES     ] = "&iquest;Desea cambiar su idioma a espa&ntilde;ol?",
	[Lan_LANGUAGE_FR     ] = "Voulez-vous changer votre langue au fran&ccedil;ais?",
	[Lan_LANGUAGE_GN     ] = "&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",
	[Lan_LANGUAGE_IT     ] = "Vuoi cambiare la tua lingua in italiano?",
	[Lan_LANGUAGE_PL     ] = "Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",
	[Lan_LANGUAGE_PT     ] = "Voc&ecirc; quer mudar suo idioma para portugu&ecirc;s?",
	[Lan_LANGUAGE_TR     ] = "Dilinizi T&uuml;rk&ccedil;e olarak de&gbreve;i&scedil;tirmek ister misiniz?",
	};
const char *Txt_Switch_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	[Lan_LANGUAGE_UNKNOWN] = "",
	[Lan_LANGUAGE_CA     ] = "Canviar a catal&agrave;",
	[Lan_LANGUAGE_DE     ] = "Umschalten auf Deutsch",
	[Lan_LANGUAGE_EN     ] = "Switch to English",
	[Lan_LANGUAGE_ES     ] = "Cambiar a espa&ntilde;ol",
	[Lan_LANGUAGE_FR     ] = "Passer au fran&ccedil;ais",
	[Lan_LANGUAGE_GN     ] = "Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;",
	[Lan_LANGUAGE_IT     ] = "Passare a italiano",
	[Lan_LANGUAGE_PL     ] = "Prze&lstrok;&aogon;cz na polski",
	[Lan_LANGUAGE_PT     ] = "Mudar para portugu&ecirc;s",
	[Lan_LANGUAGE_TR     ] = "T&uuml;rk&ccedil;e'ye ge&ccedil;",
	};
const char *Txt_Switching_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	[Lan_LANGUAGE_UNKNOWN] = "",
	[Lan_LANGUAGE_CA     ] = "Canviant a catal&agrave;&hellip;",
	[Lan_LANGUAGE_DE     ] = "Umschaltung auf Deutsch&hellip;",
	[Lan_LANGUAGE_EN     ] = "Switching to English&hellip;",
	[Lan_LANGUAGE_ES     ] = "Cambiando a espa&ntilde;ol&hellip;",
	[Lan_LANGUAGE_FR     ] = "Passant au fran&ccedil;ais&hellip;",
	[Lan_LANGUAGE_GN     ] = "Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;&hellip;",
	[Lan_LANGUAGE_IT     ] = "Passando a italiano&hellip;",
	[Lan_LANGUAGE_PL     ] = "Prze&lstrok;&aogon;cz na polski&hellip;",
	[Lan_LANGUAGE_PT     ] = "Mudando para portugu&ecirc;s&hellip;",
	[Lan_LANGUAGE_TR     ] = "T&uuml;rk&ccedil;eye ge&ccedil;i&scedil;&hellip;",
	};

/***** Messages translated (use HTML entities) *****/

const char *Txt_A_face_marked_in_green_has_been_detected_ =
#if   L==1	// ca
	"S'ha detectat un rostre (senyalat en verd) en posici&oacute; frontal"
	" i amb el fons prou clar.<br />"
	"Premeu sobre ell per confirmar la actualizaci&oacute; de la fotografia.";
#elif L==2	// de
	"Ein Gesicht (gr&uuml;n markiert) wurde vor Position erfasst"
	" und der Hintergrund ist genug hell.<br />"
	"Klicken Sie darauf, um das Update des Bildes zu best&auml;tigen.";
#elif L==3	// en
	"A face (marked in green) has been detected in front position"
	" and the background is light enough.<br />"
	"Click on it to confirm the update of the picture.";
#elif L==4	// es
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";
#elif L==5	// fr
	"Un visage (marqu&eacute; en vert) a &eacute;t&eacute; d&eacute;tect&eacute; en position avant"
	" et l'arri&egrave;re-plan est assez clair.<br />"
	"Cliquez sur pour confirmer la mise &agrave; jour de l'image.";
#elif L==6	// gn
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Un viso (segnato in verde) &egrave; stato individuato nella posizione centrale"
	" e lo sfondo &egrave; abbastanza chiaro.<br />"
	"Clicca su di esso per confermare l'aggiornamento dell'immagine.";
#elif L==8	// pl
	"Twarz (zaznaczone na zielono) zosta&lstrok; wykryty w przednim po&lstrok;o&zdot;eniu,"
	" a t&lstrok;o jest wystarczaj&aogon;co du&zdot;o &sacute;wiat&lstrok;a.<br />"
	"Kliknij go, aby potwierdzi&cacute; aktualizacj&eogon; obrazu.";
#elif L==9	// pt
	"Um rosto (marcado em verde) foi detectado em posi&ccedil;&atilde;o frontal"
	" eo fundo &eacute; claro o suficiente.<br />"
	"Clique nele para confirmar a atualiza&ccedil;&atilde;o da imagem.";
#elif L==10	// tr
	"A face (marked in green) has been detected in front position"
	" and the background is light enough.<br />"
	"Click on it to confirm the update of the picture.";						// eviri lazim!
#endif

const char *Txt_A_face_marked_in_red_has_been_detected_ =
#if   L==1	// ca
	"S'ha detectat un rostre (senyalat en vermell) en posici&oacute; frontal,"
	" per&ograve; el fons &eacute;s massa fosc.";
#elif L==2	// de
	"Ein Gesicht (rot markiert) wurde vor Position erkannt,"
	" aber der Hintergrund ist zu dunkel.";
#elif L==3	// en
	"A face (marked in red) has been detected in front position,"
	" but the background is too dark.";
#elif L==4	// es
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";
#elif L==5	// fr
	"Un visage (marqu&eacute; en rouge) a &eacute;t&eacute; d&eacute;tect&eacute; en position avant,"
	" mais le fond est trop sombre.";
#elif L==6	// gn
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";	// Okoteve traduccin
#elif L==7	// it
	"Un viso (segnato in rosso) &egrave; stato individuato nella posizione centrale,"
	" ma lo sfondo &egrave; troppo scuro.";
#elif L==8	// pl
	"Twarz (zaznaczone na czerwono) zosta&lstrok; wykryty w przednim po&lstrok;o&zdot;eniu,"
	" ale t&lstrok;o jest zbyt ciemne.";
#elif L==9	// pt
	"Um rosto (marcado em vermelho) foi detectado em posi&ccedil;&atilde;o frontal,"
	" mas o plano de fundo &eacute; demasiado escuro.";
#elif L==10	// tr
	"A face (marked in red) has been detected in front position,"
	" but the background is too dark.";	// eviri lazim!
#endif

const char *Txt_A_message_has_been_sent_to_email_address_X_to_confirm_that_address =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"S'ha enviat un missatge"
	" a l'adre&ccedil;a de correu <strong>%s</strong>"
	" per confirmar aquesta adre&ccedil;a.";
#elif L==2	// de
	"Es wurde eine Nachricht"
	" an die E-Mail-Adresse <strong>%s</strong> gesendet,"
	" um diese Adresse zu best&auml;tigen.";
#elif L==3	// en
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";
#elif L==4	// es
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";
#elif L==5	// fr
	"Un message a &eacute;t&eacute; envoy&eacute;"
	" &agrave; l'adresse e-mail <strong>%s</strong>"
	" pour confirmer cette adresse.";
#elif L==6	// gn
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; stato inviato un messaggio"
	" all'indirizzo email <strong>%s</strong>"
	" per confermare tale indirizzo.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; zosta&lstrok;a wys&lstrok;ana"
	" na adres e-mail <strong>%s</strong>"
	" w celu potwierdzenia tego adresu.";
#elif L==9	// pt
	"Uma mensagem foi enviada"
	" para o endere&ccedil;o de email <strong>%s</strong>"
	" para confirmar esse endere&ccedil;o.";
#elif L==10	// tr
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// eviri lazim!
#endif

const char *Txt_A_question_has_been_added =
#if   L==1	// ca
	"S'ha afegit una pregunta.";
#elif L==2	// de
	"Eine Frage wurde hinzugef&uuml;gt.";
#elif L==3	// en
	"A question has been added.";
#elif L==4	// es
	"Se ha a&ntilde;adido una pregunta.";
#elif L==5	// fr
	"Une question a &eacute;t&eacute; ajout%eacute;e.";
#elif L==6	// gn
	"Se ha a&ntilde;adido una pregunta.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; stata aggiunta una domanda.";
#elif L==8	// pl
	"Pytanie zosta&lstrok;o dodane.";
#elif L==9	// pt
	"Uma pergunta foi adicionada.";
#elif L==10	// tr
	"A question has been added.";		// eviri lazim!
#endif

const char *Txt_X_questions_have_been_added =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han afegit %u preguntes.";
#elif L==2	// de
	"%u Fragen wurden hinzugef&uuml;gt.";
#elif L==3	// en
	"%u questions have been added.";
#elif L==4	// es
	"Se han a&ntilde;adido %u preguntas.";
#elif L==5	// fr
	"%u questions ont &eacute;t&eacute; ajout&eacute;es.";
#elif L==6	// gn
	"Se han a&ntilde;adido %u preguntas.";	// Okoteve traduccin
#elif L==7	// it
	"Sono state aggiunte %u domande.";
#elif L==8	// pl
	"Dodano %u pyta&nacute;.";
#elif L==9	// pt
	"%u perguntas foram adicionadas.";
#elif L==10	// tr
	"%u questions have been added.";	// eviri lazim!
#endif

const char *Txt_A_student_can_belong_to_several_groups =
#if   L==1	// ca
	"Un estudiant pot pert&agrave;nyer a diversos grups";
#elif L==2	// de
	"Ein Student kann mehreren Gruppen angeh&ouml;ren";
#elif L==3	// en
	"A student can belong to several groups";
#elif L==4	// es
	"Un estudiante puede pertenecer a varios grupos";
#elif L==5	// fr
	"Un &eacute;tudiant peut appartenir &agrave; plusieurs groupes";
#elif L==6	// gn
	"Un estudiante puede pertenecer a varios grupos";	// Okoteve traduccin
#elif L==7	// it
	"Uno studente pu&ograve; appartenere a pi&ugrave; gruppi";
#elif L==8	// pl
	"Kazdy student moze nalezec do wielu grup";
#elif L==9	// pt
	"Um estudante pode pertencer a v&aacute;rios grupos";
#elif L==10	// tr
	"A student can belong to several groups";		// eviri lazim!
#endif

const char *Txt_A_student_can_only_belong_to_one_group =
#if   L==1	// ca
	"Un estudiant nom&eacute;s pot pert&agrave;nyer a un grup";
#elif L==2	// de
	"Ein Student kann nur einer Gruppe angeh&ouml;ren";
#elif L==3	// en
	"A student can only belong to one group";
#elif L==4	// es
	"Un estudiante solo puede pertenecer a un grupo";
#elif L==5	// fr
	"Un &eacute;tudiant ne peut appartenir &agrave; un groupe";
#elif L==6	// gn
	"Un estudiante solo puede pertenecer a un grupo";	// Okoteve traduccin
#elif L==7	// it
	"Uno studente pu&ograve; appartenere solo a un gruppo";
#elif L==8	// pl
	"student moze nalezec tylko do jednej grupy";
#elif L==9	// pt
	"Um estudante pode pertencer a um grupo";
#elif L==10	// tr
	"A student can only belong to one group";		// eviri lazim!
#endif

const char *Txt_A_teacher_or_administrator_has_enroled_you_as_X_into_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Un professor/a o un administrador/a l'ha inscrit"
	" com a <strong>%s</strong>"
	" en l'assignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;"
	" o prefereix ser eliminat de l'assignatura?";
#elif L==2	// de
	"Ein Lehrer oder ein Administrator hat Sie"
	" als <strong>%s</strong>"
	" in den Kurs <strong>%s</strong> eingeschrieben."
	" M&ouml;chten Sie die Anmeldung best&auml;tigen"
	" oder ziehen Sie es vor,"
	" aus dem Kurs entfernt zu werden?";
#elif L==3	// en
	"A teacher or an administrator has enroled you"
	" as a <strong>%s</strong>"
	" into the course <strong>%s</strong>."
	" Do you want to confirm the enrolment"
	" or do you prefer to be removed from the course?";
#elif L==4	// es
	"Un profesor/a o un administrador/a le ha inscrito"
	" como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado/a de la asignatura?";
#elif L==5	// fr
	"Un professeur ou un administrateur vous a enregistr&eacute;"
	" en tant que <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong>."
	" Voulez-vous confirmer l'inscription"
	" ou pr&eacute;f&eacute;rez-vous &ecirc;tre retir&eacute; de la mati&egrave;re?";
#elif L==6	// gn
	"Un profesor/a o un administrador/a le ha inscrito"
	" como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado/a de la asignatura?";	// Okoteve traduccin
#elif L==7	// it
	"Un insegnante o amministratore ti a registrato"
	" come <strong>%s</strong>"
	" nel corso <strong>%s</strong>."
	" Vuoi confermare la registrazione"
	" o preferisci essere rimosso da questo corso?";
#elif L==8	// pl
	"Nauczyciel lub administrator zapisa&lstrok; ci&eogon;"
	" jako <strong>%s</strong>"
	" do kursu <strong>%s</strong>."
	" Czy chcesz, aby potwierdzi&cacute; rejestracj&eogon;"
	" czy wolisz zosta&cacute; usuni&eogon;te z kursu?";
#elif L==9	// pt
	"Um professor/a ou administrador/a tem registrado voc&ecirc;"
	" como <strong>%s</strong>"
	" na disciplina <strong>%s</strong>."
	" Voc&ecirc; quer para confirmar a inscri&ccedil;&atilde;o"
	" ou voc&ecirc; prefere para ser removido/a da disciplina?";
#elif L==10	// tr
	"A teacher or an administrator has enroled you"
	" as a <strong>%s</strong>"
	" into the course <strong>%s</strong>."
	" Do you want to confirm the enrolment"
	" or do you prefer to be removed from the course?";	// eviri lazim!
#endif

const char *Txt_A_user_can_not_have_more_than_X_IDs =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Un usuari no pot tenir m&eacute;s de %u IDs (DNI/c&egrave;dulas)";
#elif L==2	// de
	"Ein Benutzer kann nicht mehr als %u Ausweis-Nr.";
#elif L==3	// en
	"A user can not have more than %u IDs.";
#elif L==4	// es
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";
#elif L==5	// fr
	"Un utilisateur ne peut pas avoir plus de %u IDs.";
#elif L==6	// gn
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";	// Okoteve traduccin
#elif L==7	// it
	"Un utente non pu&ograve; avere pi&ugrave; di %u ID.";
#elif L==8	// pl
	"U&zdot;ytkownik nie mo&zdot;e mie&cacute; wi&eogon;cej ni&zdot; %u ID.";
#elif L==9	// pt
	"Um usu&aacute;rio n&atilde;o pode ter mais de %u n&ordm;s. identif.";
#elif L==10	// tr
	"A user can not have more than %u IDs.";				// eviri lazim!
#endif

const char *Txt_About_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Quant a %s";
#elif L==2	// de
	"&Uuml;ber uns %s";
#elif L==3	// en
	"About %s";
#elif L==4	// es
	"Acerca de %s";
#elif L==5	// fr
	"&Agrave; propos de %s";
#elif L==6	// gn
	"%s rehegua";
#elif L==7	// it
	"Riguardo %s";
#elif L==8	// pl
	"O %s";
#elif L==9	// pt
	"Sobre %s";
#elif L==10	// tr
	"%s hakk&inodot;nda";
#endif

const char *Txt_Absent =
#if   L==1	// ca
	"Absent";
#elif L==2	// de
	"Nicht vorhanden";
#elif L==3	// en
	"Absent";
#elif L==4	// es
	"Ausente";
#elif L==5	// fr
	"Absent";
#elif L==6	// gn
	"Ausente";	// Okoteve traduccin
#elif L==7	// it
	"Assente";
#elif L==8	// pl
	"Nieobecny";
#elif L==9	// pt
	"Assente";
#elif L==10	// tr
	"Devams&inodot;z";
#endif

const char *Txt_Absents =
#if   L==1	// ca
	"Absents";
#elif L==2	// de
	"Nicht vorhanden";
#elif L==3	// en
	"Absents";
#elif L==4	// es
	"Ausentes";
#elif L==5	// fr
	"Absents";
#elif L==6	// gn
	"Ausentes";	// Okoteve traduccin
#elif L==7	// it
	"Assenti";
#elif L==8	// pl
	"Nieobecny";
#elif L==9	// pt
	"Assentes";
#elif L==10	// tr
	"Devams&inodot;z";
#endif

const char *Txt_Accept_third_party_cookies =
#if   L==1	// ca
	"Accepta cookies de tercers";
#elif L==2	// de
	"Akzeptieren Sie Cookies von Drittanbietern";
#elif L==3	// en
	"Accept third-party cookies";
#elif L==4	// es
	"Aceptar cookies de terceros";
#elif L==5	// fr
	"Accepter les cookies de tiers";
#elif L==6	// gn
	"Emone&itilde; umi cookie mbohap&yacute;ha tapicha rehegua";
#elif L==7	// it
	"Accettare i cookie di terze parti";
#elif L==8	// pl
	"Zaakceptuj pliki cookie stron trzecich";
#elif L==9	// pt
	"Aceitar cookies de terceiros";
#elif L==10	// tr
	"&Uuml;&ccedil;&uuml;nc&uuml; taraf tan&inodot;mlama bilgilerini kabul et";
#endif

const char *Txt_Accept_third_party_cookies_to_view_multimedia_content_from_other_websites =
#if   L==1	// ca
	"Accepta cookies de tercers"
	" per veure contingut multim&iacute;dia de outros sites";
#elif L==2	// de
	"Akzeptieren Sie Cookies von Drittanbietern,"
	" um Multimedia-Inhalte von anderen Websites anzuzeigen";
#elif L==3	// en
	"Accept third-party cookies"
	" to view multimedia content from other websites";
#elif L==4	// es
	"Aceptar cookies de terceros"
	" para ver contenido multimedia de otros sitios web";
#elif L==5	// fr
	"Accepter les cookies de tiers"
	" pour afficher le contenu multim&eacute;dia d'autres sites Web";
#elif L==6	// gn
	"Eacepta umi cookie mbohap&yacute;ha rehegua"
	" ehecha ha&gtilde;ua contenido multimedia ambue p&aacute;gina web-gui";
#elif L==7	// it
	"Accettare i cookie di terze parti"
	" per visualizzare contenuti multimediali da altri siti Web";
#elif L==8	// pl
	"Zaakceptuj pliki cookie stron trzecich,"
	" aby przegl&aogon;da&cacute; tre&sacute;ci multimedialne z innych stron internetowych";
#elif L==9	// pt
	"Aceitar cookies de terceiros"
	" para visualizar conte&uacute;do multimdia de outros sites";
#elif L==10	// tr
	"Di&gbreve;er web sitelerinden multimedya i&ccedil;eri&gbreve;ini g&ouml;r&uuml;nt&uuml;lemek i&ccedil;in"
	" &uuml;&ccedil;&uuml;nc&uuml; taraf tan&inodot;mlama bilgilerini kabul edin";
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_center =
#if   L==1	// ca
	"accessible per a lectura i escriptura per administradors del center";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Administratoren der Lehrinstitut";
#elif L==3	// en
	"accessible for reading and writing by administrators of the center";
#elif L==4	// es
	"accesible para lectura y escritura por administradores del centro";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les administrateurs du center";
#elif L==6	// gn
	"accesible para lectura y escritura por administradores del centro";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei amministratori del centro";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez administratorzy centrum";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos administradores do centro";
#elif L==10	// tr
	"accessible for reading and writing by administrators of the center";	// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_degree =
#if   L==1	// ca
	"accessible per a lectura i escriptura"
	" per administradors de la titulaci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich"
	" f&uuml;r Administratoren der Studiengang";
#elif L==3	// en
	"accessible for reading and writing"
	" by administrators of the degree";
#elif L==4	// es
	"accesible para lectura y escritura"
	" por administradores de la titulaci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture"
	" par les administrateurs de l'&eacute;tude";
#elif L==6	// gn
	"accesible para lectura y escritura"
	" por administradores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura"
	" da parte dei amministratori della laurea";
#elif L==8	// pl
	"dostep do odczytu i zapisu"
	" przez administratorzy stopnia";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita"
	" pelos administradores do grau";
#elif L==10	// tr
	"accessible for reading and writing"
	" by administrators of the degree";		// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_institution =
#if   L==1	// ca
	"accessible per a lectura i escriptura per administradors de l'instituci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Administratoren der Hochschule";
#elif L==3	// en
	"accessible for reading and writing by administrators of the institution";
#elif L==4	// es
	"accesible para lectura y escritura por administradores de la instituci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les administrateurs de l'&eacute;tablissement";
#elif L==6	// gn
	"accesible para lectura y escritura por administradores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei amministratori della istituzione";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez administratorzy instytucji";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos administradores da institui&ccedil;&atilde;o";
#elif L==10	// tr
	"accessible for reading and writing by administrators of the institution";		// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_project_members =
#if   L==1	// ca
	"accessible per a lectura i escriptura per membres del projecte";
#elif L==2	// de
	"zug&auml;nglich zum Lesen und Schreiben von Projektmitgliedern";
#elif L==3	// en
	"accessible for reading and writing by project members";
#elif L==4	// es
	"accesible para lectura y escritura por miembros del proyecto";
#elif L==5	// fr
	"accessible pour la lecture et l'&eacute;criture par les membres du projet";
#elif L==6	// gn
	"accesible para lectura y escritura por miembros del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei membri del progetto";
#elif L==8	// pl
	"dost&eogon;pne do czytania i pisania przez cz&lstrok;onk&oacute;w projektu";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita por membros do projeto";
#elif L==10	// tr
	"accessible for reading and writing by project members";	// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_project_tutors_and_evaluators =
#if   L==1	// ca
	"accessible per a lectura i escriptura per tutors i avaluadors del projecte";
#elif L==2	// de
	"zug&auml;nglich zum Lesen und Schreiben f&uuml;r Tutoren und Auswerter der Projekt";
#elif L==3	// en
	"accessible for reading and writing by project tutors and evaluators";
#elif L==4	// es
	"accesible para lectura y escritura por tutores y evaluadores del proyecto";
#elif L==5	// fr
	"accessible pour la lecture et l'&eacute;criture par tuteurs et &eacute;valuateurs du projet";
#elif L==6	// gn
	"accesible para lectura y escritura por tutores y evaluadores del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei precettori e dei valutatori del progetto";
#elif L==8	// pl
	"dost&eogon;pne do czytania i pisania przez nauczyciele i ewaluator&oacute;w projektu";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita por tutores e avaliadores do projeto";
#elif L==10	// tr
	"accessible for reading and writing by project tutors and evaluators";		// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_center =
#if   L==1	// ca
	"accessible per a lectura i escriptura per estudiants i professors del center";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Lehrinstitut";
#elif L==3	// en
	"accessible for reading and writing by students and teachers of the center";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes y profesores del centro";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants du center";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes y profesores del centro";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori del centro";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli centrum";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes e professores do centro";
#elif L==10	// tr
	"accessible for reading and writing by students and teachers of the center";	// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_course =
#if   L==1	// ca
	"accessible per a lectura i escriptura per estudiants i professors de l'assignatura";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible for reading and writing by students and teachers of the course";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori del corso";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli zajec";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes e professores da disciplina";
#elif L==10	// tr
	"accessible for reading and writing by students and teachers of the course";		// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_degree =
#if   L==1	// ca
	"accessible per a lectura i escriptura"
	" per estudiants i professors de la titulaci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich"
	" f&uuml;r Studenten und Lehrkr&auml;fte der Studiengang";
#elif L==3	// en
	"accessible for reading and writing"
	" by students and teachers of the degree";
#elif L==4	// es
	"accesible para lectura y escritura"
	" por estudiantes y profesores de la titulaci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture"
	" par les &eacute;tudiants et les enseignants de l'&eacute;tude";
#elif L==6	// gn
	"accesible para lectura y escritura"
	" por estudiantes y profesores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura"
	" da parte degli studenti e dei professori della laurea";
#elif L==8	// pl
	"dostep do odczytu i zapisu"
	" przez uczni&oacute;w i nauczycieli stopnia";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita"
	" pelos estudantes e professores do grau";
#elif L==10	// tr
	"accessible for reading and writing"
	" by students and teachers of the degree";			// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_institution =
#if   L==1	// ca
	"accessible per a lectura i escriptura per estudiants i professors de la instituci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Hochschule";
#elif L==3	// en
	"accessible for reading and writing by students and teachers of the institution";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes y profesores de la instituci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants de l'&eacute;tablissement";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes y profesores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori della istituzione";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli instytucji";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes e professores da institui&ccedil;&atilde;o";
#elif L==10	// tr
	"accessible for reading and writing by students and teachers of the institution";		// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_of_the_group_and_teachers_of_the_course =
#if   L==1	// ca
	"accesible per a lectura i escriptura per estudiants del grup i professors de l'assignatura";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible for reading and writing by students of the group and teachers of the course";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti del gruppo e dei professori del corso";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez student&oacute;w i nauczycieli, grupy kursu";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes do grupo e professores da disciplina";
#elif L==10	// tr
	"accessible for reading and writing by students of the group and teachers of the course";	// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_teachers_of_the_course =
#if   L==1	// ca
	"accessible per a lectura i escriptura per professors de l'assignatura";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible for reading and writing by teachers of the course";
#elif L==4	// es
	"accesible para lectura y escritura por profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible para lectura y escritura por profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei professori del corso";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez nauczycieli zajec";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos professores da disciplina";
#elif L==10	// tr
	"accessible for reading and writing by teachers of the course";		// eviri lazim!
#endif

const char *Txt_accessible_for_reading_and_writing_by_you_and_the_course_teachers =
#if   L==1	// ca
	"accessible per a lectura i escriptura per vosaltres i els professors de l'assignatura";
#elif L==2	// de
	"f&uuml;r Sie und die Kursleiter zum Lesen und Schreiben zug&auml;nglich";
#elif L==3	// en
	"accessible for reading and writing by you and the course teachers";
#elif L==4	// es
	"accesible para lectura y escritura por usted y los profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par vous et les professeurs de la mati&egrave;re";
#elif L==6	// gn
	"accesible omo&ntilde;e'&etilde; ha ohai hagu&atilde; nde ha umi mbo'eh&aacute;ra asignatura rehegua";
#elif L==7	// it
	"accessibile in lettura e scrittura da te e dai docenti del corso";
#elif L==8	// pl
	"dost&eogon;pne do czytania i pisania przez Ciebie i nauczycieli kursu";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita por voc&ecirc; e pelos professores da disciplina";
#elif L==10	// tr
	"siz ve kurs &ouml;&gbreve;retmenleri taraf&inodot;ndan okuma ve yazma i&ccedil;in eri&scedil;ilebilir";
#endif

const char *Txt_accessible_only_for_reading_by_the_student_and_the_course_teachers =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura per l'estudiant i els professors de l'assignatura";
#elif L==2	// de
	"nur zur Lekt&uuml;re durch Studierende und Kursleiter zug&auml;nglich";
#elif L==3	// en
	"accessible only for reading by the student and the course teachers";
#elif L==4	// es
	"accesible solo para lectura por el estudiante y los profesores de la asignatura";
#elif L==5	// fr
	"accessible uniquement &agrave; la lecture par l'&eacute;tudiant et les professeurs de la mati&egrave;re";
#elif L==6	// gn
	"ojeikekuaa omo&ntilde;e'&etilde; ha&gcirc;u&aacute;nte temimbo'e ha mbo'eh&aacute;ra asignatura rehegua";
#elif L==7	// it
	"accessibile in sola lettura da parte dello studente e dei docenti del corso";
#elif L==8	// pl
	"dost&eogon;pne wy&lstrok;&aogon;cznie do czytania przez ucznia i nauczyciela przedmiotu";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelo aluno e pelos professores da disciplina";
#elif L==10	// tr
	"yaln&inodot;zca &ouml;&gbreve;renci ve ders &ouml;&gbreve;retmenlerinin okuyabilece&gbreve;i &scedil;ekilde eri&scedil;ilebilir";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_center =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura per estudiants i professors del center";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Lehrinstitut";
#elif L==3	// en
	"accessible only for reading by students and teachers of the center";
#elif L==4	// es
	"accesible solo para lectura por estudiantes y profesores del centro";
#elif L==5	// fr
	"accessible uniquement &agrave; la lecture par les &eacute;tudiants et les enseignants du center";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes y profesores del centro";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti e dei professori del centro";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli centrum";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes e professores do centro";
#elif L==10	// tr
	"accessible only for reading by students and teachers of the center";	// eviri lazim!
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_course =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura per estudiants i professors de l'assignatura";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible only for reading by students and teachers of the course";
#elif L==4	// es
	"accesible solo para lectura por estudiantes y profesores de la asignatura";
#elif L==5	// fr
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti e dei professori del corso";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli zajec";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes e professores da disciplina";
#elif L==10	// tr
	"accessible only for reading by students and teachers of the course";		// eviri lazim!
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_degree =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura"
	" per estudiants i professors de la titulaci&oacute;";
#elif L==2	// de
	"zum Lesen zug&auml;nglich"
	" f&uuml;r Studenten und Lehrkr&auml;fte der Studiengang";
#elif L==3	// en
	"accessible only for reading"
	" by students and teachers of the degree";
#elif L==4	// es
	"accesible solo para lectura"
	" por estudiantes y profesores de la titulaci&oacute;n";
#elif L==5	// fr
	"accessible uniquement pour la lecture"
	" par les &eacute;tudiants et les enseignants de l'&eacute;tude";
#elif L==6	// gn
	"accesible solo para lectura"
	" por estudiantes y profesores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura"
	" da parte degli studenti e dei professori della laurea";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania"
	" przez uczni&oacute;w i nauczycieli stopnia";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura"
	" pelos estudantes e professores do grau";
#elif L==10	// tr
	"accessible only for reading"
	" by students and teachers of the degree";			// eviri lazim!
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_institution =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura per estudiants i professors de l'instituci&oacute;";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Hochschule";
#elif L==3	// en
	"accessible only for reading by students and teachers of the institution";
#elif L==4	// es
	"accesible solo para lectura por estudiantes y profesores de la instituci&oacute;n";
#elif L==5	// fr
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants de l'&eacute;tablissement";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes y profesores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti e dei professori della istituzione";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli instytucji";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes e professores da institui&ccedil;&atilde;o";
#elif L==10	// tr
	"accessible only for reading by students and teachers of the institution";		// eviri lazim!
#endif

const char *Txt_accessible_only_for_reading_by_students_of_the_group_and_teachers_of_the_course =
#if   L==1	// ca
	"accesible nom&eacute;s per a lectura per estudiants del grup i professors de l'assignatura";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible only for reading by students of the group and teachers of the course";
#elif L==4	// es
	"accesible solo para lectura por estudiantes del grupo y profesores de la asignatura";
#elif L==5	// fr
	"accessible uniquement pour la lecture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti del gruppo e dei professori del corso";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli grupy kursu";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes do grupo e professores da disciplina";
#elif L==10	// tr
	"accessible only for reading by students of the group and teachers of the course";	// eviri lazim!
#endif

const char *Txt_Action =
#if   L==1	// ca
	"Acci&oacute;";
#elif L==2	// de
	"Handlung";
#elif L==3	// en
	"Action";
#elif L==4	// es
	"Acci&oacute;n";
#elif L==5	// fr
	"Action";
#elif L==6	// gn
	"Tembiapo";
#elif L==7	// it
	"Azione";
#elif L==8	// pl
	"Dzia&lstrok;anie";
#elif L==9	// pt
	"A&ccedil;&atilde;o";
#elif L==10	// tr
	"Eylem";
#endif

const char *Txt_Add_questions =
#if   L==1	// ca
	"Afegir preguntes";
#elif L==2	// de
	"F&uuml;gen Fragen";
#elif L==3	// en
	"Add questions";
#elif L==4	// es
	"A&ntilde;adir preguntas";
#elif L==5	// fr
	"Ajouter questions";
#elif L==6	// gn
	"A&ntilde;adir preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Aggiungere domande";
#elif L==8	// pl
	"Dodaj pytania";
#elif L==9	// pt
	"Adicionar perguntas";
#elif L==10	// tr
	"Soru ekle";
#endif

const char *Txt_add_rubric =
#if   L==1	// ca
	"afegir r&uacute;brica";
#elif L==2	// de
	"Rubrik hinzuf&uuml;gen";
#elif L==3	// en
	"add rubric";
#elif L==4	// es
	"a&ntilde;adir r&uacute;brica";
#elif L==5	// fr
	"ajouter rubrique";
#elif L==6	// gn
	"omo&itilde;ve r&uacute;brica";
#elif L==7	// it
	"aggiungi rubrica";
#elif L==8	// pl
	"dodaj rubryk&eogon;";
#elif L==9	// pt
	"adicionar rubrica";
#elif L==10	// tr
	"de&gbreve;erlendirme listesi ekle";
#endif

const char *Txt_Add_this_ID =
#if   L==1	// ca
	"Afegir aquest ID";
#elif L==2	// de
	"F&uuml;gen Sie diese Ausweis-Nr.";
#elif L==3	// en
	"Add this ID";
#elif L==4	// es
	"A&ntilde;adir este ID";
#elif L==5	// fr
	"Ajouter ce num&eacute;ro d'identit&eacute;";
#elif L==6	// gn
	"A&ntilde;adir este ID";	// Okoteve traduccin
#elif L==7	// it
	"Aggiungere questa carta d'identit&agrave;";
#elif L==8	// pl
	"Dodaj ten ID";
#elif L==9	// pt
	"Adicionar este n&ordm; identif.";
#elif L==10	// tr
	"Bu kimli&gbreve;i ekle";
#endif

const char *Txt_Add_USERS =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Afegir %s";
#elif L==2	// de
	"F&uuml;gen %s";
#elif L==3	// en
	"Add %s";
#elif L==4	// es
	"A&ntilde;adir %s";
#elif L==5	// fr
	"Ajouter %s";
#elif L==6	// gn
	"A&ntilde;adir %s";	// Okoteve traduccin
#elif L==7	// it
	"Aggiungere %s";
#elif L==8	// pl
	"Dodaj %s";
#elif L==9	// pt
	"Adicionar %s";
#elif L==10	// tr
	"%s ekle";
#endif

const char *Txt_Administer_me =
#if   L==1	// ca
	"Administrarme";
#elif L==2	// de
	"Verwalten Sie mich";
#elif L==3	// en
	"Administer me";
#elif L==4	// es
	"Administrarme";
#elif L==5	// fr
	"G&eacute;rer moi";
#elif L==6	// gn
	"Administrarme";	// Okoteve traduccin
#elif L==7	// it
	"Gestire mi";
#elif L==8	// pl
	"Zarz&aogon;dzaj mnie";
#elif L==9	// pt
	"Gerenciar me";
#elif L==10	// tr
	"Beni y&ouml;net";
#endif

const char *Txt_Administer_multiple_students =
#if   L==1	// ca
	"Administrar diversos estudiants";
#elif L==2	// de
	"Verwalten Sie mehrere Studenten";
#elif L==3	// en
	"Administer multiple students";
#elif L==4	// es
	"Administrar varios estudiantes";
#elif L==5	// fr
	"G&eacute;rer plusieurs &eacute;tudiants";
#elif L==6	// gn
	"Administrar varios estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Gestire pi&ugrave; studenti";
#elif L==8	// pl
	"Zarz&aogon;dzaj wielu studenci";
#elif L==9	// pt
	"Gerenciar v&aacute;rios estudantes";
#elif L==10	// tr
	"Administer multiple students";		// eviri lazim!
#endif

const char *Txt_Administer_multiple_non_editing_teachers =
#if   L==1	// ca
	"Administrar diversos professors no editors";
#elif L==2	// de
	"Verwalten Sie mehrere nicht bearbeiteter Lehrkr&auml;fte";
#elif L==3	// en
	"Administer multiple non-editing teachers";
#elif L==4	// es
	"Administrar varios profesores no editores";
#elif L==5	// fr
	"G&eacute;rer plusieurs enseignants non-&eacute;diteurs";
#elif L==6	// gn
	"Administrar varios profesores no editores";	// Okoteve traduccin
#elif L==7	// it
	"Gestire pi&ugrave; professori non-editing";
#elif L==8	// pl
	"Zarz&aogon;dzaj wielu nauczycieli nie edytuj&aogon;cy";
#elif L==9	// pt
	"Gerenciar v&aacute;rios professores n&atilde;o editores";
#elif L==10	// tr
	"Administer multiple non-editing teachers";	// eviri lazim!
#endif

const char *Txt_Administer_multiple_teachers =
#if   L==1	// ca
	"Administrar diversos professors";
#elif L==2	// de
	"Verwalten Sie mehrere Lehrkr&auml;fte";
#elif L==3	// en
	"Administer multiple teachers";
#elif L==4	// es
	"Administrar varios profesores";
#elif L==5	// fr
	"G&eacute;rer plusieurs enseignants";
#elif L==6	// gn
	"Administrar varios profesores";	// Okoteve traduccin
#elif L==7	// it
	"Gestire pi&ugrave; professori";
#elif L==8	// pl
	"Zarz&aogon;dzaj wielu nauczycieli";
#elif L==9	// pt
	"Gerenciar v&aacute;rios professores";
#elif L==10	// tr
	"Administer multiple teachers";		// eviri lazim!
#endif

const char *Txt_Administer_one_user =
#if   L==1	// ca
	"Administrar un usuari";
#elif L==2	// de
	"Verwalten eines Benutzer";
#elif L==3	// en
	"Administer one user";
#elif L==4	// es
	"Administrar un usuario";
#elif L==5	// fr
	"G&eacute;rer un utilisateur";
#elif L==6	// gn
	"Administrar un usuario";	// Okoteve traduccin
#elif L==7	// it
	"Gestire un utente";
#elif L==8	// pl
	"Zarz&aogon;dzaj u&zdot;ytkownika";
#elif L==9	// pt
	"Gerenciar um utilizador";
#elif L==10	// tr
	"Administer one user";		// eviri lazim!
#endif

const char *Txt_AGENDA_PAST___FUTURE_EVENTS[Agd_NUM_PAST_FUTURE_EVENTS] =
	{
	// Agd_PAST___EVENTS
#if   L==1	// ca
	"Esdeveniments passats i d'avui"
#elif L==2	// de
	"Vergangenheit und Gegenwart Ereignisse"
#elif L==3	// en
	"Past and present events"
#elif L==4	// es
	"Eventos pasados y de hoy"
#elif L==5	// fr
	"&Eacute;v&eacute;nements pass&eacute;s et pr&eacute;sents"
#elif L==6	// gn
	"Eventos pasados y de hoy"	// Okoteve traduccin
#elif L==7	// it
	"Eventi passati e presenti"
#elif L==8	// pl
	"Wydarzenia minione i obecne"
#elif L==9	// pt
	"Eventos passados e presentes"
#elif L==10	// tr
	"Past and present events"	// eviri lazim!
#endif
	,
	// Agd_FUTURE_EVENTS
#if   L==1	// ca
	"Esdeveniments d'avui i futurs"
#elif L==2	// de
	"Aktuelle und zuk&uuml;nftige Ereignisse"
#elif L==3	// en
	"Current and future events"
#elif L==4	// es
	"Eventos de hoy y futuros"
#elif L==5	// fr
	"&Eacute;v&eacute;nements actuels et futurs"
#elif L==6	// gn
	"Eventos de hoy y futuros"	// Okoteve traduccin
#elif L==7	// it
	"Eventi attuali e futuri"
#elif L==8	// pl
	"Obecne i przysz&lstrok;e wydarzenia"
#elif L==9	// pt
	"Eventos atuais e futuros"
#elif L==10	// tr
	"Current and future events"	// eviri lazim!
#endif
	};

const char *Txt_AGENDA_PRIVAT_PUBLIC_EVENTS[2] =
	{
	// Agd_PRIVAT_EVENTS
#if   L==1	// ca
	"Esdeveniments privados"
#elif L==2	// de
	"Private Ereignisse"
#elif L==3	// en
	"Private events"
#elif L==4	// es
	"Eventos privados"
#elif L==5	// fr
	"&Eacute;v&eacute;nements priv&eacute;s"
#elif L==6	// gn
	"Eventos privados"	// Okoteve traduccin
#elif L==7	// it
	"Eventi privati"
#elif L==8	// pl
	"Prywatne wydarzenia"
#elif L==9	// pt
	"Eventos privados"
#elif L==10	// tr
	"&Ouml;zel etkinlikler"
#endif
	,
	// Agd_PUBLIC_EVENTS
#if   L==1	// ca
	"Esdeveniments p&uacute;blics"
#elif L==2	// de
	"&Ouml;ffentliche Ereignisse"
#elif L==3	// en
	"Public events"
#elif L==4	// es
	"Eventos p&uacute;blicos"
#elif L==5	// fr
	"&Eacute;v&eacute;nements publics"
#elif L==6	// gn
	"Eventos p&uacute;blicos"	// Okoteve traduccin
#elif L==7	// it
	"Eventi pubblici"
#elif L==8	// pl
	"Publiczne wydarzenia"
#elif L==9	// pt
	"Eventos p&uacute;blicos"
#elif L==10	// tr
	"Halka a&ccedil;&inodot;k etkinlikler"
#endif
	};

const char *Txt_AGENDA_HIDDEN_VISIBLE_EVENTS[HidVis_NUM_HIDDEN_VISIBLE] =
	{
	// HidVis_HIDDEN
#if   L==1	// ca
	"Esdeveniments ocults"
#elif L==2	// de
	"Unsichtbare Ereignisse"
#elif L==3	// en
	"Hidden events"
#elif L==4	// es
	"Eventos ocultos"
#elif L==5	// fr
	"&Eacute;v&eacute;nements cach&eacute;s"
#elif L==6	// gn
	"Eventos ocultos"	// Okoteve traduccin
#elif L==7	// it
	"Eventi nascosti"
#elif L==8	// pl
	"Ukryte wydarzenia"
#elif L==9	// pt
	"Eventos ocultos"
#elif L==10	// tr
	"Gizli etkinlikler"
#endif
	,
	// HidVis_VISIBLE
#if   L==1	// ca
	"Esdeveniments visibles"
#elif L==2	// de
	"Sichtbar Ereignisse"
#elif L==3	// en
	"Visible events"
#elif L==4	// es
	"Eventos visibles"
#elif L==5	// fr
	"&Eacute;v&eacute;nements visibles"
#elif L==6	// gn
	"Eventos visibles"	// Okoteve traduccin
#elif L==7	// it
	"Eventi visibili"
#elif L==8	// pl
	"Widoczne wydarzenia"
#elif L==9	// pt
	"Eventos vis&iacute;veis"
#elif L==10	// tr
	"G&ouml;r&uuml;n&uuml;r etkinlikler"
#endif
	};

const char *Txt_all =
#if   L==1	// ca
	"tot";
#elif L==2	// de
	"alle";
#elif L==3	// en
	"all";
#elif L==4	// es
	"todo";
#elif L==5	// fr
	"tout";
#elif L==6	// gn
	"opakatu";
#elif L==7	// it
	"tutto";
#elif L==8	// pl
	"wszystko";
#elif L==9	// pt
	"tudo";
#elif L==10	// tr
	"t&uuml;m";
#endif

const char *Txt_all_degrees =
#if   L==1	// ca
	"Totes les titulacions";
#elif L==2	// de
	"alle Abschl&uuml;sse";
#elif L==3	// en
	"all degrees";
#elif L==4	// es
	"todas las titulaciones";
#elif L==5	// fr
	"tous les &eacute;tudes";
#elif L==6	// gn
	"todas las titulaciones";	// Okoteve traduccin
#elif L==7	// it
	"tutte le lauree";
#elif L==8	// pl
	"wszystkich stopni";
#elif L==9	// pt
	"todos os graus";
#elif L==10	// tr
	"all degrees";			// eviri lazim!
#endif

const char *Txt_all_files_inside_the_root_folder =
#if   L==1	// ca
	"tots els arxius dins de la carpeta arrel";
#elif L==2	// de
	"alle Dateien im Stammordner";
#elif L==3	// en
	"all files inside the root folder";
#elif L==4	// es
	"todos los archivos dentro de la carpeta ra&iacute;z";
#elif L==5	// fr
	"tous les fichiers dans le r&eacute;pertoire racine";
#elif L==6	// gn
	"todos los archivos dentro de la carpeta ra&iacute;z";	// Okoteve traduccin
#elif L==7	// it
	"tutti i file all'interno della cartella principale";
#elif L==8	// pl
	"wszystkie pliki znajduj&aogon;ce si&eogon; w folderze g&lstrok;&oacute;wnym";
#elif L==9	// pt
	"todos os arquivos dentro do diret&oacute;rio raiz";
#elif L==10	// tr
	"all files inside the root folder";			// eviri lazim!
#endif

const char *Txt_All_groups =
#if   L==1	// ca
	"Tots els grups";
#elif L==2	// de
	"Alle Gruppen";
#elif L==3	// en
	"All groups";
#elif L==4	// es
	"Todos los grupos";
#elif L==5	// fr
	"Tous les groupes";
#elif L==6	// gn
	"Mayma aty";
#elif L==7	// it
	"Tutti i gruppi";
#elif L==8	// pl
	"Wszystkie grupy";
#elif L==9	// pt
	"Todos os grupos";
#elif L==10	// tr
	"T&uuml;m gruplar";
#endif

const char *Txt_All_places =
#if   L==1	// ca
	"Tots els llocs";
#elif L==2	// de
	"Alle Standorte";
#elif L==3	// en
	"All places";
#elif L==4	// es
	"Todos los lugares";
#elif L==5	// fr
	"Tous les emplacements";
#elif L==6	// gn
	"Todos los lugares";	// Okoteve traduccin
#elif L==7	// it
	"Tutte le localit&agrave;";
#elif L==8	// pl
	"Wszystkie miejsca";
#elif L==9	// pt
	"Todas as localiza&ccedil;oes";
#elif L==10	// tr
	"All places";		// eviri lazim!
#endif

const char *Txt_All_questions =
#if   L==1	// ca
	"Totes les preguntes";
#elif L==2	// de
	"Alle Fragen";
#elif L==3	// en
	"All questions";
#elif L==4	// es
	"Todas las preguntas";
#elif L==5	// fr
	"Toutes les questions";
#elif L==6	// gn
	"Todas las preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Tutte le domande";
#elif L==8	// pl
	"Wszystkie pytania";
#elif L==9	// pt
	"Todas as perguntas";
#elif L==10	// tr
	"All questions";	// eviri lazim!
#endif

const char *Txt_All_tags =		// Tags used in test
#if   L==1	// ca
	"Tots els descriptors";
#elif L==2	// de
	"Alle Tags";
#elif L==3	// en
	"All tags";
#elif L==4	// es
	"Todos los descriptores";
#elif L==5	// fr
	"Tous les descripteurs";
#elif L==6	// gn
	"Todos los descriptores";	// Okoteve traduccin
#elif L==7	// it
	"Tutte le etichette";
#elif L==8	// pl
	"Wszystkie tagi";
#elif L==9	// pt
	"Todos os descritores";
#elif L==10	// tr
	"All tags";			// eviri lazim!
#endif

const char *Txt_All_types_of_answers =		// Answers of a test
#if   L==1	// ca
	"Tots els tipus de respostes";
#elif L==2	// de
	"Alle Antworttypen";
#elif L==3	// en
	"All types of answers";
#elif L==4	// es
	"Todos los tipos de respuestas";
#elif L==5	// fr
	"Tous les types de r&eacute;ponses";
#elif L==6	// gn
	"Todos los tipos de respuestas";	// Okoteve traduccin
#elif L==7	// it
	"Tutti i tipi di risposte";
#elif L==8	// pl
	"Wszystkie typy odpowiedzi";
#elif L==9	// pt
	"Todos os tipos de respostas";
#elif L==10	// tr
	"All types of answers";			// eviri lazim!
#endif

const char *Txt_Allow_teachers_to_consult_this_test =
#if   L==1	// ca
	"Permetre als professors consultar aquest test";
#elif L==2	// de
	"Erm&ouml;glichen es den Lehrern, diese Test zu konsultieren";
#elif L==3	// en
	"Allow teachers to consult this test";
#elif L==4	// es
	"Permitir a los profesores consultar este test";
#elif L==5	// fr
	"Permettre aux enseignants de consulter ce test";
#elif L==6	// gn
	"Permitir a los profesores consultar este test";	// Okoteve traduccin
#elif L==7	// it
	"Lasciare gli insegnanti di consultare questo test";
#elif L==8	// pl
	"Pozwalaj&aogon; nauczycielom skonsultowa&cacute; ten test";
#elif L==9	// pt
	"Permitir que os professores de consultar este teste";
#elif L==10	// tr
	"Allow teachers to consult this test";			// eviri lazim!
#endif

const char *Txt_Alphabetic_BR_code_BR_ISO_3166_1 =
#if   L==1	// ca
	"Codi<br />alfab&egrave;tic<br />ISO 3166-1";
#elif L==2	// de
	"Alphabetischer<br />Code<br />ISO 3166-1";
#elif L==3	// en
	"Alphabetic<br />code<br />ISO 3166-1";
#elif L==4	// es
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#elif L==5	// fr
	"Code<br />alphab&eacute;tique<br />ISO 3166-1";
#elif L==6	// gn
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==7	// it
	"Codice<br />alfabetico<br />ISO 3166-1";
#elif L==8	// pl
	"Alfabetycznego<br />kodu<br />ISO 3166-1";
#elif L==9	// pt
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#elif L==10	// tr
	"Alphabetic<br />code<br />ISO 3166-1";			// eviri lazim!
#endif

const char *Txt_Already_existed_a_criterion_in_this_rubric_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un criteri en aquesta r&uacute;brica amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"In dieser Rubrik existierte bereits ein Kriterium mit dem Titel <strong>%s</strong>.";
#elif L==3	// en
	"Already existed a criterion in this rubric with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un criterio en esta r&uacute;brica con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un crit&egrave;re dans cette rubrique avec le titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un criterio en esta r&uacute;brica con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un criterio in questa rubrica con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"By&lstrok;o ju&zdot; kryterium w tej rubryce z tytu&lstrok;em <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existia um crit&eacute;rio nesta rubrica com o t&iacute;tulo <strong>%s</strong>.";
#elif L==10	// tr
	"Bu de&gbreve;erlendirme tablosunda <strong>%s</strong> ba&scedil;l&inodot;kl&inodot; bir &ouml;l&ccedil;&uuml;t zaten vard&inodot;.";
#endif

const char *Txt_Already_existed_a_game_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un joc amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Spiel mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed a game with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un juego con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un jeu du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un juego con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un gioco con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"By&lstrok;o ju&zdot; gra z tytu&lstrok;em <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um jogo com o t&iacute;tulo <strong>%s</strong>.";
#elif L==10	// tr
	"Already existed a game with the title <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_Already_existed_a_project_with_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un projecte amb la carpeta <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Projekt im Verzeichnis <strong>%s</strong>.";
#elif L==3	// en
	"There is already a project with folder <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un proyecto con la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un projet avec le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un proyecto con la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un progetto con la cartella <strong>%s</strong>.";
#elif L==8	// pl
	"Istnia&lstrok; projekt z folderem <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um projeto com o diret&oacute;rio <strong>%s</strong>.";
#elif L==10	// tr
	"There is already a project with folder <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_Already_existed_a_set_of_questions_in_this_exam_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un conjunt de preguntes en aquest examen amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"In dieser Pr&uuml;fung gab es bereits eine Reihe von Fragen mit dem Titel <strong>%s</strong>.";
#elif L==3	// en
	"Already existed a set of questions in this exam with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un conjunto de preguntas en este examen con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un ensemble de questions dans cet examen avec le titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un conjunto de preguntas en este examen con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"In questo esame esisteva gi&agrave; un set di domande con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"Na tym egzaminie istnia&lstrok; ju&zdot; zestaw pyta&nacute; o tytule <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existia um conjunto de perguntas neste exame com o t&iacute;tulo <strong>%s</strong>.";
#elif L==10	// tr
	"Already existed a set of questions in this exam with the title <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_Already_existed_a_survey_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia una enquesta amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Umfrage mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed a survey with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un sondage du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un sondaggio con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"By&lstrok;o ju&zdot; ankieta z tytu&lstrok;em <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um inqu&eacute;rito com o t&iacute;tulo <strong>%s</strong>.";
#elif L==10	// tr
	"Already existed a survey with the title <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_Already_existed_an_assignment_with_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia una activitat amb la carpeta <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Aufgabe im Verzeichnis <strong>%s</strong>.";
#elif L==3	// en
	"There is already an assignment with folder <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; une activit&eacute; avec le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un'attivit&agrave; con la cartella <strong>%s</strong>.";
#elif L==8	// pl
	"Jest juz zadanie z folderu <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe uma atividade com o diret&oacute;rio <strong>%s</strong>.";
#elif L==10	// tr
	"There is already an assignment with folder <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_Already_existed_an_assignment_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia una activitat amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Aufgabe mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed an assignment with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; une activit&eacute; du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un'attivit&agrave; con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"Istniala juz przydzialu z tytulu <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe uma atividade com o t&iacute;tulo <strong>%s</strong>.";
#elif L==10	// tr
	"Already existed an assignment with the title <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_Already_existed_an_event_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un esdeveniment amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Ereignis mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed an event with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un &eacute;v&eacute;nement du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un evento con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"Istniala juz wydarzenie z tytulu <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um evento com o t&iacute;tulo <strong>%s</strong>.";
#elif L==10	// tr
	"Already existed an event with the title <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_Already_existed_an_exam_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un examen amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Pr&uuml;fung mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed an exam with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un examen con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un examen du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un examen con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un esame con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"Istniala juz egzamin z tytulu <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um exame com o t&iacute;tulo <strong>%s</strong>.";
#elif L==10	// tr
	"Already existed an exam with the title <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ja existeix.";
#elif L==2	// de
	"<strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"<strong>%s</strong> already exists.";
#elif L==4	// es
	"<strong>%s</strong> ya existe.";
#elif L==5	// fr
	"<strong>%s</strong> existe d&eacute;j&grave;.";
#elif L==6	// gn
	"<strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"<strong>%s</strong> ju&zdot; istnieje.";
#elif L==9	// pt
	"<strong>%s</strong> j&aacute; existe.";
#elif L==10	// tr
	"<strong>%s</strong> already exists.";		// eviri lazim!
#endif

const char *Txt_Altitude =
#if   L==1	// ca
	"Altitud";
#elif L==2	// de
	"H&ouml;he";
#elif L==3	// en
	"Altitude";
#elif L==4	// es
	"Altitud";
#elif L==5	// fr
	"Altitude";
#elif L==6	// gn
	"Altitud";	// Okoteve traduccin
#elif L==7	// it
	"Altitudine";
#elif L==8	// pl
	"Wysoko&sacute;&cacute; bezwzgl&eogon;dna";
#elif L==9	// pt
	"Altitude";
#elif L==10	// tr
	"Altitude";	// eviri lazim!
#endif

const char *Txt_and =
#if   L==1	// ca
	"i";
#elif L==2	// de
	"und";
#elif L==3	// en
	"and";
#elif L==4	// es
	"y";
#elif L==5	// fr
	"et";
#elif L==6	// gn
	"y";	// Okoteve traduccin
#elif L==7	// it
	"e";
#elif L==8	// pl
	"i";
#elif L==9	// pt
	"e";
#elif L==10	// tr
	"and";	// eviri lazim!
#endif

const char *Txt_and_X_other_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"&hellip;i altres %u destinataris";
#elif L==2	// de
	"&hellip;und %u andere Empf&auml;nger";
#elif L==3	// en
	"&hellip;and %u other recipients";
#elif L==4	// es
	"&hellip;y otros %u destinatarios";
#elif L==5	// fr
	"&hellip;et %u autres destinataires";
#elif L==6	// gn
	"&hellip;y otros %u destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;e altri %u destinatari";
#elif L==8	// pl
	"&hellip;i %u innych odbiorc&oacute;w";
#elif L==9	// pt
	"&hellip;e %u outros destinat&aacute;rios";
#elif L==10	// tr
	"&hellip;and %u other recipients";	// eviri lazim!
#endif

const char *Txt_Announcements =
#if   L==1	// ca
	"Anuncis";
#elif L==2	// de
	"Bekanntmachungen";
#elif L==3	// en
	"Announcements";
#elif L==4	// es
	"Anuncios";
#elif L==5	// fr
	"Annonces";
#elif L==6	// gn
	"Anuncios";	// Okoteve traduccin
#elif L==7	// it
	"Annunci";
#elif L==8	// pl
	"Og&lstrok;oszenia";
#elif L==9	// pt
	"An&uacute;ncios";
#elif L==10	// tr
	"Duyurular";
#endif

const char *Txt_Another_building =
#if   L==1	// ca
	"Un altre edifici";
#elif L==2	// de
	"Eine weitere Geb&auml;ude";
#elif L==3	// en
	"Another building";
#elif L==4	// es
	"Otro edificio";
#elif L==5	// fr
	"Un autre b&acirc;timent";
#elif L==6	// gn
	"Otro edificio";	// Okoteve traduccin
#elif L==7	// it
	"Un altro edificio";
#elif L==8	// pl
	"Kolejna budynek";
#elif L==9	// pt
	"Outro edif&iacute;cio";
#elif L==10	// tr
	"Another building";	// eviri lazim!
#endif

const char *Txt_Another_center =
#if   L==1	// ca
	"Un altre center";
#elif L==2	// de
	"Ein weiteres Lehrinstitut";
#elif L==3	// en
	"Another center";
#elif L==4	// es
	"Otro centro";
#elif L==5	// fr
	"Un autre center";
#elif L==6	// gn
	"Otro centro";		// Okoteve traduccin
#elif L==7	// it
	"Un altro centro";
#elif L==8	// pl
	"Innym centrum";
#elif L==9	// pt
	"Outro centro";
#elif L==10	// tr
	"Another center";	// eviri lazim!
#endif

const char *Txt_Another_room =
#if   L==1	// ca
	"Una altra sala";
#elif L==2	// de
	"Eine weitere Raum";
#elif L==3	// en
	"Another room";
#elif L==4	// es
	"Otra sala";
#elif L==5	// fr
	"Une autre salle";
#elif L==6	// gn
	"Otra sala";	// Okoteve traduccin
#elif L==7	// it
	"Un'altra aula";
#elif L==8	// pl
	"Kolejna klasa";
#elif L==9	// pt
	"Outra sala";
#elif L==10	// tr
	"Another room";	// eviri lazim!
#endif

const char *Txt_Another_country =
#if   L==1	// ca
	"Un altre pa&iacute;s";
#elif L==2	// de
	"Ein weiterer Land";
#elif L==3	// en
	"Another country";
#elif L==4	// es
	"Otro pa&iacute;s";
#elif L==5	// fr
	"Un autre pays";
#elif L==6	// gn
	"Otro pa&iacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Un altro paese";
#elif L==8	// pl
	"Kolejny kraj";
#elif L==9	// pt
	"Outro pa&iacute;s";
#elif L==10	// tr
	"Another country";	// eviri lazim!
#endif

const char *Txt_Another_department =
#if   L==1	// ca
	"Un altre departament";
#elif L==2	// de
	"Eine weitere Abteilung";
#elif L==3	// en
	"Another department";
#elif L==4	// es
	"Otro departamento";
#elif L==5	// fr
	"Une autre d&eacute;partement";
#elif L==6	// gn
	"Otro departamento";	// Okoteve traduccin
#elif L==7	// it
	"Un altro dipartimento";
#elif L==8	// pl
	"Kolejny dzia&lstrok;";
#elif L==9	// pt
	"Outro departamento";
#elif L==10	// tr
	"Another department";	// eviri lazim!
#endif

const char *Txt_Another_ID =
#if   L==1	// ca
	"Un altre ID";
#elif L==2	// de
	"Ein anderer Ausweis-Nr.";
#elif L==3	// en
	"Another ID";
#elif L==4	// es
	"Otro ID";
#elif L==5	// fr
	"Un autre num&eacute;ro d'identit&eacute;";
#elif L==6	// gn
	"Otro ID";	// Okoteve traduccin
#elif L==7	// it
	"Un'altra carta d'identit&agrave;";
#elif L==8	// pl
	"Kolejny dow&oacute;d to&zdot;samo&sacute;ci";
#elif L==9	// pt
	"Outro n&ordm; de identif.";
#elif L==10	// tr
	"Ba&scedil;ka bir kimlik";
#endif

const char *Txt_Another_institution =
#if   L==1	// ca
	"Una altra instituci&oacute;";
#elif L==2	// de
	"Eine weitere Hochschule";
#elif L==3	// en
	"Another institution";
#elif L==4	// es
	"Otra instituci&oacute;n";
#elif L==5	// fr
	"Un autre &eacute;tablissement";
#elif L==6	// gn
	"Otra instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Un'altra istituzione";
#elif L==8	// pl
	"Kolejna instytucja";
#elif L==9	// pt
	"Outra institui&ccedil;&atilde;o";
#elif L==10	// tr
	"Another institution";		// eviri lazim!
#endif

const char *Txt_Another_place =
#if   L==1	// ca
	"Un altre lloc";
#elif L==2	// de
	"Ein weiterer Standort";
#elif L==3	// en
	"Another place";
#elif L==4	// es
	"Otro lugar";
#elif L==5	// fr
	"Un autre emplacement";
#elif L==6	// gn
	"Otro lugar";		// Okoteve traduccin
#elif L==7	// it
	"Un'altra localit&grave;";
#elif L==8	// pl
	"Innym miejscem";
#elif L==9	// pt
	"Outra localiza&ccedil;&atilde;o";
#elif L==10	// tr
	"Another place";	// eviri lazim!
#endif

const char *Txt_Another_user_s_profile =
#if   L==1	// ca
	"Perfil d'un altre usuari";
#elif L==2	// de
	"Profil eines anderen Benutzers";
#elif L==3	// en
	"Another user's profile";
#elif L==4	// es
	"Perfil de otro usuario";
#elif L==5	// fr
	"Profil d'un autre utilisateur";
#elif L==6	// gn
	"Perfil de otro usuario";	// Okoteve traduccin
#elif L==7	// it
	"Altro profilo di utente";
#elif L==8	// pl
	"Profil innego u&zdot;ytkownika";
#elif L==9	// pt
	"Perfil de outro usu&aacute;rio";
#elif L==10	// tr
	"Another user's profile";	// eviri lazim!
#endif

const char *Txt_Announcement =
#if   L==1	// ca
	"Anunci";
#elif L==2	// de
	"Bekanntmachung";
#elif L==3	// en
	"Announcement";
#elif L==4	// es
	"Anuncio";
#elif L==5	// fr
	"Annonce";
#elif L==6	// gn
	"Anuncio";		// Okoteve traduccin
#elif L==7	// it
	"Annuncio";
#elif L==8	// pl
	"Og&lstrok;oszenia";
#elif L==9	// pt
	"An&uacute;ncio";
#elif L==10	// tr
	"Announcement";		// eviri lazim!
#endif

const char *Txt_Announcement_created =
#if   L==1	// ca
	"Anunci creat.";
#elif L==2	// de
	"Bekanntmachung erstellt.";
#elif L==3	// en
	"Announcement created.";
#elif L==4	// es
	"Anuncio creado.";
#elif L==5	// fr
	"Annonce cr&eacute;&eacute;.";
#elif L==6	// gn
	"Anuncio creado.";		// Okoteve traduccin
#elif L==7	// it
	"Annuncio creato.";
#elif L==8	// pl
	"Og&lstrok;oszenia stworzony.";
#elif L==9	// pt
	"An&uacute;ncio criado.";
#elif L==10	// tr
	"Announcement created.";	// eviri lazim!
#endif

const char *Txt_Announcement_removed =
#if   L==1	// ca
	"Anunci eliminat.";
#elif L==2	// de
	"Bekanntmachung entfernt.";
#elif L==3	// en
	"Announcement removed.";
#elif L==4	// es
	"Anuncio eliminado.";
#elif L==5	// fr
	"Annonce supprim&eacute;.";
#elif L==6	// gn
	"Anuncio eliminado.";		// Okoteve traduccin
#elif L==7	// it
	"Annuncio rimosso.";
#elif L==8	// pl
	"Og&lstrok;oszenia usuniete.";
#elif L==9	// pt
	"An&uacute;ncio removido.";
#elif L==10	// tr
	"Announcement removed.";	// eviri lazim!
#endif

const char *Txt_Answer_survey =
#if   L==1	// ca
	"Respondre enquesta";
#elif L==2	// de
	"Beantwortung Umfrage";
#elif L==3	// en
	"Answer survey";
#elif L==4	// es
	"Responder encuesta";
#elif L==5	// fr
	"R&eacute;pondre sondage";
#elif L==6	// gn
	"Responder encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Rispondere sondaggio";
#elif L==8	// pl
	"Badanie odpowiedzi";
#elif L==9	// pt
	"Responder inqu&eacute;rito";
#elif L==10	// tr
	"Answer survey";	// eviri lazim!
#endif

const char *Txt_Answers =		// Answers of test
#if   L==1	// ca
	"Respostes";
#elif L==2	// de
	"Antworten";
#elif L==3	// en
	"Answers";
#elif L==4	// es
	"Respuestas";
#elif L==5	// fr
	"R&eacute;ponses";
#elif L==6	// gn
	"&Ntilde;embohov&aacute;i";
#elif L==7	// it
	"Risposte";
#elif L==8	// pl
	"Odpowiedzi";
#elif L==9	// pt
	"Respostas";
#elif L==10	// tr
	"Answers";	// eviri lazim!
#endif

const char *Txt_ANSWERS_blank =
#if   L==1	// ca
	"en&nbsp;blanc";
#elif L==2	// de
	"leere";
#elif L==3	// en
	"blank";
#elif L==4	// es
	"en&nbsp;blanco";
#elif L==5	// fr
	"vides";
#elif L==6	// gn
	"en&nbsp;blanco";	// Okoteve traduccin
#elif L==7	// it
	"vuote";
#elif L==8	// pl
	"puste";
#elif L==9	// pt
	"em&nbsp;branco";
#elif L==10	// tr
	"blank";		// eviri lazim!
#endif

const char *Txt_ANSWERS_correct =
#if   L==1	// ca
	"correctes";
#elif L==2	// de
	"richtige";
#elif L==3	// en
	"correct";
#elif L==4	// es
	"correctas";
#elif L==5	// fr
	"bonnes";
#elif L==6	// gn
	"correctas";	// Okoteve traduccin
#elif L==7	// it
	"corrette";
#elif L==8	// pl
	"prawid&lstrok;owe";
#elif L==9	// pt
	"corretas";
#elif L==10	// tr
	"correct";	// eviri lazim!
#endif

const char *Txt_ANSWERS_non_blank =
#if   L==1	// ca
	"contestades";
#elif L==2	// de
	"non-blank";
#elif L==3	// en
	"non-blank";
#elif L==4	// es
	"contestadas";
#elif L==5	// fr
	"non vide";
#elif L==6	// gn
	"contestadas";	// Okoteve traduccin
#elif L==7	// it
	"risposte";
#elif L==8	// pl
	"niepustych";
#elif L==9	// pt
	"respondidas";
#elif L==10	// tr
	"non-blank";	// eviri lazim!
#endif

const char *Txt_ANSWERS_wrong =
#if   L==1	// ca
	"incorrectes";
#elif L==2	// de
	"falsche";
#elif L==3	// en
	"wrong";
#elif L==4	// es
	"err&oacute;neas";
#elif L==5	// fr
	"mauvaises";
#elif L==6	// gn
	"err&oacute;neas";	// Okoteve traduccin
#elif L==7	// it
	"sbagliate";
#elif L==8	// pl
	"z&lstrok;e";
#elif L==9	// pt
	"erradas";
#elif L==10	// tr
	"wrong";		// eviri lazim!
#endif

const char *Txt_Any_action =
#if   L==1	// ca
	"Qualsevol acci&oacute;";
#elif L==2	// de
	"Alle Handlung";
#elif L==3	// en
	"Any action";
#elif L==4	// es
	"Cualquier acci&oacute;n";
#elif L==5	// fr
	"Toute action";
#elif L==6	// gn
	"Cualquier acci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Qualsiasi azione";
#elif L==8	// pl
	"Wszystkie  dzia&lstrok;ania";
#elif L==9	// pt
	"Qualquer a&ccedil;&atilde;o";
#elif L==10	// tr
	"Any action";			// eviri lazim!
#endif

const char *Txt_any_course =
#if   L==1	// ca
	"qualsevol assignatura";
#elif L==2	// de
	"alle Kursen";
#elif L==3	// en
	"any course";
#elif L==4	// es
	"cualquier asignatura";
#elif L==5	// fr
	"toute mati&egrave;re";
#elif L==6	// gn
	"cualquier asignatura";	// Okoteve traduccin
#elif L==7	// it
	"qualsiasi corso";
#elif L==8	// pl
	"kurs";
#elif L==9	// pt
	"qualquer disciplina";
#elif L==10	// tr
	"any course";		// eviri lazim!
#endif

const char *Txt_Any_department =
#if   L==1	// ca
	"Qualsevol departament";
#elif L==2	// de
	"Alle Abteilung";
#elif L==3	// en
	"Any department";
#elif L==4	// es
	"Cualquier departamento";
#elif L==5	// fr
	"Tout d&eacute;partement";
#elif L==6	// gn
	"Cualquier departamento";	// Okoteve traduccin
#elif L==7	// it
	"Qualsiasi dipartimento";
#elif L==8	// pl
	"Wszystkie dzia&lstrok;y";
#elif L==9	// pt
	"Qualquer departamento";
#elif L==10	// tr
	"Any department";		// eviri lazim!
#endif

const char *Txt_Any_type_of_degree =
#if   L==1	// ca
	"Qualsevol tipus de titulaci&oacute;";
#elif L==2	// de
	"Alle Abschlussart";
#elif L==3	// en
	"Any type of degree";
#elif L==4	// es
	"Cualquier tipo de titulaci&oacute;n";
#elif L==5	// fr
	"Tout type d'&eacute;tude";
#elif L==6	// gn
	"Cualquier tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Qualsiasi tipo di laurea";
#elif L==8	// pl
	"Dowolny rodzaj stopnia";
#elif L==9	// pt
	"Qualquer tipo de grau";
#elif L==10	// tr
	"Any type of degree";			// eviri lazim!
#endif

const char *Txt_any_user =
#if   L==1	// ca
	"qualsevol usuari";
#elif L==2	// de
	"alle Benutzer";
#elif L==3	// en
	"any user";
#elif L==4	// es
	"cualquier usuario";
#elif L==5	// fr
	"n'importe qu'utilisateur";
#elif L==6	// gn
	"cualquier usuario";	// Okoteve traduccin
#elif L==7	// it
	"qualsiasi utente";
#elif L==8	// pl
	"wszystkich uzytkownik&oacute;w";
#elif L==9	// pt
	"qualquer utilizador";
#elif L==10	// tr
	"any user";		// eviri lazim!
#endif

const char *Txt_Application_key =
#if   L==1	// ca
	"Clau de aplicaci&oacute;";
#elif L==2	// de
	"Anwendungsschl&uuml;ssel";
#elif L==3	// en
	"Application key";
#elif L==4	// es
	"Clave de aplicaci&oacute;n";
#elif L==5	// fr
	"Cl&eacute; d'application";
#elif L==6	// gn
	"Clave de aplicaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Codice di applicazione";
#elif L==8	// pl
	"Klawisz aplikacji";
#elif L==9	// pt
	"Chave de aplica&ccedil;&atilde;o";
#elif L==10	// tr
	"Application key";		// eviri lazim!
#endif

const char *Txt_Assessment =
#if   L==1	// ca
	"Avaluaci&oacute;";
#elif L==2	// de
	"Bewertung";
#elif L==3	// en
	"Assessment";
#elif L==4	// es
	"Evaluaci&oacute;n";
#elif L==5	// fr
	"&Eacute;valuation";
#elif L==6	// gn
	"Mbohepy";
#elif L==7	// it
	"Valutazione";
#elif L==8	// pl
	"Ocena";
#elif L==9	// pt
	"Avalia&ccedil;&atilde;o";
#elif L==10	// tr
	"De&gbreve; erlendirme";
#endif

const char *Txt_Assessment_criteria =
#if   L==1	// ca
	"Criteris d'avaluaci&oacute;";
#elif L==2	// de
	"Bewertungskriterien";
#elif L==3	// en
	"Assessment criteria";
#elif L==4	// es
	"Criterios de evaluaci&oacute;n";
#elif L==5	// fr
	"Crit&egrave;res d'&eacute;valuation";
#elif L==6	// gn
	"Criterios de evaluaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Criteri di valutazione";
#elif L==8	// pl
	"Kryteria oceny";
#elif L==9	// pt
	"Crit&eacute;rios de avalia&ccedil;&atilde;o";
#elif L==10	// tr
	"Assessment criteria";			// eviri lazim!
#endif

const char *Txt_Assigned_QUESTION =
#if   L==1	// ca
	"Assignat?";
#elif L==2	// de
	"Vorbelegt?";
#elif L==3	// en
	"Assigned?";
#elif L==4	// es
	"&iquest;Asignado?";
#elif L==5	// fr
	"Assign&eacute;?";
#elif L==6	// gn
	"&iquest;Asignado?";	// Okoteve traduccin
#elif L==7	// it
	"Assegnato?";
#elif L==8	// pl
	"Zadany?";
#elif L==9	// pt
	"Atribu&iacute;do?";
#elif L==10	// tr
	"Atand&inodot; m&inodot;?";
#endif

const char *Txt_Assignment =
#if   L==1	// ca
	"Activitat";
#elif L==2	// de
	"Aufgabe";
#elif L==3	// en
	"Assignment";
#elif L==4	// es
	"Actividad";
#elif L==5	// fr
	"Activit&eacute;";
#elif L==6	// gn
	"Tar&eacute;a";
#elif L==7	// it
	"Attivit&agrave;";
#elif L==8	// pl
	"Przyporzadkowanie";
#elif L==9	// pt
	"Atividade";
#elif L==10	// tr
	"Assignment";	// eviri lazim!
#endif

const char *Txt_Assignment_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Activitat <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Aufgabe <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Assignment <strong>%s</strong> removed.";
#elif L==4	// es
	"Actividad <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Activit&eacute; <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"Actividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Attivit&agrave; <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Zadanie <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Atividade <strong>%s</strong> removida.";
#elif L==10	// tr
	"Assignment <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_Assignments =
#if   L==1	// ca
	"Activitats";
#elif L==2	// de
	"Aufgaben";
#elif L==3	// en
	"Assignments";
#elif L==4	// es
	"Actividades";
#elif L==5	// fr
	"Activit&eacute;s";
#elif L==6	// gn
	"Tar&eacute;a";
#elif L==7	// it
	"Attivit&agrave;";
#elif L==8	// pl
	"Zadania";
#elif L==9	// pt
	"Atividades";
#elif L==10	// tr
	"&Ouml;devler";
#endif

const char *Txt_Assignments_and_other_works =
#if   L==1	// ca
	"Activitats i altres treballs";
#elif L==2	// de
	"Aufgaben und andere Hausarbeiten";
#elif L==3	// en
	"Assignments and other works";
#elif L==4	// es
	"Actividades y otros trabajos";		// Okoteve traduccin
#elif L==5	// fr
	"Activit&eacute;s et autres travaux";
#elif L==6	// gn
	"Actividades y otros trabajos";
#elif L==7	// it
	"Attivit&agrave; e altri lavori";
#elif L==8	// pl
	"Assignments and other works";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Atividades e outros trabalhos";
#elif L==10	// tr
	"Assignments and other works";		// eviri lazim!
#endif

const char *Txt_START_END_TIME_HELP[Dat_NUM_START_END_TIME] =
	{
 	[Dat_STR_TIME] =
#if   L==1	// ca
	"Ordenar per data de comen&ccedil;ament"
#elif L==2	// de
	"Sortieren nach Startdatum"
#elif L==3	// en
	"Sort by start date"
#elif L==4	// es
	"Ordenar por fecha de comienzo"
#elif L==5	// fr
	"Trier par date de d&eacute;but"
#elif L==6	// gn
	"Ordenar por fecha de comienzo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data di inizio"
#elif L==8	// pl
	"Sortuj wedlug daty rozpoczecia"
#elif L==9	// pt
	"Classificar por data de in&iacute;cio"
#elif L==10	// tr
	"Sort by start date"		// eviri lazim!
#endif
	,
	[Dat_END_TIME] =
#if   L==1	// ca
	"Ordenar per data de finalitzaci&oacute;"
#elif L==2	// de
	"Sortieren nach Enddatum"
#elif L==3	// en
	"Sort by end date"
#elif L==4	// es
	"Ordenar por fecha de finalizaci&oacute;n"
#elif L==5	// fr
	"Trier par date de fin"
#elif L==6	// gn
	"Ordenar por fecha de finalizaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data di fine"
#elif L==8	// pl
	"Sortuj wedlug daty zakonczenia"
#elif L==9	// pt
	"Classificar por data de t&eacute;rmino"
#elif L==10	// tr
	"Sort by end date"				// eviri lazim!
#endif
	};

const char *Txt_START_END_TIME[Dat_NUM_START_END_TIME] =
	{
 	[Dat_STR_TIME] =
#if   L==1	// ca
	"Inici"
#elif L==2	// de
	"Start"
#elif L==3	// en
	"Start"
#elif L==4	// es
	"Inicio"
#elif L==5	// fr
	"D&eacute;but"
#elif L==6	// gn
	"&Ntilde;epyr&utilde;"
#elif L==7	// it
	"Inizio"
#elif L==8	// pl
	"Pocz&aogon;tek"
#elif L==9	// pt
	"In&iacute;cio"
#elif L==10	// tr
	"Ba&scedil;lama"
#endif
	,
	[Dat_END_TIME] =
#if   L==1	// ca
	"Final"
#elif L==2	// de
	"Ende"
#elif L==3	// en
	"End"
#elif L==4	// es
	"Final"
#elif L==5	// fr
	"Fin"
#elif L==6	// gn
	"Paha"
#elif L==7	// it
	"Fine"
#elif L==8	// pl
	"Koniec"
#elif L==9	// pt
	"Fim"
#elif L==10	// tr
	"Son"
#endif
	};

const char *Txt_Assignments_area =
#if   L==1	// ca
	"Zona d'activitats";
#elif L==2	// de
	"Aufgaben-Bereich";
#elif L==3	// en
	"Assignments area";
#elif L==4	// es
	"Zona de actividades";
#elif L==5	// fr
	"Zone d'activit&eacute;s";
#elif L==6	// gn
	"Zona de actividades";	// Okoteve traduccin
#elif L==7	// it
	"Area di attivit&agrave;";
#elif L==8	// pl
	"Obszarze Zadania";
#elif L==9	// pt
	"Zona de atividades";
#elif L==10	// tr
	"Assignments area";	// eviri lazim!
#endif

const char *Txt_assignments_area =
#if   L==1	// ca
	"zona d'activitats";
#elif L==2	// de
	"Aufgaben-Bereich";
#elif L==3	// en
	"assignments area";
#elif L==4	// es
	"zona de actividades";
#elif L==5	// fr
	"zone d'activit&eacute;s";
#elif L==6	// gn
	"zona de actividades";	// Okoteve traduccin
#elif L==7	// it
	"area di attivit&agrave;";
#elif L==8	// pl
	"obszarze zadan";
#elif L==9	// pt
	"zona de atividades";
#elif L==10	// tr
	"assignments area";	// eviri lazim!
#endif

const char *Txt_Attendance =	// Number of times a students has attended to face-to-face class
#if   L==1	// ca
	"Assist&egrave;ncia";
#elif L==2	// de
	"Anwesenheit";
#elif L==3	// en
	"Attendance";
#elif L==4	// es
	"Asistencia";
#elif L==5	// fr
	"Pr&eacute;sence";
#elif L==6	// gn
	"Asistencia";
#elif L==7	// it
	"Presenza";
#elif L==8	// pl
	"Bytno&sacute;&cacute;";
#elif L==9	// pt
	"Presen&ccedil;a";
#elif L==10	// tr
	"Kat&inodot;l&inodot;m";
#endif

const char *Txt_Attendance_list =
#if   L==1	// ca
	"Llista assistencia";
#elif L==2	// de
	"Anwesenheitsliste";
#elif L==3	// en
	"Attendance list";
#elif L==4	// es
	"Lista asistencia";
#elif L==5	// fr
	"Liste fr&eacute;quentation";
#elif L==6	// gn
	"Lista asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Elenco presenze";
#elif L==8	// pl
	"Lista obecno&sacute;ci";
#elif L==9	// pt
	"Lista assiduidade";
#elif L==10	// tr
	"Attendance list";	// eviri lazim!
#endif

const char *Txt_Availability =
#if   L==1	// ca
	"Disponibilitat";
#elif L==2	// de
	"Verf&uuml;gbarkeit";
#elif L==3	// en
	"Availability";
#elif L==4	// es
	"Disponibilidad";
#elif L==5	// fr
	"Disponibilit&eacute;";
#elif L==6	// gn
	"Disponibilidad";	// Okoteve traduccin
#elif L==7	// it
	"Disponibilit&agrave;";
#elif L==8	// pl
	"Dost&eogon;pno&sacute;&cacute;";
#elif L==9	// pt
	"Disponibilidade";
#elif L==10	// tr
	"Bulunma";
#endif

const char *Txt_average =
#if   L==1	// ca
	"mitjana";
#elif L==2	// de
	"Mittel";
#elif L==3	// en
	"average";
#elif L==4	// es
	"media";
#elif L==5	// fr
	"moyenne";
#elif L==6	// gn
	"media";	// Okoteve traduccin
#elif L==7	// it
	"media";
#elif L==8	// pl
	"&sacute;rednia";
#elif L==9	// pt
	"m&eacute;dia";
#elif L==10	// tr
	"ortalama";
#endif

const char *Txt_Average_number_BR_of_ASSIG_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />d'activitats<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Aufgaben<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of assignments<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de actividades<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />d'activit&eacute;s<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de actividades<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di attivit&agrave;<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />zadania<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de atividades<br />por disciplina";
#elif L==10	// tr
	"Average number<br />of assignments<br />per course";		// eviri lazim!
#endif

const char *Txt_Average_number_of_courses_to_which_a_user_belongs =
#if   L==1	// ca
	"Nombre mitj&agrave; d'assignatures a las quals pertany cada usuari";
#elif L==2	// de
	"Durchschnittliche Anzahl von Kursen, an denen der Benutzer teilnimmt";
#elif L==3	// en
	"Average number of courses to which a user belongs";
#elif L==4	// es
	"N.&ordm; medio de asignaturas a las que pertenece cada usuario";
#elif L==5	// fr
	"Nombre moyen de mati&egrave;res &agrave; qui appartient un utilisateur";
#elif L==6	// gn
	"N.&ordm; medio de asignaturas a las que pertenece cada usuario";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio di corsi a cui appartiene un utente";
#elif L==8	// pl
	"&Sacute;rednia liczba kurs&oacute;w do kt&oacute;rej nalezy uzytkownik";
#elif L==9	// pt
	"N.&ordm; medio de disciplinas em que cada utilizador pertence";
#elif L==10	// tr
	"Average number of courses to which a user belongs";			// eviri lazim!
#endif

const char *Txt_Average_number_BR_of_criteria_BR_per_rubric =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />de criteris<br />per r&uacute;brica";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Kriterien<br />pro Rubrik";
#elif L==3	// en
	"Average number<br />of criteria<br />per rubric";
#elif L==4	// es
	"N.&ordm; medio<br />de criterios<br />por r&uacute;brica";
#elif L==5	// fr
	"Nombre moyen<br />de crit&egrave;res<br />par rubrique";
#elif L==6	// gn
	"N.&ordm; medio<br />de criterios<br />por r&uacute;brica";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di criteri<br />per rubrica";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />kryteri&oacute;w<br />na rubryk&eogon;";
#elif L==9	// pt
	"N.&ordm; medio<br />de crit&eacute;rios<br />por rubrica";
#elif L==10	// tr
	"Dereceli puanlama<br />anahtar&inodot; ba&scedil;&inodot;na<br />ortalama &ouml;l&ccedil;&uuml;t say&inodot;s&inodot;";
#endif

const char *Txt_Average_number_BR_of_exams_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />d'ex&agrave;mens<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Pr&uuml;fung<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of exams<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de ex&aacute;menes<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />d'examens<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de ex&aacute;menes<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />d'esame<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />egzamin&oacute;w<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de exames<br />por disciplina";
#elif L==10	// tr
	"Average number<br />of exams<br />per course";			// eviri lazim!
#endif

const char *Txt_Average_number_BR_of_games_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />de jocs<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Spiele<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of games<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de juegos<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de jeux<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de juegos<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di giochi<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />gry<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de jogos<br />por disciplina";
#elif L==10	// tr
	"Average number<br />of games<br />per course";		// eviri lazim!
#endif

const char *Txt_Average_number_BR_of_items_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />d'items<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Programmelemente<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of items<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de items<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />d'&eacute;l&eacute;ments<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de items<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di elementi<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />pozycji<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de itens<br />por disciplina";
#elif L==10	// tr
	"Average number<br />of items<br />per course";		// eviri lazim!
#endif

const char *Txt_Average_number_BR_of_projects_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />de projectes<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Projekte<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of projects<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de proyectos<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de projets<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de proyectos<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di progetti<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />projekty<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de projetos<br />por disciplina";
#elif L==10	// tr
	"Average number<br />of projects<br />per course";	// eviri lazim!
#endif

const char *Txt_Average_number_BR_of_questions_BR_per_survey =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />de preguntes<br />per enquesta";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Fragen<br />pro Umfrage";
#elif L==3	// en
	"Average number<br />of questions<br />per survey";
#elif L==4	// es
	"N.&ordm; medio<br />de preguntas<br />por encuesta";
#elif L==5	// fr
	"Nombre moyen<br />de questions<br />par sondage";
#elif L==6	// gn
	"N.&ordm; medio<br />de preguntas<br />por encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di domande<br />per sondaggio";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />pyta&nacute;<br />na ankiety";
#elif L==9	// pt
	"N.&ordm; medio<br />de perguntas<br />por inqu&eacute;rito";
#elif L==10	// tr
	"Average number<br />of questions<br />per survey";	// eviri lazim!
#endif

const char *Txt_Average_number_BR_of_rubrics_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />de r&uacute;briques<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Rubriken<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of rubrics<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de r&uacute;bricas<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de rubriques<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de r&uacute;bricas<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di rubriche<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />badania<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de rubricas<br />por disciplina";
#elif L==10	// tr
	"Kurs ba&scedil;&inodot;na<br />ortalama de&gbreve;erlendirme listesi<br />say&inodot;s&inodot;";
#endif

const char *Txt_Average_number_BR_of_surveys_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />d'enquestes<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Umfragen<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of surveys<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de encuestas<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de sondages<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de encuestas<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di sondaggi<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />badania<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de inqu&eacute;ritos<br />por disciplina";
#elif L==10	// tr
	"Kurs ba&scedil;&inodot;na<br />ortalama anket<br />say&inodot;s&inodot;";
#endif

const char *Txt_Average_BR_number_BR_of_test_BR_questions_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&agrave;<br />de preguntes<br />de test per<br />assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Testfragen<br />pro Kurs";
#elif L==3	// en
	"Average<br />number<br />of test<br />questions<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de preguntas<br />de test por<br />asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de questions<br />de test par<br />mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de preguntas<br />de test por<br />asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di domande<br />di test<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />pyta&nacute;<br />testowych<br />na kurs";
#elif L==9	// pt
	"N.&ordm; medio<br />de perguntas<br />de test por<br />disciplina";
#elif L==10	// tr
	"Average<br />number<br />of test<br />questions<br />per course";	// eviri lazim!
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_a_question_BR_has_been_BR_responded =
#if   L==1	// ca
	"Mitj&agrave; de<br />vegades<br />que s'ha<br />respost<br />cada pregunta";
#elif L==2	// de
	"Durchschnittliche<br />H&auml;ufigkeit,<br />mit der<br />eine Frage<br />beantwortet wurde";
#elif L==3	// en
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";
#elif L==4	// es
	"N.&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";
#elif L==5	// fr
	"Nombre moyen<br />de fois<br />qu'une question<br />a re&ccedil;u<br />une r&eacute;ponse";
#elif L==6	// gn
	"N.&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di volte<br />in cui<br />una domanda<br />ha ricevuto<br />risposta";
#elif L==8	// pl
	"&Sacute;rednia<br />liczba<br />odpowiedzi<br />na pytanie";
#elif L==9	// pt
	"N.&ordm; m&eacute;dio<br />de vezes<br />que uma<br />pergunta<br />foi respondida";
#elif L==10	// tr
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// eviri lazim!
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_questions_BR_have_been_BR_responded_BR_per_course =
#if   L==1	// ca
	"Mitj&agrave; de<br />vegades<br />que s'ha<br />respost<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche<br />H&auml;ufigkeit,<br />mit der<br />eine Frage<br />pro Kurs<br />beantwortet wurde";
#elif L==3	// en
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";
#elif L==4	// es
	"N.&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de fois<br />qu'une question<br />a re&ccedil;u<br />une r&eacute;ponse<br />par mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";			// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di volte<br />in cui<br />una domanda<br />ha ricevuto<br />risposta<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia<br />liczba<br />odpowiedzi<br />na pytanie<br />na kurs";
#elif L==9	// pt
	"N.&ordm; m&eacute;dio<br />de vezes<br />que uma<br />pergunta<br />foi respondida<br />por disciplina";
#elif L==10	// tr
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// eviri lazim!
#endif

const char *Txt_Average_number_of_users_belonging_to_a_course =
#if   L==1	// ca
	"Nombre mitj&agrave; de usuaris que pertanyen a cada assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl der Benutzer, die an einer Kurs teilnehmen";
#elif L==3	// en
	"Average number of users belonging to a course";
#elif L==4	// es
	"N.&ordm; medio de usuarios que pertenecen a cada asignatura";
#elif L==5	// fr
	"Nombre moyen d'utilisateurs appartenant &agrave; une mati&egrave;re";
#elif L==6	// gn
	"N.&ordm; medio de usuarios que pertenecen a cada asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio di utenti appartenenti a un corso";
#elif L==8	// pl
	"&Sacute;rednia liczba uzytkownik&oacute;w nalezacych do kursu";
#elif L==9	// pt
	"N.&ordm; medio de utilizadores que pertencem a uma disciplina";
#elif L==10	// tr
	"Average number of users belonging to a course";		// eviri lazim!
#endif

const char *Txt_AVERAGE_PHOTO_TYPES[Pho_NUM_AVERAGE_PHOTO_TYPES] =
	{
 	[Pho_PHOTO_MEDIAN_ALL] =
#if   L==1	// ca
	"mitjana de totes les fotos"
#elif L==2	// de
	"Median aller Fotos"
#elif L==3	// en
	"median of all photos"
#elif L==4	// es
	"mediana de todas las fotos"
#elif L==5	// fr
	"m&eacute;dian de toutes les photos"
#elif L==6	// gn
	"mediana de todas las fotos"	// Okoteve traduccin
#elif L==7	// it
	"mediana di tutte le foto"
#elif L==8	// pl
	"mediana wszystkich zdjec"
#elif L==9	// pt
	"mediana de todas as fotos"
#elif L==10	// tr
	"median of all photos"		// eviri lazim!
#endif
	,
 	[Pho_PHOTO_AVERAGE_ALL] =
#if   L==1	// ca
	"mitjana aritm&egrave;tica de totes les fotos"
#elif L==2	// de
	"arithmetischer Mittelwert aller Fotos"
#elif L==3	// en
	"arithmetic mean of all photos"
#elif L==4	// es
	"media aritm&eacute;tica de todas las fotos"
#elif L==5	// fr
	"moyenne arithm&eacute;tique de toutes les photos"
#elif L==6	// gn
	"media aritm&eacute;tica de todas las fotos"	// Okoteve traduccin
#elif L==7	// it
	"media aritmetica di tutte le foto"
#elif L==8	// pl
	"&sacute;rednia arytmetyczna wszystkich zdjec"
#elif L==9	// pt
	"m&eacute;dia aritm&eacute;tica de todas as fotos"
#elif L==10	// tr
	"arithmetic mean of all photos"			// eviri lazim!
#endif
	};

const char *Txt_Average_BR_score_BR_per_question =
#if   L==1	// ca
	"Puntuaci&oacute;<br />mitjana<br />per pregunta";
#elif L==2	// de
	"Durchschnittliche<br />Punktzahl<br />pro Frage";
#elif L==3	// en
	"Average<br />score<br />per question";
#elif L==4	// es
	"Puntuaci&oacute;n<br />media<br />por pregunta";
#elif L==5	// fr
	"Score<br />moyen<br />par question";
#elif L==6	// gn
	"Puntuaci&oacute;n<br />media<br />por pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Punteggio<br />medio<br />per domanda";
#elif L==8	// pl
	"&Sacute;rednia<br />ocena<br />za pytanie";
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia por<br />pergunta";
#elif L==10	// tr
	"Average<br />score<br />per question";	// eviri lazim!
#endif

const char *Txt_Average_type =
#if   L==1	// ca
	"Tipus de mitjana";
#elif L==2	// de
	"Art des Mittelwerts";
#elif L==3	// en
	"Average type";
#elif L==4	// es
	"Tipo de promedio";
#elif L==5	// fr
	"Type de moyenne";
#elif L==6	// gn
	"Tipo de promedio";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di media";
#elif L==8	// pl
	"&Sacute;rednia typu";
#elif L==9	// pt
	"Tipo de m&eacute;dia";
#elif L==10	// tr
	"Ortalama t&uuml;r";
#endif

const char *Txt_Banned_users =
#if   L==1	// ca
	"Usuaris bloquejats";
#elif L==2	// de
	"Gebannt Benutzer";
#elif L==3	// en
	"Banned users";
#elif L==4	// es
	"Usuarios bloqueados";
#elif L==5	// fr
	"Utilisateurs bannis";
#elif L==6	// gn
	"Usuarios bloqueados";	// Okoteve traduccin
#elif L==7	// it
	"Utenti vietati";
#elif L==8	// pl
	"Zablokowani u&zdot;ytkownicy";
#elif L==9	// pt
	"Usu&aacute;rios banidos";
#elif L==10	// tr
	"Banned users";		// eviri lazim!
#endif

const char *Txt_Banner =
#if   L==1	// ca
	"Banner";
#elif L==2	// de
	"Werbebanner";
#elif L==3	// en
	"Banner";
#elif L==4	// es
	"Banner";
#elif L==5	// fr
	"Banni&eacute;re";
#elif L==6	// gn
	"Banner";
#elif L==7	// it
	"Banner";
#elif L==8	// pl
	"Banner";
#elif L==9	// pt
	"Banner";
#elif L==10	// tr
	"Afi&scedil;";
#endif

const char *Txt_Banner_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Banner <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Werbebanner <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Banner <strong>%s</strong> removed.";
#elif L==4	// es
	"Banner <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Banni&eacute;re <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"Banner <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Banner <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Banner <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Banner <strong>%s</strong> removido.";
#elif L==10	// tr
	"Banner <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Banners =
#if   L==1	// ca
	"Banners";
#elif L==2	// de
	"Werbebanners";
#elif L==3	// en
	"Banners";
#elif L==4	// es
	"Banners";
#elif L==5	// fr
	"Banni&eacute;res";
#elif L==6	// gn
	"Banners";
#elif L==7	// it
	"Banners";
#elif L==8	// pl
	"Banners";
#elif L==9	// pt
	"Banners";
#elif L==10	// tr
	"Afi&scedil;ler";
#endif

const char *Txt_Basic_public_profile =
#if   L==1	// ca
	"Perfil p&uacute;blic b&agrave;sic";
#elif L==2	// de
	"Grundlegendes &ouml;ffentliches Profile";
#elif L==3	// en
	"Basic public profile";
#elif L==4	// es
	"Perfil p&uacute;blico b&aacute;sico";
#elif L==5	// fr
	"Profil public de base";
#elif L==6	// gn
	"Perfil p&uacute;blico b&aacute;sico";	// Okoteve traduccin
#elif L==7	// it
	"Profilo pubblico di base";
#elif L==8	// pl
	"Podstawowy profil publiczny";
#elif L==9	// pt
	"Perfil p&uacute;blico b&aacute;sico";
#elif L==10	// tr
	"Basic public profile";			// eviri lazim!
#endif

const char *Txt_Before_creating_a_new_account_check_if_you_have_been_already_registered =
#if   L==1	// ca
	"Abans de crear un compte nou,"
	" comprovi si ja li han inscrit";
#elif L==2	// de
	"Bevor ein neues Benutzerkonto erstellen,"
	" wenn Sie bereits registriert worden sind";
#elif L==3	// en
	"Before creating a new account,"
	" check if you have been already registered";
#elif L==4	// es
	"Antes de crear una nueva cuenta,"
	" compruebe si ya le han inscrito";
#elif L==5	// fr
	"Avant de cr&eacute;er un nouveau compte,"
	" v&eacute;rifiez si vous avez d&eacute;j&agrave; enregistr&eacute;";
#elif L==6	// gn
	"Antes de crear una nueva cuenta,"
	" compruebe si ya le han inscrito";	// Okoteve traduccin
#elif L==7	// it
	"Prima di creare un nuovo account,"
	" verificare se sono stati gi&agrave; registrato";
#elif L==8	// pl
	"Przed utworzeniem nowego konta sprawd&zacute;,"
	" czy zosta&lstrok;e&sacute; ju&zdot; zarejestrowany";
#elif L==9	// pt
	"Antes de criar uma nova conta,"
	" verifique se voc&ecirc; j&aacute; foi registrado/a";
#elif L==10	// tr
	"Yeni bir hesap olu&scedil;turmadan &ouml;nce,"
	" &ouml;nceden kay&inodot;tl&inodot; olup"
	" olmad&inodot;&gbreve;&inodot;n&inodot;z&inodot; kontrol edin.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_nickname =
#if   L==1	// ca
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de crear el seu sobrenom.";
#elif L==2	// de
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr Benutzernamen erstellen.";
#elif L==3	// en
	"Before going to any other option"
	" you must create your nickname.";
#elif L==4	// es
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su apodo.";
#elif L==5	// fr
	"Avant d'aller &agrave; une autre option"
	" vous devez cr&eacute;er votre surnom.";
#elif L==6	// gn
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su apodo.";		// Okoteve traduccin
#elif L==7	// it
	"Prima di scegliere qualsiasi altra opzione"
	" devi creare il tuo soprannome.";
#elif L==8	// pl
	"Przed udaniem si&eogon; do innej opcji"
	" nale&zdot;y wype&lstrok;ni&cacute; sw&oacute;j nick";
#elif L==9	// pt
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve criar sua alcunha.";
#elif L==10	// tr
	"Before going to any other option"
	" you must create your nickname.";	// eviri lazim!
#endif

const char *Txt_Before_going_to_any_other_option_you_must_create_your_password =
#if   L==1	// ca
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de crear la seva contrasenya.";
#elif L==2	// de
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr Passwort erstellen.";
#elif L==3	// en
	"Before going to any other option"
	" you must create your password.";
#elif L==4	// es
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";
#elif L==5	// fr
	"Avant d'aller &agrave; une autre option"
	" vous devez cr&eacute;er votre mot de passe.";
#elif L==6	// gn
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";	// Okoteve traduccin
#elif L==7	// it
	"Prima di scegliere qualsiasi altra opzione"
	" devi creare una password.";
#elif L==8	// pl
	"Przed p&oacute;jsciem do jakiejkolwiek innej opcji"
	" nalezy utworzy&cacute; has&lstrok;o.";
#elif L==9	// pt
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	"voc&ecirc; deve criar sua senha.";
#elif L==10	// tr
	"Before going to any other option"
	" you must create your password.";	// eviri lazim!
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_in_your_email_address =
#if   L==1	// ca
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir la seva adre&ccedil;a de correu.";
#elif L==2	// de
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr E-Mail-Adresse ausf&uuml;llen.";
#elif L==3	// en
	"Before going to any other option"
	" you must fill in your email address.";
#elif L==4	// es
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su direcci&oacute;n de correo.";
#elif L==5	// fr
	"Avant d'aller &agrave; une autre option"
	" vous devez indiquer votre adresse email.";
#elif L==6	// gn
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Prima di scegliere qualsiasi altra opzione"
	" devi inserire il tuo indirizzo email.";
#elif L==8	// pl
	"Przed p&oacute;jsciem do jakiejkolwiek innej opcji"
	" wype&lstrok;ni&cacute; sw&oacute;j adres e-mail.";
#elif L==9	// pt
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	"voc&ecirc; deve preencher seu endere&ccedil;o de email.";
#elif L==10	// tr
	"Before going to any other option"
	" you must fill in your email address.";		// eviri lazim!
#endif

const char *Txt_Briefcase_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"S'ha eliminat el malet&iacute; de <strong>%s</strong>.";
#elif L==2	// de
	"Aktentasche von <strong>%s</strong> wurde entfernt.";
#elif L==3	// en
	"Briefcase of <strong>%s</strong> has been removed.";
#elif L==4	// es
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";
#elif L==5	// fr
	"Le porte-documents de <strong>%s</strong> a &eacute;t&eacute; retir&eacute;.";
#elif L==6	// gn
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La penna USB de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8	// pl
	"Teczka <strong>%s</strong> zosta&lstrok;a usuni&eogon;ta.";
#elif L==9	// pt
	"Pasta de <strong>%s</strong> foi removida.";
#elif L==10	// tr
	"Briefcase of <strong>%s</strong> has been removed.";		// eviri lazim!
#endif

const char *Txt_Briefcases =
#if   L==1	// ca
	"Maletins";
#elif L==2	// de
	"Aktentasche";
#elif L==3	// en
	"Briefcases";
#elif L==4	// es
	"Maletines";
#elif L==5	// fr
	"Porte-documents";
#elif L==6	// gn
	"Maletines";	// Okoteve traduccin
#elif L==7	// it
	"Penna USB";
#elif L==8	// pl
	"Teczki";
#elif L==9	// pt
	"Pasta";
#elif L==10	// tr
	"Evrak &ccedil;antalar&inodot;";
#endif

const char *Txt_Building =
#if   L==1	// ca
	"Edifici";
#elif L==2	// de
	"Geb&auml;ude";
#elif L==3	// en
	"Building";
#elif L==4	// es
	"Edificio";
#elif L==5	// fr
	"B&acirc;timent";
#elif L==6	// gn
	"Edificio";	// Okoteve traduccin
#elif L==7	// it
	"Edificio";
#elif L==8	// pl
	"Budynek";
#elif L==9	// pt
	"Edif&iacute;cio";
#elif L==10	// tr
	"Bina";
#endif

const char *Txt_Building_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Edifici <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Geb&auml;ude <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Building <strong>%s</strong> removed.";
#elif L==4	// es
	"Edificio <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"B&acirc;timent <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Edificio <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Edificio <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Budynek <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Edif&iacute;cio <strong>%s</strong> removido.";
#elif L==10	// tr
	"Building <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_BUILDINGS_HELP_ORDER[Bld_NUM_ORDERS] =
	{
	[Bld_ORDER_BY_SHRT_NAME] =
#if   L==1	// ca
	"Ordenar per nom breu"
#elif L==2	// de
	"Nach Kurzname sortieren"
#elif L==3	// en
	"Sort by short name"
#elif L==4	// es
	"Ordenar por nombre breve"
#elif L==5	// fr
	"Trier par nom abr&eacute;g&eacute;"
#elif L==6	// gn
	"Ordenar por nombre breve"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per nome breve"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug kr&oacute;tkiej nazwy"
#elif L==9	// pt
	"Classificar por nome abreviado"
#elif L==10	// tr
	"Sort by short name"		// eviri lazim!
#endif
	,
	[Bld_ORDER_BY_FULL_NAME] =
#if   L==1	// ca
	"Ordenar per nom complet"
#elif L==2	// de
	"Nach vollst&auml;ndiger Name sortieren"
#elif L==3	// en
	"Sort by full name"
#elif L==4	// es
	"Ordenar por nombre completo"
#elif L==5	// fr
	"Trier par nom complet"
#elif L==6	// gn
	"Ordenar por nombre completo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per nome completo"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug pe&lstrok;na nazwa"
#elif L==9	// pt
	"Classificar por nome completo"
#elif L==10	// tr
	"Sort by full name"		// eviri lazim!
#endif
	,
	[Bld_ORDER_BY_LOCATION] =
#if   L==1	// ca
	"Ordenar per ubicaci&oacute;"
#elif L==2	// de
	"Nach Standort sortieren"
#elif L==3	// en
	"Sort by location"
#elif L==4	// es
	"Ordenar por ubicaci&oacute;n"
#elif L==5	// fr
	"Trier par emplacement"
#elif L==6	// gn
	"Ordenar por ubicaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per posizione"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug lokacja"
#elif L==9	// pt
	"Classificar por localiza&ccedil;&atilde;o"
#elif L==10	// tr
	"Sort by location"		// eviri lazim!
#endif
  };

const char *Txt_BUILDINGS_ORDER[Bld_NUM_ORDERS] =
	{
	[Bld_ORDER_BY_SHRT_NAME] =
#if   L==1	// ca
	"Nom breu"
#elif L==2	// de
	"Kurzname"
#elif L==3	// en
	"Short name"
#elif L==4	// es
	"Nombre breve"
#elif L==5	// fr
	"Nom abr&eacute;g&eacute;"
#elif L==6	// gn
	"Nombre breve"	// Okoteve traduccin
#elif L==7	// it
	"Nome breve"
#elif L==8	// pl
	"Kr&oacute;tkiej nazwy"
#elif L==9	// pt
	"Nome abreviado"
#elif L==10	// tr
	"K&inodot;sa isim"
#endif
	,
	[Bld_ORDER_BY_FULL_NAME] =
#if   L==1	// ca
	"Nom complet"
#elif L==2	// de
	"Vollst&auml;ndiger Name"
#elif L==3	// en
	"Full name"
#elif L==4	// es
	"Nombre completo"
#elif L==5	// fr
	"Nom complet"
#elif L==6	// gn
	"Nombre completo"	// Okoteve traduccin
#elif L==7	// it
	"Nome completo"
#elif L==8	// pl
	"Pe&lstrok;na nazwa"
#elif L==9	// pt
	"Nome completo"
#elif L==10	// tr
	"Tam isim"
#endif
	,
	[Bld_ORDER_BY_LOCATION] =
#if   L==1	// ca
	"Ubicaci&oacute;"
#elif L==2	// de
	"Standort"
#elif L==3	// en
	"Location"
#elif L==4	// es
	"Ubicaci&oacute;n"
#elif L==5	// fr
	"Emplacement"
#elif L==6	// gn
	"&Ntilde;emohenda"
#elif L==7	// it
	"Posizione"
#elif L==8	// pl
	"Lokacja"
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o"
#elif L==10	// tr
	"Location"	// eviri lazim!
#endif
	};

const char *Txt_Buildings =
#if   L==1	// ca
	"Edificis";
#elif L==2	// de
	"Geb&auml;ude";
#elif L==3	// en
	"Buildings";
#elif L==4	// es
	"Edificios";
#elif L==5	// fr
	"B&acirc;timents";
#elif L==6	// gn
	"Edificios";	// Okoteve traduccin
#elif L==7	// it
	"Edifici";
#elif L==8	// pl
	"Budynki";
#elif L==9	// pt
	"Edif&iacute;cios";
#elif L==10	// tr
	"Binalar";
#endif

const char *Txt_Calculate =
#if   L==1	// ca
	"Calcular";
#elif L==2	// de
	"Berechnen";
#elif L==3	// en
	"Calculate";
#elif L==4	// es
	"Calcular";
#elif L==5	// fr
	"Calculer";
#elif L==6	// gn
	"Calcular";	// Okoteve traduccin
#elif L==7	// it
	"Calcolare";
#elif L==8	// pl
	"Oblicza&cacute;";
#elif L==9	// pt
	"Calcular";
#elif L==10	// tr
	"Calculate";	// eviri lazim!
#endif

const char *Txt_Calculate_average_photo_of_THE_DEGREE_X =	// Warning: should end in space
#if   L==1	// ca
	"Calcular fotografia mitjana de ";
#elif L==2	// de
	"Berechnen Sie das durchschnittliche Foto von ";
#elif L==3	// en
	"Calculate average photo of ";
#elif L==4	// es
	"Calcular fotograf&iacute;a promedio de ";
#elif L==5	// fr
	"Calculer la photo moyenne de ";
#elif L==6	// gn
	"Okalkula ta'anga promedio ";
#elif L==7	// it
	"Calcolare la media della foto di ";
#elif L==8	// pl
	"Obliczyc &sacute;rednia zdjecie ";
#elif L==9	// pt
	"Calcular a foto m&eacute;dia de ";
#elif L==10	// tr
	"Ortalama foto&gbreve;raf&inodot;n&inodot; hesaplay&inodot;n ";
#endif

const char *Txt_Calendar =
#if   L==1	// ca
	"Calendari";
#elif L==2	// de
	"Kalender";
#elif L==3	// en
	"Calendar";
#elif L==4	// es
	"Calendario";
#elif L==5	// fr
	"Calendrier";
#elif L==6	// gn
	"Arapapaha";
#elif L==7	// it
	"Calendario";
#elif L==8	// pl
	"Kalendarz";
#elif L==9	// pt
	"Calend&aacute;rio";
#elif L==10	// tr
	"Takvim";
#endif

const char *Txt_CALL_FOR_EXAM_Approximate_duration =
#if   L==1	// ca
	"Durada aproximada";
#elif L==2	// de
	"Voraussichtliche Dauer";
#elif L==3	// en
	"Approximate duration";
#elif L==4	// es
	"Duraci&oacute;n aproximada";
#elif L==5	// fr
	"Dur&eacute;e approximative";
#elif L==6	// gn
	"Duraci&oacute;n aproximada";	// Okoteve traduccin
#elif L==7	// it
	"Durata approssimativa";
#elif L==8	// pl
	"Przyblizony czas trwania";
#elif L==9	// pt
	"Dura&ccedil;&atilde;o aproximada";
#elif L==10	// tr
	"Approximate duration";		// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Course =
#if   L==1	// ca
	"Assignatura";
#elif L==2	// de
	"Kurs";
#elif L==3	// en
	"Course";
#elif L==4	// es
	"Asignatura";
#elif L==5	// fr
	"Mati&egrave;re";
#elif L==6	// gn
	"Mbo'esyry";
#elif L==7	// it
	"Corso";
#elif L==8	// pl
	"Kurs";
#elif L==9	// pt
	"Disciplina";
#elif L==10	// tr
	"Kurs";
#endif

const char *Txt_CALL_FOR_EXAM_Documentation_required =
#if   L==1	// ca
	"Documentaci&oacute; exigida";
#elif L==2	// de
	"Ausweis erforderlich";
#elif L==3	// en
	"ID required";
#elif L==4	// es
	"Documentaci&oacute;n exigida";
#elif L==5	// fr
	"Pi&egrave;ce d'identit obligatoire";
#elif L==6	// gn
	"Documentaci&oacute;n exigida";	// Okoteve traduccin
#elif L==7	// it
	"Documentazione richiesta";
#elif L==8	// pl
	"ID wymagane";
#elif L==9	// pt
	"Documenta&ccedil;&atilde;o necess&aacute;ria";
#elif L==10	// tr
	"ID required";			// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Exam_date =
#if   L==1	// ca
	"Data de l'examen";
#elif L==2	// de
	"Pr&uuml;fungsdatum";
#elif L==3	// en
	"Exam date";
#elif L==4	// es
	"Fecha del examen";
#elif L==5	// fr
	"Date de l'examen";
#elif L==6	// gn
	"Fecha del examen";	// Okoteve traduccin
#elif L==7	// it
	"Data d'esame";
#elif L==8	// pl
	"Termin egzaminu";
#elif L==9	// pt
	"Exame data";
#elif L==10	// tr
	"Exam date";	// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Material_allowed =
#if   L==1	// ca
	"Material perm&egrave;s";
#elif L==2	// de
	"Erlaubte Unterlagen";
#elif L==3	// en
	"Material allowed";
#elif L==4	// es
	"Material permitido";
#elif L==5	// fr
	"Mat&eacute;riel autoris&eacute;";
#elif L==6	// gn
	"Material permitido";	// Okoteve traduccin
#elif L==7	// it
	"Materiale permesso";
#elif L==8	// pl
	"Material dozwolone";
#elif L==9	// pt
	"Material autorizado";
#elif L==10	// tr
	"Material allowed";	// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Material_required =
#if   L==1	// ca
	"Material obligatori";
#elif L==2	// de
	"Erforderliche Unterlagen";
#elif L==3	// en
	"Material required";
#elif L==4	// es
	"Material obligatorio";
#elif L==5	// fr
	"Mat&eacute;riel n&eacute;cessaire";
#elif L==6	// gn
	"Material obligatorio";	// Okoteve traduccin
#elif L==7	// it
	"Materiale richiesto";
#elif L==8	// pl
	"Materialy niezbedne do";
#elif L==9	// pt
	"Material obrigat&oacute;rio";
#elif L==10	// tr
	"Material required";	// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Mode =
#if   L==1	// ca
	"Modalitat<br />(escrit, oral,&hellip;)";
#elif L==2	// de
	"Pr&uuml;fungsart<br />(schriftlich, m&uuml;ndlich,&hellip;)";
#elif L==3	// en
	"Mode<br />(written, oral,&hellip;)";
#elif L==4	// es
	"Modalidad<br />(escrito, oral,&hellip;)";
#elif L==5	// fr
	"Mode<br />(&eacute;crit, oral,&hellip;)";
#elif L==6	// gn
	"Modalidad<br />(escrito, oral,&hellip;)";	// Okoteve traduccin
#elif L==7	// it
	"Modalit&agrave;<br />(scritto, orale,&hellip;)";
#elif L==8	// pl
	"Tryb<br />(pisemnej, ustnej,&hellip;)";
#elif L==9	// pt
	"Modalidade<br />(escrita, oral,&hellip;)";
#elif L==10	// tr
	"Mode<br />(written, oral,&hellip;)";		// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Other_information =
#if   L==1	// ca
	"Altres indicacions";
#elif L==2	// de
	"Weitere Informationen";
#elif L==3	// en
	"Other information";
#elif L==4	// es
	"Otras indicaciones";
#elif L==5	// fr
	"Autres informations";
#elif L==6	// gn
	"Otras indicaciones";	// Okoteve traduccin
#elif L==7	// it
	"Altre informazioni";
#elif L==8	// pl
	"Inne informacje";
#elif L==9	// pt
	"Outras informa&ccedil;&otilde;es";
#elif L==10	// tr
	"Other information";	// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Place_of_exam =
#if   L==1	// ca
	"Lloc de realitzaci&oacute;";
#elif L==2	// de
	"Pr&uuml;fungsort";
#elif L==3	// en
	"Place of exam";
#elif L==4	// es
	"Lugar de realizaci&oacute;n";
#elif L==5	// fr
	"Lieu de l'examen";
#elif L==6	// gn
	"Lugar de realizaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Luogo d'esame";
#elif L==8	// pl
	"Miejsce egzaminu";
#elif L==9	// pt
	"Local do exame";
#elif L==10	// tr
	"Place of exam";		// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Session =
#if   L==1	// ca
	"Convocat&ograve;ria<br />(juny, setembre,&hellip;)";
#elif L==2	// de
	"Einberufung<br />(Juni, September,&hellip;)";
#elif L==3	// en
	"Session<br />(june, september,&hellip;)";
#elif L==4	// es
	"Convocatoria<br />(junio, septiembre,&hellip;)";
#elif L==5	// fr
	"P&eacute;riode<br />(juin, septembre,&hellip;)";
#elif L==6	// gn
	"Convocatoria<br />(junio, septiembre,&hellip;)";	// Okoteve traduccin
#elif L==7	// it
	"Sessione<br />(giugno, settembre,&hellip;)";
#elif L==8	// pl
	"Sesja<br />(czerwiec, wrzesien,&hellip;)";
#elif L==9	// pt
	"Per&iacute;odo<br />(junho, setembro,&hellip;)";
#elif L==10	// tr
	"Session<br />(june, september,&hellip;)";		// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Start_time =
#if   L==1	// ca
	"Hora d'inici";
#elif L==2	// de
	"Beginn um";
#elif L==3	// en
	"Start time";
#elif L==4	// es
	"Hora de inicio";
#elif L==5	// fr
	"Heure de d&eacute;but";
#elif L==6	// gn
	"Hora de inicio";	// Okoteve traduccin
#elif L==7	// it
	"Ora d'inizio";
#elif L==8	// pl
	"Czas rozpoczecia";
#elif L==9	// pt
	"Hora de in&iacute;cio";
#elif L==10	// tr
	"Start time";		// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Structure_of_the_exam =
#if   L==1	// ca
	"Estructura de l'examen";
#elif L==2	// de
	"Pr&uuml;fungsstruktur";
#elif L==3	// en
	"Structure of the exam";
#elif L==4	// es
	"Estructura del examen";
#elif L==5	// fr
	"Structure de l'examen";
#elif L==6	// gn
	"Estructura del examen";	// Okoteve traduccin
#elif L==7	// it
	"Struttura d'esame";
#elif L==8	// pl
	"Struktura egzaminu";
#elif L==9	// pt
	"Estrutura do exame";
#elif L==10	// tr
	"Structure of the exam";	// eviri lazim!
#endif

const char *Txt_CALL_FOR_EXAM_Year_or_semester =	// 1, 2, 3..., not 1984, 2038...
#if   L==1	// ca
	"Any o semestre";
#elif L==2	// de
	"Jahr oder Semester";
#elif L==3	// en
	"Year or semester";
#elif L==4	// es
	"A&ntilde;o o semestre";
#elif L==5	// fr
	"Ann&eacute;e ou semestre";
#elif L==6	// gn
	"A&ntilde;o o semestre";	// Okoteve traduccin
#elif L==7	// it
	"Anno o semestre";
#elif L==8	// pl
	"Rok lub semestr";
#elif L==9	// pt
	"Ano ou semestre";
#elif L==10	// tr
	"Year or semester";		// eviri lazim!
#endif

const char *Txt_Call_for_exam =
#if   L==1	// ca
	"Convocat&ograve;ria d'examen";
#elif L==2	// de
	"Aufrufe fr Pr&uuml;fung";
#elif L==3	// en
	"Call for exam";
#elif L==4	// es
	"Convocatoria de examen";
#elif L==5	// fr
	"Convocation &agrave; un examen";
#elif L==6	// gn
	"Convocatoria de examen";	// Okoteve traduccin
#elif L==7	// it
	"Appello d'esame";
#elif L==8	// pl
	"Ogloszenie egzamin";
#elif L==9	// pt
	"Chamada para exame";
#elif L==10	// tr
	"S&inodot;nav i&ccedil;in aray&inodot;n";
#endif

const char *Txt_Call_for_exam_removed =
#if   L==1	// ca
	"Convocat&ograve;ria d'examen eliminada.";
#elif L==2	// de
	"Aufruf zur Pr&uuml;fung entfernt.";
#elif L==3	// en
	"Call for exam removed.";
#elif L==4	// es
	"Convocatoria de examen eliminada.";
#elif L==5	// fr
	"Convocation pour l'examen supprim&eacute;.";
#elif L==6	// gn
	"Convocatoria de examen eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Appello d'esame rimosso.";
#elif L==8	// pl
	"Ogloszenie egzamin usuniete.";
#elif L==9	// pt
	"Chamada para exame removida.";
#elif L==10	// tr
	"Call for exam removed.";		// eviri lazim!
#endif

const char *Txt_Call_the_roll =
#if   L==1	// ca
	"Passar llista";
#elif L==2	// de
	"Anwesenheit feststellen";
#elif L==3	// en
	"Call the roll";
#elif L==4	// es
	"Pasar lista";
#elif L==5	// fr
	"Faire l'appel";
#elif L==6	// gn
	"Pasar lista";		// Okoteve traduccin
#elif L==7	// it
	"Appello";
#elif L==8	// pl
	"Imienne";
#elif L==9	// pt
	"Lista de presen&ccedil;a";
#elif L==10	// tr
	"Call the roll";	// eviri lazim!
#endif

const char *Txt_Calls_for_exams =
#if   L==1	// ca
	"Convocat&ograve;ries d'ex&agrave;mens";
#elif L==2	// de
	"Aufrufe f&uuml;r Pr&uuml;fung";
#elif L==3	// en
	"Calls for exams";
#elif L==4	// es
	"Convocatorias de ex&aacute;menes";
#elif L==5	// fr
	"Convocations &agrave; examens";
#elif L==6	// gn
	"Convocatorias de ex&aacute;menes";	// Okoteve traduccin
#elif L==7	// it
	"Appelli d'esame";
#elif L==8	// pl
	"Ogloszenia egzamin&oacute;w";
#elif L==9	// pt
	"Chamadas para exames";
#elif L==10	// tr
	"S&inodot;nav &ccedil;a&gbreve;r&inodot;lar&inodot;";
#endif

const char *Txt_Can_not_create_the_folder_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==2	// de
	"Verzeichnis <strong>%s</strong> kann aus Platzgrnden"
	" nicht erstellt werden.";
#elif L==3	// en
	"Cannot create the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==6	// gn
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; creare la cartella <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#elif L==10	// tr
	"Cannot create the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// eviri lazim!
#endif

const char *Txt_Can_not_create_the_folder_X_because_there_is_already_a_folder_or_a_file_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque ja existeix una carpeta o un fitxer amb aquest nom.";
#elif L==2	// de
	"Der Ordner <strong>%s</strong> kann nicht erstellt werden,"
	" da bereits ein Ordner oder eine Datei mit diesem Namen vorhanden ist.";
#elif L==3	// en
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";
#elif L==4	// es
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un fichier avec ce nom.";
#elif L==6	// gn
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare la cartella <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un file con quel nome.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>"
	" poniewaz istnieje juz folder lub plik o tej nazwie.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou um arquivo com esse nome.";
#elif L==10	// tr
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";	// eviri lazim!
#endif

const char *Txt_Can_not_create_the_link_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==2	// de
	"Link <strong>%s</strong> kann aus Platzgr&uuml;nden"
	" nicht erstellt werden.";
#elif L==3	// en
	"Cannot create the link <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==6	// gn
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; creare il link <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; link <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#elif L==10	// tr
	"Cannot create the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// eviri lazim!
#endif

const char *Txt_Can_not_create_the_link_X_because_there_is_already_a_folder_or_a_link_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque ja existeix una carpeta o un enlla&ccedil; amb aquest nom.";
#elif L==2	// de
	"Der Link <strong>%s</strong> kann nicht erstellt werden,"
	" da bereits ein Ordner oder Link mit diesem Namen vorhanden ist.";
#elif L==3	// en
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";
#elif L==4	// es
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un lien avec ce nom.";
#elif L==6	// gn
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare il link <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un link con quel nome.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; link <strong>%s</strong>"
	" poniewaz istnieje juz folder lub link o tej nazwie.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou uma liga&ccedil;&atilde;o com esse nome.";
#elif L==10	// tr
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";	// eviri lazim!
#endif

const char *Txt_Can_not_rename_a_folder_of_assignment =
#if   L==1	// ca
	"No es pot canviar el nom de una carpeta de activitat.";
#elif L==2	// de
	"Das Verzeichnis der Aufgabe kann nicht umbenannt werden.";
#elif L==3	// en
	"Cannot rename a folder of assignment.";
#elif L==4	// es
	"No se puede cambiar el nombre de una carpeta de actividad.";
#elif L==5	// fr
	"Impossible de renommer un r&eacute;pertoire d'activit&eacute;.";
#elif L==6	// gn
	"No se puede cambiar el nombre de una carpeta de actividad.";	// Okoteve traduccin
#elif L==7	// it
	"Non &egrave; possibile rinominare una cartella di attivit&agrave;.";
#elif L==8	// pl
	"Nie mozna zmienic nazwy folderu zadania.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel renomear um diret&oacute;rio de actividade.";
#elif L==10	// tr
	"Cannot rename a folder of assignment.";			// eviri lazim!
#endif

const char *Txt_Capacity_OF_A_ROOM =
#if   L==1	// ca
	"Aforo";
#elif L==2	// de
	"Kapazit&auml;t";
#elif L==3	// en
	"Capacity";
#elif L==4	// es
	"Aforo";
#elif L==5	// fr
	"Capacit&eacute;";
#elif L==6	// gn
	"Aforo";	// Okoteve traduccin
#elif L==7	// it
	"Capacit&agrave;";
#elif L==8	// pl
	"Pojemno&sacute;&cacute;";
#elif L==9	// pt
	"Capacidade";
#elif L==10	// tr
	"Capacity";	// eviri lazim!
#endif

const char *Txt_Center_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Center <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Lehrinstitut <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Center <strong>%s</strong> removed.";
#elif L==4	// es
	"Centro <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Centre <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Centro <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Centro <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Centrum <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Centro <strong>%s</strong> removido.";
#elif L==10	// tr
	"Center <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_CENTER_STATUS[Hie_NUM_STATUS_TXT] =
	{
 	[Hie_STATUS_UNKNOWN] =
#if   L==1	// ca
	"Desconegudo"
#elif L==2	// de
	"Unbekannt"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
 	[Hie_STATUS_ACTIVE] =
#if   L==1	// ca
	"Activo"
#elif L==2	// de
	"Aktive"
#elif L==3	// en
	"Active"
#elif L==4	// es
	"Activo"
#elif L==5	// fr
	"Actif"
#elif L==6	// gn
	"Activo"	// Okoteve traduccin
#elif L==7	// it
	"Attivo"
#elif L==8	// pl
	"Aktywne"
#elif L==9	// pt
	"Ativo"
#elif L==10	// tr
	"Active"	// eviri lazim!
#endif
	,
 	[Hie_STATUS_PENDING] =
#if   L==1	// ca
	"Novo"
#elif L==2	// de
	"Neu"
#elif L==3	// en
	"New"
#elif L==4	// es
	"Nuevo"
#elif L==5	// fr
	"Nouveau"
#elif L==6	// gn
	"Pyahu"
#elif L==7	// it
	"Nuovo"
#elif L==8	// pl
	"Nowy"
#elif L==9	// pt
	"Novo"
#elif L==10	// tr
	"New"	// eviri lazim!
#endif
	,
 	[Hie_STATUS_REMOVED] =
#if   L==1	// ca
	"Eliminado"
#elif L==2	// de
	"Entfernt"
#elif L==3	// en
	"Removed"
#elif L==4	// es
	"Eliminado"
#elif L==5	// fr
	"Supprim&eacute;"
#elif L==6	// gn
	"Eliminado"	// Okoteve traduccin
#elif L==7	// it
	"Rimosso"
#elif L==8	// pl
	"Usuniete"
#elif L==9	// pt
	"Suprimido"
#elif L==10	// tr
	"Removed"	// eviri lazim!
#endif
	};

const char *Txt_Centers_ABBREVIATION =
#if   L==1	// ca
	"Cent.";
#elif L==2	// de
	"Lehrin.";
#elif L==3	// en
	"Cent.";
#elif L==4	// es
	"Cent.";
#elif L==5	// fr
	"Cent.";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Cent.";
#elif L==8	// pl
	"Cent.";
#elif L==9	// pt
	"Cent.";
#elif L==10	// tr
	"Merk.";
#endif

const char *Txt_CENTERS_HELP_ORDER[Hie_NUM_ORDERS] =
	{
 	[Hie_ORDER_BY_NAME] =
#if   L==1	// ca
	"Ordenar per center"
#elif L==2	// de
	"Nach Lehrinstitut Sortieren"
#elif L==3	// en
	"Sort by center"
#elif L==4	// es
	"Ordenar por centro"
#elif L==5	// fr
	"Trier par center"
#elif L==6	// gn
	"Ordenar por centro"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per centro"
#elif L==8	// pl
	"Sortuj centrum"
#elif L==9	// pt
	"Classificar por centro"
#elif L==10	// tr
	"Sort by center"	// eviri lazim!
#endif
	,
 	[Hie_ORDER_BY_NUM_USRS] =
#if   L==1	// ca
	"Ordenar per n&ordm; d'usuaris"
#elif L==2	// de
	"Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	"Sort by no. of users"
#elif L==4	// es
	"Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	"Trier par nombre d'utilisateurs"
#elif L==6	// gn
	"Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di utenti"
#elif L==8	// pl
	"Sortuj wedlug liczby uzytkownikow"
#elif L==9	// pt
	"Classificar por n&ordm; de utilizadores"
#elif L==10	// tr
	"Sort by no. of users"			// eviri lazim!
#endif
	};

const char *Txt_Centers_of_INSTITUTION_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Centers de %s";
#elif L==2	// de
	"Lehrinstitute von %s";
#elif L==3	// en
	"Centers of %s";
#elif L==4	// es
	"Centros de %s";
#elif L==5	// fr
	"Centers de %s";
#elif L==6	// gn
	"Centros de %s";	// Okoteve traduccin
#elif L==7	// it
	"Centri di %s";
#elif L==8	// pl
	"Centra %s";
#elif L==9	// pt
	"Centros de %s";
#elif L==10	// tr
	"Centers of %s";	// eviri lazim!
#endif

const char *Txt_CENTERS_ORDER[Hie_NUM_ORDERS] =
	{
 	[Hie_ORDER_BY_NAME] =
#if   L==1	// ca
	"Center"
#elif L==2	// de
	"Lehrinstitut"
#elif L==3	// en
	"Center"
#elif L==4	// es
	"Centro"
#elif L==5	// fr
	"Center"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centro"
#elif L==8	// pl
	"Centrum"
#elif L==9	// pt
	"Centro"
#elif L==10	// tr
	"Merkez"
#endif
	,
 	[Hie_ORDER_BY_NUM_USRS] =
#if   L==1	// ca
	"Usuaris del center"
#elif L==2	// de
	"Benutzer der Lehrinstitut"
#elif L==3	// en
	"Users of the center"
#elif L==4	// es
	"Usuarios del centro"
#elif L==5	// fr
	"Utilisateurs du center"
#elif L==6	// gn
	"Usuarios del centro"	// Okoteve traduccin
#elif L==7	// it
	"Utenti del centro"
#elif L==8	// pl
	"U&zdot;ytkownicy centrum"
#elif L==9	// pt
	"Utilizadores do centro"
#elif L==10	// tr
	"Users of the center"	// eviri lazim!
#endif
	};

const char *Txt_Centers_with_map =
#if   L==1	// ca
	"Centers amb mapa";
#elif L==2	// de
	"Lehrinstitute mit Karte";
#elif L==3	// en
	"Centers with map";
#elif L==4	// es
	"Centros con mapa";
#elif L==5	// fr
	"Centers avec carte";
#elif L==6	// gn
	"Centros con mapa";	// Okoteve traduccin
#elif L==7	// it
	"Centri con mappa";
#elif L==8	// pl
	"Centra z map&aogon;";
#elif L==9	// pt
	"Centros com mapa";
#elif L==10	// tr
	"Centers with map";	// eviri lazim!
#endif

const char *Txt_Centers_with_pending_degrees =
#if   L==1	// ca
	"Centers amb titulacions pendents d'activar";
#elif L==2	// de
	"Lehrinstitute mit anh&auml;ngig Studiengang";
#elif L==3	// en
	"Centers with pending degrees";
#elif L==4	// es
	"Centros con titulaciones pendientes de activar";
#elif L==5	// fr
	"Centers avec des &eacute;tudes en attente";
#elif L==6	// gn
	"Centros con titulaciones pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Centri con lauree in sospeso";
#elif L==8	// pl
	"Centra z stopnie oczekuj&aogon;cy";
#elif L==9	// pt
	"Centros com graus pendentes";
#elif L==10	// tr
	"Centers with pending degrees";				// eviri lazim!
#endif

const char *Txt_Change_email =
#if   L==1	// ca
	"Canviar correu";
#elif L==2	// de
	"E-Mail &auml;ndern";
#elif L==3	// en
	"Change email";
#elif L==4	// es
	"Cambiar correo";
#elif L==5	// fr
	"Changer courrier";
#elif L==6	// gn
	"Cambiar correo";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare email";
#elif L==8	// pl
	"Zmie&nacute; email";
#elif L==9	// pt
	"Alterar email";
#elif L==10	// tr
	"Change email";		// eviri lazim!
#endif

const char *Txt_Change_IDs =
#if   L==1	// ca
	"Canviar IDs";
#elif L==2	// de
	"Ausweis-Nr. &auml;ndern";
#elif L==3	// en
	"Change IDs";
#elif L==4	// es
	"Cambiar ID";
#elif L==5	// fr
	"Changer IDs";
#elif L==6	// gn
	"Cambiar ID";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare ID";
#elif L==8	// pl
	"Zmie&nacute; ID";
#elif L==9	// pt
	"Alterar n&ordm;s. identif.";
#elif L==10	// tr
	"Change IDs";	// eviri lazim!
#endif

const char *Txt_Change_image_video =
#if   L==1	// ca
	"Canviar imatge / v&iacute;deo";
#elif L==2	// de
	"Abbild / Video &auml;ndern";
#elif L==3	// en
	"Change image / video";
#elif L==4	// es
	"Cambiar imagen / v&iacute;deo";
#elif L==5	// fr
	"Changer image / vid&eacute;o";
#elif L==6	// gn
	"Moambue ta'&atilde;nga / video";
#elif L==7	// it
	"Cambiare immagine / video";
#elif L==8	// pl
	"Zmie&nacute; obraz / wideo";
#elif L==9	// pt
	"Alterar imagem / v&iacute;deo";
#elif L==10	// tr
	"Change image / video";	// eviri lazim!
#endif

const char *Txt_Change_my_groups =
#if   L==1	// ca
	"Canviar de grups";
#elif L==2	// de
	"Gruppen &auml;ndern";
#elif L==3	// en
	"Change my groups";
#elif L==4	// es
	"Cambiar de grupos";
#elif L==5	// fr
	"Modifier mes groupes";
#elif L==6	// gn
	"Cambiar de grupos";	// Okoteve traduccin
#elif L==7	// it
	"Cambia i miei gruppi";
#elif L==8	// pl
	"Zmie&nacute;moje grupy";
#elif L==9	// pt
	"Alterar meus grupos";
#elif L==10	// tr
	"Change my groups";	// eviri lazim!
#endif

const char *Txt_Change_nickname =
#if   L==1	// ca
	"Canviar sobrenom";
#elif L==2	// de
	"Benutzername &auml;ndern";
#elif L==3	// en
	"Change nickname";
#elif L==4	// es
	"Cambiar apodo";
#elif L==5	// fr
	"Changer le surnom";
#elif L==6	// gn
	"Cambiar apodo";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare nome utente";
#elif L==8	// pl
	"Zmie&nacute; nick";
#elif L==9	// pt
	"Alterar alcunha";
#elif L==10	// tr
	"Change nickname";	// eviri lazim!
#endif

const char *Txt_Change_password =
#if   L==1	// ca
	"Canviar contrasenya";
#elif L==2	// de
	"Passwort &auml;ndern";
#elif L==3	// en
	"Change password";
#elif L==4	// es
	"Cambiar contrase&ntilde;a";
#elif L==5	// fr
	"Changer mot de passe";
#elif L==6	// gn
	"Cambiar contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare password";
#elif L==8	// pl
	"Zmie&nacute; has&lstrok;o";
#elif L==9	// pt
	"Alterar senha";
#elif L==10	// tr
	"&Scedil;ifre de&gbreve;i&scedil;tir";
#endif

const char *Txt_CHAT_Room =
#if   L==1	// ca
	"Sala";
#elif L==2	// de
	"Raum";
#elif L==3	// en
	"Room";
#elif L==4	// es
	"Sala";
#elif L==5	// fr
	"Salle";
#elif L==6	// gn
	"Sala";	// Okoteve traduccin
#elif L==7	// it
	"Room";
#elif L==8	// pl
	"Pok&oacute;j";
#elif L==9	// pt
	"Room";
#elif L==10	// tr
	"Room";	// eviri lazim!
#endif

const char *Txt_CHAT_Room_code =
#if   L==1	// ca
	"Codi de sala";
#elif L==2	// de
	"Raum-code";
#elif L==3	// en
	"Room code";
#elif L==4	// es
	"C&oacute;digo de sala";
#elif L==5	// fr
	"Code de salle";
#elif L==6	// gn
	"C&oacute;digo de sala";	// Okoteve traduccin
#elif L==7	// it
	"Chiave di room";
#elif L==8	// pl
	"Kod pokoju";
#elif L==9	// pt
	"C&oacute;digo de room";
#elif L==10	// tr
	"Room code";			// eviri lazim!
#endif

const char *Txt_Chat_rooms =
#if   L==1	// ca
	"Sales de xat";
#elif L==2	// de
	"Chat-R&auml;ume";
#elif L==3	// en
	"Chat rooms";
#elif L==4	// es
	"Salas de chat";
#elif L==5	// fr
	"Salles de chat";
#elif L==6	// gn
	"Salas de chat";	// Okoteve traduccin
#elif L==7	// it
	"Chat rooms";
#elif L==8	// pl
	"Czat";
#elif L==9	// pt
	"Chat rooms";
#elif L==10	// tr
	"Chat rooms";		// eviri lazim!
#endif

const char *Txt_Check =
#if   L==1	// ca
	"Comprovar";
#elif L==2	// de
	"&Uuml;berpr&uuml;fen";
#elif L==3	// en
	"Check";
#elif L==4	// es
	"Comprobar";
#elif L==5	// fr
	"V&eacute;rifier";
#elif L==6	// gn
	"Comprobar";	// Okoteve traduccin
#elif L==7	// it
	"Verificare";
#elif L==8	// pl
	"Sprawd&zacute;";
#elif L==9	// pt
	"Verificar";
#elif L==10	// tr
	"Kontrol";
#endif

const char *Txt_Check_another_ID =
#if   L==1	// ca
	"Comprovar altre ID?";
#elif L==2	// de
	"&Uuml;berpr&uuml;fen Sie eine andere ID?";
#elif L==3	// en
	"Check another ID?";
#elif L==4	// es
	"&iquest;Comprobar otro ID?";
#elif L==5	// fr
	"V&eacute;rifier un autre ID?";
#elif L==6	// gn
	"&iquest;Comprobar otro ID?";	// Okoteve traduccin
#elif L==7	// it
	"Verificare un altro ID?";
#elif L==8	// pl
	"Sprawdzi&cacute; inny ID?";
#elif L==9	// pt
	"Verificar outra ID?";
#elif L==10	// tr
	"Ba&scedil;ka bir kimli&gbreve;i kontrol et?";
#endif

const char *Txt_Check_marks_in_the_file =
#if   L==1	// ca
	"Consultar qualificacions al fitxer";
#elif L==2	// de
	"&Uuml;berpr&uuml;fen Sie die Noten in der Datei";
#elif L==3	// en
	"Check marks in the file";
#elif L==4	// es
	"Consultar calificaciones en el archivo";
#elif L==5	// fr
	"Consulter les notes dans le fichier";
#elif L==6	// gn
	"Consultar calificaciones en el archivo";	// Okoteve traduccin
#elif L==7	// it
	"Consultare le valutazioni nel file";
#elif L==8	// pl
	"Sprawd&zacute; oceny w pliku";
#elif L==9	// pt
	"Verifique as notas no arquivo";
#elif L==10	// tr
	"Check marks in the file";			// eviri lazim!
#endif

const char *Txt_Click =
#if   L==1	// ca
	"Clic";
#elif L==2	// de
	"Klick";
#elif L==3	// en
	"Click";
#elif L==4	// es
	"Clic";
#elif L==5	// fr
	"Clic";
#elif L==6	// gn
	"Clic";		// Okoteve traduccin
#elif L==7	// it
	"Clic";
#elif L==8	// pl
	"Kliknij";
#elif L==9	// pt
	"Clique";
#elif L==10	// tr
	"Click";	// eviri lazim!
#endif

const char *Txt_Clicks =
#if   L==1	// ca
	"Clics";
#elif L==2	// de
	"Klicks";
#elif L==3	// en
	"Clicks";
#elif L==4	// es
	"Clics";
#elif L==5	// fr
	"Clics";
#elif L==6	// gn
	"Clics";	// Okoteve traduccin
#elif L==7	// it
	"Clic";
#elif L==8	// pl
	"Klikni&eogon;cia";
#elif L==9	// pt
	"Cliques";
#elif L==10	// tr
	"Clicks";	// eviri lazim!
#endif

const char *Txt_clicks =
#if   L==1	// ca
	"clics";
#elif L==2	// de
	"Klicks";
#elif L==3	// en
	"clicks";
#elif L==4	// es
	"clics";
#elif L==5	// fr
	"clics";
#elif L==6	// gn
	"clics";	// Okoteve traduccin
#elif L==7	// it
	"clic";
#elif L==8	// pl
	"klikni&eogon;cia";
#elif L==9	// pt
	"cliques";
#elif L==10	// tr
	"clicks";	// eviri lazim!
#endif

const char *Txt_Clicks_per_day =
#if   L==1	// ca
	"Clics per dia";
#elif L==2	// de
	"Klicks pro Tag";
#elif L==3	// en
	"Clicks per day";
#elif L==4	// es
	"Clics por d&iacute;a";
#elif L==5	// fr
	"Clics par jour";
#elif L==6	// gn
	"Clics por d&iacute;a";	// Okoteve traduccin
#elif L==7	// it
	"Clic al giorno";
#elif L==8	// pl
	"Klikni&eogon;&cacute; dziennie";
#elif L==9	// pt
	"Cliques por dia";
#elif L==10	// tr
	"Clicks per day";	// eviri lazim!
#endif

const char *Txt_Close =
#if   L==1	// ca
	"Tancar";
#elif L==2	// de
	"Schlie&szlig;en";
#elif L==3	// en
	"Close";
#elif L==4	// es
	"Cerrar";
#elif L==5	// fr
	"Fermer";
#elif L==6	// gn
	"S&#7869;";
#elif L==7	// it
	"Chiudi";
#elif L==8	// pl
	"Zamknij";
#elif L==9	// pt
	"Fechar";
#elif L==10	// tr
	"Close";	// eviri lazim!
#endif

const char *Txt_Closed_survey =
#if   L==1	// ca
	"Enquesta tancada";
#elif L==2	// de
	"Geschlossene Umfrage";
#elif L==3	// en
	"Closed survey";
#elif L==4	// es
	"Encuesta cerrada";
#elif L==5	// fr
	"Sondage ferm&eacute;";
#elif L==6	// gn
	"Encuesta cerrada";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio chiuso";
#elif L==8	// pl
	"Zamkniete badania";
#elif L==9	// pt
	"Inqu&eacute;rito fechado";
#elif L==10	// tr
	"Closed survey";	// eviri lazim!
#endif

const char *Txt_Code =
#if   L==1	// ca
	"Codi";
#elif L==2	// de
	"Code";
#elif L==3	// en
	"Code";
#elif L==4	// es
	"C&oacute;d.";
#elif L==5	// fr
	"Code";
#elif L==6	// gn
	"Papanga";
#elif L==7	// it
	"Cod.";
#elif L==8	// pl
	"Kod";
#elif L==9	// pt
	"C&oacute;d.";
#elif L==10	// tr
	"Code";	// eviri lazim!
#endif

const char *Txt_Color_of_the_graphic =
#if   L==1	// ca
	"Color de la gr&agrave;fica";
#elif L==2	// de
	"Farbe der Grafik";
#elif L==3	// en
	"Color of the graphic";
#elif L==4	// es
	"Color de la gr&aacute;fica";
#elif L==5	// fr
	"Couleur de l'image";
#elif L==6	// gn
	"Color de la gr&aacute;fica";	// Okoteve traduccin
#elif L==7	// it
	"Colore del grafico";
#elif L==8	// pl
	"Kolor grafiki";
#elif L==9	// pt
	"Cor do gr&aacute;fico";
#elif L==10	// tr
	"Color of the graphic";		// eviri lazim!
#endif

const char *Txt_column =
#if   L==1	// ca
	"columna";
#elif L==2	// de
	"Spalte";
#elif L==3	// en
	"column";
#elif L==4	// es
	"columna";
#elif L==5	// fr
	"colonne";
#elif L==6	// gn
	"columna";	// Okoteve traduccin
#elif L==7	// it
	"colonna";
#elif L==8	// pl
	"kolumna";
#elif L==9	// pt
	"coluna";
#elif L==10	// tr
	"column";	// eviri lazim!
#endif

const char *Txt_Columns =
#if   L==1	// ca
	"Columnes";
#elif L==2	// de
	"Spalten";
#elif L==3	// en
	"Columns";
#elif L==4	// es
	"Columnas";
#elif L==5	// fr
	"Colonnes";
#elif L==6	// gn
	"Columnas";	// Okoteve traduccin
#elif L==7	// it
	"Colonne";
#elif L==8	// pl
	"Kolumny";
#elif L==9	// pt
	"Colunas";
#elif L==10	// tr
	"S&uuml;tunlar";
#endif

const char *Txt_columns =
#if   L==1	// ca
	"columnes";
#elif L==2	// de
	"Spalten";
#elif L==3	// en
	"columns";
#elif L==4	// es
	"columnas";
#elif L==5	// fr
	"colonnes";
#elif L==6	// gn
	"columnas";	// Okoteve traduccin
#elif L==7	// it
	"colonne";
#elif L==8	// pl
	"kolumny";
#elif L==9	// pt
	"colunas";
#elif L==10	// tr
	"S&uuml;tunlar";
#endif

const char *Txt_Comment_removed =
#if   L==1	// ca
	"Comentari eliminat.";
#elif L==2	// de
	"Kommentar entfernt.";
#elif L==3	// en
	"Comment removed.";
#elif L==4	// es
	"Comentario eliminado.";
#elif L==5	// fr
	"Commentaire supprim&eacute;.";
#elif L==6	// gn
	"Comentario eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Commento rimosso.";
#elif L==8	// pl
	"Komentarz usuniety.";
#elif L==9	// pt
	"Coment&aacute;rio removido.";
#elif L==10	// tr
	"Comment removed.";		// eviri lazim!
#endif

const char *Txt_Comments =
#if   L==1	// ca
	"Comentaris";
#elif L==2	// de
	"Kommentare";
#elif L==3	// en
	"Comments";
#elif L==4	// es
	"Comentarios";
#elif L==5	// fr
	"Commentaires";
#elif L==6	// gn
	"Comentarios";	// Okoteve traduccin
#elif L==7	// it
	"Commenti";
#elif L==8	// pl
	"Komentarze";
#elif L==9	// pt
	"Coment&aacute;rios";
#elif L==10	// tr
	"Yorumlar";
#endif

const char *Txt_Comments_allowed =
#if   L==1	// ca
	"Comentaris permesos";
#elif L==2	// de
	"Kommentare erlaubt";
#elif L==3	// en
	"Comments allowed";
#elif L==4	// es
	"Comentarios permitidos";
#elif L==5	// fr
	"Commentaires autoris&eacute;s";
#elif L==6	// gn
	"Ojeheja umi comentario";
#elif L==7	// it
	"Commenti consentiti";
#elif L==8	// pl
	"Komentarze dozwolone";
#elif L==9	// pt
	"Coment&aacute;rios permitidos";
#elif L==10	// tr
	"Yorumlara izin verilir";
#endif

const char *Txt_Configure_projects =
#if   L==1	// ca
	"Configurar projectes";
#elif L==2	// de
	"Konfigurieren Projekte";
#elif L==3	// en
	"Configure projects";
#elif L==4	// es
	"Configurar proyectos";
#elif L==5	// fr
	"Configurer projets";
#elif L==6	// gn
	"Omohenda umi proyecto";
#elif L==7	// it
	"Configurare progetti";
#elif L==8	// pl
	"Projekty konfiguracja";
#elif L==9	// pt
	"Configurar projetos";
#elif L==10	// tr
	"Projeleri yap&inodot;land&inodot;r";
#endif

const char *Txt_Configure_tests =
#if   L==1	// ca
	"Configurar tests";
#elif L==2	// de
	"Konfigurieren Tests";
#elif L==3	// en
	"Configure tests";
#elif L==4	// es
	"Configurar test";
#elif L==5	// fr
	"Configurer tests";
#elif L==6	// gn
	"Configurar test";	// Okoteve traduccin
#elif L==7	// it
	"Configurare test";
#elif L==8	// pl
	"Test konfiguracja";
#elif L==9	// pt
	"Configurar testes";
#elif L==10	// tr
	"Configure tests";	// eviri lazim!
#endif

const char *Txt_Confirm =
#if   L==1	// ca
	"Confirmar";
#elif L==2	// de
	"Best&auml;tigen";
#elif L==3	// en
	"Confirm";
#elif L==4	// es
	"Confirmar";
#elif L==5	// fr
	"Confirmer";
#elif L==6	// gn
	"Confirmar";	// Okoteve traduccin
#elif L==7	// it
	"Confermare";
#elif L==8	// pl
	"Potwierdzenie";
#elif L==9	// pt
	"Confirmar";
#elif L==10	// tr
	"Confirm";	// eviri lazim!
#endif

const char *Txt_Confirm_email =
#if   L==1	// ca
	"Confirmar correu";
#elif L==2	// de
	"E-Mail best&auml;tigen";
#elif L==3	// en
	"Confirm email";
#elif L==4	// es
	"Confirmar correo";
#elif L==5	// fr
	"Confirmer courrier";
#elif L==6	// gn
	"Confirmar correo";	// Okoteve traduccin
#elif L==7	// it
	"Confermare email";
#elif L==8	// pl
	"Potwierdzenie email";
#elif L==9	// pt
	"Confirmar email";
#elif L==10	// tr
	"Confirm email";	// eviri lazim!
#endif

const char *Txt_Confirm_ID =
#if   L==1	// ca
	"Confirmar ID";
#elif L==2	// de
	"ID best&auml;tigen";
#elif L==3	// en
	"Confirm ID";
#elif L==4	// es
	"Confirmar ID";
#elif L==5	// fr
	"Confirmer ID";
#elif L==6	// gn
	"Confirmar ID";	// Okoteve traduccin
#elif L==7	// it
	"Confermare ID";
#elif L==8	// pl
	"Potwierdzenie ID";
#elif L==9	// pt
	"Confirmar ID";
#elif L==10	// tr
	"Confirm ID";	// eviri lazim!
#endif

const char *Txt_Confirm_my_enrolment =
#if   L==1	// ca
	"Confirmar la meva inscripci&oacute;";
#elif L==2	// de
	"Best&auml;tigen Sie meine Registrierung";
#elif L==3	// en
	"Confirm my enrolment";
#elif L==4	// es
	"Confirmar mi inscripci&oacute;n";
#elif L==5	// fr
	"Confirmer mon inscription";
#elif L==6	// gn
	"Confirmar mi inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Confermare la mia iscrizione";
#elif L==8	// pl
	"Potwierdzenie mojej rejestracji";
#elif L==9	// pt
	"Confirmar o meu registo";
#elif L==10	// tr
	"Confirm my enrolment";			// eviri lazim!
#endif

const char *Txt_Congratulations_You_have_created_your_account_X_Now_Y_will_request_you_ =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Enhorabona! Ha creat el seu compte @%s."
	" Ara %s li demanar&agrave; algunes dades per completar el seu perfil."
	" Despr&eacute;s podr&agrave; sol&middot;licitar la seva inscripci&oacute; en assignatures."
	" Si no hi ha una assignatura,"
	" vost&egrave; pot crear abans de sol&middot;licitar la seva inscripci&oacute; en ella"
	" (el mateix &eacute;s aplicable a institucions, centers i titulacions).";
#elif L==2	// de
	"Herzlichen Gl&uuml;ckwunsch! Du hast Dein Benutzerkonto <strong>@%s</strong> erstellt."
	" Jetzt %s finden Sie einige Daten in Ihrem Profil."
	" Vervollst&auml;ndigen anfordern und dann werden Sie in der Lage,"
	" f&uuml;r die Einschreibung in Kurse an."
	" Wenn ein Kurs nicht vorhanden ist,"
	" k&ouml;nnen Sie es vor der Anwendung f&uuml;r die Immatrikulation in es zu erstellen"
	" (das gleiche gilt f&uuml;r Einrichtungen, Zentren und Grad).";
#elif L==3	// en
	"Congratulations! You have created your account <strong>@%s</strong>."
	" Now %s will request you some data to complete your profile."
	" Then you will be able to apply for enrolment in courses."
	" If a course does not exist,"
	" you can create it before applying for enrolment in it"
	" (the same applies to institutions, centers and degrees).";
#elif L==4	// es
	"&iexcl;Enhorabuena! Ha creado su cuenta <strong>@%s</strong>."
	" Ahora %s le solicitar&aacute; algunos datos para completar su perfil."
	" Usted podr&aacute; despu&eacute;s solicitar su inscripci&oacute;n en asignaturas."
	" Si una asignatura no existe,"
	" puede crearla antes de solicitar su inscripci&oacute;n en ella"
	" (lo mismo es aplicable a instituciones, centros y titulaciones).";
#elif L==5	// fr
	"F&eacute;licitations! Vous avez cr&eacute;&eacute; votre compte <strong>@%s</strong>."
	" Maintenant %s vous demander&aacute; certaines donn&eacute;es pour compl&eacute;ter votre profil."
	" Vous serez alors en mesure de demander l'inscription &agrave; des mati&egrave;res."
	" Si une mati&egrave;re ne existe pas,"
	" vous pouvez le cr&eacute;er avant de demander l'inscription en elle"
	" (la m&ecirc;me chose se applique aux institutions, centers et &eacute;tudes).";
#elif L==6	// gn
	"&iexcl;Enhorabuena! Ha creado su cuenta <strong>@%s</strong>."
	" Ahora %s le solicitar&aacute; algunos datos para completar su perfil."
	" Usted podr&aacute; despu&eacute;s solicitar su inscripci&oacute;n en asignaturas."
	" Si una asignatura no existe,"
	" puede crearla antes de solicitar su inscripci&oacute;n en ella"
	" (lo mismo es aplicable a instituciones, centros y titulaciones).";	// Okoteve traduccin
#elif L==7	// it
	"Congratulazioni! Avete creato il vostro account <strong>@%s</strong>."
	" Ora %s chieder&agrave; alcuni dati per completare il tuo profilo."
	" Allora si sar&agrave; in grado di applicare per l'iscrizione ai corsi."
	" Se un corso non esiste, &egrave; possibile crearlo prima di applicare per l'iscrizione in esso"
	" (lo stesso vale per le istituzioni, centri e gradi).";
#elif L==8	// pl
	"Gratulujemy! Utworzeniu konta <strong>@%s</strong>."
	" Teraz %s zwr&oacute;ci Ci kilka danych, aby zako&nacute;czy&cacute; sw&oacute;j profil."
	" Nast&eogon;pnie b&eogon;dzie mo&zdot;na ubiega&cacute; si&eogon; o przyj&eogon;cie na kursy."
	" Je&sacute;li kurs nie istnieje,"
	" mo&zdot;na go utworzy&cacute; przed ubieganiem si&egrave; o przyj&eogon;cie w nim"
	" (to samo odnosi si&eogon; do instytucji, plac&oacute;wek i stopni).";
#elif L==9	// pt
	"Parab&eacute;ns! Voc&ecirc; criou seu conta <strong>@%s</strong>."
	" Agora %s vai pedir-lhe alguns dados para completar o seu perfil."
	" Em seguida, voc&ecirc; ser&aacute; capaz de aplicar para a matrcula em cursos."
	" Se uma disciplina n&atilde;o existir,"
	" voc&ecirc; pode cri&aacute;-lo antes de solicitar a inscri&ccedil;&atilde;o do mesmo"
	" (o mesmo se aplica &agrave;s institui&ccedil;&otilde;es, centros e graus).";
#elif L==10	// tr
	"Congratulations! You have created your account <strong>@%s</strong>."
	" Now %s will request you some data to complete your profile."
	" Then you will be able to apply for enrolment in courses."
	" If a course does not exist,"
	" you can create it before applying for enrolment in it"
	" (the same applies to institutions, centers and degrees).";	// eviri lazim!
#endif

const char *Txt_Connected_PLURAL =
#if   L==1	// ca
	"Connectats";
#elif L==2	// de
	"Angemeldet";
#elif L==3	// en
	"Connected";
#elif L==4	// es
	"Conectados";
#elif L==5	// fr
	"Connect&eacute;s";
#elif L==6	// gn
	"Conectados";	// Okoteve traduccin
#elif L==7	// it
	"Connessi";
#elif L==8	// pl
	"Online";
#elif L==9	// pt
	"Conectados";
#elif L==10	// tr
	"Connected";	// eviri lazim!
#endif

const char *Txt_connected_PLURAL =
#if   L==1	// ca
	"connectats";
#elif L==2	// de
	"angemeldet";
#elif L==3	// en
	"connected";
#elif L==4	// es
	"conectados";
#elif L==5	// fr
	"connect&eacute;s";
#elif L==6	// gn
	"conectados";	// Okoteve traduccin
#elif L==7	// it
	"connessi";
#elif L==8	// pl
	"online";
#elif L==9	// pt
	"conectados";
#elif L==10	// tr
	"connected";	// eviri lazim!
#endif

const char *Txt_connected_SINGULAR =
#if   L==1	// ca
	"connectat";
#elif L==2	// de
	"angemeldet";
#elif L==3	// en
	"connected";
#elif L==4	// es
	"conectado";
#elif L==5	// fr
	"connect&eacute;";
#elif L==6	// gn
	"conectado";	// Okoteve traduccin
#elif L==7	// it
	"connesso";
#elif L==8	// pl
	"online";
#elif L==9	// pt
	"conectado";
#elif L==10	// tr
	"connected";	// eviri lazim!
#endif

const char *Txt_Connected_users =
#if   L==1	// ca
	"Usuaris connectats";
#elif L==2	// de
	"Angemeldet Benutzer";
#elif L==3	// en
	"Connected users";
#elif L==4	// es
	"Usuarios conectados";
#elif L==5	// fr
	"Utilisateurs connect&eacute;s";
#elif L==6	// gn
	"Umi oipor&uacute;va en l&iacute;nea";
#elif L==7	// it
	"Utenti connessi";
#elif L==8	// pl
	"U&zdot;ytkownicy online";
#elif L==9	// pt
	"Utilizadores conectados";
#elif L==10	// tr
	"Ba&gbreve;l&inodot; kullan&inodot;c&inodot;lar";
#endif

const char *Txt_Continue =
#if   L==1	// ca
	"Continuar";
#elif L==2	// de
	"Weiter";
#elif L==3	// en
	"Continue";
#elif L==4	// es
	"Continuar";
#elif L==5	// fr
	"Continuer";
#elif L==6	// gn
	"Hemo&ntilde;a";
#elif L==7	// it
	"Continua";
#elif L==8	// pl
	"Kontynuuj";
#elif L==9	// pt
	"Continuar";
#elif L==10	// tr
	"Continue";	// eviri lazim!
#endif

const char *Txt_Contract =	// As a verb
#if   L==1	// ca
	"Contreure";
#elif L==2	// de
	"Zusammenziehen";
#elif L==3	// en
	"Contract";
#elif L==4	// es
	"Contraer";
#elif L==5	// fr
	"Contracter";
#elif L==6	// gn
	"Contraer";	// Okoteve traduccin
#elif L==7	// it
	"Contratto";
#elif L==8	// pl
	"Umowy";
#elif L==9	// pt
	"Contrair";
#elif L==10	// tr
	"Contract";	// eviri lazim!
#endif

const char *Txt_Control_of_class_attendance =
#if   L==1	// ca
	"Control d'assistencia a classe";
#elif L==2	// de
	"Anwesenheitskontrolle";
#elif L==3	// en
	"Control of class attendance";
#elif L==4	// es
	"Control de asistencia a clase";
#elif L==5	// fr
	"Contr&ouml;le de la fr&eacute;quentation scolaire";
#elif L==6	// gn
	"Control de asistencia a clase";	// Okoteve traduccin
#elif L==7	// it
	"Controllo delle presenze in classe";
#elif L==8	// pl
	"Kontrola obecnosc na zajeciach";
#elif L==9	// pt
	"Controlo de assiduidade de classe";
#elif L==10	// tr
	"Control of class attendance";		// eviri lazim!
#endif

const char *Txt_Cookies =
#if   L==1	// ca
	"Cookies";
#elif L==2	// de
	"Cookies";
#elif L==3	// en
	"Cookies";
#elif L==4	// es
	"Cookies";
#elif L==5	// fr
	"Cookies";
#elif L==6	// gn
	"Cookies";
#elif L==7	// it
	"Cookies";
#elif L==8	// pl
	"Cookies";
#elif L==9	// pt
	"Cookies";
#elif L==10	// tr
	"&Ccedil;erezleri";
#endif

const char *Txt_Copy_not_allowed =
#if   L==1	// ca
	"C&ograve;pia no perm&egrave;s";
#elif L==2	// de
	"Kopier nicht erlaubt";
#elif L==3	// en
	"Copy not allowed";
#elif L==4	// es
	"Copia no permitida";
#elif L==5	// fr
	"Copie pas autoris&eacute;e";
#elif L==6	// gn
	"Copia no permitida";	// Okoteve traduccin
#elif L==7	// it
	"Copia non ammessi";
#elif L==8	// pl
	"Kopiowanie nie wolno";
#elif L==9	// pt
	"Copia n&atilde;o permitida";
#elif L==10	// tr
	"Copy not allowed";	// eviri lazim!
#endif

const char *Txt_Copy_source =
#if   L==1	// ca
	"Origen de c&ograve;pia";
#elif L==2	// de
	"Kopier-Quelle";
#elif L==3	// en
	"Copy source";
#elif L==4	// es
	"Origen de copia";
#elif L==5	// fr
	"Source de copie";
#elif L==6	// gn
	"Origen de copia";	// Okoteve traduccin
#elif L==7	// it
	"Copia sorgente";
#elif L==8	// pl
	"Kopiowanie zr&oacute;dla";
#elif L==9	// pt
	"Origem da copia";
#elif L==10	// tr
	"Copy source";		// eviri lazim!
#endif

const char *Txt_Could_not_detect_any_face_in_front_position_ =
#if   L==1	// ca
	"No s'ha detectat cap rostre en posici&oacute; frontal.";
#elif L==2	// de
	"Es konnte kein Gesicht in Frontposition erkannt werden.";
#elif L==3	// en
	"Could not detect any face in front position.";
#elif L==4	// es
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";
#elif L==5	// fr
	"Impossible de d&eacute;tecter un visage en position avant.";
#elif L==6	// gn
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rilevare qualsiasi viso in posizione frontale.";
#elif L==8	// pl
	"Nie wykryto zadnej twarzy w przednim.";
#elif L==9	// pt
	"N&atilde;o foi poss&iacute;vel detectar nenhum rosto na posi&ccedil;&atilde;o frontal.";
#elif L==10	// tr
	"Could not detect any face in front position.";				// eviri lazim!
#endif

const char *Txt_Countdown =
#if   L==1	// ca
	"Compte enrere";
#elif L==2	// de
	"Countdown";
#elif L==3	// en
	"Countdown";
#elif L==4	// es
	"Cuenta atr&aacute;s";
#elif L==5	// fr
	"Compte &agrave; rebours";
#elif L==6	// gn
	"Cuenta atr&aacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Conto alla rovescia";
#elif L==8	// pl
	"Odliczanie";
#elif L==9	// pt
	"Contagem regressiva";
#elif L==10	// tr
	"Countdown";		// eviri lazim!
#endif

const char *Txt_COUNTRIES_HELP_ORDER[Hie_NUM_ORDERS] =
	{
	[Hie_ORDER_BY_NAME] =
#if   L==1	// ca
	"Ordenar per pa&iacute;s"
#elif L==2	// de
	"Nach Land sortieren"
#elif L==3	// en
	"Sort by country"
#elif L==4	// es
	"Ordenar por pa&iacute;s"
#elif L==5	// fr
	"Trier par pays"
#elif L==6	// gn
	"Ordenar por pa&iacute;s"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per paese"
#elif L==8	// pl
	"Sortuj wedlug kraju"
#elif L==9	// pt
	"Classificar por pa&iacute;s"
#elif L==10	// tr
	"Sort by country"		// eviri lazim!
#endif
	,
	[Hie_ORDER_BY_NUM_USRS] =
#if   L==1	// ca
	"Ordenar per n&ordm; d'usuaris"
#elif L==2	// de
	"Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	"Sort by no. of users"
#elif L==4	// es
	"Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	"Trier par nombre d'utilisateurs"
#elif L==6	// gn
	"Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di utenti"
#elif L==8	// pl
	"Sortuj wedlug liczby uzytkownikow"
#elif L==9	// pt
	"Classificar por n&ordm; de utilizadores"
#elif L==10	// tr
	"Sort by no. of users"			// eviri lazim!
#endif
	};

const char *Txt_Countries_with_pending_institutions =
#if   L==1	// ca
	"Pa&iuml;sos amb institucions pendents d'activar";
#elif L==2	// de
	"L&auml;nder mit anh&auml;ngig Hochschulen";
#elif L==3	// en
	"Countries with pending institutions";
#elif L==4	// es
	"Pa&iacute;ses con instituciones pendientes de activar";
#elif L==5	// fr
	"Pays avec des &eacute;tablissements en attente";
#elif L==6	// gn
	"Pa&iacute;ses con instituciones pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Paesi con istituzioni in sospeso";
#elif L==8	// pl
	"Kraje z instytucje oczekuj&aogon;cy";
#elif L==9	// pt
	"Pa&iacute;ses com institu&ccedil;&otilde;es pendentes";
#elif L==10	// tr
	"Countries with pending institutions";				// eviri lazim!
#endif

const char *Txt_COUNTRIES_ORDER[Hie_NUM_ORDERS] =
	{
	[Hie_ORDER_BY_NAME] =
#if   L==1	// ca
	"Pa&iacute;s"
#elif L==2	// de
	"Land"
#elif L==3	// en
	"Country"
#elif L==4	// es
	"Pa&iacute;s"
#elif L==5	// fr
	"Pays"
#elif L==6	// gn
	"Tet&atilde;"
#elif L==7	// it
	"Paese"
#elif L==8	// pl
	"Kraj"
#elif L==9	// pt
	"Pa&iacute;s"
#elif L==10	// tr
	"&Uuml;lke"
#endif
	,
	[Hie_ORDER_BY_NUM_USRS] =
#if   L==1	// ca
	"Usuaris del pa&iacute;s"
#elif L==2	// de
	"Benutzer des Landes"
#elif L==3	// en
	"Users of the country"
#elif L==4	// es
	"Usuarios del pa&iacute;s"
#elif L==5	// fr
	"Utilisateurs du pays"
#elif L==6	// gn
	"Usuarios del pa&iacute;s"	// Okoteve traduccin
#elif L==7	// it
	"Utenti del paese"
#elif L==8	// pl
	"U&zdot;ytkownicy kraju"
#elif L==9	// pt
	"Utilizadores do pa&iacute;s"
#elif L==10	// tr
	"Users of the country"		// eviri lazim!
#endif
	};

const char *Txt_Country_unspecified =
#if   L==1	// ca
	"Pa&iacute;s sense especificar";
#elif L==2	// de
	"Land nicht angegeben";
#elif L==3	// en
	"Country unspecified";
#elif L==4	// es
	"Pa&iacute;s sin especificar";
#elif L==5	// fr
	"Pays non sp&eacute;cifi&eacute;";
#elif L==6	// gn
	"Pa&iacute;s sin especificar";	// Okoteve traduccin
#elif L==7	// it
	"Paese non specificato";
#elif L==8	// pl
	"Kraj nieokreslony";
#elif L==9	// pt
	"Pa&iacute;s indeterminado";
#elif L==10	// tr
	"Country unspecified";		// eviri lazim!
#endif

const char *Txt_Country_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Pa&iacute;s <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Land <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Country <strong>%s</strong> removed.";
#elif L==4	// es
	"Pa&iacute;s <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Pays <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Pa&iacute;s <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Paese <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Kraj <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Pa&iacute;s <strong>%s</strong> removido.";
#elif L==10	// tr
	"Country <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Course_forum =
#if   L==1	// ca
	"F&ograve;rum de l'assignatura";
#elif L==2	// de
	"Kursforum";
#elif L==3	// en
	"Course forum";
#elif L==4	// es
	"Foro de la asignatura";
#elif L==5	// fr
	"Forum de mati&egrave;re";
#elif L==6	// gn
	"Foro de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Forum del corso";
#elif L==8	// pl
	"Forum kursu";
#elif L==9	// pt
	"F&oacute;rum da disciplina";
#elif L==10	// tr
	"Kurs forumu";
#endif

const char *Txt_COURSE_program =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Programa de %s";
#elif L==2	// de
	"Programm %s";
#elif L==3	// en
	"%s program";
#elif L==4	// es
	"Programa de %s";
#elif L==5	// fr
	"Programme de %s";
#elif L==6	// gn
	"Programa de %s";	// Okoteve traduccin
#elif L==7	// it
	"Programma di %s";
#elif L==8	// pl
	"Program %s";
#elif L==9	// pt
	"Programa de %s";
#elif L==10	// tr
	"%s program&inodot;";
#endif

const char *Txt_COURSE_STATUS[Hie_NUM_STATUS_TXT] =
	{
	[Hie_STATUS_UNKNOWN] =
#if   L==1	// ca
	"Desconegudo"
#elif L==2	// de
	"Unbekannt"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Hie_STATUS_ACTIVE] =
#if   L==1	// ca
	"Activa"
#elif L==2	// de
	"Aktive"
#elif L==3	// en
	"Active"
#elif L==4	// es
	"Activa"
#elif L==5	// fr
	"Actif"
#elif L==6	// gn
	"Activa"	// Okoteve traduccin
#elif L==7	// it
	"Attivo"
#elif L==8	// pl
	"Aktywne"
#elif L==9	// pt
	"Ativa"
#elif L==10	// tr
	"Active"	// eviri lazim!
#endif
	,
	[Hie_STATUS_PENDING] =
#if   L==1	// ca
	"Nova"
#elif L==2	// de
	"Neu"
#elif L==3	// en
	"New"
#elif L==4	// es
	"Nueva"
#elif L==5	// fr
	"Nouvelle"
#elif L==6	// gn
	"Pyahu"
#elif L==7	// it
	"Nuovo"
#elif L==8	// pl
	"Nowy"
#elif L==9	// pt
	"Nova"
#elif L==10	// tr
	"New"	// eviri lazim!
#endif
	,
	[Hie_STATUS_REMOVED] =
#if   L==1	// ca
	"Eliminada"
#elif L==2	// de
	"Entfernt"
#elif L==3	// en
	"Removed"
#elif L==4	// es
	"Eliminada"
#elif L==5	// fr
	"Supprim&eacute;"
#elif L==6	// gn
	"Eliminada"	// Okoteve traduccin
#elif L==7	// it
	"Rimosso"
#elif L==8	// pl
	"Usuniete"
#elif L==9	// pt
	"Suprimida"
#elif L==10	// tr
	"Removed"	// eviri lazim!
#endif
	};

const char *Txt_COURSE_With_users =
#if   L==1	// ca
	"Amb usuaris";
#elif L==2	// de
	"Mit Benutzer";
#elif L==3	// en
	"With users";
#elif L==4	// es
	"Con usuarios";
#elif L==5	// fr
	"Avec utilisateurs";
#elif L==6	// gn
	"Con usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Con utenti";
#elif L==8	// pl
	"Z uzytkownikow";
#elif L==9	// pt
	"Com utilizadores";
#elif L==10	// tr
	"With users";	// eviri lazim!
#endif

const char *Txt_COURSE_Without_users =
#if   L==1	// ca
	"Sense usuaris";
#elif L==2	// de
	"Ohne Benutzer";
#elif L==3	// en
	"Without users";
#elif L==4	// es
	"Sin usuarios";
#elif L==5	// fr
	"Sans utilisateurs";
#elif L==6	// gn
	"Sin usuarios";		// Okoteve traduccin
#elif L==7	// it
	"Senza utenti";
#elif L==8	// pl
	"Bez uzytkownikow";
#elif L==9	// pt
	"Sem utilizadores";
#elif L==10	// tr
	"Without users";	// eviri lazim!
#endif

const char *Txt_Course_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Assignatura <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Kurs <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Course <strong>%s</strong> removed.";
#elif L==4	// es
	"Asignatura <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Mati&egrave;re <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Asignatura <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Corso <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Kurs <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Disciplina <strong>%s</strong> removida.";
#elif L==10	// tr
	"Course <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Courses_ABBREVIATION =
#if   L==1	// ca
	"Assig.";
#elif L==2	// de
	"Veran.";
#elif L==3	// en
	"Cours.";
#elif L==4	// es
	"Asig.";
#elif L==5	// fr
	"Mat.";
#elif L==6	// gn
	"Asig.";	// Okoteve traduccin
#elif L==7	// it
	"Corsi";
#elif L==8	// pl
	"Kurs.";
#elif L==9	// pt
	"Disc.";
#elif L==10	// tr
	"Ders.";
#endif

const char *Txt_courses_ABBREVIATION =
#if   L==1	// ca
	"assig.";
#elif L==2	// de
	"Veran.";
#elif L==3	// en
	"cours.";
#elif L==4	// es
	"asig.";
#elif L==5	// fr
	"mat.";
#elif L==6	// gn
	"asig.";	// Okoteve traduccin
#elif L==7	// it
	"corsi";
#elif L==8	// pl
	"kurs.";
#elif L==9	// pt
	"disc.";
#elif L==10	// tr
	"ders.";
#endif

const char *Txt_X_courses_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'ha(n) eliminat %u assignatura(s).";
#elif L==2	// de
	"%u Kurse wurden gestrichen.";
#elif L==3	// en
	"%u course(s) have been eliminated.";
#elif L==4	// es
	"Se ha(n) eliminado %u asignatura(s).";
#elif L==5	// fr
	"%u mati&egrave;re(s) ont &eacute;t&eacute; supprim&eacute;es.";
#elif L==6	// gn
	"Se ha(n) eliminado %u asignatura(s).";		// Okoteve traduccin
#elif L==7	// it
	"%u corsi sono stati rimossi.";
#elif L==8	// pl
	"%u kurs&oacute;w zosta&lstrok;o wyeliminowanych.";
#elif L==9	// pt
	"%u disciplina(s) foram eliminadas.";
#elif L==10	// tr
	"%u course(s) have been eliminated.";		// eviri lazim!
#endif

const char *Txt_Courses_of_DEGREE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Assignatures de %s";
#elif L==2	// de
	"Kursen der %s";
#elif L==3	// en
	"Courses of %s";
#elif L==4	// es
	"Asignaturas de %s";
#elif L==5	// fr
	"Mati&egrave;res de %s";
#elif L==6	// gn
	"Asignaturas de %s";	// Okoteve traduccin
#elif L==7	// it
	"Corsi di %s";
#elif L==8	// pl
	"Kurs&oacute;w %s";
#elif L==9	// pt
	"Disciplinas de %s";
#elif L==10	// tr
	"Courses of %s";	// eviri lazim!
#endif

const char *Txt_Create =
#if   L==1	// ca
	"Crear";
#elif L==2	// de
	"Erstellen";
#elif L==3	// en
	"Create";
#elif L==4	// es
	"Crear";
#elif L==5	// fr
	"Cr&eacute;er";
#elif L==6	// gn
	"Omohe&ntilde;&oacute;i hagu&atilde;";
#elif L==7	// it
	"Crea";
#elif L==8	// pl
	"Utw&oacute;rz";
#elif L==9	// pt
	"Criar";
#elif L==10	// tr
	"Yaratmak";
#endif

const char *Txt_Create_email_message =
#if   L==1	// ca
	"Crear missatge de correu";
#elif L==2	// de
	"Neue E-Mail-Nachricht";
#elif L==3	// en
	"Create email message";
#elif L==4	// es
	"Crear mensaje de correo";
#elif L==5	// fr
	"Cr&eacute;er un email message";
#elif L==6	// gn
	"Crear mensaje de correo";	// Okoteve traduccin
#elif L==7	// it
	"Crea messaggio email";
#elif L==8	// pl
	"Utw&oacute;rz wiadomosci email";
#elif L==9	// pt
	"Criar uma mensagem de email";
#elif L==10	// tr
	"Create email message";		// eviri lazim!
#endif

const char *Txt_Create_folder =
#if   L==1	// ca
	"Crear carpeta";
#elif L==2	// de
	"Verzeichnis anlegen";
#elif L==3	// en
	"Create folder";
#elif L==4	// es
	"Crear carpeta";
#elif L==5	// fr
	"Cr&eacute;er r&eacute;pertoire";
#elif L==6	// gn
	"Crear carpeta";	// Okoteve traduccin
#elif L==7	// it
	"Crea cartella";
#elif L==8	// pl
	"Utw&oacute;rz folder";
#elif L==9	// pt
	"Criar diret&oacute;rio";
#elif L==10	// tr
	"Create folder";	// eviri lazim!
#endif

const char *Txt_Create_link =
#if   L==1	// ca
	"Crear enlla&ccedil;";
#elif L==2	// de
	"Link erstellen";
#elif L==3	// en
	"Create link";
#elif L==4	// es
	"Crear enlace";
#elif L==5	// fr
	"Cr&eacute;er lien";
#elif L==6	// gn
	"Crear enlace";	// Okoteve traduccin
#elif L==7	// it
	"Crea link";
#elif L==8	// pl
	"Utw&oacute;rz link";
#elif L==9	// pt
	"Criar liga&ccedil;&atilde;o";
#elif L==10	// tr
	"Create link";	// eviri lazim!
#endif

const char *Txt_Create_BR_notification =
#if   L==1	// ca
	"Crear<br />notificaci&oacute;";
#elif L==2	// de
	"Anmeldung<br />eingeben";
#elif L==3	// en
	"Create<br />notification";
#elif L==4	// es
	"Crear<br />notificaci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er<br />notification";
#elif L==6	// gn
	"Crear<br />notificaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea<br />notifica";
#elif L==8	// pl
	"Utw&oacute;rz<br />powiadomienie";
#elif L==9	// pt
	"Criar<br />notifica&ccedil;&atilde;o";
#elif L==10	// tr
	"Create<br />notification";		// eviri lazim!
#endif

const char *Txt_Create_ZIP_file =
#if   L==1	// ca
	"Crear arxiu ZIP";
#elif L==2	// de
	"ZIP-Datei erstellen";
#elif L==3	// en
	"Create ZIP file";
#elif L==4	// es
	"Crear archivo ZIP";
#elif L==5	// fr
	"Cr&eacute;er fichier ZIP";
#elif L==6	// gn
	"Crear archivo ZIP";	// Okoteve traduccin
#elif L==7	// it
	"Crea file ZIP";
#elif L==8	// pl
	"Utw&oacute;rz pliku ZIP";
#elif L==9	// pt
	"Criar arquivo ZIP";
#elif L==10	// tr
	"Create ZIP file";	// eviri lazim!
#endif

const char *Txt_Created_new_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova activitat <strong>%s</strong>.";
#elif L==2	// de
	"Neue Aufgabe <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new assignment <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva actividad <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvelle activit&eacute; <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva actividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova attivit&agrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa zadania <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova atividade <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni atama <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_banner_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou banner <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Werbebanner <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new banner <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo banner <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle banni&eacute;re <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo banner <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo banner <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy banner <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo banner <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni banner <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_building_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou edifici <strong>%s</strong>.";
#elif L==2	// de
	"Neues Geb&auml;ude <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new building <strong>%s</strong>.";
#elif L==4	// es
	"Creada nuevo edificio <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau b&acirc;timent <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nuevo edificio <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo edificio <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe budynek <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo edif&iacute;cio <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni bina <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_call_for_exam =
#if   L==1	// ca
	"Creada nova convocat&ograve;ria dexamen.";
#elif L==2	// de
	"Neuer Aufruf zur Pr&uuml;fung erstellt.";
#elif L==3	// en
	"Created new call for exam.";
#elif L==4	// es
	"Creada nueva convocatoria de examen.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle convocation &agrave; un examen.";
#elif L==6	// gn
	"Creada nueva convocatoria de examen.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo appello d'esame.";
#elif L==8	// pl
	"Utworzono nowe ogloszenie egzamin.";
#elif L==9	// pt
	"Criado nova chamada para exame.";
#elif L==10	// tr
	"S&inodot;nav i&ccedil;in yeni &ccedil;a&gbreve;r&inodot; olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_center_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou centre <strong>%s</strong>.";
#elif L==2	// de
	"Neues Lehrinstitut <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new center <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo centro <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau center <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo centro <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo centro <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe centrum <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo centro <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni merkez <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_country_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou pa&iacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Neues Land <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new country <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau pays <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo paese <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa kraju <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo pa&iacute;s <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni &uuml;lke <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova assignatura <strong>%s</strong>.";
#elif L==2	// de
	"Neue Kurs <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new course <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvelle mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo corso <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa oczywiscie <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni kurs <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_criterion_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou criteri <strong>%s</strong>.";
#elif L==2	// de
	"Neues Kriterium <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new criterion <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo criterio <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau crit&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo criterio <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo criterio <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe kryterium <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo crit&eacute;rio <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni kriter <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova titulaci&oacute; <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Studiengang <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new degree <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvel &eacute;tude <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova laurea <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy poziom <strong>%s</strong>.";
#elif L==9	// pt
	"Criada novo grau <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni derece <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_department_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou departament <strong>%s</strong>.";
#elif L==2	// de
	"Neue Abteilung <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new department <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo departamento <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau d&eacute;partement <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo departamento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo dipartimento <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy dzia&lstrok; <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo departamento <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni departman <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_email_domain_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou domini de correu <strong>%s</strong>.";
#elif L==2	// de
	"Neue E-Mail-Dom&auml;ne <strong>%s</strong> angelegt.";
#elif L==3	// en
	"Created new email domain <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo dominio de correo <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvel domaine de messagerie <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo dominio de correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo campo mail <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa domena mail <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo dom&iacute;nio de email <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni e-posta etki alan&inodot; <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"reat nou esdeveniment <strong>%s</strong>.";
#elif L==2	// de
	"Neues Ereignis <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new event <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo evento <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau &eacute;v&eacute;nement <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo evento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo evento <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe wydarzenie <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo evento <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni etkinlik <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_exam_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou examen <strong>%s</strong>.";
#elif L==2	// de
	"Neues Pr&uuml;fung <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new exam <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo examen <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau examen <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo examen <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo esame <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe egzaminu <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo exame <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni s&inodot;nav <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_game_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada novo joc <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Spiel <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new game <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo juego <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau jeu <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo juego <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo gioco <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe gra <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo jogo <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni oyun <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou grup <strong>%s</strong>.";
#elif L==2	// de
	"Neue Gruppe <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new group <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo grupo <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau groupe <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo gruppo <strong>%s</strong>.";
#elif L==8	// pl
	"Stworzyl nowa grupe <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo grupo <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni grup <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_holiday_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova festivitat <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Feiertag <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new holiday <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva festividad <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle f&ecirc;te <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva festividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova festivit&agrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe swieto <strong>%s</strong>.";
#elif L==9	// pt
	"Criado nova f&eacute;ria <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni tatil <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova instituci&oacute; <strong>%s</strong>.";
#elif L==2	// de
	"Neue Hochschule <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new institution <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvel &eacute;tablissement <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova istituzione <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa instytucje <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova institu&ccedil;&atilde;o <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni kurum <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_link_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou enlla&ccedil; <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Link <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new link <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo enlace <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau lien <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo enlace <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo link <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy link <strong>%s</strong>.";
#elif L==9	// pt
	"Criado nova liga&ccedil;&atilde;o <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni ba&gbreve;lant&inodot; <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_place_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou lloc <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Standort <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new place <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo lugar <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvel emplacement <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo lugar <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova localit&grave; <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe miejsce <strong>%s</strong>.";
#elif L==9	// pt
	"Criado nova localiza&ccedil;&atilde;o <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni yer <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_plugin_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou complement <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Plugin <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new plugin <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo complemento <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau plugin <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo complemento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo plugin <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy plugin <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo plugin <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni eklenti <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_project_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou projecte <strong>%s</strong>.";
#elif L==2	// de
	"Neues Projekt <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new project <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo proyecto <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau projet <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo proyecto <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo progetto <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe projekt <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo projeto <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni proje <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_record_field_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou camp de fitxa <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Datenfeld <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new record field <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo campo de ficha <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau champ de carte <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo campo de ficha <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo campo di scheda <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe pola rekordu <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo campo de cart&atilde;o <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni kay&inodot;t alan&inodot; <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_room_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova sala <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Raum <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new room <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva sala <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle salle <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva sala <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova aula <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe klasa <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova sala <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni oda <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_rubric_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova r&uacute;brica <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Rubrik <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new rubric <strong>%s</strong>.";
#elif L==4	// es
	"Creado nueva r&uacute;brica <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle rubrique <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nueva r&uacute;brica <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova rubrica <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono now&aogon; rubryk&eogon; <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova rubrica <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni de&gbreve;erlendirme listesi <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_session_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nova sessi&oacute; <strong>%s</strong>.";
#elif L==2	// de
	"Neue Sitzung <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new session <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva sesi&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvelle session <strong>%s</strong>.";
#elif L==6	// gn
	"Ojejapo sesi&oacute;n pyahu <strong>%s</strong>.";
#elif L==7	// it
	"Creata nuova sessione <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono now&aogon; sesj&eogon; <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova sess&atilde;o <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni oturum <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_set_of_questions_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou conjunt de preguntes <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Satz von Fragen <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new set of questions <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo conjunto de preguntas <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau ensemble de questions <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo conjunto de preguntas <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo set di domande <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe zestaw pyta&nacute; <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo conjunto de perguntas <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni soru grubu <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova enquesta <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Umfrage <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new survey <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva encuesta <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau sondage <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva encuesta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo sondaggio <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe badania <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo inqu&eacute;rito <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni anket <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_type_of_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou tipus de titulaci&oacute; <strong>%s</strong>.";
#elif L==2	// de
	"Neue Abschlussart <strong>%s</strong> definiert.";
#elif L==3	// en
	"Created new type of degree <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau type d'&eacute;tude <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo tipo di laurea <strong>%s</strong>.";
#elif L==8	// pl
	"Stworzyl nowy typ stopnia <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo tipo de grau <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni tip derecesi <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_new_type_of_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou tipus de grup <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Gruppen-Typ <strong>%s</strong> definiert.";
#elif L==3	// en
	"Created new type of group <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo tipo de grupo <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau type de groupe <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo tipo de grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo tipo di gruppo <strong>%s</strong>.";
#elif L==8	// pl
	"Stworzyl nowy typ grupy <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo tipo de grupo <strong>%s</strong>.";
#elif L==10	// tr
	"Yeni tip grubu <strong>%s</strong> olu&scedil;turuldu.";
#endif

const char *Txt_Created_tables_in_the_database_that_did_not_exist =
#if   L==1	// ca
	"Creades les taules de la base de dades que no existien.";
#elif L==2	// de
	"Neue Tabellen in der Datenbank erstellt.";
#elif L==3	// en
	"Created tables in the database that did not exist.";
#elif L==4	// es
	"Creadas las tablas de la base de datos que no exist&iacute;an.";
#elif L==5	// fr
	"Tables cr&eacute;&eacute;es dans la base de donn&eacute;es qui n'existaient pas.";
#elif L==6	// gn
	"Creadas las tablas de la base de datos que no exist&iacute;an.";	// Okoteve traduccin
#elif L==7	// it
	"Tabelle create in un database inesistente.";
#elif L==8	// pl
	"Wykonawca tabel w bazie danych, kt&oacute;re nie istnialy.";
#elif L==9	// pt
	"Criadas tabelas no banco de dados que n&atilde;o existiam.";
#elif L==10	// tr
	"Created tables in the database that did not exist.";			// eviri lazim!
#endif

const char *Txt_Creating_database_tables_if_they_do_not_exist =
#if   L==1	// ca
	"Creant taules de la base de dades si no existeixen&hellip;";
#elif L==2	// de
	"Datenbanktabellen erstellen, wenn sie nicht existieren&hellip;";
#elif L==3	// en
	"Creating database tables if they do not exist&hellip;";
#elif L==4	// es
	"Creando tablas de la base de datos si no existen&hellip;";
#elif L==5	// fr
	"Cr&eacute;ation de tables de base de donn&eacute;es si elles n'existent pas&hellip;";
#elif L==6	// gn
	"Creando tablas de la base de datos si no existen&hellip;";	// Okoteve traduccin
#elif L==7	// it
	"Creazione tabelle della base di dati se non esistono&hellip;";
#elif L==8	// pl
	"Utw&oacute;rz tabel bazy danych, jesli ich nie ma &hellip;";
#elif L==9	// pt
	"Criando tabelas da base de dados, se elas n&atilde;o existirem&hellip;";
#elif L==10	// tr
	"Creating database tables if they do not exist&hellip;";	// eviri lazim!
#endif

const char *Txt_Criteria =
#if   L==1	// ca
	"Criteris";
#elif L==2	// de
	"Kriterien";
#elif L==3	// en
	"Criteria";
#elif L==4	// es
	"Criterios";
#elif L==5	// fr
	"Crit&egrave;res";
#elif L==6	// gn
	"Criterios";	// Okoteve traduccin
#elif L==7	// it
	"Criteri";
#elif L==8	// pl
	"Kryteria";
#elif L==9	// pt
	"Crit&eacute;rios";
#elif L==10	// tr
	"Kriterler";
#endif

const char *Txt_Criterion =
#if   L==1	// ca
	"Criteri";
#elif L==2	// de
	"Kriterium";
#elif L==3	// en
	"Criterion";
#elif L==4	// es
	"Criterio";
#elif L==5	// fr
	"Crit&egrave;re";
#elif L==6	// gn
	"Criterio";	// Okoteve traduccin
#elif L==7	// it
	"Criterio";
#elif L==8	// pl
	"Kryterium";
#elif L==9	// pt
	"Crit&eacute;rio";
#elif L==10	// tr
	"Kriter";
#endif

const char *Txt_Criterion_removed =
#if   L==1	// ca
	"Criteri eliminat.";
#elif L==2	// de
	"Kriterium entfernt.";
#elif L==3	// en
	"Criterion removed.";
#elif L==4	// es
	"Criterio eliminado.";
#elif L==5	// fr
	"Crit&egrave;re supprim&eacute;.";
#elif L==6	// gn
	"Criterio eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Criterio rimosso.";
#elif L==8	// pl
	"Usuni&eogon;to kryterium.";
#elif L==9	// pt
	"Crit&eacute;rio removido.";
#elif L==10	// tr
	"Kriter kald&inodot;r&inodot;ld&inodot;.";
#endif

const char *Txt_Current_email =
#if   L==1	// ca
	"Correu actual";
#elif L==2	// de
	"Aktueller E-Mail";
#elif L==3	// en
	"Current email";
#elif L==4	// es
	"Correo actual";
#elif L==5	// fr
	"Courrier actuel";
#elif L==6	// gn
	"Correo actual";	// Okoteve traduccin
#elif L==7	// it
	"Email corrente";
#elif L==8	// pl
	"Obecny email";
#elif L==9	// pt
	"Email atual";
#elif L==10	// tr
	"Current email";	// eviri lazim!
#endif

const char *Txt_Current_image_video =
#if   L==1	// ca
	"Imatge / v&iacute;deo actual";
#elif L==2	// de
	"Aktuelles Abbild / Video";
#elif L==3	// en
	"Current image / video";
#elif L==4	// es
	"Imagen / v&iacute;deo actual";
#elif L==5	// fr
	"Image / vid&eacute;o actuelle";
#elif L==6	// gn
	"Imagen / v&iacute;deo actual";	// Okoteve traduccin
#elif L==7	// it
	"Immagine / video corrente";
#elif L==8	// pl
	"Bie&zdot;&aogon;cy obraz / wideo";
#elif L==9	// pt
	"Imagem / v&iacute;deo atual";
#elif L==10	// tr
	"Current image / video";	// eviri lazim!
#endif

const char *Txt_Current_nickname =
#if   L==1	// ca
	"Sobrenom actual";
#elif L==2	// de
	"Aktueller Benutzername";
#elif L==3	// en
	"Current nickname";
#elif L==4	// es
	"Apodo actual";
#elif L==5	// fr
	"Surnom actuel";
#elif L==6	// gn
	"Apodo actual";		// Okoteve traduccin
#elif L==7	// it
	"Nome utente corrente";
#elif L==8	// pl
	"Obecny pseudonim";
#elif L==9	// pt
	"Alcunha atual";
#elif L==10	// tr
	"Current nickname";	// eviri lazim!
#endif

const char *Txt_Current_password =
#if   L==1	// ca
	"Contrasenya actual";
#elif L==2	// de
	"Aktuelles Passwort";
#elif L==3	// en
	"Current password";
#elif L==4	// es
	"Contrase&ntilde;a actual";
#elif L==5	// fr
	"Mot de passe actuel";
#elif L==6	// gn
	"Contrase&ntilde;a actual";	// Okoteve traduccin
#elif L==7	// it
	"Password corrente";
#elif L==8	// pl
	"Obecne has&lstrok;o";
#elif L==9	// pt
	"Senha atual";
#elif L==10	// tr
	"Mevcut &scedil;ifre";
#endif

const char *Txt_DATA_PROTECTION_POLICY =
#if   L==1	// ca
	"Pol&iacute;tica de protecci&oacute; de dades";
#elif L==2	// de
	"Datenschutzrichtlinie";
#elif L==3	// en
	"Data protection policy";
#elif L==4	// es
	"Pol&iacute;tica de protecci&oacute;n de datos";
#elif L==5	// fr
	"Clause de protection des donn&eacute;es";
#elif L==6	// gn
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Okoteve traduccin
#elif L==7	// it
	"Politica sulla protezione dei dati";
#elif L==8	// pl
	"Polityka ochrony danych";
#elif L==9	// pt
	"Pol&iacute;tica de prote&ccedil;&atilde;o de dados";
#elif L==10	// tr
	"Data protection policy";				// eviri lazim!
#endif

const char *Txt_Date =
#if   L==1	// ca
	"Data";
#elif L==2	// de
	"Datum";
#elif L==3	// en
	"Date";
#elif L==4	// es
	"Fecha";
#elif L==5	// fr
	"Date";
#elif L==6	// gn
	"Arange";
#elif L==7	// it
	"Data";
#elif L==8	// pl
	"Data";
#elif L==9	// pt
	"Data";
#elif L==10	// tr
	"Date";	// eviri lazim!
#endif

const char *Txt_Date_and_time =
#if   L==1	// ca
	"Data i hora";
#elif L==2	// de
	"Datum und Uhrzeit";
#elif L==3	// en
	"Date and time";
#elif L==4	// es
	"Fecha y hora";
#elif L==5	// fr
	"Date et heure";
#elif L==6	// gn
	"Fecha y hora";		// Okoteve traduccin
#elif L==7	// it
	"Data e ora";
#elif L==8	// pl
	"Data i godzina";
#elif L==9	// pt
	"Data e hora";
#elif L==10	// tr
	"Date and time";	// eviri lazim!
#endif

const char *Txt_Date_of_birth =
#if   L==1	// ca
	"Data naixement";
#elif L==2	// de
	"Geburtsdatum";
#elif L==3	// en
	"Date of birth";
#elif L==4	// es
	"Fecha nacimiento";
#elif L==5	// fr
	"Date naissance";
#elif L==6	// gn
	"Fecha nacimiento";	// Okoteve traduccin
#elif L==7	// it
	"Data di nascita";
#elif L==8	// pl
	"Data urodzenia";
#elif L==9	// pt
	"Data nascimento";
#elif L==10	// tr
	"Date of birth";	// eviri lazim!
#endif

const char *Txt_Date_of_creation =
#if   L==1	// ca
	"Data de creaci&oacute;";
#elif L==2	// de
	"Datum der Erstellung";
#elif L==3	// en
	"Date of creation";
#elif L==4	// es
	"Fecha de creaci&oacute;n";
#elif L==5	// fr
	"Date de cr&eacute;ation";
#elif L==6	// gn
	"Fecha de creaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Data di creazione";
#elif L==8	// pl
	"Data utworzenia";
#elif L==9	// pt
	"Data da cria&ccedil;&atilde;o";
#elif L==10	// tr
	"Date of creation";		// eviri lazim!
#endif

const char *Txt_Dates =
#if   L==1	// ca
	"Dates";
#elif L==2	// de
	"Datum";
#elif L==3	// en
	"Dates";
#elif L==4	// es
	"Fechas";
#elif L==5	// fr
	"Dates";
#elif L==6	// gn
	"Arange";
#elif L==7	// it
	"Date";
#elif L==8	// pl
	"Daty";
#elif L==9	// pt
	"Datas";
#elif L==10	// tr
	"Tarihleri";
#endif

const char *Txt_Day =
#if   L==1	// ca
	"Dia";
#elif L==2	// de
	"Tag";
#elif L==3	// en
	"Day";
#elif L==4	// es
	"D&iacute;a";
#elif L==5	// fr
	"Jour";
#elif L==6	// gn
	"&Aacute;ra";
#elif L==7	// it
	"Giorno";
#elif L==8	// pl
	"Dzien";
#elif L==9	// pt
	"Dia";
#elif L==10	// tr
	"Day";	// eviri lazim!
#endif

const char *Txt_day =
#if   L==1	// ca
	"dia";
#elif L==2	// de
	"Tag";
#elif L==3	// en
	"day";
#elif L==4	// es
	"d&iacute;a";
#elif L==5	// fr
	"jour";
#elif L==6	// gn
	"&aacute;ra";
#elif L==7	// it
	"giorno";
#elif L==8	// pl
	"dzie&nacute;";
#elif L==9	// pt
	"dia";
#elif L==10	// tr
	"day";	// eviri lazim!
#endif

const char *Txt_days =
#if   L==1	// ca
	"dies";
#elif L==2	// de
	"Tage";
#elif L==3	// en
	"days";
#elif L==4	// es
	"d&iacute;as";
#elif L==5	// fr
	"jours";
#elif L==6	// gn
	"&aacute;ra";
#elif L==7	// it
	"giorni";
#elif L==8	// pl
	"dni;";
#elif L==9	// pt
	"dias";
#elif L==10	// tr
	"days";	// eviri lazim!
#endif

const char *Txt_DAYS[7] =
	{
#if   L==1	// ca
	"dilluns"
#elif L==2	// de
	"Montag"
#elif L==3	// en
	"Monday"
#elif L==4	// es
	"lunes"
#elif L==5	// fr
	"Lundi"
#elif L==6	// gn
	"arak&otilde;i"
#elif L==7	// it
	"Luned&igrave;"
#elif L==8	// pl
	"Poniedzialek"
#elif L==9	// pt
	"Segunda-feira"
#elif L==10	// tr
	"Pazartesi"
#endif
	,
#if   L==1	// ca
	"dimarts"
#elif L==2	// de
	"Dienstag"
#elif L==3	// en
	"Tuesday"
#elif L==4	// es
	"martes"
#elif L==5	// fr
	"Mardi"
#elif L==6	// gn
	"araapy"
#elif L==7	// it
	"Marted&igrave;"
#elif L==8	// pl
	"Wtorek"
#elif L==9	// pt
	"Ter&ccedil;a-feira"
#elif L==10	// tr
	"Sal&inodot;"
#endif
	,
#if   L==1	// ca
	"dimecres"
#elif L==2	// de
	"Mittwoch"
#elif L==3	// en
	"Wednesday"
#elif L==4	// es
	"mi&eacute;rcoles"
#elif L==5	// fr
	"Mercredi"
#elif L==6	// gn
	"ararundy"
#elif L==7	// it
	"Mercoled&igrave;"
#elif L==8	// pl
	"Sroda"
#elif L==9	// pt
	"Quarta-feira"
#elif L==10	// tr
	"&Ccedil;ar&scedil;amba"
#endif
	,
#if   L==1	// ca
	"dijous"
#elif L==2	// de
	"Donnerstag"
#elif L==3	// en
	"Thursday"
#elif L==4	// es
	"jueves"
#elif L==5	// fr
	"Jeudi"
#elif L==6	// gn
	"arapo"
#elif L==7	// it
	"Gioved&igrave;"
#elif L==8	// pl
	"Czwartek"
#elif L==9	// pt
	"Quinta-feira"
#elif L==10	// tr
	"Per&scedil;embe"
#endif
	,
#if   L==1	// ca
	"divendres"
#elif L==2	// de
	"Freitag"
#elif L==3	// en
	"Friday"
#elif L==4	// es
	"viernes"
#elif L==5	// fr
	"Vendredi"
#elif L==6	// gn
	"arapote&#297;"
#elif L==7	// it
	"Venerd&igrave;"
#elif L==8	// pl
	"Piatek"
#elif L==9	// pt
	"Sexta-feira"
#elif L==10	// tr
	"Cuma"
#endif
	,
#if   L==1	// ca
	"dissabte"
#elif L==2	// de
	"Samstag"
#elif L==3	// en
	"Saturday"
#elif L==4	// es
	"s&aacute;bado"
#elif L==5	// fr
	"Samedi"
#elif L==6	// gn
	"arapok&otilde;i"
#elif L==7	// it
	"Sabato"
#elif L==8	// pl
	"Sobota"
#elif L==9	// pt
	"Sabado"
#elif L==10	// tr
	"Cumartesi"
#endif
	,
#if   L==1	// ca
	"diumenge"
#elif L==2	// de
	"Sonntag"
#elif L==3	// en
	"Sunday"
#elif L==4	// es
	"domingo"
#elif L==5	// fr
	"Dimanche"
#elif L==6	// gn
	"arate&#297;"
#elif L==7	// it
	"Domenica"
#elif L==8	// pl
	"Niedziela"
#elif L==9	// pt
	"Domingo"
#elif L==10	// tr
	"Pazar"
#endif
	};

const char *Txt_DAYS2[7] =
	{
#if   L==1	// ca
	"di"
#elif L==2	// de
	"Mo"
#elif L==3	// en
	"Mo"
#elif L==4	// es
	"lu"
#elif L==5	// fr
	"Lu"
#elif L==6	// gn
	"ar"
#elif L==7	// it
	"Lu"
#elif L==8	// pl
	"Po"
#elif L==9	// pt
	"Se"
#elif L==10	// tr
	"Pa"
#endif
	,
#if   L==1	// ca
	"di"
#elif L==2	// de
	"Di"
#elif L==3	// en
	"Tu"
#elif L==4	// es
	"ma"
#elif L==5	// fr
	"Ma"
#elif L==6	// gn
	"ar"
#elif L==7	// it
	"Ma"
#elif L==8	// pl
	"Wt"
#elif L==9	// pt
	"Te"
#elif L==10	// tr
	"Sa"
#endif
	,
#if   L==1	// ca
	"di"
#elif L==2	// de
	"Mi"
#elif L==3	// en
	"We"
#elif L==4	// es
	"mi"
#elif L==5	// fr
	"Me"
#elif L==6	// gn
	"ar"
#elif L==7	// it
	"Me"
#elif L==8	// pl
	"Sr"
#elif L==9	// pt
	"Qu"
#elif L==10	// tr
	"&Ccedil;a"
#endif
	,
#if   L==1	// ca
	"di"
#elif L==2	// de
	"Do"
#elif L==3	// en
	"Th"
#elif L==4	// es
	"ju"
#elif L==5	// fr
	"Je"
#elif L==6	// gn
	"ar"
#elif L==7	// it
	"Gi"
#elif L==8	// pl
	"Cz"
#elif L==9	// pt
	"Qu"
#elif L==10	// tr
	"Pe"
#endif
	,
#if   L==1	// ca
	"di"
#elif L==2	// de
	"Fr"
#elif L==3	// en
	"Fr"
#elif L==4	// es
	"vi"
#elif L==5	// fr
	"Ve"
#elif L==6	// gn
	"ar"
#elif L==7	// it
	"Ve"
#elif L==8	// pl
	"Pi"
#elif L==9	// pt
	"Se"
#elif L==10	// tr
	"Cu"
#endif
	,
#if   L==1	// ca
	"di"
#elif L==2	// de
	"Sa"
#elif L==3	// en
	"Sa"
#elif L==4	// es
	"s&aacute;"
#elif L==5	// fr
	"Sa"
#elif L==6	// gn
	"ar"
#elif L==7	// it
	"Sa"
#elif L==8	// pl
	"So"
#elif L==9	// pt
	"Sa"
#elif L==10	// tr
	"Cu"
#endif
	,
#if   L==1	// ca
	"di"
#elif L==2	// de
	"So"
#elif L==3	// en
	"Su"
#elif L==4	// es
	"do"
#elif L==5	// fr
	"Di"
#elif L==6	// gn
	"ar"
#elif L==7	// it
	"Do"
#elif L==8	// pl
	"Ni"
#elif L==9	// pt
	"Do"
#elif L==10	// tr
	"Pa"
#endif
	};

const char *Txt_default =
#if   L==1	// ca
	"predeterminat";
#elif L==2	// de
	"Vorgabe";
#elif L==3	// en
	"default";
#elif L==4	// es
	"predeterminado";
#elif L==5	// fr
	"d&eacute;faut";
#elif L==6	// gn
	"predeterminado";	// Okoteve traduccin
#elif L==7	// it
	"predefinito";
#elif L==8	// pl
	"Domyslnie";
#elif L==9	// pt
	"padr&atilde;o";
#elif L==10	// tr
	"default";	// eviri lazim!
#endif

const char *Txt_DEGREE_STATUS[Hie_NUM_STATUS_TXT] =
	{
	[Hie_STATUS_UNKNOWN] =
#if   L==1	// ca
	"Desconegudo"
#elif L==2	// de
	"Unbekannt"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Hie_STATUS_ACTIVE] =
#if   L==1	// ca
	"Activa"
#elif L==2	// de
	"Aktive"
#elif L==3	// en
	"Active"
#elif L==4	// es
	"Activa"
#elif L==5	// fr
	"Actif"
#elif L==6	// gn
	"Activa"	// Okoteve traduccin
#elif L==7	// it
	"Attiva"
#elif L==8	// pl
	"Aktywne"
#elif L==9	// pt
	"Ativa"
#elif L==10	// tr
	"Active"	// eviri lazim!
#endif
	,
	[Hie_STATUS_PENDING] =
#if   L==1	// ca
	"Nova"
#elif L==2	// de
	"Neu"
#elif L==3	// en
	"New"
#elif L==4	// es
	"Nueva"
#elif L==5	// fr
	"Nouveau"
#elif L==6	// gn
	"Pyahu"
#elif L==7	// it
	"Nuovo"
#elif L==8	// pl
	"Nowy"
#elif L==9	// pt
	"Nova"
#elif L==10	// tr
	"New"	// eviri lazim!
#endif
	,
	[Hie_STATUS_REMOVED] =
#if   L==1	// ca
	"Eliminada"
#elif L==2	// de
	"Entfernt"
#elif L==3	// en
	"Removed"
#elif L==4	// es
	"Eliminada"
#elif L==5	// fr
	"Supprim&eacute;"
#elif L==6	// gn
	"Eliminada"	// Okoteve traduccin
#elif L==7	// it
	"Rimossa"
#elif L==8	// pl
	"Usuniete"
#elif L==9	// pt
	"Suprimida"
#elif L==10	// tr
	"Removed"	// eviri lazim!
#endif
	};

const char *Txt_DEGREE_TYPES_HELP_ORDER[DegTyp_NUM_ORDERS] =
	{
	[DegTyp_ORDER_BY_DEGREE_TYPE] =
#if   L==1	// ca
	"Ordenar por tipus de titulaci&oacute;"
#elif L==2	// de
	"Nach Art des Grades sortieren"
#elif L==3	// en
	"Sort by type of degree"
#elif L==4	// es
	"Ordenar por tipo de titulaci&oacute;n"
#elif L==5	// fr
	"Trier par type d'&eacute;tude"
#elif L==6	// gn
	"Ordenar por tipo de titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per tipo di laurea"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug rodzaju stopnia"
#elif L==9	// pt
	"Classificar por tipo de grau"
#elif L==10	// tr
	"Sort by type of degree"		// eviri lazim!
#endif
	,
	[DegTyp_ORDER_BY_NUM_DEGREES] =
#if   L==1	// ca
	"Ordenar per nombre de titulacions"
#elif L==2	// de
	"Nach Anzahl der Studieng&auml;nge sortieren"
#elif L==3	// en
	"Sort by no. of degrees"
#elif L==4	// es
	"Ordenar por n&ordm; de titulaciones"
#elif L==5	// fr
	"Trier par nombre d'&eacute;tudes"
#elif L==6	// gn
	"Ordenar por n&ordm; de titulaciones"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di lauree"
#elif L==8	// pl
	"Sortuj wedlug stopni"
#elif L==9	// pt
	"Classificar por n&ordm; de graus"
#elif L==10	// tr
	"Sort by no. of degrees"		// eviri lazim!
#endif
	};

const char *Txt_DEGREE_TYPES_ORDER[DegTyp_NUM_ORDERS] =
	{
	[DegTyp_ORDER_BY_DEGREE_TYPE] =
#if   L==1	// ca
	"Tipus de titulaci&oacute;"
#elif L==2	// de
	"Abschlussart"
#elif L==3	// en
	"Types of degree"
#elif L==4	// es
	"Tipos de titulaci&oacute;n"
#elif L==5	// fr
	"Types d'&eacute;tude"
#elif L==6	// gn
	"Arandur&atilde; l&aacute;ja"
#elif L==7	// it
	"Tipi di laurea"
#elif L==8	// pl
	"Rodzaje stopni"
#elif L==9	// pt
	"Tipos de grau"
#elif L==10	// tr
	"Types of degree"	// eviri lazim!
#endif
	,
	[DegTyp_ORDER_BY_NUM_DEGREES] =
#if   L==1	// ca
	"Titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degrees"
#elif L==4	// es
	"Titulaciones"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#elif L==10	// tr
	"Derece"
#endif
	};

const char *Txt_DEGREE_With_courses =
#if   L==1	// ca
	"Amb assignatures";
#elif L==2	// de
	"Mit Kursen";
#elif L==3	// en
	"With courses";
#elif L==4	// es
	"Con asignaturas";
#elif L==5	// fr
	"Avec des mati&egrave;res";
#elif L==6	// gn
	"Con asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Con corsi";
#elif L==8	// pl
	"Z kurs&oacute;w";
#elif L==9	// pt
	"Com disciplinas";
#elif L==10	// tr
	"With courses";		// eviri lazim!
#endif

const char *Txt_DEGREE_Without_courses =
#if   L==1	// ca
	"Sense assignatures";
#elif L==2	// de
	"Ohne Kursen";
#elif L==3	// en
	"Without courses";
#elif L==4	// es
	"Sin asignaturas";
#elif L==5	// fr
	"Sans mati&egrave;res";
#elif L==6	// gn
	"Sin asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Senza corsi";
#elif L==8	// pl
	"Bez kurs&oacute;w";
#elif L==9	// pt
	"Sem disciplinas";
#elif L==10	// tr
	"Without courses";	// eviri lazim!
#endif

const char *Txt_Degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Titulaci&oacute; <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Studiengang <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Degree <strong>%s</strong> removed.";
#elif L==4	// es
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Degr&eacute; <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Laurea <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Studia <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Grau <strong>%s</strong> removido.";
#elif L==10	// tr
	"Degree <strong>%s</strong> removed.";			// eviri lazim!
#endif

const char *Txt_Degrees_ABBREVIATION =
#if   L==1	// ca
	"Titul.";
#elif L==2	// de
	"Studieng.";
#elif L==3	// en
	"Degrees";
#elif L==4	// es
	"Titul.";
#elif L==5	// fr
	"&Eacute;tudes";
#elif L==6	// gn
	"Titul.";	// Okoteve traduccin
#elif L==7	// it
	"Lauree";
#elif L==8	// pl
	"Stopni";
#elif L==9	// pt
	"Titul.";
#elif L==10	// tr
	"Derece";
#endif

const char *Txt_Degrees_of_CENTER_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Titulacions de %s";
#elif L==2	// de
	"Studiengang von %s";
#elif L==3	// en
	"Degrees of %s";
#elif L==4	// es
	"Titulaciones de %s";
#elif L==5	// fr
	"&Eacute;tudes de %s";
#elif L==6	// gn
	"Titulaciones de %s";	// Okoteve traduccin
#elif L==7	// it
	"Lauree di %s";
#elif L==8	// pl
	"Stopnie %s";
#elif L==9	// pt
	"Graus de %s";
#elif L==10	// tr
	"Degrees of %s";	// eviri lazim!
#endif

const char *Txt_Degrees_with_pending_courses =
#if   L==1	// ca
	"Titulacions amb assignatures pendents d'activar";
#elif L==2	// de
	"Studiengang mit anh&auml;ngig Kursen";
#elif L==3	// en
	"Degrees with pending courses";
#elif L==4	// es
	"Titulaciones con asignaturas pendientes de activar";
#elif L==5	// fr
	"&Eacute;tudes avec des mati&egrave;res en attente";
#elif L==6	// gn
	"Titulaciones con asignaturas pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Lauree con corsi in sospeso";
#elif L==8	// pl
	"Stopnie z kurs&otilde;w oczekuj&aogon;cy";
#elif L==9	// pt
	"Graus com disciplinas pendentes";
#elif L==10	// tr
	"Degrees with pending courses";	// eviri lazim!
#endif

const char *Txt_Delete_messages_received =
#if   L==1	// ca
	"Eliminar missatges rebuts";
#elif L==2	// de
	"Entfernen empfangene Nachrichten";
#elif L==3	// en
	"Delete messages received";
#elif L==4	// es
	"Eliminar mensajes recibidos";
#elif L==5	// fr
	"&Eacute;liminer messages re&ccedil;us";
#elif L==6	// gn
	"Eliminar mensajes recibidos";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere messaggi ricevuti";
#elif L==8	// pl
	"Usu&nacute; wiadomo&sacute;ci otrzymane";
#elif L==9	// pt
	"Eliminar mensagens recebidas";
#elif L==10	// tr
	"Delete messages received";	// eviri lazim!
#endif

const char *Txt_Delete_messages_sent =
#if   L==1	// ca
	"Eliminar missatges enviats";
#elif L==2	// de
	"Entfernen gesendete Nachrichten";
#elif L==3	// en
	"Delete messages sent";
#elif L==4	// es
	"Eliminar mensajes enviados";
#elif L==5	// fr
	"&Eacute;liminer messages envoy&eacute;s";
#elif L==6	// gn
	"Eliminar mensajes enviados";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere messaggi inviati";
#elif L==8	// pl
	"Usu&nacute; wiadomo&sacute;ci wyslanych";
#elif L==9	// pt
	"Eliminar mensagens enviadas";
#elif L==10	// tr
	"Delete messages sent";		// eviri lazim!
#endif

const char *Txt_Delete_my_answer =
#if   L==1	// ca
	"Eliminar la meva resposta";
#elif L==2	// de
	"L&ouml;sche meine Antwort";
#elif L==3	// en
	"Delete my answer";
#elif L==4	// es
	"Eliminar mi respuesta";
#elif L==5	// fr
	"Supprimer ma r&eacute;ponse";
#elif L==6	// gn
	"Eliminar mi respuesta";	// Okoteve traduccin
#elif L==7	// it
	"Elimina la mia risposta";
#elif L==8	// pl
	"Usu&nacute; moj&aogon; odpowied&zacute;";
#elif L==9	// pt
	"Excluir minha resposta";
#elif L==10	// tr
	"Delete my answer";		// eviri lazim!
#endif

const char *Txt_Department =
#if   L==1	// ca
	"Departament";
#elif L==2	// de
	"Abteilung";
#elif L==3	// en
	"Department";
#elif L==4	// es
	"Departamento";
#elif L==5	// fr
	"D&eacute;partement";
#elif L==6	// gn
	"Peh&#7869;";
#elif L==7	// it
	"Dipartimento";
#elif L==8	// pl
	"Wydzia&lstrok;";
#elif L==9	// pt
	"Departamento";
#elif L==10	// tr
	"Department";	// eviri lazim!
#endif

const char *Txt_Department_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Departament <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Abteilung <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Department <strong>%s</strong> removed.";
#elif L==4	// es
	"Departamento <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"D&eacute;partement <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Departamento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Dipartamento <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Zaklad <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Departamento <strong>%s</strong> removido.";
#elif L==10	// tr
	"Department <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_Department_unspecified =
#if   L==1	// ca
	"Departament sense especificar";
#elif L==2	// de
	"Abteilung nicht angegeben";
#elif L==3	// en
	"Department unspecified";
#elif L==4	// es
	"Departamento sin especificar";
#elif L==5	// fr
	"D&eacute;partement non sp&eacute;cifi&eacute;";
#elif L==6	// gn
	"Departamento sin especificar";	// Okoteve traduccin
#elif L==7	// it
	"Dipartimento non specificato";
#elif L==8	// pl
	"Departament nieokreslone";
#elif L==9	// pt
	"Departamento indeterminado";
#elif L==10	// tr
	"Department unspecified";	// eviri lazim!
#endif

const char *Txt_Departments =
#if   L==1	// ca
	"Departaments";
#elif L==2	// de
	"Abteilungen";
#elif L==3	// en
	"Departments";
#elif L==4	// es
	"Departamentos";
#elif L==5	// fr
	"D&eacute;partements";
#elif L==6	// gn
	"Peh&#7869;";
#elif L==7	// it
	"Dipartimenti";
#elif L==8	// pl
	"Wydzia&lstrok;y";
#elif L==9	// pt
	"Departamentos";
#elif L==10	// tr
	"B&ouml;l&uuml;mler";
#endif

const char *Txt_Departments_of_INSTITUTION_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Departaments de %s";
#elif L==2	// de
	"Abteilungen von %s";
#elif L==3	// en
	"Departments of %s";
#elif L==4	// es
	"Departamentos de %s";
#elif L==5	// fr
	"D&eacute;partements de %s";
#elif L==6	// gn
	"Peh&#7869; de %s";	// Okoteve traduccin
#elif L==7	// it
	"Dipartimenti di %s";
#elif L==8	// pl
	"Wydzia&lstrok;y %s";
#elif L==9	// pt
	"Departamentos de %s";
#elif L==10	// tr
	"Departments of %s";	// eviri lazim!
#endif

const char *Txt_Departments_ABBREVIATION =
#if   L==1	// ca
	"Depar.";
#elif L==2	// de
	"Abteilung";
#elif L==3	// en
	"Depar";
#elif L==4	// es
	"Depar.";
#elif L==5	// fr
	"Depar.";
#elif L==6	// gn
	"Depar.";	// Okoteve traduccin
#elif L==7	// it
	"Dipar.";
#elif L==8	// pl
	"Wydzialy";
#elif L==9	// pt
	"Depar.";
#elif L==10	// tr
	"Depar";	// eviri lazim!
#endif

const char *Txt_DEPARTMENTS_HELP_ORDER[Dpt_NUM_ORDERS] =
	{
	[Dpt_ORDER_BY_DEPARTMENT] =
#if   L==1	// ca
	"Ordenar pel departament"
#elif L==2	// de
	"Sortieren nach Abteilung"
#elif L==3	// en
	"Sort by department"
#elif L==4	// es
	"Ordenar por departamento"
#elif L==5	// fr
	"Trier par d&eacute;partement"
#elif L==6	// gn
	"Ordenar por departamento"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per dipartimento"
#elif L==8	// pl
	"Sortuj wedlug dzial&oacute;w"
#elif L==9	// pt
	"Classificar por departamento"
#elif L==10	// tr
	"Sort by department"		// eviri lazim!
#endif
	,
	[Dpt_ORDER_BY_NUM_TCHS] =
#if   L==1	// ca
	"Ordenar pel nombre de professors"
#elif L==2	// de
	"Sortieren nach Anzahl der Lehrkr&auml;fte"
#elif L==3	// en
	"Sort by no. of teachers"
#elif L==4	// es
	"Ordenar por n&ordm; de profesores"
#elif L==5	// fr
	"Trier par nombre d'enseignants"
#elif L==6	// gn
	"Ordenar por n&ordm; de profesores"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di professori"
#elif L==8	// pl
	"Sortuj wedlug liczby nauczycieli"
#elif L==9	// pt
	"Classificar por n&ordm; de professores"
#elif L==10	// tr
	"Sort by no. of teachers"		// eviri lazim!
#endif
	};

const char *Txt_DEPARTMENTS_ORDER[Dpt_NUM_ORDERS] =
	{
	[Dpt_ORDER_BY_DEPARTMENT] =
#if   L==1	// ca
	"Departament"
#elif L==2	// de
	"Abteilung"
#elif L==3	// en
	"Department"
#elif L==4	// es
	"Departamento"
#elif L==5	// fr
	"D&eacute;partement"
#elif L==6	// gn
	"Peh&#7869;"
#elif L==7	// it
	"Dipartimento"
#elif L==8	// pl
	"Wydzia&lstrok;"
#elif L==9	// pt
	"Departamento"
#elif L==10	// tr
	"Department"	// eviri lazim!
#endif
	,
	[Dpt_ORDER_BY_NUM_TCHS] =
#if   L==1	// ca
	"Professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Profesores"	// Okoteve traduccin
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczycieli"
#elif L==9	// pt
	"Professores"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	};

const char *Txt_Description =
#if   L==1	// ca
	"Descripci&oacute;";
#elif L==2	// de
	"Beschreibung";
#elif L==3	// en
	"Description";
#elif L==4	// es
	"Descripci&oacute;n";
#elif L==5	// fr
	"Description";
#elif L==6	// gn
	"Descripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Descrizione";
#elif L==8	// pl
	"Opis";
#elif L==9	// pt
	"Descri&ccedil;&atilde;o";
#elif L==10	// tr
	"Description";		// eviri lazim!
#endif

const char *Txt_Design =
#if   L==1	// ca
	"Disseny";
#elif L==2	// de
	"Design";
#elif L==3	// en
	"Design";
#elif L==4	// es
	"Dise&ntilde;o";
#elif L==5	// fr
	"Design";
#elif L==6	// gn
	"Ta'&atilde;ngahai";
#elif L==7	// it
	"Disegno";
#elif L==8	// pl
	"Design";
#elif L==9	// pt
	"Design";
#elif L==10	// tr
	"Design";	// eviri lazim!
#endif

const char *Txt_Details =
#if   L==1	// ca
	"Detalls";
#elif L==2	// de
	"Einzelheiten";
#elif L==3	// en
	"Details";
#elif L==4	// es
	"Detalles";
#elif L==5	// fr
	"D&eacute;tails";
#elif L==6	// gn
	"Sa'&ytilde;ijo pete&itilde;te&itilde;";
#elif L==7	// it
	"Dettagli";
#elif L==8	// pl
	"Szczeg&oacute;&lstrok;y";
#elif L==9	// pt
	"Detalhes";
#elif L==10	// tr
	"Detaylar";
#endif

const char *Txt_Disclaimer_the_files_hosted_here_ =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Av&iacute;s legal: els arxius allotjats aqu&iacute;,"
	" salvo que indica el contrari,"
	" estan subjectes als drets de propietat intellectual"
	" i la seva titularitat correspone als usuaris que han subido."
	" %s no es responsabilitza de la informaci&oacute; contenida en dichos arxius."
	" Si vost&egrave; creu convenient retirar qualsevol fitxer"
	" que contingui contingut li pertenezca"
	" o que infringeix la llei, pot comunicar-lo a %s.";
#elif L==2	// de
	"Haftungsausschluss: Die hier gehosteten Dateien unterliegen,"
	" sofern nicht anders angegeben,"
	" geistigen Eigentumsrechten und ihr Eigentum geh&ouml;rt den Benutzern,"
	" die sie hochgeladen haben."
	" %s ist nicht verantwortlich f&uuml;r die in diesen Dateien enthaltenen Informationen."
	" Wenn Sie es f&uuml;r angebracht halten, eine Datei zu entfernen,"
	" deren Inhalt Ihnen geh&ouml;rt oder die gegen das Gesetz verst&ouml;&szlig;t,"
	" k&ouml;nnen Sie sich an %s wenden.";
#elif L==3	// en
	"Disclaimer: the files hosted here,"
	" unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file"
	" whose contents belongs to you"
	" or that violates the law, you can contact %s.";
#elif L==4	// es
	"Aviso legal: los archivos alojados aqu&iacute;,"
	" salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo"
	" cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";
#elif L==5	// fr
	"Avis de non-responsabilit&eacute;: les fichiers h&eacute;berg&eacute;s ici,"
	" sauf indication contraire,"
	" sont soumis &agrave; des droits de propri&eacute;t&eacute; intellectuelle"
	" et leur propri&eacute;t&eacute; appartient aux utilisateurs qui les ont t&eacute;l&eacute;charg&eacute;s."
	" %s n'est pas responsable des informations contenues dans ces fichiers."
	" Si vous jugez opportun de supprimer tout fichier"
	" dont le contenu vous appartient"
	" ou qui enfreint la loi, vous pouvez contacter %s.";
#elif L==6	// gn
	"Aviso legal: los archivos alojados aqu&iacute;,"
	" salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo"
	" cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";	// Okoteve traduccin
#elif L==7	// it
	"Disclaimer: i file qui ospitati,"
	" salvo diversa indicazione,"
	" sono soggetti a diritti di propriet intellettuale"
	" e la loro propriet appartiene agli utenti che li hanno caricati."
	" %s non &egrave; responsabile per le informazioni contenute in tali file."
	" Se ritieni opportuno rimuovere qualsiasi file"
	" il cui contenuto ti appartiene"
	" o che viola la legge, puoi contattare %s.";
#elif L==8	// pl
	"Zastrze&zdot;enie: pliki hostowane tutaj, o ile nie zaznaczono inaczej,"
	" podlegaj&aogon; prawom w&lstrok;asno&sacute;ci intelektualnej,"
	" a ich w&lstrok;asno&sacute;&cacute; nale&zdot;y do u&zdot;ytkownik&oacute;w, kt&oacute;rzy je przes&lstrok;ali."
	" %s nie ponosi odpowiedzialno&sacute;ci za informacje zawarte w tych plikach. "
	" Je&sacute;li uwa&zdot;asz za wygodne usuni&eogon;cie dowolnego pliku,"
	" kt&oacute;rego zawarto&sacute;&cacute; nale&zdot;y do Ciebie lub kt&oacute;ry narusza prawo,"
	" mo&zdot;esz skontaktowa&cacute; si&eogon; z %s.";
#elif L==9	// pt
	"Isen&ccedil;&atilde;o de responsabilidade: os arquivos hospedados aqui,"
	" salvo indica&ccedil;&atilde;o em contr&aacute;rio,"
	" est&atilde;o sujeitos a direitos de propriedade intelectual"
	" e sua propriedade pertence aos usu&aacute;rios que fizeram o upload deles."
	" %s n&atilde;o &eacute; respons&aacute;vel pelas informa&ccedil;&otilde;es contidas nesses arquivos."
	" Se voc&ecirc; acha conveniente remover qualquer arquivo"
	" cujo conte&uacute;do perten&ccedil;a a voc&ecirc;"
	" ou que viole a lei, entre em contato com %s.";
#elif L==10	// tr
	"Disclaimer: the files hosted here,"
	" unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file"
	" whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// eviri lazim!
#endif

const char *Txt_Display_photos =
#if   L==1	// ca
	"Incloure fotos";
#elif L==2	// de
	"Bilder anzeigen";
#elif L==3	// en
	"Display photos";
#elif L==4	// es
	"Incluir fotos";
#elif L==5	// fr
	"Inclure des photos";
#elif L==6	// gn
	"Incluir fotos";	// Okoteve traduccin
#elif L==7	// it
	"Mostra foto";
#elif L==8	// pl
	"Wyswietlanie zdjec";
#elif L==9	// pt
	"Incluir fotos";
#elif L==10	// tr
	"Display photos";	// eviri lazim!
#endif

const char *Txt_distributed_by =
#if   L==1	// ca
	"distribu&iuml;t per";
#elif L==2	// de
	"vertrieben von";
#elif L==3	// en
	"distributed by";
#elif L==4	// es
	"distribuido por";
#elif L==5	// fr
	"distribu&eacute; par";
#elif L==6	// gn
	"distribuido por";	// Okoteve traduccin
#elif L==7	// it
	"distribuito da";
#elif L==8	// pl
	"wyplacanych przez";
#elif L==9	// pt
	"distribu&iacute;do por";
#elif L==10	// tr
	"distributed by";	// eviri lazim!
#endif

const char *Txt_Do_not_accept_third_party_cookies =
#if   L==1	// ca
	"No accepteu cookies de tercers";
#elif L==2	// de
	"Akzeptieren Sie keine Cookies von Drittanbietern";
#elif L==3	// en
	"Do no accept third-party cookies";
#elif L==4	// es
	"No aceptar cookies de terceros";
#elif L==5	// fr
	"Ne pas accepter les cookies tiers";
#elif L==6	// gn
	"Ani reacepta umi cookie mbohap&yacute;ha tapicha rehegua";
#elif L==7	// it
	"Non accettare i cookie di terze parti";
#elif L==8	// pl
	"Nie akceptuj plik&oacute;w cookie stron trzecich";
#elif L==9	// pt
	"N&atilde;o aceitar cookies de terceiros";
#elif L==10	// tr
	"&Uuml;&ccedil;&uuml;nc&uuml; taraf tan&inodot;mlama bilgilerini kabul etmeyin";
#endif

const char *Txt_Do_not_show_again =
#if   L==1	// ca
	"No mostrar de nou";
#elif L==2	// de
	"Nicht mehr anzeigen";
#elif L==3	// en
	"Do not show again";
#elif L==4	// es
	"No mostrar de nuevo";
#elif L==5	// fr
	"Ne plus afficher";
#elif L==6	// gn
	"No mostrar de nuevo";	// Okoteve traduccin
#elif L==7	// it
	"Non mostrare pi&ugrave;";
#elif L==8	// pl
	"Nie pokazuj ponownie";
#elif L==9	// pt
	"N&atilde;o mostrar novamente";
#elif L==10	// tr
	"Do not show again";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_be_removed_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realment desitja deixar ser administrador/a"
	" de <strong>%s</strong>?";
#elif L==2	// de
	"M&ouml;chten Sie wirklich als Administrator"
	" von <strong>%s</strong> entfernt werden?";
#elif L==3	// en
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment &ecirc;tre supprim&eacute; en tant qu'administrateur"
	" de <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuole realmente essere rimosso come amministratore"
	" de <strong>%s</strong>?";
#elif L==8	// pl
	"Czy naprawde chcesz byc usuniety jako administrator"
	" <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer ser removido/a como administrador/a"
	" de <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_be_removed_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente vol ser eliminat de l'assignatura <strong>%s</strong>"
	" (eliminant els seus treballs, adscripci&oacute; a grups, etc.)?";
#elif L==2	// de
	"Wollen Sie wirklich aus dem Kurs <strong>%s</strong> exmatrikuliert werden"
	" (Entfernung Ihrer Arbeiten, Einschreibung in Gruppen etc.)?";
#elif L==3	// en
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrolment in groups, etc.)?";
#elif L==4	// es
	"&iquest;Realmente desea ser eliminado/a de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==5	// fr
	"Souhaitez-vous vraiment &ecirc;tre retir&eacute; de la mati&egrave;re <strong>%s</strong>"
	" (retrait de vos travaux, inscription &agrave; des groupes, etc.)?";
#elif L==6	// gn
	"&iquest;Realmente desea ser eliminado/a de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente essere rimosso dal corso <strong>%s</strong>"
	" (rimuovendo i tuoi lavori, registrazione in gruppi, etc.)?";
#elif L==8	// pl
	"Czy na pewno chcesz zostac usuniete z kursu <strong>%s</strong>"
	" (usuwanie swoja prace, rejestracja w grupach, itp.)?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer ser removido/a da disciplina <strong>%s</strong>"
	" (removendo seus trabalhos, inscri&ccedil;&atilde;o em grupos, etc.)?";
#elif L==10	// tr
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrolment in groups, etc.)?";			// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_be_removed_as_a_X_from_the_project_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu ser eliminat"
	" como a %s del projecte <strong>%s</strong>?";
#elif L==2	// de
        "M&ouml;chten Sie wirklich"
        " als %s aus dem Projekt <strong>%s</strong> entfernt werden?";
#elif L==3	// en
	"Do you really want to be removed"
	" as a %s from the project <strong>%s</strong>";
#elif L==4	// es
	"&iquest;Realmente desea ser eliminado"
	" como %s del proyecto <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment &ecirc;tre retir&eacute;"
	" en tant que %sdu projet <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea ser eliminado"
	" como %s del proyecto <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere"
	" come %s dal progetto <strong>%s</strong>?";
#elif L==8	// pl
	"Czy naprawd&eogon; chcesz zosta&cacute; usuni&eogon;ty"
	" jako %s z projektu <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer ser removido"
	" como %s do projeto <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to be removed"
	" as a %s from the project <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_the_following_user =
#if   L==1	// ca
	"Realment voleu eliminar completament l'usuari seg&uuml;ent?";
#elif L==2	// de
	"Wollen Sie den folgenden Benutzer wirklich vollst&auml;ndig eliminieren?";
#elif L==3	// en
	"Do you really want to completely eliminate the following user?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar por completo la cuenta del siguiente usuario/a?";
#elif L==5	// fr
	"Voulez-vous vraiment &eacute;liminer compl&egrave;tement l'utilisateur suivant?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar por completo la cuenta del siguiente usuario/a?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere totalmente il seguente utente?";
#elif L==8	// pl
	"Czy na pewno chcesz, aby calkowicie wyeliminowac nastepujacych uzytkownik&oacute;w?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer eliminar completamente o seguinte usu&aacute;rio/a?";
#elif L==10	// tr
	"Do you really want to completely eliminate the following user?";			// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_your_user_account =
#if   L==1	// ca
	"Realment voleu eliminar completament el vostre compte d'usuari?";
#elif L==2	// de
	"M&ouml;chten Sie Ihr Benutzerkonto wirklich vollst&auml;ndig l&ouml;schen?";
#elif L==3	// en
	"Do you really want to completely eliminate your user account?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer compl&egrave;tement votre compte utilisateur?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere totalmente il tuo account?";
#elif L==8	// pl
	"Czy na pewno chcesz, aby calkowicie wyeliminowac konta uzytkownika?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer eliminar completamente sua conta de usu&aacute;rio?";
#elif L==10	// tr
	"Do you really want to completely eliminate your user account?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Realment voleu suprimir tots els missatges rebuts de %s des de %s?";
#elif L==2	// de
	"Wollen Sie wirklich alle von %s erhaltenen Nachrichten von %s l&ouml;schen?";
#elif L==3	// en
	"Do you really want to delete all messages received from %s from %s?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer tous les messages re&ccedil;us de %s depuis %s?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi ricevuti da %s da %s?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; wszystkie wiadomosci otrzymane od %s z %s?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens recebidas de %s de %s?";
#elif L==10	// tr
	"Do you really want to delete all messages received from %s from %s?";			// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"Realment voleu suprimir tots els missatges"
	" rebuts de %s des de %s relacionats amb <strong>%s</strong>?";
#elif L==2	// de
	"M&ouml;chten Sie wirklich alle Nachrichten l&ouml;schen,"
	" die Sie von %s von %s erhalten haben und die sich auf <strong>%s</strong> beziehen?";
#elif L==3	// en
	"Do you really want to delete all messages"
	" received from %s from %s related to <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes"
	" recibidos de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer tous les messages"
	" re&ccedil;us de %s depuis %s li&eacute;s &agrave; <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes"
	" recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi"
	" ricevuti da %s da %s relativi a <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; wszystkie wiadomosci"
	" otrzymane od %s z %s podobne do <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens"
	" recebidas de %s de %s relacionadas com <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to delete all messages"
	" received from %s from %s related to <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Realment voleu suprimir tots els missatges enviats a %s des de %s?";
#elif L==2	// de
	"M&ouml;chten Sie wirklich alle von %s an %s gesendeten Nachrichten l&ouml;schen?";
#elif L==3	// en
	"Do you really want to delete all messages sent to %s from %s?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer tous les messages envoy&eacute;s &agrave; %s depuis %s?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; wszystkie wiadomosci wyslane do %s z %s?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens enviadas para %s de %s?";
#elif L==10	// tr
	"Do you really want to delete all messages sent to %s from %s?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"Realment voleu suprimir tots els missatges enviats a %s des de %s relacionats amb <strong>%s</strong>?";
#elif L==2	// de
	"M&ouml;chten Sie wirklich alle Nachrichten l&ouml;schen, die von %s an %s gesendet wurden und sich auf <strong>%s</strong> beziehen?";
#elif L==3	// en
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer tous les messages envoy&eacute;s &agrave; %s depuis %s concernant <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s relativi a<strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; wszystkie wiadomosci wyslane do %s z %s podobne do <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens enviadas para %s de %s relacionadas com <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Realment voleu suprimir els missatges no llegits rebuts de %s des de %s?";
#elif L==2	// de
	"M&ouml;chten Sie wirklich die ungelesenen Nachrichten von %s l&ouml;schen, die Sie von %s erhalten haben?";
#elif L==3	// en
	"Do you really want to delete the unread messages received from %s from %s?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer les messages non lus re&ccedil;us de %s depuis %s?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi non letti ricevuti da %s da %s?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nieprzeczytanych wiadomosci otrzymane od %s z %s?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar as mensagens n&atilde;o lidas recebidas de %s de %s?";
#elif L==10	// tr
	"Do you really want to delete the unread messages received from %s from %s?";			// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"Realment voleu suprimir els missatges no llegits"
	" rebuts de %s des de %s relacionats amb <strong>%s</strong>?";
#elif L==2	// de
	"M&ouml;chten Sie wirklich die ungelesenen Nachrichten l&louml;schen,"
	" die Sie von %s von %s erhalten haben und die sich auf <strong>%s</strong> beziehen?";
#elif L==3	// en
	"Do you really want to delete the unread messages"
	" received from %s from %s related to <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos"
	" recibidos de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer les messages non lus"
	" re&ccedil;us de %s depuis %s li&eacute;s &agrave; <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos"
	" recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi non letti"
	" ricevuti da %s da %s relativi a <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nieprzeczytanych wiadomosci"
	" otrzymane od %s z %s podobne do <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar as mensagens n&atilde;o lidas"
	" recebidas de %s de %s relacionadas com <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to delete the unread messages"
	" received from %s from %s related to <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_register_the_following_user_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu registrar el seg&uuml;ent usuari"
	" com a administrador de <strong>%s</strong>?";
#elif L==2	// de
	"Mchten Sie den folgenden Benutzer wirklich"
	" als Administrator von <strong>%s</strong> registrieren?";
#elif L==3	// en
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea inscribir el siguiente usuario/a"
	" como administrador/a de <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment enregistrer l'utilisateur suivant"
	" en tant qu'administrateur de <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea inscribir el siguiente usuario/a"
	" como administrador/a de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente registrare il seguente utente"
	" come amministratore de <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz sie zarejestrowac nastepujace u&zdot;ytkownika"
	" jako administratora <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja registrar o seguinte usu&aacute;rio/a"
	" como administrador/a de <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_reject_the_enrolment_request_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"Realment vols rebutjar la sollicitud de matr&iacute;cula"
	" de <strong>%s</strong> com a <strong>%s</strong>"
	" a l'assignatura <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie den Immatrikulationsantrag"
	" von <strong>%s</strong> als <strong>%s</strong>"
	" in den Kurs <strong>%s</strong> wirklich ablehnen?";
#elif L==3	// en
	"Do you really want to reject the enrolment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";
#elif L==4	// es
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment rejeter la demande d'inscription"
	" de <strong>%s</strong> en tant que <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong>?";
#elif L==6	// gn
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi davvero rifiutare la richiesta di iscrizione"
	" di <strong>%s</strong> come <strong>%s</strong>"
	" al corso <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz odrzuci&cacute; pro&sacute;b&eogon; o przyj&eogon;cie"
	" od <strong>%s</strong> jako <strong>%s</strong>"
	" na kurs <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer rejeitar a solicita&ccedil;&atilde;o de inscri&ccedil;&atilde;o"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" na disciplina <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to reject the enrolment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_X_students_from_the_course_Y_ =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1	// ca
	"Realment voleu eliminar els %u estudiants de l'assignatura <strong>%s</strong>?<br />"
	"Encara que no s'eliminen les dades dels estudiants en altres assignatures,"
	" ja no apareixeran inscrits en aquesta assignatura,"
	" i a m&eacute;s <strong>s'eliminaran tots els treballs enviats per ells en aquesta assignatura,"
	" les seves fitxes personalitzades per aquesta assignatura,"
	" i les seves inscripcions en grups d'aquesta assignatura.</strong>";
#elif L==2	// de
	"Wollen Sie die %u Studenten wirklich aus dem Kurs <strong>%s</strong> entfernen?<br />"
	"Die Daten der Studierenden in anderen Studieng&auml;ngen werden zwar nicht gel&ouml;scht,"
	" aber sie werden in diesem Studiengang nicht mehr eingeschrieben,"
	" und auch <strong>alle von ihnen in diesem Studiengang eingereichten Arbeiten,"
	" ihre Karteikarten f&uuml;r diesen Studiengang"
	" und ihre Einschreibung in Gruppen dieses Studiengangs werden gel&ouml;scht.</strong>";
#elif L==3	// en
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enroled in this course,"
	" and also <strong>all works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrolment in groups of this course will be removed.</strong>";
#elif L==4	// es
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y sus inscripciones en grupos de esta asignatura.</strong>";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer les %u &eacute;tudiants de la mati&eacute;re <strong>%s</strong>?<br />"
	"Bien que les donn&eacute;es des &eacute;tudiants ne soient pas supprim&eacute;es dans d'autres cours,"
	" ils ne seront plus inscrits &agrave; cette mati&egrave;re,"
	" ainsi que <strong>tous les travaux soumis par eux dans cette mati&egrave;re,"
	" leurs fiches pour cette mati&egrave;re"
	" et leur inscription &agrave; des groupes de cette mati&egrave;re seront supprim&eacute;s.</strong>";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere gli %u studenti dal corso <strong>%s</strong>?<br />"
	"Sebbene i dati degli studenti non saranno rimossi in altri corsi,"
	" essi non saranno pi&ugrave; registrati in questo corso,"
	" e cos <strong>tutti i lavori presentati da loro in questo corso,"
	" le loro schede per questo corso,"
	" e la loro registrazione nei gruppi di questo corso saranno rimossi.</strong>";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; %u student&oacute;w z kursu <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" nie beda juz zarejestrowane w tym kurs"
	" a takze <strong>wszystkie prace zlozone przez nich w tym kursie,"
	" swoich kart zapisu na ten kurs,"
	" i ich rejestracja w grupach zajec zostana usuniete.</strong>";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover os %u estudantes do curso <strong>%s</strong>?<br />"
	"Embora os dados dos estudantes n&atilde;o sejam removidos em outras disciplinas,"
	" eles n&atilde;o ser&atilde;o mais inscritos nesta disciplina,"
	" e tamb&eacute;m <strong>ser&atilde;o removidos todos os trabalhos enviados por eles nesta disciplina,"
	" suos cart&otilde;es para esta disciplina,"
	" e sua inscri&ccedil;&atilde;o em grupos desta disciplina.</strong>";
#elif L==10	// tr
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enroled in this course,"
	" and also <strong>all works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrolment in groups of this course will be removed.</strong>";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar l'activitat <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Aufgabe <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the assignment <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'activit&eacute; <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'attivit&agrave; <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; przypisanie <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a atividade <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the assignment <strong>%s</strong>?";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_criterion_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar el criteri <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie der Kriterium <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the criterion <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el criterio <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le crit&egrave;re <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el criterio <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il criterio <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; kryterium <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o crit&eacute;rio <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> kriterini ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_entire_thread_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar tota la discussi&oacute; <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die gesamten Thread <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the entire thread <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer tout le fil <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'intera discussione <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; caly watek <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover toda a discuss&atilde;o <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the entire thread <strong>%s</strong>?";			// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar l'esdevenimen <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie der Ereignis <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the event <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'&eacute;v&eacute;nement <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'evento <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; wydarzenie <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o evento <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the event <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_exam_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar l'examen <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie der Pr&uuml;fung <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the exam <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el examen <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'examen <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el examen <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'esame <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; egzaminu <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o exame <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the exam <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_field_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu eliminar el camp <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie das Feld <strong>%s</strong>?";
#elif L==3	// en
	"Do you really want to remove the field <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le champ <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il campo <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; pole <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o campo <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the field <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";		// Necessita traducci
#elif L==2	// de
	"Wollen Sie wirklich"
	" das Verzeichnis <strong>%s</strong> und alle Inhalte entfernen?";
#elif L==3	// en
	"Do you really want to remove"
	" the folder <strong>%s</strong> and all its contents?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer"
	" le r&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere"
	" la cartella <strong>%s</strong> e tutto il suo contenuto?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute;"
	" folder <strong>%s</strong> i wszystkie jego skladniki?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover"
	" o diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do?";
#elif L==10	// tr
	"Do you really want to remove"
	" the folder <strong>%s</strong> and all its contents?";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_call_for_exam =
#if   L==1	// ca
	"De veres voleu eliminar la seg&uuml;ent convocat&ograve;ria d'examen?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Aufruf f&uuml;r Pr&uuml;fung?";
#elif L==3	// en
	"Do you really want to remove the following call for exam?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la siguiente convocatoria de examen?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la convocation &agrave; un examen suivante?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la siguiente convocatoria de examen?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente appello d'esame?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nast&eogon;puj&aogon;c&aogon; zapowied&zacute; egzaminu?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a seguinte chamada para exame?";
#elif L==10	// tr
	"Do you really want to remove the following call for exam?";			// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_notice =
#if   L==1	// ca
	"De veres voleu eliminar el seg&uuml;ent av&iacute;s?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Ank&uuml;ndigung zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the following notice?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente aviso?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'avis suivant?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente aviso?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente avviso?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nastepujace obwieszczenie?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte aviso?";
#elif L==10	// tr
	"Do you really want to remove the following notice?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_comment =
#if   L==1	// ca
	"De veres voleu eliminar el seg&uuml;ent comentari?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Kommentar zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the following comment?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente comentario?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le commentaire suivant?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente comentario?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente commento?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nastepujace komentarz?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte coment&aacute;rio?";
#elif L==10	// tr
	"Do you really want to remove the following comment?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_post =
#if   L==1	// ca
	"De veres voleu eliminar la seg&uuml;ent publicaci&oacute;?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Beitrag zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the following post?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la siguiente publicaci&oacute;n?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le post suivant?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la siguiente publicaci&oacute;n?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente post?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nastepujace post?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte post?";
#elif L==10	// tr
	"Do you really want to remove the following post?";			// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu eliminar l'usuari seg&uuml;ent"
	" com a administrador de <strong>%s</strong>?";
#elif L==2	// de
	"M&ouml;chten Sie den folgenden Benutzer wirklich"
	" als Administrator von <strong>%s</strong> entfernen?";
#elif L==3	// en
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador/a de <strong>%s</strong>?";
#elif L==5	// fr
	"Realment voleu eliminar l'usuari seg&uuml;ent"
	" com a administrador de <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador/a de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente utente"
	" come amministratore de <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nastepujace u&zdot;ytkownika"
	" jako administratora <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte usu&aacute;rio"
	" como administrador de <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar el seg&uuml;ent usuari de l'assignatura <strong>%s</strong>"
	" (eliminant els seus treballs, adscripci&oacute; a grups, etc.)?";
#elif L==2	// de
        "M&ouml;chten Sie den folgenden Benutzer wirklich aus dem Kurs <strong>%s</strong> entfernen"
	" (Beseitigung ihrer Werke, Einschreibung in Gruppen usw.)?";
#elif L==3	// en
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrolment in groups, etc.)?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'utilisateur suivant de la mati&egrave;re <strong>%s</strong>"
	" (&eacute;liminant ses travaux, inscription en groupes, etc.)?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente utente dal corso <strong>%s</strong>"
	" (rimuovendo i suoi lavori, registrazione in gruppi, etc.)?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nast&eogon;puj&aogon;cy u&zdot;ytkownik z kursu <strong>%s</strong>"
	" (usuni&eogon;cie jego utwor&oacute;w, zapisanie si&eogon; w grupach itp.)?";
#elif L==9	// pt
	"Voc&ecirc; deseja realmente remover o seguinte usu&aacute;rio da disciplina <strong>%s</strong>"
	" (removendo seus trabalhos, inscri&ccedil;&atilde;o em grupos, etc.)?";
#elif L==10	// tr
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrolment in groups, etc.)?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_a_X_from_the_project_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar el seg&uuml;ent usuari"
	" com a %s del projecte <strong>%s</strong>?";
#elif L==2	// de
        "M&ouml;chten Sie den folgenden Benutzer wirklich"
        " als %s aus dem Projekt <strong>%s</strong> entfernen?";
#elif L==3	// en
	"Do you really want to remove the following user"
	" as a %s from the project <strong>%s</strong>";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como %s del proyecto <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'utilisateur suivant"
	" en tant que %s du projet <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como %s del proyecto <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente utente"
	" como %s dal progetto <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nast&eogon;puj&aogon;cy u&zdot;ytkownik"
	" jako %s z projektu <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; deseja realmente remover o seguinte usu&aacute;rio"
	" como %s do projeto <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the following user"
	" as a %s from the project <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_game_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar le joc <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Spiel <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the game <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el juego <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le jeu <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el juego <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il gioco <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; gra <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o jogo <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> oyununu ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu suprimir el grup <strong>%s</strong>?<br />"
	"Si ho fa, els usuaris inscrits al grup deixaran de pert&agrave;nyer-hi.";
#elif L==2	// de
	"M&ouml;chten Sie Gruppe <strong>%s</strong> wirklich entfernen?<br />"
	"Wenn Sie dies tun, geh&ouml;ren die in der Gruppe registrierten Benutzer nicht mehr dazu.";
#elif L==3	// en
	"Do you really want to remove group <strong>%s</strong>?<br />"
	"If you do, users enrolled in the group will no longer belong to it.";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, los usuarios inscritos en el grupo dejar&aacute;n de pertenecer a &eacute;l.";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le groupe <strong>%s</strong>?<br />"
	"Si vous le faites, les utilisateurs inscrits au groupe n'en feront plus partie.";
#elif L==6	// gn
	"&iquest;A&ntilde;eteh&aacute;pe piko reipe'ase pe grupo <strong>%s</strong>?<br />"
	"Rejap&oacute;ramo up&eacute;icha, umi puruh&aacute;ra o&ntilde;einskriv&iacute;va pe aty'&iacute;pe ndaha'emo'&atilde;v&eacute;ima ipypegua.";
#elif L==7	// it
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?<br />"
	"Se lo fai, gli utenti iscritti al gruppo non ne faranno pi&ugrave; parte.";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; grup&eogon; <strong>%s</strong>?<br />"
	"Je&sacute;li to zrobisz, u&zdot;ytkownicy zarejestrowani w grupie przestan&aogon; do niej nale&zdot;e&cacute;.";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o grupo <strong>%s</strong>?<br />"
	"Se o fizer, os usu&aacute;rios inscritos no grupo n&atilde;o far&atilde;o mais parte dele.";
#elif L==10	// tr
	"<strong>%s</strong> grubunu ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?<br />"
	"Bunu yaparsan&inodot;z, gruba kay&inodot;tl&inodot; kullan&inodot;c&inodot;lar art&inodot;k gruba ait olmaz.";
#endif

const char *Txt_Do_you_really_want_to_remove_the_item_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar l'&iacute;tem <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Artikel <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the item <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el &iacute;tem <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'article <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el &iacute;tem <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il articolo <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; przedmiot <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o item <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> &ouml;&gbreve;esini ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_match_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar la partida <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Spiel <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the match <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la partida <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le match <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la partida <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la partita <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; mecz <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o jogo <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> e&scedil;le&scedil;mesini ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_photo_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar la foto de <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die wirklich das Foto von <strong>%s</strong> zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the photo of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la foto de <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la photo de <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la foto de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la foto di <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; zdj&eogon;cie <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover la foto de <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to remove the photo of <strong>%s</strong>?";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_project_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar el projecte <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Projekt <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the project <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el proyecto <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le projet <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el proyecto <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il progetto <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; projekt <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o projeto <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> projesini ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_question_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar la pregunta <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Frage <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the question <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la pregunta <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la question <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la pregunta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la domanda <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; pytanie <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a pergunta <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> sorusunu ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_resource_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar el recurs <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Ressource <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the resource <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el recurso <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la ressource <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el recurso <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la risorsa <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; zas&oacute;b <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o recurso <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> kayna&gbreve;&inodot;n&inodot; ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_rubric_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu eliminar la r&uacute;brica <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Rubrik <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the rubric <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la r&uacute;brica <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la rubrique <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la r&uacute;brica <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la rubrica <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; rubryk&eogon; <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a rubrica <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> de&gbreve;erlendirme tablosunu ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_selected_questions =
#if   L==1	// ca
       "Realment voleu eliminar les preguntes seleccionades?";
#elif L==2	// de
       "Wollen Sie die ausgew&auml;hlten Fragen wirklich entfernen?";
#elif L==3	// en
       "Do you really want to remove the selected questions?";
#elif L==4	// es
       "&iquest;Realmente desea eliminar las preguntas seleccionadas?";
#elif L==5	// fr
       "Voulez-vous vraiment supprimer les questions s&eacute;lectionn&eacute;es?";
#elif L==6	// gn
       "&iquest;Realmente desea eliminar las preguntas seleccionadas?";	// Okoteve traduccin
#elif L==7	// it
       "Vuoi realmente rimuovere le domande selezionate?";
#elif L==8	// pl
       "Czy na pewno chcesz usun&aogon;&cacute; wybrane pytania?";
#elif L==9	// pt
       "Voc&ecirc; realmente deseja remover as perguntas selecionadas?";
#elif L==10	// tr
	"Do you really want to remove the selected questions?";		// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_remove_the_session_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar la sessi&oacute; <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie der Sitzung <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the session <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la sesi&oacute;n <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la session <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la sesi&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la sessione <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; sesji <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a sess&atilde;o <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> oturumunu ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_set_of_questions_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar el conjunt de preguntes <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Satz von Fragen <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the set of questions <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el conjunto de preguntas <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'ensemble de questions <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el conjunto de preguntas <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il set di domande <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; zestaw pyta&nacute; <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o conjunto de perguntas <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> soru grubunu ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu eliminar l'enquesta <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the survey <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le sondage <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il sondaggio <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; badania <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o inqu&eacute;rito <strong>%s</strong>?";
#elif L==10	// tr
	"<strong>%s</strong> anketini ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_type_of_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu eliminar el tipus de grup <strong>%s</strong>?<br />"
	"En fer-ho, tamb&eacute; s'eliminaran els grups d'aquest tipus.";
#elif L==2	// de
	"M&ouml;chten Sie den Typ der Gruppe <strong>%s</strong> wirklich entfernen?<br />"
	"Dadurch werden auch die Gruppen dieses Typs entfernt.";
#elif L==3	// en
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove the groups of that type.";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n los grupos de ese tipo.";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le type de groupe <strong>%s</strong>?<br />"
	"Cela supprimera &eacute;galement les groupes de ce type.";
#elif L==6	// gn
	"&iquest;A&ntilde;eteh&aacute;pe piko reipe'ase pe tipo de grupo <strong>%s</strong>?<br />"
	"P&eacute;icha jajap&oacute;ramo ojeipe'&aacute;ta avei umi aty upeichagua.";
#elif L==7	// it
	"Vuoi realmente rimuovere il tipo di gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai anche i gruppi di questo tipo.";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; typ grupy <strong>%s</strong>?<br />"
	"Spowoduje to r&oacute;wnie&zdot; usuni&eogon;cie grup tego typu.";
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o tipo de grupo <strong>%s</strong>?<br />"
	"Isso tamb&eacute;m remover&aacute; os grupos desse tipo.";
#elif L==10	// tr
	"<strong>%s</strong> grubunun t&uuml;r&uuml;n&uuml; ger&ccedil;ekten kald&inodot;rmak istiyor musunuz?<br />"
	"Bunu yapmak, o t&uuml;rdeki gruplar&inodot; da kald&inodot;racakt&inodot;r.";
#endif

const char *Txt_Do_you_really_want_to_remove_your_photo =
#if   L==1	// ca
	"De veres voleu eliminar la seva foto?";
#elif L==2	// de
	"Wollen Sie die Ihr Foto zu entfernen?";
#elif L==3	// en
	"Do you really want to remove your photo?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar su foto?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer votre photo?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar su foto?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la tua foto?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; swoje zdj&eogon;cie?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a sua foto?";
#elif L==10	// tr
	"Do you really want to remove your photo?";	// eviri lazim!
#endif

const char *Txt_Do_you_really_want_to_reset_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu posar a zero l'enquesta <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich reset?";
#elif L==3	// en
	"Do you really want to reset the survey <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment reset le sondage <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente resettare il sondaggio <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz zresetowac badania <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja reiniciar o inqu&eacute;rito <strong>%s</strong>?";
#elif L==10	// tr
	"Do you really want to reset the survey <strong>%s</strong>?";			// eviri lazim!
#endif

const char *Txt_Do_you_think_you_are_one_of_these_users =
#if   L==1	// ca
	"Creu que vost&egrave; &eacute;s un d'aquests usuaris?";
#elif L==2	// de
	"Denken Sie, Sie einer dieser Benutzer sind?";
#elif L==3	// en
	"Do you think you are one of these users?";
#elif L==4	// es
	"&iquest;Cree que usted es uno de estos usuarios?";
#elif L==5	// fr
	"Pensez-vous que vous &ecirc;tes un de ces utilisateurs?";
#elif L==6	// gn
	"&iquest;Cree que usted es uno de estos usuarios?";	// Okoteve traduccin
#elif L==7	// it
	"Pensi che sei uno di questi utenti?";
#elif L==8	// pl
	"My&sacute;lisz, &zdot;e jeste&sacute;"
	" jednym z tych u&zdot;ytkownik&oacute;w?";
#elif L==9	// pt
	"Voc&ecirc; acha que voc&ecirc; &eacute; um desses usu&aacute;rios?";
#elif L==10	// tr
	"Bu kullan&inodot;c&inodot;lardan biri oldu&gbreve;unuzu"
	" d&uuml;&scedil;&uuml;n&uuml;yor musunuz?";
#endif

const char *Txt_Do_you_think_you_are_this_user =
#if   L==1	// ca
	"Creu que vost&egrave; &eacute;s aquest usuari?";
#elif L==2	// de
	"Denken Sie, Sie dieser Benutzer?";
#elif L==3	// en
	"Do you think you are this user?";
#elif L==4	// es
	"&iquest;Cree que usted es este usuario/a?";
#elif L==5	// fr
	"Pensez-vous que vous &ecirc;tes cet utilisateur?";
#elif L==6	// gn
	"&iquest;Cree que usted es este usuario/a?";	// Okoteve traduccin
#elif L==7	// it
	"Pensi che sei questo utente?";
#elif L==8	// pl
	"Czy uwa&zdot;asz, &zdot;e ta osoba?";
#elif L==9	// pt
	"Voc&ecirc; acha que voc&ecirc; &eacute; deste usu&aacute;rio?";
#elif L==10	// tr
	"Bu kullan&inodot;c&inodot; oldu&gbreve;unuzu"
	" d&uuml;&scedil;&uuml;n&uuml;yor musunuz?";
#endif

const char *Txt_Do_you_want_to_follow_the_selected_user_whom_you_do_not_follow_yet =
#if   L==1	// ca
	"Voleu seguir a l'usuari seleccionat a qui encara no segueix?";
#elif L==2	// de
	"M&ouml;chten Sie dem ausgew&auml;hlten Benutzer folgen, dem Sie noch nicht folgen?";
#elif L==3	// en
	"Do you want to follow the selected user whom you do not follow yet?";
#elif L==4	// es
	"&iquest;Desea seguir al usuario seleccionado que a&uacute;n no sigue?";
#elif L==5	// fr
	"Voulez-vous suivre l'utilisateur s&eacute;lectionn&eacute; que vous ne suivez pas encore?";
#elif L==6	// gn
	"&iquest;Desea seguir al usuario seleccionado que a&uacute;n no sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi seguire l'utente selezionato che non segui ancora?";
#elif L==8	// pl
	"Czy chcesz &sacute;ledzi&cacute; wybranego u&zdot;ytkownika, kt&oacute;rego jeszcze nie &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer seguir o usu&aacute;rio selecionado que voc&ecirc; n&atilde;o segue ainda?";
#elif L==10	// tr
	"Do you want to follow the selected user whom you do not follow yet?";		// eviri lazim!
#endif

const char *Txt_Do_you_want_to_follow_the_X_selected_users_whom_you_do_not_follow_yet =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Voleu seguir als %u usuaris seleccionats als que encara no segueix?";
#elif L==2	// de
	"M&ouml;chten Sie den %u ausgew&auml;hlten Benutzern folgen, denen Sie noch nicht folgen?";
#elif L==3	// en
	"Do you want to follow the %u selected users whom you do not follow yet?";
#elif L==4	// es
	"&iquest;Desea seguir a los %u usuarios seleccionados que a&uacute;n no sigue?";
#elif L==5	// fr
	"Voulez-vous suivre les %u utilisateurs s&eacute;lectionn&eacute;s que vous ne suivez pas encore?";
#elif L==6	// gn
	"&iquest;Desea seguir a los %u usuarios seleccionados que a&uacute;n no sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi seguire i %u utenti selezionati che non segui ancora?";
#elif L==8	// pl
	"Czy chcesz &sacute;ledzi&cacute; %u wybranych u&zdot;ytkownik&oacute;w, kt&oacute;rych jeszcze nie &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer seguir os %u usu&aacute;rios selecionados que voc&ecirc; n&atilde;o segue ainda?";
#elif L==10	// tr
	"Do you want to follow the %u selected users whom you do not follow yet?";		// eviri lazim!
#endif

const char *Txt_Do_you_want_to_lock_the_editing_of_the_X_selected_projects =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Voleu bloquejar l'edici&oacute;n dels %u projectes seleccionats?";
#elif L==2	// de
	"M&ouml;chten Sie die Bearbeitung der ausgew&auml;hlten %u-Projekte sperren?";
#elif L==3	// en
	"Do you want to lock the editing of the %u selected projects?";
#elif L==4	// es
	"&iquest;Desea bloquear la edici&oacute;n de los %u proyectos seleccionados?";
#elif L==5	// fr
	"Voulez-vous verrouiller l'&eacute;dition des %u projets s&eacute;lectionn&eacute;s?";
#elif L==6	// gn
	"&iquest;Desea bloquear la edici&oacute;n de los %u proyectos seleccionados?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi bloccare la modifica dei %u progetti selezionati?";
#elif L==8	// pl
	"Czy chcesz zablokowa&cacute; edycj&eogon; %u wybranych projekt&oacute;w?";
#elif L==9	// pt
	"Voc&ecirc; quer bloquear a edi&ccedil;&atilde;o dos %u projetos selecionados?";
#elif L==10	// tr
	"Do you want to lock the editing of the %u selected projects?";			// eviri lazim!
#endif

const char *Txt_Do_you_want_to_stop_following_the_selected_user_whom_you_follow =
#if   L==1	// ca
	"Voleu deixar de seguir a l'usuari seleccionat a qui segueix?";
#elif L==2	// de
	"M&ouml;chten Sie dem ausgew&auml;hlten Benutzer, dem Sie folgen, nicht mehr folgen?";
#elif L==3	// en
	"Do you want to stop following the selected user whom you follow?";
#elif L==4	// es
	"&iquest;Desea dejar de seguir al usuario seleccionado que sigue?";
#elif L==5	// fr
	"Voulez-vous arr&ecirc;ter de suivre l'utilisateur s&eacute;lectionn&eacute; que vous suivez?";
#elif L==6	// gn
	"&iquest;Desea dejar de seguir al usuario seleccionado que sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi smettere di seguire l'utente selezionato che segui?";
#elif L==8	// pl
	"Czy chcesz przesta&cacute; &sacute;ledzi&cacute; wybranego u&zdot;ytkownika, kt&oacute;rego &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer parar de seguir o usu&aacute;rio selecionado que voc&ecirc; segue?";
#elif L==10	// tr
	"Do you want to stop following the selected user whom you follow?";	// eviri lazim!
#endif

const char *Txt_Do_you_want_to_stop_following_the_X_selected_users_whom_you_follow =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Voleu deixar de seguir als %u usuaris seleccionats als que segueix?";
#elif L==2	// de
	"M&ouml;chten Sie nicht mehr den %u ausgew&auml;hlten Nutzern folgen, denen Sie folgen?";
#elif L==3	// en
	"Do you want to stop following the %u selected users whom you follow?";
#elif L==4	// es
	"&iquest;Desea dejar de seguir a los %u usuarios seleccionados que sigue?";
#elif L==5	// fr
	"Voulez-vous arr&ecirc;ter de suivre les %u utilisateurs s&eacute;lectionn&eacute;s que vous suivez?";
#elif L==6	// gn
	"&iquest;Desea dejar de seguir a los %u usuarios seleccionados que sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi smettere di seguire i %u utenti selezionati che segui?";
#elif L==8	// pl
	"Czy chcesz przesta&cacute; &sacute;ledzi&cacute; %u wybranych u&zdot;ytkownik&oacute;w, kt&oacute;rych &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer parar de seguir os %u usu&aacute;rios selecionados que voc&ecirc; segue?";
#elif L==10	// tr
	"Do you want to stop following the %u selected users whom you follow";		// eviri lazim!
#endif

const char *Txt_Do_you_want_to_unlock_the_editing_of_the_X_selected_projects =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Voleu desbloquejar l'edici&oacute; dels %u projectes seleccionats?";
#elif L==2	// de
	"M&ouml;chten Sie die Bearbeitung der ausgew&auml;hlten %u-Projekte freischalten?";
#elif L==3	// en
	"Do you want to unlock the editing of the %u selected projects?";
#elif L==4	// es
	"&iquest;Desea desbloquear la edici&oacute;n de los %u proyectos seleccionados?";
#elif L==5	// fr
	"Voulez-vous d&eacute;verrouiller l'&eacute;dition des %u projets s&eacute;lectionn&eacute;s?";
#elif L==6	// gn
	"&iquest;Desea desbloquear la edici&oacute;n de los %u proyectos seleccionados?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi sbloccare la modifica dei %u progetti selezionati?";
#elif L==8	// pl
	"Czy chcesz odblokowa&cacute; edycj&eogon; %u wybranych projekt&oacute;w?";
#elif L==9	// pt
	"Voc&ecirc; quer desbloquear a edi&ccedil;&atilde;o dos %u projetos selecionados?";
#elif L==10	// tr
	"Do you want to unlock the editing of the %u selected projects?";			// eviri lazim!
#endif

const char *Txt_Document =
#if   L==1	// ca
	"Document";
#elif L==2	// de
	"Dokument";
#elif L==3	// en
	"Document";
#elif L==4	// es
	"Documento";
#elif L==5	// fr
	"Document";
#elif L==6	// gn
	"Terakuatia";
#elif L==7	// it
	"Documento";
#elif L==8	// pl
	"Dokument";
#elif L==9	// pt
	"Documento";
#elif L==10	// tr
	"Belge";
#endif

const char *Txt_document_in_my_courses =
#if   L==1	// ca
	"document en les meves assignatures";
#elif L==2	// de
	"Dokument in meinen Kursen";
#elif L==3	// en
	"document in my courses";
#elif L==4	// es
	"documento en mis asignaturas";
#elif L==5	// fr
	"document &agrave; mes mati&egrave;res";
#elif L==6	// gn
	"documento en mis asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"documento in miei corsi";
#elif L==8	// pl
	"dokument w moich kursach";
#elif L==9	// pt
	"documento em minha disciplinas";
#elif L==10	// tr
	"document in my courses";	// eviri lazim!
#endif

const char *Txt_Documents =
#if   L==1	// ca
	"Documents";
#elif L==2	// de
	"Dokumente";
#elif L==3	// en
	"Documents";
#elif L==4	// es
	"Documentos";
#elif L==5	// fr
	"Documents";
#elif L==6	// gn
	"Documentos";	// Okoteve traduccin
#elif L==7	// it
	"Documenti";
#elif L==8	// pl
	"Dokumenty";
#elif L==9	// pt
	"Documentos";
#elif L==10	// tr
	"Belgeler";
#endif

const char *Txt_document_from_me =
#if   L==1	// ca
	"document meu";
#elif L==2	// de
	"Dokument von mir";
#elif L==3	// en
	"document from me";
#elif L==4	// es
	"documento m&iacute;o";
#elif L==5	// fr
	"document de moi";
#elif L==6	// gn
	"documento m&iacute;o";	// Okoteve traduccin
#elif L==7	// it
	"documento da me";
#elif L==8	// pl
	"dokument ode mnie";
#elif L==9	// pt
	"documento de mim";
#elif L==10	// tr
	"document from me";	// eviri lazim!
#endif

const char *Txt_documents_from_me =
#if   L==1	// ca
	"documents meus";
#elif L==2	// de
	"Dokumente von mir";
#elif L==3	// en
	"documents from me";
#elif L==4	// es
	"documentos m&iacute;os";
#elif L==5	// fr
	"documents de moi";
#elif L==6	// gn
	"documentos m&iacute;os";	// Okoteve traduccin
#elif L==7	// it
	"documenti da me";
#elif L==8	// pl
	"dokumenty ode mnie";
#elif L==9	// pt
	"documentos de mim";
#elif L==10	// tr
	"documents from me";		// eviri lazim!
#endif

const char *Txt_documents_in_my_courses =
#if   L==1	// ca
	"documents en les meves assignatures";
#elif L==2	// de
	"Dokumente in meinen Kursen";
#elif L==3	// en
	"documents in my courses";
#elif L==4	// es
	"documentos en mis asignaturas";
#elif L==5	// fr
	"documents &agrave; mes mati&egrave;res";
#elif L==6	// gn
	"documentos en mis asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"documenti in miei corsi";
#elif L==8	// pl
	"dokumenty w moich kursach";
#elif L==9	// pt
	"documentos em minha disciplinas";
#elif L==10	// tr
	"documents in my courses";		// eviri lazim!
#endif

const char *Txt_Documents_area =
#if   L==1	// ca
	"Zona de documents";
#elif L==2	// de
	"Dokumente-Bereich";
#elif L==3	// en
	"Documents area";
#elif L==4	// es
	"Zona de documentos";
#elif L==5	// fr
	"Zone de documents";
#elif L==6	// gn
	"Zona de documentos";	// Okoteve traduccin
#elif L==7	// it
	"Area documenti";
#elif L==8	// pl
	"Obszar Dokument&oacute;w";
#elif L==9	// pt
	"Zona de documentos";
#elif L==10	// tr
	"Documents area";	// eviri lazim!
#endif

const char *Txt_Documents_management_area =
#if   L==1	// ca
	"Zona d'administraci&oacute; de documents";
#elif L==2	// de
	"Dokumente-Verwaltung";
#elif L==3	// en
	"Documents management area";
#elif L==4	// es
	"Zona de administraci&oacute;n de documentos";
#elif L==5	// fr
	"Zone de gestion de documents";
#elif L==6	// gn
	"Zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==7	// it
	"Area di gestione dei documenti";
#elif L==8	// pl
	"Obszar zarzadzania Dokumenty";
#elif L==9	// pt
	"Zona de gest&atilde;o de documentos";
#elif L==10	// tr
	"Documents management area";			// eviri lazim!
#endif

const char *Txt_documents_management_area =
#if   L==1	// ca
	"zona de administraci&oacute;n de documentos";	// Necessita traducci
#elif L==2	// de
	"Dokumente-Verwaltung";
#elif L==3	// en
	"documents management area";
#elif L==4	// es
	"zona de administraci&oacute;n de documentos";
#elif L==5	// fr
	"zone de gestion de documents";
#elif L==6	// gn
	"zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==7	// it
	"area di gestione dei documenti";
#elif L==8	// pl
	"obszaru zarzadzania dokumentami";
#elif L==9	// pt
	"zona de gest&atilde;o de documentos";
#elif L==10	// tr
	"documents management area";			// eviri lazim!
#endif

const char *Txt_Domains =
#if   L==1	// ca
	"Dominis";
#elif L==2	// de
	"Dom&auml;nen";
#elif L==3	// en
	"Domains";
#elif L==4	// es
	"Dominios";
#elif L==5	// fr
	"Domaines";
#elif L==6	// gn
	"Dominios";	// Okoteve traduccin
#elif L==7	// it
	"Campi";
#elif L==8	// pl
	"Domen";
#elif L==9	// pt
	"Dom&iacute;nios";
#elif L==10	// tr
	"Domains";	// eviri lazim!
#endif

const char *Txt_Done =
#if   L==1	// ca
	"Fet";
#elif L==2	// de
	"Gemacht";
#elif L==3	// en
	"Done";
#elif L==4	// es
	"Hecho";
#elif L==5	// fr
	"Termin&eacute;";
#elif L==6	// gn
	"Hecho";	// Okoteve traduccin
#elif L==7	// it
	"Fatto";
#elif L==8	// pl
	"Gotowe";
#elif L==9	// pt
	"Feito";
#elif L==10	// tr
	"Done";		// eviri lazim!
#endif

const char *Txt_Download =	// As a verb
#if   L==1	// ca
	"Descarregar";
#elif L==2	// de
	"Download";
#elif L==3	// en
	"Download";
#elif L==4	// es
	"Descargar";
#elif L==5	// fr
	"T&eacute;l&eacute;charger";
#elif L==6	// gn
	"Descargar";	// Okoteve traduccin
#elif L==7	// it
	"Scarica";
#elif L==8	// pl
	"Pobierz";
#elif L==9	// pt
	"Baixar";
#elif L==10	// tr
	"Download";	// eviri lazim!
#endif

const char *Txt_download =	// As a noun
#if   L==1	// ca
	"desc&agrave;rrega";
#elif L==2	// de
	"Download";
#elif L==3	// en
	"download";
#elif L==4	// es
	"descarga";
#elif L==5	// fr
	"t&eacute;l&eacute;chargement";
#elif L==6	// gn
	"descarga";	// Okoteve traduccin
#elif L==7	// it
	"download";
#elif L==8	// pl
	"pobrania";
#elif L==9	// pt
	"download";
#elif L==10	// tr
	"download";	// eviri lazim!
#endif

const char *Txt_Downloads =	// As a noun
#if   L==1	// ca
	"Desc&agrave;rregues";
#elif L==2	// de
	"Downloads";
#elif L==3	// en
	"Downloads";
#elif L==4	// es
	"Descargas";
#elif L==5	// fr
	"T&eacute;l&eacute;chargements";
#elif L==6	// gn
	"Descargas";	// Okoteve traduccin
#elif L==7	// it
	"Downloads";
#elif L==8	// pl
	"Pobrania";
#elif L==9	// pt
	"Downloads";
#elif L==10	// tr
	"Downloads";	// eviri lazim!
#endif

const char *Txt_downloads =	// As a noun
#if   L==1	// ca
	"desc&agrave;rregues";
#elif L==2	// de
	"Downloads";
#elif L==3	// en
	"downloads";
#elif L==4	// es
	"descargas";
#elif L==5	// fr
	"t&eacute;l&eacute;chargements";
#elif L==6	// gn
	"descargas";	// Okoteve traduccin
#elif L==7	// it
	"downloads";
#elif L==8	// pl
	"pobrania";
#elif L==9	// pt
	"downloads";
#elif L==10	// tr
	"downloads";	// eviri lazim!
#endif

const char *Txt_Duplicate_USERS =
#if   L==1	// ca
	"Duplicats";
#elif L==2	// de
	"Doppelte";
#elif L==3	// en
	"Duplicates";
#elif L==4	// es
	"Duplicados";
#elif L==5	// fr
	"En double";
#elif L==6	// gn
	"Duplicados";	// Okoteve traduccin
#elif L==7	// it
	"Duplicati";
#elif L==8	// pl
	"Duplikaty";
#elif L==9	// pt
	"Duplicados";
#elif L==10	// tr
	"Duplicates";	// eviri lazim!
#endif

const char *Txt_Edit_plain_text =
#if   L==1	// ca
	"Editar text sense format";
#elif L==2	// de
	"Plain Text bearbeiten";
#elif L==3	// en
	"Edit plain text";
#elif L==4	// es
	"Editar texto sin formato";
#elif L==5	// fr
	"&Eacute;diter texte simple";
#elif L==6	// gn
	"Editar texto sin formato";	// Okoteve traduccin
#elif L==7	// it
	"Editare testo normale";
#elif L==8	// pl
	"Edycja tekstu zwyk&lstrok;y";
#elif L==9	// pt
	"Editar texto simples";
#elif L==10	// tr
	"Edit plain text";		// eviri lazim!
#endif

const char *Txt_Edit_record_fields =
#if   L==1	// ca
	"Editar camps de fitxes";
#elif L==2	// de
	"Datensatzfelder bearbeiten";
#elif L==3	// en
	"Edit record fields";
#elif L==4	// es
	"Editar campos de fichas";
#elif L==5	// fr
	"Modifier champs de cartes";
#elif L==6	// gn
	"Editar campos de fichas";	// Okoteve traduccin
#elif L==7	// it
	"Editare campo schede";
#elif L==8	// pl
	"Edycja rekord w dziedzinie";
#elif L==9	// pt
	"Editar campos cart&otilde;es";
#elif L==10	// tr
	"Edit record fields";		// eviri lazim!
#endif

const char *Txt_Edit_rich_text =
#if   L==1	// ca
	"Editar text enriquit";
#elif L==2	// de
	"Rich Text bearbeiten";
#elif L==3	// en
	"Edit rich text";
#elif L==4	// es
	"Editar texto enriquecido";
#elif L==5	// fr
	"&Eacute;diter texte enrichi";
#elif L==6	// gn
	"Editar texto enriquecido";	// Okoteve traduccin
#elif L==7	// it
	"Editare testo formattato";
#elif L==8	// pl
	"Edycja tekstu sformatowany";
#elif L==9	// pt
	"Editar texto rico";
#elif L==10	// tr
	"Edit rich text";		// eviri lazim!
#endif

const char *Txt_eg_A_B =
#if   L==1	// ca
	"per ex.: A, B&hellip;";
#elif L==2	// de
	"z. B.: A, B&hellip;";
#elif L==3	// en
	"e.g., A, B&hellip;";
#elif L==4	// es
	"por ej.: A, B&hellip;";
#elif L==5	// fr
	"par exemple: A, B&hellip;";
#elif L==6	// gn
	"techapyr&atilde; ramo: A, B&hellip;";
#elif L==7	// it
	"ad esempio: A, B&hellip;";
#elif L==8	// pl
	"np. A, B&hellip;";
#elif L==9	// pt
	"ex.: A, B&hellip;";
#elif L==10	// tr
	"&ouml;rne&gbreve;in: A, B&hellip;";
#endif

const char *Txt_eg_Lectures_Practicals =
#if   L==1	// ca
	"per ex.: Teoria, Pr&agrave;ctiques&hellip;";
#elif L==2	// de
	"z. B.: Vorlesung, &Uuml;bung&hellip;";
#elif L==3	// en
	"e.g., Lectures, Practicals&hellip;";
#elif L==4	// es
	"por ej.: Teor&iacute;a, Pr&aacute;cticas&hellip;";
#elif L==5	// fr
	"par exemple: Th&eacute;orie, Pratique&hellip;";
#elif L==6	// gn
	"techapyr&atilde; ramo: Teor&iacute;a, Pr&aacute;cticas rehegua&hellip;";	// Okoteve traduccin
#elif L==7	// it
	"ad esempio: Teoria, Pratica&hellip;";
#elif L==8	// pl
	"np. Wyklady, Praktyki &hellip;";
#elif L==9	// pt
	"ex.: Teoria, Pr&aacute;tica&hellip;";
#elif L==10	// tr
	"&ouml;rne&gbreve;in: Teori, Uygulamalar&hellip;";
#endif

const char *Txt_ELAPSED_TIME =
#if   L==1	// ca
	"Fa";
#elif L==2	// de
	"Vor";
#elif L==3	// en
	"Ago";
#elif L==4	// es
	"Hace";
#elif L==5	// fr
	"Y a";
#elif L==6	// gn
	"Ojapo";
#elif L==7	// it
	"Fa";
#elif L==8	// pl
	"Temu";
#elif L==9	// pt
	"Atr&aacute;s";
#elif L==10	// tr
	"&Ouml;nce";
#endif

const char *Txt_Eliminate =
#if   L==1	// ca
	"Eliminar";
#elif L==2	// de
	"Entfernen";
#elif L==3	// en
	"Eliminate";
#elif L==4	// es
	"Eliminar";
#elif L==5	// fr
	"&Eacute;liminer";
#elif L==6	// gn
	"Pe'a";
#elif L==7	// it
	"Rimuovere";
#elif L==8	// pl
	"Wyeliminowac";
#elif L==9	// pt
	"Eliminar";
#elif L==10	// tr
	"Elemek";
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_1_OF_2 =
#if   L==1	// ca
	"Elimineu tots els cursos sense usuaris i sense acc&eacute;s en els &uacute;ltims";
#elif L==2	// de
	"Beseitigen Sie alle Kurse ohne Benutzer und ohne Zugriffe in den letzten";
#elif L==3	// en
	"Eliminate all courses without users and without accesses in the last";
#elif L==4	// es
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";
#elif L==5	// fr
	"&Eacute;liminer tous les mati&egrave;res sans utilisateurs et sans acc&egrave;s au cours des";
#elif L==6	// gn
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";	// Okoteve traduccin
#elif L==7	// it
	"Elimina tutti i corsi senza utenti e senza accessi negli ultimi";
#elif L==8	// pl
	"Wyeliminuj wszystkie kursy bez u&zdot;ytkownik&oacute;w i bez dost&eogon;pu w ci&aogon;gu ostatnich";
#elif L==9	// pt
	"Eliminar todas as disciplinas sem usu&aacute;rios e sem acessos nos &uacute;ltimos";
#elif L==10	// tr
	"Eliminate all courses without users and without accesses in the last";			// eviri lazim!
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_2_OF_2 =
#if   L==1	// ca
	"mesos.";
#elif L==2	// de
	"Monaten.";
#elif L==3	// en
	"months.";
#elif L==4	// es
	"meses.";
#elif L==5	// fr
	"derniers mois.";
#elif L==6	// gn
	"meses.";	// Okoteve traduccin
#elif L==7	// it
	"mesi.";
#elif L==8	// pl
	"miesi&eogon;cy.";
#elif L==9	// pt
	"meses.";
#elif L==10	// tr
	"months.";	// eviri lazim!
#endif

const char *Txt_Eliminate_all_users_who_are_not_enroled_on_any_courses_PART_1_OF_2 =
#if   L==1	// ca
	"Elimineu tots els usuaris"
	" que no estiguin inscrits en cap curs"
	" i amb m&eacute;s de";
#elif L==2	// de
	"Beseitigen Sie alle Benutzer"
	" die nicht an Kursen angemeldet sind"
	" und mehr als";
#elif L==3	// en
	"Eliminate all users"
	" who are not enroled on any courses"
	" and with more than";
#elif L==4	// es
	"Eliminar todos los usuarios"
	" que no est&eacute;n inscritos en ninguna asignatura"
	" y que lleven m&aacute;s de";
#elif L==5	// fr
	"&Eacute;liminer tous les utilisateurs"
	" qui ne sont inscrits &agrave; aucune mati&egrave;re"
	" et qui ont plus de";
#elif L==6	// gn
	"Eliminar todos los usuarios"
	" que no est&eacute;n inscritos en ninguna asignatura"
	" y que lleven m&aacute;s de";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere tutti gli utenti"
	" che non sono registrati ad alcun corso"
	" e con pi&ugrave; di";
#elif L==8	// pl
	"Wyeliminuj wszystkich u&zdot;ytkownik&oacute;w"
	" kt&oacute;rzy nie s&aogon; zapisani na &zdot;adne kursy"
	" i d&lstrok;u&zdot;ej ni&zdot;";
#elif L==9	// pt
	"Eliminar todos os usu&aacute;rios"
	" que n&atilde,o est&atilde;o matriculados em nenhuma disciplina"
	" e com mais de";
#elif L==10	// tr
	"Eliminate all users"
	" who are not enroled on any courses"
	" and with more than";		// eviri lazim!
#endif

const char *Txt_Eliminate_all_users_who_are_not_enroled_on_any_courses_PART_2_OF_2 =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"mesos sense acc&eacute;s a %s.";
#elif L==2	// de
	"Monate ohne Zugriff auf %s.";
#elif L==3	// en
	"months without access to %s.";
#elif L==4	// es
	"meses sin acceder a %s.";
#elif L==5	// fr
	"mois sans acc&egrave;s &agrave; %s.";
#elif L==6	// gn
	"meses sin acceder a %s.";	// Okoteve traduccin
#elif L==7	// it
	"mesi senza accedere a %s.";
#elif L==8	// pl
	"miesi&aogon;ce bez dost&eogon;pu do %s.";
#elif L==9	// pt
	"meses sem acesso a %s.";
#elif L==10	// tr
	"months without access to %s.";	// eviri lazim!
#endif

const char *Txt_Eliminate_from_the_platform_the_users_indicated_in_step_1 =
#if   L==1	// ca
	"Eliminar completament de la plataforma"
	" els usuaris indicats al pas 1";
#elif L==2	// de
	"Entfernen Sie die in Schritt 1 angegebenen"
	" Benutzer vollst&auml;ndig von der Plattform";
#elif L==3	// en
	"Eliminate from the platform"
	" the users indicated on step 1";
#elif L==4	// es
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";
#elif L==5	// fr
	"&Eacute;liminer de la plate-forme"
	" les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";		// Okoteve traduccin
#elif L==7	// it
	"Rimuovere dalla piattaforma"
	" gli utenti che sono in lista";
#elif L==8	// pl
	"Wyeliminuj z platformy"
	" uzytkownikow kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Eliminar da plataforma"
	" os utilizadores que est&atilde;o na lista";
#elif L==10	// tr
	"Eliminate from the platform"
	" the users indicated on step 1";		// eviri lazim!
#endif

const char *Txt_Eliminate_old_courses =
#if   L==1	// ca
	"Eliminar assignatures antics";
#elif L==2	// de
	"Fr&uuml;here Kursen entfernen";
#elif L==3	// en
	"Eliminate old courses";
#elif L==4	// es
	"Eliminar asignaturas antiguas";
#elif L==5	// fr
	"&Eacute;liminer anciens mati&egrave;res";
#elif L==6	// gn
	"Eliminar asignaturas antiguas";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere vecchi corsi";
#elif L==8	// pl
	"Wyeleminuj starych kurs&oacute;w";
#elif L==9	// pt
	"Eliminar antigas disciplinas";
#elif L==10	// tr
	"Eliminate old courses";		// eviri lazim!
#endif

const char *Txt_Eliminate_old_users =
#if   L==1	// ca
	"Eliminar usuaris antics";
#elif L==2	// de
	"Fr&uuml;here Benutzer entfernen";
#elif L==3	// en
	"Eliminate old users";
#elif L==4	// es
	"Eliminar usuarios antiguos";
#elif L==5	// fr
	"&Eacute;liminer anciens utilisateurs";
#elif L==6	// gn
	"Eliminar usuarios antiguos";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere vecchi utenti";
#elif L==8	// pl
	"Usu&nacute; starych uzytkownik&oacute;w";
#elif L==9	// pt
	"Eliminar antigos utilizadores";
#elif L==10	// tr
	"Eliminate old users";		// eviri lazim!
#endif

const char *Txt_Eliminating_X_courses_whithout_users_and_with_more_than_Y_months_without_access =	// Warning: it is very important to include %lu and %u in the following sentences
#if   L==1	// ca
	"Eliminant %lu assignatures sense usuaris"
	" i amb m&eacute;s de %u mesos sense acc&eacute;s.";
#elif L==2	// de
	"Eliminierung von %lu Kurs(en) ohne Benutzer"
	" und mit mehr als %u Monaten ohne Zugang.";
#elif L==3	// en
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";
#elif L==4	// es
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";
#elif L==5	// fr
	"Suppression de %lu cours sans utilisateurs"
	" et avec plus de %u mois sans acc&egrave;s.";
#elif L==6	// gn
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";	// Okoteve traduccin
#elif L==7	// it
	"Eliminazione di %lu corsi senza utenti"
	" e con pi&ugrave; di %u mesi senza accesso.";
#elif L==8	// pl
	"Eliminacja %lu kurs&oacute;w bez u&zdot;ytkownik&oacute;w"
	" i z ponad %u miesi&aogon;cami bez dost&eogon;pu.";
#elif L==9	// pt
	"Eliminando %lu disciplina(s) sem usu&aacute;rios"
	" e com mais de %u meses sem acesso.";
#elif L==10	// tr
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// eviri lazim!
#endif

const char *Txt_Eliminate_my_user_account =
#if   L==1	// ca
	"Eliminar la meva compte d'usuari";
#elif L==2	// de
	"Entfernen mein Benutzerkonto";
#elif L==3	// en
	"Eliminate my user account";
#elif L==4	// es
	"Eliminar mi cuenta de usuario/a";
#elif L==5	// fr
	"Supprimer mon compte d'utilisateur";
#elif L==6	// gn
	"Eliminar mi cuenta de usuario/a";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere il mio account utente";
#elif L==8	// pl
	"Usu&nacute; moje konto u&zdot;ytkownika";
#elif L==9	// pt
	"Remover minha conta de usu&aacute;rio";
#elif L==10	// tr
	"Eliminate my user account";		// eviri lazim!
#endif

const char *Txt_Eliminate_user_account =
#if   L==1	// ca
	"Eliminar compte d'usuari";
#elif L==2	// de
	"Entfernen Benutzerkonto";
#elif L==3	// en
	"Eliminate user account";
#elif L==4	// es
	"Eliminar cuenta de usuario/a";
#elif L==5	// fr
	"Supprimer compte d'utilisateur";
#elif L==6	// gn
	"Eliminar cuenta de usuario/a";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere account utente";
#elif L==8	// pl
	"Usu&nacute; konto u&zdot;ytkownika";
#elif L==9	// pt
	"Remover conta de usu&aacute;rio";
#elif L==10	// tr
	"Eliminate user account";	// eviri lazim!
#endif

const char *Txt_Eliminating_X_users_who_were_not_enroled_in_any_course_and_with_more_than_Y_months_without_access_to_Z =	// Warning: it is very important to include %lu, %u and %s in the following sentences
#if   L==1	// ca
	"Eliminant %lu usuaris que no estaven matriculats en cap assignatura"
	" i amb m&eacute;s de %u mesos sense acc&eacute;s a %s.";
#elif L==2	// de
	"Eliminierung von %lu Benutzern, die in keinem Kurs eingeschrieben waren"
	" und seit mehr als %u Monaten keinen Zugriff auf %s haben.";
#elif L==3	// en
	"Eliminating %lu user(s) who were not enroled in any course"
	" and with more than %u months without access to %s.";
#elif L==4	// es
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";
#elif L==5	// fr
	"Suppression de %lu utilisateur(s) qui n'&eacute;taient inscrits &agrave; aucune mati&egrave;re"
	" et avec plus de %u mois sans acc&egrave;s &agrave; %s.";
#elif L==6	// gn
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovendo %lu utenti che non siano registrati in alcun corso"
	" e con pi&ugrave; di %u mesi senza accedere a %s.";
#elif L==8	// pl
	"Eliminowanie %lu u&zdot;ytkownik&oacute;w, kt&oacute;rzy nie byli zapisani na &zdot;aden kurs"
	" i d&lstrok;u&zdor;j ni&zdot; %u miesi&aogon;ce bez dost&eogon;pu do %s.";
#elif L==9	// pt
	"Eliminando %lu usu&aacute;rio(s) que n&atilde;o estavam matriculados em nenhuma disciplina"
	" e com mais de %u meses sem acesso a %s.";
#elif L==10	// tr
	"Eliminating %lu user(s) who were not enroled in any course"
	" and with more than %u months without access to %s.";		// eviri lazim!
#endif

const char *Txt_Email =
#if   L==1	// ca
	"Correu";
#elif L==2	// de
	"E-Mail";
#elif L==3	// en
	"Email";
#elif L==4	// es
	"Correo";
#elif L==5	// fr
	"Courrier";
#elif L==6	// gn
	"&Ntilde;e'&#7869;veve";
#elif L==7	// it
	"Email";
#elif L==8	// pl
	"Email";
#elif L==9	// pt
	"Email";
#elif L==10	// tr
	"E-posta";
#endif

const char *Txt_Email_addresses =
#if   L==1	// ca
	"Adreces de correu";
#elif L==2	// de
	"E-Mail-Adressen";
#elif L==3	// en
	"Email addresses";
#elif L==4	// es
	"Direcciones de correo";
#elif L==5	// fr
	"Adresses e-mail";
#elif L==6	// gn
	"Direcciones de correo";	// Okoteve traduccin
#elif L==7	// it
	"Indirizzi email";
#elif L==8	// pl
	"Adresy e-mail";
#elif L==9	// pt
	"Endere&ccedil;os de email";
#elif L==10	// tr
	"Email addresses";		// eviri lazim!
#endif

const char *Txt_Email_domain_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Domini de correu <strong>%s</strong> eliminat.";
#elif L==2	// de
	"E-Mail-Dom&auml;ne <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Email domain <strong>%s</strong> removed.";
#elif L==4	// es
	"Dominio de correo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Domaine de messagerie <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Dominio de correo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Campo mail <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Domeny poczty <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Dom&iacute;nio de email <strong>%s</strong> removido.";
#elif L==10	// tr
	"Email domain <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Email_domains_allowed_for_notifications =
#if   L==1	// ca
	"Dominis de correu permesos per a notificacions";
#elif L==2	// de
	"E-Mail-Dom&auml;nen, an die Benachrichtigungen gesendet werden k&ouml;nnen";
#elif L==3	// en
	"Email domains allowed for notifications";
#elif L==4	// es
	"Dominios de correo permitidos para notificaciones";
#elif L==5	// fr
	"Domaines de messagerie autoris&eacute;s pour les notifications";
#elif L==6	// gn
	"Dominios de correo permitidos para notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Campo mail permesso per notifiche";
#elif L==8	// pl
	"Mail domen dozwolone powiadomien";
#elif L==9	// pt
	"Dom&iacute;nios de email permitidos para notifica&ccedil;&otilde;es";
#elif L==10	// tr
	"Email domains allowed for notifications";		// eviri lazim!
#endif

const char *Txt_EMAIL_DOMAIN_HELP_ORDER[Mai_NUM_ORDERS] =
	{
	[Mai_ORDER_BY_DOMAIN] =
#if   L==1	// ca
	"Ordenar per domini de correu"
#elif L==2	// de
	"Nach E-Mail-Dom&auml;ne sortieren"
#elif L==3	// en
	"Sort by email domain"
#elif L==4	// es
	"Ordenar por dominio de correo"
#elif L==5	// fr
	"Trier par domaine de messagerie"
#elif L==6	// gn
	"Ordenar por dominio de correo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per campo mail"
#elif L==8	// pl
	"Sortuj wedlug domeny poczty"
#elif L==9	// pt
	"Classificar por dom&iacute;nio de email"
#elif L==10	// tr
	"Sort by email domain"		// eviri lazim!
#endif
	,
	[Mai_ORDER_BY_INFO] =
#if   L==1	// ca
	"Ordenar per informaci&oacute;"
#elif L==2	// de
	"Nach Information sortieren"
#elif L==3	// en
	"Sort by information"
#elif L==4	// es
	"Ordenar por informaci&oacute;n"
#elif L==5	// fr
	"Trier par information"
#elif L==6	// gn
	"Ordenar por informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per informazione"
#elif L==8	// pl
	"Sortuj wedlug informacji"
#elif L==9	// pt
	"Classificar por informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Sort by information"			// eviri lazim!
#endif
	,
	[Mai_ORDER_BY_USERS] =
#if   L==1	// ca
	"Ordenar per nombre d'usuaris"
#elif L==2	// de
	"Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	"Sort by no. of users"
#elif L==4	// es
	"Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	"Trier par nombre d'utilisateurs"
#elif L==6	// gn
	"Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di utenti"
#elif L==8	// pl
	"Sortuj wedlug numerow uzytkownikow"
#elif L==9	// pt
	"Classificar por n&ordm; de utilizadores"
#elif L==10	// tr
	"Sort by no. of users"			// eviri lazim!
#endif
	};

const char *Txt_EMAIL_DOMAIN_ORDER[Mai_NUM_ORDERS] =
	{
	[Mai_ORDER_BY_DOMAIN] =
#if   L==1	// ca
	"Domini de correu"
#elif L==2	// de
	"E-Mail-Dom&auml;ne"
#elif L==3	// en
	"Email domain"
#elif L==4	// es
	"Dominio de correo"
#elif L==5	// fr
	"Domaine de messagerie"
#elif L==6	// gn
	"Dominio correo"
#elif L==7	// it
	"Campo mail"
#elif L==8	// pl
	"Domeny poczty"
#elif L==9	// pt
	"Dom&iacute;nio de email"
#elif L==10	// tr
	"E-posta etki alan&inodot;"
#endif
	,
	[Mai_ORDER_BY_INFO] =
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	[Mai_ORDER_BY_USERS] =
#if   L==1	// ca
	"Usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarios"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"Uzytkownikow"
#elif L==9	// pt
	"Utilizadores"
#elif L==10	// tr
	"Users"		// eviri lazim!
#endif
	};

const char *Txt_Email_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Correu %s confirmat";
#elif L==2	// de
	"E-Mail %s best&auml;tigt";
#elif L==3	// en
	"Email %s confirmed";
#elif L==4	// es
	"Correo %s confirmado";
#elif L==5	// fr
	"E-mail %s confirm&eacute;";
#elif L==6	// gn
	"Correo %s confirmado";	// Okoteve traduccin
#elif L==7	// it
	"E-mail %s confermata";
#elif L==8	// pl
	"E-mail %s potwierdzony";
#elif L==9	// pt
	"Email %s confirmado";
#elif L==10	// tr
	"Email %s confirmed";	// eviri lazim!
#endif

const char *Txt_Email_X_has_already_been_confirmed_before =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El correu <strong>%s</strong>"
	" ja s'ha confirmat abans.";
#elif L==2	// de
	"E-Mail <strong>%s</strong>"
	" wurde bereits zuvor best&auml;tigt.";
#elif L==3	// en
	"Email <strong>%s</strong>"
	" has already been confirmed before.";
#elif L==4	// es
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";
#elif L==5	// fr
	"L'e-mail <strong>%s</strong>"
	" a d&eacute;j&agrave; &eacute;t&eacute; confirm&eacute; auparavant.";
#elif L==6	// gn
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";	// Okoteve traduccin
#elif L==7	// it
	"L'email <strong>%s</strong>"
	" &egrave; gi&agrave; stata confermata in precedenza.";
#elif L==8	// pl
	"E-mail <strong>%s</strong>"
	" zosta&lstrok; ju&zdot; wcze&sacute;niej potwierdzony.";
#elif L==9	// pt
	"Email <strong>%s</strong>"
	" j&aacute; foi confirmado antes.";
#elif L==10	// tr
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// eviri lazim!
#endif

const char *Txt_Email_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Correu <strong>%s</strong> eliminat.";
#elif L==2	// de
	"E-Mail <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Email <strong>%s</strong> removed.";
#elif L==4	// es
	"Correo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Courrier <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Correo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Email <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Email <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Email <strong>%s</strong> removido.";
#elif L==10	// tr
	"Email <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_End_date =
#if   L==1	// ca
	"Data final";
#elif L==2	// de
	"Enddatum";
#elif L==3	// en
	"End date";
#elif L==4	// es
	"Fecha final";
#elif L==5	// fr
	"Date finale";
#elif L==6	// gn
	"Fecha final";	// Okoteve traduccin
#elif L==7	// it
	"Data finale";
#elif L==8	// pl
	"Data koncowa";
#elif L==9	// pt
	"Data final";
#elif L==10	// tr
	"End date";	// eviri lazim!
#endif

const char *Txt_Enrol_in_groups =
#if   L==1	// ca
	"Inscriure en grups";
#elif L==2	// de
	"Anmeldung mich in Gruppen";
#elif L==3	// en
	"Enrol in groups";
#elif L==4	// es
	"Inscribirme en grupos";
#elif L==5	// fr
	"Inscriver moi dans groupes";
#elif L==6	// gn
	"Inscribirme en grupos";	// Okoteve traduccin
#elif L==7	// it
	"Iscriver mi in gruppi";
#elif L==8	// pl
	"Zarejestruj mnie w grupach";
#elif L==9	// pt
	"Inscreva-me em grupos ";
#elif L==10	// tr
	"Enrol in groups";		// eviri lazim!
#endif

const char *Txt_Enrolment =
#if   L==1	// ca
	"Inscripci&oacute;";
#elif L==2	// de
	"Einschreibung";
#elif L==3	// en
	"Enrolment";
#elif L==4	// es
	"Inscripci&oacute;n";
#elif L==5	// fr
	"Inscription";
#elif L==6	// gn
	"Inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione";
#elif L==8	// pl
	"Rejestracja";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o";
#elif L==10	// tr
	"Enrolment";		// eviri lazim!
#endif

const char *Txt_Enrolment_confirmed =
#if   L==1	// ca
	"Inscripci&oacute; confirmada";
#elif L==2	// de
	"Anmeldung best&auml;tigt";
#elif L==3	// en
	"Enrolment confirmed";
#elif L==4	// es
	"Inscripci&oacute;n confirmada";
#elif L==5	// fr
	"Inscription confirm&eacute;e";
#elif L==6	// gn
	"Inscripci&oacute;n confirmada";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione confermata";
#elif L==8	// pl
	"Rejestracja potwierdzona";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o confirmada";
#elif L==10	// tr
	"Enrolment confirmed";			// eviri lazim!
#endif

const char *Txt_Enrolment_of_X_rejected =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscripci&oacute; de <strong>%s</strong> rebutjada.";
#elif L==2	// de
	"Anmeldung von <strong>%s</strong> abgelehnt.";
#elif L==3	// en
	"Enrolment of <strong>%s</strong> rejected.";
#elif L==4	// es
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";
#elif L==5	// fr
	"Inscription de <strong>%s</strong> rejet&eacute;e.";
#elif L==6	// gn
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";	// Okoteve traduccin
#elif L==7	// it
	"Iscrizione di <strong>%s</strong> rifiutata.";
#elif L==8	// pl
	"Rejestracja <strong>%s</strong> zosta&lstrok;a odrzucona.";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o de <strong>%s</strong> rejeitada.";
#elif L==10	// tr
	"Enrolment of <strong>%s</strong> rejected.";		// eviri lazim!
#endif

const char *Txt_Enrolment_not_confirmed =
#if   L==1	// ca
	"Inscripci&oacute; pendent de confirmaci&oacute;";
#elif L==2	// de
	"Anmeldung nicht best&auml;tigt";
#elif L==3	// en
	"Enrolment not confirmed";
#elif L==4	// es
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";
#elif L==5	// fr
	"Inscription non confirm&eacute;e";
#elif L==6	// gn
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione non confermata";
#elif L==8	// pl
	"Rejestracja nie potwierdzona";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o n&atilde;o confirmada";
#elif L==10	// tr
	"Enrolment not confirmed";				// eviri lazim!
#endif

const char *Txt_Enrolment_requests =
#if   L==1	// ca
	"Peticions d'inscripci&oacute;";
#elif L==2	// de
	"Registrierungsanforderungen";
#elif L==3	// en
	"Enrolment requests";
#elif L==4	// es
	"Solicitudes de inscripci&oacute;n";
#elif L==5	// fr
	"Demandes d'inscription";
#elif L==6	// gn
	"Solicitudes de inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Richieste di registrazione";
#elif L==8	// pl
	"Pro&sacute;by o rejestracji";
#elif L==9	// pt
	"Pedidos de inscri&ccedil;&atilde;o";
#elif L==10	// tr
	"Enrolment requests";			// eviri lazim!
#endif

const char *Txt_Enter_a_new_item_here =
#if   L==1	// ca
	"Escriviu aqu&iacute; un nou apartat";
#elif L==2	// de
	"Neuen Eintrag hier eingeben";
#elif L==3	// en
	"Enter a new item here";
#elif L==4	// es
	"Escriba aqu&iacute; un nuevo apartado";
#elif L==5	// fr
	"Ecrivez ici un nouveau point";
#elif L==6	// gn
	"Escriba aqu&iacute; un nuevo apartado";	// Okoteve traduccin
#elif L==7	// it
	"Inserisci qui un nuovo oggetto(tambi&egrave;n item est&agrave; bien)";
#elif L==8	// pl
	"Wpisz nowy element o";
#elif L==9	// pt
	"Escreva aqui uma nova rubrica";
#elif L==10	// tr
	"Enter a new item here";			// eviri lazim!
#endif

const char *Txt_Enter_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Entrar des %s";
#elif L==2	// de
	"Geben Sie %s";
#elif L==3	// en
	"Enter from %s";
#elif L==4	// es
	"Entrar desde %s";
#elif L==5	// fr
	"Entrez du %s";
#elif L==6	// gn
	"Entrar desde %s";	// Okoteve traduccin
#elif L==7	// it
	"Inserisci da %s";
#elif L==8	// pl
	"Wprowadzi&cacute; z %s";
#elif L==9	// pt
	"Digite partir %s";
#elif L==10	// tr
	"Enter from %s";	// eviri lazim!
#endif

const char *Txt_Equation_centered =
#if   L==1	// ca
	"Equaci&oacute; centrada";
#elif L==2	// de
	"Gleichung zentriert";
#elif L==3	// en
	"Equation centered";
#elif L==4	// es
	"Ecuaci&oacute;n centrada";
#elif L==5	// fr
	"Equation centr&eacute;";
#elif L==6	// gn
	"Ecuaci&oacute;n centrada";	// Okoteve traduccin
#elif L==7	// it
	"Equazione centrato";
#elif L==8	// pl
	"R&oacute;wnanie centered";
#elif L==9	// pt
	"equa&ccedil;&atilde;o centrado";
#elif L==10	// tr
	"Equation centered";		// eviri lazim!
#endif

const char *Txt_Error_getting_data_from_a_recipient =
#if   L==1	// ca
	"Error obtenint dades dun destinatari.";
#elif L==2	// de
	"Fehler beim Abrufen von Daten von einem Empf&auml;nger.";
#elif L==3	// en
	"Error getting data from a recipient.";
#elif L==4	// es
	"Error obteniendo datos de un destinatario.";
#elif L==5	// fr
	"Erreur lors de l'obtention des donn&eacute;es d'un destinataire.";
#elif L==6	// gn
	"Error obteniendo datos de un destinatario.";	// Okoteve traduccin
#elif L==7	// it
	"Errore acquisizione dati da un destinatario.";
#elif L==8	// pl
	"Blad podczas pobierania danych z odbiorca.";
#elif L==9	// pt
	"Erro ao obter dados de um destinat&aacute;rio.";
#elif L==10	// tr
	"Error getting data from a recipient.";		// eviri lazim!
#endif

const char *Txt_Error_sending_or_processing_image_video =
#if   L==1	// ca
	"Error enviant o processant imatge / v&iacute;deo.";
#elif L==2	// de
	"Fehler beim Senden oder Verarbeiten eines Abbild / Video.";
#elif L==3	// en
	"Error sending or processing image / video.";
#elif L==4	// es
	"Error enviando o procesando imagen / v&iacute;deo.";
#elif L==5	// fr
	"Erreur d'envoi ou de traitement de l'image / vid&eacute;o.";
#elif L==6	// gn
	"Error enviando o procesando imagen / v&iacute;deo.";	// Okoteve traduccin
#elif L==7	// it
	"Errore nell'invio o nell'elaborazione dell'immagine / video.";
#elif L==8	// pl
	"B&lstrok;&aogon;d podczas wysy&lstrok;ania lub przetwarzania obrazu / wideo.";
#elif L==9	// pt
	"Erro ao enviar ou processar imagem / v&iacute;deo.";
#elif L==10	// tr
	"Error sending or processing image / video.";		// eviri lazim!
#endif

const char *Txt_Event =
#if   L==1	// ca
	"Esdeveniment";
#elif L==2	// de
	"Ereignis";
#elif L==3	// en
	"Event";
#elif L==4	// es
	"Evento";
#elif L==5	// fr
	"&Eacute;v&eacute;nement";
#elif L==6	// gn
	"Evento";	// Okoteve traduccin
#elif L==7	// it
	"Evento";
#elif L==8	// pl
	"Wydarzenie";
#elif L==9	// pt
	"Evento";
#elif L==10	// tr
	"Event";	// eviri lazim!
#endif

const char *Txt_Event_X_is_now_private =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'esdevenimen <strong>%s</strong> ara &eacute;s privat.";
#elif L==2	// de
	"Der Ereignis <strong>%s</strong> ist jetzt privat.";
#elif L==3	// en
	"Event <strong>%s</strong> is now private.";
#elif L==4	// es
	"El evento <strong>%s</strong> ahora es privado.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant priv&eacute;.";
#elif L==6	// gn
	"El evento <strong>%s</strong> ahora es privado.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento <strong>%s</strong> &egrave; ora privato.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> jest obecnie prywatne.";
#elif L==9	// pt
	"O evento <strong>%s</strong> &eacute; agora privado.";
#elif L==10	// tr
	"Event <strong>%s</strong> is now private.";	// eviri lazim!
#endif

const char *Txt_Event_X_is_now_visible_to_users_of_your_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'esdevenimen <strong>%s</strong> ara &eacute;s visible"
	" pels usuaris de les seves assignatures.";
#elif L==2	// de
	"Der Ereignis <strong>%s</strong> ist jetzt sichtbar"
	" f&uuml;r Benutzer Ihrer Kurse.";
#elif L==3	// en
	"Event <strong>%s</strong> is now visible"
	" to users of your courses.";
#elif L==4	// es
	"El evento <strong>%s</strong> ahora es visible"
	" por los usuarios de sus asignaturas.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant visible"
	" pour les utilisateurs de vos mati&egrave;res.";
#elif L==6	// gn
	"El evento <strong>%s</strong> ahora es visible"
	" por los usuarios de sus asignaturas.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento <strong>%s</strong> &egrave; ora visibile"
	" agli utenti dei corsi.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> jest teraz widoczne"
	" dla u&zdot;ytkownik&oacute;w swoich kursach.";
#elif L==9	// pt
	"O evento <strong>%s</strong> &eacute; agora vis&iacute;vel"
	" para os usu&aacute;rios de as suas disciplinas.";
#elif L==10	// tr
	"Event <strong>%s</strong> is now visible"
	" to users of your courses.";			// eviri lazim!
#endif

const char *Txt_Event_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Esdeveniment <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Ereignis <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Event <strong>%s</strong> removed.";
#elif L==4	// es
	"Evento <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"&Eacute;v&eacute;nement <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Event <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Evento <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Evento <strong>%s</strong> removido.";
#elif L==10	// tr
	"Event <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_Events =
#if   L==1	// ca
	"Esdeveniments";
#elif L==2	// de
	"Ereignisse";
#elif L==3	// en
	"Events";
#elif L==4	// es
	"Eventos";
#elif L==5	// fr
	"&Eacute;v&eacute;nements";
#elif L==6	// gn
	"Eventos";	// Okoteve traduccin
#elif L==7	// it
	"Eventi";
#elif L==8	// pl
	"Wydarzen";
#elif L==9	// pt
	"Eventos";
#elif L==10	// tr
	"Events";	// eviri lazim!
#endif

const char *Txt_Exam =
#if   L==1	// ca
	"Ex&agrave;men";
#elif L==2	// de
	"Pr&uuml;fung";
#elif L==3	// en
	"Exam";
#elif L==4	// es
	"Examen";
#elif L==5	// fr
	"Examen";
#elif L==6	// gn
	"Aranduchauka";
#elif L==7	// it
	"Esame";
#elif L==8	// pl
	"Egzamin";
#elif L==9	// pt
	"Exame";
#elif L==10	// tr
	"S&inodot;nav";
#endif

const char *Txt_EXAM_LOG_ACTIONS[ExaLog_NUM_ACTIONS] =
	{
	[ExaLog_UNKNOWN_ACTION] =
#if   L==1	// ca
	"Acci&oacute; desconeguda"
#elif L==2	// de
	"Unbekannte Aktion"
#elif L==3	// en
	"Unknown action"
#elif L==4	// es
	"Acci&oacute;n desconocida"
#elif L==5	// fr
	"Action inconnue"
#elif L==6	// gn
	"Acci&oacute;n desconocida"	// Okoteve traduccin
#elif L==7	// it
	"Azione sconosciuta"
#elif L==8	// pl
	"Nieznana akcja"
#elif L==9	// pt
	"A&ccedil;&atilde;o desconhecida"
#elif L==10	// tr
	"Unknown action"		// eviri lazim!
#endif
	,
	[ExaLog_START_EXAM] =
#if   L==1	// ca
	"Va comen&ccedil;ar l'examen"
#elif L==2	// de
	"Begann die Pr&uacute;fung"
#elif L==3	// en
	"Started the exam"
#elif L==4	// es
	"Comenz&oacute; el examen"
#elif L==5	// fr
	"A commenc&eacute; l'examen"
#elif L==6	// gn
	"Comenz&oacute; el examen"	// Okoteve traduccin
#elif L==7	// it
	"Ha iniziato l'esame"
#elif L==8	// pl
	"Rozpocz&aogon;&lstrok; egzamin"
#elif L==9	// pt
	"Iniciou o exame"
#elif L==10	// tr
	"Started the exam"		// eviri lazim!
#endif
	,
	[ExaLog_RESUME_EXAM] =
#if   L==1	// ca
	"Va reprendre l'examen"
#elif L==2	// de
	"Nahm die Pr&uuml;fung wieder auf"
#elif L==3	// en
	"Resumed the exam"
#elif L==4	// es
	"Reanud&oacute; el examen"
#elif L==5	// fr
	"A repris l'examen"
#elif L==6	// gn
	"Reanud&oacute; el examen"	// Okoteve traduccin
#elif L==7	// it
	"Ha ripreso l'esame"
#elif L==8	// pl
	"Wznowi&lstrok; egzamin"
#elif L==9	// pt
	"Retomou o exame"
#elif L==10	// tr
	"Resumed the exam"		// eviri lazim!
#endif
	,
	[ExaLog_ANSWER_QUESTION] =
#if   L==1	// ca
	"Va respondre una pregunta"
#elif L==2	// de
	"Beantwortete eine Frage"
#elif L==3	// en
	"Answer exam question"
#elif L==4	// es
	"Respondi&oacute; una pregunta"
#elif L==5	// fr
	"A r&eacute;pondu &agrave; une question"
#elif L==6	// gn
	"Respondi&oacute; una pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Ha risposto a una domanda"
#elif L==8	// pl
	"Odpowiedzia&lstrok; na pytanie"
#elif L==9	// pt
	"Respondeu a uma pergunta"
#elif L==10	// tr
	"Answer exam question"		// eviri lazim!
#endif
	,
	[ExaLog_FINISH_EXAM] =
#if   L==1	// ca
	"Va acabar l'examen"
#elif L==2	// de
	"Beendete die Pr&uuml;fung"
#elif L==3	// en
	"Finished the exam"
#elif L==4	// es
	"Termin&oacute; el examen"
#elif L==5	// fr
	"A termin&eacute; l'examen"
#elif L==6	// gn
	"Termin&oacute; el examen"		// Okoteve traduccin
#elif L==7	// it
	"Ha terminato l'esame"
#elif L==8	// pl
	"Zako&nacute;czy&lstrok; egzamin"
#elif L==9	// pt
	"Terminou o exame"
#elif L==10	// tr
	"Finished the exam"	// eviri lazim!
#endif
	};

const char *Txt_Exam_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Examen de %s";
#elif L==2	// de
	"Pr&uuml;fung der %s";
#elif L==3	// en
	"Exam of %s";
#elif L==4	// es
	"Examen de %s";
#elif L==5	// fr
	"Examen de %s";
#elif L==6	// gn
	"Examen de %s";		// Okoteve traduccin
#elif L==7	// it
	"Esame di %s";
#elif L==8	// pl
	"Egzamin z %s";
#elif L==9	// pt
	"Exame de %s";
#elif L==10	// tr
	"Exam of %s";		// eviri lazim!
#endif

const char *Txt_EXAM_Open =
#if   L==1	// ca
	"Obert";
#elif L==2	// de
	"Ge&ouml;ffneten";
#elif L==3	// en
	"Open";
#elif L==4	// es
	"Abierto";
#elif L==5	// fr
	"Ouvert";
#elif L==6	// gn
	"Ojei";
#elif L==7	// it
	"Aperto";
#elif L==8	// pl
	"otwarte";
#elif L==9	// pt
	"Aberta";
#elif L==10	// tr
	"Open";		// eviri lazim!
#endif

const char *Txt_Exam_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Examen <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Pr&uuml;fung <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Exam <strong>%s</strong> removed.";
#elif L==4	// es
	"Examen <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Examen <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Examen <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Esame <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Egzamin <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Exame <strong>%s</strong> removido.";
#elif L==10	// tr
	"Exam <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Exams =
#if   L==1	// ca
	"Ex&agrave;mens";
#elif L==2	// de
	"Pr&uuml;fungen";
#elif L==3	// en
	"Exams";
#elif L==4	// es
	"Ex&aacute;menes";
#elif L==5	// fr
	"Examens";
#elif L==6	// gn
	"Ex&aacute;menes";	// Okoteve traduccin
#elif L==7	// it
	"Esami";
#elif L==8	// pl
	"Egzaminy";
#elif L==9	// pt
	"Exames";
#elif L==10	// tr
	"S&inodot;navlar";
#endif

const char *Txt_EXAMS_ORDER[Exa_NUM_ORDERS] =
	{
	[Exa_ORDER_BY_START_DATE] =
#if   L==1	// ca
	"Inici"
#elif L==2	// de
	"Start"
#elif L==3	// en
	"Start"
#elif L==4	// es
	"Inicio"
#elif L==5	// fr
	"D&eacute;but"
#elif L==6	// gn
	"&Ntilde;epyr&utilde;"
#elif L==7	// it
	"Inizio"
#elif L==8	// pl
	"Pocz&aogon;tek"
#elif L==9	// pt
	"In&iacute;cio"
#elif L==10	// tr
	"Ba&scedil;lama"
#endif
	,
	[Exa_ORDER_BY_END_DATE] =
#if   L==1	// ca
	"Final"
#elif L==2	// de
	"Ende"
#elif L==3	// en
	"End"
#elif L==4	// es
	"Final"
#elif L==5	// fr
	"Fin"
#elif L==6	// gn
	"Paha"
#elif L==7	// it
	"Fine"
#elif L==8	// pl
	"Koniec"
#elif L==9	// pt
	"Fim"
#elif L==10	// tr
	"Son"
#endif
	,
	[Exa_ORDER_BY_TITLE] =
#if   L==1	// ca
	"Ex&agrave;men"
#elif L==2	// de
	"Pr&uuml;fung"
#elif L==3	// en
	"Exam"
#elif L==4	// es
	"Examen"
#elif L==5	// fr
	"Examen"
#elif L==6	// gn
	"Aranduchauka"
#elif L==7	// it
	"Esame"
#elif L==8	// pl
	"Egzamin"
#elif L==9	// pt
	"Exame"
#elif L==10	// tr
	"Exam"	// eviri lazim!
#endif
	};

const char *Txt_EXAMS_ORDER_HELP[Gam_NUM_ORDERS] =
	{
	[Exa_ORDER_BY_START_DATE] =
#if   L==1	// ca
	"Ordenar per data de comen&ccedil;ament"
#elif L==2	// de
	"Sortieren nach Startdatum"
#elif L==3	// en
	"Sort by start date"
#elif L==4	// es
	"Ordenar por fecha de comienzo"
#elif L==5	// fr
	"Trier par date de d&eacute;but"
#elif L==6	// gn
	"Ordenar por fecha de comienzo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data di inizio"
#elif L==8	// pl
	"Sortuj wedlug daty rozpoczecia"
#elif L==9	// pt
	"Classificar por data de in&iacute;cio"
#elif L==10	// tr
	"Sort by start date"		// eviri lazim!
#endif
	,
	[Exa_ORDER_BY_END_DATE] =
#if   L==1	// ca
	"Ordenar per data de finalitzaci&oacute;"
#elif L==2	// de
	"Sortieren nach Enddatum"
#elif L==3	// en
	"Sort by end date"
#elif L==4	// es
	"Ordenar por fecha de finalizaci&oacute;n"
#elif L==5	// fr
	"Trier par date de fin"
#elif L==6	// gn
	"Ordenar por fecha de finalizaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data di fine"
#elif L==8	// pl
	"Sortuj wedlug daty zakonczenia"
#elif L==9	// pt
	"Classificar por data de t&eacute;rmino"
#elif L==10	// tr
	"Sort by end date"				// eviri lazim!
#endif
	,
	[Exa_ORDER_BY_TITLE] =
#if   L==1	// ca
	"Ordenar per t&iacute;tol del ex&agrave;men"
#elif L==2	// de
	"Sortieren nach Pr&uuml;fungtitel"
#elif L==3	// en
	"Sort by exam title"
#elif L==4	// es
	"Ordenar por t&iacute;tulo del examen"
#elif L==5	// fr
	"Trier par titre de l'examen"
#elif L==6	// gn
	"Ordenar por t&iacute;tulo del examen"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per titolo del gioco"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug tytu&lstrok;u egzamin"
#elif L==9	// pt
	"Classificar por t&iacute;tulo do exame"
#elif L==10	// tr
	"Sort by exam title"			// eviri lazim!
#endif
	};

const char *Txt_Existing_question =
#if   L==1	// ca
	"Pregunta ja existent";
#elif L==2	// de
	"Bestehende Frage";
#elif L==3	// en
	"Existing question";
#elif L==4	// es
	"Pregunta ya existente";
#elif L==5	// fr
	"Question existante";
#elif L==6	// gn
	"Pregunta ya existente";	// Okoteve traduccin
#elif L==7	// it
	"Domanda esistente";
#elif L==8	// pl
	"Istniej&aogon;ca pytanie";
#elif L==9	// pt
	"Pergunta existente";
#elif L==10	// tr
	"Existing question";		// eviri lazim!
#endif

const char *Txt_Expand =
#if   L==1	// ca
	"Expandir";
#elif L==2	// de
	"Erweitern";
#elif L==3	// en
	"Expand";
#elif L==4	// es
	"Expandir";
#elif L==5	// fr
	"&Eacute;largir";
#elif L==6	// gn
	"Expandir";	// Okoteve traduccin
#elif L==7	// it
	"Espandere";
#elif L==8	// pl
	"Rozwin";
#elif L==9	// pt
	"Expandir";
#elif L==10	// tr
	"Expand";	// eviri lazim!
#endif

const char *Txt_Extended_public_profile =
#if   L==1	// ca
	"Perfil p&uacute;blic ampliat";
#elif L==2	// de
	"Erweitertes &ouml;ffentliches Profile";
#elif L==3	// en
	"Extended public profile";
#elif L==4	// es
	"Perfil p&uacute;blico extendido";
#elif L==5	// fr
	"Profil public &eacute;tendu";
#elif L==6	// gn
	"Perfil p&uacute;blico extendido";	// Okoteve traduccin
#elif L==7	// it
	"Profilo pubblico esteso";
#elif L==8	// pl
	"Rozszerzony profil publiczny";
#elif L==9	// pt
	"Perfil p&uacute;blico estendido";
#elif L==10	// tr
	"Extended public profile";		// eviri lazim!
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_1_Z_ =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"S'han detectat %u cares en posici&oacute; frontal:<br />"
	"1 amb prou llum de fons (marcat en verd),<br />"
	"%u amb el fons massa fosc (marcat en vermell).<br />"
	"Si vols triar la cara marcada en verd, fes-hi clic.";
#elif L==2	// de
	"%u Gesichter wurden in vorderer Position erkannt:<br />"
	"1 mit ausreichend hellem Hintergrund (gr&uuml;n markiert),<br />"
	"%u mit zu dunklem Hintergrund (rot markiert).<br />"
	"Wenn Sie das gr&uuml;n markierte Gesicht ausw&auml;hlen m&ouml;chten, klicken Sie darauf.";
#elif L==3	// en
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";
#elif L==4	// es
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";
#elif L==5	// fr
	"%u visages ont &eacute;t&eacute; d&eacute;tect&eacute;s en position de face:<br />"
	"1 avec le fond assez clair (marqu&eacute; en vert),<br />"
	"%u avec le fond trop sombre (marqu&eacute; en rouge).<br />"
	"Si vous voulez choisir le visage marqu&eacute; en vert, cliquez dessus.";
#elif L==6	// gn
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi sono stati individuati in posizione centrale:<br />"
	"1 con lo sfondo abbastanza chiaro (segnato in verde),<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere il viso segnato in verde, clicca su di esso.";
#elif L==8	// pl
	"%u twarze zostaly wykryte w przednim:<br />"
	"1 z dosc jasnym tle (zaznaczone na zielono),<br />"
	"%u z tlem zbyt ciemny (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac twarz zaznaczone na zielono, kliknij na niego.";
#elif L==9	// pt
	"%u faces foram detectadas na posi&ccedil;&atilde;o frontal:<br />"
	"1 com o fundo claro o suficiente (marcado em verde),<br />"
	"%u com o fundo muito escuro (marcado em vermelho).<br />"
	"Se voc&ecirc; quiser escolher o rosto marcado em verde, clique nele.";
#elif L==10	// tr
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";			// eviri lazim!
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_Y_Z_ =	// Warning: it is very important to include three %u in the following sentences
#if   L==1	// ca
	"S'han detectat %u cares en posici&oacute; frontal:<br />"
	"%u amb prou llum de fons (marcat en verd),<br />"
	"%u amb el fons massa fosc (marcat en vermell).<br />"
	"Si vols triar una de les cares marcades en verd, fes-hi clic.";
#elif L==2	// de
	"%u Gesichter wurden in vorderer Position erkannt:<br />"
	"%u mit ausreichend hellem Hintergrund (gr&uuml;n markiert),<br />"
	"%u mit zu dunklem Hintergrund (rot markiert).<br />"
	"Wenn Sie eines der gr&uuml;n markierten Gesichter ausw&auml;hlen m&ouml;chten, klicken Sie darauf.";
#elif L==3	// en
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green),<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";
#elif L==4	// es
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";
#elif L==5	// fr
	"%u visages ont &eacute;t&eacute; d&eacute;tect&eacute;s en position de face:<br />"
	"%u avec un fond suffisamment clair (marqu&eacute; en vert),<br />"
	"%u avec un arri&egrave;re-plan trop sombre (marqu&eacute; en rouge).<br />"
	"Si vous voulez choisir l'une des faces marqu&eacute;es en vert, cliquez dessus.";
#elif L==6	// gn
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi sono stati individuati in posizione centrale:<br />"
	"%u con lo sfondo abbastanza chiaro (segnato il verde),<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere uno dei visi segnati in verde, clicca su di esso.";
#elif L==8	// pl
	"%u twarze zostaly wykryte w przednim:<br />"
	"%u z dosc jasnym tlem (zaznaczone na zielono),<br />"
	"%u z tlem zbyt ciemnym (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac jedna z twarzy zaznaczone na zielono, kliknij na niego.";
#elif L==9	// pt
	"%u faces foram detectadas na posi&ccedil;&atilde;o frontal:<br />"
	"%u com o fundo claro o suficiente (marcado em verde),<br />"
	"%u com o fundo muito escuro (marcado em vermelho).<br />"
	"Se voc&ecirc; quiser escolher um dos rostos marcados em verde, clique nele.";
#elif L==10	// tr
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green),<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// eviri lazim!
#endif

const char *Txt_X_faces_marked_in_green_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han detectat %u cares (assenyalats en verd) en posici&oacute; frontal"
	" i amb el fons prou clar.<br />"
	"Si voleu triar un d'aquests rostres, premi sobre ell.";
#elif L==2	// de
	"%u Fl&auml;chen (gr&uuml;n markiert) wurden vor Position erfasst,"
	" und der Hintergrund ist leicht genug.<br />"
	"Wenn Sie eine dieser Fl&auml;chen zu w&auml;hlen, klicken Sie darauf.";
#elif L==3	// en
	"%u faces (marked in green) have been detected in front position,"
	" and the background is light enough.<br />"
	"If you want to choose one of those faces, click on it.";
#elif L==4	// es
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";
#elif L==5	// fr
	"%u visages ont &eacute;t&eacute; d&eacute;tect&eacute;s (indiqu&eacute;s en vert) dans la position avant"
	" et avec un fond suffisamment claire.<br />"
	"Si vous voulez choisir un de ces visages, cliquez dessus.";
#elif L==6	// gn
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi (segnati in verde) sono stati individuati in posizione frontale,"
	" e lo sfondo &egrave; abbastanza chiaro.<br />"
	"Se vuoi scegliere uno di questi visi, clicca su di esso.";
#elif L==8	// pl
	"%u powierzchnie zosta&lstrok;y wykryte (oznaczony na zielono) w po&lstrok;o&zdot;eniu przednim,"
	" o dostatecznie jasne t&lstrok;o.<br />"
	"Je&zdot;eli chcesz wybra&cacute; jedn&aogon; z tych twarzy, kliknij na ni&aogon;.";
#elif L==9	// pt
	"%u faces foram detectados (indicado em verde) na posi&ccedil;&atilde;o frontal"
	" e com fundo suficientemente claro.<br />"
	"Se voc&ecirc; quer escolher um daqueles rostos, clique sobre ele.";
#elif L==10	// tr
	"%u faces (marked in green) have been detected in front position,"
	" and the background is light enough.<br />"
	"If you want to choose one of those faces, click on it.";	// eviri lazim!
#endif

const char *Txt_X_faces_marked_in_red_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han detectat %u cares (assenyalats en vermell) en posici&oacute; frontal,"
	" per&ograve; el fons &eacute;s massa fosc.";
#elif L==2	// de
	"%u wurden Fl&auml;chen (in rot abgebildet) in der vorderen Position erkannt,"
	" aber der Hintergrund ist zu dunkel.";
#elif L==3	// en
	"%u faces (marked in red) have been detected in front position,"
	" but the background is too dark.";
#elif L==4	// es
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";
#elif L==5	// fr
	"%u faces (en rouge) ont &eacute;t&eacute; d&eacute;tect&eacute;s dans la position avant,"
	" mais le fond est trop sombre.";
#elif L==6	// gn
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi (segnati in rosso) sono stati individuati in posizione centrale,"
	" ma lo sfondo &egrave; troppo scuro.";
#elif L==8	// pl
	"%u wykryto twarze (pokazane na czerwono) w przednim po&lstrok;o&ztot;eniu,"
	" ale t&lstrok;o jest zbyt ciemne.";
#elif L==9	// pt
	"%u rostos foram detectados (mostrados em vermelho) na posi&ccedil;&atilde;o frontal,"
	" mas o fundo &eacute; muito escuro.";
#elif L==10	// tr
	"%u faces (marked in red) have been detected in front position,"
	" but the background is too dark.";	// eviri lazim!
#endif

const char *Txt_Faces_detected =
#if   L==1	// ca
	"Rostres detectats";
#elif L==2	// de
	"Erkannte Gesichter";
#elif L==3	// en
	"Faces detected";
#elif L==4	// es
	"Rostros detectados";
#elif L==5	// fr
	"Visages d&eacute;tect&eacute;s";
#elif L==6	// gn
	"Rostros detectados";		// Okoteve traduccin
#elif L==7	// it
	"Volti rilevati";
#elif L==8	// pl
	"Wykryto twarze";
#elif L==9	// pt
	"Rostos detectados";
#elif L==10	// tr
	"Faces detected";		// eviri lazim!
#endif

const char *Txt_Failed_email_confirmation_key =
#if   L==1	// ca
	"Error en la clave de confirmaci&oacute;n de correo.";	// Necessita traducci
#elif L==2	// de
	"E-Mail-Best&auml;tigungsschl&uuml;ssel fehlgeschlagen.";
#elif L==3	// en
	"Failed email confirmation key.";
#elif L==4	// es
	"Error en la clave de confirmaci&oacute;n de correo.";
#elif L==5	// fr
	"&Eacute;chec de la cl&eacute; de confirmation de l'e-mail.";
#elif L==6	// gn
	"Error en la clave de confirmaci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Chiave di conferma e-mail non riuscita.";
#elif L==8	// pl
	"Niepowodzenie klucza potwierdzaj&aogon;cego e-mail.";
#elif L==9	// pt
	"Erro na chave de confirma&ccedil;&atilde;o de email.";
#elif L==10	// tr
	"Failed email confirmation key.";			// eviri lazim!
#endif

const char *Txt_Feedback =
#if   L==1	// ca
	"Realimentaci&oacute;";
#elif L==2	// de
	"Feedback";
#elif L==3	// en
	"Feedback";
#elif L==4	// es
	"Realimentaci&oacute;n";
#elif L==5	// fr
	"Feedback";
#elif L==6	// gn
	"Realimentaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Feedback";
#elif L==8	// pl
	"Opinie";
#elif L==9	// pt
	"Feedback";
#elif L==10	// tr
	"Feedback";			// eviri lazim!
#endif

const char *Txt_Field_BR_name =
#if   L==1	// ca
	"Nom<br />del camp";
#elif L==2	// de
	"Name<br />des Feldes";
#elif L==3	// en
	"Field<br />name";
#elif L==4	// es
	"Nombre<br />del campo";
#elif L==5	// fr
	"Nom<br />du champ";
#elif L==6	// gn
	"Nombre<br />del campo";	// Okoteve traduccin
#elif L==7	// it
	"Nome<br />del campo";
#elif L==8	// pl
	"Nazwa<br />pola";
#elif L==9	// pt
	"Nome<br />do campo";
#elif L==10	// tr
	"Field<br />name";		// eviri lazim!
#endif

const char *Txt_Figures =
#if   L==1	// ca
	"Xifres";
#elif L==2	// de
	"Zahlen";
#elif L==3	// en
	"Figures";
#elif L==4	// es
	"Cifras";
#elif L==5	// fr
	"Chiffres";
#elif L==6	// gn
	"Cifras";	// Okoteve traduccin
#elif L==7	// it
	"Cifre";
#elif L==8	// pl
	"Cyfry";
#elif L==9	// pt
	"N&uacute;meros";
#elif L==10	// tr
	"Rakamlar";
#endif

const char *Txt_File =
#if   L==1	// ca
	"Arxiu";
#elif L==2	// de
	"Datei";
#elif L==3	// en
	"File";
#elif L==4	// es
	"Archivo";
#elif L==5	// fr
	"Fichier";
#elif L==6	// gn
	"Jehai'aty";
#elif L==7	// it
	"File";
#elif L==8	// pl
	"Plik";
#elif L==9	// pt
	"Arquivo";
#elif L==10	// tr
	"File";	// eviri lazim!
#endif

const char *Txt_file =
#if   L==1	// ca
	"arxiu";
#elif L==2	// de
	"Datei";
#elif L==3	// en
	"file";
#elif L==4	// es
	"archivo";
#elif L==5	// fr
	"fichier";
#elif L==6	// gn
	"jehai'aty";
#elif L==7	// it
	"file";
#elif L==8	// pl
	"Plik";
#elif L==9	// pt
	"arquivo";
#elif L==10	// tr
	"file";	// eviri lazim!
#endif

const char *Txt_X_file =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Arxiu %s";
#elif L==2	// de
	"%s-Datei";
#elif L==3	// en
	"%s file";
#elif L==4	// es
	"Archivo %s";
#elif L==5	// fr
	"Fichier %s";
#elif L==6	// gn
	"Archivo %s";	// Okoteve traduccin
#elif L==7	// it
	"File %s";
#elif L==8	// pl
	"Plik %s";
#elif L==9	// pt
	"Arquivo %s";
#elif L==10	// tr
	"%s file";	// eviri lazim!
#endif

const char *Txt_File_not_found =
#if   L==1	// ca
	"Arxiu no trobat";
#elif L==2	// de
	"Datei nicht gefunden";
#elif L==3	// en
	"File not found";
#elif L==4	// es
	"Archivo no encontrado";
#elif L==5	// fr
	"Fichier non trouv&eacute;";
#elif L==6	// gn
	"Archivo no encontrado";	// Okoteve traduccin
#elif L==7	// it
	"File non trovato";
#elif L==8	// pl
	"Nie znaleziono pliku";
#elif L==9	// pt
	"Arquivo n&atilde;o encontrado";
#elif L==10	// tr
	"File not found";		// eviri lazim!
#endif

const char *Txt_FILE_uncompressed =
#if   L==1	// ca
	"sense comprimir";
#elif L==2	// de
	"unkomprimierte";
#elif L==3	// en
	"uncrompressed";
#elif L==4	// es
	"sin comprimir";
#elif L==5	// fr
	"non compress&eacute;";
#elif L==6	// gn
	"sin comprimir";	// Okoteve traduccin
#elif L==7	// it
	"non compresso";
#elif L==8	// pl
	"nieskompresowane";
#elif L==9	// pt
	"descompactado";
#elif L==10	// tr
	"uncrompressed";	// eviri lazim!
#endif

const char *Txt_Filename =
#if   L==1	// ca
	"Nom del arxiu";
#elif L==2	// de
	"Dateiname";
#elif L==3	// en
	"Filename";
#elif L==4	// es
	"Nombre del archivo";
#elif L==5	// fr
	"Nom du fichier";
#elif L==6	// gn
	"Nombre del archivo";	// Okoteve traduccin
#elif L==7	// it
	"Nome del file";
#elif L==8	// pl
	"Nazwa";
#elif L==9	// pt
	"Nome do arquivo";
#elif L==10	// tr
	"Filename";		// eviri lazim!
#endif

const char *Txt_file_folder =
#if   L==1	// ca
	"arxiu/carpeta";
#elif L==2	// de
	"Datei/Verzeichnis";
#elif L==3	// en
	"file/folder";
#elif L==4	// es
	"archivo/carpeta";
#elif L==5	// fr
	"fichier/r&eacute;pertoire";
#elif L==6	// gn
	"archivo/carpeta";	// Okoteve traduccin
#elif L==7	// it
	"file/cartella";
#elif L==8	// pl
	"Plik/folder";
#elif L==9	// pt
	"arquivo/diret&oacute;rio";
#elif L==10	// tr
	"file/folder";		// eviri lazim!
#endif

const char *Txt_FILE_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> eliminat.";
#elif L==2	// de
	"<strong>%s</strong> entfernt.";
#elif L==3	// en
	"<strong>%s</strong> removed.";
#elif L==4	// es
	"<strong>%s</strong> eliminado.";
#elif L==5	// fr
	"<strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"<strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> rimosso.";
#elif L==8	// pl
	"<strong>%s</strong> usuniety.";
#elif L==9	// pt
	"<strong>%s</strong> removido.";
#elif L==10	// tr
	"<strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_File_size =
#if   L==1	// ca
	"Mida del fitxer";
#elif L==2	// de
	"Dateigr&ouml;&szlig;e";
#elif L==3	// en
	"File size";
#elif L==4	// es
	"Tama&ntilde;o del archivo";
#elif L==5	// fr
	"Taille du fichier";
#elif L==6	// gn
	"Tama&ntilde;o del archivo";	// Okoteve traduccin
#elif L==7	// it
	"Dimensioni del file";
#elif L==8	// pl
	"Rozmiar pliku";
#elif L==9	// pt
	"Tamanho do arquivo";
#elif L==10	// tr
	"File size";			// eviri lazim!
#endif

const char *Txt_File_with_the_logo =
#if   L==1	// ca
	"Fitxer amb el logo";
#elif L==2	// de
	"Datei mit dem Logo";
#elif L==3	// en
	"File with the logo";
#elif L==4	// es
	"Archivo con el logo";
#elif L==5	// fr
	"Fichier avec le logo";
#elif L==6	// gn
	"Archivo con el logo";	// Okoteve traduccin
#elif L==7	// it
	"File con il logo";
#elif L==8	// pl
	"Plik ze logo";
#elif L==9	// pt
	"Arquivo com o logotipo";
#elif L==10	// tr
	"File with the logo";	// eviri lazim!
#endif

const char *Txt_File_with_the_photo =
#if   L==1	// ca
	"Fitxer amb la foto";
#elif L==2	// de
	"Datei mit dem Foto";
#elif L==3	// en
	"File with the photo";
#elif L==4	// es
	"Archivo con la foto";
#elif L==5	// fr
	"Fichier avec la photo";
#elif L==6	// gn
	"Archivo orek&oacute;va ta'anga";
#elif L==7	// it
	"File con la foto";
#elif L==8	// pl
	"Plik ze zdjeciem";
#elif L==9	// pt
	"Arquivo com a foto";
#elif L==10	// tr
	"Foto&gbreve;rafl&inodot; dosya";
#endif

const char *Txt_File_zone =
#if   L==1	// ca
	"Zona d'arxius";
#elif L==2	// de
	"Dateienbereich";
#elif L==3	// en
	"File zone";
#elif L==4	// es
	"Zona de archivos";
#elif L==5	// fr
	"Zone de fichiers";
#elif L==6	// gn
	"Zona de archivos";	// Okoteve traduccin
#elif L==7	// it
	"Area file";
#elif L==8	// pl
	"Strefa plik&oacute;w";
#elif L==9	// pt
	"Zona de arquivos";
#elif L==10	// tr
	"File zone";		// eviri lazim!
#endif

const char *Txt_File_zones =
#if   L==1	// ca
	"Zones d'arxius";
#elif L==2	// de
	"Dateienbereiche";
#elif L==3	// en
	"File zones";
#elif L==4	// es
	"Zonas de archivos";
#elif L==5	// fr
	"Zones de fichiers";
#elif L==6	// gn
	"Zonas de archivos";	// Okoteve traduccin
#elif L==7	// it
	"Area file";
#elif L==8	// pl
	"Plik obszar&oacute;w";
#elif L==9	// pt
	"Zonas de arquivos";
#elif L==10	// tr
	"File zones";		// eviri lazim!
#endif

const char *Txt_File_zones_disabled =
#if   L==1	// ca
	"Zones d'arxius inhabilitades";
#elif L==2	// de
	"Dateizonen deaktiviert";
#elif L==3	// en
	"File zones disabled";
#elif L==4	// es
	"Zonas de archivos inhabilitadas";
#elif L==5	// fr
	"Zones de fichiers d&eacute;sactiv&eacute;es";
#elif L==6	// gn
	"Zonas de archivos inhabilitadas";	// Okoteve traduccin
#elif L==7	// it
	"Area file disabilitata";
#elif L==8	// pl
	"Plik strefy niedostepnej";
#elif L==9	// pt
	"Zonas de arquivos desativadas";
#elif L==10	// tr
	"File zones disabled";			// eviri lazim!
#endif

const char *Txt_File_zones_of_the_group_X_are_now_disabled =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Les zones d'arxius del grup %s ara estan desactivades.";
#elif L==2	// de
	"Dateizonen der Gruppe %s sind jetzt deaktiviert.";
#elif L==3	// en
	"File zones of the group %s are now disabled.";
#elif L==4	// es
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";
#elif L==5	// fr
	"Les zones de fichiers du groupe %s sont maintenant d&eacute;sactiv&eacute;es.";
#elif L==6	// gn
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";	// Okoteve traduccin
#elif L==7	// it
	"Le aree file del gruppo %s sono ora disabilitate.";
#elif L==8	// pl
	"Plik strefy grupy %s jest teraz niedostepny.";
#elif L==9	// pt
	"As zonas de arquivos do grupo %s est&atilde;o agora desativadas.";
#elif L==10	// tr
	"File zones of the group %s are now disabled.";				// eviri lazim!
#endif

const char *Txt_File_zones_of_the_group_X_are_now_enabled =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Les zones d'arxius del grup %s ara estan habilitades.";
#elif L==2	// de
	"Dateizonen der Gruppe %s sind jetzt aktiviert.";
#elif L==3	// en
	"File zones of the group %s are now enabled.";
#elif L==4	// es
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";
#elif L==5	// fr
	"Les zones de fichiers du groupe %s sont maintenant activ&eacute;es.";
#elif L==6	// gn
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";	// Okoteve traduccin
#elif L==7	// it
	"Le aree file del gruppo %s sono ora abilitate.";
#elif L==8	// pl
	"Plik strefy grupa %s sa obecnie dost&eogon;pne.";
#elif L==9	// pt
	"As zonas de arquivos do grupo %s est&atilde;o agora ativadas.";
#elif L==10	// tr
	"File zones of the group %s are now enabled.";				// eviri lazim!
#endif

const char *Txt_Files =
#if   L==1	// ca
	"Fitxers";
#elif L==2	// de
	"Dateien";
#elif L==3	// en
	"Files";
#elif L==4	// es
	"Archivos";
#elif L==5	// fr
	"Fichiers";
#elif L==6	// gn
	"Archivos";	// Okoteve traduccin
#elif L==7	// it
	"File";
#elif L==8	// pl
	"Pliki";
#elif L==9	// pt
	"Arquivos";
#elif L==10	// tr
	"Files";	// eviri lazim!
#endif

const char *Txt_files =
#if   L==1	// ca
	"arxius";
#elif L==2	// de
	"Dateien";
#elif L==3	// en
	"files";
#elif L==4	// es
	"archivos";
#elif L==5	// fr
	"fichiers";
#elif L==6	// gn
	"archivos";	// Okoteve traduccin
#elif L==7	// it
	"file";
#elif L==8	// pl
	"pliki";
#elif L==9	// pt
	"arquivos";
#elif L==10	// tr
	"files";	// eviri lazim!
#endif

const char *Txt_Files_assignments =
#if   L==1	// ca
	"Arxius activitats";
#elif L==2	// de
	"Dateien der Aufgaben";
#elif L==3	// en
	"Files assignments";
#elif L==4	// es
	"Archivos actividades";
#elif L==5	// fr
	"Fichiers activites";
#elif L==6	// gn
	"Archivos actividades";	// Okoteve traduccin
#elif L==7	// it
	"File attivit&agrave";
#elif L==8	// pl
	"Pliki zadan";
#elif L==9	// pt
	"Arquivos atividades";
#elif L==10	// tr
	"Files assignments";	// eviri lazim!
#endif

const char *Txt_Files_copied =
#if   L==1	// ca
	"Arxius copiats";
#elif L==2	// de
	"Dateien kopiert";
#elif L==3	// en
	"Files copied";
#elif L==4	// es
	"Archivos copiados";
#elif L==5	// fr
	"Fichiers copi&eacute;s";
#elif L==6	// gn
	"Archivos copiados";	// Okoteve traduccin
#elif L==7	// it
	"File copiati";
#elif L==8	// pl
	"Pliki kopiowane";
#elif L==9	// pt
	"Arquivos copiados";
#elif L==10	// tr
	"Files copied";		// eviri lazim!
#endif

const char *Txt_Files_uploaded =
#if   L==1	// ca
	"Fitxers pujats";
#elif L==2	// de
	"Dateien hochgeladen";
#elif L==3	// en
	"Files uploaded";
#elif L==4	// es
	"Archivos subidos";
#elif L==5	// fr
	"Fichiers t&eacute;l&eacute;charg&eacute;s";
#elif L==6	// gn
	"Archivos subidos";	// Okoteve traduccin
#elif L==7	// it
	"File caricati";
#elif L==8	// pl
	"Pliki dodane";
#elif L==9	// pt
	"Arquivos enviados";
#elif L==10	// tr
	"Files uploaded";	// eviri lazim!
#endif

const char *Txt_Files_of_marks_must_contain_a_table_in_HTML_format_ =
#if   L==1	// ca
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Necessita traducci
#elif L==2	// de
	"Bewertungsdateien mssen eine Tabelle im <strong>HTML-Format</strong> ohne Bilder enthalten."
	" Die HTML-Dateien k&ouml;nnen z.B. aus einer Kalkulationstabelle erstellt werden,"
	" indem der gewnschte Bereich aus der Tabelle ausgew&auml;hlt und als HTML gespeichert wird.<br />"
	" Eine Tabelle kann <strong>mehrere Kopf- und Fuzeilen enthalten</strong>."
	" Zwischen solchen Zeilen muss eine <strong>einzige Zeile f&uuml; jeden Studenten bestehen</strong>,"
	" mit einer beliebigen Anzahl von Spalten, <strong>wovon die erste die Ausweis-Nr. enthalten muss</strong>.<br />"
	" Ruft ein Student die Tabelle auf, sieht er nur die f&uuml;r ihn bestimmte Zeile.";
#elif L==3	// en
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with his/her marks,"
	" and footer rows.";
#elif L==4	// es
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";
#elif L==5	// fr
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with his/her marks,"
	" and footer rows.";				// Besoin de traduction
#elif L==6	// gn
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Okoteve traduccin
#elif L==7	// it
	"I file dei risultati devono apparire in una tabella in <strong>formato HTML</strong> senza immagini."
	" Ogni file HTML pu&ograve; essere generato, per esempio, da un foglio di calcolo elettronico,"
	" selezionando la desiderata porzione di tabella e salvandolo come una pagina web.<br />"
	" Una tabella pu&ograve; avere <strong>intestazioni multiple e righe a pi&egrave; di pagina</strong>."
	" Tra di loro dovrebbe esserci <strong>una singola riga per ogni studente</strong>,"
	" con alcuni numeri di colonne, <strong>la prima colonna contiene l'ID dello studente</strong>.<br />"
	" Uno studente vedr&agrave; una tabella con righe intestate,"
	" seguite da una riga con i loro risultati,"
	" e righe a pi&egrave; di pagina.";
#elif L==8	// pl
	"Pliki znak&oacute;w musza zawierac tabele <strong>w formacie HTML</strong> bez zdjecia."
	" Kazdy plik HTML moze zostac wygenerowany, na przyklad z arkusza kalkulacyjnego,"
	" wybierajac odpowiednia czesc tabeli i zapisanie go jako strone sieci Web.<br />"
	" Tabela moze miec <strong>wielu wiersze nagl&oacute;wka i stopki</strong>."
	" Pomiedzy nimi nalezy <strong>jednym wierszu dla kazdego ucznia</strong>,"
	" z dowolna iloscia kolumn, <strong>pierwszej kolumnie zawierajacej studenta ID</strong>.<br />"
	" Student zobaczy tabele z wierszy nagl&oacute;wka, a nastepnie przez rzad z ich znak&oacute;w towarowych i stopki.";
#elif L==9	// pt
	"Os arquivos de notas devem conter uma tabela no <strong>formato HTML</strong> sem imagens."
	" Cada arquivo HTML pode ser gerado, por exemplo, a partir de uma planilha eletr&ocirc;nica,"
	" selecionando a parte desejada da tabela e salvando-a como p&aacute;gina web.<br />"
	" Uma tabela pode ter <strong>v&aacute;rias linhas de cabe&ccedil;alho e rodap&eacute;</strong>."
	" Entre eles deve haver <strong>uma &uacute;nica linha para cada estudante</strong>,"
	" com qualquer n&uacute;mero de colunas, <strong>a primeira coluna contendo o ID do estudante</strong>.<br />"
	" Um estudante ver&aacute; uma tabela com linhas de cabe&ccedil;alho,"
	" seguida pela linha com suas notas,"
	" e linhas de rodap&eacute;.";
#elif L==10	// tr
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with his/her marks,"
	" and footer rows.";				// eviri lazim!
#endif

const char *Txt_Files_removed =
#if   L==1	// ca
	"Arxius eliminats";
#elif L==2	// de
	"Dateien entfernt";
#elif L==3	// en
	"Files removed";
#elif L==4	// es
	"Archivos eliminados";
#elif L==5	// fr
	"Fichiers supprim&eacute;s";
#elif L==6	// gn
	"Archivos eliminados";	// Okoteve traduccin
#elif L==7	// it
	"File rimossi";
#elif L==8	// pl
	"Pliki usuni&eogon;te";
#elif L==9	// pt
	"Arquivos removidos";
#elif L==10	// tr
	"Files removed";	// eviri lazim!
#endif

const char *Txt_Files_works =
#if   L==1	// ca
	"Arxius treballs";
#elif L==2	// de
	"Dateien Hausarbeiten";
#elif L==3	// en
	"Files homework";
#elif L==4	// es
	"Archivos trabajos";
#elif L==5	// fr
	"Fichiers travaux";
#elif L==6	// gn
	"Archivos trabajos";	// Okoteve traduccin
#elif L==7	// it
	"File lavori";
#elif L==8	// pl
	"Pliki domowe";
#elif L==9	// pt
	"Arquivos trabalhos";
#elif L==10	// tr
	"Files homework";	// eviri lazim!
#endif

const char *Txt_Filter =	// As a noun
#if   L==1	// ca
	"Filtre";
#elif L==2	// de
	"Filter";
#elif L==3	// en
	"Filter";
#elif L==4	// es
	"Filtro";
#elif L==5	// fr
	"Filtre";
#elif L==6	// gn
	"Ayga";
#elif L==7	// it
	"Filtro";
#elif L==8	// pl
	"Filtr";
#elif L==9	// pt
	"Filtro";
#elif L==10	// tr
	"Filter";	// eviri lazim!
#endif

const char *Txt_First_day_of_the_week_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Primer dia de la setmana: %s";
#elif L==2	// de
	"Erster Tag der Woche: %s";
#elif L==3	// en
	"First day of the week: %s";
#elif L==4	// es
	"Primer d&iacute;a de la semana: %s";
#elif L==5	// fr
	"Premier jour de la semaine: %s";
#elif L==6	// gn
	"Primer d&iacute;a de la semana: %s";	// Okoteve traduccin
#elif L==7	// it
	"Primo giorno della settimana: %s";
#elif L==8	// pl
	"Pierwszy dzie&nacute; tygodnia: %s";
#elif L==9	// pt
	"Primeiro dia da semana: %s";
#elif L==10	// tr
	"Haftan&inodot;n ilk g&uuml;n&uuml;: %s";
#endif

const char *Txt_First_name =
#if   L==1	// ca
	"Nom";
#elif L==2	// de
	"Vorname";
#elif L==3	// en
	"First name";
#elif L==4	// es
	"Nombre";
#elif L==5	// fr
	"Pr&eacute;nom";
#elif L==6	// gn
	"T&eacute;ra";
#elif L==7	// it
	"Primo nome";
#elif L==8	// pl
	"Imi&eogon;";
#elif L==9	// pt
	"Nome";
#elif L==10	// tr
	"First name";	// eviri lazim!
#endif

const char *Txt_Floor =
#if   L==1	// ca
	"Planta";
#elif L==2	// de
	"Stock";
#elif L==3	// en
	"Floor";
#elif L==4	// es
	"Planta";
#elif L==5	// fr
	"&Eacute;tage";
#elif L==6	// gn
	"Planta";	// Okoteve traduccin
#elif L==7	// it
	"Piano";
#elif L==8	// pl
	"Pi&eogon;trze";
#elif L==9	// pt
	"Andar";
#elif L==10	// tr
	"Floor";	// eviri lazim!
#endif

const char *Txt_Folder =
#if   L==1	// ca
	"Carpeta";
#elif L==2	// de
	"Verzeichnis";
#elif L==3	// en
	"Folder";
#elif L==4	// es
	"Carpeta";
#elif L==5	// fr
	"R&eacute;pertoire";
#elif L==6	// gn
	"Carpeta";	// Okoteve traduccin
#elif L==7	// it
	"Cartella";
#elif L==8	// pl
	"Folder";
#elif L==9	// pt
	"Diret&oacute;rio";
#elif L==10	// tr
	"Folder";	// eviri lazim!
#endif

const char *Txt_folder =
#if   L==1	// ca
	"carpeta";
#elif L==2	// de
	"Verzeichnis";
#elif L==3	// en
	"folder";
#elif L==4	// es
	"carpeta";
#elif L==5	// fr
	"r&eacute;pertoire";
#elif L==6	// gn
	"carpeta";
#elif L==7	// it
	"cartella";
#elif L==8	// pl
	"folder";
#elif L==9	// pt
	"diret&oacute;rio";
#elif L==10	// tr
	"dosya";
#endif

const char *Txt_Folder_X_and_all_its_contents_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Carpeta <strong>%s</strong> i tot el contingut eliminats.";
#elif L==2	// de
	"Verzeichnis <strong>%s</strong> und alle Inhalte wurden entfernt.";
#elif L==3	// en
	"Folder <strong>%s</strong> and all its contents removed.";
#elif L==4	// es
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";
#elif L==5	// fr
	"R&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu supprim&eacute;s.";
#elif L==6	// gn
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";	// Okoteve traduccin
#elif L==7	// it
	"Cartella <strong>%s</strong> e tutto il suo contenuto rimossi.";
#elif L==8	// pl
	"Folder <strong>%s</strong> i cala jego zawartosc usunieta.";
#elif L==9	// pt
	"Diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do removidos.";
#elif L==10	// tr
	"Folder <strong>%s</strong> and all its contents removed.";	// eviri lazim!
#endif

const char *Txt_Folder_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Carpeta <strong>%s</strong> suprimida.";
#elif L==2	// de
	"Verzeichnis <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Folder <strong>%s</strong> removed.";
#elif L==4	// es
	"Carpeta <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"R&eacute;pertoire <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Carpeta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Cartella <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Folder <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Diret&oacute;rio <strong>%s</strong> removido.";
#elif L==10	// tr
	"Folder <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Folder_to_upload_files =
#if   L==1	// ca
	"Carpeta per pujar arxius?";
#elif L==2	// de
	"Ordner zum Hochladen von Dateien?";
#elif L==3	// en
	"Folder to upload files?";
#elif L==4	// es
	"&iquest;Carpeta para subir archivos?";
#elif L==5	// fr
	"Dossier pour t%eacute;l&eacute;charger des fichiers?";
#elif L==6	// gn
	"Carpeta ojegueraha hag&gtilde;ua archivo-ku&eacute;ra?";
#elif L==7	// it
	"Cartella per caricare i file?";
#elif L==8	// pl
	"Folder do przes&lstrok;ania plik&oacute;w?";
#elif L==9	// pt
	"Pasta para fazer upload de arquivos?";
#elif L==10	// tr
	"Dosyalar&inodot;n y&uuml;klenece&gbreve;i klas&ouml;r?";
#endif

const char *Txt_Folders =
#if   L==1	// ca
	"Carpetes";
#elif L==2	// de
	"Verzeichnisse";
#elif L==3	// en
	"Folders";
#elif L==4	// es
	"Carpetas";
#elif L==5	// fr
	"R&eacute;pertoires";
#elif L==6	// gn
	"Umi carpeta";
#elif L==7	// it
	"Cartelle";
#elif L==8	// pl
	"Foldery";
#elif L==9	// pt
	"Diret&oacute;rios";
#elif L==10	// tr
	"Klas&ouml;rler";
#endif

const char *Txt_folders =
#if   L==1	// ca
	"carpetes";
#elif L==2	// de
	"Verzeichnisse";
#elif L==3	// en
	"folders";
#elif L==4	// es
	"carpetas";
#elif L==5	// fr
	"r&eacute;pertoires";
#elif L==6	// gn
	"umi carpeta";
#elif L==7	// it
	"cartelle";
#elif L==8	// pl
	"foldery";
#elif L==9	// pt
	"diret&oacute;rios";
#elif L==10	// tr
	"klas&ouml;rler";
#endif

const char *Txt_Folders_copied =
#if   L==1	// ca
	"Carpetes copiades";
#elif L==2	// de
	"Verzeichnisse kopiert";
#elif L==3	// en
	"Folders copied";
#elif L==4	// es
	"Carpetas copiadas";
#elif L==5	// fr
	"R&eacute;pertoires copi&eacute;s";
#elif L==6	// gn
	"Carpetas copiadas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle copiate";
#elif L==8	// pl
	"foldery skopiowane";
#elif L==9	// pt
	"Diret&oacute;rios copiados";
#elif L==10	// tr
	"Folders copied";	// eviri lazim!
#endif

const char *Txt_Folders_renamed =
#if   L==1	// ca
	"Carpetes anomenades";
#elif L==2	// de
	"Verzeichnisse umbenannt";
#elif L==3	// en
	"Folders renamed";
#elif L==4	// es
	"Carpetas renombradas";
#elif L==5	// fr
	"R&eacute;pertoires renomm&eacute;s";
#elif L==6	// gn
	"Carpetas renombradas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle rinominate";
#elif L==8	// pl
	"Foldery zmienione";
#elif L==9	// pt
	"Diret&oacute;rios renomeados";
#elif L==10	// tr
	"Folders renamed";	// eviri lazim!
#endif

const char *Txt_Folders_removed =
#if   L==1	// ca
	"Carpetes eliminades";
#elif L==2	// de
	"Verzeichnisse entfernt";
#elif L==3	// en
	"Folders removed";
#elif L==4	// es
	"Carpetas eliminadas";
#elif L==5	// fr
	"R&eacute;pertoires supprim&eacute;s";
#elif L==6	// gn
	"Carpetas eliminadas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle rimossi";
#elif L==8	// pl
	"Foldery usuni&eogon;te";
#elif L==9	// pt
	"Diret&oacute;rios removidos";
#elif L==10	// tr
	"Folders removed";	// eviri lazim!
#endif

const char *Txt_Folders_not_renamed =
#if   L==1	// ca
	"Carpetes no anomenades";
#elif L==2	// de
	"Verzeichnisse nicht umbenannt";
#elif L==3	// en
	"Folders not renamed";
#elif L==4	// es
	"Carpetas no renombradas";
#elif L==5	// fr
	"R&eacute;pertoires pas renomm&eacute;s";
#elif L==6	// gn
	"Carpetas no renombradas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle non rinominate";
#elif L==8	// pl
	"Foldery nie zmienione";
#elif L==9	// pt
	"Diret&oacute;rios n&atilde;o renomeados";
#elif L==10	// tr
	"Folders not renamed";		// eviri lazim!
#endif

const char *Txt_Follow =
#if   L==1	// ca
	"Seguir";
#elif L==2	// de
	"Folgen";
#elif L==3	// en
	"Follow";
#elif L==4	// es
	"Seguir";
#elif L==5	// fr
	"Suivre";
#elif L==6	// gn
	"Seguir";	// Okoteve traduccin
#elif L==7	// it
	"Seguire";
#elif L==8	// pl
	"&Sacute;led&zacute;";
#elif L==9	// pt
	"Seguir";
#elif L==10	// tr
	"Follow";	// eviri lazim!
#endif

const char *Txt_FollowPerFollow[2] =
   	{
#if   L==1	// ca
	"Mitjana de seguits per cada seguidor"
#elif L==2	// de
	"Durchschnittliche gefolgt pro anh&auml;nger"
#elif L==3	// en
	"Average followed per follower"
#elif L==4	// es
	"Promedio de seguidos por cada seguidor"
#elif L==5	// fr
	"Moyenne de suivis par suiveur"
#elif L==6	// gn
	"Promedio de seguidos por cada seguidor"	// Okoteve traduccin
#elif L==7	// it
	"Media di seguiti per seguace"
#elif L==8	// pl
        "&Sacute;rednia liczba obserwowani za obserwuj&aogon;"
#elif L==9	// pt
	"M&eacute;dia por seguidos por seguidor"
#elif L==10	// tr
	"Average followed per follower"			// eviri lazim!
#endif
      ,
#if   L==1	// ca
	"Mitjana de seguidors per cada seguit"
#elif L==2	// de
	"Durchschnittliche anh&auml;nger pro gefolgt"
#elif L==3	// en
	"Average followers per followed"
#elif L==4	// es
	"Promedio de seguidores por cada seguido"
#elif L==5	// fr
	"Moyenne de suiveurs par suivie"
#elif L==6	// gn
	"Promedio de seguidores por cada seguido"	// Okoteve traduccin
#elif L==7	// it
	"Media di seguace per seguito"
#elif L==8	// pl
        "&Sacute;rednia liczba obserwuj&aogon; za obserwowani"
#elif L==9	// pt
	"M&eacute;dia por seguidores por seguido"
#elif L==10	// tr
	"Average followers per followed"		// eviri lazim!
#endif
      };

const char *Txt_Followed =
#if   L==1	// ca
	"Seguits";
#elif L==2	// de
	"Gefolgt";
#elif L==3	// en
	"Followed";
#elif L==4	// es
	"Seguidos";
#elif L==5	// fr
	"Suivis";
#elif L==6	// gn
	"Seguidos";	// Okoteve traduccin
#elif L==7	// it
	"Seguiti";
#elif L==8	// pl
	"Obserwowani";
#elif L==9	// pt
	"Seguidos";
#elif L==10	// tr
	"Followed";	// eviri lazim!
#endif

const char *Txt_Followers =
#if   L==1	// ca
	"Seguidors";
#elif L==2	// de
	"Anh&auml;nger";
#elif L==3	// en
	"Followers";
#elif L==4	// es
	"Seguidores";
#elif L==5	// fr
	"Suiveurs";
#elif L==6	// gn
	"Seguidores";	// Okoteve traduccin
#elif L==7	// it
	"Seguaci";
#elif L==8	// pl
	"Obserwuj&aogon;";
#elif L==9	// pt
	"Seguidores";
#elif L==10	// tr
	"Followers";	// eviri lazim!
#endif

const char *Txt_Following =
#if   L==1	// ca
	"Seg&uuml;ent";
#elif L==2	// de
	"Folgende";
#elif L==3	// en
	"Following";
#elif L==4	// es
	"Siguiendo";
#elif L==5	// fr
	"Suivant";
#elif L==6	// gn
	"Siguiendo";	// Okoteve traduccin
#elif L==7	// it
	"Seguendo";
#elif L==8	// pl
	"Nast&eogon;puj&aogon;cy";
#elif L==9	// pt
	"Seguinte";
#elif L==10	// tr
	"Following";	// eviri lazim!
#endif

const char *Txt_Following_unfollow =
#if   L==1	// ca
	"Seg&uuml;ent, deixar de seguir";
#elif L==2	// de
	"Folgende, entfolgen";
#elif L==3	// en
	"Following, unfollow";
#elif L==4	// es
	"Siguiendo, dejar de seguir";
#elif L==5	// fr
	"Suivant, se d&eacute;sabonner";
#elif L==6	// gn
	"Siguiendo, dejar de seguir";	// Okoteve traduccin
#elif L==7	// it
	"Seguendo, smetti";
#elif L==8	// pl
	"Nast&eogon;puj&aogon;cy, nie obserwuj";
#elif L==9	// pt
	"Seguinte, deixar de seguir";
#elif L==10	// tr
	"Following, unfollow";		// eviri lazim!
#endif

const char *Txt_FOLLOWS_YOU =
#if   L==1	// ca
	"ET SEGUEIX";
#elif L==2	// de
	"FOLGT DIR";
#elif L==3	// en
	"FOLLOWS YOU";
#elif L==4	// es
	"TE SIGUE";
#elif L==5	// fr
	"VOUS SUIT";
#elif L==6	// gn
	"TE SIGUE";	// Okoteve traduccin
#elif L==7	// it
	"TI SEGUE";
#elif L==8	// pl
	"OBSERWUJE CI&Eogon;";
#elif L==9	// pt
	"SEGUE VOC&Ecirc;";
#elif L==10	// tr
	"FOLLOWS YOU";	// eviri lazim!
#endif

const char *Txt_For_security_enter_your_password =
#if   L==1	// ca
	"Per a m&eacute;s seguretat, introdueixi la seva contrasenya";
#elif L==2	// de
	"Aus Sicherheitsgr&uuml;nden geben Sie Ihr Passwort";
#elif L==3	// en
	"For security, enter your password";
#elif L==4	// es
	"Para mayor seguridad, introduzca su contrase&ntilde;a";
#elif L==5	// fr
	"Pour plus de s&eacute;curit&eacute;, entrez votre mot de passe";
#elif L==6	// gn
	"Para mayor seguridad, introduzca su contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Per motivi di sicurezza, immettere la password";
#elif L==8	// pl
	"Dla bezpiecze&nacute;stwa, wprowad&zacute; has&lstrok;o";
#elif L==9	// pt
	"Por seguran&ccedil;a, digite a sua senha";
#elif L==10	// tr
	"For security, enter your password";				// eviri lazim!
#endif

const char *Txt_Force_students_to_read_this_information =
#if   L==1	// ca
	"Obligar els estudiants a llegir aquesta informaci&oacute;";
#elif L==2	// de
	"Erzwingen Studenten lesen diese Informationen";
#elif L==3	// en
	"Force students to read this information";
#elif L==4	// es
	"Obligar a los estudiantes a leer esta informaci&oacute;n";
#elif L==5	// fr
	"Forcer les &eacute;tudiants &agrave; lire cette information";
#elif L==6	// gn
	"Obligar a los estudiantes a leer esta informaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Obbligare gli studenti a leggere questa informazione";
#elif L==8	// pl
	"Studenci maja przeczytac ta informacje";
#elif L==9	// pt
	"For&ccedil;ar os estudantes a ler esta informa&ccedil;&atilde;o";
#elif L==10	// tr
	"&Ouml;&gbreve;rencileri bu bilgileri okumaya zorla";
#endif

const char *Txt_Forgotten_password =
#if   L==1	// ca
	"Contrasenya oblidada?";
#elif L==2	// de
	"Passwort vergessen?";
#elif L==3	// en
	"Forgotten password?";
#elif L==4	// es
	"&iquest;Contrasea olvidada?";
#elif L==5	// fr
	"Mot de passe oubli&eacute;?";
#elif L==6	// gn
	"&iquest;Contrasea olvidada?";		// Okoteve traduccin
#elif L==7	// it
	"Password dimenticata?";
#elif L==8	// pl
	"Zapomniane has&lstrok;o?";
#elif L==9	// pt
	"Senha esquecida?";
#elif L==10	// tr
	"Unutulan &scedil;ifre?";
#endif

const char *Txt_Format =
#if   L==1	// ca
	"Format";
#elif L==2	// de
	"Format";
#elif L==3	// en
	"Format";
#elif L==4	// es
	"Formato";
#elif L==5	// fr
	"Format";
#elif L==6	// gn
	"Formato";		// Okoteve traduccin
#elif L==7	// it
	"Formato";
#elif L==8	// pl
	"Format";
#elif L==9	// pt
	"Formato";
#elif L==10	// tr
	"Format";	// eviri lazim!
#endif

const char *Txt_Forum =
#if   L==1	// ca
	"F&ograve;rum";
#elif L==2	// de
	"Forum";
#elif L==3	// en
	"Forum";
#elif L==4	// es
	"Foro";
#elif L==5	// fr
	"Forum";
#elif L==6	// gn
	"Foro";
#elif L==7	// it
	"Forum";
#elif L==8	// pl
	"Forum";
#elif L==9	// pt
	"F&oacute;rum";
#elif L==10	// tr
	"Forum";
#endif

const char *Txt_Forum_posts =
#if   L==1	// ca
	"Comentaris en f&ograve;rums";
#elif L==2	// de
	"Forum-Beitr&auml;ge";
#elif L==3	// en
	"Forum posts";
#elif L==4	// es
	"Comentarios en foros";
#elif L==5	// fr
	"Posts de forum";
#elif L==6	// gn
	"Comentarios en foros";	// Okoteve traduccin
#elif L==7	// it
	"Post del forum";
#elif L==8	// pl
	"Forum postow";
#elif L==9	// pt
	"Posts de f&oacute;rum";
#elif L==10	// tr
	"Forum posts";		// eviri lazim!
#endif

const char *Txt_FORUM_post =
#if   L==1	// ca
	"comentari";
#elif L==2	// de
	"Post";
#elif L==3	// en
	"post";
#elif L==4	// es
	"comentario";
#elif L==5	// fr
	"post";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"post";
#elif L==8	// pl
	"post";
#elif L==9	// pt
	"post";
#elif L==10	// tr
	"post";	// eviri lazim!
#endif

const char *Txt_FORUM_posts =
#if   L==1	// ca
	"comentaris";
#elif L==2	// de
	"Nachrichten";
#elif L==3	// en
	"posts";
#elif L==4	// es
	"comentarios";
#elif L==5	// fr
	"posts";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"post";
#elif L==8	// pl
	"posty";
#elif L==9	// pt
	"posts";
#elif L==10	// tr
	"posts";	// eviri lazim!
#endif

const char *Txt_FORUM_Post_X_allowed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Comentari %u perm&egrave;s.";
#elif L==2	// de
	"Beitrag %u erlaubt.";
#elif L==3	// en
	"Post %u allowed.";
#elif L==4	// es
	"Comentario %u permitido.";
#elif L==5	// fr
	"Poste %u autoris&eacute;.";
#elif L==6	// gn
	"Comentario %u permitido.";	// Okoteve traduccin
#elif L==7	// it
	"Post %u autorizzato.";
#elif L==8	// pl
	"Post %u dozwolone.";
#elif L==9	// pt
	"Post %u permitido.";
#elif L==10	// tr
	"Post %u allowed.";		// eviri lazim!
#endif

const char *Txt_FORUM_Post_and_thread_removed =
#if   L==1	// ca
	"Comentari i discussi&oacute; eliminats.";
#elif L==2	// de
	"Nachricht und Faden entfernt.";
#elif L==3	// en
	"Post and thread removed.";
#elif L==4	// es
	"Comentario y discusi&oacute;n eliminados.";
#elif L==5	// fr
	"Message et sujet supprim&eacute;s.";
#elif L==6	// gn
	"Comentario y discusi&oacute;n eliminados.";	// Okoteve traduccin
#elif L==7	// it
	"Post e discussione rimossi.";
#elif L==8	// pl
	"Post watku usuniety.";
#elif L==9	// pt
	"Post e discuss&atilde;o removidos.";
#elif L==10	// tr
	"Post and thread removed.";			// eviri lazim!
#endif

const char *Txt_FORUM_Post_banned =
#if   L==1	// ca
	"Comentari no perm&egrave;s";
#elif L==2	// de
	"Beitrag gesperrt.";
#elif L==3	// en
	"Post banned.";
#elif L==4	// es
	"Comentario no permitido";
#elif L==5	// fr
	"Message interdit.";
#elif L==6	// gn
	"Comentario no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Post censurato.";
#elif L==8	// pl
	"Post zakazany.";
#elif L==9	// pt
	"Post n&atilde;o permitido.";
#elif L==10	// tr
	"Post banned.";			// eviri lazim!
#endif

const char *Txt_FORUM_Post_X_banned =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Comentari %u no perm&egrave;s";
#elif L==2	// de
	"Kommentar %u keine Genehmigung";
#elif L==3	// en
	"Post %u banned.";
#elif L==4	// es
	"Comentario %u no permitido";
#elif L==5	// fr
	"Commentaire %u non autoris&eacute;";
#elif L==6	// gn
	"Comentario %u no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Post %u censurato.";
#elif L==8	// pl
	"Post %u zakazany.";
#elif L==9	// pt
	"Post %u n&atilde;o permitido.";
#elif L==10	// tr
	"Post %u banned.";		// eviri lazim!
#endif

const char *Txt_FORUM_Post_sent =
#if   L==1	// ca
	"Comentari enviat.";
#elif L==2	// de
	"Post verschickt.";
#elif L==3	// en
	"Post sent.";
#elif L==4	// es
	"Comentario enviado.";
#elif L==5	// fr
	"Message envoy&eacute;.";
#elif L==6	// gn
	"Comentario enviado.";	// Okoteve traduccin
#elif L==7	// it
	"Post inviato.";
#elif L==8	// pl
	"Post wyslany.";
#elif L==9	// pt
	"Post enviado.";
#elif L==10	// tr
	"Post sent.";		// eviri lazim!
#endif

const char *Txt_FORUM_Post_removed =
#if   L==1	// ca
	"Comentari eliminat.";
#elif L==2	// de
	"Nachricht entfernt.";
#elif L==3	// en
	"Post removed.";
#elif L==4	// es
	"Comentario eliminado.";
#elif L==5	// fr
	"Message supprim&eacute;.";
#elif L==6	// gn
	"Comentario eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Post rimosso.";
#elif L==8	// pl
	"Post usuniety.";
#elif L==9	// pt
	"Post removido.";
#elif L==10	// tr
	"Post removed.";		// eviri lazim!
#endif

const char *Txt_FORUM_Post_unbanned =
#if   L==1	// ca
	"Comentari perm&egrave;s";
#elif L==2	// de
	"Beitrag entbannt.";
#elif L==3	// en
	"Post unbanned";
#elif L==4	// es
	"Comentario permitido";
#elif L==5	// fr
	"Publication non bannie.";
#elif L==6	// gn
	"Comentario permitido";	// Okoteve traduccin
#elif L==7	// it
	"Post autorizzato.";
#elif L==8	// pl
	"Post odbanowany";
#elif L==9	// pt
	"Post permitido.";
#elif L==10	// tr
	"Post unbanned";	// eviri lazim!
#endif

const char *Txt_Forum_threads =
#if   L==1	// ca
	"Discussions de f&ograve;rum";
#elif L==2	// de
	"Forum-Threads";
#elif L==3	// en
	"Forum threads";
#elif L==4	// es
	"Discusiones de foro";
#elif L==5	// fr
	"Fils de forum";
#elif L==6	// gn
	"Discusiones de foro";	// Okoteve traduccin
#elif L==7	// it
	"Discussione del forum";
#elif L==8	// pl
	"Watki forum";
#elif L==9	// pt
	"Discuss&otilde;es de f&oacute;rum";
#elif L==10	// tr
	"Forum konular&inodot;";
#endif

const char *Txt_FORUM_THREAD_HELP_ORDER[Dat_NUM_START_END_TIME] =
	{
	[Dat_STR_TIME] =
#if   L==1	// ca
	"Ordenar per data del primer missatge"
#elif L==2	// de
	"Nach Datum der Nachrichten sortieren (aufsteigend)"
#elif L==3	// en
	"Sort by date of first message"
#elif L==4	// es
	"Ordenar por fecha del primer mensaje"
#elif L==5	// fr
	"Trier par date du premier message"
#elif L==6	// gn
	"Ordenar por fecha del primer mensaje"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data del primo messaggio"
#elif L==8	// pl
	"Sortuj wedlug daty pierwszej wiadomosci"
#elif L==9	// pt
	"Classificar por data da primeira mensagem"
#elif L==10	// tr
	"Sort by date of first message"		// eviri lazim!
#endif
	,
	[Dat_END_TIME] =
#if   L==1	// ca
	"Ordenar per data del darrer missatge"
#elif L==2	// de
	"Nach Datum der Nachrichten sortieren (absteigend)"
#elif L==3	// en
	"Sort by date of the last message"
#elif L==4	// es
	"Ordenar por fecha del &uacute;ltimo mensaje"
#elif L==5	// fr
	"Trier par date du dernier message"
#elif L==6	// gn
	"Ordenar por fecha del &uacute;ltimo mensaje"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data dell'ultimo messaggio"
#elif L==8	// pl
	"Sortuj wedlug daty ostatniego komunikatu"
#elif L==9	// pt
	"Classificar pela data da &uacute;ltima mensagem"
#elif L==10	// tr
	"Sort by date of the last message"		// eviri lazim!
#endif
	};

const char *Txt_FORUM_THREAD_ORDER[Dat_NUM_START_END_TIME] =
	{
	[Dat_STR_TIME] =
#if   L==1	// ca
	"Primer comentari"
#elif L==2	// de
	"Erste Post"
#elif L==3	// en
	"First post"
#elif L==4	// es
	"Primer comentario"
#elif L==5	// fr
	"Premier post"
#elif L==6	// gn
	"Primer comentario"	// Okoteve traduccin
#elif L==7	// it
	"Primo post"
#elif L==8	// pl
	"Pierwsza post"
#elif L==9	// pt
	"Primeira post"
#elif L==10	// tr
	"&Idot;lk posta"
#endif
	,
	[Dat_END_TIME] =
#if   L==1	// ca
	"&Uacute;ltim comentari"
#elif L==2	// de
	"Letzte Post"
#elif L==3	// en
	"Last post"
#elif L==4	// es
	"&Uacute;ltimo comentario"
#elif L==5	// fr
	"Dernier post"
#elif L==6	// gn
	"&Uacute;ltimo comentario"	// Okoteve traduccin
#elif L==7	// it
	"Ultimo post"
#elif L==8	// pl
	"Ostatnia post"
#elif L==9	// pt
	"&Uacute;ltima post"
#elif L==10	// tr
	"Son posta"
#endif
	};

const char *Txt_FORUM_WHICH_FORUM[For_NUM_FORUM_SETS] =
	{
	[For_ONLY_CURRENT_FORUMS] =
#if   L==1	// ca
	"F&ograve;rums d'aquest lloc"
#elif L==2	// de
	"Foren zu diesem Standort"
#elif L==3	// en
	"Forums of this location"
#elif L==4	// es
	"Foros de este sitio"
#elif L==5	// fr
	"Forums de cet emplacement"
#elif L==6	// gn
	"Foros de este sitio"		// Okoteve traduccin
#elif L==7	// it
	"Forum di questo sito"
#elif L==8	// pl
	"Forum tej lokalizacji"
#elif L==9	// pt
	"F&oacute;runs deste site"
#elif L==10	// tr
	"Forums of this location"	// eviri lazim!
#endif
	,
	[For_ALL_MY_FORUMS] =
#if   L==1	// ca
	"Tots els meus f&ograve;rums"
#elif L==2	// de
	"Alle meine Foren"
#elif L==3	// en
	"All my forums"
#elif L==4	// es
	"Todos mis foros"
#elif L==5	// fr
	"Tous mes forums"
#elif L==6	// gn
	"Todos mis foros"	// Okoteve traduccin
#elif L==7	// it
	"Tutti i miei forum"
#elif L==8	// pl
	"Wszystkie moje forum"
#elif L==9	// pt
	"Todos os meus f&oacute;runs"
#elif L==10	// tr
	"All my forums"		// eviri lazim!
#endif
	};

const char *Txt_Forums =
#if   L==1	// ca
	"F&ograve;rums";
#elif L==2	// de
	"Foren";
#elif L==3	// en
	"Forums";
#elif L==4	// es
	"Foros";
#elif L==5	// fr
	"Forums";
#elif L==6	// gn
	"Foros";	// Okoteve traduccin
#elif L==7	// it
	"Forum";
#elif L==8	// pl
	"Forum";
#elif L==9	// pt
	"F&oacute;runs";
#elif L==10	// tr
	"Forumlar";
#endif

const char *Txt_Found_an_index_html_file =
#if   L==1	// ca
	"S'ha trobat un arxiu <em>index.htm/index.html</em>.";
#elif L==2	// de
	"Eine <em>index.htm/index.html</em>-Datei gefunden.";
#elif L==3	// en
	"Found an <em>index.htm/index.html</em> file.";
#elif L==4	// es
	"Se ha encontrado un archivo <em>index.htm/index.html</em>.";
#elif L==5	// fr
	"Trouv&eacute; un fichier <em>index.htm/index.html</em>";
#elif L==6	// gn
	"Se ha encontrado un archivo <em>index.htm/index.html</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Si &egrave; trovato un file <em>index.htm/index.html</em>.";
#elif L==8	// pl
	"Znaleziono <em>index.htm/index.html</em> plik.";
#elif L==9	// pt
	"Um arquivo <em>index.htm/index.html</em> foi encontrado.";
#elif L==10	// tr
	"Found an <em>index.htm/index.html</em> file.";			// eviri lazim!
#endif

const char *Txt_Frequent_ACTIONS =	// Check that this sentence fits within the allocated space
#if   L==1	// ca
	"Freq&uuml;ents";
#elif L==2	// de
	"H&auml;ufige";
#elif L==3	// en
	"Frequent";
#elif L==4	// es
	"Frecuentes";
#elif L==5	// fr
	"Frequent";
#elif L==6	// gn
	"Jepigu&aacute;va";
#elif L==7	// it
	"Frequenti";
#elif L==8	// pl
	"Cz&eogon;ste";
#elif L==9	// pt
	"Freq&uuml;entes";
#elif L==10	// tr
	"S&inodot;k";
#endif

const char *Txt_from =
#if   L==1	// ca
	"de";	// Necessita traducci
#elif L==2	// de
	"von";
#elif L==3	// en
	"from";
#elif L==4	// es
	"de";
#elif L==5	// fr
	"de";
#elif L==6	// gn
	"de";	// Okoteve traduccin
#elif L==7	// it
	"da";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"de";
#elif L==10	// tr
	"from";	// eviri lazim!
#endif

const char *Txt_From_now_on_the_number_of_editing_lines_of_the_field_X_is_Y =	// Warning: it is very important to include %s and %u (in this order) in the following sentences
#if   L==1	// ca
	"A partir d'ara"
	" el nombre de l&iacute;nies d'edici&oacute; del camp <strong>%s</strong>"
	" &eacute;s <strong>%u</strong>.";
#elif L==2	// de
	"Von nun an"
	" ist die Anzahl der Bearbeitungszeilen des Feldes <strong>%s</strong>"
	" gleich <strong>%u</strong>.";
#elif L==3	// en
	"From now on"
	" the number of editing lines of the field <strong>%s</strong>"
	" is <strong>%u</strong>.";
#elif L==4	// es
	"A partir de ahora"
	" el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong>"
	" es <strong>%u</strong>.";
#elif L==5	// fr
	"D&eacute;sormais"
	" le nombre de lignes d'&eacute;dition du champ <strong>%s</strong>"
	" est <strong>%u</strong>.";
#elif L==6	// gn
	"A partir de ahora"
	" el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong>"
	" es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"A partire da ora"
	" il numero di linee editate <strong>%s</strong>"
	" &egrave; <strong>%u</strong>.";
#elif L==8	// pl
	"Od tej pory"
	" liczba linii w polu edycji <strong>%s</strong>"
	" jest <strong>%u</strong>.";
#elif L==9	// pt
	"A partir de agora"
	" o n&uacute;mero de linhas de edi&ccedil;&atilde;o no campo <strong>%s</strong>"
	" &eacute; <strong>%u</strong>.";
#elif L==10	// tr
	"From now on"
	" the number of editing lines of the field <strong>%s</strong>"
	" is <strong>%u</strong>.";	// eviri lazim!
#endif

const char *Txt_from_this_course =
#if   L==1	// ca
	"desde esta asignatura";	// Necessita traducci
#elif L==2	// de
	"von dieser Kurs";
#elif L==3	// en
	"from this course";
#elif L==4	// es
	"desde esta asignatura";
#elif L==5	// fr
	"de cette mati&egrave;re";
#elif L==6	// gn
	"desde esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"da questo corso";
#elif L==8	// pl
	"z tego kursu";
#elif L==9	// pt
	"desta disciplina";
#elif L==10	// tr
	"from this course";		// eviri lazim!
#endif

const char *Txt_From_this_time_you_can_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"A partir d'aquest moment, podeu rebre missatges de %s.";
#elif L==2	// de
	"Ab diesem Zeitpunkt k&ouml;nnen Sie Nachrichten von %s erhalten.";
#elif L==3	// en
	"From this time, you can receive messages from %s.";
#elif L==4	// es
	"A partir de este momento, puede recibir mensajes de %s.";
#elif L==5	// fr
	"&Agrave; partir de ce moment, vous pouvez recevoir des messages de %s.";
#elif L==6	// gn
	"A partir de este momento, puede recibir mensajes de %s.";	// Okoteve traduccin
#elif L==7	// it
	"Da questo momento, puoi ricevere messaggi da %s.";
#elif L==8	// pl
	"Od tej chwili mo&zdot;esz otrzymywa&cacute; wiadomo&sacute;ci od %s.";
#elif L==9	// pt
	"A partir deste momento, voc&ecirc; pode receber mensagens de %s.";
#elif L==10	// tr
	"From this time, you can receive messages from %s.";		// eviri lazim!
#endif

const char *Txt_From_this_time_you_will_not_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"A partir d'aquest moment, no rebr&agrave;s missatges de %s.";
#elif L==2	// de
	"Ab diesem Zeitpunkt erhalten Sie keine Nachrichten mehr von %s.";
#elif L==3	// en
	"From this time, you will not receive messages from %s.";
#elif L==4	// es
	"A partir de este momento, no recibir&aacute; mensajes de %s.";
#elif L==5	// fr
	"&Agrave; partir de ce moment, vous ne recevrez plus de messages de %s.";
#elif L==6	// gn
	"A partir de este momento, no recibir&aacute; mensajes de %s.";	// Okoteve traduccin
#elif L==7	// it
	"Da questo momento, non riceverai pi&ugrave; messaggi da %s.";
#elif L==8	// pl
	"Od tej chwili nie b&eogon;dziesz otrzymywa&cacute; wiadomo&sacute;ci od %s.";
#elif L==9	// pt
	"A partir deste momento, voc&ecirc; n&atilde;o receber&aacute; mensagens de %s.";
#elif L==10	// tr
	"From this time, you will not receive messages from %s.";	// eviri lazim!
#endif

const char *Txt_Full_name =
#if   L==1	// ca
	"Nom complet";
#elif L==2	// de
	"Vollst&auml;ndiger Name";
#elif L==3	// en
	"Full name";
#elif L==4	// es
	"Nombre completo";
#elif L==5	// fr
	"Nom complet";
#elif L==6	// gn
	"Nombre completo";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo";
#elif L==8	// pl
	"Pe&lstrok;na nazwa";
#elif L==9	// pt
	"Nome completo";
#elif L==10	// tr
	"Tam isim";
#endif

const char *Txt_Full_name_of_the_degree =
#if   L==1	// ca
	"Nom complet de la titulaci&oacute;";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Studiengang";
#elif L==3	// en
	"Full name of the degree";
#elif L==4	// es
	"Nombre completo de la titulaci&oacute;n";
#elif L==5	// fr
	"Nom complet du &eacute;tude";
#elif L==6	// gn
	"Nombre completo de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo della laurea";
#elif L==8	// pl
	"Pe&lstrok;na nazwa stopie&nacute;";
#elif L==9	// pt
	"Nome completo do grau";
#elif L==10	// tr
	"Full name of the degree";			// eviri lazim!
#endif

const char *Txt_Full_name_of_the_center =
#if   L==1	// ca
	"Nom complet del center";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Lehrinstitut";
#elif L==3	// en
	"Full name of the center";
#elif L==4	// es
	"Nombre completo del centro";
#elif L==5	// fr
	"Nom complet du center";
#elif L==6	// gn
	"Nombre completo del centro";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo del centro";
#elif L==8	// pl
	"Pe&lstrok;na nazwa centrum";
#elif L==9	// pt
	"Nome completo do centro";
#elif L==10	// tr
	"Full name of the center";	// eviri lazim!
#endif

const char *Txt_Full_name_of_the_course =
#if   L==1	// ca
	"Nom complet de l'assignatura";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Kurs";
#elif L==3	// en
	"Full name of the course";
#elif L==4	// es
	"Nombre completo de la asignatura";
#elif L==5	// fr
	"Nom complet de la mati&egrave;re";
#elif L==6	// gn
	"Nombre completo de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo del corso";
#elif L==8	// pl
	"Pe&lstrok;na nazwa kurs";
#elif L==9	// pt
	"Nome completo da disciplina";
#elif L==10	// tr
	"Full name of the course";		// eviri lazim!
#endif

const char *Txt_Full_name_of_the_institution =
#if   L==1	// ca
	"Nom complet de la instituci&oacute;";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Institution";
#elif L==3	// en
	"Full name of the institution";
#elif L==4	// es
	"Nombre completo de la instituci&oacute;n";
#elif L==5	// fr
	"Nom complet de l'&eacute;tablissement";
#elif L==6	// gn
	"Nombre completo de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo dell'istituzione";
#elif L==8	// pl
	"Pe&lstrok;na nazwa instytucji";
#elif L==9	// pt
	"Nome completo da institui&ccedil;&atilde;o";
#elif L==10	// tr
	"Full name of the institution";			// eviri lazim!
#endif

const char *Txt_Function =
#if   L==1	// ca
	"Funci&oacute;";
#elif L==2	// de
	"Funktion";
#elif L==3	// en
	"Function";
#elif L==4	// es
	"Funci&oacute;n";
#elif L==5	// fr
	"Fonction";
#elif L==6	// gn
	"Funci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Funzione";
#elif L==8	// pl
	"Funkcja";
#elif L==9	// pt
	"Fun&ccedil;&atilde;o";
#elif L==10	// tr
	"Function";		// eviri lazim!
#endif

const char *Txt_Game =
#if   L==1	// ca
	"Joc";
#elif L==2	// de
	"Spiel";
#elif L==3	// en
	"Game";
#elif L==4	// es
	"Juego";
#elif L==5	// fr
	"Jeu";
#elif L==6	// gn
	"&Ntilde;eha'a";
#elif L==7	// it
	"Gioco";
#elif L==8	// pl
	"Gra";
#elif L==9	// pt
	"Jogo";
#elif L==10	// tr
	"Oyun";
#endif

const char *Txt_Game_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Joc <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Spiel <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Game <strong>%s</strong> removed.";
#elif L==4	// es
	"Juego <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Jeu <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Juego <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Gioco <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Gra <strong>%s</strong> usuni&eogon;te.";
#elif L==9	// pt
	"Jogo <strong>%s</strong> removido.";
#elif L==10	// tr
	"Game <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_Games =
#if   L==1	// ca
	"Jocs";
#elif L==2	// de
	"Spiele";
#elif L==3	// en
	"Games";
#elif L==4	// es
	"Juegos";
#elif L==5	// fr
	"Jeux";
#elif L==6	// gn
	"Juegos";	// Okoteve traduccin
#elif L==7	// it
	"Giochi";
#elif L==8	// pl
	"Gry";
#elif L==9	// pt
	"Jogos";
#elif L==10	// tr
	"Oyunlar";
#endif

const char *Txt_GAMES_ORDER[Gam_NUM_ORDERS] =
	{
	[Gam_ORDER_BY_START_DATE] =
#if   L==1	// ca
	"Inici"
#elif L==2	// de
	"Start"
#elif L==3	// en
	"Start"
#elif L==4	// es
	"Inicio"
#elif L==5	// fr
	"D&eacute;but"
#elif L==6	// gn
	"&Ntilde;epyr&utilde;"
#elif L==7	// it
	"Inizio"
#elif L==8	// pl
	"Pocz&aogon;tek"
#elif L==9	// pt
	"In&iacute;cio"
#elif L==10	// tr
	"Ba&scedil;lama"
#endif
	,
	[Gam_ORDER_BY_END_DATE] =
#if   L==1	// ca
	"Final"
#elif L==2	// de
	"Ende"
#elif L==3	// en
	"End"
#elif L==4	// es
	"Final"
#elif L==5	// fr
	"Fin"
#elif L==6	// gn
	"Paha"
#elif L==7	// it
	"Fine"
#elif L==8	// pl
	"Koniec"
#elif L==9	// pt
	"Fim"
#elif L==10	// tr
	"Son"
#endif
	,
	[Gam_ORDER_BY_TITLE] =
#if   L==1	// ca
	"Joc"
#elif L==2	// de
	"Spiel"
#elif L==3	// en
	"Game"
#elif L==4	// es
	"Juego"
#elif L==5	// fr
	"Jeu"
#elif L==6	// gn
	"Juego"	// Okoteve traduccin
#elif L==7	// it
	"Gioco"
#elif L==8	// pl
	"Gra"
#elif L==9	// pt
	"Jogo"
#elif L==10	// tr
	"Game"	// eviri lazim!
#endif
	};

const char *Txt_GAMES_ORDER_HELP[Gam_NUM_ORDERS] =
	{
	[Gam_ORDER_BY_START_DATE] =
#if   L==1	// ca
	"Ordenar per data de comen&ccedil;ament"
#elif L==2	// de
	"Sortieren nach Startdatum"
#elif L==3	// en
	"Sort by start date"
#elif L==4	// es
	"Ordenar por fecha de comienzo"
#elif L==5	// fr
	"Trier par date de d&eacute;but"
#elif L==6	// gn
	"Ordenar por fecha de comienzo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data di inizio"
#elif L==8	// pl
	"Sortuj wedlug daty rozpoczecia"
#elif L==9	// pt
	"Classificar por data de in&iacute;cio"
#elif L==10	// tr
	"Sort by start date"		// eviri lazim!
#endif
	,
	[Gam_ORDER_BY_END_DATE] =
#if   L==1	// ca
	"Ordenar per data de finalitzaci&oacute;"
#elif L==2	// de
	"Sortieren nach Enddatum"
#elif L==3	// en
	"Sort by end date"
#elif L==4	// es
	"Ordenar por fecha de finalizaci&oacute;n"
#elif L==5	// fr
	"Trier par date de fin"
#elif L==6	// gn
	"Ordenar por fecha de finalizaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data di fine"
#elif L==8	// pl
	"Sortuj wedlug daty zakonczenia"
#elif L==9	// pt
	"Classificar por data de t&eacute;rmino"
#elif L==10	// tr
	"Sort by end date"				// eviri lazim!
#endif
	,
	[Gam_ORDER_BY_TITLE] =
#if   L==1	// ca
	"Ordenar per t&iacute;tol del joc"
#elif L==2	// de
	"Sortieren nach Spieltitel"
#elif L==3	// en
	"Sort by game title"
#elif L==4	// es
	"Ordenar por t&iacute;tulo del juego"
#elif L==5	// fr
	"Trier par titre de jeu"
#elif L==6	// gn
	"Ordenar por t&iacute;tulo del juego"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per titolo del gioco"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug tytu&lstrok;u gry"
#elif L==9	// pt
	"Classificar por t&iacute;tulo do jogo"
#elif L==10	// tr
	"Sort by game title"			// eviri lazim!
#endif
	};

const char *Txt_General =
#if   L==1	// ca
	"General";
#elif L==2	// de
	"Allgemein";
#elif L==3	// en
	"General";
#elif L==4	// es
	"General";
#elif L==5	// fr
	"G&eacute;n&eacute;ral";
#elif L==6	// gn
	"General";	// Okoteve traduccin
#elif L==7	// it
	"Generale";
#elif L==8	// pl
	"Ogolne";
#elif L==9	// pt
	"Geral";
#elif L==10	// tr
	"General";	// eviri lazim!
#endif

const char *Txt_general_topics =
#if   L==1	// ca
	"temas generales";	// Necessita traducci
#elif L==2	// de
	"allgemeine Themen";
#elif L==3	// en
	"general topics";
#elif L==4	// es
	"temas generales";
#elif L==5	// fr
	"affaires g&eacute;n&eacute;rales";
#elif L==6	// gn
	"temas generales";	// Okoteve traduccin
#elif L==7	// it
	"temi generali";
#elif L==8	// pl
	"oglne tematy";
#elif L==9	// pt
	"temas gerais";
#elif L==10	// tr
	"general topics";	// eviri lazim!
#endif

const char *Txt_Generate_report =
#if   L==1	// ca
	"Generar informe";
#elif L==2	// de
	"Generieren Nutzungsbericht";
#elif L==3	// en
	"Generate report";
#elif L==4	// es
	"Generar informe";
#elif L==5	// fr
	"G&eacute;n&eacute;rer le rapport";
#elif L==6	// gn
	"Generar informe";	// Okoteve traduccin
#elif L==7	// it
	"Crea rapporto";
#elif L==8	// pl
	"Wygeneruj raport";
#elif L==9	// pt
	"Gerar relat&oacute;rio";
#elif L==10	// tr
	"Generate report";	// eviri lazim!
#endif

const char *Txt_Generate_test =
#if   L==1	// ca
	"Generar test";
#elif L==2	// de
	"Generieren Test";
#elif L==3	// en
	"Generate test";
#elif L==4	// es
	"Generar test";
#elif L==5	// fr
	"G&eacute;n&eacute;rer test";
#elif L==6	// gn
	"Generar test";		// Okoteve traduccin
#elif L==7	// it
	"Crea test";
#elif L==8	// pl
	"Wygeneruj test";
#elif L==9	// pt
	"Gerar teste";
#elif L==10	// tr
	"Generate test";	// eviri lazim!
#endif

const char *Txt_Get_a_new_password =
#if   L==1	// ca
	"Rebre una nova contrasenya";
#elif L==2	// de
	"Neues Passwort beantragen";
#elif L==3	// en
	"Get a new password";
#elif L==4	// es
	"Recibir una nueva contrase&ntilde;a";
#elif L==5	// fr
	"Recevoir un nouveau mot de passe";
#elif L==6	// gn
	"Recibir una nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Ottenere una nuova password";
#elif L==8	// pl
	"Uzyskaj nowe has&lstrok;o";
#elif L==9	// pt
	"Obter uma nova senha";
#elif L==10	// tr
	"Yeni bir &scedil;ifre al&inodot;n";
#endif

const char *Txt_Global_visits =
#if   L==1	// ca
	"Accessos globals";
#elif L==2	// de
	"Globale Anmeldungen";
#elif L==3	// en
	"Global visits";
#elif L==4	// es
	"Accesos globales";
#elif L==5	// fr
	"Visites globales";
#elif L==6	// gn
	"Accesos globales";	// Okoteve traduccin
#elif L==7	// it
	"Accessi globali";
#elif L==8	// pl
	"Globalne wizyty";
#elif L==9	// pt
	"Visitas globais";
#elif L==10	// tr
	"Global visits";	// eviri lazim!
#endif

const char *Txt_Go_back =
#if   L==1	// ca
	"Tornar enrere";
#elif L==2	// de
	"Zur&uuml;ckgehen";
#elif L==3	// en
	"Go back";
#elif L==4	// es
	"Volver atr&aacute;s";
#elif L==5	// fr
	"Retour";
#elif L==6	// gn
	"Volver atr&aacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Tornare";
#elif L==8	// pl
	"Wraca&cacute;";
#elif L==9	// pt
	"Voltar";
#elif L==10	// tr
	"Go back";		// eviri lazim!
#endif

const char *Txt_Go_forward =
#if   L==1	// ca
	"Anar endavant";
#elif L==2	// de
	"Vorw&auml;rts gehen";
#elif L==3	// en
	"Go forward";
#elif L==4	// es
	"Avanzar";
#elif L==5	// fr
	"Aller de l'avant";
#elif L==6	// gn
	"Avanzar";	// Okoteve traduccin
#elif L==7	// it
	"Avanzare";
#elif L==8	// pl
	"Id&zacute; napr&oacute;d";
#elif L==9	// pt
	"Avan&ccedil;ar";
#elif L==10	// tr
	"Go forward";	// eviri lazim!
#endif

const char *Txt_Go_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ir a %s";	// Necessita traducci
#elif L==2	// de
	"Gehe zum %s";
#elif L==3	// en
	"Go to %s";
#elif L==4	// es
	"Ir a %s";
#elif L==5	// fr
	"Aller &agrave; %s";
#elif L==6	// gn
	"Ir a %s";	// Okoteve traduccin
#elif L==7	// it
	"Vai a %s";
#elif L==8	// pl
	"Przejdz do %s";
#elif L==9	// pt
	"Ir para %s";
#elif L==10	// tr
	"Go to %s";	// eviri lazim!
#endif

const char *Txt_Go_to_one_of_my_courses =
#if   L==1	// ca
	"Anar a una de les meves assignatures";
#elif L==2	// de
	"Gehen Sie zu einem meiner Kurse";
#elif L==3	// en
	"Go to one of my courses";
#elif L==4	// es
	"Ir a una de mis asignaturas";
#elif L==5	// fr
	"Allez &agrave; l'une de mes mati&egrave;res";
#elif L==6	// gn
	"Ir a una de mis asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Vai a uno dei miei corsi";
#elif L==8	// pl
	"Id&zacute; do jednej z moich kurs&oacute;w";
#elif L==9	// pt
	"V&aacute; para uma das minha disciplinas";
#elif L==10	// tr
	"Go to one of my courses";	// eviri lazim!
#endif

const char *Txt_Grade =
#if   L==1	// ca
	"Nota";
#elif L==2	// de
	"Note";
#elif L==3	// en
	"Grade";
#elif L==4	// es
	"Nota";
#elif L==5	// fr
	"Note";
#elif L==6	// gn
	"Nota";		// Okoteve traduccin
#elif L==7	// it
	"Voto";
#elif L==8	// pl
	"Ocena";
#elif L==9	// pt
	"Nota";
#elif L==10	// tr
	"Vas&inodot;f";
#endif

const char *Txt_Graph =
#if   L==1	// ca
	"Gr&agrave;fica";
#elif L==2	// de
	"Graph";
#elif L==3	// en
	"Graph";
#elif L==4	// es
	"Gr&aacute;fica";
#elif L==5	// fr
	"Graphique";
#elif L==6	// gn
	"Gr&aacute;fica";	// Okoteve traduccin
#elif L==7	// it
	"Grafico";
#elif L==8	// pl
	"Wykres";
#elif L==9	// pt
	"Gr&aacute;fico";
#elif L==10	// tr
	"Grafik";
#endif

const char *Txt_Group =
#if   L==1	// ca
	"Grup";
#elif L==2	// de
	"Gruppe";
#elif L==3	// en
	"Group";
#elif L==4	// es
	"Grupo";
#elif L==5	// fr
	"Groupe";
#elif L==6	// gn
	"Aty";
#elif L==7	// it
	"Gruppo";
#elif L==8	// pl
	"Grupa";
#elif L==9	// pt
	"Grupo";
#elif L==10	// tr
	"Group";	// eviri lazim!
#endif

const char *Txt_group =
#if   L==1	// ca
	"grup";
#elif L==2	// de
	"Gruppe";
#elif L==3	// en
	"group";
#elif L==4	// es
	"grupo";
#elif L==5	// fr
	"groupe";
#elif L==6	// gn
	"aty";
#elif L==7	// it
	"gruppo";
#elif L==8	// pl
	"grupa";
#elif L==9	// pt
	"grupo";
#elif L==10	// tr
	"group";	// eviri lazim!
#endif

const char *Txt_Group_closed =
#if   L==1	// ca
	"Grup tancat";
#elif L==2	// de
	"Gruppe geschlossen";
#elif L==3	// en
	"Group closed";
#elif L==4	// es
	"Grupo cerrado";
#elif L==5	// fr
	"Groupe ferm&eacute;";
#elif L==6	// gn
	"Grupo cerrado";	// Okoteve traduccin
#elif L==7	// it
	"Gruppo chiuso";
#elif L==8	// pl
	"Grupa zamknieta";
#elif L==9	// pt
	"Grupo fechado";
#elif L==10	// tr
	"Group closed";		// eviri lazim!
#endif

const char *Txt_Group_X_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup %s tancat";
#elif L==2	// de
	"Gruppe %s geschlossen";
#elif L==3	// en
	"Group %s closed";
#elif L==4	// es
	"Grupo %s cerrado";
#elif L==5	// fr
	"Groupe %s ferm&eacute;";
#elif L==6	// gn
	"Grupo %s cerrado";	// Okoteve traduccin
#elif L==7	// it
	"Gruppo %s chiuso";
#elif L==8	// pl
	"Grupa %s zamknieta";
#elif L==9	// pt
	"Grupo %s fechado";
#elif L==10	// tr
	"Group %s closed";	// eviri lazim!
#endif

const char *Txt_Group_name =
#if   L==1	// ca
	"Nom del grup";
#elif L==2	// de
	"Name der Gruppe";
#elif L==3	// en
	"Group name";
#elif L==4	// es
	"Nombre del grupo";
#elif L==5	// fr
	"Nom du groupe";
#elif L==6	// gn
	"Nombre del grupo";	// Okoteve traduccin
#elif L==7	// it
	"Nome del gruppo";
#elif L==8	// pl
	"Nazwa grupy";
#elif L==9	// pt
	"Nome do grupo";
#elif L==10	// tr
	"Group name";		// eviri lazim!
#endif

const char *Txt_Group_X_open =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup %s obert";
#elif L==2	// de
	"Gruppe %s &ouml;ffnen";
#elif L==3	// en
	"Group %s open";
#elif L==4	// es
	"Grupo %s abierto";
#elif L==5	// fr
	"Groupe %s ouvert";
#elif L==6	// gn
	"Grupo %s abierto";	// Okoteve traduccin
#elif L==7	// it
	"Gruppo %s aperto";
#elif L==8	// pl
	"Grupa %s otwarta";
#elif L==9	// pt
	"Grupo %s aberto";
#elif L==10	// tr
	"Group %s open";	// eviri lazim!
#endif

const char *Txt_Group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Gruppe <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Group <strong>%s</strong> removed.";
#elif L==4	// es
	"Grupo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Groupe <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Grupo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Gruppo <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Grupa <strong>%s</strong> usunieta.";
#elif L==9	// pt
	"Grupo <strong>%s</strong> removido.";
#elif L==10	// tr
	"Group <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_GROUP_WHICH_GROUPS[2] =
	{
#if   L==1	// ca
	"Nom&eacute;s els meus grups"
#elif L==2	// de
	"Nur meine Gruppen"
#elif L==3	// en
	"Only my groups"
#elif L==4	// es
	"Solo mis grupos"
#elif L==5	// fr
	"Uniquement mes groupes"
#elif L==6	// gn
	"Solo mis grupos"	// Okoteve traduccin
#elif L==7	// it
	"Solo i miei gruppi"
#elif L==8	// pl
	"Tylko moje grupy"
#elif L==9	// pt
	"Apenas os meus grupos"
#elif L==10	// tr
	"Only my groups"	// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Tots els grups"
#elif L==2	// de
	"Alle Gruppen"
#elif L==3	// en
	"All groups"
#elif L==4	// es
	"Todos los grupos"
#elif L==5	// fr
	"Tous les groupes"
#elif L==6	// gn
	"Todos los grupos"	// Okoteve traduccin
#elif L==7	// it
	"Tutti i gruppi"
#elif L==8	// pl
	"Wszystkie grupy"
#elif L==9	// pt
	"Todos os grupos"
#elif L==10	// tr
	"All groups"		// eviri lazim!
#endif
	};

const char *Txt_Groups =
#if   L==1	// ca
	"Grups";
#elif L==2	// de
	"Gruppen";
#elif L==3	// en
	"Groups";
#elif L==4	// es
	"Grupos";
#elif L==5	// fr
	"Groupes";
#elif L==6	// gn
	"Aty";
#elif L==7	// it
	"Gruppi";
#elif L==8	// pl
	"Grupy";
#elif L==9	// pt
	"Grupos";
#elif L==10	// tr
	"Gruplar";
#endif

const char *Txt_Groups_OF_A_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grups de %s";
#elif L==2	// de
	"Gruppen von %s";
#elif L==3	// en
	"%s groups";
#elif L==4	// es
	"Grupos de %s";
#elif L==5	// fr
	"Groupes de %s";
#elif L==6	// gn
	"Grupos de %s";	// Okoteve traduccin
#elif L==7	// it
	"Gruppi di %s";
#elif L==8	// pl
	"%s grupy";
#elif L==9	// pt
	"Grupos de %s";
#elif L==10	// tr
	"%s groups";	// eviri lazim!
#endif

const char *Txt_Guided_academic_assignments =
#if   L==1	// ca
	"Activitats acad&egrave;miques dirigides";
#elif L==2	// de
	"Gef&uuml;hrte akademische Aufgaben";
#elif L==3	// en
	"Guided academic assignments";
#elif L==4	// es
	"Actividades acad&eacute;micas dirigidas";
#elif L==5	// fr
	"Activit&eacute;s acad&eacute;miques guid&eacute;es";
#elif L==6	// gn
	"Actividades acad&eacute;micas dirigidas";	// Okoteve traduccin
#elif L==7	// it
	"Attivit&agrave; accademiche guidate";
#elif L==8	// pl
	"Zwiedzanie dyplomowe";
#elif L==9	// pt
	"Atividades acad&eacute;micas guided";
#elif L==10	// tr
	"Guided academic assignments";			// eviri lazim!
#endif

const char *Txt_Happy_birthday_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Feli&ccedil; aniversari, %s!";
#elif L==2	// de
	"Gute zum Geburtstag, %s!";
#elif L==3	// en
	"Happy birthday, %s!";
#elif L==4	// es
	"&iexcl;Feliz cumplea&ntilde;os, %s!";
#elif L==5	// fr
	"Joyeux anniversaire, %s!";
#elif L==6	// gn
	"&iexcl;Feliz cumplea&ntilde;os, %s!";	// Okoteve traduccin
#elif L==7	// it
	"Buon compleanno, %s!";
#elif L==8	// pl
	"Wszystkiego najlepszego, %s!";
#elif L==9	// pt
	"Feliz anivers&aacute;rio, %s!";
#elif L==10	// tr
	"Happy birthday, %s!";			// eviri lazim!
#endif

const char *Txt_Help =
#if   L==1	// ca
	"Ajuda";
#elif L==2	// de
	"Hilfe";
#elif L==3	// en
	"Help";
#elif L==4	// es
	"Ayuda";
#elif L==5	// fr
	"Aide";
#elif L==6	// gn
	"Pytyv&otilde;";
#elif L==7	// it
	"Aiuto";
#elif L==8	// pl
	"Pomoc";
#elif L==9	// pt
	"Ajuda";
#elif L==10	// tr
	"Help";	// eviri lazim!
#endif

const char *Txt_HELP_email =
#if   L==1	// ca
	"Exemple: garcia@exemplemail.com";
#elif L==2	// de
	"Beispiel: muller@beispielmail.com";
#elif L==3	// en
	"Example: smith@examplemail.com";
#elif L==4	// es
	"Ejemplo: gonzalez@ejemplomail.com";
#elif L==5	// fr
	"Exemple: dubois@exemplemail.com";
#elif L==6	// gn
	"Ejemplo: gonzalez@ejemplomail.com";	// Okoteve traduccin
#elif L==7	// it
	"Esempio: rossi@esempiomail.com";
#elif L==8	// pl
	"Przyk&lstrok;ad: nowak@przykladmail.com";
#elif L==9	// pt
	"Exemplo: silva@exemplomail.com";
#elif L==10	// tr
	"Example: smith@examplemail.com";	// eviri lazim!
#endif

const char *Txt_HELP_nickname =
#if   L==1	// ca
	"Exemple: @garcia";
#elif L==2	// de
	"Beispiel: @muller";
#elif L==3	// en
	"Example: @smith";
#elif L==4	// es
	"Ejemplo: @gonzalez";
#elif L==5	// fr
	"Exemple: @dubois";
#elif L==6	// gn
	"Ejemplo: @gonzalez";	// Okoteve traduccin
#elif L==7	// it
	"Esempio: @rossi";
#elif L==8	// pl
	"Przyk&lstrok;ad: @nowak";
#elif L==9	// pt
	"Exemplo: @silva";
#elif L==10	// tr
	"Example: @smith";	// eviri lazim!
#endif

const char *Txt_HELP_password =
#if   L==1	// ca
	"8 o m&eacute;s car&agrave;cters";
#elif L==2	// de
	"8 oder mehr Zeichen";
#elif L==3	// en
	"8 or more characters";
#elif L==4	// es
	"8 o m&aacute;s caracteres";
#elif L==5	// fr
	"8 ou plusieurs caract&egrave;res";
#elif L==6	// gn
	"8 o m&aacute;s caracteres";	// Okoteve traduccin
#elif L==7	// it
	"8 o pi&ugrave; caratteri";
#elif L==8	// pl
	"8 lub wi&eogon;cej znak&oacute;w";
#elif L==9	// pt
	"8 ou mais caracteres";
#elif L==10	// tr
	"8 or more characters";		// eviri lazim!
#endif

const char *Txt_Hidden_MALE_PLURAL =
#if   L==1	// ca
	"Ocults";
#elif L==2	// de
	"Versteckt";
#elif L==3	// en
	"Hidden";
#elif L==4	// es
	"Ocultos";
#elif L==5	// fr
	"Cach&eacute;s";
#elif L==6	// gn
	"Ocultos";	// Okoteve traduccin
#elif L==7	// it
	"Nascosti";
#elif L==8	// pl
	"Ukryty";
#elif L==9	// pt
	"Ocultos";
#elif L==10	// tr
	"Hidden";	// eviri lazim!
#endif

const char *Txt_hidden_document =
#if   L==1	// ca
	"document ocult";
#elif L==2	// de
	"Dokument verborgen";
#elif L==3	// en
	"hidden document";
#elif L==4	// es
	"documento oculto";
#elif L==5	// fr
	"document cach&eacute;";
#elif L==6	// gn
	"documento oculto";	// Okoteve traduccin
#elif L==7	// it
	"documento nascosto";
#elif L==8	// pl
	"hidden document";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"documento oculto";
#elif L==10	// tr
	"hidden document";	// eviri lazim!
#endif

const char *Txt_hidden_documents =
#if   L==1	// ca
	"documents ocults";
#elif L==2	// de
	"Dokumente verborgen";
#elif L==3	// en
	"hidden documents";
#elif L==4	// es
	"documentos ocultos";
#elif L==5	// fr
	"documents cach&eacute;s";
#elif L==6	// gn
	"documentos ocultos";	// Okoteve traduccin
#elif L==7	// it
	"documenti nascosti";
#elif L==8	// pl
	"hidden documents";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"documentos ocultos";
#elif L==10	// tr
	"hidden documents";	// eviri lazim!
#endif

const char *Txt_Hidden_results =
#if   L==1	// ca
	"Resultats ocults";
#elif L==2	// de
	"Ausgeblendet Ergebnisse";
#elif L==3	// en
	"Hidden results";
#elif L==4	// es
	"Resultados ocultos";
#elif L==5	// fr
	"R&eacute;sultats cach&eacute;s";
#elif L==6	// gn
	"Resultados ocultos";	// Okoteve traduccin
#elif L==7	// it
	"Risultati nascosti";
#elif L==8	// pl
	"Ukryte wynik&oacute;w";
#elif L==9	// pt
	"Resultados ocultos";
#elif L==10	// tr
	"Hidden results";	// eviri lazim!
#endif

const char *Txt_Hidden_survey =
#if   L==1	// ca
	"Enquesta oculta";
#elif L==2	// de
	"Ausgeblendet Umfrage";
#elif L==3	// en
	"Hidden survey";
#elif L==4	// es
	"Encuesta oculta";
#elif L==5	// fr
	"Sondage cach&eacute;";
#elif L==6	// gn
	"Encuesta oculta";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio nascosto";
#elif L==8	// pl
	"Ukryte badania";
#elif L==9	// pt
	"Inqu&eacute;rito oculto";
#elif L==10	// tr
	"Hidden survey";	// eviri lazim!
#endif

const char *Txt_Hide_left_column =
#if   L==1	// ca
	"Amagar columna esquerra";
#elif L==2	// de
	"Links Spalte ausblenden";
#elif L==3	// en
	"Hide left column";
#elif L==4	// es
	"Ocultar columna izquierda";
#elif L==5	// fr
	"Masquer la colonne de gauche";
#elif L==6	// gn
	"Ocultar columna izquierda";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi colonna di sinistra";
#elif L==8	// pl
	"Ukryj lewa kolumne";
#elif L==9	// pt
	"Ocultar coluna da esquerda";
#elif L==10	// tr
	"Hide left column";		// eviri lazim!
#endif

const char *Txt_Hide_message =
#if   L==1	// ca
	"Amaga missatge";
#elif L==2	// de
	"Ausblende Nachricht";
#elif L==3	// en
	"Hide message";
#elif L==4	// es
	"Ocultar mensaje";
#elif L==5	// fr
	"Masquer message";
#elif L==6	// gn
	"Ocultar mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi messaggio";
#elif L==8	// pl
	"Hide message";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ocultar mensagem";
#elif L==10	// tr
	"Hide message";		// eviri lazim!
#endif

const char *Txt_Hide_right_column =
#if   L==1	// ca
	"Amagar columna dreta";
#elif L==2	// de
	"Rechte Spalte ausblenden";
#elif L==3	// en
	"Hide right column";
#elif L==4	// es
	"Ocultar columna derecha";
#elif L==5	// fr
	"Masquer la colonne de droite";
#elif L==6	// gn
	"Ocultar columna derecha";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi colonna di destra";
#elif L==8	// pl
	"Ukryj prawa kolumne";
#elif L==9	// pt
	"Ocultar coluna da direita";
#elif L==10	// tr
	"Hide right column";		// eviri lazim!
#endif

const char *Txt_HIERARCHY_PLURAL_Abc[Hie_NUM_LEVELS] =
	{
	[Hie_CTY] =
#if   L==1	// ca
	"Pa&iuml;sos"
#elif L==2	// de
	"L&auml;nder"
#elif L==3	// en
	"Countries"
#elif L==4	// es
	"Pa&iacute;ses"
#elif L==5	// fr
	"Pays"
#elif L==6	// gn
	"Tet&atilde;"
#elif L==7	// it
	"Paesi"
#elif L==8	// pl
	"Kraje"
#elif L==9	// pt
	"Pa&iacute;ses"
#elif L==10	// tr
	"Countries"	// eviri lazim!
#endif
	,
	[Hie_INS] =
#if   L==1	// ca
	"Institucions"
#elif L==2	// de
	"Hochschulen"
#elif L==3	// en
	"Institutions"
#elif L==4	// es
	"Instituciones"
#elif L==5	// fr
	"&Eacute;tablissements"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzioni"
#elif L==8	// pl
	"Instytucje"
#elif L==9	// pt
	"Institu&ccedil;&otilde;es"
#elif L==10	// tr
	"Institutions"	// eviri lazim!
#endif
	,
	[Hie_CTR] =
#if   L==1	// ca
	"Centers"
#elif L==2	// de
	"Lehrinstitute"
#elif L==3	// en
	"Centers"
#elif L==4	// es
	"Centros"
#elif L==5	// fr
	"Centers"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centri"
#elif L==8	// pl
	"Centra"
#elif L==9	// pt
	"Centros"
#elif L==10	// tr
	"Merkezler"
#endif
	,
	[Hie_DEG] =
#if   L==1	// ca
	"Titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degrees"
#elif L==4	// es
	"Titulaciones"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#elif L==10	// tr
	"Derece"
#endif
	,
	[Hie_CRS] =
#if   L==1	// ca
	"Assignatures"
#elif L==2	// de
	"Kursen"
#elif L==3	// en
	"Courses"
#elif L==4	// es
	"Asignaturas"
#elif L==5	// fr
	"Mati&egrave;res"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corsi"
#elif L==8	// pl
	"Kurs&oacute;w"
#elif L==9	// pt
	"Disciplinas"
#elif L==10	// tr
	"Dersler"
#endif
	};

const char *Txt_HIERARCHY_PLURAL_abc[Hie_NUM_LEVELS] =
	{
	[Hie_CTY] =
#if   L==1	// ca
	"pa&iuml;sos"
#elif L==2	// de
	"L&auml;nder"
#elif L==3	// en
	"countries"
#elif L==4	// es
	"pa&iacute;ses"
#elif L==5	// fr
	"pays"
#elif L==6	// gn
	"tet&atilde;"
#elif L==7	// it
	"paesi"
#elif L==8	// pl
	"kraje"
#elif L==9	// pt
	"pa&iacute;ses"
#elif L==10	// tr
	"countries"	// eviri lazim!
#endif
	,
	[Hie_INS] =
#if   L==1	// ca
	"institucions"
#elif L==2	// de
	"Hochschulen"
#elif L==3	// en
	"institutions"
#elif L==4	// es
	"instituciones"
#elif L==5	// fr
	"&eacute;tablissements"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"istituzioni"
#elif L==8	// pl
	"instytucje"
#elif L==9	// pt
	"institu&ccedil;&otilde;es"
#elif L==10	// tr
	"institutions"	// eviri lazim!
#endif
	,
	[Hie_CTR] =
#if   L==1	// ca
	"centers"
#elif L==2	// de
	"Lehrinstitute"
#elif L==3	// en
	"centers"
#elif L==4	// es
	"centros"
#elif L==5	// fr
	"centers"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"centri"
#elif L==8	// pl
	"centra"
#elif L==9	// pt
	"centros"
#elif L==10	// tr
	"merkezler"
#endif
	,
	[Hie_DEG] =
#if   L==1	// ca
	"titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"degrees"
#elif L==4	// es
	"titulaciones"
#elif L==5	// fr
	"&eacute;tudes"
#elif L==6	// gn
	"arandur&atilde;"
#elif L==7	// it
	"lauree"
#elif L==8	// pl
	"stopnie"
#elif L==9	// pt
	"graus"
#elif L==10	// tr
	"derece"
#endif
	,
	[Hie_CRS] =
#if   L==1	// ca
	"assignatures"
#elif L==2	// de
	"Kursen"
#elif L==3	// en
	"courses"
#elif L==4	// es
	"asignaturas"
#elif L==5	// fr
	"mati&egrave;res"
#elif L==6	// gn
	"mbo'esyry"
#elif L==7	// it
	"corsi"
#elif L==8	// pl
	"kurs&oacute;w"
#elif L==9	// pt
	"disciplinas"
#elif L==10	// tr
	"dersler"
#endif
	};

const char *Txt_HIERARCHY_SINGUL_Abc[Hie_NUM_LEVELS] =
	{
	[Hie_SYS] =
#if   L==1	// ca
	"Plataforma"
#elif L==2	// de
	"Plattform"
#elif L==3	// en
	"System"
#elif L==4	// es
	"Plataforma"
#elif L==5	// fr
	"Plate-forme"
#elif L==6	// gn
	"Ypykatu"
#elif L==7	// it
	"Piattaforma"
#elif L==8	// pl
	"System"
#elif L==9	// pt
	"Sistema"
#elif L==10	// tr
	"Platformu"
#endif
	,
	[Hie_CTY] =
#if   L==1	// ca
	"Pa&iacute;s"
#elif L==2	// de
	"Land"
#elif L==3	// en
	"Country"
#elif L==4	// es
	"Pa&iacute;s"
#elif L==5	// fr
	"Pays;"
#elif L==6	// gn
	"Tet&atilde;"
#elif L==7	// it
	"Paese"
#elif L==8	// pl
	"Kraj"
#elif L==9	// pt
	"Pa&iacute;s"
#elif L==10	// tr
	"&Uuml;lke"
#endif
	,
	[Hie_INS] =
#if   L==1	// ca
	"Instituci&oacute;"
#elif L==2	// de
	"Hochschule"
#elif L==3	// en
	"Institution"
#elif L==4	// es
	"Instituci&oacute;n"
#elif L==5	// fr
	"&Eacute;tablissement"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzione"
#elif L==8	// pl
	"Instytucji"
#elif L==9	// pt
	"Institu&ccedil;&atilde;o"
#elif L==10	// tr
	"Kurum"
#endif
	,
	[Hie_CTR] =
#if   L==1	// ca
	"Center"
#elif L==2	// de
	"Lehrinstitut"
#elif L==3	// en
	"Center"
#elif L==4	// es
	"Centro"
#elif L==5	// fr
	"Center"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centro"
#elif L==8	// pl
	"Centrum"
#elif L==9	// pt
	"Centro"
#elif L==10	// tr
	"Merkez"
#endif
	,
	[Hie_DEG] =
#if   L==1	// ca
	"Titulaci&oacute;"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degree"
#elif L==4	// es
	"Titulaci&oacute;n"
#elif L==5	// fr
	"&Eacute;tude"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Laurea"
#elif L==8	// pl
	"Stopien"
#elif L==9	// pt
	"Grau"
#elif L==10	// tr
	"Derece"
#endif
	,
	[Hie_CRS] =
#if   L==1	// ca
	"Assignatura"
#elif L==2	// de
	"Kurs"
#elif L==3	// en
	"Course"
#elif L==4	// es
	"Asignatura"
#elif L==5	// fr
	"Mati&egrave;re"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corso"
#elif L==8	// pl
	"Kurs"
#elif L==9	// pt
	"Disciplina"
#elif L==10	// tr
	"Kurs"
#endif
	};

const char *Txt_HIERARCHY_SINGUL_abc[Hie_NUM_LEVELS] =
	{
	[Hie_SYS] =
#if   L==1	// ca
	"plataforma"
#elif L==2	// de
	"Plattform"
#elif L==3	// en
	"system"
#elif L==4	// es
	"plataforma"
#elif L==5	// fr
	"plate-forme"
#elif L==6	// gn
	"ypykatu"
#elif L==7	// it
	"piattaforma"
#elif L==8	// pl
	"system"
#elif L==9	// pt
	"sistema"
#elif L==10	// tr
	"platformu"
#endif
	,
	[Hie_CTY] =
#if   L==1	// ca
	"pa&iacute;s"
#elif L==2	// de
	"Land"
#elif L==3	// en
	"country"
#elif L==4	// es
	"pa&iacute;s"
#elif L==5	// fr
	"pays;"
#elif L==6	// gn
	"tet&atilde;"
#elif L==7	// it
	"paese"
#elif L==8	// pl
	"kraj"
#elif L==9	// pt
	"pa&iacute;s"
#elif L==10	// tr
	"&uuml;lke"
#endif
	,
	[Hie_INS] =
#if   L==1	// ca
	"instituci&oacute;"
#elif L==2	// de
	"Hochschule"
#elif L==3	// en
	"institution"
#elif L==4	// es
	"instituci&oacute;n"
#elif L==5	// fr
	"&eacute;tablissement"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"istituzione"
#elif L==8	// pl
	"instytucji"
#elif L==9	// pt
	"institu&ccedil;&atilde;o"
#elif L==10	// tr
	"kurum"
#endif
	,
	[Hie_CTR] =
#if   L==1	// ca
	"center"
#elif L==2	// de
	"Lehrinstitut"
#elif L==3	// en
	"center"
#elif L==4	// es
	"centro"
#elif L==5	// fr
	"center"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"centro"
#elif L==8	// pl
	"centrum"
#elif L==9	// pt
	"centro"
#elif L==10	// tr
	"merkez"
#endif
	,
	[Hie_DEG] =
#if   L==1	// ca
	"titulaci&oacute;"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"degree"
#elif L==4	// es
	"titulaci&oacute;n"
#elif L==5	// fr
	"&eacute;tude"
#elif L==6	// gn
	"arandur&atilde;"
#elif L==7	// it
	"laurea"
#elif L==8	// pl
	"stopien"
#elif L==9	// pt
	"grau"
#elif L==10	// tr
	"derece"
#endif
	,
	[Hie_CRS] =
#if   L==1	// ca
	"assignatura"
#elif L==2	// de
	"Kurs"
#elif L==3	// en
	"course"
#elif L==4	// es
	"asignatura"
#elif L==5	// fr
	"mati&egrave;re"
#elif L==6	// gn
	"mbo'esyry"
#elif L==7	// it
	"corso"
#elif L==8	// pl
	"kurs"
#elif L==9	// pt
	"disciplina"
#elif L==10	// tr
	"kurs"
#endif
	};

const char *Txt_historical_log =
#if   L==1	// ca
	"registre hist&ograve;ric";
#elif L==2	// de
	"historische log";
#elif L==3	// en
	"historical log";
#elif L==4	// es
	"registro hist&oacute;rico";
#elif L==5	// fr
	"journal historique";
#elif L==6	// gn
	"registro hist&oacute;rico";	// Okoteve traduccin
#elif L==7	// it
	"registro storico";
#elif L==8	// pl
	"dziennik historyczny";
#elif L==9	// pt
	"log hist&oacute;rico";
#elif L==10	// tr
	"historical log";		// eviri lazim!
#endif

const char *Txt_Hits =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos";
#elif L==2	// de
	"Anmeldungen";
#elif L==3	// en
	"Hits";
#elif L==4	// es
	"Accesos (clics)";
#elif L==5	// fr
	"Visites";
#elif L==6	// gn
	"Accesos (clics)";	// Okoteve traduccin
#elif L==7	// it
	"Visite";
#elif L==8	// pl
	"Ods&lstrok;on";
#elif L==9	// pt
	"Visitas";
#elif L==10	// tr
	"Hits";			// eviri lazim!
#endif

const char *Txt_Hits_as_a_USER =	// hits = visits, clicks, page views... Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Accessos (clics) com a %s";
#elif L==2	// de
	"Anmeldungen (Klicks) als %s";
#elif L==3	// en
	"Hits (clicks) as a %s";
#elif L==4	// es
	"Accesos (clics) como %s";
#elif L==5	// fr
	"Visites (clics) en tant que %s";
#elif L==6	// gn
	"Accesos (clics) como %s";	// Okoteve traduccin
#elif L==7	// it
	"Visite (scatti) come %s";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) jako %s";
#elif L==9	// pt
	"Visitas (cliques) como %s";
#elif L==10	// tr
	"Hits (clicks) as a %s";	// eviri lazim!
#endif

const char *Txt_Hits_per_action =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos (clics) per acci&oacute;";
#elif L==2	// de
	"Anmeldungen (Klicks) pro Handlung";
#elif L==3	// en
	"Hits (clicks) per action";
#elif L==4	// es
	"Accesos (clics) por acci&oacute;n";
#elif L==5	// fr
	"Visites (clics) par action";
#elif L==6	// gn
	"Accesos (clics) por acci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Visite (clic) per azione";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) dzia&lstrok;anie";
#elif L==9	// pt
	"Visitas (cliques) por a&ccedil;&atilde;o";
#elif L==10	// tr
	"Hits (clicks) per action";		// eviri lazim!
#endif

const char *Txt_Hits_per_year =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos (clics) per any";
#elif L==2	// de
	"Anmeldungen (Klicks) pro Jahr";
#elif L==3	// en
	"Hits (clicks) per year";
#elif L==4	// es
	"Accesos (clics) por a&ntilde;o";
#elif L==5	// fr
	"Visites (clics) par an";
#elif L==6	// gn
	"Accesos (clics) por a&ntilde;o";	// Okoteve traduccin
#elif L==7	// it
	"Visite (clic) all'anno";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) rocznie";
#elif L==9	// pt
	"Visitas (cliques) por ano";
#elif L==10	// tr
	"Hits (clicks) per year";		// eviri lazim!
#endif

const char *Txt_Hits_without_course_selected =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos (clics) sense assignatura seleccionada";
#elif L==2	// de
	"Anmeldungen (Klicks) ohne Kurs ausgew&auml;hlt";
#elif L==3	// en
	"Hits (clicks) without course selected";
#elif L==4	// es
	"Accesos (clics) sin asignatura seleccionada";
#elif L==5	// fr
	"Visites (clics) sans mati&egrave;re s&eacute;lectionn&eacute;e";
#elif L==6	// gn
	"Accesos (clics) sin asignatura seleccionada";	// Okoteve traduccin
#elif L==7	// it
	"Visite (scatti) senza corso selezionato";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) bez oczywi&sacute;cie wybrana";
#elif L==9	// pt
	"Visitas (cliques) sem disciplina selecionada";
#elif L==10	// tr
	"Hits (clicks) without course selected";	// eviri lazim!
#endif

const char *Txt_Holiday =
#if   L==1	// ca
	"Festivitat";
#elif L==2	// de
	"Feiertag";
#elif L==3	// en
	"Holiday";
#elif L==4	// es
	"Festividad";
#elif L==5	// fr
	"F&ecirc;te";
#elif L==6	// gn
	"Arete";
#elif L==7	// it
	"Festivit&agrave;";
#elif L==8	// pl
	"Wakacje";
#elif L==9	// pt
	"F&eacute;ria";
#elif L==10	// tr
	"Tatil";
#endif

const char *Txt_Holiday_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Festivitat <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Feiertag <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Holiday <strong>%s</strong> removed.";
#elif L==4	// es
	"Festividad <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Holiday <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Festividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Festivit&agrave; <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Wakacje <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Festividade <strong>%s</strong> eliminada.";
#elif L==10	// tr
	"Holiday <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_HOLIDAY_TYPES[Hld_NUM_TYPES_HOLIDAY] =
	{
	[Hld_HOLIDAY] =
#if   L==1	// ca
	"Dia festiu"
#elif L==2	// de
	"Urlaub"
#elif L==3	// en
	"Holiday"
#elif L==4	// es
	"D&iacute;a festivo"
#elif L==5	// fr
	"Jour f&eacute;ri&eacute;"
#elif L==6	// gn
	"Arete"
#elif L==7	// it
	"Festivit&agrave;"
#elif L==8	// pl
	"wakacje"
#elif L==9	// pt
	"Feriado"
#elif L==10	// tr
	"Tatil"
#endif
	,
	[Hld_NON_SCHOOL_PERIOD] =
#if   L==1	// ca
	"Per&iacute;ode no lectiu"
#elif L==2	// de
	"Non-Schule Zeitraum"
#elif L==3	// en
	"Non school period"
#elif L==4	// es
	"Per&iacute;odo no lectivo"
#elif L==5	// fr
	"P&eacute;riode sans &eacute;cole"
#elif L==6	// gn
	"Per&iacute;odo no lectivo"	// Okoteve traduccin
#elif L==7	// it
	"Periodo senza lezione"
#elif L==8	// pl
	"Okres nauki szkolnej dla"
#elif L==9	// pt
	"Per&iacute;odo sem escola"
#elif L==10	// tr
	"Non school period"		// eviri lazim!
#endif
	};

const char *Txt_Holidays =
#if   L==1	// ca
	"Festivitats";
#elif L==2	// de
	"Feiertage";
#elif L==3	// en
	"Holidays";
#elif L==4	// es
	"Festividades";
#elif L==5	// fr
	"Festivit&eacute;s";
#elif L==6	// gn
	"Arete";
#elif L==7	// it
	"Festivit&agrave;";
#elif L==8	// pl
	"Noclegi";
#elif L==9	// pt
	"F&eacute;rias";
#elif L==10	// tr
	"Tatil";
#endif

const char *Txt_HOLIDAYS_HELP_ORDER[Hld_NUM_ORDERS] =
	{
	[Hld_ORDER_BY_PLACE] =
#if   L==1	// ca
	"Ordenar per lloc"
#elif L==2	// de
	"Nach Standort sortieren"
#elif L==3	// en
	"Sort by place"
#elif L==4	// es
	"Ordenar por lugar"
#elif L==5	// fr
	"Trier par emplacement"
#elif L==6	// gn
	"Ordenar por lugar"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per localit&grave;"
#elif L==8	// pl
	"Sortuj wedlug miejsca"
#elif L==9	// pt
	"Classificar por localiza&ccedil;&atilde;o"
#elif L==10	// tr
	"Sort by place"		// eviri lazim!
#endif
	,
	[Hld_ORDER_BY_START_DATE] =
#if   L==1	// ca
	"Ordenar per data"
#elif L==2	// de
	"Sortieren nach Datum"
#elif L==3	// en
	"Sort by date"
#elif L==4	// es
	"Ordenar por fecha"
#elif L==5	// fr
	"Trier par date"
#elif L==6	// gn
	"Ordenar por fecha"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data"
#elif L==8	// pl
	"Sortuj wedlug daty"
#elif L==9	// pt
	"Classificar por data"
#elif L==10	// tr
	"Sort by date"		// eviri lazim!
#endif
	};

const char *Txt_HOLIDAYS_ORDER[Hld_NUM_ORDERS] =
	{
	[Hld_ORDER_BY_PLACE] =
#if   L==1	// ca
	"Lloc"
#elif L==2	// de
	"Standort"
#elif L==3	// en
	"Place"
#elif L==4	// es
	"Lugar"
#elif L==5	// fr
	"Emplacement"
#elif L==6	// gn
	"Tenda"
#elif L==7	// it
	"Luogo"
#elif L==8	// pl
	"Miejsce"
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o"
#elif L==10	// tr
	"Place"	// eviri lazim!
#endif
	,
	[Hld_ORDER_BY_START_DATE] =
#if   L==1	// ca
	"Data inicial"
#elif L==2	// de
	"Startdatum"
#elif L==3	// en
	"Start date"
#elif L==4	// es
	"Fecha inicial"
#elif L==5	// fr
	"Date initiale"
#elif L==6	// gn
	"Fecha inicial"	// Okoteve traduccin
#elif L==7	// it
	"Data iniziale"
#elif L==8	// pl
	"Data rozpoczecia"
#elif L==9	// pt
	"Data de in&iacute;cio"
#elif L==10	// tr
	"Start date"	// eviri lazim!
#endif
	};

const char *Txt_Home_PAGE =
#if   L==1	// ca
	"Inici";
#elif L==2	// de
	"Home";
#elif L==3	// en
	"Home";
#elif L==4	// es
	"Inicio";
#elif L==5	// fr
	"Accueil";
#elif L==6	// gn
	"Inicio";	// Okoteve traduccin
#elif L==7	// it
	"Home";
#elif L==8	// pl
	"Dom";
#elif L==9	// pt
	"Home";
#elif L==10	// tr
	"Home";		// eviri lazim!
#endif

const char *Txt_hour =
#if   L==1	// ca
	"hora";
#elif L==2	// de
	"Stunde";
#elif L==3	// en
	"hour";
#elif L==4	// es
	"hora";
#elif L==5	// fr
	"heure";
#elif L==6	// gn
	"aravo";
#elif L==7	// it
	"ora";
#elif L==8	// pl
	"godzina";
#elif L==9	// pt
	"hora";
#elif L==10	// tr
	"hour";	// eviri lazim!
#endif

const char *Txt_hours =
#if   L==1	// ca
	"hores";
#elif L==2	// de
	"Stunden";
#elif L==3	// en
	"hours";
#elif L==4	// es
	"horas";
#elif L==5	// fr
	"heures";
#elif L==6	// gn
	"aravo";
#elif L==7	// it
	"ore";
#elif L==8	// pl
	"godziny";
#elif L==9	// pt
	"horas";
#elif L==10	// tr
	"hours";	// eviri lazim!
#endif

const char *Txt_hours_ABBREVIATION =	// Abbreviation of "hours"
#if   L==1	// ca
	"h";
#elif L==2	// de
	"S";
#elif L==3	// en
	"h";
#elif L==4	// es
	"h";
#elif L==5	// fr
	"h";
#elif L==6	// gn
	"a";
#elif L==7	// it
	"o";
#elif L==8	// pl
	"g";
#elif L==9	// pt
	"h";
#elif L==10	// tr
	"h";	// eviri lazim!
#endif

const char *Txt_I_have_read_this_information =
#if   L==1	// ca
	"He llegit aquesta informaci&oacute;";
#elif L==2	// de
	"Ich habe diese Information lesen";
#elif L==3	// en
	"I have read this information";
#elif L==4	// es
	"He le&iacute;do esta informaci&oacute;n";
#elif L==5	// fr
	"J'ai lu cette information";
#elif L==6	// gn
	"He le&iacute;do esta informaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Ho letto questa informazione";
#elif L==8	// pl
	"Czytalem te informacje";
#elif L==9	// pt
	"Eu li esta informa&ccedil;&atilde;o";
#elif L==10	// tr
	"I have read this information";			// eviri lazim!
#endif

const char *Txt_I_understand_that_this_action_can_not_be_undone =
#if   L==1	// ca
	"Entenc que aquesta acci&oacute; no es pot desfer.";
#elif L==2	// de
	"Ich verstehe, dass diese Aktion kann nicht r&uuml;ckg&auml;ngig gemacht werden.";
#elif L==3	// en
	"I understand that this action can not be undone.";
#elif L==4	// es
	"Entiendo que esta acci&oacute;n no se puede deshacer.";
#elif L==5	// fr
	"Je comprends que cette action ne peut &ecirc;tre annul&eacute;e.";
#elif L==6	// gn
	"Entiendo que esta acci&oacute;n no se puede deshacer.";	// Okoteve traduccin
#elif L==7	// it
	"Mi rendo conto che questa azione non pu&ograve; essere annullata.";
#elif L==8	// pl
	"Rozumiem, &zdot;e dzia&lstrok;anie nie mo&zdot;na cofn&aogon;&cacute;.";
#elif L==9	// pt
	"Eu entendo que esta a&ccedil;&atilde;o n&atilde;o pode ser desfeita.";
#elif L==10	// tr
	"I understand that this action can not be undone.";		// eviri lazim!
#endif

const char *Txt_I_have_finished =
#if   L==1	// ca
	"He acabat";
#elif L==2	// de
	"Ich bin fertig";
#elif L==3	// en
	"I have finished";
#elif L==4	// es
	"He terminado";
#elif L==5	// fr
	"J'ai fini";
#elif L==6	// gn
	"He terminado";		// Okoteve traduccin
#elif L==7	// it
	"Ho finito";
#elif L==8	// pl
	"Sko&nacute;czy&lstrok;em";
#elif L==9	// pt
	"Acabei";
#elif L==10	// tr
	"I have finished";	// eviri lazim!
#endif

const char *Txt_Icons =
#if   L==1	// ca
	"Icones";
#elif L==2	// de
	"Symbole";
#elif L==3	// en
	"Icons";
#elif L==4	// es
	"Iconos";
#elif L==5	// fr
	"Ic&ocirc;nes";
#elif L==6	// gn
	"Iconos";	// Okoteve traduccin
#elif L==7	// it
	"Icone";
#elif L==8	// pl
	"Ikony";
#elif L==9	// pt
	"&Iacute;cones";
#elif L==10	// tr
	"Simgeler";
#endif

const char *Txt_ID =
#if   L==1	// ca
	"ID";
#elif L==2	// de
	"Ausweis";
#elif L==3	// en
	"ID";
#elif L==4	// es
	"ID";
#elif L==5	// fr
	"ID";
#elif L==6	// gn
	"ID";	// Okoteve traduccin
#elif L==7	// it
	"ID";
#elif L==8	// pl
	"ID";
#elif L==9	// pt
	"ID";
#elif L==10	// tr
	"Kimlik";
#endif

const char *Txt_ID_identity_number =
#if   L==1	// ca
	"ID (DNI/c&egrave;dula)";
#elif L==2	// de
	"Ausweis-Nr.";
#elif L==3	// en
	"ID (identity number/passport)";
#elif L==4	// es
	"ID (DNI/c&eacute;dula)";
#elif L==5	// fr
	"Num. d'identit&eacute;";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==7	// it
	"Numero carta d'identit&agrave;";
#elif L==8	// pl
	"Dow&oacute;d osobisty";
#elif L==9	// pt
	"N.&ordm; identif.";
#elif L==10	// tr
	"Kimlik (kimlik numaras&inodot;/pasaport)";
#endif

const char *Txt_ID_X_had_already_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El ID (DNI/c&egrave;dula) %s ja havia estat confirmat.";
#elif L==2	// de
	"ID %s hatte bereits best&auml;tigt worden.";
#elif L==3	// en
	"ID %s had already been confirmed.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) %s ya hab&iacute;a sido confirmado.";
#elif L==5	// fr
	"L'ID %s avait d&eacute;j&agrave; &eacute;t&eacute; confirm&eacute;e.";
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) %s ya hab&iacute;a sido confirmado.";	// Okoteve traduccin
#elif L==7	// it
	"L'ID %s era gi&agrave; stato confermato.";
#elif L==8	// pl
	"ID %s zosta&lstrok;a ju&zdot; potwierdzona.";
#elif L==9	// pt
	"O ID %s j&aacute; havia sido confirmada.";
#elif L==10	// tr
	"ID %s had already been confirmed.";					// eviri lazim!
#endif

const char *Txt_ID_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"ID (DNI/c&egrave;dula) %s confirmat";
#elif L==2	// de
	"ID %s best&auml;tigt";
#elif L==3	// en
	"ID %s confirmed";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) %s confirmado";
#elif L==5	// fr
	"ID %s confirm&eacute;";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) %s confirmado";		// Okoteve traduccin
#elif L==7	// it
	"ID %s confermato";
#elif L==8	// pl
	"ID %s potwierdzone";
#elif L==9	// pt
	"ID %s confirmado";
#elif L==10	// tr
	"ID %s confirmed";				// eviri lazim!
#endif

const char *Txt_ID_X_not_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"ID (DNI/c&egrave;dula) %s no confirmat";
#elif L==2	// de
	"ID %s nicht best&auml;tigt";
#elif L==3	// en
	"ID %s not confirmed";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) %s no confirmado";
#elif L==5	// fr
	"ID %s pas confirm&eacute;";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) %s no confirmado";	// Okoteve traduccin
#elif L==7	// it
	"ID %s non confermato";
#elif L==8	// pl
	"ID %s nie potwierdzone";
#elif L==9	// pt
	"ID %s n&atilde;o confirmado";
#elif L==10	// tr
	"ID %s not confirmed";				// eviri lazim!
#endif

const char *Txt_ID_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"ID (DNI/c&egrave;dula) <strong>%s</strong> eliminat.";
#elif L==2	// de
	"ID <strong>%s</strong> entfernt.";
#elif L==3	// en
	"ID <strong>%s</strong> removed.";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"ID <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"ID <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"ID <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Email <strong>%s</strong> removido.";
#elif L==10	// tr
	"ID <strong>%s</strong> removed.";				// eviri lazim!
#endif

const char *Txt_Identified_views =
#if   L==1	// ca
	"Accessos identificats";
#elif L==2	// de
	"Identifizierte Ansichten";
#elif L==3	// en
	"Identified views";
#elif L==4	// es
	"Accesos identificados";
#elif L==5	// fr
	"Vues identifi&eacute;es";
#elif L==6	// gn
	"Accesos identificados";	// Okoteve traduccin
#elif L==7	// it
	"Visualizzazioni identificati";
#elif L==8	// pl
	"Zidentyfikowane ods&lstrok;on";
#elif L==9	// pt
	"Acessos identificados";
#elif L==10	// tr
	"Identified views";		// eviri lazim!
#endif

const char *Txt_If_this_is_a_new_user_in_X_you_should_indicate_her_his_ID =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Si es tracta d'un usuari nou a %s,"
	" haureu d'indicar el seu ID (DNI/c&egrave;dula).";
#elif L==2	// de
	"Wenn dies ein neuer Benutzer in %s ist,"
	" sollten Sie seine/ihre ID angeben.";
#elif L==3	// en
	"If this is a new user in %s,"
	" you should indicate her/his ID.";
#elif L==4	// es
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"S'il s'agit d'un nouvel utilisateur dans %s,"
	" vous devez indiquer son ID.";
#elif L==6	// gn
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7	// it
	"Se questo &egrave; un nuovo utente in %s,"
	" dovresti indicare il suo ID.";
#elif L==8	// pl
	"Je&sacute;li jest to nowy u&zdot;ytkownik w %s,"
	" nale&zdot;y poda&cacute; jego ID.";
#elif L==9	// pt
	"Se este for um novo usu&aacute;rio em %s,"
	" voc&ecirc; deve indicar seu ID.";
#elif L==10	// tr
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// eviri lazim!
#endif

const char *Txt_If_you_can_not_find_your_institution_your_center_your_degree_or_your_courses_you_can_create_them =
#if   L==1	// ca
	"Si no trobes la seva instituci&oacute;, el seu center,"
	" la seva titulaci&oacute; o les seves assignatures, pot crear-los.";
#elif L==2	// de
	"Wenn Sie Ihre Institution, Ihre Zentrum, Ihr Studium"
	" oder Ihre Kurse nicht finden k&oacute;nnen, k&oacute;nnen Sie sie erstellen.";
#elif L==3	// en
	"If you can not find your institution, your center,"
	" your degree or your courses, you can create them.";
#elif L==4	// es
	"Si no encuentra su instituci&oacute;n, su centro,"
	" su titulaci&oacute;n o sus asignaturas, puede crearlos.";
#elif L==5	// fr
	"Si vous ne trouvez pas votre &eacute;tablissement, votre center,"
	" votre &eacute;tude ou de vos mati&egrave;res, vous pouvez les cr&eacute;er.";
#elif L==6	// gn
	"Si no encuentra su instituci&oacute;n, su centro,"
	" su titulaci&oacute;n o sus asignaturas, puede crearlos.";	// Okoteve traduccin
#elif L==7	// it
	"Se non riesci a trovare il vostro istituto, il centro,"
	" il vostro grado o vostri corsi, &egrave; possibile creare.";
#elif L==8	// pl
	"Je&sacute;li nie mo&zdot;esz znale&zacute;&cacute; instytucj&eogo, Centrum,"
	" studia lub kursy swoich, mo&zdot;na je tworzy&cacute;.";
#elif L==9	// pt
	"Se voc&ecirc; n&atilde;o consegue encontrar a sua institui&ccedil;&atilde;o, o seu centro,"
	" o seu grau ou as suas disciplinas, voc&ecirc; pode cri&aacute;-los.";
#elif L==10	// tr
	"If you can not find your institution, your center,"
	" your degree or your courses, you can create them.";		// eviri lazim!
#endif

const char *Txt_If_you_have_an_Android_device_try_SWADroid =
#if   L==1	// ca
	"Si teniu un dispositiu Android, provi SWADroid";
#elif L==2	// de
	"Wenn Sie ein Android-Ger&auml;t haben, versuchen SWADroid";
#elif L==3	// en
	"If you have an Android device, try SWADroid";
#elif L==4	// es
	"Si tiene un dispositivo Android, pruebe SWADroid";
#elif L==5	// fr
	"Si vous avez un appareil Android, essayer SWADroid";
#elif L==6	// gn
	"Si tiene un dispositivo Android, pruebe SWADroid";	// Okoteve traduccin
#elif L==7	// it
	"Se avete un dispositivo Android, provate SWADroid";
#elif L==8	// pl
	"Je&sacute;li masz telefon z systemem Android, spr&oacute;buj SWADroid";
#elif L==9	// pt
	"Se voc&ecirc; tiver um dispositivo Android, tente SWADroid";
#elif L==10	// tr
	"If you have an Android device, try SWADroid";		// eviri lazim!
#endif

const char *Txt_If_you_have_forgotten_your_password_ =
#if   L==1	// ca
	"Si vost&egrave; ha oblidat la seva contrasenya,"
	" se li enviar&agrave; una de nova"
	" a la vostra adre&ccedil;a de correu.";
#elif L==2	// de
	"Wenn Sie Ihr Passwort vergessen haben,"
	" wird ein neues"
	" an Ihre E-Mail-Adresse gesendet.";
#elif L==3	// en
	"If you have forgotten your password,"
	" a new one will be sent"
	" to your email address.";
#elif L==4	// es
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a su direcci&oacute;n de correo.";
#elif L==5	// fr
	"Si vous avez oubli&eacute; votre mot de passe,"
	" un nouveau mot de passe vous sera envoy&eacute;"
	" &agrave; votre adresse e-mail.";
#elif L==6	// gn
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Se hai dimenticato la tua password,"
	" ne verr&agrave; inviata una nuova"
	" al tuo indirizzo email.";
#elif L==8	// pl
	"Je&sacute;li zapomnia&lstrok;e&sacute; has&lstrok;a,"
	" nowe zostanie wys&lstrok;ane"
	" na Tw&oacute;j adres e-mail.";
#elif L==9	// pt
	"Se voc&ecirc; tiver esquecido sua senha,"
	" uma nova ser&aacute; enviada"
	" para o seu endere&ccedil;o de email.";
#elif L==10	// tr
	"If you have forgotten your password,"
	" a new one will be sent"
	" to your email address.";		// eviri lazim!
#endif

const char *Txt_If_you_have_written_your_ID_nickname_or_email_correctly_ =
#if   L==1	// ca
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";	// Necessita traducci
#elif L==2	// de
	"Wenn Sie Ihre @Benutzername, Email oder Ausweis-Nr.,"
	" korrekt geschrieben wurden, und die E-Mail-Adresse auf Ihren Profil korrekt ist,"
	" erhalten Sie eine E-Mail mit einem neuen Passwort.";
#elif L==3	// en
	"If you have written your @nick, email or ID correctly,"
	" and the email address on your record is correct,"
	" you will receive an email with a new password.";
#elif L==4	// es
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";
#elif L==5	// fr
	"Si vous avez &eacute;crit votre @surnom, courrier ou num&eacute;ro d'identit&eacute; correctement,"
	" ainsi que l'adresse email sur votre carte est correct,"
	" vous recevrez un email avec un nouveau mot de passe.";
#elif L==6	// gn
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";	// Okoteve traduccin
#elif L==7	// it
	"Se hai scritto il tuo nome utente, email o numero di carta d'identit&agrave; correttamente,"
	" e l'indirizzo email della tua scheda &egrave; corretto,"
	" riceverai una email con una nuova password.";
#elif L==8	// pl
	"Jesli napisales swoja nazwe lub nick poprawnie,"
	" a adres email na zapis jest poprawny,"
	" otrzymasz maila z nowym haslem.";
#elif L==9	// pt
	"Se voc&ecirc; ter escrito o suo alcunha, email ou n&ordm; identif. corretamente,"
	" bem como o endere&ccedil;o de email em seu cart&atilde;o est&aacute; correto,"
	" voc&ecirc; receber&aacute; um email com uma nova senha.";
#elif L==10	// tr
	"If you have written your @nick, email or ID correctly,"
	" and the email address on your record is correct,"
	" you will receive an email with a new password.";		// eviri lazim!
#endif

const char *Txt_If_you_think_you_may_have_been_registered_ =
#if   L==1	// ca
	"Si creu que pot haver estat inscrit/a"
	" per un professor o administrador,"
	" comprovi escrivint algun dels seus possibles ID"
	" (DNI amb lletra final o sense, passaport, etc.).";
#elif L==2	// de
	"Wenn Sie glauben, dass Sie"
	" von einem Lehrer oder Administrator registriert worden sind,"
	" &uuml;berpr&uuml;fen Sie ihn, indem Sie eine Ihrer m&ouml;glichen IDs"
	" (ID, Reisepass usw.) schreiben.";
#elif L==3	// en
	"If you think you may have been registered"
	" by a teacher or administrator,"
	" check it by writing one of your possible IDs (ID, passport, etc.).";
#elif L==4	// es
	"Si cree que puede haber sido inscrito/a"
	" por un profesor o administrador,"
	" compru&eacute;belo escribiendo alguno de sus posibles ID"
	" (DNI con letra final o sin ella, c&eacute;dula, pasaporte, etc.).";
#elif L==5	// fr
	"Si vous pensez que vous avez &eacute;t&eacute; enregistr&eacute;"
	" par un enseignant ou un administrateur,"
	" v&eacute;rifiez-le en &eacute;crivant l'un de vos identifiants possibles"
	" (carte d'identit&eacute;, passeport, etc.).";
#elif L==6	// gn
	"Si cree que puede haber sido inscrito/a"
	" por un profesor o administrador,"
	" compru&eacute;belo escribiendo alguno de sus posibles ID"
	" (DNI con letra final o sin ella, c&eacute;dula, pasaporte, etc.).";	// Okoteve traduccin
#elif L==7	// it
	"Se pensi di essere stato registrato"
	" da un insegnante o da un amministratore,"
	" controllarlo scrivendo uno dei tuoi possibili ID"
	" (ID, passaporto, ecc.).";
#elif L==8	// pl
	"Je&sacute;li uwa&zdot;asz, &zdot;e mo&zdot;esz zosta&cacute; zarejestrowany"
	" przez nauczyciela lub administratora,"
	" sprawd&zacute; go, wpisuj&aogon;c jeden z mo&zdot;liwych ID"
	" (dowodu to&zdot;samo&sacute;ci, paszportu itp.).";
#elif L==9	// pt
	"Se voc&ecirc; acha que pode ter sido registrado"
	" por um professor ou administrador,"
	" verifique-o escrevendo uma das suas poss&iacute;veis IDs"
	" (identifica&ccedil;&atilde;o, passaporte, etc.).";
#elif L==10	// tr
	"Bir &ouml;&gbreve;retmen veya y&ouml;netici taraf&inodot;ndan"
	" kay&inodot;tl&inodot; olabilece&gbreve;inizi d&uuml;&scedil;&uuml;n&uuml;yorsan&inodot;z,"
	" olas&inodot; kimliklerinizden (kimlik, pasaport vb.)"
	" birini yazarak kontrol edin.";
#endif

const char *Txt_Image =
#if   L==1	// ca
	"Imatge";
#elif L==2	// de
	"Abbild";
#elif L==3	// en
	"Image";
#elif L==4	// es
	"Imagen";
#elif L==5	// fr
	"Image";
#elif L==6	// gn
	"Ta'&atilde;nga";
#elif L==7	// it
	"Immagine";
#elif L==8	// pl
	"Obraz";
#elif L==9	// pt
	"Imagem";
#elif L==10	// tr
	"Image";	// eviri lazim!
#endif

const char *Txt_Image_video =
#if   L==1	// ca
	"Imatge / v&iacute;deo";
#elif L==2	// de
	"Abbild / Video";
#elif L==3	// en
	"Image / video";
#elif L==4	// es
	"Imagen / v&iacute;deo";
#elif L==5	// fr
	"Image / vid&eacute;o";
#elif L==6	// gn
	"Ta'&atilde;nga / video";
#elif L==7	// it
	"Immagine / video";
#elif L==8	// pl
	"Obraz / wideo";
#elif L==9	// pt
	"Imagem / v&iacute;deo";
#elif L==10	// tr
	"Image / video";	// eviri lazim!
#endif

const char *Txt_Import_questions =
#if   L==1	// ca
	"Importa preguntes";
#elif L==2	// de
	"Fragen importieren";
#elif L==3	// en
	"Import questions";
#elif L==4	// es
	"Importar preguntas";
#elif L==5	// fr
	"Importer questions";
#elif L==6	// gn
	"Importar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Importa domande";
#elif L==8	// pl
	"Importuj pytania";
#elif L==9	// pt
	"Importar perguntas";
#elif L==10	// tr
	"Import questions";	// eviri lazim!
#endif

const char *Txt_Imported_questions =
#if   L==1	// ca
	"Preguntes importades";
#elif L==2	// de
	"Importierte Fragen";
#elif L==3	// en
	"Imported questions";
#elif L==4	// es
	"Preguntas importadas";
#elif L==5	// fr
	"Questions import&eacute;es";
#elif L==6	// gn
	"Preguntas importadas";	// Okoteve traduccin
#elif L==7	// it
	"Domande importate";
#elif L==8	// pl
	"Zaimportowane pytania";
#elif L==9	// pt
	"Perguntas importadas";
#elif L==10	// tr
	"Imported questions";	// eviri lazim!
#endif

const char *Txt_SEARCH_X_in_Y =	// Search string "in" documentos, users, etc.
#if   L==1	// ca
	"en";
#elif L==2	// de
	"im";
#elif L==3	// en
	"in";
#elif L==4	// es
	"en";
#elif L==5	// fr
	"dans";
#elif L==6	// gn
	"en";	// Okoteve traduccin
#elif L==7	// it
	"in";
#elif L==8	// pl
	"w";
#elif L==9	// pt
	"em";
#elif L==10	// tr
	"in";	// eviri lazim!
#endif

const char *Txt_In_a_type_of_group_with_single_enrolment_students_can_not_be_registered_in_more_than_one_group =
#if   L==1	// ca
	"En un tipus de grup amb matr&iacute;cula &uacute;nica,"
	" els alumnes no poden estar matriculats en m&eacute;s d'un grup.";
#elif L==2	// de
	"Bei einem Gruppentyp mit Einzelanmeldung k&ouml;nnen"
	" Studierende nicht in mehr als einer Gruppe angemeldet werden.";
#elif L==3	// en
	"In a type of group with single enrolment,"
	" students can not be registered in more than one group.";
#elif L==4	// es
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";
#elif L==5	// fr
	"Dans un type de groupe &agrave; inscription unique,"
	" les &eacute;tudiants ne peuvent pas &ecirc;tre inscrits dans plus d'un groupe.";
#elif L==6	// gn
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";	// Okoteve traduccin
#elif L==7	// it
	"In un tipo di gruppo a singola registrazione,"
	" non puoi registrare studenti in pi&ugrave; di un gruppo.";
#elif L==8	// pl
	"W rodzaju grupy z jednej rejestracji,"
	" nie mozesz rejestrowac uczni&oacute;w w wiecej niz jednej grupy.";
#elif L==9	// pt
	"Em um tipo de grupo com inscri&ccedil;&atilde;o &uacute;nica,"
	" os estudantes n&atilde;o podem ser registrados em mais de um grupo.";
#elif L==10	// tr
	"In a type of group with single enrolment,"
	" students can not be registered in more than one group.";	// eviri lazim!
#endif

const char *Txt_In_the_year_X_of_the_degree_Y_already_existed_a_course_with_the_name_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"L'any/semestre <strong>%s</strong> de la titulaci&oacute; <strong>%s</strong>"
	" ja existia una assignatura amb el nom <strong>%s</strong>.";
#elif L==2	// de
	"Im <strong>%s</strong> Studienjahr/Semester des Studiengangs <strong>%s</strong>"
	" existierte bereits ein Studiengang mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";
#elif L==4	// es
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"Dans l'ann&eacute;e/semestre <strong>%s</strong> de l'&eacute;tude <strong>%s</strong>"
	" existait d&eacute;j&agrave; une mati&egrave;re avec le nom <strong>%s</strong>.";
#elif L==6	// gn
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Nell'anno/semester <strong>%s</strong> di laurea <strong>%s</strong>"
	" gi&agrave; esiste un corso con il nome <strong>%s</strong>.";
#elif L==8	// pl
	"W roku/semestrze <strong>%s</strong> stopnia <strong>%s</strong>"
	" istnia&lstrok; ju&zdot; kierunek o nazwie <strong>%s</strong>.";
#elif L==9	// pt
	"No ano/semestre <strong>%s</strong> do grau <strong>%s</strong>"
	" j&aacute; existia uma disciplina com o nome <strong>%s</strong>.";
#elif L==10	// tr
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_Incomplete_timetable_for_lack_of_space =
#if   L==1	// ca
	"Horari incomplet per falta d'espai.";
#elif L==2	// de
	"Unvollst&auml;ndige Studenplan f&uuml;r Platzmangel.";
#elif L==3	// en
	"Incomplete timetable for lack of space.";
#elif L==4	// es
	"Horario incompleto por falta de espacio.";
#elif L==5	// fr
	"Horaire incompl&egrave;te par manque de place.";
#elif L==6	// gn
	"Horario incompleto por falta de espacio.";	// Okoteve traduccin
#elif L==7	// it
	"Orario incompleto per mancanza di spazio.";
#elif L==8	// pl
	"Niekompletne harmonogram brak miejsca.";
#elif L==9	// pt
	"Hor&aacute;rio incompleto por falta de espa&ccedil;o.";
#elif L==10	// tr
	"Incomplete timetable for lack of space.";	// eviri lazim!
#endif

const char *Txt_Indicators =
#if   L==1	// ca
	"Indicadors";
#elif L==2	// de
	"Indikatoren";
#elif L==3	// en
	"Indicators";
#elif L==4	// es
	"Indicadores";
#elif L==5	// fr
	"Indicateurs";
#elif L==6	// gn
	"Indicadores";	// Okoteve traduccin
#elif L==7	// it
	"Indicatori";
#elif L==8	// pl
	"Wska&zacute;niki";
#elif L==9	// pt
	"Indicadores";
#elif L==10	// tr
	"G&ouml;stergeler";
#endif

const char *Txt_Indicators_of_courses =
#if   L==1	// ca
	"Indicadors d'assignatures";
#elif L==2	// de
	"Anzeigen von Kursen";
#elif L==3	// en
	"Indicators of courses";
#elif L==4	// es
	"Indicadores de asignaturas";
#elif L==5	// fr
	"Indicateurs de mati&egrave;res";
#elif L==6	// gn
	"Umi mba'e ohechauk&aacute;va tema rehegua";
#elif L==7	// it
	"Indicatori di corsi";
#elif L==8	// pl
	"Wska&zacute;niki kurs&oacute;w";
#elif L==9	// pt
	"Indicadores de disciplinas";
#elif L==10	// tr
	"Kurs g&ouml;stergeleri";
#endif

const char *Txt_Info =
#if   L==1	// ca
	"Info";
#elif L==2	// de
	"Info";
#elif L==3	// en
	"Info";
#elif L==4	// es
	"Info";
#elif L==5	// fr
	"Info";
#elif L==6	// gn
	"Info";	// Okoteve traduccin
#elif L==7	// it
	"Informazione";
#elif L==8	// pl
	"Info";
#elif L==9	// pt
	"Info";
#elif L==10	// tr
	"Info";	// eviri lazim!
#endif

const char *Txt_INFO_TITLE[Inf_NUM_TYPES] =
	{
	[Inf_INFORMATION] =
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	[Inf_TEACHING_GUIDE] =
#if   L==1	// ca
	"Guia docent"
#elif L==2	// de
	"Hinweise zur Kurs"
#elif L==3	// en
	"Teaching guide"
#elif L==4	// es
	"Gu&iacute;a docente"
#elif L==5	// fr
	"Guide p&eacute;dagogique"
#elif L==6	// gn
	"Gu&iacute;a docente"	// Okoteve traduccin
#elif L==7	// it
	"Guida didattica"
#elif L==8	// pl
	"Nauczanie przypomnienie"
#elif L==9	// pt
	"Guia de ensino"
#elif L==10	// tr
	"Teaching guide"	// eviri lazim!
#endif
	,
	[Inf_LECTURES] =
#if   L==1	// ca
	"Temari"
#elif L==2	// de
	"Studienplan"
#elif L==3	// en
	"Topics"
#elif L==4	// es
	"Temario"
#elif L==5	// fr
	"Plan"
#elif L==6	// gn
	"Temario"	// Okoteve traduccin
#elif L==7	// it
	"Temi"
#elif L==8	// pl
	"Sylabus"
#elif L==9	// pt
	"Conte&uacute;do"
#elif L==10	// tr
	"Konular"
#endif
	,
	[Inf_PRACTICALS] =
#if   L==1	// ca
	"Temari"
#elif L==2	// de
	"Studienplan"
#elif L==3	// en
	"Topics"
#elif L==4	// es
	"Temario"
#elif L==5	// fr
	"Plan"
#elif L==6	// gn
	"Temario"	// Okoteve traduccin
#elif L==7	// it
	"Temi"
#elif L==8	// pl
	"Sylabus"
#elif L==9	// pt
	"Conte&uacute;do"
#elif L==10	// tr
	"Konular"
#endif
	,
	[Inf_BIBLIOGRAPHY] =
#if   L==1	// ca
	"Bibliografia"
#elif L==2	// de
	"Literatur"
#elif L==3	// en
	"Bibliography"
#elif L==4	// es
	"Bibliograf&iacute;a"
#elif L==5	// fr
	"Bibliographie"
#elif L==6	// gn
	"Arandukapurupyre"
#elif L==7	// it
	"Bibliografia"
#elif L==8	// pl
	"Bibliografia"
#elif L==9	// pt
	"Bibliografia"
#elif L==10	// tr
	"Bibliyografya"
#endif
	,
	[Inf_FAQ] =
#if   L==1	// ca
	"FAQ"
#elif L==2	// de
	"FAQ"
#elif L==3	// en
	"FAQ"
#elif L==4	// es
	"FAQ"
#elif L==5	// fr
	"FAQ"
#elif L==6	// gn
	"FAQ"	// Okoteve traduccin
#elif L==7	// it
	"FAQ"
#elif L==8	// pl
	"FAQ"
#elif L==9	// pt
	"FAQ"
#elif L==10	// tr
	"SSS"
#endif
	,
	[Inf_LINKS] =
#if   L==1	// ca
	"Enlla&ccedil;os"
#elif L==2	// de
	"Links"
#elif L==3	// en
	"Links"
#elif L==4	// es
	"Enlaces"
#elif L==5	// fr
	"Liens"
#elif L==6	// gn
	"Ambu&eacute;re joaju"
#elif L==7	// it
	"Link"
#elif L==8	// pl
	"Linki"
#elif L==9	// pt
	"Liga&ccedil;&otilde;es"
#elif L==10	// tr
	"Ba&gbreve;lant&inodot;lar"
#endif
	,
	[Inf_ASSESSMENT] =
#if   L==1	// ca
	"Sistema d'avaluaci&oacute;"
#elif L==2	// de
	"Bewertungssystem"
#elif L==3	// en
	"Assessment system"
#elif L==4	// es
	"Sistema de evaluaci&oacute;n"
#elif L==5	// fr
	"Syst&egrave;me d'&eacute;valuation"
#elif L==6	// gn
	"Sistema de evaluaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Sistema di valutazione"
#elif L==8	// pl
	"System oceniania"
#elif L==9	// pt
	"Sistema de avalia&ccedil;&atilde;o"
#elif L==10	// tr
	"Assessment system"		// eviri lazim!
#endif
	};

const char *Txt_INFO_SRC_FULL_TEXT[Inf_NUM_SOURCES] =
	{
	[Inf_NONE] =
#if   L==1	// ca
	"No oferir cap informaci&oacute;"
#elif L==2	// de
	"Geben keine Auskunft"
#elif L==3	// en
	"Do not give any information"
#elif L==4	// es
	"No ofrecer ninguna informaci&oacute;n"
#elif L==5	// fr
	"Ne pas donner d'information"
#elif L==6	// gn
	"No ofrecer ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Non dare alcuna informazione"
#elif L==8	// pl
	"Nie podawaj zadnych informacji"
#elif L==9	// pt
	"N&atilde;o d&ecirc; nenhuma informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Do not give any information"		// eviri lazim!
#endif
	,
	[Inf_EDITOR] =
#if   L==1	// ca
	"Editor intern"
#elif L==2	// de
	"Interner Editor"
#elif L==3	// en
	"Internal editor"
#elif L==4	// es
	"Editor interno"
#elif L==5	// fr
	"&Eacute;diteur interne"
#elif L==6	// gn
	"Editor interno"	// Okoteve traduccin
#elif L==7	// it
	"Editor interno"
#elif L==8	// pl
	"Wewnetrzny edytor"
#elif L==9	// pt
	"Interno editor"
#elif L==10	// tr
	"Internal editor"	// eviri lazim!
#endif
	,
	[Inf_PLAIN_TEXT] =
#if   L==1	// ca
	"Text sense format"
#elif L==2	// de
	"Plain Text"
#elif L==3	// en
	"Plain text"
#elif L==4	// es
	"Texto sin formato"
#elif L==5	// fr
	"Texte brut"
#elif L==6	// gn
	"Texto sin formato"	// Okoteve traduccin
#elif L==7	// it
	"Testo normale"
#elif L==8	// pl
	"Zwykly tekst"
#elif L==9	// pt
	"Texto plano"
#elif L==10	// tr
	"Plain text"		// eviri lazim!
#endif
	,
	[Inf_RICH_TEXT] =
#if   L==1	// ca
	"Text enriquit"
#elif L==2	// de
	"Rich Text"
#elif L==3	// en
	"Rich text"
#elif L==4	// es
	"Texto enriquecido"
#elif L==5	// fr
	"Texte enrichi"
#elif L==6	// gn
	"Texto enriquecido"	// Okoteve traduccin
#elif L==7	// it
	"Testo formattato"
#elif L==8	// pl
	"Tekst sformatowany"
#elif L==9	// pt
	"Texto enriquecido"
#elif L==10	// tr
	"Rich text"		// eviri lazim!
#endif
	,
	[Inf_PAGE] =
#if   L==1	// ca
	"P&agrave;gina web allotjada aqu&iacute;"
#elif L==2	// de
	"Webseite gehostet hier"
#elif L==3	// en
	"Web page hosted here"
#elif L==4	// es
	"P&aacute;gina web alojada aqu&iacute;"
#elif L==5	// fr
	"Page web h&eacute;berg&eacute;e ici"
#elif L==6	// gn
	"P&aacute;gina web alojada aqu&iacute;"	// Okoteve traduccin
#elif L==7	// it
	"Pagina web ospitata qui"
#elif L==8	// pl
	"Strona internetowa hostowana tutaj"
#elif L==9	// pt
	"P&aacute;gina Web hospedada aqui"
#elif L==10	// tr
	"Web page hosted here"			// eviri lazim!
#endif
	,
	[Inf_URL] =
#if   L==1	// ca
	"P&agrave;gina web allotjada a un altre servidor"
#elif L==2	// de
	"Webseite auf einem anderen Server gehostet"
#elif L==3	// en
	"Web page hosted on another server"
#elif L==4	// es
	"P&aacute;gina web alojada en otro servidor"
#elif L==5	// fr
	"Page Web h&eacute;berg&eacute;e sur un autre serveur"
#elif L==6	// gn
	"P&aacute;gina web alojada en otro servidor"	// Okoteve traduccin
#elif L==7	// it
	"Pagina Web ospitata su un altro server"
#elif L==8	// pl
	"Strona internetowa umieszczona na innym serwerze"
#elif L==9	// pt
	"P&aacute;gina Web hospedada em outro servidor"
#elif L==10	// tr
	"Web page hosted on another server"		// eviri lazim!
#endif
	};

const char *Txt_INFO_SRC_HELP[Inf_NUM_SOURCES] =
	{
	[Inf_NONE] =
	NULL,
	[Inf_EDITOR] =
#if   L==1	// ca
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"	// Necessita traducci
#elif L==2	// de
	"Interner Editor, nur f&uuml;r Studienplan (Vorlesung und &Uuml;bungen)"
#elif L==3	// en
	"internal editor, only for syllabus (lectures and practicals)"
#elif L==4	// es
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"
#elif L==5	// fr
	"&eacute;diteur interne, que pour les plans (th&eacute;orie et pratique)"
#elif L==6	// gn
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"	// Okoteve traduccin
#elif L==7	// it
	"editor interno, solo per programmi di teoria e di pratica"
#elif L==8	// pl
	"Wewnetrzny edytor, tylko program (wyklady i practicals)"
#elif L==9	// pt
	"interno editor, apenas para programas de teoria y pr&aacute;tica"
#elif L==10	// tr
	"internal editor, only for syllabus (lectures and practicals)"			// eviri lazim!
#endif
	,
	[Inf_PLAIN_TEXT] =
#if   L==1	// ca
	"editor de text sense format"
#elif L==2	// de
	"Plain Text Editor"
#elif L==3	// en
	"plain text editor"
#elif L==4	// es
	"editor de texto sin formato"
#elif L==5	// fr
	"&eacute;diteur de texte simple"
#elif L==6	// gn
	"editor de texto sin formato"	// Okoteve traduccin
#elif L==7	// it
	"editor di testo normale"
#elif L==8	// pl
	"edytor tekstu zwyk&lstrok;y"
#elif L==9	// pt
	"editor de texto simples"
#elif L==10	// tr
	"plain text editor"		// eviri lazim!
#endif
	,
	[Inf_RICH_TEXT] =
#if   L==1	// ca
	"editor de text enriquit"
#elif L==2	// de
	"Rich Text Editor"
#elif L==3	// en
	"rich text editor"
#elif L==4	// es
	"editor de texto enriquecido"
#elif L==5	// fr
	"&eacute;diteur de texte enrichi"
#elif L==6	// gn
	"editor de texto enriquecido"	// Okoteve traduccin
#elif L==7	// it
	"editor di testo formattato"
#elif L==8	// pl
	"edytor tekstu sformatowany"
#elif L==9	// pt
	"editor de texto rico"
#elif L==10	// tr
	"rich text editor"		// eviri lazim!
#endif
	,
	[Inf_PAGE] =
#if   L==1	// ca
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Necessita traducci
#elif L==2	// de
	"senden Sie eine HTML-Datei, oder als ZIP-Datei mit einer Datei index.html / index.htm und Bild-Dateien"
#elif L==3	// en
	"send an HTML file, or a ZIP file containing a file index.html / index.htm and image files"
#elif L==4	// es
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"
#elif L==5	// fr
	"envoyer un fichier HTML, ou un fichier ZIP contenant un fichier index.html / index.htm et les fichiers d'images"
#elif L==6	// gn
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Okoteve traduccin
#elif L==7	// it
	"inviare un file HTML, o un file ZIP contenente un file index.html / index.htm e file di immagine"
#elif L==8	// pl
	"wyslac plik HTML lub plik ZIP zawierajacy plik index.html / index.htm i pliki graficzne"
#elif L==9	// pt
	"enviar um arquivo HTML, ou um arquivo ZIP que cont&eacute;m um arquivo index.html / index.htm e limas da imagem"
#elif L==10	// tr
	"send an HTML file, or a ZIP file containing a file index.html / index.htm and image files"					// eviri lazim!
#endif
	,
	[Inf_URL] =
#if   L==1	// ca
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Necessita traducci
#elif L==2	// de
	"Geben Sie die Adresse einer Web-Seite geleitet werden, der sich auf einem anderen Server"
#elif L==3	// en
	"enter the address of a web page that is hosted on another server"
#elif L==4	// es
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"
#elif L==5	// fr
	"entrer l'adresse d'une page Web qui est h&eacute;berg&eacute;e sur un autre serveur"
#elif L==6	// gn
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Okoteve traduccin
#elif L==7	// it
	"immettere l'indirizzo di una pagina web che &egrave; ospitata su un altro server"
#elif L==8	// pl
	"wprowadzic adres strony internetowej, kt&oacute;ra znajduje sie na innym serwerze"
#elif L==9	// pt
	"introduza o endere&ccedil;o de uma p&aacute;gina Web que est&aacute; hospedada em outro servidor"
#elif L==10	// tr
	"enter the address of a web page that is hosted on another server"					// eviri lazim!
#endif
	};

const char *Txt_INFO_SRC_SHORT_TEXT[Inf_NUM_SOURCES] =
	{
	[Inf_NONE] =
#if   L==1	// ca
	"Cap informaci&oacute;"
#elif L==2	// de
	"Keine Angaben"
#elif L==3	// en
	"No information"
#elif L==4	// es
	"Ninguna informaci&oacute;n"
#elif L==5	// fr
	"Aucune information"
#elif L==6	// gn
	"Ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Nessuna informazione"
#elif L==8	// pl
	"Brak informacji"
#elif L==9	// pt
	"Sem informa&ccedil;&atilde;o"
#elif L==10	// tr
	"No information"		// eviri lazim!
#endif
	,
	[Inf_EDITOR] =
#if   L==1	// ca
	"Editor intern"
#elif L==2	// de
	"Interner Editor"
#elif L==3	// en
	"Internal editor"
#elif L==4	// es
	"Editor interno"
#elif L==5	// fr
	"&Eacute;diteur interne"
#elif L==6	// gn
	"Editor interno"	// Okoteve traduccin
#elif L==7	// it
	"Editor interno"
#elif L==8	// pl
	"Wewnetrzny edytor"
#elif L==9	// pt
	"Interno editor"
#elif L==10	// tr
	"Internal editor"	// eviri lazim!
#endif
	,
	[Inf_PLAIN_TEXT] =
#if   L==1	// ca
	"Text sense format"
#elif L==2	// de
	"Plain Text"
#elif L==3	// en
	"Plain text"
#elif L==4	// es
	"Texto sin formato"
#elif L==5	// fr
	"Plain text"
#elif L==6	// gn
	"Texto sin formato"	// Okoteve traduccin
#elif L==7	// it
	"Testo normale"
#elif L==8	// pl
	"Zwykly tekst"
#elif L==9	// pt
	"Texto plano"
#elif L==10	// tr
	"Plain text"		// eviri lazim!
#endif
	,
	[Inf_RICH_TEXT] =
#if   L==1	// ca
	"Text enriquit"
#elif L==2	// de
	"Rich Text"
#elif L==3	// en
	"Rich text"
#elif L==4	// es
	"Texto enriquecido"
#elif L==5	// fr
	"Texte enrichi"
#elif L==6	// gn
	"Texto enriquecido"	// Okoteve traduccin
#elif L==7	// it
	"Testo formattato"
#elif L==8	// pl
	"Tekst sformatowany"
#elif L==9	// pt
	"Texto enriquecido"
#elif L==10	// tr
	"Rich text"		// eviri lazim!
#endif
	,
	[Inf_PAGE] =
#if   L==1	// ca
	"P&agrave;gina interna"
#elif L==2	// de
	"Interne Webseite"
#elif L==3	// en
	"Internal page"
#elif L==4	// es
	"P&aacute;gina interna"
#elif L==5	// fr
	"Page interne"
#elif L==6	// gn
	"P&aacute;gina interna"	// Okoteve traduccin
#elif L==7	// it
	"Pagina interna"
#elif L==8	// pl
	"Strona wewnetrzna"
#elif L==9	// pt
	"Interna p&aacute;gina"
#elif L==10	// tr
	"Internal page"		// eviri lazim!
#endif
	,
	[Inf_URL] =
#if   L==1	// ca
	"P&agrave;gina externa"
#elif L==2	// de
	"Externe Webseite"
#elif L==3	// en
	"External page"
#elif L==4	// es
	"P&aacute;gina externa"
#elif L==5	// fr
	"Page externe"
#elif L==6	// gn
	"P&aacute;gina externa"	// Okoteve traduccin
#elif L==7	// it
	"Pagina esterna"
#elif L==8	// pl
	"Strona zewnetrzna"
#elif L==9	// pt
	"Externa p&aacute;gina"
#elif L==10	// tr
	"External page"		// eviri lazim!
#endif
	};

const char *Txt_Informants =
#if   L==1	// ca
	"Informadors";
#elif L==2	// de
	"Informanten";
#elif L==3	// en
	"Informants";
#elif L==4	// es
	"Informantes";
#elif L==5	// fr
	"Informateurs";
#elif L==6	// gn
	"Informantes";	// Okoteve traduccin
#elif L==7	// it
	"Informatori";
#elif L==8	// pl
	"informator&oacute;w";
#elif L==9	// pt
	"Informantes";
#elif L==10	// tr
	"Informants";	// eviri lazim!
#endif

const char *Txt_Inline_math =
#if   L==1	// ca
	"F&oacute;rmula";
#elif L==2	// de
	"Inline Mathematik";
#elif L==3	// en
	"Inline math";
#elif L==4	// es
	"F&oacute;rmula";
#elif L==5	// fr
	"Formule";
#elif L==6	// gn
	"F&oacute;rmula";	// Okoteve traduccin
#elif L==7	// it
	"Matematica inline";
#elif L==8	// pl
	"Jazda matematyki";
#elif L==9	// pt
	"F&oacute;rmula";
#elif L==10	// tr
	"Inline math";		// eviri lazim!
#endif

const char *Txt_Institution_center_and_department =
#if   L==1	// ca
	"Instituci&oacute;, center i departament";
#elif L==2	// de
	"Hochschule, Lehrinstitut und Abteilung";
#elif L==3	// en
	"Institution, center and department";
#elif L==4	// es
	"Instituci&oacute;n, centro y departamento";
#elif L==5	// fr
	"&Eacute;tablissement, center et d&eacute;partement";
#elif L==6	// gn
	"Mbo'ehao, mbo'ehao, peh&#7869;";
#elif L==7	// it
	"Istituzione, centro e dipartimento";
#elif L==8	// pl
	"Instytucji, centrum i wydzia&lstrok;";
#elif L==9	// pt
	"Institu&ccedil;&atilde;o, centro e departamento";
#elif L==10	// tr
	"Institution, center and department";	// eviri lazim!
#endif

const char *Txt_Institution_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Instituci&oacute; <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Hochschule <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Institution <strong>%s</strong> removed.";
#elif L==4	// es
	"Instituci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"&Eacute;tablissement <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Instituci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Istituzione <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Instytucja <strong>%s</strong> usunieta.";
#elif L==9	// pt
	"Institui&ccedil;&atilde;o <strong>%s</strong> removida.";
#elif L==10	// tr
	"Institution <strong>%s</strong> removed.";		// eviri lazim!
#endif


const char *Txt_INSTITUTION_STATUS[Hie_NUM_STATUS_TXT] =
	{
	[Hie_STATUS_UNKNOWN] =
#if   L==1	// ca
	"Desconegudo"
#elif L==2	// de
	"Unbekannt"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Hie_STATUS_ACTIVE] =
#if   L==1	// ca
	"Activa"
#elif L==2	// de
	"Aktive"
#elif L==3	// en
	"Active"
#elif L==4	// es
	"Activa"
#elif L==5	// fr
	"Actif"
#elif L==6	// gn
	"Activa"	// Okoteve traduccin
#elif L==7	// it
	"Attiva"
#elif L==8	// pl
	"Aktywne"
#elif L==9	// pt
	"Ativa"
#elif L==10	// tr
	"Active"	// eviri lazim!
#endif
	,
	[Hie_STATUS_PENDING] =
#if   L==1	// ca
	"Nova"
#elif L==2	// de
	"Neu"
#elif L==3	// en
	"New"
#elif L==4	// es
	"Nueva"
#elif L==5	// fr
	"Nouvelle"
#elif L==6	// gn
	"Pyahu"
#elif L==7	// it
	"Nuovo"
#elif L==8	// pl
	"Nowy"
#elif L==9	// pt
	"Nova"
#elif L==10	// tr
	"New"	// eviri lazim!
#endif
	,
	[Hie_STATUS_REMOVED] =
#if   L==1	// ca
	"Eliminada"
#elif L==2	// de
	"Entfernt"
#elif L==3	// en
	"Removed"
#elif L==4	// es
	"Eliminada"
#elif L==5	// fr
	"Supprim&eacute;"
#elif L==6	// gn
	"Eliminada"	// Okoteve traduccin
#elif L==7	// it
	"Rimossa"
#elif L==8	// pl
	"Usuniete"
#elif L==9	// pt
	"Suprimida"
#elif L==10	// tr
	"Removed"	// eviri lazim!
#endif
	};

const char *Txt_Institutional_code =
#if   L==1	// ca
	"Codi institucional";
#elif L==2	// de
	"Institutionelle Code";
#elif L==3	// en
	"Institutional code";
#elif L==4	// es
	"C&oacute;digo institucional";
#elif L==5	// fr
	"Code institutionnel";
#elif L==6	// gn
	"C&oacute;digo institucional";	// Okoteve traduccin
#elif L==7	// it
	"Codice istituzionale";
#elif L==8	// pl
	"Kod instytucjonalna";
#elif L==9	// pt
	"C&oacute;digo institutional";
#elif L==10	// tr
	"Institutional code";		// eviri lazim!
#endif

const char *Txt_Institutional_BR_code =
#if   L==1	// ca
	"Codi<br />instituc.";
#elif L==2	// de
	"Institutionelle<br />Code";
#elif L==3	// en
	"Institut.<br />code";
#elif L==4	// es
	"C&oacute;digo<br />instituc.";
#elif L==5	// fr
	"Code<br />institut.";
#elif L==6	// gn
	"C&oacute;digo<br />instituc.";	// Okoteve traduccin
#elif L==7	// it
	"Codice<br />istituz.";
#elif L==8	// pl
	"Kod<br />instytuc.";
#elif L==9	// pt
	"C&oacute;digo<br />instituc.";
#elif L==10	// tr
	"Institut.<br />code";		// eviri lazim!
#endif

const char *Txt_Institutions_ABBREVIATION =
#if   L==1	// ca
	"Instit.";
#elif L==2	// de
	"Hochsc.";
#elif L==3	// en
	"Instit.";
#elif L==4	// es
	"Instit.";
#elif L==5	// fr
	"&Eacute;tabl.";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Istit.";
#elif L==8	// pl
	"Instyt.";
#elif L==9	// pt
	"Instit.";
#elif L==10	// tr
	"Instit.";	// eviri lazim!
#endif

const char *Txt_INSTITUTIONS_HELP_ORDER[Hie_NUM_ORDERS] =
	{
	[Hie_ORDER_BY_NAME] =
#if   L==1	// ca
	"Ordenar per instituci&oacute;"
#elif L==2	// de
	"Nach Hochschule sortieren"
#elif L==3	// en
	"Sort by institution"
#elif L==4	// es
	"Ordenar por instituci&oacute;n"
#elif L==5	// fr
	"Trier par &eacute;tablissement"
#elif L==6	// gn
	"Ordenar por instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per istituzione"
#elif L==8	// pl
	"Sortuj wedlug instytucji"
#elif L==9	// pt
	"Classificar por institui&ccedil;&atilde;o"
#elif L==10	// tr
	"Sort by institution"			// eviri lazim!
#endif
	,
	[Hie_ORDER_BY_NUM_USRS] =
#if   L==1	// ca
	"Ordenar per nombre d'usuaris"
#elif L==2	// de
	"Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	"Sort by no. of users"
#elif L==4	// es
	"Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	"Trier par nombre d'utilisateurs"
#elif L==6	// gn
	"Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di utenti"
#elif L==8	// pl
	"Sortowanie wedlug liczby uzytkownikow"
#elif L==9	// pt
	"Classificar por n&ordm; de utilizadores"
#elif L==10	// tr
	"Sort by no. of users"			// eviri lazim!
#endif
	};

const char *Txt_Institutions_by_number_of_centers =
#if   L==1	// ca
	"Institucions segons nombre de centers";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Lehrinstitute";
#elif L==3	// en
	"Institutions by number of centers";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de centros";
#elif L==5	// fr
	"&Eacute;tablissements par nombre de centers";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de centros";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di centri";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby o&sacute;rodk&oacute;w";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de centros";
#elif L==10	// tr
	"Institutions by number of centers";			// eviri lazim!
#endif

const char *Txt_Institutions_by_number_of_degrees =
#if   L==1	// ca
	"Institucions segons nombre de titulacions";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Studiengang";
#elif L==3	// en
	"Institutions by number of degrees";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de titulaciones";
#elif L==5	// fr
	"&Eacute;tablissements par nombre d'&eacute;tudes";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de titulaciones";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di lauree";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby stopni";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de graus";
#elif L==10	// tr
	"Institutions by number of degrees";				// eviri lazim!
#endif

const char *Txt_Institutions_by_number_of_courses =
#if   L==1	// ca
	"Institucions segons nombre de assignatures";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Kursen";
#elif L==3	// en
	"Institutions by number of courses";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de asignaturas";
#elif L==5	// fr
	"&Eacute;tablissements par nombre de mati&egrave;res";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di corsi";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby kurs&otilde;w";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de disciplinas";
#elif L==10	// tr
	"Institutions by number of courses";				// eviri lazim!
#endif

const char *Txt_Institutions_by_number_of_users_in_courses =
#if   L==1	// ca
	"Institucions segons nombre d'usuaris en assignatures";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Benutzer in Kurse";
#elif L==3	// en
	"Institutions by number of users in courses";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de usuarios en asignaturas";
#elif L==5	// fr
	"&Eacute;tablissements par nombre d'utilisateurs dans mati&egrave;res";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de usuarios en asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di utenti in corsi";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby u&zdot;ytkownik&oacute;w w kurs&otilde;w";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de usu&aacute;rios em disciplinas";
#elif L==10	// tr
	"Institutions by number of users in courses";				// eviri lazim!
#endif

const char *Txt_Institutions_by_number_of_users_who_claim_to_belong_to_them =
#if   L==1	// ca
	"Institucions segons nombre d'usuaris que diuen pert&agrave;nyer-hi";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Benutzer, die behaupten, zu ihnen geh&ouml;ren";
#elif L==3	// en
	"Institutions by number of users who claim to belong to them";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de usuarios que dicen pertenecer a ellas";
#elif L==5	// fr
	"&Eacute;tablissements par nombre d'utilisateurs qui pr&eacute;tendent leur appartiennent";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de usuarios que dicen pertenecer a ellas";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di utenti che affermano di appartenere a loro";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby u&zdot;ytkownik&oacute;w, kt&oacute;rzy twierdz&aogon;, &zdot;e nale&zdot;&aogon; do nich";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de usu&aacute;rios que afirmam pertencer a elas";
#elif L==10	// tr
	"Institutions by number of users who claim to belong to them";				// eviri lazim!
#endif

const char *Txt_Institutions_of_COUNTRY_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Institucions de %s";
#elif L==2	// de
	"Hochschulen von %s";
#elif L==3	// en
	"Institutions of %s";
#elif L==4	// es
	"Instituciones de %s";
#elif L==5	// fr
	"&Eacute;tablissements de %s";
#elif L==6	// gn
	"Instituciones %s-pegua";
#elif L==7	// it
	"Istituzioni di %s";
#elif L==8	// pl
	"Instytucje %s";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es de %s";
#elif L==10	// tr
	"%s Kurumlar&inodot;";
#endif

const char *Txt_INSTITUTIONS_ORDER[Hie_NUM_ORDERS] =
	{
	[Hie_ORDER_BY_NAME] =
#if   L==1	// ca
	"Instituci&oacute;"
#elif L==2	// de
	"Hochschule"
#elif L==3	// en
	"Institution"
#elif L==4	// es
	"Instituci&oacute;n"
#elif L==5	// fr
	"&Eacute;tablissement"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzione"
#elif L==8	// pl
	"Instytucji"
#elif L==9	// pt
	"Institui&ccedil;&atilde;o"
#elif L==10	// tr
	"Kurum"
#endif
	,
	[Hie_ORDER_BY_NUM_USRS] =
#if   L==1	// ca
	"Usuaris de la instituci&oacute;"
#elif L==2	// de
	"Benutzer der Hochschule"
#elif L==3	// en
	"Users of the institution"
#elif L==4	// es
	"Usuarios de la instituci&oacute;n"
#elif L==5	// fr
	"Utilisateurs de l'&eacute;tablissement"
#elif L==6	// gn
	"Usuarios de la instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Utenti della istituzione"
#elif L==8	// pl
	"U&zdot;ytkownicy instytucji"
#elif L==9	// pt
	"Utilizadores da institu&ccedil;&atilde;o"
#elif L==10	// tr
	"Users of the institution"		// eviri lazim!
#endif
	};

const char *Txt_Institutions_with_pending_centers =
#if   L==1	// ca
	"Institucions amb centers pendents d'activar";
#elif L==2	// de
	"Hochschulen mit anh&auml;ngig Lehrinstitute";
#elif L==3	// en
	"Institutions with pending centers";
#elif L==4	// es
	"Instituciones con centros pendientes de activar";
#elif L==5	// fr
	"&Eacute;tablissements avec des centers en attente";
#elif L==6	// gn
	"Instituciones con centros pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni con centri in sospeso";
#elif L==8	// pl
	"Instytucji z centra oczekuj&aogon;cy";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es com centros pendentes";
#elif L==10	// tr
	"Institutions with pending centers";			// eviri lazim!
#endif

const char *Txt_Integer_number =
#if   L==1	// ca
	"Nombre enter";
#elif L==2	// de
	"Ganze Zahl";
#elif L==3	// en
	"Integer number";
#elif L==4	// es
	"N.&ordm; entero";
#elif L==5	// fr
	"Nombre entier";
#elif L==6	// gn
	"N.&ordm; entero";	// Okoteve traduccin
#elif L==7	// it
	"Numero intero";
#elif L==8	// pl
	"Liczba calkowita";
#elif L==9	// pt
	"N.&ordm; inteiro";
#elif L==10	// tr
	"Integer number";	// eviri lazim!
#endif

const char *Txt_Internal_code =
#if   L==1	// ca
	"Codi intern";
#elif L==2	// de
	"Interner Code";
#elif L==3	// en
	"Internal code";
#elif L==4	// es
	"C&oacute;digo interno";
#elif L==5	// fr
	"Code interne";
#elif L==6	// gn
	"C&oacute;digo interno";	// Okoteve traduccin
#elif L==7	// it
	"Codice interno";
#elif L==8	// pl
	"Kod wewn&eogon;trzny";
#elif L==9	// pt
	"C&oacute;digo interno";
#elif L==10	// tr
	"Internal code";		// eviri lazim!
#endif

const char *Txt_Internationalization =
#if   L==1	// ca
	"Internacionalitzaci&oacute;";
#elif L==2	// de
	"Internationalisierung";
#elif L==3	// en
	"Internationalization";
#elif L==4	// es
	"Internacionalizaci&oacute;n";
#elif L==5	// fr
	"Internationalisation";
#elif L==6	// gn
	"Internacionalizaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Internazionalizzazione";
#elif L==8	// pl
	"Umi&eogon;dzynarodowienie";
#elif L==9	// pt
	"Internacionaliza&ccedil;&atilde;o";
#elif L==10	// tr
	"Uluslararas&inodot;la&scedil;ma";
#endif

const char *Txt_Invalid_question =
#if   L==1	// ca
	"Pregunta anul&middot;lada";
#elif L==2	// de
	"Frage storniert";
#elif L==3	// en
	"Invalid question";
#elif L==4	// es
	"Pregunta anulada";
#elif L==5	// fr
	"Question annul&eacute;e";
#elif L==6	// gn
	"Pregunta anulada";	// Okoteve traduccin
#elif L==7	// it
	"Domanda annullata";
#elif L==8	// pl
	"Pytanie anulowane";
#elif L==9	// pt
	"Pergunta cancelada";
#elif L==10	// tr
	"Invalid question";	// eviri lazim!
#endif

const char *Txt_IP =		// Internet Protocol address
#if   L==1	// ca
	"IP";
#elif L==2	// de
	"IP";
#elif L==3	// en
	"IP";
#elif L==4	// es
	"IP";
#elif L==5	// fr
	"IP";
#elif L==6	// gn
	"IP";	// Okoteve traduccin
#elif L==7	// it
	"IP";
#elif L==8	// pl
	"IP";
#elif L==9	// pt
	"IP";
#elif L==10	// tr
	"IP";	// eviri lazim!
#endif

const char *Txt_It_is_mandatory_to_choose_a_group =
#if   L==1	// ca
	"Escollir grup &eacute;s obligatori";
#elif L==2	// de
	"Es muss eine Gruppe ausgew&auml;hlt werden";
#elif L==3	// en
	"It is mandatory to choose a group";
#elif L==4	// es
	"Elegir grupo es obligatorio";
#elif L==5	// fr
	"Il est obligatoire de choisir un groupe";
#elif L==6	// gn
	"Elegir grupo es obligatorio";		// Okoteve traduccin
#elif L==7	// it
	"E 'obbligatorio scegliere un gruppo";
#elif L==8	// pl
	"Jest to obowiazkowe, aby wybrac grupe";
#elif L==9	// pt
	"Escolher um grupo &eacute; obrigat&oacute;rio";
#elif L==10	// tr
	"It is mandatory to choose a group";	// eviri lazim!
#endif

const char *Txt_It_is_optional_to_choose_a_group =
#if   L==1	// ca
	"Escollir grup &eacute;s voluntari";
#elif L==2	// de
	"Die Ausw&auml;hl einer Gruppe ist freiwillig";
#elif L==3	// en
	"It is optional to choose a group";
#elif L==4	// es
	"Elegir grupo es voluntario";
#elif L==5	// fr
	"Il est volontaire de choisir un groupe";
#elif L==6	// gn
	"Elegir grupo es voluntario";		// Okoteve traduccin
#elif L==7	// it
	"E' facoltativo scegliere un gruppo";
#elif L==8	// pl
	"Opcjonalnie mozna wybrac grupe";
#elif L==9	// pt
	"Escolher um grupo &eacute; volunt&aacute;rio";
#elif L==10	// tr
	"It is optional to choose a group";	// eviri lazim!
#endif

const char *Txt_Item_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&Iacute;tem <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Programmelement <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Item <strong>s</strong> removed.";
#elif L==4	// es
	"&Iacute;tem <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"&Eacute;l&eacute;ment <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"&Iacute;tem <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Elemento <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Pozycja <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Item <strong>%s</strong> removido.";
#elif L==10	// tr
	"Item <strong>s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Its_me =
#if   L==1	// ca
	"S&oacute;c jo!";
#elif L==2	// de
	"Da ich bin!";
#elif L==3	// en
	"It's me!";
#elif L==4	// es
	"&iexcl;Soy yo!";
#elif L==5	// fr
	"C'est moi!";
#elif L==6	// gn
	"&iexcl;Soy yo!";	// Okoteve traduccin
#elif L==7	// it
	"Sono io!";
#elif L==8	// pl
	"To ja!";
#elif L==9	// pt
	"Sou eu!";
#elif L==10	// tr
	"It's me!";		// eviri lazim!
#endif

const char *Txt_Language =
#if   L==1	// ca
	"Idioma";
#elif L==2	// de
	"Sprache";
#elif L==3	// en
	"Language";
#elif L==4	// es
	"Idioma";
#elif L==5	// fr
	"Langue";
#elif L==6	// gn
	"&Ntilde;e'&#7869;";
#elif L==7	// it
	"Lingua";
#elif L==8	// pl
	"J&eogon;zyk";
#elif L==9	// pt
	"Lingua";
#elif L==10	// tr
	"Dil";
#endif

const char *Txt_Last_clicks =
#if   L==1	// ca
	"&Uacute;ltims clics";
#elif L==2	// de
	"Letzte Klicks";
#elif L==3	// en
	"Last clicks";
#elif L==4	// es
	"&Uacute;ltimos clics";
#elif L==5	// fr
	"Derni&egrave;re clics";
#elif L==6	// gn
	"&Uacute;ltimos clics";	// Okoteve traduccin
#elif L==7	// it
	"Ultimi clic";
#elif L==8	// pl
	"Ostatnie dwa klikni&eogon;cia";
#elif L==9	// pt
	"&Uacute;ltimos cliques";
#elif L==10	// tr
	"Last clicks";		// eviri lazim!
#endif

const char *Txt_Last_clicks_in_real_time =
#if   L==1	// ca
	"&Uacute;ltims clics en temps real";
#elif L==2	// de
	"Letzte Klicks in Echtzeit";
#elif L==3	// en
	"Last clicks in real time";
#elif L==4	// es
	"&Uacute;ltimos clics en tiempo real";
#elif L==5	// fr
	"Derni&egrave;re clics en temps r&eacute;el";
#elif L==6	// gn
	"&Uacute;ltimos clics en tiempo real";	// Okoteve traduccin
#elif L==7	// it
	"Ultimi clic in tempo reale";
#elif L==8	// pl
	"Ostatnie dwa klikni&eogon;cia w czasie rzeczywistym";
#elif L==9	// pt
	"&Uacute;ltimos cliques em tempo real";
#elif L==10	// tr
	"Ger&ccedil;ek zamanl&inodot; son t&inodot;klamalar";
#endif

const char *Txt_Latitude =
#if   L==1	// ca
	"Latitud";
#elif L==2	// de
	"Geographische Breite";
#elif L==3	// en
	"Latitude";
#elif L==4	// es
	"Latitud";
#elif L==5	// fr
	"Latitude";
#elif L==6	// gn
	"Latitud";	// Okoteve traduccin
#elif L==7	// it
	"Latitudine";
#elif L==8	// pl
	"Szeroko&sacute;&cacute; geograficzna";
#elif L==9	// pt
	"Latitude";
#elif L==10	// tr
	"Enlem";
#endif

const char *Txt_LAYOUT_SIDE_COLUMNS[4] =
	{
#if   L==1	// ca
	"Nom&eacute;s zona central"
#elif L==2	// de
	"Nur mittleren Bereich anzeigen"
#elif L==3	// en
	"Display central area only"
#elif L==4	// es
	"Solo zona central"
#elif L==5	// fr
	"Afficher la zone centrale uniquement"
#elif L==6	// gn
	"Solo zona central"		// Okoteve traduccin
#elif L==7	// it
	"Mostra solo la parte centrale"
#elif L==8	// pl
	"Wyswietl centralnej czesci tylko"
#elif L==9	// pt
	"Somente &aacute;rea central"
#elif L==10	// tr
	"Display central area only"	// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Columna dreta visible"
#elif L==2	// de
	"Rechte Spalte anzeigen"
#elif L==3	// en
	"Display right column"
#elif L==4	// es
	"Columna derecha visible"
#elif L==5	// fr
	"Afficher la colonne de droite"
#elif L==6	// gn
	"Columna derecha visible"	// Okoteve traduccin
#elif L==7	// it
	"Mostra la colonna destra"
#elif L==8	// pl
	"Wyswietl prawa kolumne"
#elif L==9	// pt
	"Coluna direita vis&iacute;vel"
#elif L==10	// tr
	"Display right column"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Columna esquerra visible"
#elif L==2	// de
	"Linke Spalte anzeigen"
#elif L==3	// en
	"Display left column"
#elif L==4	// es
	"Columna izquierda visible"
#elif L==5	// fr
	"Afficher la colonne de gauche"
#elif L==6	// gn
	"Columna izquierda visible"	// Okoteve traduccin
#elif L==7	// it
	"Mostra la colonna sinistra"
#elif L==8	// pl
	"Wyswietl lewa kolumne"
#elif L==9	// pt
	"Coluna esquerda vis&iacute;vel"
#elif L==10	// tr
	"Display left column"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Columnes esquerra i dreta visibles"
#elif L==2	// de
	"Linke und rechte Spalte anzeigen"
#elif L==3	// en
	"Display left and right columns"
#elif L==4	// es
	"Columnas izquierda y derecha visibles"
#elif L==5	// fr
	"Afficher les colonnes de gauche et de droite"
#elif L==6	// gn
	"Columnas izquierda y derecha visibles"	// Okoteve traduccin
#elif L==7	// it
	"Mostra la colonna sinistra e destra"
#elif L==8	// pl
	"Wyswietl lewa i prawa kolumne"
#elif L==9	// pt
	"Colunas esquerda e direita vis&iacute;veis"
#elif L==10	// tr
	"Display left and right columns"	// eviri lazim!
#endif
	};

const char *Txt_level =
#if   L==1	// ca
	"nivell";
#elif L==2	// de
	"Ebene";
#elif L==3	// en
	"level";
#elif L==4	// es
	"nivel";
#elif L==5	// fr
	"niveau";
#elif L==6	// gn
	"nivel";	// Okoteve traduccin
#elif L==7	// it
	"livello";
#elif L==8	// pl
	"poziom";
#elif L==9	// pt
	"n&iacute;vel";
#elif L==10	// tr
	"level";	// eviri lazim!
#endif

const char *Txt_Levels =
#if   L==1	// ca
	"Nivells";
#elif L==2	// de
	"Ebenen";
#elif L==3	// en
	"Levels";
#elif L==4	// es
	"Niveles";
#elif L==5	// fr
	"Niveaux";
#elif L==6	// gn
	"Niveles";	// Okoteve traduccin
#elif L==7	// it
	"Livelli";
#elif L==8	// pl
	"Poziomy";
#elif L==9	// pt
	"N&iacute;veis";
#elif L==10	// tr
	"Levels";	// eviri lazim!
#endif

const char *Txt_levels =
#if   L==1	// ca
	"nivells";
#elif L==2	// de
	"Ebenen";
#elif L==3	// en
	"levels";
#elif L==4	// es
	"niveles";
#elif L==5	// fr
	"niveaux";
#elif L==6	// gn
	"niveles";	// Okoteve traduccin
#elif L==7	// it
	"livelli";
#elif L==8	// pl
	"poziomy";
#elif L==9	// pt
	"n&iacute;veis";
#elif L==10	// tr
	"levels";	// eviri lazim!
#endif

const char *Txt_License =
#if   L==1	// ca
	"Llic&egrave;ncia";
#elif L==2	// de
	"Lizenz";
#elif L==3	// en
	"License";
#elif L==4	// es
	"Licencia";
#elif L==5	// fr
	"Licence";
#elif L==6	// gn
	"Licencia";	// Okoteve traduccin
#elif L==7	// it
	"Licenza";
#elif L==8	// pl
	"Pozwolenie";
#elif L==9	// pt
	"Licen&ccedil;a";
#elif L==10	// tr
	"License";	// eviri lazim!
#endif

const char *Txt_LICENSES[Brw_NUM_LICENSES] =
	{
	[Brw_LICENSE_UNKNOWN] =
#if   L==1	// ca
	"desconeguda / altra"
#elif L==2	// de
	"Unbekannte / sonstige"
#elif L==3	// en
	"unknown / other"
#elif L==4	// es
	"desconocida / otra"
#elif L==5	// fr
	"inconnue / autre"
#elif L==6	// gn
	"desconocida / otra"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuta / altri"
#elif L==8	// pl
	"nieznana / inny"
#elif L==9	// pt
	"desconhecida / outra"
#elif L==10	// tr
	"unknown / other"	// eviri lazim!
#endif
	,
	[Brw_LICENSE_ALL_RIGHTS_RESERVED] =
#if   L==1	// ca
	"Tots els drets reservats"		// All Rights Reserved
#elif L==2	// de
	"Alle Rechte vorbehalten"		// All Rights Reserved
#elif L==3	// en
	"All rights reserved"			// All Rights Reserved
#elif L==4	// es
	"Todos los derechos reservados"		// All Rights Reserved
#elif L==5	// fr
	"Tous droits r&eacute;serv&eacute;s"	// All Rights Reserved
#elif L==6	// gn
	"Todos los derechos reservados"		// All Rights Reserved	// Okoteve traduccin
#elif L==7	// it
	"Tutti i diritti riservati"		// All Rights Reserved
#elif L==8	// pl
	"Wszelkie prawa zastrze&zdot;one"	// All Rights Reserved
#elif L==9	// pt
	"Todos os direitos reservados"		// All Rights Reserved
#elif L==10	// tr
	"All rights reserved"			// All Rights Reserved	// eviri lazim!
#endif
	,
	[Brw_LICENSE_CC_BY] =
#if   L==1	// ca
	"CC Reconeixement"			// CC Attribution License
#elif L==2	// de
	"CC Namensnennung"			// CC Attribution License
#elif L==3	// en
	"CC Attribution"			// CC Attribution License
#elif L==4	// es
	"CC Reconocimiento"			// CC Attribution License
#elif L==5	// fr
	"CC Attribution"			// CC Attribution License
#elif L==6	// gn
	"CC Reconocimiento"			// CC Attribution License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione"			// CC Attribution License
#elif L==8	// pl
	"CC Uznanie autorstwa"			// CC Attribution License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o"		// CC Attribution License
#elif L==10	// tr
	"CC Attribution"			// CC Attribution License	// eviri lazim!
#endif
	,
	[Brw_LICENSE_CC_BY_SA] =
#if   L==1	// ca
	"CC Reconeixement - Compartir igual"				// CC Attribution-ShareAlike License
#elif L==2	// de
	"CC Namensnennung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-ShareAlike License
#elif L==3	// en
	"CC Attribution - Share Alike"					// CC Attribution-ShareAlike License
#elif L==4	// es
	"CC Reconocimiento - Compartir bajo la misma licencia"		// CC Attribution-ShareAlike License
#elif L==5	// fr
	"CC Attribution - Partage &agrave; l'Identique"			// CC Attribution-ShareAlike License
#elif L==6	// gn
	"CC Reconocimiento - Compartir bajo la misma licencia"		// CC Attribution-ShareAlike License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Condividi allo stesso modo"			// CC Attribution-ShareAlike License
#elif L==8	// pl
	"CC Uznanie autorstwa - Na tych samych warunkach"		// CC Attribution-ShareAlike License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - Compartilha Igual"		// CC Attribution-ShareAlike License
#elif L==10	// tr
	"CC Attribution - Share Alike"					// CC Attribution-ShareAlike License	// eviri lazim!
#endif
	,
	[Brw_LICENSE_CC_BY_ND] =
#if   L==1	// ca
	"CC Reconeixement - Sense obra derivada"				// CC Attribution-NoDerivs License
#elif L==2	// de
	"CC Namensnennung - Keine Bearbeitung"					// CC Attribution-NoDerivs License
#elif L==3	// en
	"CC Attribution - No Derivative Works"					// CC Attribution-NoDerivs License
#elif L==4	// es
	"CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License
#elif L==5	// fr
	"CC Attribution - Pas de travaux d&eacute;riv&eacute;s"			// CC Attribution-NoDerivs License
#elif L==6	// gn
	"CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non opere derivate"					// CC Attribution-NoDerivs License
#elif L==8	// pl
	"CC Uznanie autorstwa - Bez utwor&oacute;w zale&zdot;nych"		// CC Attribution-NoDerivs License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NoDerivs License
#elif L==10	// tr
	"CC Attribution - No Derivative Works"					// CC Attribution-NoDerivs License	// eviri lazim!
#endif
	,
	[Brw_LICENSE_CC_BY_NC] =
#if   L==1	// ca
	"CC Reconeixement - No comercial"			// CC Attribution-NonCommercial License
#elif L==2	// de
	"CC Namensnennung - Keine kommerzielle Nutzung"		// CC Attribution-NonCommercial License
#elif L==3	// en
	"CC Attribution - Noncommercial"			// CC Attribution-NonCommercial License
#elif L==4	// es
	"CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License
#elif L==5	// fr
	"CC Attribution - Pas d'Utilisation Commerciale"	// CC Attribution-NonCommercial License
#elif L==6	// gn
	"CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non commerciale"			// CC Attribution-NonCommercial License
#elif L==8	// pl
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne"	// CC Attribution-NonCommercial License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial"	// CC Attribution-NonCommercial License
#elif L==10	// tr
	"CC Attribution - Noncommercial"			// CC Attribution-NonCommercial License	// eviri lazim!
#endif
	,
	[Brw_LICENSE_CC_BY_NC_SA] =
#if   L==1	// ca
	"CC Reconeixement - No comercial - Compartir igual"					// CC Attribution-NonCommercial-ShareAlike License
#elif L==2	// de
	"CC Namensnennung - Keine kommerzielle Nutzung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-NonCommercial-ShareAlike License
#elif L==3	// en
	"CC Attribution - Noncommercial - Share Alike"						// CC Attribution-NonCommercial-ShareAlike License
#elif L==4	// es
	"CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==5	// fr
	"CC Attribution - Pas d'Utilisation Commerciale - Partage &agrave; l'Identique"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==6	// gn
	"CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non commerciale - Condividi allo stesso modo"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==8	// pl
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Na tych samych warunkach"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Compartilha Igual"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==10	// tr
	"CC Attribution - Noncommercial - Share Alike"						// CC Attribution-NonCommercial-ShareAlike License	// eviri lazim!
#endif
	,
	[Brw_LICENSE_CC_BY_NC_ND] =
#if   L==1	// ca
	"CC Reconeixement - No comercial - Sense obra derivada"						// CC Attribution-NonCommercial-NoDerivs License
#elif L==2	// de
	"CC Namensnennung - Keine kommerzielle Nutzung - Keine Bearbeitung"				// CC Attribution-NonCommercial-NoDerivs License
#elif L==3	// en
	"CC Attribution - Noncommercial - No Derivative Works"						// CC Attribution-NonCommercial-NoDerivs License
#elif L==4	// es
	"CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==5	// fr
	"CC Attribution - Pas dUtilisation Commerciale - Pas de travaux d&eacute;riv&eacute;s"		// CC Attribution-NonCommercial-NoDerivs License
#elif L==6	// gn
	"CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non commerciale - Non opere derivate"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==8	// pl
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Bez utwor&oacute;w zale&zdot;nych"		// CC Attribution-NonCommercial-NoDerivs License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NonCommercial-NoDerivs License
#elif L==10	// tr
	"CC Attribution - Noncommercial - No Derivative Works"						// CC Attribution-NonCommercial-NoDerivs License	// eviri lazim!
#endif
	};

const char *Txt_Link =
#if   L==1	// ca
	"Enlla&ccedil;";
#elif L==2	// de
	"Link";
#elif L==3	// en
	"Link";
#elif L==4	// es
	"Enlace";
#elif L==5	// fr
	"Lien";
#elif L==6	// gn
	"Enlace";	// Okoteve traduccin
#elif L==7	// it
	"Link";
#elif L==8	// pl
	"Link";
#elif L==9	// pt
	"Liga&ccedil;&atilde;o.";
#elif L==10	// tr
	"Ba&gbreve;lant&inodot;";
#endif

const char *Txt_link =
#if   L==1	// ca
	"enlla&ccedil;";
#elif L==2	// de
	"Link";
#elif L==3	// en
	"link";
#elif L==4	// es
	"enlace";
#elif L==5	// fr
	"lien";
#elif L==6	// gn
	"enlace";	// Okoteve traduccin
#elif L==7	// it
	"link";
#elif L==8	// pl
	"link";
#elif L==9	// pt
	"liga&ccedil;&atilde;o.";
#elif L==10	// tr
	"ba&gbreve;lant&inodot;";
#endif

const char *Txt_Link_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Enlla&ccedil; <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Link <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Link <strong>%s</strong> removed.";
#elif L==4	// es
	"Enlace <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Lien <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Enlace <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Link <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Link <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Liga&ccedil;&atilde;o <strong>%s</strong> removida.";
#elif L==10	// tr
	"Link <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Link_to_call_for_exam =
#if   L==1	// ca
	"Enlla&ccedil; a convocat&ograve;ria d'examen";
#elif L==2	// de
	"Link zu Aufruf f&uuml;r Pr&uuml;fung";
#elif L==3	// en
	"Link to call for exam";
#elif L==4	// es
	"Enlace a convocatoria de examen";
#elif L==5	// fr
	"Lien vers convocation &agrave; un examen";
#elif L==6	// gn
	"Enlace a convocatoria de examen";	// Okoteve traduccin
#elif L==7	// it
	"Link al appello d'esame";
#elif L==8	// pl
	"Link do ogloszenie egzaminu";
#elif L==9	// pt
	"Link para chamada para exame";
#elif L==10	// tr
	"Link to call for exam";		// eviri lazim!
#endif

const char *Txt_Link_to_resource_X_copied_into_clipboard =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Enlla&ccedil; al recurs <strong>%s</strong> copiat al porta-retalls.";
#elif L==2	// de
	"Link zu Ressource <strong>%s</strong> in die Zwischenablage kopiert.";
#elif L==3	// en
	"Link to resource <strong>%s</strong> copied into clipboard.";
#elif L==4	// es
	"Enlace al recurso <strong>%s</strong> copiado en el portapapeles.";
#elif L==5	// fr
	"Lien vers la ressource <strong>%s</strong> copi&eacute; dans le presse-papiers.";
#elif L==6	// gn
	"Enlace al recurso <strong>%s</strong> copiado en el portapapeles.";	// Okoteve traduccin
#elif L==7	// it
	"Collegamento alla risorsa <strong>%s</strong> copiata negli appunti.";
#elif L==8	// pl
	"Link do zasobu <strong>%s</strong> skopiowany do schowka.";
#elif L==9	// pt
	"Link para o recurso <strong>%s</strong> copiado para a &aacute;rea de transfer&ecirc;ncia.";
#elif L==10	// tr
	"Panoya kopyalanan <strong>%s</strong> kayna&gbreve;&inodot;na ba&gbreve;lant&inodot;.";
#endif

const char *Txt_List_of_detailed_clicks =
#if   L==1	// ca
	"Llistat de clics detallats";
#elif L==2	// de
	"Liste der detaillierten Klicks";
#elif L==3	// en
	"List of detailed clicks";
#elif L==4	// es
	"Listado de clics detallados";
#elif L==5	// fr
	"Liste des clics d&eacute;taill&eacute;s";
#elif L==6	// gn
	"Listado de clics detallados";	// Okoteve traduccin
#elif L==7	// it
	"Lista di scatti dettagliati";
#elif L==8	// pl
	"Lista szczeg&oacute;&lstrok;owych klikni&eogon;&cacute;";
#elif L==9	// pt
	"Lista de cliques detalhados";
#elif L==10	// tr
	"List of detailed clicks";	// eviri lazim!
#endif

const char *Txt_List_of_nicks_emails_or_IDs =
#if   L==1	// ca
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Necessita traducci
#elif L==2	// de
	"Liste von @Benutzernamen, Emails oder Ausweis-Nr.";
#elif L==3	// en
	"List of @nicks, emails or IDs";
#elif L==4	// es
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";
#elif L==5	// fr
	"Liste de num&eacute;ros d'identit&eacute;";
#elif L==6	// gn
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Okoteve traduccin
#elif L==7	// it
	"Lista di @nomi-utenti, email o numero carte d'identit&agrave;";
#elif L==8	// pl
	"List of @nicks, emails or IDs";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Lista de @alcunhas, emails ou n&ordm;s. identif.";
#elif L==10	// tr
	"List of @nicks, emails or IDs";			// eviri lazim!
#endif

const char *Txt_Links =
#if   L==1	// ca
	"Enlla&ccedil;os";
#elif L==2	// de
	"Links";
#elif L==3	// en
	"Links";
#elif L==4	// es
	"Enlaces";
#elif L==5	// fr
	"Liens";
#elif L==6	// gn
	"Ambu&eacute;re joaju";
#elif L==7	// it
	"Link";
#elif L==8	// pl
	"Linki";
#elif L==9	// pt
	"Liga&ccedil;&otilde;es";
#elif L==10	// tr
	"Ba&gbreve;lant&inodot;lar";
#endif

const char *Txt_Links_copied =
#if   L==1	// ca
	"Enlla&ccedil;os copiats";
#elif L==2	// de
	"Links kopiert";
#elif L==3	// en
	"Links copied";
#elif L==4	// es
	"Enlaces copiados";
#elif L==5	// fr
	"Liens copi&eacute;s";
#elif L==6	// gn
	"Enlaces copiados";	// Okoteve traduccin
#elif L==7	// it
	"Link copiati";
#elif L==8	// pl
	"Linki kopiowane";
#elif L==9	// pt
	"Liga&ccedil;&otilde;es copiadas";
#elif L==10	// tr
	"Links copied";		// eviri lazim!
#endif

const char *Txt_Links_removed =
#if   L==1	// ca
	"Enlla&ccedil;os eliminats";
#elif L==2	// de
	"Links entfernt";
#elif L==3	// en
	"Links removed";
#elif L==4	// es
	"Enlaces eliminados";
#elif L==5	// fr
	"Liens supprim&eacute;s";
#elif L==6	// gn
	"Enlaces eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Link rimossi";
#elif L==8	// pl
	"Linki usuni&eogon;te";
#elif L==9	// pt
	"Liga&ccedil;&otilde;es removidas";
#elif L==10	// tr
	"Links removed";	// eviri lazim!
#endif

const char *Txt_Location =
#if   L==1	// ca
	"Ubicaci&oacute;";
#elif L==2	// de
	"Standort";
#elif L==3	// en
	"Location";
#elif L==4	// es
	"Ubicaci&oacute;n";
#elif L==5	// fr
	"Emplacement";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Posizione";
#elif L==8	// pl
	"Lokacja";
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o";
#elif L==10	// tr
	"Location";	// eviri lazim!
#endif

const char *Txt_Locations =
#if   L==1	// ca
	"Ubicacions";
#elif L==2	// de
	"Standorte";
#elif L==3	// en
	"Locations";
#elif L==4	// es
	"Ubicaciones";
#elif L==5	// fr
	"Emplacements";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Posizioni";
#elif L==8	// pl
	"Lokalizacje";
#elif L==9	// pt
	"Localiza&ccedil;&otilde;es";
#elif L==10	// tr
	"Locations";	// eviri lazim!
#endif

const char *Txt_LOCKED_UNLOCKED[Prj_NUM_LOCKED_UNLOCKED] =
	{
	[Prj_LOCKED] =
#if   L==1	// ca
	"Edici&oacute; bloquejada"
#elif L==2	// de
	"Gesperrte Ausgabe"
#elif L==3	// en
	"Locked edition"
#elif L==4	// es
	"Edici&oacute;n bloqueada"
#elif L==5	// fr
	"Edition verrouill&eacute;e"
#elif L==6	// gn
	"Edici&oacute;n bloqueada"	// Okoteve traduccin
#elif L==7	// it
	"Edizione bloccata"
#elif L==8	// pl
	"Zamkni&eogon;ta edycja"
#elif L==9	// pt
	"Edi&ccedil;&atilde;o bloqueada"
#elif L==10	// tr
	"Locked edition"		// eviri lazim!
#endif
	,
	[Prj_UNLOCKED] =
#if   L==1	// ca
	"Edici&oacute; permesa"
#elif L==2	// de
	"Erlaubte Ausgabe"
#elif L==3	// en
	"Allowed edition"
#elif L==4	// es
	"Edici&oacute;n permitida"
#elif L==5	// fr
	"&Eacute;dition autoris&eacute;e"
#elif L==6	// gn
	"Edici&oacute;n permitida"	// Okoteve traduccin
#elif L==7	// it
	"Edizione consentita"
#elif L==8	// pl
	"Dozwolone wydanie"
#elif L==9	// pt
	"Edi&ccedil;&atilde;o permitida"
#elif L==10	// tr
	"Allowed edition"		// eviri lazim!
#endif
   };

const char *Txt_Lock_editing =
#if   L==1	// ca
	"Bloquejar edici&oacute;";
#elif L==2	// de
	"Bearbeitung sperren";
#elif L==3	// en
	"Lock editing";
#elif L==4	// es
	"Bloquear edici&oacute;n";
#elif L==5	// fr
	"Verrouiller l'&eacute;dition";
#elif L==6	// gn
	"Bloquear edici&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Blocca la modifica";
#elif L==8	// pl
	"Zablokuj edycj&eogon;";
#elif L==9	// pt
	"Bloquear edi&ccedil;&atilde;o";
#elif L==10	// tr
	"Lock editing";	// eviri lazim!
#endif

const char *Txt_LOG_More_info =
#if   L==1	// ca
	"Observacions";
#elif L==2	// de
	"Weitere Informationen";
#elif L==3	// en
	"More info";
#elif L==4	// es
	"Observaciones";
#elif L==5	// fr
	"Plus d'informations";
#elif L==6	// gn
	"Observaciones";	// Okoteve traduccin
#elif L==7	// it
	"Maggiori informazioni";
#elif L==8	// pl
	"Wiecej informacji";
#elif L==9	// pt
	"Mais informa&ccedil;&atilde;o";
#elif L==10	// tr
	"More info";		// eviri lazim!
#endif

const char *Txt_Log_in =
#if   L==1	// ca
	"Iniciar sessi&oacute;";
#elif L==2	// de
	"Anmelden";
#elif L==3	// en
	"Log in";
#elif L==4	// es
	"Iniciar sesi&oacute;n";
#elif L==5	// fr
	"Entrer";
#elif L==6	// gn
	"Tera&ntilde;emboguapy";
#elif L==7	// it
	"Inizia sessione";
#elif L==8	// pl
	"Zaloguj si&eogon;";
#elif L==9	// pt
	"Entrar";
#elif L==10	// tr
	"Giri&scedil; yapmak";
#endif

const char *Txt_logged[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"identificat/da"
#elif L==2	// de
	"angemeldet"
#elif L==3	// en
	"logged"
#elif L==4	// es
	"identificado/a"
#elif L==5	// fr
	"connect&eacute;/e"
#elif L==6	// gn
	"identificado/a"	// Okoteve traduccin
#elif L==7	// it
	"loggato/a"
#elif L==8	// pl
	"zalogowany"
#elif L==9	// pt
	"logado/a"
#elif L==10	// tr
	"logged"		// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"identificada"
#elif L==2	// de
	"angemeldet"
#elif L==3	// en
	"logged"
#elif L==4	// es
	"identificada"
#elif L==5	// fr
	"connect&eacute;e"
#elif L==6	// gn
	"identificada"	// Okoteve traduccin
#elif L==7	// it
	"loggata"
#elif L==8	// pl
	"zalogowana"
#elif L==9	// pt
	"logada"
#elif L==10	// tr
	"logged"	// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"identificat"
#elif L==2	// de
	"angemeldet"
#elif L==3	// en
	"logged"
#elif L==4	// es
	"identificado"
#elif L==5	// fr
	"connect&eacute;"
#elif L==6	// gn
	"identificado"	// Okoteve traduccin
#elif L==7	// it
	"loggato"
#elif L==8	// pl
	"zalogowany"
#elif L==9	// pt
	"logado"
#elif L==10	// tr
	"logged"	// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"identificat/da"
#elif L==2	// de
	"angemeldet"
#elif L==3	// en
	"logged"
#elif L==4	// es
	"identificado/a"
#elif L==5	// fr
	"connect&eacute;/e"
#elif L==6	// gn
	"identificado/a"	// Okoteve traduccin
#elif L==7	// it
	"loggato/a"
#elif L==8	// pl
	"zalogowani"
#elif L==9	// pt
	"logado/a"
#elif L==10	// tr
	"logged"		// eviri lazim!
#endif
	};

const char *Txt_Logo =
#if   L==1	// ca
	"Logo";
#elif L==2	// de
	"Logo";
#elif L==3	// en
	"Logo";
#elif L==4	// es
	"Logo";
#elif L==5	// fr
	"Logo";
#elif L==6	// gn
	"Logo";	// Okoteve traduccin
#elif L==7	// it
	"Logo";
#elif L==8	// pl
	"Logo";
#elif L==9	// pt
	"Logotipo";
#elif L==10	// tr
	"Logo";	// eviri lazim!
#endif

const char *Txt_Longitude =
#if   L==1	// ca
	"Longitud";
#elif L==2	// de
	"Geographische L&auml;nge";
#elif L==3	// en
	"Longitude";
#elif L==4	// es
	"Longitud";
#elif L==5	// fr
	"Longitude";
#elif L==6	// gn
	"Longitud";	// Okoteve traduccin
#elif L==7	// it
	"Longitudine";
#elif L==8	// pl
	"D&lstrok;ugo&sacute;&cacute; geograficzna";
#elif L==9	// pt
	"Longitude";
#elif L==10	// tr
	"Longitude";	// eviri lazim!
#endif

const char *Txt_MAC_address =
#if   L==1	// ca
	"Adre&ccedil;a MAC";
#elif L==2	// de
	"MAC-Adresse";
#elif L==3	// en
	"MAC address";
#elif L==4	// es
	"Direcci&oacute;n MAC";
#elif L==5	// fr
	"Adresse MAC";
#elif L==6	// gn
	"Direcci&oacute;n MAC";	// Okoteve traduccin
#elif L==7	// it
	"Indirizzo MAC";
#elif L==8	// pl
	"Adres MAC";
#elif L==9	// pt
	"Endere&ccedil;o MAC";
#elif L==10	// tr
	"MAC address";		// eviri lazim!
#endif

const char *Txt_Mandatory_enrolment =		// (to a type of group)
#if   L==1	// ca
	"&iquest;Adscripci&oacute;n obligatoria?";	// Necessita traducci
#elif L==2	// de
	"Obligatorische beitreten?";
#elif L==3	// en
	"Mandatory enrolment?";
#elif L==4	// es
	"&iquest;Adscripci&oacute;n obligatoria?";
#elif L==5	// fr
	"Inscription obligatoire?";
#elif L==6	// gn
	"&iquest;Adscripci&oacute;n obligatoria?";	// Okoteve traduccin
#elif L==7	// it
	"Iscrizione obbligatoria?";
#elif L==8	// pl
	"Obowiazkowa rejestracja?";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o obrigat&oacute;ria?";
#elif L==10	// tr
	"Mandatory enrolment?";				// eviri lazim!
#endif

const char *Txt_Mark_all_NOTIFICATIONS_as_read =
#if   L==1	// ca
	"Marca totes com llegides";
#elif L==2	// de
	"Markieren Sie alle als gelesen";
#elif L==3	// en
	"Mark all as read";
#elif L==4	// es
	"Marcar todas como le&iacute;das";
#elif L==5	// fr
	"Marquer tous comme lus";
#elif L==6	// gn
	"Marcar todas como le&iacute;das";	// Okoteve traduccin
#elif L==7	// it
	"Segnare tutte come gi&grave; letti";
#elif L==8	// pl
	"Oznacz wszystko jako czytasz";
#elif L==9	// pt
	"Marcar todas como lidas";
#elif L==10	// tr
	"Mark all as read";			// eviri lazim!
#endif

const char *Txt_Marks_management_area =
#if   L==1	// ca
	"Zona d'administraci&oacute; de qualificacions";
#elif L==2	// de
	"Bewertungs-Verwaltung";
#elif L==3	// en
	"Marks management area";
#elif L==4	// es
	"Zona de administraci&oacute;n de calificaciones";
#elif L==5	// fr
	"Zone de gestion de notes";
#elif L==6	// gn
	"Zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Area di gestione dei risultati";
#elif L==8	// pl
	"Obszar zarzadzania znak&oacute;w";
#elif L==9	// pt
	"Zona de gest&atilde;o de notas";
#elif L==10	// tr
	"Marks management area";				// eviri lazim!
#endif

const char *Txt_marks_management_area =
#if   L==1	// ca
	"zona de administraci&oacute;n de calificaciones";	// Necessita traducci
#elif L==2	// de
	"Bewertungs-Verwaltung";
#elif L==3	// en
	"marks management area";
#elif L==4	// es
	"zona de administraci&oacute;n de calificaciones";
#elif L==5	// fr
	"zone de gestion de notes";
#elif L==6	// gn
	"zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==7	// it
	"area di gestione dei risultati";
#elif L==8	// pl
	"obszar zarzadzania znak&oacute;w";
#elif L==9	// pt
	"zona de gest&atilde;o de notas";
#elif L==10	// tr
	"marks management area";				// eviri lazim!
#endif

const char *Txt_Marks_area =
#if   L==1	// ca
	"Zona de qualificacions";
#elif L==2	// de
	"Bewertungs-Bereich";
#elif L==3	// en
	"Marks area";
#elif L==4	// es
	"Zona de calificaciones";
#elif L==5	// fr
	"Zone de notes";
#elif L==6	// gn
	"Zona de calificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Area risultati";
#elif L==8	// pl
	"Powierzchnia znak&oacute;w";
#elif L==9	// pt
	"Zona de notas";
#elif L==10	// tr
	"Marks area";			// eviri lazim!
#endif

const char *Txt_Match =		// of a game
#if   L==1	// ca
	"Partida";
#elif L==2	// de
	"Spiel";
#elif L==3	// en
	"Match";
#elif L==4	// es
	"Partida";
#elif L==5	// fr
	"Match";
#elif L==6	// gn
	"&Ntilde;es&etilde;ha";
#elif L==7	// it
	"Partita";
#elif L==8	// pl
	"Mecz";
#elif L==9	// pt
	"Jogo";
#elif L==10	// tr
	"Ma&ccedil;";
#endif

const char *Txt_MATCH_End =
#if   L==1	// ca
	"Final";
#elif L==2	// de
	"Ende";
#elif L==3	// en
	"End";
#elif L==4	// es
	"Final";
#elif L==5	// fr
	"Fin";
#elif L==6	// gn
	"Paha";
#elif L==7	// it
	"Fine";
#elif L==8	// pl
	"Koniec";
#elif L==9	// pt
	"Fim";
#elif L==10	// tr
	"Son";
#endif

const char *Txt_MATCH_Paused =
#if   L==1	// ca
	"En pausa";
#elif L==2	// de
	"Angehalten";
#elif L==3	// en
	"Paused";
#elif L==4	// es
	"En pausa";
#elif L==5	// fr
	"En pause";
#elif L==6	// gn
	"En pausa";	// Okoteve traduccin
#elif L==7	// it
	"In pausa";
#elif L==8	// pl
	"Wstrzymany";
#elif L==9	// pt
	"Pausado";
#elif L==10	// tr
	"Paused";	// eviri lazim!
#endif

const char *Txt_MATCH_QUESTION_Answered =
#if   L==1	// ca
	"Resposta";
#elif L==2	// de
	"Beantwortet";
#elif L==3	// en
	"Answered";
#elif L==4	// es
	"Respondida";
#elif L==5	// fr
	"R&eacute;pondue";
#elif L==6	// gn
	"Respondida";	// Okoteve traduccin
#elif L==7	// it
	"Risposta";
#elif L==8	// pl
	"Odpowiedziano";
#elif L==9	// pt
	"Respondida";
#elif L==10	// tr
	"Answered";	// eviri lazim!
#endif

const char *Txt_MATCH_QUESTION_Unanswered =
#if   L==1	// ca
	"No resposta";
#elif L==2	// de
	"Unbeantwortete";
#elif L==3	// en
	"Unanswered";
#elif L==4	// es
	"No respondida";
#elif L==5	// fr
	"Pas r&eacute;pondue";
#elif L==6	// gn
	"No respondida";	// Okoteve traduccin
#elif L==7	// it
	"Non risposta";
#elif L==8	// pl
	"Nie odpowiedziano";
#elif L==9	// pt
	"N&atilde;o respondida";
#elif L==10	// tr
	"Unanswered";		// eviri lazim!
#endif

const char *Txt_MATCH_Start =
#if   L==1	// ca
	"Inici";
#elif L==2	// de
	"Start";
#elif L==3	// en
	"Start";
#elif L==4	// es
	"Inicio";
#elif L==5	// fr
	"D&eacute;but";
#elif L==6	// gn
	"&Ntilde;epyr&utilde;";
#elif L==7	// it
	"Inizio";
#elif L==8	// pl
	"Pocz&aogon;tek";
#elif L==9	// pt
	"In&iacute;cio";
#elif L==10	// tr
	"Ba&scedil;lama";
#endif

const char *Txt_Match_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Partida <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Spiel <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Match <strong>%s</strong> removed.";
#elif L==4	// es
	"Partida <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Match <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Partida <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Partita <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Mecz <strong>%s</strong> usuni&eogon;te.";
#elif L==9	// pt
	"Jogo <strong>%s</strong> removido.";
#elif L==10	// tr
	"Match <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_MATCH_respond =
#if   L==1	// ca
	"responen";
#elif L==2	// de
	"antworten";
#elif L==3	// en
	"respond";
#elif L==4	// es
	"responden";
#elif L==5	// fr
	"r&eacute;pondent";
#elif L==6	// gn
	"responden";	// Okoteve traduccin
#elif L==7	// it
	"rispondono";
#elif L==8	// pl
	"odpowiadaj&aogon;";
#elif L==9	// pt
	"respondem";
#elif L==10	// tr
	"respond";	// eviri lazim!
#endif

const char *Txt_Matches =	// of a game
#if   L==1	// ca
	"Partides";
#elif L==2	// de
	"Spiele";
#elif L==3	// en
	"Matches";
#elif L==4	// es
	"Partidas";
#elif L==5	// fr
	"Matchs";
#elif L==6	// gn
	"Partidas";	// Okoteve traduccin
#elif L==7	// it
	"Partite";
#elif L==8	// pl
	"Mecze";
#elif L==9	// pt
	"Partidas";
#elif L==10	// tr
	"Matches";	// eviri lazim!
#endif

const char *Txt_Materials =
#if   L==1	// ca
	"Materials";
#elif L==2	// de
	"Material";
#elif L==3	// en
	"Materials";
#elif L==4	// es
	"Materiales";
#elif L==5	// fr
	"Mat&eacute;riaux";
#elif L==6	// gn
	"Materiales";	// Okoteve traduccin
#elif L==7	// it
	"Materiale";
#elif L==8	// pl
	"Materialy";
#elif L==9	// pt
	"Materiais";
#elif L==10	// tr
	"Materials";	// eviri lazim!
#endif

const char *Txt_Max_levels =
#if   L==1	// ca
	"M&agrave;x. nivells";
#elif L==2	// de
	"Max. Ebenen";
#elif L==3	// en
	"Max. levels";
#elif L==4	// es
	"M&aacute;x. niveles";
#elif L==5	// fr
	"Max. niveaux";
#elif L==6	// gn
	"M&aacute;x. niveles";	// Okoteve traduccin
#elif L==7	// it
	"Max. livelli";
#elif L==8	// pl
	"Max. poziom&oacute;w";
#elif L==9	// pt
	"M&aacute;x. n&iacute;veis";
#elif L==10	// tr
	"Max. levels";		// eviri lazim!
#endif

const char *Txt_Max_BR_students =
#if   L==1	// ca
	"M&agrave;xim<br />d'est.";
#elif L==2	// de
	"Max.<br />Stud.";
#elif L==3	// en
	"Max.<br />stud.";
#elif L==4	// es
	"M&aacute;x.<br />estu.";
#elif L==5	// fr
	"Max.<br />&eacute;tud.";
#elif L==6	// gn
	"M&aacute;x.<br />estu.";	// Okoteve traduccin
#elif L==7	// it
	"Max.<br />stud.";
#elif L==8	// pl
	"Max.<br />stud.";
#elif L==9	// pt
	"M&aacute;x.<br />estu.";
#elif L==10	// tr
	"Max.<br />stud.";		// eviri lazim!
#endif

const char *Txt_Maximum =
#if   L==1	// ca
	"M&agrave;xim";
#elif L==2	// de
	"Maximum";
#elif L==3	// en
	"Maximum";
#elif L==4	// es
	"M&aacute;ximo";
#elif L==5	// fr
	"Maximum";
#elif L==6	// gn
	"M&aacute;ximo";	// Okoteve traduccin
#elif L==7	// it
	"Massimo";
#elif L==8	// pl
	"Maksimum";
#elif L==9	// pt
	"M&aacute;ximo";
#elif L==10	// tr
	"Maksimum";
#endif

const char *Txt_maximum =
#if   L==1	// ca
	"m&agrave;xim";
#elif L==2	// de
	"Maximum";
#elif L==3	// en
	"maximum";
#elif L==4	// es
	"m&aacute;ximo";
#elif L==5	// fr
	"maximum";
#elif L==6	// gn
	"m&aacute;ximo";	// Okoteve traduccin
#elif L==7	// it
	"massimo";
#elif L==8	// pl
	"maksimum";
#elif L==9	// pt
	"m&aacute;ximo";
#elif L==10	// tr
	"maksimum";
#endif

const char *Txt_Maximum_grade =
#if   L==1	// ca
	"Nota m&agrave;xima";
#elif L==2	// de
	"H&ouml;chstnote";
#elif L==3	// en
	"Maximum grade";
#elif L==4	// es
	"Nota m&aacute;xima";
#elif L==5	// fr
	"Note maximale";
#elif L==6	// gn
	"Nota m&aacute;xima";	// Okoteve traduccin
#elif L==7	// it
	"Voto massimo";
#elif L==8	// pl
	"Maksymalna ocena";
#elif L==9	// pt
	"Nota m&aacute;xima";
#elif L==10	// tr
	"Maksimum derece";
#endif

const char *Txt_Members =	// Project members
#if   L==1	// ca
	"Membres";
#elif L==2	// de
	"Mitarbeiter";
#elif L==3	// en
	"Members";
#elif L==4	// es
	"Miembros";
#elif L==5	// fr
	"Membres";
#elif L==6	// gn
	"Miembros";	// Okoteve traduccin
#elif L==7	// it
	"Membri";
#elif L==8	// pl
	"Cz&lstrok;onkowie";
#elif L==9	// pt
	"Membros";
#elif L==10	// tr
	"Members";	// eviri lazim!
#endif

const char *Txt_Menu =
#if   L==1	// ca
	"Men&uacute;";
#elif L==2	// de
	"Men&uuml;";
#elif L==3	// en
	"Menu";
#elif L==4	// es
	"Men&uacute;";
#elif L==5	// fr
	"Menu";
#elif L==6	// gn
	"Men&uacute;";	// Okoteve traduccin
#elif L==7	// it
	"Menu";
#elif L==8	// pl
	"Menu";
#elif L==9	// pt
	"Menu";
#elif L==10	// tr
	"Men&uuml;";
#endif

const char *Txt_MENU_NAMES[Mnu_NUM_MENUS] =
	{
	[Mnu_MENU_HORIZONTAL] =
#if   L==1	// ca
	"Horitzontal"
#elif L==2	// de
	"Horizontal"
#elif L==3	// en
	"Horizontal"
#elif L==4	// es
	"Horizontal"
#elif L==5	// fr
	"Horizontal"
#elif L==6	// gn
	"Horizontal"	// Okoteve traduccin
#elif L==7	// it
	"Orizzontale"
#elif L==8	// pl
	"Poziomy"
#elif L==9	// pt
	"Horizontal"
#elif L==10	// tr
	"Horizontal"	// eviri lazim!
#endif
	,
	[Mnu_MENU_VERTICAL] =
#if   L==1	// ca
	"Vertical"
#elif L==2	// de
	"Vertikal"
#elif L==3	// en
	"Vertical"
#elif L==4	// es
	"Vertical"
#elif L==5	// fr
	"Vertical"
#elif L==6	// gn
	"Vertical"		// Okoteve traduccin
#elif L==7	// it
	"Verticale"
#elif L==8	// pl
	"Pionowy"
#elif L==9	// pt
	"Vertical"
#elif L==10	// tr
	"Vertical"	// eviri lazim!
#endif
	};

const char *Txt_MENU_TITLE[Tab_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB] =
{
[TabStr] =
	{
	//  0: ActFrmLogIn
#if   L==1	// ca
	"Entrar"
#elif L==2	// de
	"Einloggen"
#elif L==3	// en
	"Log in"
#elif L==4	// es
	"Entrar"
#elif L==5	// fr
	"S'identifier"
#elif L==6	// gn
	"Entrar"	// Okoteve traduccin
#elif L==7	// it
	"Accesso"
#elif L==8	// pl
	"Zaloguj Si&eogon;"
#elif L==9	// pt
	"Entrar"
#elif L==10	// tr
	"Giri&scedil; yapmak"
#endif
	,
	//  1: ActReqSch
#if   L==1	// ca
	"Cercar"
#elif L==2	// de
	"Suche"
#elif L==3	// en
	"Search"
#elif L==4	// es
	"Buscar"
#elif L==5	// fr
	"Chercher"
#elif L==6	// gn
	"Heka"
#elif L==7	// it
	"Cercare"
#elif L==8	// pl
	"Szuka&cacute;"
#elif L==9	// pt
	"Procurar"
#elif L==10	// tr
	"Aramak"
#endif
	,
	//  2: ActSeeTmlGbl
#if   L==1	// ca
	"Timeline"
#elif L==2	// de
	"Timeline"
#elif L==3	// en
	"Timeline"
#elif L==4	// es
	"Timeline"
#elif L==5	// fr
	"Chronologie"
#elif L==6	// gn
	"Timeline"	// Okoteve traduccin
#elif L==7	// it
	"Timeline"
#elif L==8	// pl
	"Timeline"
#elif L==9	// pt
	"Timeline"
#elif L==10	// tr
	"Zaman &ccedil;izelgesi"
#endif
	,
	//  3: ActSeeSocPrf
#if   L==1	// ca
	"Perfils"
#elif L==2	// de
	"Profiles"
#elif L==3	// en
	"Profiles"
#elif L==4	// es
	"Perfiles"
#elif L==5	// fr
	"Profils"
#elif L==6	// gn
	"Che"
#elif L==7	// it
	"Profili"
#elif L==8	// pl
	"Profile"
#elif L==9	// pt
	"Perfiles"
#elif L==10	// tr
	"Profiller"
#endif
	,
	//  4: ActSeeCal
#if   L==1	// ca
	"Calendari"
#elif L==2	// de
	"Kalender"
#elif L==3	// en
	"Calendar"
#elif L==4	// es
	"Calendario"
#elif L==5	// fr
	"Calendrier"
#elif L==6	// gn
	"Arapapaha"
#elif L==7	// it
	"Calendario"
#elif L==8	// pl
	"Kalendarz"
#elif L==9	// pt
	"Calend&aacute;rio"
#elif L==10	// tr
	"Takvim"
#endif
	,
	//  5: ActSeeNtf
#if   L==1	// ca
	"Notificacions"
#elif L==2	// de
	"Anmeldungen"
#elif L==3	// en
	"Notifications"
#elif L==4	// es
	"Notificaciones"
#elif L==5	// fr
	"Notifications"
#elif L==6	// gn
	"Momarandu"
#elif L==7	// it
	"Notifiche"
#elif L==8	// pl
	"Powiadomienia"
#elif L==9	// pt
	"Notifica&ccedil;&otilde;es"
#elif L==10	// tr
	"Bildirimler"
#endif
	},
[TabSys] =
	{
	//  0: ActSeeSysInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	//  1: ActSeeCty
#if   L==1	// ca
	"Pa&iuml;sos"
#elif L==2	// de
	"L&auml;nder"
#elif L==3	// en
	"Countries"
#elif L==4	// es
	"Pa&iacute;ses"
#elif L==5	// fr
	"Pays"
#elif L==6	// gn
	"Tet&atilde;"
#elif L==7	// it
	"Paesi"
#elif L==8	// pl
	"Kraje"
#elif L==9	// pt
	"Pa&iacute;ses"
#elif L==10	// tr
	"&Uuml;lkeler"
#endif
	,
	//  2: ActSeePen
#if   L==1	// ca
	"Pendents"
#elif L==2	// de
	"Anh&auml;ngig"
#elif L==3	// en
	"Pending"
#elif L==4	// es
	"Pendientes"
#elif L==5	// fr
	"En&nbsp;attente"
#elif L==6	// gn
	"Pendientes"	// Okoteve traduccin
#elif L==7	// it
	"In&nbsp;attesa"
#elif L==8	// pl
	"Oczekuj&aogon;cy"
#elif L==9	// pt
	"Pendentes"
#elif L==10	// tr
	"Bekleyen"
#endif
	,
	//  3: ActSeeLnk
#if   L==1	// ca
	"Enlla&ccedil;os"
#elif L==2	// de
	"Links"
#elif L==3	// en
	"Links"
#elif L==4	// es
	"Enlaces"
#elif L==5	// fr
	"Liens"
#elif L==6	// gn
	"Ambu&eacute;re"
#elif L==7	// it
	"Link"
#elif L==8	// pl
	"Linki"
#elif L==9	// pt
	"Liga&ccedil;&otilde;es"
#elif L==10	// tr
	"Ba&gbreve;lant&inodot;lar"
#endif
	,
	//  4: ActSeePlg
#if   L==1	// ca
	"Complements"
#elif L==2	// de
	"Plugins"
#elif L==3	// en
	"Plugins"
#elif L==4	// es
	"Complementos"
#elif L==5	// fr
	"Plugins"
#elif L==6	// gn
	"Complementos"
#elif L==7	// it
	"Plugin"
#elif L==8	// pl
	"Plugin&oacute;w"
#elif L==9	// pt
	"Plugins"
#elif L==10	// tr
	"Eklentiler"
#endif
	,
	//  5: ActMtn
#if   L==1	// ca
	"Manteniment"
#elif L==2	// de
	"Erhaltung"
#elif L==3	// en
	"Maintenance"
#elif L==4	// es
	"Mantenimiento"
#elif L==5	// fr
	"Entretien"
#elif L==6	// gn
	"&Ntilde;emyatyr&otilde;"
#elif L==7	// it
	"Manutenzione"
#elif L==8	// pl
	"Utrzymanie"
#elif L==9	// pt
	"Manuten&ccedil;&atilde;o"
#elif L==10	// tr
	"Bak&inodot;m onar&inodot;m"
#endif
	},
[TabCty] =
	{
	//  0: ActSeeCtyInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	//  1: ActSeeIns
#if   L==1	// ca
	"Institucions"
#elif L==2	// de
	"Hochschulen"
#elif L==3	// en
	"Institutions"
#elif L==4	// es
	"Instituciones"
#elif L==5	// fr
	"&Eacute;tablissements"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzioni"
#elif L==8	// pl
	"Instytucje"
#elif L==9	// pt
	"Institu&ccedil;&otilde;es"
#elif L==10	// tr
	"Kurumlar"
#endif
	},
[TabIns] =
	{
	//  0: ActSeeInsInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	//  1: ActSeeCtr
#if   L==1	// ca
	"Centers"
#elif L==2	// de
	"Lehrinstitute"
#elif L==3	// en
	"Centers"
#elif L==4	// es
	"Centros"
#elif L==5	// fr
	"Centers"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centri"
#elif L==8	// pl
	"Centra"
#elif L==9	// pt
	"Centros"
#elif L==10	// tr
	"Merkezler"
#endif
	,
	//  2: ActSeePlc
#if   L==1	// ca
	"Llocs"
#elif L==2	// de
	"Standorte"
#elif L==3	// en
	"Places"
#elif L==4	// es
	"Lugares"
#elif L==5	// fr
	"Emplacements"
#elif L==6	// gn
	"Tenda"
#elif L==7	// it
	"Localit&grave;"
#elif L==8	// pl
	"Miejsca"
#elif L==9	// pt
	"Localiza&ccedil;oes"
#elif L==10	// tr
	"Yer"
#endif
	,
	//  3: ActSeeDpt
#if   L==1	// ca
	"Departaments"
#elif L==2	// de
	"Abteilungen"
#elif L==3	// en
	"Departments"
#elif L==4	// es
	"Departamentos"
#elif L==5	// fr
	"D&eacute;partements"
#elif L==6	// gn
	"Peh&#7869;"
#elif L==7	// it
	"Dipartimenti"
#elif L==8	// pl
	"Wydzia&lstrok;y"
#elif L==9	// pt
	"Departamentos"
#elif L==10	// tr
	"B&ouml;l&uuml;mler"
#endif
	,
	//  4: ActSeeHld
#if   L==1	// ca
	"Festius"
#elif L==2	// de
	"Feiertage"
#elif L==3	// en
	"Holidays"
#elif L==4	// es
	"Festivos"
#elif L==5	// fr
	"F&eacute;ri&eacute;s"
#elif L==6	// gn
	"Arete"
#elif L==7	// it
	"Festivit&agrave;"
#elif L==8	// pl
	"Noclegi"
#elif L==9	// pt
	"F&eacute;rias"
#elif L==10	// tr
	"Bayram"
#endif
	},
[TabCtr] =
	{
	//  0: ActSeeCtrInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	//  1: ActSeeDeg
#if   L==1	// ca
	"Titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degrees"
#elif L==4	// es
	"Titulaciones"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#elif L==10	// tr
	"Derece"
#endif
	,
	//  2: ActSeeBld
#if   L==1	// ca
	"Edificis"
#elif L==2	// de
	"Geb&auml;ude"
#elif L==3	// en
	"Buildings"
#elif L==4	// es
	"Edificios"
#elif L==5	// fr
	"B&acirc;timents"
#elif L==6	// gn
	"Edificios"	// Okoteve traduccin
#elif L==7	// it
	"Edifici"
#elif L==8	// pl
	"Budynki"
#elif L==9	// pt
	"Edif&iacute;cios"
#elif L==10	// tr
	"Binalar"
#endif
	,
	//  3: ActSeeRoo
#if   L==1	// ca
	"Sales"
#elif L==2	// de
	"R&auml;ume"
#elif L==3	// en
	"Rooms"
#elif L==4	// es
	"Salas"
#elif L==5	// fr
	"Salles"
#elif L==6	// gn
	"Mbo'ehakoty"
#elif L==7	// it
	"Aule"
#elif L==8	// pl
	"Sale"
#elif L==9	// pt
	"Salas"
#elif L==10	// tr
	"Odalar"
#endif
	},
[TabDeg] =
	{
	//  0: ActSeeDegInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	//  1: ActSeeCrs
#if   L==1	// ca
	"Assignatures"
#elif L==2	// de
	"Kursen"
#elif L==3	// en
	"Courses"
#elif L==4	// es
	"Asignaturas"
#elif L==5	// fr
	"Mati&egrave;res"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corsi"
#elif L==8	// pl
	"Kurs&oacute;w"
#elif L==9	// pt
	"Disciplinas"
#elif L==10	// tr
	"Dersler"
#endif
	},
[TabCrs] =
	{
	//  0: ActSeeCrsInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#elif L==10	// tr
	"Bilgi"
#endif
	,
	//  1: ActSeePrg
#if   L==1	// ca
	"Programa"
#elif L==2	// de
	"Programm"
#elif L==3	// en
	"Program"
#elif L==4	// es
	"Programa"
#elif L==5	// fr
	"Programme"
#elif L==6	// gn
	"Programa"	// Okoteve traduccin
#elif L==7	// it
	"Programma"
#elif L==8	// pl
	"Program"
#elif L==9	// pt
	"Programa"
#elif L==10	// tr
	"Program&inodot;"
#endif
	,
	//  2: ActSeeTchGui
#if   L==1	// ca
	"Guia"
#elif L==2	// de
	"Hinweise"
#elif L==3	// en
	"Guide"
#elif L==4	// es
	"Gu&iacute;a"
#elif L==5	// fr
	"Guide"
#elif L==6	// gn
	"Gu&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"Guida"
#elif L==8	// pl
	"Podr&eogon;cznik"
#elif L==9	// pt
	"Guia"
#elif L==10	// tr
	"K&inodot;lavuz"
#endif
	,
	//  3: ActSeeSyl
#if   L==1	// ca
	"Temari"
#elif L==2	// de
	"Studienplan"
#elif L==3	// en
	"Topics"
#elif L==4	// es
	"Temario"
#elif L==5	// fr
	"Plan"
#elif L==6	// gn
	"Temario"	// Okoteve traduccin
#elif L==7	// it
	"Temi"
#elif L==8	// pl
	"Sylabus"
#elif L==9	// pt
	"Conte&uacute;do"
#elif L==10	// tr
	"Konular"
#endif
	,
	//  4: ActSeeBib
#if   L==1	// ca
	"Bibliografia"
#elif L==2	// de
	"Literatur"
#elif L==3	// en
	"Bibliography"
#elif L==4	// es
	"Bibliograf&iacute;a"
#elif L==5	// fr
	"Bibliographie"
#elif L==6	// gn
	"Arandukapurupyre"
#elif L==7	// it
	"Bibliografia"
#elif L==8	// pl
	"Bibliografia"
#elif L==9	// pt
	"Bibliografia"
#elif L==10	// tr
	"Bibliyografya"
#endif
	,
	//  5: ActSeeFAQ
#if   L==1	// ca
	"FAQ"
#elif L==2	// de
	"FAQ"
#elif L==3	// en
	"FAQ"
#elif L==4	// es
	"FAQ"
#elif L==5	// fr
	"FAQ"
#elif L==6	// gn
	"FAQ"	// Okoteve traduccin
#elif L==7	// it
	"FAQ"
#elif L==8	// pl
	"FAQ"
#elif L==9	// pt
	"FAQ"
#elif L==10	// tr
	"SSS"
#endif
	,
	//  6: ActSeeCrsLnk
#if   L==1	// ca
	"Enlla&ccedil;os"
#elif L==2	// de
	"Links"
#elif L==3	// en
	"Links"
#elif L==4	// es
	"Enlaces"
#elif L==5	// fr
	"Liens"
#elif L==6	// gn
	"Ambu&eacute;re"
#elif L==7	// it
	"Link"
#elif L==8	// pl
	"Linki"
#elif L==9	// pt
	"Liga&ccedil;&otilde;es"
#elif L==10	// tr
	"Ba&gbreve;lant&inodot;lar"
#endif
	,
	//  7: ActSeeAss
#if   L==1	// ca
	"Avaluaci&oacute;"
#elif L==2	// de
	"Bewertung"
#elif L==3	// en
	"Assessment"
#elif L==4	// es
	"Evaluaci&oacute;n"
#elif L==5	// fr
	"&Eacute;valuation"
#elif L==6	// gn
	"Mbohepy"
#elif L==7	// it
	"Valutazione"
#elif L==8	// pl
	"Ocena"
#elif L==9	// pt
	"Avalia&ccedil;&atilde;o"
#elif L==10	// tr
	"De&gbreve; erlendirme"
#endif
	,
	//  8: ActSeeCrsTT
#if   L==1	// ca
	"Horari"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"Timetable"
#elif L==4	// es
	"Horario"
#elif L==5	// fr
	"Horaire"
#elif L==6	// gn
	"Horario"	// Okoteve traduccin
#elif L==7	// it
	"Orario"
#elif L==8	// pl
	"Plan"
#elif L==9	// pt
	"Hor&aacute;rio"
#elif L==10	// tr
	"Program&inodot;"
#endif
	},
[TabAss] =
	{
	//  0: ActSeeAllAsg
#if   L==1	// ca
	"Activitats"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"Assignments"
#elif L==4	// es
	"Actividades"
#elif L==5	// fr
	"Activit&eacute;s"
#elif L==6	// gn
	"Tar&eacute;a"
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Zadania"
#elif L==9	// pt
	"Atividades"
#elif L==10	// tr
	"&Ouml;devler"
#endif
	,
	//  1: ActSeeAllPrj
#if   L==1	// ca
	"Projectes"
#elif L==2	// de
	"Projekte"
#elif L==3	// en
	"Projects"
#elif L==4	// es
	"Proyectos"
#elif L==5	// fr
	"Projets"
#elif L==6	// gn
	"Proyectos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti"
#elif L==8	// pl
	"Projekty"
#elif L==9	// pt
	"Projetos"
#elif L==10	// tr
	"Projeler"
#endif
	,
	//  2: ActSeeAllExaAnn
#if   L==1	// ca
	"Convocat&ograve;ries"
#elif L==2	// de
	"Pr&uuml;fungsaufrufe"
#elif L==3	// en
	"Calls"
#elif L==4	// es
	"Convocatorias"
#elif L==5	// fr
	"Convocations"
#elif L==6	// gn
	"Convocatorias"	// Okoteve traduccin
#elif L==7	// it
	"Appelli"
#elif L==8	// pl
	"Ogloszenia"
#elif L==9	// pt
	"Chamadas"
#elif L==10	// tr
	"Aramalar"	// eviri lazim!
#endif
	,
	//  3: ActEdiTstQst
#if   L==1	// ca
	"Preguntes"
#elif L==2	// de
	"Fragen"
#elif L==3	// en
	"Questions"
#elif L==4	// es
	"Preguntas"
#elif L==5	// fr
	"Questions"
#elif L==6	// gn
	"Preguntas"	// Okoteve traduccin
#elif L==7	// it
	"Domande"
#elif L==8	// pl
	"Pyta&nacute;"
#elif L==9	// pt
	"Perguntas"
#elif L==10	// tr
	"Sorular"
#endif
	,
	//  4: ActReqTst
#if   L==1	// ca
	"Tests"
#elif L==2	// de
	"Tests"
#elif L==3	// en
	"Tests"
#elif L==4	// es
	"Test"
#elif L==5	// fr
	"Tests"
#elif L==6	// gn
	"Test"	// Okoteve traduccin
#elif L==7	// it
	"Test"
#elif L==8	// pl
	"Test&oacute;w"
#elif L==9	// pt
	"Testes"
#elif L==10	// tr
	"Testler"
#endif
	,
	//  5: ActSeeAllExa
#if   L==1	// ca
	"Ex&agrave;mens"
#elif L==2	// de
	"Pr&uuml;fungen"
#elif L==3	// en
	"Exams"
#elif L==4	// es
	"Ex&aacute;menes"
#elif L==5	// fr
	"Examens"
#elif L==6	// gn
	"Ex&aacute;menes"	// Okoteve traduccin
#elif L==7	// it
	"Esami"
#elif L==8	// pl
	"Egzaminy"
#elif L==9	// pt
	"Exames"
#elif L==10	// tr
	"S&inodot;navlar"
#endif
	,
	//  6: ActSeeAllGam
#if   L==1	// ca
	"Jocs"
#elif L==2	// de
	"Spiele"
#elif L==3	// en
	"Games"
#elif L==4	// es
	"Juegos"
#elif L==5	// fr
	"Jeux"
#elif L==6	// gn
	"Juegos"	// Okoteve traduccin
#elif L==7	// it
	"Giochi"
#elif L==8	// pl
	"Gry"
#elif L==9	// pt
	"Jogos"
#elif L==10	// tr
	"Oyunlar"
#endif
	,
	//  7: ActSeeAllRub
#if   L==1	// ca
	"R&uacute;briques"
#elif L==2	// de
	"Rubriken"
#elif L==3	// en
	"Rubrics"
#elif L==4	// es
	"R&uacute;bricas"
#elif L==5	// fr
	"Rubriques"
#elif L==6	// gn
	"R&uacute;bricas"	// Okoteve traduccin
#elif L==7	// it
	"Rubriche"
#elif L==8	// pl
	"Rubryki"
#elif L==9	// pt
	"Rubricas"
#elif L==10	// tr
	"De&gbreve;erlendirme"
#endif
	},
[TabFil] =
	{
	//  0: ActSeeAdmDocIns
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#elif L==10	// tr
	"Belgeler"
#endif
	,
	//  1: ActAdmShaIns
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#elif L==10	// tr
	"Payla&scedil;&inodot;lan"
#endif
	,
	//  2: ActSeeAdmDocCtr
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#elif L==10	// tr
	"Belgeler"
#endif
	,
	//  3: ActAdmShaCtr
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#elif L==10	// tr
	"Payla&scedil;&inodot;lan"
#endif
	,
	//  4: ActSeeAdmDocDeg
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#elif L==10	// tr
	"Belgeler"
#endif
	,
	//  5: ActAdmShaDeg
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#elif L==10	// tr
	"Payla&scedil;&inodot;lan"
#endif
	,
	//  6: ActSeeAdmDocCrsGrp
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#elif L==10	// tr
	"Belgeler"
#endif
	,
	//  7: ActAdmTchCrsGrp
#if   L==1	// ca
	"Privats"
#elif L==2	// de
	"Private"
#elif L==3	// en
	"Private"
#elif L==4	// es
	"Privados"
#elif L==5	// fr
	"Priv&eacute;s"
#elif L==6	// gn
	"Privados"	// Okoteve traduccin
#elif L==7	// it
	"Privati"
#elif L==8	// pl
	"Prywatne"
#elif L==9	// pt
	"Privados"
#elif L==10	// tr
	"&Ouml;zel"
#endif
	,
	//  8: ActAdmShaCrsGrp
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#elif L==10	// tr
	"Payla&scedil;&inodot;lan"
#endif
	,
	//  9: ActAdmAsgWrkUsr
#if   L==1	// ca
	"Treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"Homework"
#elif L==4	// es
	"Trabajos"
#elif L==5	// fr
	"Travaux"
#elif L==6	// gn
	"Tembiapo"
#elif L==7	// it
	"Lavori"
#elif L==8	// pl
	"Zadanie"
#elif L==9	// pt
	"Trabalhos"
#elif L==10	// tr
	"Ev &ouml;devi"
#endif
	,
	// 10: ActReqAsgWrkCrs
#if   L==1	// ca
	"Treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"Homework"
#elif L==4	// es
	"Trabajos"
#elif L==5	// fr
	"Travaux"
#elif L==6	// gn
	"Tembiapo"
#elif L==7	// it
	"Lavori"
#elif L==8	// pl
	"Zadanie"
#elif L==9	// pt
	"Trabalhos"
#elif L==10	// tr
	"Ev &ouml;devi"
#endif
	,
	// 11: ActSeeMrk
#if   L==1	// ca
	"Qualificacions"
#elif L==2	// de
	"Bewertungen"
#elif L==3	// en
	"Marks"
#elif L==4	// es
	"Calificaciones"
#elif L==5	// fr
	"Notes"
#elif L==6	// gn
	"Calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Risultati"
#elif L==8	// pl
	"Marks"
#elif L==9	// pt
	"Notas"
#elif L==10	// tr
	"I&scedil;aretler"
#endif
	,
	// 12: ActAdmBrf
#if   L==1	// ca
	"Malet&iacute;"
#elif L==2	// de
	"Aktentasche"
#elif L==3	// en
	"Briefcase"
#elif L==4	// es
	"Malet&iacute;n"
#elif L==5	// fr
	"Porte-documents"
#elif L==6	// gn
	"Malet&iacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Cartella"
#elif L==8	// pl
	"Teczka"
#elif L==9	// pt
	"Pasta"
#elif L==10	// tr
	"I&scedil; &ccedil;antas&inodot;"
#endif
	},
[TabUsr] =
	{
	//  0: ActReqSelGrp
#if   L==1	// ca
	"Grups"
#elif L==2	// de
	"Gruppen"
#elif L==3	// en
	"Groups"
#elif L==4	// es
	"Grupos"
#elif L==5	// fr
	"Groupes"
#elif L==6	// gn
	"Aty"
#elif L==7	// it
	"Gruppi"
#elif L==8	// pl
	"Grupy"
#elif L==9	// pt
	"Grupos"
#elif L==10	// tr
	"Gruplar"
#endif
	,
	//  1: ActLstStd
#if   L==1	// ca
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'eku&eacute;ra"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Student&oacute;w"
#elif L==9	// pt
	"Estudantes"
#elif L==10	// tr
	"&Ouml;&gbreve;renciler"
#endif
	,
	//  2: ActLstTch
#if   L==1	// ca
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	,
	//  3: ActLstOth
#if   L==1	// ca
	"Altres"
#elif L==2	// de
	"Andere"
#elif L==3	// en
	"Others"
#elif L==4	// es
	"Otros"
#elif L==5	// fr
	"Autres"
#elif L==6	// gn
	"Otros"		// Okoteve traduccin
#elif L==7	// it
	"Altri"
#elif L==8	// pl
	"Inne"
#elif L==9	// pt
	"Outros"
#elif L==10	// tr
	"Di&gbreve;erleri"
#endif
	,
	//  4: ActSeeAllAtt
#if   L==1	// ca
	"Assist&egrave;ncia"
#elif L==2	// de
	"Anwesenheit"
#elif L==3	// en
	"Attendance"
#elif L==4	// es
	"Asistencia"
#elif L==5	// fr
	"Pr&eacute;sence"
#elif L==6	// gn
	"Asistencia"	// Okoteve traduccin
#elif L==7	// it
	"Frequenza"
#elif L==8	// pl
	"Frekwencji"
#elif L==9	// pt
	"Presen&ccedil;a"
#elif L==10	// tr
	"Kat&inodot;l&inodot;m"
#endif
	,
	//  5: ActReqSignUp
#if   L==1	// ca
	"Inscriure"
#elif L==2	// de
	"Anmeldung"
#elif L==3	// en
	"Sign up"
#elif L==4	// es
	"Inscribirme"
#elif L==5	// fr
	"Signer"
#elif L==6	// gn
	"Inscribirme"	// Okoteve traduccin
#elif L==7	// it
	"Iscrivi"
#elif L==8	// pl
	"Zapisa&cacute;"
#elif L==9	// pt
	"Inscrever"
#elif L==10	// tr
	"&Uuml;ye olmak"
#endif
	,
	//  6: ActSeeSignUpReq
#if   L==1	// ca
	"Peticions"
#elif L==2	// de
	"Ersuchen"
#elif L==3	// en
	"Requests"
#elif L==4	// es
	"Solicitudes"
#elif L==5	// fr
	"Demandes"
#elif L==6	// gn
	"Solicitudes"// Okoteve traduccin
#elif L==7	// it
	"Richieste"
#elif L==8	// pl
	"Upraszanie"
#elif L==9	// pt
	"Solicita&ccedil;&otilde;es"
#elif L==10	// tr
	"Talepleri"
#endif
	,
	//  7: ActLstCon
#if   L==1	// ca
	"Connectats"
#elif L==2	// de
	"Angemeldet"
#elif L==3	// en
	"Connected"
#elif L==4	// es
	"Conectados"
#elif L==5	// fr
	"Connect&eacute;s"
#elif L==6	// gn
	"Conectados"	// Okoteve traduccin
#elif L==7	// it
	"Connessi"
#elif L==8	// pl
	"Online"
#elif L==9	// pt
	"Conectados"
#elif L==10	// tr
	"Ba&gbreve;l&inodot;"
#endif
	},
[TabMsg] =
	{
	//  0: ActSeeAnn
#if   L==1	// ca
	"Anuncis"
#elif L==2	// de
	"Bekanntmachungen"
#elif L==3	// en
	"Announcements"
#elif L==4	// es
	"Anuncios"
#elif L==5	// fr
	"Annonces"
#elif L==6	// gn
	"Anuncios"	// Okoteve traduccin
#elif L==7	// it
	"Annunci"
#elif L==8	// pl
	"Og&lstrok;oszenia"
#elif L==9	// pt
	"An&uacute;ncios"
#elif L==10	// tr
	"Duyurular"
#endif
	,
	//  1: ActSeeAllNot
#if   L==1	// ca
	"Avisos"
#elif L==2	// de
	"Ank&uuml;ndigungen"
#elif L==3	// en
	"Notices"
#elif L==4	// es
	"Avisos"
#elif L==5	// fr
	"Avis"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Avvisi"
#elif L==8	// pl
	"Ogloszenia"
#elif L==9	// pt
	"Avisos"
#elif L==10	// tr
	"Bildirimler"
#endif
	,
	//  2: ActSeeFor
#if   L==1	// ca
	"F&ograve;rums"
#elif L==2	// de
	"Foren"
#elif L==3	// en
	"Forums"
#elif L==4	// es
	"Foros"
#elif L==5	// fr
	"Forums"
#elif L==6	// gn
	"Foros"	// Okoteve traduccin
#elif L==7	// it
	"Forum"
#elif L==8	// pl
	"Forum"
#elif L==9	// pt
	"F&oacute;runs"
#elif L==10	// tr
	"Forumlar"
#endif
	,
	//  3: ActSeeChtRms
#if   L==1	// ca
	"Xat"
#elif L==2	// de
	"Chat"
#elif L==3	// en
	"Chat"
#elif L==4	// es
	"Chat"
#elif L==5	// fr
	"Chat"
#elif L==6	// gn
	"&Ntilde;omongeta"
#elif L==7	// it
	"Chat"
#elif L==8	// pl
	"Chat"
#elif L==9	// pt
	"Chat"
#elif L==10	// tr
	"Sohbet"
#endif
	,
	//  4: ActMsgUsr
#if   L==1	// ca
	"Missatges"
#elif L==2	// de
	"Nachrichten"
#elif L==3	// en
	"Messages"
#elif L==4	// es
	"Mensajes"
#elif L==5	// fr
	"Messages"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Messaggi"
#elif L==8	// pl
	"Wiadomo&sacute;ci"
#elif L==9	// pt
	"Mensagens"
#elif L==10	// tr
	"Mesajlar"
#endif
	,
	//  5: ActReqMaiUsr
#if   L==1	// ca
	"Correu"
#elif L==2	// de
	"E-Mail"
#elif L==3	// en
	"Email"
#elif L==4	// es
	"Correo"
#elif L==5	// fr
	"Courrier"
#elif L==6	// gn
	"Correo"	// Okoteve traduccin
#elif L==7	// it
	"Email"
#elif L==8	// pl
	"Email"
#elif L==9	// pt
	"Email"
#elif L==10	// tr
	"E-posta"
#endif
	},
[TabAna] =
	{
	//  0: ActReqUseGbl
#if   L==1	// ca
	"Xifres"
#elif L==2	// de
	"Zahlen"
#elif L==3	// en
	"Figures"
#elif L==4	// es
	"Cifras"
#elif L==5	// fr
	"Chiffres"
#elif L==6	// gn
	"Cifras"	// Okoteve traduccin
#elif L==7	// it
	"Cifre"
#elif L==8	// pl
	"Cyfry"
#elif L==9	// pt
	"N&uacute;meros"
#elif L==10	// tr
	"Rakamlar"
#endif
	,
	//  1: ActSeePhoDeg
#if   L==1	// ca
	"Titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degrees"
#elif L==4	// es
	"Titulaciones"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#elif L==10	// tr
	"Derece"
#endif
	,
	//  2: ActReqStaCrs
#if   L==1	// ca
	"Indicadors"
#elif L==2	// de
	"Anzeigen"
#elif L==3	// en
	"Indicators"
#elif L==4	// es
	"Indicadores"
#elif L==5	// fr
	"Indicateurs"
#elif L==6	// gn
	"Indicadores"	// Okoteve traduccin
#elif L==7	// it
	"Indicatori"
#elif L==8	// pl
	"Wska&zacute;niki"
#elif L==9	// pt
	"Indicadores"
#elif L==10	// tr
	"G&ouml;stergeler"
#endif
	,
	//  3: ActSeeAllSvy
#if   L==1	// ca
	"Enquestes"
#elif L==2	// de
	"Umfragen"
#elif L==3	// en
	"Surveys"
#elif L==4	// es
	"Encuestas"
#elif L==5	// fr
	"Sondages"
#elif L==6	// gn
	"Encuestas"	// Okoteve traduccin
#elif L==7	// it
	"Sondaggi"
#elif L==8	// pl
	"Ankiety"
#elif L==9	// pt
	"Inqu&eacute;ritos"
#elif L==10	// tr
	"Anketler"
#endif
	,
	//  4: ActReqAccGbl
#if   L==1	// ca
	"Accessos"
#elif L==2	// de
	"Anmeldungen"
#elif L==3	// en
	"Visits"
#elif L==4	// es
	"Accesos"
#elif L==5	// fr
	"Visites"
#elif L==6	// gn
	"Accesos"	// Okoteve traduccin
#elif L==7	// it
	"Accessi"
#elif L==8	// pl
	"Logowanie"
#elif L==9	// pt
	"Visitas"
#elif L==10	// tr
	"Ziyaretler"
#endif
	,
	//  5: ActReqMyUsgRep
#if   L==1	// ca
	"Informe"
#elif L==2	// de
	"Nutzungsbericht"
#elif L==3	// en
	"Report"
#elif L==4	// es
	"Informe"
#elif L==5	// fr
	"Rapport"
#elif L==6	// gn
	"Informe"	// Okoteve traduccin
#elif L==7	// it
	"Rapporto"
#elif L==8	// pl
	"Raport"
#elif L==9	// pt
	"Relat&oacute;rio"
#elif L==10	// tr
	"Bildiri"
#endif
	,
	//  6: ActMFUAct
#if   L==1	// ca
	"Freq&uuml;ents"
#elif L==2	// de
	"H&auml;ufige"
#elif L==3	// en
	"Frequent"
#elif L==4	// es
	"Frecuentes"
#elif L==5	// fr
	"Frequent"
#elif L==6	// gn
	"Jepigu&aacute;va"
#elif L==7	// it
	"Frequenti"
#elif L==8	// pl
	"Cz&eogon;ste"
#elif L==9	// pt
	"Freq&uuml;entes"
#elif L==10	// tr
	"S&inodot;k"
#endif
	},
[TabPrf] =
	{
	//  0: ActFrmRolSes
#if   L==1	// ca
	"Sessi&oacute;"
#elif L==2	// de
	"Sitzung"
#elif L==3	// en
	"Session"
#elif L==4	// es
	"Sesi&oacute;n"
#elif L==5	// fr
	"Session"
#elif L==6	// gn
	"Sesi&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Sessione"
#elif L==8	// pl
	"Sesja"
#elif L==9	// pt
	"Sess&atilde;o"
#elif L==10	// tr
	"Oturum"
#endif
	,
	//  1: ActMyCrs
#if   L==1	// ca
	"Assignatures"
#elif L==2	// de
	"Kursen"
#elif L==3	// en
	"Courses"
#elif L==4	// es
	"Asignaturas"
#elif L==5	// fr
	"Mati&egrave;res"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corsi"
#elif L==8	// pl
	"Kurs&oacute;w"
#elif L==9	// pt
	"Disciplinas"
#elif L==10	// tr
	"Dersler"
#endif
	,
	//  2: ActSeeMyTT
#if   L==1	// ca
	"Horari"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"Timetable"
#elif L==4	// es
	"Horario"
#elif L==5	// fr
	"Horaire"
#elif L==6	// gn
	"Horario"	// Okoteve traduccin
#elif L==7	// it
	"Orario"
#elif L==8	// pl
	"Plan"
#elif L==9	// pt
	"Hor&aacute;rio"
#elif L==10	// tr
	"Program&inodot;"
#endif
	,
	//  3: ActSeeMyAgd
#if   L==1	// ca
	"Agenda"
#elif L==2	// de
	"Agenda"
#elif L==3	// en
	"Agenda"
#elif L==4	// es
	"Agenda"
#elif L==5	// fr
	"Agenda"
#elif L==6	// gn
	"Aporyr&atilde;"
#elif L==7	// it
	"Agenda"
#elif L==8	// pl
	"Pami&eogon;tnik"
#elif L==9	// pt
	"Agenda"
#elif L==10	// tr
	"G&uuml;nl&uuml;k"
#endif
	,
	//  4: ActFrmMyAcc
#if   L==1	// ca
	"Compte"
#elif L==2	// de
	"Benutzerkonto"
#elif L==3	// en
	"Account"
#elif L==4	// es
	"Cuenta"
#elif L==5	// fr
	"Compte"
#elif L==6	// gn
	"Cuenta"	// Okoteve traduccin
#elif L==7	// it
	"Account"
#elif L==8	// pl
	"Konto"
#elif L==9	// pt
	"Conta"
#elif L==10	// tr
	"Hesab&inodot;"
#endif
	,
	//  5: ActReqEdiRecSha
#if   L==1	// ca
	"Fitxa"
#elif L==2	// de
	"Profil"
#elif L==3	// en
	"Record"
#elif L==4	// es
	"Ficha"
#elif L==5	// fr
	"Carte"
#elif L==6	// gn
	"Ficha"	// Okoteve traduccin
#elif L==7	// it
	"Scheda"
#elif L==8	// pl
	"Karta"
#elif L==9	// pt
	"Cart&atilde;o"
#elif L==10	// tr
	"Kart&inodot;"
#endif
	,
	//  6: ActReqEdiSet
#if   L==1	// ca
	"Configuraci&oacute;"
#elif L==2	// de
	"Einstellungen"
#elif L==3	// en
	"Settings"
#elif L==4	// es
	"Ajustes"
#elif L==5	// fr
	"R&eacute;glages"
#elif L==6	// gn
	"Mbohoryha"
#elif L==7	// it
	"Impostazioni"
#elif L==8	// pl
	"Ustawienia"
#elif L==9	// pt
	"Configura&ccedil;&atilde;o"
#elif L==10	// tr
	"Ayarlar"
#endif
	}
};

const char *Txt_Message =
#if   L==1	// ca
	"Missatge";
#elif L==2	// de
	"Nachricht";
#elif L==3	// en
	"Message";
#elif L==4	// es
	"Mensaje";
#elif L==5	// fr
	"Message";
#elif L==6	// gn
	"Marandu";
#elif L==7	// it
	"Messaggio";
#elif L==8	// pl
	"Wiadomosc";
#elif L==9	// pt
	"Mensagem";
#elif L==10	// tr
	"Message";	// eviri lazim!
#endif

const char *Txt_message =
#if   L==1	// ca
	"missatge";
#elif L==2	// de
	"Nachricht";
#elif L==3	// en
	"message";
#elif L==4	// es
	"mensaje";
#elif L==5	// fr
	"message";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"messaggio";
#elif L==8	// pl
	"wiadomosc";
#elif L==9	// pt
	"mensagem";
#elif L==10	// tr
	"message";	// eviri lazim!
#endif

const char *Txt_Message_deleted =
#if   L==1	// ca
	"Missatge eliminat.";
#elif L==2	// de
	"Nachricht entfernt.";
#elif L==3	// en
	"Message deleted.";
#elif L==4	// es
	"Mensaje eliminado.";
#elif L==5	// fr
	"Message supprim&eacute;.";
#elif L==6	// gn
	"Mensaje eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio rimosso.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; usunieta.";
#elif L==9	// pt
	"Mensagem removida.";
#elif L==10	// tr
	"Message deleted.";	// eviri lazim!
#endif

const char *Txt_message_not_sent_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Missatge no enviat a %s.";
#elif L==2	// de
	"Nachricht nicht gesendet, um %s.";
#elif L==3	// en
	"Message not sent to %s.";
#elif L==4	// es
	"Mensaje no enviado a %s.";
#elif L==5	// fr
	"Message non envoy&eacute; &agrave; %s.";
#elif L==6	// gn
	"Mensaje no enviado a %s.";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio non inviato a %s.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; nie zostala wyslana do %s.";
#elif L==9	// pt
	"Mensagem n&atilde;o enviada para %s.";
#elif L==10	// tr
	"Message not sent to %s.";	// eviri lazim!
#endif

const char *Txt_message_sent_to_X_not_notified_by_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Missatge enviat a %s (no notificat per email).";
#elif L==2	// de
	"Nachricht an %s gesendet (nicht per E-Mail benachrichtigt).";
#elif L==3	// en
	"Message sent to %s (not notified by email).";
#elif L==4	// es
	"Mensaje enviado a %s (no notificado por email).";
#elif L==5	// fr
	"Message envoy&eacute; &agrave; %s (non notifi&eacute; par email).";
#elif L==6	// gn
	"Mensaje enviado a %s (no notificado por email).";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio inviato a %s (non notificato per email).";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; wyslana do %s (nie zgloszonych przez email).";
#elif L==9	// pt
	"Mensagem enviada para %s (n&atilde;o notificada por email).";
#elif L==10	// tr
	"Message sent to %s (not notified by email).";		// eviri lazim!
#endif

const char *Txt_message_sent_to_X_notified_by_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Missatge enviat a %s (notificat per email).";
#elif L==2	// de
	"Nachricht an %s gesendet (Benachrichtigung per E-Mail).";
#elif L==3	// en
	"Message sent to %s (notified by email).";
#elif L==4	// es
	"Mensaje enviado a %s (notificado por email).";
#elif L==5	// fr
	"Message envoy&eacute; &agrave; %s (notifi&eacute; par email).";
#elif L==6	// gn
	"Mensaje enviado a %s (notificado por email).";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio inviato a %s (notificato per email).";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; wyslana do %s (zgloszone przez email).";
#elif L==9	// pt
	"Mensagem enviada para %s (notificada por email).";
#elif L==10	// tr
	"Message sent to %s (notified by email).";	// eviri lazim!
#endif

const char *Txt_Messages =
#if   L==1	// ca
	"Missatges";
#elif L==2	// de
	"Nachrichten";
#elif L==3	// en
	"Messages";
#elif L==4	// es
	"Mensajes";
#elif L==5	// fr
	"Messages";
#elif L==6	// gn
	"Marandu";
#elif L==7	// it
	"Messaggi";
#elif L==8	// pl
	"Wiadomo&sacute;ci";
#elif L==9	// pt
	"Mensagens";
#elif L==10	// tr
	"Mesajlar";
#endif

const char *Txt_messages =
#if   L==1	// ca
	"missatges";
#elif L==2	// de
	"Nachrichten";
#elif L==3	// en
	"messages";
#elif L==4	// es
	"mensajes";
#elif L==5	// fr
	"messages";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"messaggi";
#elif L==8	// pl
	"wiadomo&sacute;ci";
#elif L==9	// pt
	"mensagens";
#elif L==10	// tr
	"messages";	// eviri lazim!
#endif

const char *Txt_X_messages_have_been_deleted =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han suprimit %u missatges.";
#elif L==2	// de
	"%u Nachrichten wurden gel&ouml;scht.";
#elif L==3	// en
	"%u messages have been deleted.";
#elif L==4	// es
	"Se han eliminado %u mensajes.";
#elif L==5	// fr
	"%u messages ont &eacute;t&eacute; supprim&eacute;s.";
#elif L==6	// gn
	"Se han eliminado %u mensajes.";	// Okoteve traduccin
#elif L==7	// it
	"%u messaggi sono stati rimossi.";
#elif L==8	// pl
	"wiadomosci% u zostaly usuniete.";
#elif L==9	// pt
	"%u mensagens foram removidas.";
#elif L==10	// tr
	"%u messages have been deleted.";	// eviri lazim!
#endif

const char *Txt_Messages_of_THE_USER_X_have_been_deleted =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"S'han eliminat els missatges de <strong>%s</strong>.";
#elif L==2	// de
	"Nachrichten von <strong>%s</strong> wurden gel&ouml;scht.";
#elif L==3	// en
	"Messages of <strong>%s</strong> have been deleted.";
#elif L==4	// es
	"Se han eliminado los mensajes de <strong>%s</strong>.";
#elif L==5	// fr
	"Les messages de <strong>%s</strong> ont &eacute;t&eacute; supprim&eacute;s.";
#elif L==6	// gn
	"Se han eliminado los mensajes de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi de <strong>%s</strong> sono stati rimossi.";
#elif L==8	// pl
	"Wiadomo&sacute;ci od <strong>%s</strong> zostaly usuniete.";
#elif L==9	// pt
	"Mensagens de <strong>%s</strong> foram removidas.";
#elif L==10	// tr
	"Messages of <strong>%s</strong> have been deleted.";		// eviri lazim!
#endif

const char *Txt_message_received =
#if   L==1	// ca
	"missatge rebut";
#elif L==2	// de
	"Nachricht empfangen";
#elif L==3	// en
	"message received";
#elif L==4	// es
	"mensaje recibido";
#elif L==5	// fr
	"message re&ccedil;u";
#elif L==6	// gn
	"mensaje recibido";	// Okoteve traduccin
#elif L==7	// it
	"messaggio ricevuto";
#elif L==8	// pl
	"odebranej wiadomo&sacute;ci";
#elif L==9	// pt
	"mensagem recebida";
#elif L==10	// tr
	"message received";	// eviri lazim!
#endif

const char *Txt_messages_received =
#if   L==1	// ca
	"missatges rebuts";
#elif L==2	// de
	"empfangene Nachrichten";
#elif L==3	// en
	"messages received";
#elif L==4	// es
	"mensajes recibidos";
#elif L==5	// fr
	"messages re&ccedil;us";
#elif L==6	// gn
	"mensajes recibidos";	// Okoteve traduccin
#elif L==7	// it
	"messaggi ricevuti";
#elif L==8	// pl
	"wiadomo&sacute;ci odebrane";
#elif L==9	// pt
	"mensagens recebidas";
#elif L==10	// tr
	"messages received";	// eviri lazim!
#endif

const char *Txt_Messages_received_from_A_COURSE =
#if   L==1	// ca
	"Missatges rebuts des de";
#elif L==2	// de
	"Empfangene Nachrichten aus";
#elif L==3	// en
	"Messages received from";
#elif L==4	// es
	"Mensajes recibidos desde";
#elif L==5	// fr
	"Messages re&ccedil;us de";
#elif L==6	// gn
	"Mensajes recibidos desde";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi ricevuti da";
#elif L==8	// pl
	"Wiadomo&sacute;ci odebrane";
#elif L==9	// pt
	"Mensagens recebidas de";
#elif L==10	// tr
	"Messages received from";	// eviri lazim!
#endif

const char *Txt_message_sent =
#if   L==1	// ca
	"missatge enviat";
#elif L==2	// de
	"Nachricht gesendet";
#elif L==3	// en
	"message sent";
#elif L==4	// es
	"mensaje enviado";
#elif L==5	// fr
	"message envoy&eacute;";
#elif L==6	// gn
	"mensaje enviado";	// Okoteve traduccin
#elif L==7	// it
	"messaggio inviato";
#elif L==8	// pl
	"wiadomo&sacute;&cacute; wys&lstrok;ana";
#elif L==9	// pt
	"mensagem enviada";
#elif L==10	// tr
	"message sent";		// eviri lazim!
#endif

const char *Txt_messages_sent =
#if   L==1	// ca
	"missatges enviats";
#elif L==2	// de
	"gesendete Nachrichten";
#elif L==3	// en
	"messages sent";
#elif L==4	// es
	"mensajes enviados";
#elif L==5	// fr
	"messages envoy&eacute;s";
#elif L==6	// gn
	"mensajes enviados";	// Okoteve traduccin
#elif L==7	// it
	"messaggi inviati";
#elif L==8	// pl
	"wiadomo&sacute;ci wysylane";
#elif L==9	// pt
	"mensagens enviadas";
#elif L==10	// tr
	"messages sent";	// eviri lazim!
#endif

const char *Txt_Messages_sent =
#if   L==1	// ca
	"Missatges enviats";
#elif L==2	// de
	"Gesendete Nachrichten";
#elif L==3	// en
	"Messages sent";
#elif L==4	// es
	"Mensajes enviados";
#elif L==5	// fr
	"Messages envoy&eacute;s";
#elif L==6	// gn
	"Mensajes enviados";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi inviati";
#elif L==8	// pl
	"Wiadomo&sacute;ci wysylane";
#elif L==9	// pt
	"Mensagens enviadas";
#elif L==10	// tr
	"Messages sent";	// eviri lazim!
#endif

const char *Txt_Messages_sent_by_teachers =
#if   L==1	// ca
	"Missatges enviats per professors";
#elif L==2	// de
	"Gesendete Nachrichten, die von Lehrkr&auml;ften";
#elif L==3	// en
	"Messages sent by teachers";
#elif L==4	// es
	"Mensajes enviados por profesores";
#elif L==5	// fr
	"Messages envoy&eacute;s par enseignants";
#elif L==6	// gn
	"Mensajes enviados por profesores";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi inviati dai professori";
#elif L==8	// pl
	"Wiadomo&sacute;ci wysylane przez nauczycieli";
#elif L==9	// pt
	"Mensagens enviadas por professores";
#elif L==10	// tr
	"Messages sent by teachers";		// eviri lazim!
#endif

const char *Txt_Messages_sent_from_A_COURSE =
#if   L==1	// ca
	"Missatges enviats des de";
#elif L==2	// de
	"Gesendete Nachrichten aus";
#elif L==3	// en
	"Messages sent from";
#elif L==4	// es
	"Mensajes enviados desde";
#elif L==5	// fr
	"Messages envoy&eacute;s de";
#elif L==6	// gn
	"Mensajes enviados desde";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi inviati da";
#elif L==8	// pl
	"Wiadomo&sacute;ci wyslane z";
#elif L==9	// pt
	"Mensagens enviadas de";
#elif L==10	// tr
	"Messages sent from";		// eviri lazim!
#endif

const char *Txt_Minimum =
#if   L==1	// ca
	"M&iacute;nim";
#elif L==2	// de
	"Mindest";
#elif L==3	// en
	"Minimum";
#elif L==4	// es
	"M&iacute;nimo";
#elif L==5	// fr
	"Minimum";
#elif L==6	// gn
	"M&iacute;nimo";	// Okoteve traduccin
#elif L==7	// it
	"Minimo";
#elif L==8	// pl
	"Minimum";
#elif L==9	// pt
	"M&iacute;nimo";
#elif L==10	// tr
	"Asgari";
#endif

const char *Txt_minimum =
#if   L==1	// ca
	"m&iacute;nim";
#elif L==2	// de
	"Mindest";
#elif L==3	// en
	"minimum";
#elif L==4	// es
	"m&iacute;nimo";
#elif L==5	// fr
	"minimum";
#elif L==6	// gn
	"m&iacute;nimo";	// Okoteve traduccin
#elif L==7	// it
	"minimo";
#elif L==8	// pl
	"minimum";
#elif L==9	// pt
	"m&iacute;nimo";
#elif L==10	// tr
	"asgari";
#endif

const char *Txt_Minimum_time_seconds_per_question_between_two_tests =
#if   L==1	// ca
	"Temps m&iacute;nim (segons)<br />per pregunta<br />entre dues tests";
#elif L==2	// de
	"Minimale Zeit (Sekunden)<br />pro Frage<br />zwischen zwei Tests";
#elif L==3	// en
	"Minimum time (seconds)<br />per question<br />between two tests";
#elif L==4	// es
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos test";
#elif L==5	// fr
	"Minimum de temps (en secondes)<br />par question<br />entre deux tests";
#elif L==6	// gn
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos test";	// Okoteve traduccin
#elif L==7	// it
	"Minimo tempo (in secondi)<br />per domanda<br />tra due test";
#elif L==8	// pl
	"Minimalny czas (w sekundach)<br />na pytanie miedzy<br />dwoma badaniami";
#elif L==9	// pt
	"Tempo m&iacute;nimo (segundos)<br />por pergunta<br />entre dois testes";
#elif L==10	// tr
	"Minimum time (seconds)<br />per question<br />between two tests";		// eviri lazim!
#endif

const char *Txt_minute =
#if   L==1	// ca
	"minut";
#elif L==2	// de
	"Minute";
#elif L==3	// en
	"minute";
#elif L==4	// es
	"minuto";
#elif L==5	// fr
	"minute";
#elif L==6	// gn
	"aravo'i";
#elif L==7	// it
	"minuto";
#elif L==8	// pl
	"minut&eogon;";
#elif L==9	// pt
	"minuto";
#elif L==10	// tr
	"minute";	// eviri lazim!
#endif

const char *Txt_minutes =
#if   L==1	// ca
	"minuts";
#elif L==2	// de
	"Minute";
#elif L==3	// en
	"minutes";
#elif L==4	// es
	"minutos";
#elif L==5	// fr
	"minutes";
#elif L==6	// gn
	"aravo'i";
#elif L==7	// it
	"minuti";
#elif L==8	// pl
	"minuty";
#elif L==9	// pt
	"minutos";
#elif L==10	// tr
	"minutes";	// eviri lazim!
#endif

const char *Txt_Modify_me_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Modificarme en l'assignatura <strong>%s</strong>";
#elif L==2	// de
	"&Auml;ndern Sie mir im Kurs <strong>%s</strong>";
#elif L==3	// en
	"Modify me in the course <strong>%s</strong>";
#elif L==4	// es
	"Modificarme en la asignatura <strong>%s</strong>";
#elif L==5	// fr
	"Modifier-moi dans la mati&egrave;re <strong>%s</strong>";
#elif L==6	// gn
	"Modificarme en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Modificami utente nel corso <strong>%s</strong>";
#elif L==8	// pl
	"Modyfikacja mnie w kurs <strong>%s</strong>";
#elif L==9	// pt
	"Modificar me na disciplina <strong>%s</strong>";
#elif L==10	// tr
	"Modify me in the course <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Modify_user_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Modificar usuari en l'assignatura <strong>%s</strong>";
#elif L==2	// de
	"&Auml;ndern Benutzer im Kurs <strong>%s</strong>";
#elif L==3	// en
	"Modify user in the course <strong>%s</strong>";
#elif L==4	// es
	"Modificar usuario en la asignatura <strong>%s</strong>";
#elif L==5	// fr
	"Modifier utilisateur dans la mati&egrave;re <strong>%s</strong>";
#elif L==6	// gn
	"Modificar usuario en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Modifica utente nel corso <strong>%s</strong>";
#elif L==8	// pl
	"Modyfikacja u&zdot;ytkownika w kurs <strong>%s</strong>";
#elif L==9	// pt
	"Modificar utilizador na disciplina <strong>%s</strong>";
#elif L==10	// tr
	"Modify user in the course <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Month =
#if   L==1	// ca
	"Mes";
#elif L==2	// de
	"Monat";
#elif L==3	// en
	"Month";
#elif L==4	// es
	"Mes";
#elif L==5	// fr
	"Mois";
#elif L==6	// gn
	"Jasy";
#elif L==7	// it
	"Mese";
#elif L==8	// pl
	"Miesi&aogon;c";
#elif L==9	// pt
	"M&ecirc;s";
#elif L==10	// tr
	"Month";	// eviri lazim!
#endif

const char *Txt_month =
#if   L==1	// ca
	"mes";
#elif L==2	// de
	"Monat";
#elif L==3	// en
	"month";
#elif L==4	// es
	"mes";
#elif L==5	// fr
	"mois";
#elif L==6	// gn
	"jasy";
#elif L==7	// it
	"mese";
#elif L==8	// pl
	"miesi&aogon;c";
#elif L==9	// pt
	"m&ecirc;s";
#elif L==10	// tr
	"month";	// eviri lazim!
#endif

const char *Txt_months =
#if   L==1	// ca
	"mesos";
#elif L==2	// de
	"Monate";
#elif L==3	// en
	"months";
#elif L==4	// es
	"meses";
#elif L==5	// fr
	"mois";
#elif L==6	// gn
	"meses";	// Okoteve traduccin
#elif L==7	// it
	"mesi";
#elif L==8	// pl
	"miesi&aogon;ce";
#elif L==9	// pt
	"meses";
#elif L==10	// tr
	"aylar";
#endif

const char *Txt_MONTHS_SMALL[12] =
	{
#if   L==1	// ca
	"gener"
#elif L==2	// de
	"Januar"
#elif L==3	// en
	"January"
#elif L==4	// es
	"enero"
#elif L==5	// fr
	"janvier"
#elif L==6	// gn
	"jasyte&#297;"
#elif L==7	// it
	"gennaio"
#elif L==8	// pl
	"Stycznia"
#elif L==9	// pt
	"janeiro"
#elif L==10	// tr
	"Ocak"
#endif
	,
#if   L==1	// ca
	"febrer"
#elif L==2	// de
	"Februar"
#elif L==3	// en
	"February"
#elif L==4	// es
	"febrero"
#elif L==5	// fr
	"f&eacute;vrier"
#elif L==6	// gn
	"jasyk&otilde;i"
#elif L==7	// it
	"febbraio"
#elif L==8	// pl
	"Lutego"
#elif L==9	// pt
	"fevereiro"
#elif L==10	// tr
	"&Scedil;ubat"
#endif
	,
#if   L==1	// ca
	"mar&ccedil;"
#elif L==2	// de
	"M;&auml;rz"
#elif L==3	// en
	"March"
#elif L==4	// es
	"marzo"
#elif L==5	// fr
	"mars"
#elif L==6	// gn
	"jasyapy"
#elif L==7	// it
	"marzo"
#elif L==8	// pl
	"Marca"
#elif L==9	// pt
	"mar&ccedil;o"
#elif L==10	// tr
	"Mart"
#endif
	,
#if   L==1	// ca
	"abril"
#elif L==2	// de
	"April"
#elif L==3	// en
	"April"
#elif L==4	// es
	"abril"
#elif L==5	// fr
	"avril"
#elif L==6	// gn
	"jasyrundy"
#elif L==7	// it
	"aprile"
#elif L==8	// pl
	"Kwietnia"
#elif L==9	// pt
	"abril"
#elif L==10	// tr
	"Nisan"
#endif
	,
#if   L==1	// ca
	"maig"
#elif L==2	// de
	"Mai"
#elif L==3	// en
	"May"
#elif L==4	// es
	"mayo"
#elif L==5	// fr
	"mai"
#elif L==6	// gn
	"jasypo"
#elif L==7	// it
	"maggio"
#elif L==8	// pl
	"Maja"
#elif L==9	// pt
	"maio"
#elif L==10	// tr
	"May&inodot;s"
#endif
	,
#if   L==1	// ca
	"juny"
#elif L==2	// de
	"Juni"
#elif L==3	// en
	"June"
#elif L==4	// es
	"junio"
#elif L==5	// fr
	"juin"
#elif L==6	// gn
	"jasypote&#297;"
#elif L==7	// it
	"giugno"
#elif L==8	// pl
	"Czerwca"
#elif L==9	// pt
	"junho"
#elif L==10	// tr
	"Haziran"
#endif
	,
#if   L==1	// ca
	"juliol"
#elif L==2	// de
	"Juli"
#elif L==3	// en
	"July"
#elif L==4	// es
	"julio"
#elif L==5	// fr
	"juillet"
#elif L==6	// gn
	"jasypok&otilde;i"
#elif L==7	// it
	"luglio"
#elif L==8	// pl
	"Lipca"
#elif L==9	// pt
	"julho"
#elif L==10	// tr
	"Temmuz"
#endif
	,
#if   L==1	// ca
	"agost"
#elif L==2	// de
	"August"
#elif L==3	// en
	"August"
#elif L==4	// es
	"agosto"
#elif L==5	// fr
	"ao&ucirc;t"
#elif L==6	// gn
	"jasypoapy"
#elif L==7	// it
	"agosto"
#elif L==8	// pl
	"Sierpnia"
#elif L==9	// pt
	"agosto"
#elif L==10	// tr
	"A&gbreve;ustos"
#endif
	,
#if   L==1	// ca
	"setembre"
#elif L==2	// de
	"September"
#elif L==3	// en
	"September"
#elif L==4	// es
	"septiembre"
#elif L==5	// fr
	"septembre"
#elif L==6	// gn
	"jasyporundy"
#elif L==7	// it
	"settembre"
#elif L==8	// pl
	"Wrze&sacute;nia"
#elif L==9	// pt
	"setembro"
#elif L==10	// tr
	"Eyl&uuml;l"
#endif
	,
#if   L==1	// ca
	"octubre"
#elif L==2	// de
	"Oktober"
#elif L==3	// en
	"October"
#elif L==4	// es
	"octubre"
#elif L==5	// fr
	"octobre"
#elif L==6	// gn
	"jasypa"
#elif L==7	// it
	"ottobre"
#elif L==8	// pl
	"Pa&zacute;dziernika"
#elif L==9	// pt
	"outubro"
#elif L==10	// tr
	"Ekim"
#endif
	,
#if   L==1	// ca
	"novembre"
#elif L==2	// de
	"November"
#elif L==3	// en
	"November"
#elif L==4	// es
	"noviembre"
#elif L==5	// fr
	"novembre"
#elif L==6	// gn
	"jasypate&#297;"
#elif L==7	// it
	"novembre"
#elif L==8	// pl
	"Listopada"
#elif L==9	// pt
	"novembro"
#elif L==10	// tr
	"Kas&inodot;m"
#endif
	,
#if   L==1	// ca
	"desembre"
#elif L==2	// de
	"Dezember"
#elif L==3	// en
	"December"
#elif L==4	// es
	"diciembre"
#elif L==5	// fr
	"d&eacute;cembre"
#elif L==6	// gn
	"jasypak&otilde;i"
#elif L==7	// it
	"dicembre"
#elif L==8	// pl
	"Grudnia"
#elif L==9	// pt
	"dezembro"
#elif L==10	// tr
	"Aral&inodot;k"
#endif
	};

const char *Txt_MONTHS_SMALL_SHORT[12] =
	{
#if   L==1	// ca
	"gen"
#elif L==2	// de
	"Jan"
#elif L==3	// en
	"Jan"
#elif L==4	// es
	"ene"
#elif L==5	// fr
	"jan"
#elif L==6	// gn
	"ene"	// Okoteve traduccin
#elif L==7	// it
	"gen"
#elif L==8	// pl
	"Sty"
#elif L==9	// pt
	"jan"
#elif L==10	// tr
	"Oca"
#endif
	,
#if   L==1	// ca
	"feb"
#elif L==2	// de
	"Feb"
#elif L==3	// en
	"Feb"
#elif L==4	// es
	"feb"
#elif L==5	// fr
	"f&eacute;v"
#elif L==6	// gn
	"feb"	// Okoteve traduccin
#elif L==7	// it
	"feb"
#elif L==8	// pl
	"Lut"
#elif L==9	// pt
	"fev"
#elif L==10	// tr
	"&Scedil;ub"
#endif
	,
#if   L==1	// ca
	"mar"
#elif L==2	// de
	"M;&auml;r"
#elif L==3	// en
	"Mar"
#elif L==4	// es
	"mar"
#elif L==5	// fr
	"mar"
#elif L==6	// gn
	"mar"	// Okoteve traduccin
#elif L==7	// it
	"mar"
#elif L==8	// pl
	"Mar"
#elif L==9	// pt
	"mar"
#elif L==10	// tr
	"Mar"
#endif
	,
#if   L==1	// ca
	"abr"
#elif L==2	// de
	"Apr"
#elif L==3	// en
	"Apr"
#elif L==4	// es
	"abr"
#elif L==5	// fr
	"avr"
#elif L==6	// gn
	"abr"	// Okoteve traduccin
#elif L==7	// it
	"apr"
#elif L==8	// pl
	"Kwi"
#elif L==9	// pt
	"abr"
#elif L==10	// tr
	"Nis"
#endif
	,
#if   L==1	// ca
	"mai"
#elif L==2	// de
	"Mai"
#elif L==3	// en
	"May"
#elif L==4	// es
	"may"
#elif L==5	// fr
	"mai"
#elif L==6	// gn
	"may"	// Okoteve traduccin
#elif L==7	// it
	"mag"
#elif L==8	// pl
	"Maj"
#elif L==9	// pt
	"mai"
#elif L==10	// tr
	"May"
#endif
	,
#if   L==1	// ca
	"jun"
#elif L==2	// de
	"Jun"
#elif L==3	// en
	"Jun"
#elif L==4	// es
	"jun"
#elif L==5	// fr
	"jui"
#elif L==6	// gn
	"jun"	// Okoteve traduccin
#elif L==7	// it
	"giu"
#elif L==8	// pl
	"Cze"
#elif L==9	// pt
	"jun"
#elif L==10	// tr
	"Haz"
#endif
	,
#if   L==1	// ca
	"jul"
#elif L==2	// de
	"Jul"
#elif L==3	// en
	"Jul"
#elif L==4	// es
	"jul"
#elif L==5	// fr
	"jui"
#elif L==6	// gn
	"jul"	// Okoteve traduccin
#elif L==7	// it
	"lug"
#elif L==8	// pl
	"Lip"
#elif L==9	// pt
	"jul"
#elif L==10	// tr
	"Tem"
#endif
	,
#if   L==1	// ca
	"ago"
#elif L==2	// de
	"Aug"
#elif L==3	// en
	"Aug"
#elif L==4	// es
	"ago"
#elif L==5	// fr
	"ao&ucirc;"
#elif L==6	// gn
	"ago"	// Okoteve traduccin
#elif L==7	// it
	"ago"
#elif L==8	// pl
	"Sie"
#elif L==9	// pt
	"ago"
#elif L==10	// tr
	"A&gbreve;u"
#endif
	,
#if   L==1	// ca
	"set"
#elif L==2	// de
	"Sep"
#elif L==3	// en
	"Sep"
#elif L==4	// es
	"sep"
#elif L==5	// fr
	"sep"
#elif L==6	// gn
	"sep"	// Okoteve traduccin
#elif L==7	// it
	"set"
#elif L==8	// pl
	"Wrz"
#elif L==9	// pt
	"set"
#elif L==10	// tr
	"Eyl"
#endif
	,
#if   L==1	// ca
	"oct"
#elif L==2	// de
	"Okt"
#elif L==3	// en
	"Oct"
#elif L==4	// es
	"oct"
#elif L==5	// fr
	"oct"
#elif L==6	// gn
	"oct"	// Okoteve traduccin
#elif L==7	// it
	"ott"
#elif L==8	// pl
	"Pa&zacute;"
#elif L==9	// pt
	"out"
#elif L==10	// tr
	"Eki"
#endif
	,
#if   L==1	// ca
	"nov"
#elif L==2	// de
	"Nov"
#elif L==3	// en
	"Nov"
#elif L==4	// es
	"nov"
#elif L==5	// fr
	"nov"
#elif L==6	// gn
	"nov"	// Okoteve traduccin
#elif L==7	// it
	"nov"
#elif L==8	// pl
	"Lis"
#elif L==9	// pt
	"nov"
#elif L==10	// tr
	"Kas"
#endif
	,
#if   L==1	// ca
	"des"
#elif L==2	// de
	"Dez"
#elif L==3	// en
	"Dec"
#elif L==4	// es
	"dic"
#elif L==5	// fr
	"d&eacute;c"
#elif L==6	// gn
	"dic"	// Okoteve traduccin
#elif L==7	// it
	"dic"
#elif L==8	// pl
	"Gru"
#elif L==9	// pt
	"dez"
#elif L==10	// tr
	"Ara"
#endif
	};

const char *Txt_Movement_not_allowed =
#if   L==1	// ca
	"Moviment no perm&egrave;s";
#elif L==2	// de
	"Bewegung nicht erlaubt";
#elif L==3	// en
	"Move up not allowed";
#elif L==4	// es
	"Movimiento no permitido";
#elif L==5	// fr
	"Mouvement pas autoris&eacute;";
#elif L==6	// gn
	"Movimiento no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Movimento non consentito";
#elif L==8	// pl
	"Ruch nie jest dozwolone";
#elif L==9	// pt
	"Movimento n&atilde;o permitido";
#elif L==10	// tr
	"Move up not allowed	";	// eviri lazim!
#endif

const char *Txt_MSG_Content =	// of a post or message
#if   L==1	// ca
	"Contingut";
#elif L==2	// de
	"Inhalt";
#elif L==3	// en
	"Content";
#elif L==4	// es
	"Contenido";
#elif L==5	// fr
	"Contenu";
#elif L==6	// gn
	"Contenido";	// Okoteve traduccin
#elif L==7	// it
	"Contenuto";
#elif L==8	// pl
	"Tre&sacute;&cacute;";
#elif L==9	// pt
	"Conte&uacute;do";
#elif L==10	// tr
	"Content";	// eviri lazim!
#endif

const char *Txt_MSG_Deleted_without_opening =
#if   L==1	// ca
	"Eliminat sense obrir";
#elif L==2	// de
	"Gel&ouml;scht ohne &Ouml;ffnen";
#elif L==3	// en
	"Deleted without opening";
#elif L==4	// es
	"Eliminado sin abrir";
#elif L==5	// fr
	"Supprim&eacute; sans ouvrir";
#elif L==6	// gn
	"Eliminado sin abrir";		// Okoteve traduccin
#elif L==7	// it
	"Rimosso senza aprire";
#elif L==8	// pl
	"Usuniete bez otwierania";
#elif L==9	// pt
	"Suprimida sem abrir";
#elif L==10	// tr
	"Deleted without opening";	// eviri lazim!
#endif

const char *Txt_MSG_From =
#if   L==1	// ca
	"De";
#elif L==2	// de
	"Von";
#elif L==3	// en
	"From";
#elif L==4	// es
	"De";
#elif L==5	// fr
	"De";
#elif L==6	// gn
	"De";		// Okoteve traduccin
#elif L==7	// it
	"Da";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"De";
#elif L==10	// tr
	"From";		// eviri lazim!
#endif

const char *Txt_MSG_from =
#if   L==1	// ca
	"from";		// Necessita traducci
#elif L==2	// de
	"von";
#elif L==3	// en
	"from";
#elif L==4	// es
	"de";
#elif L==5	// fr
	"de";
#elif L==6	// gn
	"de";		// Okoteve traduccin
#elif L==7	// it
	"da";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"de";
#elif L==10	// tr
	"from";		// eviri lazim!
#endif

const char *Txt_MSG_New =
#if   L==1	// ca
	"Nou";
#elif L==2	// de
	"Neue";
#elif L==3	// en
	"New";
#elif L==4	// es
	"Nuevo";
#elif L==5	// fr
	"Nouveau";
#elif L==6	// gn
	"Nuevo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo";
#elif L==8	// pl
	"nowe";
#elif L==9	// pt
	"Nova";
#elif L==10	// tr
	"New";		// eviri lazim!
#endif

const char *Txt_MSG_Not_replied =
#if   L==1	// ca
	"No respost";
#elif L==2	// de
	"Nicht geantwortet";
#elif L==3	// en
	"Not replied";
#elif L==4	// es
	"No respondido";
#elif L==5	// fr
	"Pas r&eacute;pondu";
#elif L==6	// gn
	"No respondido";	// Okoteve traduccin
#elif L==7	// it
	"Non risposto";
#elif L==8	// pl
	"Nie odpowiedzial";
#elif L==9	// pt
	"N&atilde;o respondida";
#elif L==10	// tr
	"Not replied";		// eviri lazim!
#endif

const char *Txt_MSG_Open =
#if   L==1	// ca
	"Obert";
#elif L==2	// de
	"Ge&ouml;ffneten";
#elif L==3	// en
	"Open";
#elif L==4	// es
	"Abierto";
#elif L==5	// fr
	"Ouvert";
#elif L==6	// gn
	"Ojei";
#elif L==7	// it
	"Aperto";
#elif L==8	// pl
	"otwarte";
#elif L==9	// pt
	"Aberta";
#elif L==10	// tr
	"Open";		// eviri lazim!
#endif

const char *Txt_MSG_Open_and_deleted =
#if   L==1	// ca
	"Obert i eliminat";
#elif L==2	// de
	"Ge&ouml;ffneten und gel&ouml;scht";
#elif L==3	// en
	"Open and deleted";
#elif L==4	// es
	"Abierto y eliminado";
#elif L==5	// fr
	"Ouvert et supprim&eacute;";
#elif L==6	// gn
	"Abierto y eliminado";	// Okoteve traduccin
#elif L==7	// it
	"Aperto e rimosso";
#elif L==8	// pl
	"Open i usuniete";
#elif L==9	// pt
	"Aberta e suprimida";
#elif L==10	// tr
	"Open and deleted";	// eviri lazim!
#endif

const char *Txt_MSG_Replied =
#if   L==1	// ca
	"Respost";
#elif L==2	// de
	"Geantwortet";
#elif L==3	// en
	"Replied";
#elif L==4	// es
	"Respondido";
#elif L==5	// fr
	"R&eacute;pondu";
#elif L==6	// gn
	"Respondido";	// Okoteve traduccin
#elif L==7	// it
	"Risposto";
#elif L==8	// pl
	"Odpowiedziano";
#elif L==9	// pt
	"Respondida";
#elif L==10	// tr
	"Replied";	// eviri lazim!
#endif

const char *Txt_MSG_Sent =
#if   L==1	// ca
	"Enviat";
#elif L==2	// de
	"Gesendet";
#elif L==3	// en
	"Sent";
#elif L==4	// es
	"Enviado";
#elif L==5	// fr
	"Envoy&eacute;";
#elif L==6	// gn
	"Enviado";	// Okoteve traduccin
#elif L==7	// it
	"Inviato";
#elif L==8	// pl
	"Wyslane";
#elif L==9	// pt
	"Enviada";
#elif L==10	// tr
	"Sent";		// eviri lazim!
#endif

const char *Txt_MSG_Sent_and_deleted =
#if   L==1	// ca
	"Enviat i eliminat";
#elif L==2	// de
	"Gesendet und gel&ouml;scht";
#elif L==3	// en
	"Sent and deleted";
#elif L==4	// es
	"Enviado y eliminado";
#elif L==5	// fr
	"Envoy&eacute; et supprim&eacute;";
#elif L==6	// gn
	"Enviado y eliminado";	// Okoteve traduccin
#elif L==7	// it
	"Inviato e rimosso";
#elif L==8	// pl
	"Wyslane i usuniete";
#elif L==9	// pt
	"Enviada e suprimida";
#elif L==10	// tr
	"Sent and deleted";	// eviri lazim!
#endif

const char *Txt_MSG_Subject =
#if   L==1	// ca
	"Assumpte";
#elif L==2	// de
	"Betreff";
#elif L==3	// en
	"Subject";
#elif L==4	// es
	"Asunto";
#elif L==5	// fr
	"Objet";
#elif L==6	// gn
	"Asunto";	// Okoteve traduccin
#elif L==7	// it
	"Oggetto";
#elif L==8	// pl
	"Z zastrzezeniem";
#elif L==9	// pt
	"Assunto";
#elif L==10	// tr
	"Subject";	// eviri lazim!
#endif

const char *Txt_MSG_To =
#if   L==1	// ca
	"Per";
#elif L==2	// de
	"An";
#elif L==3	// en
	"To";
#elif L==4	// es
	"Para";
#elif L==5	// fr
	"Pour";
#elif L==6	// gn
	"Para";	// Okoteve traduccin
#elif L==7	// it
	"A";
#elif L==8	// pl
	"Aby";
#elif L==9	// pt
	"Para";
#elif L==10	// tr
	"To";	// eviri lazim!
#endif

const char *Txt_MSG_Unopened =
#if   L==1	// ca
	"Sense obrir";
#elif L==2	// de
	"Unge&ouml;ffnet";
#elif L==3	// en
	"Unopened";
#elif L==4	// es
	"Sin abrir";
#elif L==5	// fr
	"Non ouvert";
#elif L==6	// gn
	"Sin abrir";	// Okoteve traduccin
#elif L==7	// it
	"Non aperto";
#elif L==8	// pl
	"Otwarte";
#elif L==9	// pt
	"Fechada";
#elif L==10	// tr
	"Unopened";	// eviri lazim!
#endif

const char *Txt_MSGS_Deleted =
#if   L==1	// ca
	"Eliminats";
#elif L==2	// de
	"Gel&ouml;scht";
#elif L==3	// en
	"Deleted";
#elif L==4	// es
	"Eliminados";
#elif L==5	// fr
	"Supprim&eacute;s";
#elif L==6	// gn
	"Eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Rimossi";
#elif L==8	// pl
	"Usuniete";
#elif L==9	// pt
	"Suprimidas";
#elif L==10	// tr
	"Deleted";	// eviri lazim!
#endif

const char *Txt_MSGS_Not_deleted =
#if   L==1	// ca
	"No eliminats";
#elif L==2	// de
	"Nicht Gel&ouml;scht";
#elif L==3	// en
	"Not deleted";
#elif L==4	// es
	"No eliminados";
#elif L==5	// fr
	"Non supprim&eacute;s";
#elif L==6	// gn
	"No eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Non rimossi";
#elif L==8	// pl
	"Nie usuniete";
#elif L==9	// pt
	"N&atilde;o suprimidas";
#elif L==10	// tr
	"Not deleted";		// eviri lazim!
#endif

const char *Txt_MSGS_Received =
#if   L==1	// ca
	"Rebuts";
#elif L==2	// de
	"Empfangene";
#elif L==3	// en
	"Received";
#elif L==4	// es
	"Recibidos";
#elif L==5	// fr
	"Re&ccedil;us";
#elif L==6	// gn
	"Recibidos";	// Okoteve traduccin
#elif L==7	// it
	"Ricevuti";
#elif L==8	// pl
	"Odebrane";
#elif L==9	// pt
	"Recebidas";
#elif L==10	// tr
	"Received";	// eviri lazim!
#endif

const char *Txt_MSGS_Sent =
#if   L==1	// ca
	"Enviats";
#elif L==2	// de
	"Gesendete";
#elif L==3	// en
	"Sent";
#elif L==4	// es
	"Enviados";
#elif L==5	// fr
	"Envoy&eacute;s";
#elif L==6	// gn
	"Enviados";	// Okoteve traduccin
#elif L==7	// it
	"Inviati";
#elif L==8	// pl
	"Wyslane";
#elif L==9	// pt
	"Enviadas";
#elif L==10	// tr
	"Sent";		// eviri lazim!
#endif

const char *Txt_Multimedia =
#if   L==1	// ca
	"Multim&egrave;dia";
#elif L==2	// de
	"Multimedia";
#elif L==3	// en
	"Multimedia";
#elif L==4	// es
	"Multimedia";
#elif L==5	// fr
	"Multim&eacute;dia";
#elif L==6	// gn
	"Multimedia";
#elif L==7	// it
	"Multimedialit&agrave;";
#elif L==8	// pl
	"Multimedia";
#elif L==9	// pt
	"Multim&eacute;dia";
#elif L==10	// tr
	"Multimedia";	// eviri lazim!
#endif

const char *Txt_Multiple_enrolment =		// (to a type of group)
#if   L==1	// ca
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Necessita traducci
#elif L==2	// de
	"Mehrere beitreten?";
#elif L==3	// en
	"Multiple enrolment?";
#elif L==4	// es
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";
#elif L==5	// fr
	"Inscription multiple?";
#elif L==6	// gn
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione multipla?";
#elif L==8	// pl
	"Rejestracja wielu element&oacute;w?";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o m&uacute;ltiplas?";
#elif L==10	// tr
	"Multiple enrolment?";	// eviri lazim!
#endif

const char *Txt_My_agenda =
#if   L==1	// ca
	"La meva agenda";
#elif L==2	// de
	"Mein Organizer";
#elif L==3	// en
	"My agenda";
#elif L==4	// es
	"Mi agenda";
#elif L==5	// fr
	"Mon agenda";
#elif L==6	// gn
	"Che aporyr&atilde;";
#elif L==7	// it
	"La mia agenda";
#elif L==8	// pl
	"Moja pami&eogon;tnik";
#elif L==9	// pt
	"Minha agenda";
#elif L==10	// tr
	"G&uuml;ndemim";
#endif

const char *Txt_My_courses =
#if   L==1	// ca
	"Les meves assignatures";
#elif L==2	// de
	"Meine Kursen";
#elif L==3	// en
	"My courses";
#elif L==4	// es
	"Mis asignaturas";
#elif L==5	// fr
	"Mes mati&egrave;res";
#elif L==6	// gn
	"Che mbo'esyry";
#elif L==7	// it
	"Miei corsi";
#elif L==8	// pl
	"Moje kurs&oacute;w";
#elif L==9	// pt
	"As minha disciplinas";
#elif L==10	// tr
	"Kurslar&inodot;m";
#endif

const char *Txt_my_documents =
#if   L==1	// ca
	"els meus documents";
#elif L==2	// de
	"meine Dokumente";
#elif L==3	// en
	"my documents";
#elif L==4	// es
	"mis documentos";
#elif L==5	// fr
	"mes documents";
#elif L==6	// gn
	"mis documentos";	// Okoteve traduccin
#elif L==7	// it
	"i miei documenti";
#elif L==8	// pl
	"moje dokumenty";
#elif L==9	// pt
	"meus documentos";
#elif L==10	// tr
	"benim d&ouml;k&uuml;manlar&inodot;m";
#endif

const char *Txt_My_frequent_actions =
#if   L==1	// ca
	"Les meves accions freq&uuml;ents";
#elif L==2	// de
	"Meine h&auml;ufigen Handlungen";
#elif L==3	// en
	"My frequent actions";
#elif L==4	// es
	"Mis acciones frecuentes";
#elif L==5	// fr
	"Mes actions fr&eacute;quentes";
#elif L==6	// gn
	"Che rembiapo py'&ytilde;i";
#elif L==7	// it
	"Le mie azioni frequenti";
#elif L==8	// pl
	"Moi cz&eogon;ste dzia&lstrok;ania";
#elif L==9	// pt
	"Minhas a&ccedil;&otilde;es freq&uuml;entes";
#elif L==10	// tr
	"S&inodot;k yapt&inodot;&gbreve;&inodot;m eylemler";
#endif

const char *Txt_My_groups =
#if   L==1	// ca
	"Meus grups";
#elif L==2	// de
	"Meine Gruppen";
#elif L==3	// en
	"My groups";
#elif L==4	// es
	"Mis grupos";
#elif L==5	// fr
	"Mes groupes";
#elif L==6	// gn
	"Mis grupos";	// Okoteve traduccin
#elif L==7	// it
	"Miei gruppi";
#elif L==8	// pl
	"Moje grupy";
#elif L==9	// pt
	"Meus grupos";
#elif L==10	// tr
	"Gruplar&inodot;m";
#endif

const char *Txt_My_public_profile =
#if   L==1	// ca
	"El meu perfil p&uacute;blic";
#elif L==2	// de
	"Mein &ouml;ffentliches Profil";
#elif L==3	// en
	"My public profile";
#elif L==4	// es
	"Mi perfil p&uacute;blico";
#elif L==5	// fr
	"Mon profil public";
#elif L==6	// gn
	"Mi perfil p&uacute;blico";	// Okoteve traduccin
#elif L==7	// it
	"Il mio profilo pubblico";
#elif L==8	// pl
	"M&oacute;j profil publiczny";
#elif L==9	// pt
	"Meu perfil p&uacute;blico";
#elif L==10	// tr
	"Herkese a&ccedil;&inodot;k profilim";
#endif

const char *Txt_My_views =
#if   L==1	// ca
	"Els meus accessos";
#elif L==2	// de
	"Meine Ansichten";
#elif L==3	// en
	"My views";
#elif L==4	// es
	"Mis accesos";
#elif L==5	// fr
	"Mes vues";
#elif L==6	// gn
	"Mis accesos";	// Okoteve traduccin
#elif L==7	// it
	"Miei visualizzazioni";
#elif L==8	// pl
	"Moje ods&lstrok;on";
#elif L==9	// pt
	"Meus acessos";
#elif L==10	// tr
	"G&ouml;r&uuml;&scedil;lerim";
#endif

const char *Txt_Name =
#if   L==1	// ca
	"Nom";
#elif L==2	// de
	"Name";
#elif L==3	// en
	"Name";
#elif L==4	// es
	"Nombre";
#elif L==5	// fr
	"Nom";
#elif L==6	// gn
	"T&eacute;ra";
#elif L==7	// it
	"Nome";
#elif L==8	// pl
	"Nazwa";
#elif L==9	// pt
	"Nome";
#elif L==10	// tr
	"&Idot;sim";
#endif

const char *Txt_New_email =
#if   L==1	// ca
	"Nou correu";
#elif L==2	// de
	"Neues E-Mail";
#elif L==3	// en
	"New email";
#elif L==4	// es
	"Nuevo correo";
#elif L==5	// fr
	"Nouveau courrier";
#elif L==6	// gn
	"Nuevo correo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo email";
#elif L==8	// pl
	"Nowy email";
#elif L==9	// pt
	"Novo email";
#elif L==10	// tr
	"Yeni e-posta";
#endif

const char *Txt_New_FILE_OR_FOLDER =
#if   L==1	// ca
	"Nou";
#elif L==2	// de
	"Neue";
#elif L==3	// en
	"New";
#elif L==4	// es
	"Nuevo";
#elif L==5	// fr
	"Nouveau";
#elif L==6	// gn
	"Nuevo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo";
#elif L==8	// pl
	"Nowe";
#elif L==9	// pt
	"Novo";
#elif L==10	// tr
	"Yeni";
#endif

const char *Txt_New_nickname =
#if   L==1	// ca
	"Nou sobrenom";
#elif L==2	// de
	"Neues Benutzername";
#elif L==3	// en
	"New nickname";
#elif L==4	// es
	"Nuevo apodo";
#elif L==5	// fr
	"Nouveau surnom";
#elif L==6	// gn
	"Nuevo apodo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo nome utente";
#elif L==8	// pl
	"Nowy nick";
#elif L==9	// pt
	"Nova alcunha";
#elif L==10	// tr
	"Yeni takma ad";
#endif

const char *Txt_New_on_PLATFORM_Sign_up =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No t&eacute; compte a %s? Registra't ara";
#elif L==2	// de
	"Neu bei %s? Registriere Dich!";
#elif L==3	// en
	"New on %s? Sign up";
#elif L==4	// es
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";
#elif L==5	// fr
	"Nouveau sur %s? Inscrivez-vous";
#elif L==6	// gn
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo su %s? Iscriviti";
#elif L==8	// pl
	"Pierwszy raz na %s? Zarejestruj si&eogon;";
#elif L==9	// pt
	"Novo no %s? Inscreva-se";
#elif L==10	// tr
	"%s'da yeni misiniz? &Uuml;ye olmak";
#endif

const char *Txt_New_password =
#if   L==1	// ca
	"Nova contrasenya";
#elif L==2	// de
	"Neues Passwort";
#elif L==3	// en
	"New password";
#elif L==4	// es
	"Nueva contrase&ntilde;a";
#elif L==5	// fr
	"Nouveau mot de passe";
#elif L==6	// gn
	"Nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Nuova password";
#elif L==8	// pl
	"Nowe has&lstrok;o";
#elif L==9	// pt
	"Nova senha";
#elif L==10	// tr
	"Yeni &scedil;ifre";
#endif

const char *Txt_New_question =
#if   L==1	// ca
	"Nova pregunta";
#elif L==2	// de
	"Neuer Frage";
#elif L==3	// en
	"New question";
#elif L==4	// es
	"Nueva pregunta";
#elif L==5	// fr
	"Nouvelle question";
#elif L==6	// gn
	"Nueva pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Nuova domanda";
#elif L==8	// pl
	"Nowe pytanie";
#elif L==9	// pt
	"Nova pergunta";
#elif L==10	// tr
	"Yeni soru";
#endif

const char *Txt_New_resource =
#if   L==1	// ca
	"Nou recurs";
#elif L==2	// de
	"Neue Ressource";
#elif L==3	// en
	"New resource";
#elif L==4	// es
	"Nuevo recurso";
#elif L==5	// fr
	"Nouvelle ressource";
#elif L==6	// gn
	"Nuevo recurso";	// Okoteve traduccin
#elif L==7	// it
	"Nuova risorsa";
#elif L==8	// pl
	"Nowy zas&oacute;b";
#elif L==9	// pt
	"Novo recurso";
#elif L==10	// tr
	"Yeni kaynak";
#endif

const char *Txt_New_TIMELINE_comment =
#if   L==1	// ca
	"Nou comentari";
#elif L==2	// de
	"Neue Kommentar";
#elif L==3	// en
	"New comment";
#elif L==4	// es
	"Nuevo comentario";
#elif L==5	// fr
	"Nouveau commentaire";
#elif L==6	// gn
	"Nuevo comentario";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo commento";
#elif L==8	// pl
	"Nowy komentarz";
#elif L==9	// pt
	"Novo coment&aacute;rio";
#elif L==10	// tr
	"Yeni yorum";
#endif

const char *Txt_New_TIMELINE_post =
#if   L==1	// ca
	"Nova publicaci&oacute;";
#elif L==2	// de
	"Neue Beitrag";
#elif L==3	// en
	"New post";
#elif L==4	// es
	"Nueva publicaci&oacute;n";
#elif L==5	// fr
	"Nouveau post";
#elif L==6	// gn
	"Nueva publicaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo post";
#elif L==8	// pl
	"Nowy post";
#elif L==9	// pt
	"Novo post";
#elif L==10	// tr
	"Yeni posta";
#endif

const char *Txt_new_tag =		// Tag used in test
#if   L==1	// ca
	"nou descriptor";
#elif L==2	// de
	"neuen Tag";
#elif L==3	// en
	"new tag";
#elif L==4	// es
	"nuevo descriptor";
#elif L==5	// fr
	"nouveau descripteur";
#elif L==6	// gn
	"nuevo descriptor";	// Okoteve traduccin
#elif L==7	// it
	"nuova etichetta";
#elif L==8	// pl
	"nowy tag";
#elif L==9	// pt
	"novo descritor";
#elif L==10	// tr
	"yeni etiket";
#endif

const char *Txt_NEW_YEAR_GREETING =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Feli&ccedil; %u!";
#elif L==2	// de
	"Happy %u!";
#elif L==3	// en
	"Happy %u!";
#elif L==4	// es
	"&iexcl;Feliz %u!";
#elif L==5	// fr
	"Happy %u!";
#elif L==6	// gn
	"&iexcl;Feliz %u!";	// Okoteve traduccin
#elif L==7	// it
	"Felice %u!";
#elif L==8	// pl
	"Happy %u!";
#elif L==9	// pt
	"Feliz %u!";
#elif L==10	// tr
	"Happy %u!";		// eviri lazim!
#endif

const char *Txt_nick_email_or_ID =
#if   L==1	// ca
	"@sobrenom, correu o ID (DNI/c&egrave;dula)";
#elif L==2	// de
	"@Benutzername, Email oder Ausweis-Nr.";
#elif L==3	// en
	"@nick, email or ID";
#elif L==4	// es
	"@apodo, correo o ID (DNI/c&eacute;dula)";
#elif L==5	// fr
	"@surnom, courrier ou num&eacute;ro d'identit&eacute;";
#elif L==6	// gn
	"@apodo, correo o ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==7	// it
	"@nome-utente, email o numero carta d'identit&agrave;";
#elif L==8	// pl
	"@pseudonim, email lub ID";
#elif L==9	// pt
	"@alcunha, email ou n&ordm; identif.";
#elif L==10	// tr
	"@nick, e-posta veya kimlik";
#endif

const char *Txt_nicks_emails_or_IDs_separated_by_commas =
#if   L==1	// ca
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Necessita traducci
#elif L==2	// de
	"@Benutzernamen, Emails oder Ausweis-Nr. durch Kommas getrennt";
#elif L==3	// en
	"@nicks, emails or IDs separated by commas";
#elif L==4	// es
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";
#elif L==5	// fr
	"@surnoms, courriers ou num&eacute;ros d'identit&eacute; s&eacute;par&eacute;s par des virgules";
#elif L==6	// gn
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Okoteve traduccin
#elif L==7	// it
	"@nomi-utenti, email o numero carte d'identit&agrave; separati da virgole";
#elif L==8	// pl
	"@nicks, emails or IDs separated by commas";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"@alcunhas, emails ou n&ordm;s. identif. separados por v&iacute;rgulas";
#elif L==10	// tr
	"@nicks, emails or IDs separated by commas";				// eviri lazim!
#endif

const char *Txt_Nickname =
#if   L==1	// ca
	"Sobrenom";
#elif L==2	// de
	"Benutzername";
#elif L==3	// en
	"Nickname";
#elif L==4	// es
	"Apodo";
#elif L==5	// fr
	"Surnom";
#elif L==6	// gn
	"Hero";
#elif L==7	// it
	"Nome-utente";
#elif L==8	// pl
	"Pseudonim";
#elif L==9	// pt
	"Alcunha";
#elif L==10	// tr
	"Takma ad";
#endif

const char *Txt_Nickname_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Sobrenom <strong>@%s</strong> eliminat.";
#elif L==2	// de
	"Benutzername <strong>@%s</strong> entfernt.";
#elif L==3	// en
	"Nickname <strong>@%s</strong> removed.";
#elif L==4	// es
	"Apodo <strong>@%s</strong> eliminado.";
#elif L==5	// fr
	"Surnom <strong>@%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Apodo <strong>@%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Nome utente <strong>@%s</strong> rimosso.";
#elif L==8	// pl
	"Pseudonim <strong>@%s</strong> usuniete.";
#elif L==9	// pt
	"Alcunha <strong>@%s</strong> removido.";
#elif L==10	// tr
	"Nickname <strong>@%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_NO =
#if   L==1	// ca
	"NO";
#elif L==2	// de
	"NEIN";
#elif L==3	// en
	"NO";
#elif L==4	// es
	"NO";
#elif L==5	// fr
	"NON";
#elif L==6	// gn
	"NAH&aACUTE;NIRI";
#elif L==7	// it
	"NO";
#elif L==8	// pl
	"NO";
#elif L==9	// pt
	"N&Atilde;O";
#elif L==10	// tr
	"NO";	// eviri lazim!
#endif

const char *Txt_No =
#if   L==1	// ca
	"No";
#elif L==2	// de
	"Nein";
#elif L==3	// en
	"No";
#elif L==4	// es
	"No";
#elif L==5	// fr
	"Non";
#elif L==6	// gn
	"Nah&aacute;niri";
#elif L==7	// it
	"No";
#elif L==8	// pl
	"No";
#elif L==9	// pt
	"N&atilde;o";
#elif L==10	// tr
	"No";	// eviri lazim!
#endif

const char *Txt_No_announcements =
#if   L==1	// ca
	"No hi ha anuncis";
#elif L==2	// de
	"Keine Bekanntmachungen";
#elif L==3	// en
	"No announcements";
#elif L==4	// es
	"No hay anuncios";
#elif L==5	// fr
	"Il n'y a pas d'annonces";
#elif L==6	// gn
	"No hay anuncios";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono annunci";
#elif L==8	// pl
	"Brak og&lstrok;oszenia";
#elif L==9	// pt
	"N&atilde;o h&aacute; an&uacute;ncios";
#elif L==10	// tr
	"No announcements";	// eviri lazim!
#endif

const char *Txt_No_assigned_building =
#if   L==1	// ca
	"Sense edifici assignat";
#elif L==2	// de
	"Kein zugewiesenes Geb&auml;ude";
#elif L==3	// en
	"No assigned building";
#elif L==4	// es
	"Sin edificio asignado";
#elif L==5	// fr
	"Aucun b&acirc;timent assign&eacute;";
#elif L==6	// gn
	"Sin edificio asignado";	// Okoteve traduccin
#elif L==7	// it
	"Nessun edificio assegnato";
#elif L==8	// pl
	"Brak przydzielonej budynek lekcyjnej";
#elif L==9	// pt
	"Nenhum edif&iacute;cio atribu&iacute;do";
#elif L==10	// tr
	"No assigned building";		// eviri lazim!
#endif

const char *Txt_No_assigned_room =
#if   L==1	// ca
	"Sense sala assignada";
#elif L==2	// de
	"Kein zugewiesenes Raum";
#elif L==3	// en
	"No assigned room";
#elif L==4	// es
	"Sin sala asignada";
#elif L==5	// fr
	"Aucune salle assign&eacute;e";
#elif L==6	// gn
	"Sin sala asignada";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna aula assegnata";
#elif L==8	// pl
	"Brak przydzielonej sali lekcyjnej";
#elif L==9	// pt
	"Nenhuma sala atribu&iacute;da";
#elif L==10	// tr
	"No assigned room";	// eviri lazim!
#endif

const char *Txt_No_assignments =
#if   L==1	// ca
	"No hi ha activitats.";
#elif L==2	// de
	"Keine Aufgaben.";
#elif L==3	// en
	"No assignments.";
#elif L==4	// es
	"No hay actividades.";
#elif L==5	// fr
	"Il n'y a pas d'activit&eacute;s.";
#elif L==6	// gn
	"No hay actividades.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono attivit&agrave;.";
#elif L==8	// pl
	"Brak zadania.";
#elif L==9	// pt
	"N&atilde;o h&aacute; atividades.";
#elif L==10	// tr
	"No assignments.";	// eviri lazim!
#endif

const char *Txt_No_banners =
#if   L==1	// ca
	"No hi ha banners.";
#elif L==2	// de
	"Keine Werbebanners.";
#elif L==3	// en
	"No banners.";
#elif L==4	// es
	"No hay banners.";
#elif L==5	// fr
	"Il n'y a pas de banni&eacute;res.";
#elif L==6	// gn
	"No hay banners.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono banners.";
#elif L==8	// pl
	"Brak banners.";
#elif L==9	// pt
	"N&atilde;o h&aacute; banners.";
#elif L==10	// tr
	"No banners.";		// eviri lazim!
#endif

const char *Txt_No_calls_for_exams_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No hi ha convocatries dexamen de <strong>%s</strong>.";
#elif L==2	// de
	"Keine Aufrufe f&uuml;r Pr&uuml;fung von <strong>%s</strong>.";
#elif L==3	// en
	"No calls for exams of <strong>%s</strong>.";
#elif L==4	// es
	"No hay convocatorias de examen de <strong>%s</strong>.";
#elif L==5	// fr
	"Aucune convocation &agrave; un examen de <strong>%s</strong>.";
#elif L==6	// gn
	"No hay convocatorias de examen de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun appello d'esame di <strong>%s</strong>.";
#elif L==8	// pl
	"Brak ogloszen egzamin&oacute;w <strong>%s</strong>.";
#elif L==9	// pt
	"N&atilde;o chamadas para exame de <strong>%s</strong>.";
#elif L==10	// tr
	"No calls for exams of <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_no_course_of_origin =	// Means: "message sent from any course"
#if   L==1	// ca
	"sense assignatura d'origen";
#elif L==2	// de
	"kein Kurs urspr&uuml;nglich";
#elif L==3	// en
	"no course of origin";
#elif L==4	// es
	"sin asignatura de origen";
#elif L==5	// fr
	"pas de mati&egrave;re d'origine";
#elif L==6	// gn
	"sin asignatura de origen";	// Okoteve traduccin
#elif L==7	// it
	"senza corso di origine";
#elif L==8	// pl
	"kurs nie pochodzenia";
#elif L==9	// pt
	"nenhuma disciplina de origem";
#elif L==10	// tr
	"no course of origin";		// eviri lazim!
#endif

const char *Txt_no_course_selected =
#if   L==1	// ca
	"cap assignatura seleccionada";
#elif L==2	// de
	"kein Kurs ausgew&auml;hlt";
#elif L==3	// en
	"no course selected";
#elif L==4	// es
	"ninguna asignatura seleccionada";
#elif L==5	// fr
	"pas de mati&egrave;re choisi";
#elif L==6	// gn
	"ninguna asignatura seleccionada";
#elif L==7	// it
	"nessun corso scelto";
#elif L==8	// pl
	"kurs nie jest wybrany";
#elif L==9	// pt
	"nenhuma disciplina seleccionada";
#elif L==10	// tr
	"no course selected";	// eviri lazim!
#endif

const char *Txt_No_centers =
#if   L==1	// ca
	"No hi ha centers.";
#elif L==2	// de
	"Keine Lehrinstitute.";
#elif L==3	// en
	"No centers.";
#elif L==4	// es
	"No hay centros.";
#elif L==5	// fr
	"Il n'y a pas de centers.";
#elif L==6	// gn
	"No hay centros.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono centri.";
#elif L==8	// pl
	"Brak centra.";
#elif L==9	// pt
	"N&atilde;o h&aacute; centros.";
#elif L==10	// tr
	"No centers.";		// eviri lazim!
#endif

const char *Txt_No_courses =
#if   L==1	// ca
	"No hi ha assignatures.";
#elif L==2	// de
	"Keine Kursen.";
#elif L==3	// en
	"No courses.";
#elif L==4	// es
	"No hay asignaturas.";
#elif L==5	// fr
	"Il n'y a pas de mati&egrave;res.";
#elif L==6	// gn
	"No hay asignaturas.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono laurea.";
#elif L==8	// pl
	"Brak kurs&oacute;w.";
#elif L==9	// pt
	"N&atilde;o h&aacute; disciplinas.";
#elif L==10	// tr
	"No courses.";		// eviri lazim!
#endif

const char *Txt_No_degrees =
#if   L==1	// ca
	"No hi ha titulacions.";
#elif L==2	// de
	"Keine Studieng&auml;nge.";
#elif L==3	// en
	"No degrees.";
#elif L==4	// es
	"No hay titulaciones.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;tudes.";
#elif L==6	// gn
	"No hay titulaciones.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono corsi.";
#elif L==8	// pl
	"Brak stopni.";
#elif L==9	// pt
	"N&atilde;o h&aacute; graus.";
#elif L==10	// tr
	"No degrees.";		// eviri lazim!
#endif

const char *Txt_No_enrolment_requests =
#if   L==1	// ca
	"No hi ha cap sollicitud d'inscripci&oacute;.";
#elif L==2	// de
	"Keine Immatrikulationsanfragen.";
#elif L==3	// en
	"No enrolment requests.";
#elif L==4	// es
	"No hay solicitudes de inscripci&oacute;n.";
#elif L==5	// fr
	"Aucune demande d'inscription.";
#elif L==6	// gn
	"No hay solicitudes de inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna richiesta di iscrizione.";
#elif L==8	// pl
	"Brak pr&oacute;&sacute;b o rejestracj&eogon;.";
#elif L==9	// pt
	"N&atilde;o h&aacute; solicita&ccedil;&otilde;es de inscri&ccedil;&atilde;o.";
#elif L==10	// tr
	"No enrolment requests.";			// eviri lazim!
#endif

const char *Txt_No_events =
#if   L==1	// ca
	"No hi ha esdeveniments.";
#elif L==2	// de
	"Keine Ereignisse.";
#elif L==3	// en
	"No events.";
#elif L==4	// es
	"No hay eventos.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;v&eacute;nements.";
#elif L==6	// gn
	"No hay eventos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono eventi.";
#elif L==8	// pl
	"Brak wydarzen.";
#elif L==9	// pt
	"N&atilde;o h&aacute; eventos.";
#elif L==10	// tr
	"No events.";		// eviri lazim!
#endif

const char *Txt_No_exams =
#if   L==1	// ca
	"No hi ha ex&agrave;mens.";
#elif L==2	// de
	"Keine Pr&uuml;fung.";
#elif L==3	// en
	"No exams.";
#elif L==4	// es
	"No hay ex&aacute;menes.";
#elif L==5	// fr
	"Il n'y a pas d'examens.";
#elif L==6	// gn
	"No hay ex&aacute;menes.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono esame.";
#elif L==8	// pl
	"Brak egzamin&oacute;w.";
#elif L==9	// pt
	"N&atilde;o h&aacute; exames.";
#elif L==10	// tr
	"No exams.";			// eviri lazim!
#endif

const char *Txt_No_file_index_html_found_within_the_ZIP_file =
#if   L==1	// ca
	"No s'ha trobat cap fitxer <em>index.htm/index.html</em>"
	" dins de l'arxiu <em>ZIP</em>.";
#elif L==2	// de
	"Keine Datei <em>index.htm/index.html</em>"
	" in der <em>ZIP</em>-Datei gefunden.";
#elif L==3	// en
	"No file <em>index.htm/index.html</em> found"
	" within the <em>ZIP</em> file.";
#elif L==4	// es
	"No se ha encontrado un archivo <em>index.htm/index.html</em>"
	" dentro del archivo <em>ZIP</em>.";
#elif L==5	// fr
	"Aucun fichier <em>index.htm/index.html</em>"
	" trouv&eacute; dans le fichier <em>ZIP</em>.";
#elif L==6	// gn
	"No se ha encontrado un archivo <em>index.htm/index.html</em>"
	" dentro del archivo <em>ZIP</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun documento <em>index.htm/index.html</em> trovato"
	" nel file <em>ZIP</em>.";
#elif L==8	// pl
	"Nie znaleziono pliku <em>index.htm/index.html</em>"
	" w pliku <em>ZIP</em>.";
#elif L==9	// pt
	"Um arquivo <em>index.htm/index.html</em> n&atilde;o foi encontrado"
	" dentro do arquivo <em>ZIP</em>.";
#elif L==10	// tr
	"No file <em>index.htm/index.html</em> found"
	" within the <em>ZIP</em> file.";	// eviri lazim!
#endif

const char *Txt_No_games =
#if   L==1	// ca
	"No hi ha jocs.";
#elif L==2	// de
	"Keine Spiele.";
#elif L==3	// en
	"No games.";
#elif L==4	// es
	"No hay juegos.";
#elif L==5	// fr
	"Il n'y a pas de jeux.";
#elif L==6	// gn
	"No hay juegos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono giochi.";
#elif L==8	// pl
	"Brak gry.";
#elif L==9	// pt
	"N&atilde;o h&aacute; jogos.";
#elif L==10	// tr
	"No games.";		// eviri lazim!
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'han creat grups a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"Im Kurs <strong>%s</strong> wurden keine Gruppen erstellt.";
#elif L==3	// en
	"No groups have been created in the course <strong>%s</strong>.";
#elif L==4	// es
	"No se han creado grupos en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Aucun groupe n'a &eacute;t&eacute; cr&eacute;&eacute; dans la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"No se han creado grupos en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>.";
#elif L==9	// pt
	"Nenhum grupo foi criado na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"No groups have been created in the course <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X_Therefore_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat cap grup a l'assignatura <strong>%s</strong>."
	" Per tant, els usuaris especificats"
	" seran inscrits/eliminats en/de l'assignatura.";
#elif L==2	// de
	"Im Kurs <strong>%s</strong> wurden keine Gruppen erstellt."
	" Daher werden die angegebenen Benutzer"
	" in den Kurs eingeschrieben/aus diesem entfernt.";
#elif L==3	// en
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users"
	" will be enroled/removed in/from the course.";
#elif L==4	// es
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados"
	" ser&aacute;n inscritos/eliminados en/de la asignatura.";
#elif L==5	// fr
	"Aucun groupe n'a &eacute;t&eacute; cr&eacute;&eacute; dans la mati&egrave;re <strong>%s</strong>."
	" Par cons&eacute;quent, les utilisateurs sp&eacute;cifi&eacute;s"
	" seront inscrits/retir&eacute;s de la mati&egrave;re.";
#elif L==6	// gn
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados"
	" ser&aacute;n inscritos/eliminados en/de la asignatura.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>."
	"Dunque, gli utenti specificati"
	" saranno registrati/rimossi in/dal corso.";
#elif L==8	// pl
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>."
	"Dlatego okreslonych uzytkownik&oacute;w"
	" zostanie zarejestrowany/usuniete w/z kursu.";
#elif L==9	// pt
	"Nenhum grupo foi criado na disciplina <strong>%s</strong>."
	" Portanto, os usu&aacute;rios especificados"
	" ser&atilde;o inscritos/removidos da disciplina.";
#elif L==10	// tr
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users"
	" will be enroled/removed in/from the course.";			// eviri lazim!
#endif

const char *Txt_No_holidays =
#if   L==1	// ca
	"No hi ha festivitats.";
#elif L==2	// de
	"Keine Feiertage.";
#elif L==3	// en
	"No holidays.";
#elif L==4	// es
	"No hay festividades.";
#elif L==5	// fr
	"Pas de festivit&eacute;s.";
#elif L==6	// gn
	"Ndaip&oacute;ri arete.";
#elif L==7	// it
	"Non ci sono festivit&agrave;.";
#elif L==8	// pl
	"&Zdot;adnych &sacute;wi&aogon;t.";
#elif L==9	// pt
	"N&atilde;o h&aacute; feriados.";
#elif L==10	// tr
	"Tatil yok.";
#endif

const char *Txt_No_image_video =	// Without any image
#if   L==1	// ca
	"Sense imatge / v&iacute;deo";
#elif L==2	// de
	"Ohne Abbild / Video";
#elif L==3	// en
	"No image / video";
#elif L==4	// es
	"Sin imagen / v&iacute;deo";
#elif L==5	// fr
	"Pas d'image / vid&eacute;o";
#elif L==6	// gn
	"Sin imagen / v&iacute;deo";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna immagine / video";
#elif L==8	// pl
	"Brak obrazka / wideo";
#elif L==9	// pt
	"Sem imagem / v&iacute;deo";
#elif L==10	// tr
	"No image / video";		// eviri lazim!
#endif

const char *Txt_No_INDEX =	// Short version of "Number" (as an index)...
#if   L==1	// ca
	"N.&ordm;";	// Necessita traducci
#elif L==2	// de
	"Anzahl";
#elif L==3	// en
	"#";
#elif L==4	// es
	"N.&ordm;";
#elif L==5	// fr
	"Nombre";
#elif L==6	// gn
	"Papanga";
#elif L==7	// it
	"Numero";
#elif L==8	// pl
	"Numer";
#elif L==9	// pt
	"N.&ordm;";
#elif L==10	// tr
	"Number";	// eviri lazim!
#endif

const char *Txt_No_information =
#if   L==1	// ca
	"No hi ha informaci&oacute;.";
#elif L==2	// de
	"Keine Informationen.";
#elif L==3	// en
	"No information.";
#elif L==4	// es
	"No hay informaci&oacute;n.";
#elif L==5	// fr
	"Aucune information.";
#elif L==6	// gn
	"No hay informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna informazione.";
#elif L==8	// pl
	"Brak informacji.";
#elif L==9	// pt
	"N&atilde;o h&aacute; informa&ccedil;&atilde;o.";
#elif L==10	// tr
	"No information.";		// eviri lazim!
#endif

const char *Txt_No_institutions =
#if   L==1	// ca
	"No hi ha institucions.";
#elif L==2	// de
	"Keine Hochschulen.";
#elif L==3	// en
	"No institutions.";
#elif L==4	// es
	"No hay instituciones.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;tablissements.";
#elif L==6	// gn
	"No hay instituciones.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono istituzioni.";
#elif L==8	// pl
	"Brak instytucje.";
#elif L==9	// pt
	"N&atilde;o h&aacute; institu&ccedil;&otilde;es.";
#elif L==10	// tr
	"No institutions.";		// eviri lazim!
#endif

const char *Txt_No_links =
#if   L==1	// ca
	"No hi ha enlla&ccedil;os.";
#elif L==2	// de
	"Keine Links.";
#elif L==3	// en
	"No links.";
#elif L==4	// es
	"No hay enlaces.";
#elif L==5	// fr
	"Il n'y a pas de liens.";
#elif L==6	// gn
	"No hay enlaces.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono link.";
#elif L==8	// pl
	"Brak linki.";
#elif L==9	// pt
	"N&atilde;o h&aacute; liga&ccedil;&otilde;es.";
#elif L==10	// tr
	"No links.";		// eviri lazim!
#endif

const char *Txt_No_new_posts =
#if   L==1	// ca
	"No hi ha comentaris nous";
#elif L==2	// de
	"Keine neuen Nachrichten";
#elif L==3	// en
	"No new posts";
#elif L==4	// es
	"No hay comentarios nuevos";
#elif L==5	// fr
	"Pas de nouveaux posts";
#elif L==6	// gn
	"No hay comentarios nuevos";	// Okoteve traduccin
#elif L==7	// it
	"Nessun nuovo post";
#elif L==8	// pl
	"Brak nowych post&oacute;w";
#elif L==9	// pt
	"Sem posts novos";
#elif L==10	// tr
	"No new posts";			// eviri lazim!
#endif

const char *Txt_No_notices =
#if   L==1	// ca
	"No hi ha avisos.";
#elif L==2	// de
	"Keine Ank&uuml;ndigungen.";
#elif L==3	// en
	"No notices.";
#elif L==4	// es
	"No hay avisos.";
#elif L==5	// fr
	"Il n'y a pas d'avis.";
#elif L==6	// gn
	"No hay avisos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono avvisi.";
#elif L==8	// pl
	"Brak ogloszen.";
#elif L==9	// pt
	"N&atilde;o h&aacute; avisos.";
#elif L==10	// tr
	"No notices.";		// eviri lazim!
#endif

const char *Txt_No_projects =
#if   L==1	// ca
	"No hi ha projectes.";
#elif L==2	// de
	"Keine Projekte.";
#elif L==3	// en
	"No projects.";
#elif L==4	// es
	"No hay proyectos.";
#elif L==5	// fr
	"Il n'y a pas de projets.";
#elif L==6	// gn
	"No hay proyectos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono progetti.";
#elif L==8	// pl
	"Brak projekty.";
#elif L==9	// pt
	"N&atilde;o h&aacute; projetos.";
#elif L==10	// tr
	"No projects.";		// eviri lazim!
#endif

const char *Txt_No_questions =
#if   L==1	// ca
	"No hi ha preguntes.";
#elif L==2	// de
	"Keine Fragen.";
#elif L==3	// en
	"No questions.";
#elif L==4	// es
	"No hay preguntas.";
#elif L==5	// fr
	"Il n'y a pas de questions.";
#elif L==6	// gn
	"Ndaip&oacute;ri porandu.";
#elif L==7	// it
	"Non ci sono domande.";
#elif L==8	// pl
	"Brak pyta&nacute;.";
#elif L==9	// pt
	"N&atilde;o existem perguntas.";
#elif L==10	// tr
	"Soru yok.";
#endif

const char *Txt_No_questions_found_matching_your_search_criteria =
#if   L==1	// ca
	"No hi ha preguntes amb el criteri de cerca seleccionat.";
#elif L==2	// de
	"Keine Testfragen gefunden, die mit Ihrer Suchanfrage entsprechen.";
#elif L==3	// en
	"No questions found matching your search criteria.";
#elif L==4	// es
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";
#elif L==5	// fr
	"Aucune question trouv&eacute;e correspondant &agrave; vos crit&egrave;res de recherche.";
#elif L==6	// gn
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono domande trovate con il criterio di ricerca selezionato.";
#elif L==8	// pl
	"Brak pyta&nacute; spelniajacych zadane kryteria wyszukiwania.";
#elif L==9	// pt
	"N&atilde;o perguntas com os seus crit&eacute;rios de pesquisa selecionados.";
#elif L==10	// tr
	"No questions found matching your search criteria.";			// eviri lazim!
#endif

const char *Txt_No_questions_have_been_added =
#if   L==1	// ca
	"No s'ha afegit cap pregunta.";
#elif L==2	// de
	"Es wurden keine Fragen hinzugef&uuml;gt.";
#elif L==3	// en
	"No questions have been added.";
#elif L==4	// es
	"No se ha a&ntilde;adido ninguna pregunta.";
#elif L==5	// fr
	"Aucune question n'a &eacute;t&eacute; ajout&eacute;e.";
#elif L==6	// gn
	"No se ha a&ntilde;adido ninguna pregunta.";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna domanda &egrave; stata aggiunta.";
#elif L==8	// pl
	"Nie dodano &zdot;adnych pyta&nacute;.";
#elif L==9	// pt
	"Nenhuma pergunta foi adicionada.";
#elif L==10	// tr
	"No questions have been added.";		// eviri lazim!
#endif

const char *Txt_No_results =
#if   L==1	// ca
	"No hi ha resultats.";
#elif L==2	// de
	"Keine Ergebnisse.";
#elif L==3	// en
	"No results.";
#elif L==4	// es
	"No hay resultados.";
#elif L==5	// fr
	"Pas de r&eacute;sultats.";
#elif L==6	// gn
	"No hay resultados.";		// Okoteve traduccin
#elif L==7	// it
	"Nessun risultato.";
#elif L==8	// pl
	"Brak wynik&oacute;w.";
#elif L==9	// pt
	"Nenhum resultado.";
#elif L==10	// tr
	"No results.";			// eviri lazim!
#endif

const char *Txt_no_rubric =
#if   L==1	// ca
	"sense r&uacute;brica";
#elif L==2	// de
	"keine Rubrik";
#elif L==3	// en
	"no rubric";
#elif L==4	// es
	"sin r&uacute;brica";
#elif L==5	// fr
	"pas de rubrique";
#elif L==6	// gn
	"ndaip&oacute;ri r&uacute;brica";
#elif L==7	// it
	"nessuna rubrica";
#elif L==8	// pl
	"bez rubryk";
#elif L==9	// pt
	"sem rubrica";
#elif L==10	// tr
	"de&gbreve;erlendirme listesi olmadan";
#endif

const char *Txt_No_rubrics =
#if   L==1	// ca
	"No hi ha r&uacute;briques.";
#elif L==2	// de
	"Keine Rubriken.";
#elif L==3	// en
	"No rubrics.";
#elif L==4	// es
	"No hay r&uacute;bricas.";
#elif L==5	// fr
	"Pas de rubriques.";
#elif L==6	// gn
	"Ndaip&oacute;ri r&uacute;brica.";
#elif L==7	// it
	"Non ci sono rubriche.";
#elif L==8	// pl
	"Brak rubryk.";
#elif L==9	// pt
	"N&atilde;o h&aacute; rubricas.";
#elif L==10	// tr
	"Puan anahtar&inodot; yok.";
#endif

const char *Txt_no_subject =
#if   L==1	// ca
	"sense assumpte";
#elif L==2	// de
	"kein Betreff";
#elif L==3	// en
	"no subject";
#elif L==4	// es
	"sin asunto";
#elif L==5	// fr
	"pas de sujet";
#elif L==6	// gn
	"sin asunto";	// Okoteve traduccin
#elif L==7	// it
	"nessun oggetto";
#elif L==8	// pl
	"bez tematu";
#elif L==9	// pt
	"sem assunto";
#elif L==10	// tr
	"no subject";	// eviri lazim!
#endif

const char *Txt_No_surveys =
#if   L==1	// ca
	"No hi ha enquestes.";
#elif L==2	// de
	"Keine Umfragen.";
#elif L==3	// en
	"No surveys.";
#elif L==4	// es
	"No hay encuestas.";
#elif L==5	// fr
	"Il n'y a pas de sondages.";
#elif L==6	// gn
	"No hay encuestas.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono sondaggi.";
#elif L==8	// pl
	"Brak badania.";
#elif L==9	// pt
	"N&atilde;o h&aacute; inqu&eacute;ritos.";
#elif L==10	// tr
	"No surveys.";		// eviri lazim!
#endif

const char *Txt_no_tags =		// Tags used in test
#if   L==1	// ca
	"sense descriptors";
#elif L==2	// de
	"keine Tags";
#elif L==3	// en
	"no tags";
#elif L==4	// es
	"sin descriptores";
#elif L==5	// fr
	"pas de descripteurs";
#elif L==6	// gn
	"sin descriptores";	// Okoteve traduccin
#elif L==7	// it
	"senza etichette";
#elif L==8	// pl
	"zadnych tag&oacute;w";
#elif L==9	// pt
	"sem descritores";
#elif L==10	// tr
	"no tags";		// eviri lazim!
#endif

const char *Txt_No_teachers_or_students_found =
#if   L==1	// ca
	"No hi ha professors ni estudiants.";
#elif L==2	// de
	"Keine Lehrer oder Studenten gefunden.";
#elif L==3	// en
	"No teachers or students found.";
#elif L==4	// es
	"No hay profesores ni estudiantes.";
#elif L==5	// fr
	"Aucun enseignant ou &eacute;tudiant trouv&eacute;.";
#elif L==6	// gn
	"No hay profesores ni estudiantes.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun professore o studente trovato.";
#elif L==8	// pl
	"Nie znaleziono nauczycieli i uczni&oacute;w.";
#elif L==9	// pt
	"Nenhum professor ou estudante encontrado.";
#elif L==10	// tr
	"No teachers or students found.";	// eviri lazim!
#endif

const char *Txt_No_types_of_degree =
#if   L==1	// ca
	"No hi ha cap tipus de titulaci&oacute;.";
#elif L==2	// de
	"Es gibt keine Abschlussarten.";
#elif L==3	// en
	"No types of degree.";
#elif L==4	// es
	"No hay tipos de titulaci&oacute;n.";
#elif L==5	// fr
	"Il n'y a pas de types d'&eacute;tudes.";
#elif L==6	// gn
	"No hay tipos de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono tipi di lauree.";
#elif L==8	// pl
	"Nie ma rodzaj&oacute;w stopni.";
#elif L==9	// pt
	"N&atilde;o h&aacute; tipos de grau.";
#elif L==10	// tr
	"No types of degree.";			// eviri lazim!
#endif

const char *Txt_No_user_has_been_eliminated =
#if   L==1	// ca
	"No s'ha suprimit cap usuari.";
#elif L==2	// de
	"Es wurde kein Benutzer gel&ouml;scht.";
#elif L==3	// en
	"No user has been eliminated.";
#elif L==4	// es
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==5	// fr
	"Aucun utilisateur n'a &eacute;t&eacute; &eacute;limin&eacute;.";
#elif L==6	// gn
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Zaden uzytkownik nie zostal wyeliminowany.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio foi removido.";
#elif L==10	// tr
	"No user has been eliminated.";			// eviri lazim!
#endif

const char *Txt_No_user_has_been_enroled =
#if   L==1	// ca
	"No s'ha inscrit cap usuari.";
#elif L==2	// de
	"Es wurde kein Benutzer registriert.";
#elif L==3	// en
	"No user has been enroled.";
#elif L==4	// es
	"No se ha inscrito ning&uacute;n usuario.";
#elif L==5	// fr
	"Aucun utilisateur n'a &eacute;t&eacute; inscrit.";
#elif L==6	// gn
	"No se ha inscrito ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente &egrave; stato registrato.";
#elif L==8	// pl
	"Zaden uzytkownik nie zostal zarejestrowany.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio se registrou.";
#elif L==10	// tr
	"No user has been enroled.";			// eviri lazim!
#endif

const char *Txt_No_user_has_been_removed =
#if   L==1	// ca
	"No s'ha suprimit cap usuari.";
#elif L==2	// de
	"Es wurde kein Benutzer entfernt.";
#elif L==3	// en
	"No user has been removed.";
#elif L==4	// es
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==5	// fr
	"Aucun utilisateur n'a &eacute;t&eacute; supprim&eacute;.";
#elif L==6	// gn
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Zaden uzytkownik nie zostal usuniety.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio foi removido.";
#elif L==10	// tr
	"No user has been removed.";			// eviri lazim!
#endif

const char *Txt_No_user_to_whom_you_can_follow_Try_again_later =
#if   L==1	// ca
	"No hi ha cap usuari a qui vost&egrave; pugui seguir."
	" Intenteu-ho m&eacute;s endavant.";
#elif L==2	// de
	"Kein Benutzer dem Sie folgen k&ouml;nnen."
	" Versuchen Sie es sp&auml;ter noch einmal.";
#elif L==3	// en
	"No user to whom you can follow."
	" Try again later.";
#elif L==4	// es
	"No hay ning&uacute;n usuario a quien usted pueda seguir."
	" Int&eacute;ntelo m&aacute;s adelante.";
#elif L==5	// fr
	"Aucun utilisateur auquel vous pouvez suivre."
	" R&eacute;essayez plus tard.";
#elif L==6	// gn
	"No hay ning&uacute;n usuario a quien usted pueda seguir."
	" Int&eacute;ntelo m&aacute;s adelante.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente a cui &egrave; possibile seguire."
	" Riprovare pi&ugrave; tardi.";
#elif L==8	// pl
	"No u&zdot;ytkownika, do kt&oacute;rego mo&zdot;na &sacute;ledzi&cacute;."
	" Spr&oacute;buj ponownie p&oacute;&zacute;niej.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio a quem voc&ecirc; pode seguir."
	" Tente novamente mais tarde.";
#elif L==10	// tr
	"No user to whom you can follow."
	" Try again later.";				// eviri lazim!
#endif

const char *Txt_No_users_found[Rol_NUM_ROLES] =
	{
	[Rol_UNK] =
#if   L==1	// ca
	"No hi ha usuaris."
#elif L==2	// de
	"Keine Benutzer gefunden."
#elif L==3	// en
	"No users found."
#elif L==4	// es
	"No hay usuarios."
#elif L==5	// fr
	"Aucun utilisateur trouv&eacute;."
#elif L==6	// gn
	"No hay usuarios."	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente trovato."
#elif L==8	// pl
	"Nie znaleziono uzytkownik&oacute;w."
#elif L==9	// pt
	"Nenhum utilizador encontrado."
#elif L==10	// tr
	"No users found."	// eviri lazim!
#endif
	,
	[Rol_GST] =
#if   L==1	// ca
	"No hi ha convidats."
#elif L==2	// de
	"Keine Eingeladen gefunden."
#elif L==3	// en
	"No guests found."
#elif L==4	// es
	"No hay invitados."
#elif L==5	// fr
	"Aucun invit&eacute; trouv&eacute;."
#elif L==6	// gn
	"No hay invitados."	// Okoteve traduccin
#elif L==7	// it
	"Nessun ospite trovato."
#elif L==8	// pl
	"Nie znaleziono zaproszeni."
#elif L==9	// pt
	"Nenhum convidado encontrado."
#elif L==10	// tr
	"No guests found."	// eviri lazim!
#endif
	,
	[Rol_USR] =
#if   L==1	// ca
	"No hi ha usuaris."
#elif L==2	// de
	"Keine Benutzer gefunden."
#elif L==3	// en
	"No users found."
#elif L==4	// es
	"No hay usuarios."
#elif L==5	// fr
	"Aucun utilisateur trouv&eacute;."
#elif L==6	// gn
	"No hay usuarios."	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente trovato."
#elif L==8	// pl
	"Nie znaleziono uzytkownik&oacute;w."
#elif L==9	// pt
	"Nenhum utilizador encontrado."
#elif L==10	// tr
	"No users found."	// eviri lazim!
#endif
	,
	[Rol_STD] =
#if   L==1	// ca
	"No hi ha estudiants."
#elif L==2	// de
	"Keine Studenten gefunden."
#elif L==3	// en
	"No students found."
#elif L==4	// es
	"No hay estudiantes."
#elif L==5	// fr
	"Aucun &eacute;tudiant trouv&eacute;."
#elif L==6	// gn
	"No hay estudiantes."	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono studenti."
#elif L==8	// pl
	"Nie znaleziono uczni&oacute;w."
#elif L==9	// pt
	"Nenhum estudante encontrado."
#elif L==10	// tr
	"No students found."	// eviri lazim!
#endif
	,
	[Rol_NET] =
#if   L==1	// ca
	"No hi ha professors no editors."
#elif L==2	// de
	"Keine nicht bearbeiteter Lehrkr&auml;ften gefunden."
#elif L==3	// en
	"No non-editing teachers found."
#elif L==4	// es
	"No hay profesores no editores."
#elif L==5	// fr
	"Aucun enseignant non-&eacute;diteur trouv&eacute;."
#elif L==6	// gn
	"No hay profesores no editores."	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono professori non-editing."
#elif L==8	// pl
	"Nie znaleziono nauczycieli nie edytuj&aogon;cy."
#elif L==9	// pt
	"Nenhum professor n&atilde;o editor encontrado."
#elif L==10	// tr
	"No non-editing teachers found."	// eviri lazim!
#endif
	,
	[Rol_TCH] =
#if   L==1	// ca
	"No hi ha professors."
#elif L==2	// de
	"Keine Lehrkr&auml;ften gefunden."
#elif L==3	// en
	"No teachers found."
#elif L==4	// es
	"No hay profesores."
#elif L==5	// fr
	"Aucun enseignant trouv&eacute;."
#elif L==6	// gn
	"No hay profesores."	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono professori."
#elif L==8	// pl
	"Nie znaleziono nauczycieli."
#elif L==9	// pt
	"Nenhum professor encontrado."
#elif L==10	// tr
	"No teachers found."	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM] =
#if   L==1	// ca
	"No hi ha administradors."
#elif L==2	// de
	"Keine Administratoren gefunden."
#elif L==3	// en
	"No administrators found."
#elif L==4	// es
	"No hay administradores."
#elif L==5	// fr
	"Aucun administrateur trouv&eacute;."
#elif L==6	// gn
	"No hay administradores."	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono amministratori."
#elif L==8	// pl
	"Nie znaleziono administrator&oacute;w."
#elif L==9	// pt
	"Nenhum administrador encontrado."
#elif L==10	// tr
	"No administrators found."	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM] =
#if   L==1	// ca
	"No hi ha administradors."
#elif L==2	// de
	"Keine Administratoren gefunden."
#elif L==3	// en
	"No administrators found."
#elif L==4	// es
	"No hay administradores."
#elif L==5	// fr
	"Aucun administrateur trouv&eacute;."
#elif L==6	// gn
	"No hay administradores."	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono amministratori."
#elif L==8	// pl
	"Nie znaleziono administrator&oacute;w."
#elif L==9	// pt
	"Nenhum administrador encontrado."
#elif L==10	// tr
	"No administrators found."	// eviri lazim!
#endif
	,
	[Rol_INS_ADM] =
#if   L==1	// ca
	"No hi ha administradors."
#elif L==2	// de
	"Keine Administratoren gefunden."
#elif L==3	// en
	"No administrators found."
#elif L==4	// es
	"No hay administradores."
#elif L==5	// fr
	"Aucun administrateur trouv&eacute;."
#elif L==6	// gn
	"No hay administradores."	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono amministratori."
#elif L==8	// pl
	"Nie znaleziono administrator&oacute;w."
#elif L==9	// pt
	"Nenhum administrador encontrado."
#elif L==10	// tr
	"No administrators found."	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM] =
#if   L==1	// ca
	"No hi ha superusuaris."
#elif L==2	// de
	"Keine Superuser gefunden."
#elif L==3	// en
	"No superusers found."
#elif L==4	// es
	"No hay superusuarios."
#elif L==5	// fr
	"Aucun superutilisateur trouv&eacute;."
#elif L==6	// gn
	"No hay superusuarios."	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono superutenti."
#elif L==8	// pl
	"Nie znaleziono superu&zdot;ytkownika."
#elif L==9	// pt
	"Nenhum superutilizador encontrado."
#elif L==10	// tr
	"No superusers found."	// eviri lazim!
#endif
	};

const char *Txt_nobody_else_can_access_this_content =
#if   L==1	// ca
	"ning&uacute; m&eacute;s pot accedir a aquest contingut";
#elif L==2	// de
	"Niemand sonst kann auf diese Inhalte";
#elif L==3	// en
	"nobody else can access this content";
#elif L==4	// es
	"nadie m&aacute;s puede acceder a este contenido";
#elif L==5	// fr
	"personne d'autre ne peut acc&eacute;der &agrave; ce contenu";
#elif L==6	// gn
	"avave ambue ndaikat&uacute;i oike ko contenido-pe";
#elif L==7	// it
	"nessun altro pu&ograve; accedere a questo contenuto";
#elif L==8	// pl
	"nikt inny nie moze uzyskac dostepu do tej zawartosci";
#elif L==9	// pt
	"ningu&eacute;m mais pode acessar este conte&uacute;do";
#elif L==10	// tr
	"bu i&ccedil;eri&gbreve;e ba&scedil;ka kimse eri&scedil;emez";
#endif

const char *Txt_Non_editing_teachers_can_create_new_projects =
#if   L==1	// ca
	"Els professors no editors poden crear nous projectes";
#elif L==2	// de
	"Nicht bearbeitende Lehrern k&ouml;nnen neue Projekte erstellen";
#elif L==3	// en
	"Non-editing teachers can create new projects";
#elif L==4	// es
	"Los profesores no editores pueden crear nuevos proyectos";
#elif L==5	// fr
	"Les enseignants non &eacute;diteurs peuvent cr&eacute;er de nouveaux projets";
#elif L==6	// gn
	"Mbo'eh&aacute;a ndaha'&eacute;iva editor ikatu omohe&ntilde;&oacute;i proyecto pyahu";
#elif L==7	// it
	"Gli insegnanti non editori possono creare nuovi progetti";
#elif L==8	// pl
	"Nauczyciele nieedytuj&aogon;cy mog&aogon; tworzy&cacute; nowe projekty";
#elif L==9	// pt
	"Professores n&atilde;o editores podem criar novos projetos";
#elif L==10	// tr
	"Edit&ouml;r olmayan &ouml;&gbreve;retmenler yeni projeler olu&scedil;turabilir";
#endif

const char *Txt_Not_applicable =
#if   L==1	// ca
	"No aplicable";
#elif L==2	// de
	"Nicht anwendbar";
#elif L==3	// en
	"Not applicable";
#elif L==4	// es
	"No aplicable";
#elif L==5	// fr
	"Non applicable";
#elif L==6	// gn
	"No aplicable";		// Okoteve traduccin
#elif L==7	// it
	"Non applicabile";
#elif L==8	// pl
	"Nie dotyczy";
#elif L==9	// pt
	"N&atilde;o aplic&aacute;vel";
#elif L==10	// tr
	"Not applicable";	// eviri lazim!
#endif

const char *Txt_not_available =
#if   L==1	// ca
	"no disponible";
#elif L==2	// de
	"Nicht verf&uuml;gbar";
#elif L==3	// en
	"not available";
#elif L==4	// es
	"no disponible";
#elif L==5	// fr
	"pas disponible";
#elif L==6	// gn
	"no disponible";	// Okoteve traduccin
#elif L==7	// it
	"non disponibile";
#elif L==8	// pl
	"niedost&eogon;pne";
#elif L==9	// pt
	"n&atilde;o dispon&iacute;vel";
#elif L==10	// tr
	"not available";	// eviri lazim!
#endif

const char *Txt_Not_duplicated =
#if   L==1	// ca
	"No &eacute;s duplicat";
#elif L==2	// de
	"Nicht dupliziert";
#elif L==3	// en
	"Not duplicated";
#elif L==4	// es
	"No es duplicado/a";
#elif L==5	// fr
	"Pas dupliqu&eacute;";
#elif L==6	// gn
	"No es duplicado/a";	// Okoteve traduccin
#elif L==7	// it
	"Non duplicato";
#elif L==8	// pl
	"Nie powiela&cacute;";
#elif L==9	// pt
	"N&atilde;o duplicado";
#elif L==10	// tr
	"Not duplicated";	// eviri lazim!
#endif

const char *Txt_Not_visible =
#if   L==1	// ca
	"No visible";
#elif L==2	// de
	"Nicht sichtbar";
#elif L==3	// en
	"Not visible";
#elif L==4	// es
	"No visible";
#elif L==5	// fr
	"Non visible";
#elif L==6	// gn
	"No visible";	// Okoteve traduccin
#elif L==7	// it
	"Non visibile";
#elif L==8	// pl
	"Niewidoczne";
#elif L==9	// pt
	"N&atilde;o vis&iacute;vel";
#elif L==10	// tr
	"Not visible";	// eviri lazim!
#endif

const char *Txt_Nothing_has_been_pasted_because_the_clipboard_is_empty_ =
#if   L==1	// ca
	"No s'enganxa res perqu&egrave; el porta-retalls &eacute;s buit."
	" Intenta copiar i enganxar de nou.";
#elif L==2	// de
	"Es konnten keine Daten eingef&uuml;gt werden. Der Zwischenspeicher ist leer."
	" Versuchen Sie den Kopieren-Einf&uuml;gen-Vorgang bitte erneut.";
#elif L==3	// en
	"Nothing has been pasted because the clipboard is empty."
	" Try copy-paste again.";
#elif L==4	// es
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";
#elif L==5	// fr
	"Rien n'a &eacute;t&eacute; coll&eacute; car le presse-papiers est vide."
	" Essayez &agrave; nouveau de copier-coller.";
#elif L==6	// gn
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";	// Okoteve traduccin
#elif L==7	// it
	"Niente &egrave; stato copiato perch&egrave; la bacheca &egrave; piena."
	" Prova copia-incolla nuovamente.";
#elif L==8	// pl
	"Nic nie zostal wklejony bo schowek jest pusty."
	" Spr&oacute;buj ponownie kopiuj-wklej.";
#elif L==9	// pt
	"Nada foi colado porque a &aacute;rea de transfer&ecirc;ncia est&aacute; vazia."
	" Tente copiar e colar novamente.";
#elif L==10	// tr
	"Nothing has been pasted because the clipboard is empty."
	" Try copy-paste again.";		// eviri lazim!
#endif

const char *Txt_NOTICE_Active_BR_notices =
#if   L==1	// ca
	"Avisos<br />actius";
#elif L==2	// de
	"Aktive<br />Ank&uuml;ndigungen";
#elif L==3	// en
	"Active<br />notices";
#elif L==4	// es
	"Avisos<br />activos";
#elif L==5	// fr
	"Avis<br />actives";
#elif L==6	// gn
	"Avisos<br />activos";	// Okoteve traduccin
#elif L==7	// it
	"Avvisi<br />attivi";
#elif L==8	// pl
	"Active<br />Ogloszenia";
#elif L==9	// pt
	"Avisos<br />ativos";
#elif L==10	// tr
	"Active<br />notices";	// eviri lazim!
#endif

const char *Txt_NOTICE_Deleted_BR_notices =
#if   L==1	// ca
	"Avisos<br />eliminats";
#elif L==2	// de
	"Gel&ouml;scht<br />Ank&uuml;ndigungen";
#elif L==3	// en
	"Deleted<br />notices";
#elif L==4	// es
	"Avisos<br />eliminados";
#elif L==5	// fr
	"Avis<br />supprim&eacute;s";
#elif L==6	// gn
	"Avisos<br />eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Avvisi<br />rimossi";
#elif L==8	// pl
	"Usuniete<br />notices";
#elif L==9	// pt
	"Avisos<br />suprimidos";
#elif L==10	// tr
	"Deleted<br />notices";		// eviri lazim!
#endif

const char *Txt_NOTICE_Obsolete_BR_notices =
#if   L==1	// ca
	"Avisos<br />obsolets";
#elif L==2	// de
	"Veraltet<br />Ank&uuml;ndigungen";
#elif L==3	// en
	"Obsolete<br />notices";
#elif L==4	// es
	"Avisos<br />obsoletos";
#elif L==5	// fr
	"Avis<br />obsol&egrave;tes";
#elif L==6	// gn
	"Avisos<br />obsoletos";	// Okoteve traduccin
#elif L==7	// it
	"Avvisi<br />obsoleti";
#elif L==8	// pl
	"Przestarzaly<br />Ogloszenia";
#elif L==9	// pt
	"Avisos<br />obsoletos";
#elif L==10	// tr
	"Obsolete<br />notices";	// eviri lazim!
#endif

const char *Txt_Notice =
#if   L==1	// ca
	"Av&iacute;s";
#elif L==2	// de
	"Ank&uuml;ndigung";
#elif L==3	// en
	"Notice";
#elif L==4	// es
	"Aviso";
#elif L==5	// fr
	"Avis";
#elif L==6	// gn
	"Marandu";
#elif L==7	// it
	"Avviso";
#elif L==8	// pl
	"Informacja";
#elif L==9	// pt
	"Aviso";
#elif L==10	// tr
	"Uyar&inodot;";
#endif

const char *Txt_Notice_created =
#if   L==1	// ca
	"Av&iacute;s creat.";
#elif L==2	// de
	"Ank&uuml;ndigung erstellt.";
#elif L==3	// en
	"Notice created.";
#elif L==4	// es
	"Aviso creado.";
#elif L==5	// fr
	"Avis cr&eacute;&eacute;.";
#elif L==6	// gn
	"Aviso creado.";	// Okoteve traduccin
#elif L==7	// it
	"Avviso creato.";
#elif L==8	// pl
	"Zawiadomienie stworzony.";
#elif L==9	// pt
	"Aviso criado.";
#elif L==10	// tr
	"Notice created.";	// eviri lazim!
#endif

const char *Txt_Notice_removed =
#if   L==1	// ca
	"Av&iacute;s eliminat.";
#elif L==2	// de
	"Ank&uuml;ndigung entfernt.";
#elif L==3	// en
	"Notice removed.";
#elif L==4	// es
	"Aviso eliminado";
#elif L==5	// fr
	"Avis supprim&eacute;.";
#elif L==6	// gn
	"Aviso eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Avviso rimosso.";
#elif L==8	// pl
	"Informacja usuniete.";
#elif L==9	// pt
	"Aviso removido.";
#elif L==10	// tr
	"Notice removed.";	// eviri lazim!
#endif

const char *Txt_Notices =
#if   L==1	// ca
	"Avisos";
#elif L==2	// de
	"Ank&uuml;ndigungen";
#elif L==3	// en
	"Notices";
#elif L==4	// es
	"Avisos";
#elif L==5	// fr
	"Avis";
#elif L==6	// gn
	"Marandu";
#elif L==7	// it
	"Avvisi";
#elif L==8	// pl
	"Ogloszenia";
#elif L==9	// pt
	"Avisos";
#elif L==10	// tr
	"Bildirimler";
#endif

const char *Txt_NOTIF_new_SINGULAR =
#if   L==1	// ca
	"nova";
#elif L==2	// de
	"neue";
#elif L==3	// en
	"new";
#elif L==4	// es
	"nueva";
#elif L==5	// fr
	"nouvelle";
#elif L==6	// gn
	"nueva";	// Okoteve traduccin
#elif L==7	// it
	"nuova";
#elif L==8	// pl
	"nowy";
#elif L==9	// pt
	"nova";
#elif L==10	// tr
	"new";		// eviri lazim!
#endif

const char *Txt_NOTIF_new_PLURAL =
#if   L==1	// ca
	"noves";
#elif L==2	// de
	"neue";
#elif L==3	// en
	"new";
#elif L==4	// es
	"nuevas";
#elif L==5	// fr
	"nouvelles";
#elif L==6	// gn
	"nuevas";	// Okoteve traduccin
#elif L==7	// it
	"nuove";
#elif L==8	// pl
	"nowy";
#elif L==9	// pt
	"novas";
#elif L==10	// tr
	"new";		// eviri lazim!
#endif

const char *Txt_NOTIFICATION_STATUS[Ntf_NUM_STATUS_TXT] =
	{
	[Ntf_STATUS_NO_EMAIL] =
#if   L==1	// ca
	"No"
#elif L==2	// de
	"Nein"
#elif L==3	// en
	"No"
#elif L==4	// es
	"No"
#elif L==5	// fr
	"Non"
#elif L==6	// gn
	"Nah&aacute;niri"
#elif L==7	// it
	"No"
#elif L==8	// pl
	"No"
#elif L==9	// pt
	"N&atilde;o"
#elif L==10	// tr
	"No"	// eviri lazim!
#endif
	,
	[Ntf_STATUS_EMAIL_PENDING] =
#if   L==1	// ca
	"Pendent"
#elif L==2	// de
	"Offen"
#elif L==3	// en
	"Pending"
#elif L==4	// es
	"Pendiente"
#elif L==5	// fr
	"En attente"
#elif L==6	// gn
	"Pendiente"	// Okoteve traduccin
#elif L==7	// it
	"In attesa"
#elif L==8	// pl
	"Do czasu"
#elif L==9	// pt
	"Pendente"
#elif L==10	// tr
	"Pending"	// eviri lazim!
#endif
	,
	[Ntf_STATUS_EMAIL_CANCELLED] =
#if   L==1	// ca
	"Cancellat"
#elif L==2	// de
	"Abgebrochen"
#elif L==3	// en
	"Cancelled"
#elif L==4	// es
	"Cancelado"
#elif L==5	// fr
	"Annul&eacute;"
#elif L==6	// gn
	"Cancelado"	// Okoteve traduccin
#elif L==7	// it
	"Rimosso"
#elif L==8	// pl
	"Odwolany"
#elif L==9	// pt
	"Cancelado"
#elif L==10	// tr
	"Cancelled"	// eviri lazim!
#endif
	,
	[Ntf_STATUS_EMAIL_SENT] =
#if   L==1	// ca
	"Enviat"
#elif L==2	// de
	"Gesendet"
#elif L==3	// en
	"Sent"
#elif L==4	// es
	"Enviado"
#elif L==5	// fr
	"Envoy&eacute;e"
#elif L==6	// gn
	"Enviado"	// Okoteve traduccin
#elif L==7	// it
	"Inviata"
#elif L==8	// pl
	"Wyslane"
#elif L==9	// pt
	"Enviada"
#elif L==10	// tr
	"Sent"		// eviri lazim!
#endif
	};

const char *Txt_notification =
#if   L==1	// ca
	"notificaci&oacute;";
#elif L==2	// de
	"Anmeldung";
#elif L==3	// en
	"notification";
#elif L==4	// es
	"notificaci&oacute;n";
#elif L==5	// fr
	"notification";
#elif L==6	// gn
	"momarandu";
#elif L==7	// it
	"notifica";
#elif L==8	// pl
	"powiadomienie";
#elif L==9	// pt
	"notifica&ccedil;&atilde;o";
#elif L==10	// tr
	"notification";	// eviri lazim!
#endif

const char *Txt_Notifications =
#if   L==1	// ca
	"Notificacions";
#elif L==2	// de
	"Anmeldungen";
#elif L==3	// en
	"Notifications";
#elif L==4	// es
	"Notificaciones";
#elif L==5	// fr
	"Notifications";
#elif L==6	// gn
	"Momarandu";
#elif L==7	// it
	"Notifiche";
#elif L==8	// pl
	"Powiadomienia";
#elif L==9	// pt
	"Notifica&ccedil;&otilde;es";
#elif L==10	// tr
	"Bildirimler";
#endif

const char *Txt_notifications =
#if   L==1	// ca
	"notificacions";
#elif L==2	// de
	"Anmeldungen";
#elif L==3	// en
	"notifications";
#elif L==4	// es
	"notificaciones";
#elif L==5	// fr
	"notifications";
#elif L==6	// gn
	"momarandu";
#elif L==7	// it
	"notifiche";
#elif L==8	// pl
	"powiadomienia";
#elif L==9	// pt
	"notifica&ccedil;&otilde;es";
#elif L==10	// tr
	"bildirimler";
#endif

const char *Txt_NOTIFY_EVENTS_PLURAL[Ntf_NUM_NOTIFY_EVENTS] =
	{
	[Ntf_EVENT_UNKNOWN] =
#if   L==1	// ca
	"Esdeveniments desconeguts"
#elif L==2	// de
	"Unbekannte Ereignisse"
#elif L==3	// en
	"Unknown events"
#elif L==4	// es
	"Eventos desconocidos"
#elif L==5	// fr
	"&Eacute;v&eacute;nements inconnus"
#elif L==6	// gn
	"Eventos desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Eventi sconosciuto"
#elif L==8	// pl
	"Brak wydarzen"
#elif L==9	// pt
	"Eventos desconhecidos"
#elif L==10	// tr
	"Unknown events"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_DOCUMENT_FILE] =
#if   L==1	// ca
	"Nous documents"
#elif L==2	// de
	"Neue Dokumentdateien"
#elif L==3	// en
	"New document files"
#elif L==4	// es
	"Nuevos documentos"
#elif L==5	// fr
	"Nouveaux fichiers de documents"
#elif L==6	// gn
	"Nuevos documentos"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi documenti"
#elif L==8	// pl
	"Nowy plik dokumentu"
#elif L==9	// pt
	"Novos arquivos de documento"
#elif L==10	// tr
	"New document files"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_TEACHERS_FILE] =
#if   L==1	// ca
	"Nous arxius de professors"
#elif L==2	// de
	"New Lehrer Dateien"
#elif L==3	// en
	"New teachers' files"
#elif L==4	// es
	"Nuevos archivos de profesores"
#elif L==5	// fr
	"Nouveaux fichiers d'enseignants"
#elif L==6	// gn
	"Nuevos archivos de profesores"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi file di professori"
#elif L==8	// pl
	"Nowy Nauczyciela plik"
#elif L==9	// pt
	"Novos arquivos dos professores"
#elif L==10	// tr
	"New teachers' files"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_SHARED_FILE] =
#if   L==1	// ca
	"Nous arxius compartits"
#elif L==2	// de
	"New freigegebene Dateien"
#elif L==3	// en
	"New shared files"
#elif L==4	// es
	"Nuevos archivos compartidos"
#elif L==5	// fr
	"Nouveaux fichiers partag&eacute;s"
#elif L==6	// gn
	"Nuevos archivos compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi file condiviso"
#elif L==8	// pl
	"Nowy Udost&eogon;pniony plik"
#elif L==9	// pt
	"Novos arquivos compartilhado"
#elif L==10	// tr
	"New shared files"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ASSIGNMENT] =
#if   L==1	// ca
	"Noves activitats"
#elif L==2	// de
	"Neue Aufgaben"
#elif L==3	// en
	"New assignments"
#elif L==4	// es
	"Nuevas actividades"
#elif L==5	// fr
	"Nouvelles activit&eacute;s"
#elif L==6	// gn
	"Nuevas actividades"	// Okoteve traduccin
#elif L==7	// it
	"Nuove attivit&agrave;"
#elif L==8	// pl
	"Nowe zadania"
#elif L==9	// pt
	"Novas atividades"
#elif L==10	// tr
	"New assignments"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_CALL_FOR_EXAM] =
#if   L==1	// ca
	"Noves convocat&ograve;ries d'examen"
#elif L==2	// de
	"Neue Aufrufe f&uuml;r Pr&uuml;fung"
#elif L==3	// en
	"New calls for exam"
#elif L==4	// es
	"Nuevas convocatorias de examen"
#elif L==5	// fr
	"Nouvelles convocations &agrave; un examen"
#elif L==6	// gn
	"Nuevas convocatorias de examen"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi appelli d'esame"
#elif L==8	// pl
	"Nowe anonse egzamin"
#elif L==9	// pt
	"Novas chamadas para exame"
#elif L==10	// tr
	"New calls for exam"			// eviri lazim!
#endif
	,
	[Ntf_EVENT_MARKS_FILE] =
#if   L==1	// ca
	"Nous arxius de qualificacions"
#elif L==2	// de
	"Neue Dateien mit Bewertung"
#elif L==3	// en
	"New files with marks"
#elif L==4	// es
	"Nuevos archivos de calificaciones"
#elif L==5	// fr
	"Nouveaux fichiers avec des notes"
#elif L==6	// gn
	"Nuevos archivos de calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi file con risultati"
#elif L==8	// pl
	"Nowe pliki ze znakami"
#elif L==9	// pt
	"Novos arquivos de notas"
#elif L==10	// tr
	"New files with marks"			// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_STD] =
#if   L==1	// ca
	"Novas inscripci&oacute;s com a estudiant"
#elif L==2	// de
	"New Einschreibungen als Student"
#elif L==3	// en
	"New enrolments as a student"
#elif L==4	// es
	"Nuevas inscripciones como estudiante"
#elif L==5	// fr
	"Nouvelles inscriptions en tant qu'&eacute;tudiant"
#elif L==6	// gn
	"Nuevas inscripciones como estudiante"	// Okoteve traduccin
#elif L==7	// it
	"Nuova iscrizione come studente"
#elif L==8	// pl
	"Nowe zapisy jako student"
#elif L==9	// pt
	"Novas inscri&ccedil;&otilde;es como estudante"
#elif L==10	// tr
	"New enrolments as a student"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_TCH] =
#if   L==1	// ca
	"Novas inscripci&oacute;s com a professor"
#elif L==2	// de
	"New Einschreibungen als Lehrkraft"
#elif L==3	// en
	"New enrolments as a teacher"
#elif L==4	// es
	"Nuevas inscripciones como profesor/a"
#elif L==5	// fr
	"Nouvelles inscriptions en tant qu'enseignant"
#elif L==6	// gn
	"Nuevas inscripciones como profesor/a"	// Okoteve traduccin
#elif L==7	// it
	"Nuova iscrizione come professore"
#elif L==8	// pl
	"Nowe zapisy jako nauczyciel"
#elif L==9	// pt
	"Novas inscri&ccedil;&otilde;es como professor"
#elif L==10	// tr
	"New enrolments as a teacher"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_REQUEST] =
#if   L==1	// ca
	"Novas petici&oacute;s d'inscripci&oacute;"
#elif L==2	// de
	"New Registrierungsanforderungen"
#elif L==3	// en
	"New enrolment requests"
#elif L==4	// es
	"Nuevas peticiones de inscripci&oacute;n"
#elif L==5	// fr
	"Nouvelles demandes d'inscription"
#elif L==6	// gn
	"Nuevas peticiones de inscripci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Nuova richiesta di iscrizione"
#elif L==8	// pl
	"Nowe pro&sacute;by o rejestracji"
#elif L==9	// pt
	"Novos pedidos de inscri&ccedil;&atilde;o"
#elif L==10	// tr
	"New enrolment requests"			// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_COMMENT] =
#if   L==1	// ca
	"Nous comentaris als meus missatges socials"
#elif L==2	// de
	"Neue Kommentare zu meinem sozialen Beitr&auml;ge"
#elif L==3	// en
	"New comments to my social posts"
#elif L==4	// es
	"Nuevos comentarios a mis mensajes sociales"
#elif L==5	// fr
	"Nouveaux commentaires &agrave; mes messages sociaux"
#elif L==6	// gn
	"Nuevos comentarios a mis mensajes sociales"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi commenti ai miei post sociali"
#elif L==8	// pl
	"Nowe komentarze do moich post spo&lstrok;ecznej"
#elif L==9	// pt
	"Novos coment&aacute;rio &agrave;s minhos posts sociais"
#elif L==10	// tr
	"New comments to my social posts"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_FAV] =
#if   L==1	// ca
	"Nous favorits als meus missatges socials"
#elif L==2	// de
	"Neue Favorit zu meinem sozialen Beitr&auml;ge"
#elif L==3	// en
	"New favourite to my social posts"
#elif L==4	// es
	"Nuevos favoritos a mis mensajes sociales"
#elif L==5	// fr
	"Nouveaux favori &agrave; mes messages sociaux"
#elif L==6	// gn
	"Nuevos favoritos a mis mensajes sociales"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi preferito ai miei post sociali"
#elif L==8	// pl
	"Nowe ulubiony do moich post spo&lstrok;ecznej"
#elif L==9	// pt
	"Novos favoritos &agrave;s minhos posts sociais"
#elif L==10	// tr
	"New favourite to my social posts"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_SHARE] =
#if   L==1	// ca
	"Noves comparticions dels meus missatges socials"
#elif L==2	// de
	"Neue gemeinsame Nutzung zu meinem sozialen Beitr&auml;ge"
#elif L==3	// en
	"New sharings of my social posts"
#elif L==4	// es
	"Nuevas comparticiones de mis mensajes sociales"
#elif L==5	// fr
	"Nouveaux partages de mes messages sociaux"
#elif L==6	// gn
	"Nuevas comparticiones de mis mensajes sociales"	// Okoteve traduccin
#elif L==7	// it
	"Nuove condivisioni dei miei post sociali"
#elif L==8	// pl
	"Nowe podzial moich post spo&lstrok;ecznej"
#elif L==9	// pt
	"Novas partilhas de minhos posts sociais"
#elif L==10	// tr
	"New favourite to my social posts"			// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_MENTION] =
#if   L==1	// ca
	"Noves mencions"
#elif L==2	// de
	"Neue Erw&auml;hnungen"
#elif L==3	// en
	"New mentions"
#elif L==4	// es
	"Nuevas menciones"
#elif L==5	// fr
	"Nouvelles mentions"
#elif L==6	// gn
	"Nuevas menciones"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi menzioni"
#elif L==8	// pl
	"Nowe wzmianki"
#elif L==9	// pt
	"Novas men&ccedil;&otilde;es"
#elif L==10	// tr
	"New mentions"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_FOLLOWER] =
#if   L==1	// ca
	"Nous seguidors"
#elif L==2	// de
	"Neue Anh&auml;nger"
#elif L==3	// en
	"New followers"
#elif L==4	// es
	"Nuevos seguidores"
#elif L==5	// fr
	"Nouveaux suiveurs"
#elif L==6	// gn
	"Nuevos seguidores"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi followers"
#elif L==8	// pl
	"Nowe obserwuj&aogon;"
#elif L==9	// pt
	"Novos seguidores"
#elif L==10	// tr
	"New followers"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_FORUM_POST_COURSE] =
#if   L==1	// ca
	"Novos comentaris en f&ograve;rums"
#elif L==2	// de
	"Neue Beitr&auml;ge in den Foren"
#elif L==3	// en
	"New posts in course forums"
#elif L==4	// es
	"Nuevos comentarios en foros de asignaturas"
#elif L==5	// fr
	"Nouveaux posts dans les forums"
#elif L==6	// gn
	"Nuevos comentarios en foros de asignaturas"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi post nei forum"
#elif L==8	// pl
	"Nowe post&oacute;w na forum"
#elif L==9	// pt
	"Novos posts em f&oacute;runs"
#elif L==10	// tr
	"New posts in course forums"			// eviri lazim!
#endif
	,
	[Ntf_EVENT_FORUM_REPLY] =
#if   L==1	// ca
	"Noves respostes als meus comentaris en f&ograve;rums"
#elif L==2	// de
	"Neue Antworten auf meine Beitr&auml;ge in den Foren"
#elif L==3	// en
	"New replies to my posts in forums"
#elif L==4	// es
	"Nuevas respuestas a mis comentarios en foros"
#elif L==5	// fr
	"Nouvelles r&eacute;ponses &agrave; mes posts dans les forums"
#elif L==6	// gn
	"Nuevas respuestas a mis comentarios en foros"	// Okoteve traduccin
#elif L==7	// it
	"Nuove risposte ai miei post nei forum"
#elif L==8	// pl
	"Nowe podejscie do moich post&oacute;w na forum"
#elif L==9	// pt
	"Novas respostas aos meus posts em f&oacute;runs"
#elif L==10	// tr
	"New replies to my posts in forums"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_NOTICE] =
#if   L==1	// ca
	"Nous avisos"
#elif L==2	// de
	"Neue Ank&uuml;ndigungen"
#elif L==3	// en
	"New notices"
#elif L==4	// es
	"Nuevos avisos"
#elif L==5	// fr
	"Nouveaux avis"
#elif L==6	// gn
	"Nuevos avisos"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi avvisi"
#elif L==8	// pl
	"Nowe ogloszenia"
#elif L==9	// pt
	"Novos avisos"
#elif L==10	// tr
	"New notices"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_MESSAGE] =
#if   L==1	// ca
	"Nous missatges"
#elif L==2	// de
	"Neue Nachrichten"
#elif L==3	// en
	"New messages"
#elif L==4	// es
	"Nuevos mensajes"
#elif L==5	// fr
	"Nouveaux messages"
#elif L==6	// gn
	"Nuevos mensajes"	// Okoteve traduccin
#elif L==7	// it
	"Nuovi messaggi"
#elif L==8	// pl
	"Nowa wiadomosc"
#elif L==9	// pt
	"Novas mensagens"
#elif L==10	// tr
	"New messages"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_NET] =	// TODO: Move to users tab (also necessary in database) !!!!!!!!!
#if   L==1	// ca
	"Novas inscripci&oacute;s com a professor/a no editor/a"
#elif L==2	// de
	"New Einschreibungen als nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"New enrolments as a non-editing teacher"
#elif L==4	// es
	"Nuevas inscripciones como profesor/a/a no editor/a"
#elif L==5	// fr
	"Nouvelles inscriptions en tant qu'enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"Nuevas inscripciones como profesor/a no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"Nuova iscrizione come professore/ssa non-editing"
#elif L==8	// pl
	"Nowe zapisy jako nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"Novas inscri&ccedil;&otilde;es como professor/a n&atilde;o editor/a"
#elif L==10	// tr
	"New enrolments as a non-editing teacher"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_SURVEY] =
#if   L==1	// ca
	"Noves enquestes"
#elif L==2	// de
	"Neue Umfragen"
#elif L==3	// en
	"New surveys"
#elif L==4	// es
	"Nuevas encuestas"
#elif L==5	// fr
	"Nouveaux sondages"
#elif L==6	// gn
	"Nuevas encuestas"	// Okoteve traduccin
#elif L==7	// it
	"Nuove sondaggi"
#elif L==8	// pl
	"Nowe badania"
#elif L==9	// pt
	"Novos inqu&eacute;ritos"
#elif L==10	// tr
	"New surveys"		// eviri lazim!
#endif
	};

const char *Txt_NOTIFY_EVENTS_SINGULAR[Ntf_NUM_NOTIFY_EVENTS] =
	{
	[Ntf_EVENT_UNKNOWN] =
#if   L==1	// ca	//
	"Esdeveniment desconegut"
#elif L==2	// de
	"Unbekannt Ereignis"
#elif L==3	// en
	"Unknown event"
#elif L==4	// es
	"Evento desconocido"
#elif L==5	// fr
	"&Eacute;v&eacute;nement inconnu"
#elif L==6	// gn
	"Evento desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Evento sconosciuto"
#elif L==8	// pl
	"Nieznane zdarzenia"
#elif L==9	// pt
	"Evento desconhecido"
#elif L==10	// tr
	"Unknown event"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_DOCUMENT_FILE] =
#if   L==1	// ca
	"Document"
#elif L==2	// de
	"Dokumentdatei"
#elif L==3	// en
	"Document file"
#elif L==4	// es
	"Documento"
#elif L==5	// fr
	"Fichier de document"
#elif L==6	// gn
	"Terakuatia"
#elif L==7	// it
	"Documento"
#elif L==8	// pl
	"Plik dokumentu"
#elif L==9	// pt
	"Arquivo de documento"
#elif L==10	// tr
	"Document file"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_TEACHERS_FILE] =
#if   L==1	// ca
	"Arxiu de professors"
#elif L==2	// de
	"Lehrer Datei"
#elif L==3	// en
	"Teachers' file"
#elif L==4	// es
	"Archivo de profesores"
#elif L==5	// fr
	"Fichier d'enseignants"
#elif L==6	// gn
	"Archivo de profesores"	// Okoteve traduccin
#elif L==7	// it
	"File di professori"
#elif L==8	// pl
	"Nauczyciela plik"
#elif L==9	// pt
	"Arquivo dos professores"
#elif L==10	// tr
	"Teachers' file"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_SHARED_FILE] =
#if   L==1	// ca
	"Arxiu compartit"
#elif L==2	// de
	"Freigegebene Datei"
#elif L==3	// en
	"Shared file"
#elif L==4	// es
	"Archivo compartido"
#elif L==5	// fr
	"Fichier partag&eacute;"
#elif L==6	// gn
	"Archivo compartido"	// Okoteve traduccin
#elif L==7	// it
	"File condiviso"
#elif L==8	// pl
	"Udost&eogon;pniony plik"
#elif L==9	// pt
	"Arquivo compartilhado"
#elif L==10	// tr
	"Shared file"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ASSIGNMENT] =
#if   L==1	// ca
	"Activitat"
#elif L==2	// de
	"Aufgabe"
#elif L==3	// en
	"Assignment"
#elif L==4	// es
	"Actividad"
#elif L==5	// fr
	"Activit&eacute;"
#elif L==6	// gn
	"Tar&eacute;a"
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Przyporzadkowanie"
#elif L==9	// pt
	"Atividade"
#elif L==10	// tr
	"Assignment"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_CALL_FOR_EXAM] =
#if   L==1	// ca
	"Convocat&ograve;ria d'examen"
#elif L==2	// de
	"Aufrufe fr Pr&uuml;fung"
#elif L==3	// en
	"Call for exam"
#elif L==4	// es
	"Convocatoria de examen"
#elif L==5	// fr
	"Convocation &agrave; un examen"
#elif L==6	// gn
	"Convocatoria de examen"	// Okoteve traduccin
#elif L==7	// it
	"Appello d'esame"
#elif L==8	// pl
	"Ogloszenie egzamin"
#elif L==9	// pt
	"Chamada para exame"
#elif L==10	// tr
	"S&inodot;nav i&ccedil;in aray&inodot;n"
#endif
	,
	[Ntf_EVENT_MARKS_FILE] =
#if   L==1	// ca
	"Fitxer de qualificacions"
#elif L==2	// de
	"Datei mit Bewertung"
#elif L==3	// en
	"File with marks"
#elif L==4	// es
	"Archivo de calificaciones"
#elif L==5	// fr
	"Fichier avec des notes"
#elif L==6	// gn
	"Archivo de calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"File con risultati"
#elif L==8	// pl
	"Plik ze znakami"
#elif L==9	// pt
	"Arquivo de notas"
#elif L==10	// tr
	"File with marks"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_STD] =
#if   L==1	// ca
	"Inscripci&oacute; com a estudiant"
#elif L==2	// de
	"Einschreibung als Student"
#elif L==3	// en
	"Enrolment as a student"
#elif L==4	// es
	"Inscripci&oacute;n como estudiante"
#elif L==5	// fr
	"Inscription en tant qu'&eacute;tudiant"
#elif L==6	// gn
	"Inscripci&oacute;n como estudiante"	// Okoteve traduccin
#elif L==7	// it
	"Iscrizione come studente"
#elif L==8	// pl
	"Zapisy jako student"
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o como estudante"
#elif L==10	// tr
	"Enrolment as a student"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_TCH] =
#if   L==1	// ca
	"Inscripci&oacute; com a professor/a"
#elif L==2	// de
	"Einschreibung als Lehrkraft"
#elif L==3	// en
	"Enrolment as a teacher"
#elif L==4	// es
	"Inscripci&oacute;n como profesor/a"
#elif L==5	// fr
	"Inscription en tant qu'enseignant"
#elif L==6	// gn
	"Inscripci&oacute;n como profesor/a"	// Okoteve traduccin
#elif L==7	// it
	"Iscrizione come professore/ssa"
#elif L==8	// pl
	"Zapisy jako nauczyciel"
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o como professor/a"
#elif L==10	// tr
	"Enrolment as a teacher"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_REQUEST] =
#if   L==1	// ca
	"Petici&oacute; d'inscripci&oacute;"
#elif L==2	// de
	"Registrierungsanforderung"
#elif L==3	// en
	"Enrolment request"
#elif L==4	// es
	"Petici&oacute;n de inscripci&oacute;n"
#elif L==5	// fr
	"Demande d'inscription"
#elif L==6	// gn
	"Petici&oacute;n de inscripci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Richiesta di iscrizione"
#elif L==8	// pl
	"&Zdot;&aogon;danie rejestracji"
#elif L==9	// pt
	"Pedido de inscri&ccedil;&atilde;o"
#elif L==10	// tr
	"Enrolment request"			// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_COMMENT] =
#if   L==1	// ca
	"Comentari a un missatge social"
#elif L==2	// de
	"Kommentar zu soziale Beitrag"
#elif L==3	// en
	"Comment to social post"
#elif L==4	// es
	"Comentario a un mensaje social"
#elif L==5	// fr
	"Commentaire &agrave; message social"
#elif L==6	// gn
	"Comentario a un mensaje social"	// Okoteve traduccin
#elif L==7	// it
	"Commento a un post sociale"
#elif L==8	// pl
	"Komentarz do post spo&lstrok;eczna"
#elif L==9	// pt
	"Coment&aacute;rio a um post sociais"
#elif L==10	// tr
	"Comment to social post"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_FAV] =
#if   L==1	// ca
	"Missatge social marcat com favorit"
#elif L==2	// de
	"Social Beitrag als Favorit markiert"
#elif L==3	// en
	"Social post marked as favourite"
#elif L==4	// es
	"Mensaje social marcado como favorito"
#elif L==5	// fr
	"Message social marqu&eacute;e comme favori"
#elif L==6	// gn
	"Mensaje social marcado como favorito"	// Okoteve traduccin
#elif L==7	// it
	"Post sociale segnato come preferito"
#elif L==8	// pl
	"Post spo&lstrok;ecznej oznaczone jako ulubiony"
#elif L==9	// pt
	"Post sociais marcado como favorito"
#elif L==10	// tr
	"Social post marked as favourite"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_SHARE] =
#if   L==1	// ca
	"Missatge social compartit"
#elif L==2	// de
	"Social Beitrag gemeinsamer"
#elif L==3	// en
	"Social post shared"
#elif L==4	// es
	"Mensaje social compartido"
#elif L==5	// fr
	"Message social partag&eacute;"
#elif L==6	// gn
	"Mensaje social compartido"	// Okoteve traduccin
#elif L==7	// it
	"Post sociale condiviso"
#elif L==8	// pl
	"Post spo&lstrok;ecznej wsp&oacute;lne"
#elif L==9	// pt
	"Post sociais compartido"
#elif L==10	// tr
	"Social post shared"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_TML_MENTION] =
#if   L==1	// ca
	"Menci&oacute;"
#elif L==2	// de
	"Erw&auml;hnung"
#elif L==3	// en
	"Mention"
#elif L==4	// es
	"Menci&oacute;n"
#elif L==5	// fr
	"Mention"
#elif L==6	// gn
	"Menci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Menzione"
#elif L==8	// pl
	"Wzmianka"
#elif L==9	// pt
	"Men&ccedil;&atilde;o"
#elif L==10	// tr
	"Mention"		// eviri lazim!
#endif
	,
	[Ntf_EVENT_FOLLOWER] =
#if   L==1	// ca
	"Seguidor"
#elif L==2	// de
	"Anh&auml;nger"
#elif L==3	// en
	"Follower"
#elif L==4	// es
	"Seguidor"
#elif L==5	// fr
	"Suiveur"
#elif L==6	// gn
	"Seguidor"	// Okoteve traduccin
#elif L==7	// it
	"Follower"
#elif L==8	// pl
	"Obserwuj&aogon;"
#elif L==9	// pt
	"Seguidor"
#elif L==10	// tr
	"Follower"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_FORUM_POST_COURSE] =
#if   L==1	// ca
	"Comentari en un f&ograve;rum"
#elif L==2	// de
	"Beitr&auml;ge in einem Forum"
#elif L==3	// en
	"Post in a forum"
#elif L==4	// es
	"Comentario en un foro"
#elif L==5	// fr
	"Post dans un forum"
#elif L==6	// gn
	"Comentario en un foro"	// Okoteve traduccin
#elif L==7	// it
	"Post in un forum"
#elif L==8	// pl
	"Post na forum"
#elif L==9	// pt
	"Post em um f&oacute;rum"
#elif L==10	// tr
	"Post in a forum"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_FORUM_REPLY] =
#if   L==1	// ca
	"Resposta en un f&ograve;rum"
#elif L==2	// de
	"Antwort in einem Forum"
#elif L==3	// en
	"Reply in a forum"
#elif L==4	// es
	"Respuesta en un foro"
#elif L==5	// fr
	"R&eacute;ponse dans un forum"
#elif L==6	// gn
	"Respuesta en un foro"	// Okoteve traduccin
#elif L==7	// it
	"Risposta in un forum"
#elif L==8	// pl
	"Odpowiedz na forum"
#elif L==9	// pt
	"Resposta em um f&oacute;rum"
#elif L==10	// tr
	"Reply in a forum"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_NOTICE] =
#if   L==1	// ca
	"Av&iacute;s"
#elif L==2	// de
	"Ank&uuml;ndigungen"
#elif L==3	// en
	"Notice"
#elif L==4	// es
	"Aviso"
#elif L==5	// fr
	"Avis"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Avviso"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"An&uacute;ncio"
#elif L==10	// tr
	"Notice"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_MESSAGE] =
#if   L==1	// ca
	"Missatge"
#elif L==2	// de
	"Nachricht"
#elif L==3	// en
	"Message"
#elif L==4	// es
	"Mensaje"
#elif L==5	// fr
	"Message"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Messaggio"
#elif L==8	// pl
	"Wiadomo&sacute;&cacute;"
#elif L==9	// pt
	"Mensagem"
#elif L==10	// tr
	"Message"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_SURVEY] =
#if   L==1	// ca
	"Enquesta"
#elif L==2	// de
	"Umfrage"
#elif L==3	// en
	"Survey"
#elif L==4	// es
	"Encuesta"
#elif L==5	// fr
	"Sondage"
#elif L==6	// gn
	"Encuesta"	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio"
#elif L==8	// pl
	"Ankieta"
#elif L==9	// pt
	"Inqu&eacute;rito"
#elif L==10	// tr
	"Survey"	// eviri lazim!
#endif
	,
	[Ntf_EVENT_ENROLMENT_NET] =	// TODO: Move to users tab (also necessary in database) !!!!!!!!!
#if   L==1	// ca
	"Inscripci&oacute; com a professor/a no editor/a"
#elif L==2	// de
	"Einschreibung als nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Enrolment as a non-editing teacher"
#elif L==4	// es
	"Inscripci&oacute;n como profesor/a no editor/a"
#elif L==5	// fr
	"Inscription en tant qu'enseignant non-&eacute;diteur/trice"
#elif L==6	// gn
	"Inscripci&oacute;n como profesor/a no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"Iscrizione come professore/ssa non-editing"
#elif L==8	// pl
	"Zapisy jako nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o como professor/a n&atilde;o editor/a"
#elif L==10	// tr
	"Enrolment as a non-editing teacher"			// eviri lazim!
#endif
	};

const char *Txt_Notify_me_BR_by_email =
#if   L==1	// ca
	"Avisar-me<br />per correu";
#elif L==2	// de
	"Benachrichtigt mich<br />per E-Mail";
#elif L==3	// en
	"Notify me<br />by email";
#elif L==4	// es
	"Avisarme<br />por correo";
#elif L==5	// fr
	"Avertissez-moi<br />par email";
#elif L==6	// gn
	"Avisarme<br />por correo";	// Okoteve traduccin
#elif L==7	// it
	"Notificatami<br />via email";
#elif L==8	// pl
	"Informuj<br />mnie emailem";
#elif L==9	// pt
	"Notifique-me<br />por email";
#elif L==10	// tr
	"Notify me<br />by email";	// eviri lazim!
#endif

const char *Txt_Now_each_student_can_belong_to_multiple_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ara cada estudiant pot pertnyer a diversos grups tipus <strong>%s</strong>.";
#elif L==2	// de
	"Jetzt kann jeder Sch&uuml;ler mehreren Gruppen des Typs <strong>%s</strong> angeh&ouml;ren.";
#elif L==3	// en
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";
#elif L==4	// es
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";
#elif L==5	// fr
	"D&eacute;sormais, chaque &eacute;tudiant peut appartenir &agrave; plusieurs groupes de type <strong>%s</strong>.";
#elif L==6	// gn
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Ora ogni studente pu&ograve; appartenere a gruppi multipli di tipo <strong>%s</strong>.";
#elif L==8	// pl
	"Teraz kazdy student moze nalezec do wielu grup typu <strong>%s</strong>.";
#elif L==9	// pt
	"Agora cada aluno pode pertencer a v&aacute;rios grupos do tipo <strong>%s</strong>.";
#elif L==10	// tr
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_Now_each_student_can_only_belong_to_a_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ara cada estudiant nom&eacute;s pot pert&agrave;nyer a un grup de tipus <strong>%s</strong>.";
#elif L==2	// de
	"Jetzt kann jeder Sch&uuml;ler nur noch einer Gruppe vom Typ <strong>%s</strong> angeh&ouml;ren.";
#elif L==3	// en
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==4	// es
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";
#elif L==5	// fr
	"D&eacute;sormais, chaque &eacute;tudiant ne peut appartenir qu'&agrave; un groupe de type <strong>%s</strong>.";
#elif L==6	// gn
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Ora ogni studente pu&ograve; solo appartenere a un gruppo di tipo <strong>%s</strong>.";
#elif L==8	// pl
	"Teraz kazdy student moze nalezec tylko do grupy typu <strong>%s</strong>.";
#elif L==9	// pt
	"Agora cada aluno s&oacute; pode pertencer a um grupo do tipo <strong>%s</strong>.";
#elif L==10	// tr
	"Now each student can only belong to a group of type <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_Number_of_criteria =
#if   L==1	// ca
	"Nombre de criteris";
#elif L==2	// de
	"Anzahl der Kriterien";
#elif L==3	// en
	"Number of criteria";
#elif L==4	// es
	"N.&ordm; de criterios";
#elif L==5	// fr
	"Nombre de crit&egrave;res";
#elif L==6	// gn
	"N.&ordm; de criterios";	// Okoteve traduccin
#elif L==7	// it
	"Numero di criteri";
#elif L==8	// pl
	"Liczba kryteri&oacute;w";
#elif L==9	// pt
	"N.&ordm; de crit&eacute;rios";
#elif L==10	// tr
	"Kriter say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_files =
#if   L==1	// ca
	"Nombre d'arxius";
#elif L==2	// de
	"Anzahl der Dateien";
#elif L==3	// en
	"Number of files";
#elif L==4	// es
	"N.&ordm; de archivos";
#elif L==5	// fr
	"Nombre de fichiers";
#elif L==6	// gn
	"N.&ordm; de archivos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file";
#elif L==8	// pl
	"Liczba pliki";
#elif L==9	// pt
	"N.&ordm; de arquivos";
#elif L==10	// tr
	"Number of files";	// eviri lazim!
#endif

const char *Txt_Number_of_files_in_DOCUM_zones =
#if   L==1	// ca
	"Nombre d'arxius en zones de documents";
#elif L==2	// de
	"Anzahl der Dateien im Dokumente-Bereich";
#elif L==3	// en
	"Number of files in documents area";
#elif L==4	// es
	"N.&ordm; de archivos en zonas de documentos";
#elif L==5	// fr
	"Nombre de fichiers dans les zones de documents";
#elif L==6	// gn
	"N.&ordm; de archivos en zonas de documentos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file nell'area documenti";
#elif L==8	// pl
	"Number of files in documents area";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N.&ordm; de arquivos em zonas de documentos";
#elif L==10	// tr
	"Number of files in documents area";		// eviri lazim!
#endif

const char *Txt_Number_of_files_in_SHARE_zones =
#if   L==1	// ca
	"Nombre d'arxius compartits";
#elif L==2	// de
	"Anzahl der freigegebene Dateien";
#elif L==3	// en
	"Number of shared files";
#elif L==4	// es
	"N.&ordm; de archivos compartidos";
#elif L==5	// fr
	"Nombre de fichiers partag&eacute;s";
#elif L==6	// gn
	"N.&ordm; de archivos compartidos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file condivisi";
#elif L==8	// pl
	"Liczba pliki udost&eogon;pnione";
#elif L==9	// pt
	"N.&ordm; de arquivos compartilhados";
#elif L==10	// tr
	"Number of shared files";		// eviri lazim!
#endif

const char *Txt_Number_of_forums =
#if   L==1	// ca
	"Nombre de f&ograve;rums";
#elif L==2	// de
	"Anzahl der Foren";
#elif L==3	// en
	"No. of forums";
#elif L==4	// es
	"N.&ordm; de foros";
#elif L==5	// fr
	"Nombre de forums";
#elif L==6	// gn
	"N.&ordm; de foros";	// Okoteve traduccin
#elif L==7	// it
	"Numero di forum";
#elif L==8	// pl
	"Liczba forum";
#elif L==9	// pt
	"N.&ordm; de f&oacute;runs";
#elif L==10	// tr
	"No. of forums";	// eviri lazim!
#endif

const char *Txt_Number_of_BR_groups =
#if   L==1	// ca
	"Nombre de<br />grups";
#elif L==2	// de
	"Anzahl der<br />Gruppen";
#elif L==3	// en
	"Number of<br />groups";
#elif L==4	// es
	"N.&ordm; de<br />grupos";
#elif L==5	// fr
	"Nombre de<br />groupes";
#elif L==6	// gn
	"N.&ordm; de<br />grupos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />gruppi";
#elif L==8	// pl
	"Liczba z<br />grupy";
#elif L==9	// pt
	"N.&ordm; de<br />grupos";
#elif L==10	// tr
	"Number of<br />groups";	// eviri lazim!
#endif

const char *Txt_Number_of_indicators =
#if   L==1	// ca
	"Nombre d'indicadors";
#elif L==2	// de
	"Anzahl der Indikatoren";
#elif L==3	// en
	"Number of indicators";
#elif L==4	// es
	"N.&ordm; de indicadores";
#elif L==5	// fr
	"Nombre d'indicateurs";
#elif L==6	// gn
	"N.&ordm; de indicadores";	// Okoteve traduccin
#elif L==7	// it
	"Numero di indicatori";
#elif L==8	// pl
	"Liczba z wskazniki";
#elif L==9	// pt
	"N.&ordm; de indocadores";
#elif L==10	// tr
	"Number of indicators";		// eviri lazim!
#endif

const char *Txt_Number_of_BR_lines =
#if   L==1	// ca
	"Nombre de<br />l&iacute;nies";
#elif L==2	// de
	"Anzahl der<br />Zeilen";
#elif L==3	// en
	"Number of<br />lines";
#elif L==4	// es
	"N.&ordm; de<br />l&iacute;neas";
#elif L==5	// fr
	"Nombre de<br />lignes";
#elif L==6	// gn
	"N.&ordm; de<br />l&iacute;neas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />linee";
#elif L==8	// pl
	"Liczba z<br />linie";
#elif L==9	// pt
	"N.&ordm; de<br />linhas";
#elif L==10	// tr
	"Number of<br />lines";			// eviri lazim!
#endif

const char *Txt_Number_BR_msgs =
#if   L==1	// ca
	"Nombre<br />missat.";
#elif L==2	// de
	"Anzahl<br />Nachr.";
#elif L==3	// en
	"Number<br />msgs.";
#elif L==4	// es
	"N.&ordm;<br />mens.";
#elif L==5	// fr
	"Nombre<br />msgs.";
#elif L==6	// gn
	"N.&ordm;<br />mens.";	// Okoteve traduccin
#elif L==7	// it
	"Num.<br />msg.";
#elif L==8	// pl
	"Numer<br />msgs.";
#elif L==9	// pt
	"N.&ordm;<br />mens.";
#elif L==10	// tr
	"Number<br />msgs.";	// eviri lazim!
#endif

const char *Txt_Number_of_posts =
#if   L==1	// ca
	"Nombre de comentaris";
#elif L==2	// de
	"Anzahl der Nachrichten";
#elif L==3	// en
	"Number of posts";
#elif L==4	// es
	"N.&ordm; de comentarios";
#elif L==5	// fr
	"Nombre de posts";
#elif L==6	// gn
	"N.&ordm; de comentarios";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post";
#elif L==8	// pl
	"Liczba wiadomosci";
#elif L==9	// pt
	"N.&ordm; de posts";
#elif L==10	// tr
	"Number of posts";		// eviri lazim!
#endif

const char *Txt_Number_of_posts_BR_per_forum =
#if   L==1	// ca
	"Nombre de comentaris<br />per f&ograve;rum";
#elif L==2	// de
	"Anzahl der Nachrichten<br />pro Forum";
#elif L==3	// en
	"Number of posts<br />per forum";
#elif L==4	// es
	"N.&ordm; de comentarios<br />por foro";
#elif L==5	// fr
	"Nombre de posts<br />par forum";
#elif L==6	// gn
	"N.&ordm; de comentarios<br />por foro";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post<br />per forum";
#elif L==8	// pl
	"Liczba wiadomosci<br />per forum";
#elif L==9	// pt
	"N.&ordm; de posts<br />por f&oacute;rum";
#elif L==10	// tr
	"Number of posts<br />per forum";		// eviri lazim!
#endif

const char *Txt_Number_of_posts_BR_per_thread =
#if   L==1	// ca
	"Nombre de comentaris<br />per discussi&oacute;";
#elif L==2	// de
	"Anzahl der Nachrichten<br />pro Thread";
#elif L==3	// en
	"Number of posts<br />per thread";
#elif L==4	// es
	"N.&ordm; de comentarios<br />por usuario";
#elif L==5	// fr
	"Nombre de posts<br />par fil";
#elif L==6	// gn
	"N.&ordm; de comentarios<br />por discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post<br />per discussione";
#elif L==8	// pl
	"Liczba wiadomosci<br />per watku";
#elif L==9	// pt
	"N.&ordm; de posts<br />por discuss&atilde;o";
#elif L==10	// tr
	"Number of posts<br />per thread";			// eviri lazim!
#endif

const char *Txt_Number_of_posts_BR_per_user =
#if   L==1	// ca
	"Nombre de comentaris<br />per usuari";
#elif L==2	// de
	"Anzahl der Nachrichten<br />pro Benutzer";
#elif L==3	// en
	"Number of posts<br />per user";
#elif L==4	// es
	"N.&ordm; de comentarios<br />por usuario";
#elif L==5	// fr
	"Nombre de posts<br />par utilisateur";
#elif L==6	// gn
	"N.&ordm; de comentarios<br />por puruh&aacute;ra";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post<br />per utente";
#elif L==8	// pl
	"Liczba wiadomosci<br />per u&zdot;ytkownik";
#elif L==9	// pt
	"N.&ordm; de posts<br />por utilizador";
#elif L==10	// tr
	"Number of posts<br />per user";			// eviri lazim!
#endif

const char *Txt_Number_of_private_files =
#if   L==1	// ca
	"Nombre d'arxius privats";
#elif L==2	// de
	"Anzahl der private Dateien";
#elif L==3	// en
	"Number of private files";
#elif L==4	// es
	"N.&ordm; de archivos privados";
#elif L==5	// fr
	"Nombre de fichiers priv&eacute;s";
#elif L==6	// gn
	"N.&ordm; de archivos privados";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file privati";
#elif L==8	// pl
	"Liczba prywatne pliki";
#elif L==9	// pt
	"N.&ordm; de arquivos privados";
#elif L==10	// tr
	"Number of private files";		// eviri lazim!
#endif

const char *Txt_Number_of_BR_program_items =
#if   L==1	// ca
	"Nombre d'elements<br />de programes";
#elif L==2	// de
	"Anzahl der Programmelemente";
#elif L==3	// en
	"Number of<br />program items";
#elif L==4	// es
	"N.&ordm; de<br />items en programas";
#elif L==5	// fr
	"Nombre d'&eacute;l&eacute;ments<br />de programme";
#elif L==6	// gn
	"N.&ordm; de<br />items en programas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di elementi<br />di programmi";
#elif L==8	// pl
	"Liczba<br />pozycji programu";
#elif L==9	// pt
	"N.&ordm; de<br />itens de programas";
#elif L==10	// tr
	"Number of<br />program items";		// eviri lazim!
#endif

const char *Txt_Number_of_public_files =
#if   L==1	// ca
	"Nombre d'arxius p&uacute;blics";
#elif L==2	// de
	"Anzahl der &ouml;ffentliche Dateien";
#elif L==3	// en
	"Number of public files";
#elif L==4	// es
	"N.&ordm; de archivos p&uacute;blicos";
#elif L==5	// fr
	"Nombre de fichiers publics";
#elif L==6	// gn
	"N.&ordm; de archivos p&uacute;blicos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file pubblici";
#elif L==8	// pl
	"Liczba publicznych pliki";
#elif L==9	// pt
	"N.&ordm; de arquivos p&uacute;blicos";
#elif L==10	// tr
	"Number of public files";		// eviri lazim!
#endif

const char *Txt_Number_of_questions =
#if   L==1	// ca
	"Nombre de preguntes";
#elif L==2	// de
	"Anzahl der Fragen";
#elif L==3	// en
	"Number of questions";
#elif L==4	// es
	"N.&ordm; de preguntas";
#elif L==5	// fr
	"Nombre de questions";
#elif L==6	// gn
	"N.&ordm; de preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di domande";
#elif L==8	// pl
	"Liczba pyta&nacute;";
#elif L==9	// pt
	"N&uacute;mero de perguntas";
#elif L==10	// tr
	"Number of questions";		// eviri lazim!
#endif

const char *Txt_Number_of_questions_to_show =
#if   L==1	// ca
	"Nombre de preguntes a mostrar";
#elif L==2	// de
	"Anzahl der anzuzeigenden Fragen";
#elif L==3	// en
	"Number of questions to show";
#elif L==4	// es
	"N.&ordm; de preguntas a mostrar";
#elif L==5	// fr
	"Nombre de questions &agrave; afficher";
#elif L==6	// gn
	"N.&ordm; de preguntas a mostrar";	// Okoteve traduccin
#elif L==7	// it
	"Numero di domande da mostrare";
#elif L==8	// pl
	"Liczba pyta&nacute; do wy&sacute;wietlenia";
#elif L==9	// pt
	"N&uacute;mero de perguntas a mostrar";
#elif L==10	// tr
	"Number of questions to show";		// eviri lazim!
#endif

const char *Txt_Number_of_BR_times_that_BR_questions_BR_have_been_BR_responded =
#if   L==1	// ca
	"Nombre de<br />vegades<br />que s'ha<br />respost";
#elif L==2	// de
	"Anzahl der<br />beantworteten<br />Fragen";
#elif L==3	// en
	"Number of<br />times that<br />questions<br />have been<br />responded";
#elif L==4	// es
	"N.&ordm; de<br />veces<br />que se ha<br />respondido";
#elif L==5	// fr
	"Nombre de<br />fois o&ugrave;<br />les questions<br />ont &eacute;t&eacute;<br />r&eacute;pondues";
#elif L==6	// gn
	"N.&ordm; de<br />veces<br />que se ha<br />respondido";			// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />volte in cui<br />&egrave; stata<br />data risposta<br />alle domande";
#elif L==8	// pl
	"Ile razy<br />udzielono<br />odpowiedzi<br />na pytania";
#elif L==9	// pt
	"N.&ordm; de<br />vezes que<br />as perguntas<br />foram<br />respondidas";
#elif L==10	// tr
	"Number of<br />times that<br />questions<br />have been<br />responded";	// eviri lazim!
#endif

const char *Txt_Number_of_threads =
#if   L==1	// ca
	"Nombre de discussions";
#elif L==2	// de
	"Anzahl der Threads";
#elif L==3	// en
	"Number of threads";
#elif L==4	// es
	"N.&ordm; de discusiones";
#elif L==5	// fr
	"Nombre de fils";
#elif L==6	// gn
	"N.&ordm; de discusiones";	// Okoteve traduccin
#elif L==7	// it
	"Numero di discussioni";
#elif L==8	// pl
	"Liczba watk&oacute;w";
#elif L==9	// pt
	"N.&ordm; de discuss&otilde;es";
#elif L==10	// tr
	"Number of threads";		// eviri lazim!
#endif

const char *Txt_Number_of_threads_BR_per_forum =
#if   L==1	// ca
	"Nombre de discussions<br />per f&ograve;rum";
#elif L==2	// de
	"Anzahl der Threads<br />pro Forum";
#elif L==3	// en
	"Number of threads<br />per forum";
#elif L==4	// es
	"N.&ordm; de discusiones<br />por foro";
#elif L==5	// fr
	"Nombre de fils<br />par forum";
#elif L==6	// gn
	"N.&ordm; de discusiones<br />por foro";	// Okoteve traduccin
#elif L==7	// it
	"Numero di discussioni<br />per forum";
#elif L==8	// pl
	"Liczba watk&oacute;w<br />per forum";
#elif L==9	// pt
	"N.&ordm; de discuss&otilde;es<br />por f&oacute;rum";
#elif L==10	// tr
	"Number of threads<br />per forum";		// eviri lazim!
#endif

const char *Txt_Number_of_BR_assignments =
#if   L==1	// ca
	"Nombre<br />d'activitats";
#elif L==2	// de
	"Anzahl der<br />Aufgaben";
#elif L==3	// en
	"Number of<br />assignments";
#elif L==4	// es
	"N.&ordm; de<br />actividades";
#elif L==5	// fr
	"Nombre<br />d'activit&eacute;s";
#elif L==6	// gn
	"N.&ordm; de<br />actividades";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />attivit&agrave;";
#elif L==8	// pl
	"Number of<br />assignments";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N.&ordm; de<br />atividades";
#elif L==10	// tr
	"&Ouml;dev<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_with_BR_assignments =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb activitats";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Aufgaben";
#elif L==3	// en
	"Number of<br />courses with<br />assignments";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con actividades";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec activit&eacute;s";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con actividades";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />attivit&agrave;";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />zadania";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com atividades";
#elif L==10	// tr
	"&Ouml;dev verilen<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_with_BR_exams =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb ex&agrave;mens";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Pr&uuml;fung";
#elif L==3	// en
	"Number of<br />courses with<br />exams";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con ex&aacute;menes";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec examens";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con ex&aacute;menes";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />esame";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />egzamin&oacute;w";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com exames";
#elif L==10	// tr
	"S&inodot;navl&inodot;<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_with_BR_exportable_BR_test_BR_questions =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test<br />exportables";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit exportierbar<br />Testfragen";
#elif L==3	// en
	"Number of<br />courses with<br />exportable<br />test<br />questions";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test<br />exportables";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />domande<br />de test<br />esportabili";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w<br />z eksportowane<br />pyta&nacute;<br />testowych";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com<br />perguntas<br />de test<br />export&aacute;veis";
#elif L==10	// tr
	"D&inodot;&scedil;a<br />aktar&inodot;labilir<br />test<br />sorular&inodot;na<br />sahip<br />kurs";
#endif

const char *Txt_Number_of_BR_courses_with_BR_games =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb jocs";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Spiele";
#elif L==3	// en
	"Number of<br />courses with<br />games";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con juegos";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec jeux";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con juegos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />giochi";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />gry";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com jogos";
#elif L==10	// tr
	"Oyunlu<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_with_BR_program_items =
#if   L==1	// ca
	"Nombre<br />d'assignatures amb<br />elements de programes";
#elif L==2	// de
	"Anzahl der<br />Kursen mit<br />Programmelemente";
#elif L==3	// en
	"Number of<br />courses with<br />program items";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas con<br />items de programas";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res avec<br />d'&eacute;l&eacute;ments de programme";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas con<br />items de programas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />elementi di programmi";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />pozycji programu";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas com<br />itens de programas";
#elif L==10	// tr
	"Program &ouml;&gbreve;elerini i&ccedil;eren<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_with_BR_projects =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb projectes";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Projekte";
#elif L==3	// en
	"Number of<br />courses with<br />projects";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con proyectos";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec projets";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con proyectos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />progetti";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />projekty";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com projetos";
#elif L==10	// tr
	"Projeli<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_BR_with_test_BR_questions =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Testfragen";
#elif L==3	// en
	"Number of<br />courses<br />with test<br />questions";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con preguntas<br />de test";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con preguntas<br />de test";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />domande<br />de test";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w<br />z pyta&nacute;<br />testowych";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com<br />perguntas<br />de test";
#elif L==10	// tr
	"Sorulu<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_with_BR_rubrics =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb r&uacute;briques";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Rubriken";
#elif L==3	// en
	"Number of<br />courses with<br />rubrics";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con r&uacute;bricas";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec rubriques";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con r&uacute;bricas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />rubriche";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />rubryk";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com rubricas";
#elif L==10	// tr
	"Rubrikli<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_courses_with_BR_surveys =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb enquestes";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Umfragen";
#elif L==3	// en
	"Number of<br />courses with<br />surveys";
#elif L==4	// es
	"N.&ordm; de<br />asignaturas<br />con encuestas";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec sondages";
#elif L==6	// gn
	"N.&ordm; de<br />asignaturas<br />con encuestas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />sondaggi";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />badania";
#elif L==9	// pt
	"N.&ordm; de<br />disciplinas<br />com inqu&eacute;ritos";
#elif L==10	// tr
	"Anketli<br />ders<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_events =
#if   L==1	// ca
	"N.&ordm; d'esdeveniments";
#elif L==2	// de
	"Anzahl der Ereignisse";
#elif L==3	// en
	"Number of events";
#elif L==4	// es
	"N.&ordm; de eventos";
#elif L==5	// fr
	"Nombre d'&eacute;v&eacute;nements";
#elif L==6	// gn
	"N.&ordm; de eventos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di eventi";
#elif L==8	// pl
	"Liczba of zdarzenia";
#elif L==9	// pt
	"N&uacute;mero de eventos";
#elif L==10	// tr
	"Number of events";	// eviri lazim!
#endif

const char *Txt_Number_of_events_per_user =
#if   L==1	// ca
	"N.&ordm; d'esdeveniments per usuari";
#elif L==2	// de
	"Anzahl der Ereignisse pro Benutzer";
#elif L==3	// en
	"Number of events per user";
#elif L==4	// es
	"N.&ordm; de eventos por usuario";
#elif L==5	// fr
	"Nombre d'&eacute;v&eacute;nements par utilisateur";
#elif L==6	// gn
	"N.&ordm; de eventos por usuario";	// Okoteve traduccin
#elif L==7	// it
	"Numero di eventi per utente";
#elif L==8	// pl
	"Liczba zdarze&nacute; na u&zdot;ytkownika";
#elif L==9	// pt
	"N&uacute;mero de eventos por utilizador";
#elif L==10	// tr
	"Number of events per user";		// eviri lazim!
#endif

const char *Txt_Number_of_emails =
#if   L==1	// ca
	"Nombre de correus";
#elif L==2	// de
	"Anzahl der E-Mails";
#elif L==3	// en
	"Number of emails";
#elif L==4	// es
	"N.&ordm; de correos";
#elif L==5	// fr
	"Nombre d'emails";
#elif L==6	// gn
	"N.&ordm; de correos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di email";
#elif L==8	// pl
	"Liczba email";
#elif L==9	// pt
	"N&uacute;mero de emails";
#elif L==10	// tr
	"Number of emails";	// eviri lazim!
#endif

const char *Txt_Number_of_BR_exams =
#if   L==1	// ca
	"Nombre<br />d'ex&agrave;mens";
#elif L==2	// de
	"Anzahl der<br />Pr&uuml;fung";
#elif L==3	// en
	"Number of<br />exams";
#elif L==4	// es
	"N.&ordm; de<br />ex&aacute;menes";
#elif L==5	// fr
	"Nombre<br />d'examens";
#elif L==6	// gn
	"N.&ordm; de<br />ex&aacute;menes";	// Okoteve traduccin
#elif L==7	// it
	"Numero<br />d'esame";
#elif L==8	// pl
	"Liczba<br />egzamin&oacute;w";
#elif L==9	// pt
	"N&uacute;mero de<br />exames";
#elif L==10	// tr
	"Number of<br />exams";			// eviri lazim!
#endif

const char *Txt_Number_of_BR_games =
#if   L==1	// ca
	"Nombre<br />de jocs";
#elif L==2	// de
	"Anzahl der<br />Spiele";
#elif L==3	// en
	"Number of<br />games";
#elif L==4	// es
	"N.&ordm; de<br />juegos";
#elif L==5	// fr
	"Nombre<br />de jeux";
#elif L==6	// gn
	"N.&ordm; de<br />juegos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />giochi";
#elif L==8	// pl
	"Liczba<br />gier";
#elif L==9	// pt
	"N&uacute;mero de<br />jogos";
#elif L==10	// tr
	"Number of<br />games";		// eviri lazim!
#endif

const char *Txt_Number_of_BR_notifications =
#if   L==1	// ca
	"Nombre de<br />notificacions";
#elif L==2	// de
	"Anzahl der<br />Anmeldungen";
#elif L==3	// en
	"Number of<br />notifications";
#elif L==4	// es
	"N.&ordm; de<br />notificaciones";
#elif L==5	// fr
	"Nombre de<br />notifications";
#elif L==6	// gn
	"N.&ordm; de<br />notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />notifiche";
#elif L==8	// pl
	"Liczba<br />zgloszen";
#elif L==9	// pt
	"N&uacute;mero de<br />notifica&ccedil;&otilde;es";
#elif L==10	// tr
	"Number of<br />notifications";		// eviri lazim!
#endif

const char *Txt_Number_of_BR_projects =
#if   L==1	// ca
	"Nombre<br />de projectes.";
#elif L==2	// de
	"Anzahl der<br />Projekte";
#elif L==3	// en
	"Number of<br />projects";
#elif L==4	// es
	"N.&ordm; de<br />proyectos";
#elif L==5	// fr
	"Nombre<br />de projets";
#elif L==6	// gn
	"N.&ordm; de<br />proyectos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />progetti";
#elif L==8	// pl
	"Liczba<br />projekt&oacute;w.";
#elif L==9	// pt
	"N&uacute;mero de<br />projetos";
#elif L==10	// tr
	"Number of<br />projects";	// eviri lazim!
#endif

const char *Txt_Number_of_BR_rubrics =
#if   L==1	// ca
	"Nombre de<br />r&uacute;briques";
#elif L==2	// de
	"Anzahl der<br />Rubriken";
#elif L==3	// en
	"Number of<br />rubrics";
#elif L==4	// es
	"N.&ordm; de<br />r&uacute;bricas";
#elif L==5	// fr
	"Nombre de<br />rubriques";
#elif L==6	// gn
	"N.&ordm; de<br />r&uacute;bricas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />rubriche";
#elif L==8	// pl
	"Liczba<br />rubryk";
#elif L==9	// pt
	"N.&ordm; de<br />rubricas";
#elif L==10	// tr
	"De&gbreve;erlendirme listesi<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_students =
#if   L==1	// ca
	"Nombre d'estudiants";
#elif L==2	// de
	"Anzahl der Studenten";
#elif L==3	// en
	"Number of students";
#elif L==4	// es
	"N.&ordm; de estudiantes";
#elif L==5	// fr
	"Nombre d'&eacute;tudiants";
#elif L==6	// gn
	"N.&ordm; de estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Numero di studenti";
#elif L==8	// pl
	"Liczba studenci";
#elif L==9	// pt
	"N.&ordm; de estudantes";
#elif L==10	// tr
	"&Ouml;&gbreve;renci say&inodot;s&inodot;";
#endif

const char *Txt_Number_of_BR_surveys =
#if   L==1	// ca
	"Nombre<br />d'enquestes";
#elif L==2	// de
	"Anzahl der<br />Umfragen";
#elif L==3	// en
	"Number of<br />surveys";
#elif L==4	// es
	"N.&ordm; de<br />encuestas";
#elif L==5	// fr
	"Nombre de<br />sondages";
#elif L==6	// gn
	"N.&ordm; de<br />encuestas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />sondaggi";
#elif L==8	// pl
	"Liczba<br />badania";
#elif L==9	// pt
	"N.&ordm; de<br />inqu&eacute;ritos";
#elif L==10	// tr
	"Anket<br />say&inodot;s&inodot;";
#endif

const char *Txt_Number_BR_of_test_BR_questions =
#if   L==1	// ca
	"Nombre de<br />preguntes<br />de test";
#elif L==2	// de
	"Anzahl der<br />Testfragen";
#elif L==3	// en
	"Number<br />of test<br />questions";
#elif L==4	// es
	"N.&ordm; de<br />preguntas<br />de test";
#elif L==5	// fr
	"Nombre de<br />questions<br />de test";
#elif L==6	// gn
	"N.&ordm; de<br />preguntas<br />de test";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />domande<br />de test";
#elif L==8	// pl
	"Liczba<br />pyta&nacute;<br />testowych";
#elif L==9	// pt
	"N.&ordm; de<br />perguntas<br />de test";
#elif L==10	// tr
	"Number<br />of test<br />questions";		// eviri lazim!
#endif

const char *Txt_Number_of_users =
#if   L==1	// ca
	"Nombre d'usuaris";
#elif L==2	// de
	"Anzahl der Benutzer";
#elif L==3	// en
	"Number of users";
#elif L==4	// es
	"N.&ordm; de usuarios";
#elif L==5	// fr
	"Nombre d'utilisateurs";
#elif L==6	// gn
	"N.&ordm; de usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Numero di utenti";
#elif L==8	// pl
	"Liczba uzytkownik&oacute;w";
#elif L==9	// pt
	"N.&ordm; de utilizadores";
#elif L==10	// tr
	"Number of users";	// eviri lazim!
#endif

const char *Txt_Numeric_BR_code_BR_ISO_3166_1 =
#if   L==1	// ca
	"Codi<br />num&egrave;ric<br />ISO 3166-1";
#elif L==2	// de
	"Numerischen<br />Code<br />ISO 3166-1";
#elif L==3	// en
	"Numeric<br />code<br />ISO 3166-1";
#elif L==4	// es
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#elif L==5	// fr
	"Code<br />num&eacute;rique<br />ISO 3166-1";
#elif L==6	// gn
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==7	// it
	"Codice<br />numerico<br />ISO 3166-1";
#elif L==8	// pl
	"Nnumerycznych<br />kod<br />ISO 3166-1";
#elif L==9	// pt
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#elif L==10	// tr
	"Numeric<br />code<br />ISO 3166-1";	// eviri lazim!
#endif

const char *Txt_of_PART_OF_A_TOTAL =	// Example: transferred 2 of 3 files
#if   L==1	// ca
	"de";	// Necessita traducci
#elif L==2	// de
	"von";
#elif L==3	// en
	"of";
#elif L==4	// es
	"de";
#elif L==5	// fr
	"de";
#elif L==6	// gn
	"de";	// Okoteve traduccin
#elif L==7	// it
	"di";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"de";
#elif L==10	// tr
	"of";	// eviri lazim!
#endif

const char *Txt_Office =
#if   L==1	// ca
	"Despatx";
#elif L==2	// de
	"Raum";
#elif L==3	// en
	"Office";
#elif L==4	// es
	"Despacho";
#elif L==5	// fr
	"Bureau";
#elif L==6	// gn
	"Mba'apoha";
#elif L==7	// it
	"Ufficio";
#elif L==8	// pl
	"Biuro";
#elif L==9	// pt
	"Gabinete";
#elif L==10	// tr
	"Office";	// eviri lazim!
#endif

const char *Txt_One_message_has_been_deleted =
#if   L==1	// ca
	"S'ha suprimit un missatge.";
#elif L==2	// de
	"Eine Nachricht wurde gel&ouml;scht.";
#elif L==3	// en
	"One message has been deleted.";
#elif L==4	// es
	"Se ha eliminado un mensaje.";
#elif L==5	// fr
	"Un message a &eacute;t&eacute; supprim&eacute;.";
#elif L==6	// gn
	"Se ha eliminado un mensaje.";		// Okoteve traduccin
#elif L==7	// it
	"Un messaggio &egrave; stato rimosso.";
#elif L==8	// pl
	"Jedna wiadomosc zostala usunieta.";
#elif L==9	// pt
	"Uma mensagem foi removida.";
#elif L==10	// tr
	"One message has been deleted.";	// eviri lazim!
#endif

const char *Txt_One_user_has_been_eliminated =
#if   L==1	// ca
	"S'ha suprimit un usuari.";
#elif L==2	// de
	"Ein Benutzer wurde eliminiert.";
#elif L==3	// en
	"One user has been eliminated.";
#elif L==4	// es
	"Se ha eliminado un usuario.";
#elif L==5	// fr
	"Un utilisateur a &eacute;t&eacute; &eacute;limin&eacute;.";
#elif L==6	// gn
	"Se ha eliminado un usuario.";		// Okoteve traduccin
#elif L==7	// it
	"Un utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Jeden uzytkownik zostal usuniety.";
#elif L==9	// pt
	"Um usu&aacute;rio foi removido.";
#elif L==10	// tr
	"One user has been eliminated.";	// eviri lazim!
#endif

const char *Txt_One_user_has_been_enroled =
#if   L==1	// ca
	"S'hi ha inscrit un usuari.";
#elif L==2	// de
	"Ein Benutzer wurde registriert.";
#elif L==3	// en
	"One user has been enroled.";
#elif L==4	// es
	"Se ha inscrito un usuario.";
#elif L==5	// fr
	"Un utilisateur a &eacute;t&eacute; inscrit.";
#elif L==6	// gn
	"Se ha inscrito un usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Un utente &egrave; stato registrato.";
#elif L==8	// pl
	"Jeden uzytkownik zostal zarejestrowany.";
#elif L==9	// pt
	"Um usu&aacute;rio se registrou.";
#elif L==10	// tr
	"One user has been enroled.";	// eviri lazim!
#endif

const char *Txt_One_user_has_been_removed =
#if   L==1	// ca
	"S'ha suprimit un usuari.";
#elif L==2	// de
	"Ein Benutzer wurde entfernt.";
#elif L==3	// en
	"One user has been removed.";
#elif L==4	// es
	"Se ha eliminado un usuario.";
#elif L==5	// fr
	"Un utilisateur a &eacute;t&eacute; supprim&eacute;.";
#elif L==6	// gn
	"Se ha eliminado un usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Un utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Jeden z uzytkownik&oacute;w zostaly usuniete.";
#elif L==9	// pt
	"Um usu&aacute;rio foi removido.";
#elif L==10	// tr
	"One user has been removed.";	// eviri lazim!
#endif

const char *Txt_Online_tutoring =
#if   L==1	// ca
	"Tutories virtuals";
#elif L==2	// de
	"Virtual Anleitungen";
#elif L==3	// en
	"Online tutoring";
#elif L==4	// es
	"Tutor&iacute;as virtuales";
#elif L==5	// fr
	"Tutorat en ligne";
#elif L==6	// gn
	"Tutor&iacute;as virtuales";	// Okoteve traduccin
#elif L==7	// it
	"Tutoraggio on-line";
#elif L==8	// pl
	"Korepetycje online";
#elif L==9	// pt
	"Tutoria on-line";
#elif L==10	// tr
	"Online tutoring";		// eviri lazim!
#endif

const char *Txt_Only_courses_with_more_than_X_clicks_are_shown =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Nom&eacute;s es mostren les assignatures amb m&eacute;s de %u clics.";
#elif L==2	// de
	"Nur Kurse mit mehr als %u Klicks angezeigt.";
#elif L==3	// en
	"Only courses with more than %u clicks are shown.";
#elif L==4	// es
	"Solo se muestran las asignaturas con m&aacute;s de %u clics.";
#elif L==5	// fr
	"Seuls les mati&egrave;res avec plus de %u clics sont pr&eacute;sent&eacute;es.";
#elif L==6	// gn
	"Solo se muestran las asignaturas con m&aacute;s de %u clics.";	// Okoteve traduccin
#elif L==7	// it
	"Solo i corsi con pi&ugrave; di %u scatti sono mostrati.";
#elif L==8	// pl
	"Tylko kursy z ponad 100 klikni&eogon;&cacute; s&aogon; pokazane.";
#elif L==9	// pt
	"Somente disciplinas com mais de %u cliques s&atilde;o mostradas.";
#elif L==10	// tr
	"Only courses with more than %u clicks are shown.";		// eviri lazim!
#endif

const char *Txt_only_if_the_scope_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"nom&eacute;s si l'&agrave;mbit &eacute;s %s";
#elif L==2	// de
	"nur dann, wenn der Geltungsbereich ist %s";
#elif L==3	// en
	"only if the scope is %s";
#elif L==4	// es
	"solo si el &aacute;mbito es %s";
#elif L==5	// fr
	"seulement si le port&eacute;e est %s";
#elif L==6	// gn
	"solo si el &aacute;mbito es %s";	// Okoteve traduccin
#elif L==7	// it
	"solo se l'ambito &egrave; %s";
#elif L==8	// pl
	"tylko wtedy, gdy zakres jest %s";
#elif L==9	// pt
	"somente se o &acirc;mbito &eacute; %s";
#elif L==10	// tr
	"only if the scope is %s";		// eviri lazim!
#endif

const char *Txt_only_students =
#if   L==1	// ca
	", solo estudiantes";	// Necessita traducci
#elif L==2	// de
	", nur Studenten";
#elif L==3	// en
	", only students";
#elif L==4	// es
	", solo estudiantes";
#elif L==5	// fr
	", seulement &eacute;tudiants";
#elif L==6	// gn
	", solo estudiantes";	// Okoteve traduccin
#elif L==7	// it
	", solo studenti";
#elif L==8	// pl
	", tylko studenci";
#elif L==9	// pt
	", apenas estudantes";
#elif L==10	// tr
	", only students";	// eviri lazim!
#endif

const char *Txt_only_teachers =
#if   L==1	// ca
	", professors";
#elif L==2	// de
	", Lehrkr&auml;fte";
#elif L==3	// en
	", teachers";
#elif L==4	// es
	", profesores";
#elif L==5	// fr
	", enseignants";
#elif L==6	// gn
	", profesores";	// Okoteve traduccin
#elif L==7	// it
	", professori";
#elif L==8	// pl
	", nauczycieli";
#elif L==9	// pt
	", professores";
#elif L==10	// tr
	", teachers";	// eviri lazim!
#endif

const char *Txt_only_unread_messages =
#if   L==1	// ca
	"nom&eacute;s els missatges no llegits";
#elif L==2	// de
	"nur ungelesene Nachrichten";
#elif L==3	// en
	"only unread messages";
#elif L==4	// es
	"&uacute;nicamente los mensajes no le&iacute;dos";
#elif L==5	// fr
	"uniquement les messages non lus";
#elif L==6	// gn
	"&uacute;nicamente los mensajes no le&iacute;dos";	// Okoteve traduccin
#elif L==7	// it
	"solo i messaggi non letti";
#elif L==8	// pl
	"only unread messages";					// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"apenas mensagens n&atilde;o lidas";
#elif L==10	// tr
	"only unread messages";					// eviri lazim!
#endif

const char *Txt_open_document =
#if   L==1	// ca
	"document obert";
#elif L==2	// de
	"ge&ouml;ffnet Dokumente";
#elif L==3	// en
	"open document";
#elif L==4	// es
	"documento abierto";
#elif L==5	// fr
	"document ouvert";
#elif L==6	// gn
	"documento abierto";	// Okoteve traduccin
#elif L==7	// it
	"documento aperto";
#elif L==8	// pl
	"otwarty dokument";
#elif L==9	// pt
	"documento aberto";
#elif L==10	// tr
	"open document";	// eviri lazim!
#endif

const char *Txt_open_documents =
#if   L==1	// ca
	"documents oberts";
#elif L==2	// de
	"ge&ouml;ffnete Dokumente";
#elif L==3	// en
	"open documents";
#elif L==4	// es
	"documentos abiertos";
#elif L==5	// fr
	"documents ouverts";
#elif L==6	// gn
	"documentos abiertos";	// Okoteve traduccin
#elif L==7	// it
	"documenti aperti";
#elif L==8	// pl
	"otwarte dokumenty";
#elif L==9	// pt
	"documentos abertos";
#elif L==10	// tr
	"open documents";	// eviri lazim!
#endif

const char *Txt_Open_survey =
#if   L==1	// ca
	"Enquesta oberta";
#elif L==2	// de
	"Ge&ouml;ffneten Umfrage";
#elif L==3	// en
	"Open survey";
#elif L==4	// es
	"Encuesta abierta";
#elif L==5	// fr
	"Sondage ouvert";
#elif L==6	// gn
	"Encuesta abierta";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio aperto";
#elif L==8	// pl
	"Otwarte badania";
#elif L==9	// pt
	"Inqu&eacute;rito aberta";
#elif L==10	// tr
	"Open survey";		// eviri lazim!
#endif

const char *Txt_Opening_of_groups =
#if   L==1	// ca
	"Obertura de grups";
#elif L==2	// de
	"Er&ouml;ffnung von Gruppen";
#elif L==3	// en
	"Opening of groups";
#elif L==4	// es
	"Apertura de grupos";
#elif L==5	// fr
	"Ouverture des groupes";
#elif L==6	// gn
	"Apertura de grupos";		// Okoteve traduccin
#elif L==7	// it
	"Apertura di gruppi";
#elif L==8	// pl
	"Otwarcie grup";
#elif L==9	// pt
	"Abertura de grupos";
#elif L==10	// tr
	"Opening of groups";	// eviri lazim!
#endif

const char *Txt_Option_under_development =
#if   L==1	// ca
	"Opci&oacute; en desenvolupament.";
#elif L==2	// de
	"Option in der Entwicklung.";
#elif L==3	// en
	"Option under development.";
#elif L==4	// es
	"Opci&oacute;n en desarrollo.";
#elif L==5	// fr
	"Option en cours de d&eacute;veloppement.";
#elif L==6	// gn
	"Opci&oacute;n en desarrollo.";		// Okoteve traduccin
#elif L==7	// it
	"Opzione in fase di sviluppo.";
#elif L==8	// pl
	"Opcja w fazie rozwoju.";
#elif L==9	// pt
	"Op&ccedil;&atilde;o em desenvolvimento.";
#elif L==10	// tr
	"Option under development.";		// eviri lazim!
#endif

const char *Txt_optional =
#if   L==1	// ca
	"opcional";
#elif L==2	// de
	"wahlweise";
#elif L==3	// en
	"optional";
#elif L==4	// es
	"opcional";
#elif L==5	// fr
	"en option";
#elif L==6	// gn
	"opcional";	// Okoteve traduccin
#elif L==7	// it
	"opzionale";
#elif L==8	// pl
	"do wyboru";
#elif L==9	// pt
	"opcional";
#elif L==10	// tr
	"optional";	// eviri lazim!
#endif

const char *Txt_Options =
#if   L==1	// ca
	"Opcions";
#elif L==2	// de
	"Optionen";
#elif L==3	// en
	"Options";
#elif L==4	// es
	"Opciones";
#elif L==5	// fr
	"Choix";
#elif L==6	// gn
	"Opciones";
#elif L==7	// it
	"Opzioni";
#elif L==8	// pl
	"Opcje";
#elif L==9	// pt
	"Op&ccedil;&otilde;es";
#elif L==10	// tr
	"Se&ccedil;enekler";
#endif

const char *Txt_or_you_can_create_a_new_link_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o pot crear un novo enlla&ccedil;"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==2	// de
	"&hellip;Oder Sie k&ouml;nnen ein neues Link"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==3	// en
	"&hellip;or you can create a new link"
	" inside the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez cr&eacute;er un nouveau lien"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure &egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;czy mo&zdot;esz utworzy&cacute; nowy link"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode criar uma nova liga&ccedil;&atilde;o"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#elif L==10	// tr
	"&hellip;or you can create a new link"
	" inside the folder <strong>%s</strong>:";	// eviri lazim!
#endif

const char *Txt_or_you_can_make_a_file_copy_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o pot realitzar una c&ograve;pia d'arxius"
	" a la carpeta <strong>%s</strong>:";
#elif L==2	// de
	"&hellip;Oder Sie k&ouml;nnen die Kopie einer Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3	// en
	"&hellip;or you can make a file copy"
	" to the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez faire une copie de fichiers"
	" vers le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";		// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure &egrave; possibile effettuare una copia dei file"
	" nella cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;czy mozna zrobic kopie pliku"
	" do folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode fazer uma c&oacute;pia de arquivo"
	" na diret&oacute;rio <strong>%s</strong>:";
#elif L==10	// tr
	"&hellip;or you can make a file copy"
	" to the folder <strong>%s</strong>:";		// eviri lazim!
#endif

const char *Txt_or_you_can_upload_a_new_file_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Necessita traducci
#elif L==2	// de
	"&hellip;oder senden Sie eine neue Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3	// en
	"&hellip;or you can upload a new file"
	" to the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez envoyer un nouveau fichier"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure si pu&ograve; inviare un nuovo file"
	" nella cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode enviar um novo arquivo"
	" para o diret&oacute;rio <strong>%s</strong>:";
#elif L==10	// tr
	"&hellip;or you can upload a new file"
	" to the folder <strong>%s</strong>:";	// eviri lazim!
#endif

const char *Txt_or_you_can_upload_new_files_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Necessita traducci
#elif L==2	// de
	"&hellip;oder senden Sie neue Dateien"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3	// en
	"&hellip;or you can upload new files"
	" to the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez envoyer des nouveaux fichiers"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure si pu&ograve; inviare nuovi file"
	" nella cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode enviar novos arquivos"
	" para o diret&oacute;rio <strong>%s</strong>:";
#elif L==10	// tr
	"&hellip;or you can upload new files"
	" to the folder <strong>%s</strong>:";	// eviri lazim!
#endif

const char *Txt_Original_message =
#if   L==1	// ca
	"Missatge original";
#elif L==2	// de
	"Original-Nachricht";
#elif L==3	// en
	"Original message";
#elif L==4	// es
	"Mensaje original";
#elif L==5	// fr
	"Message d'origine";
#elif L==6	// gn
	"Mensaje original";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio originale";
#elif L==8	// pl
	"Oryginalna wiadomo&sacute;&cacute;";
#elif L==9	// pt
	"Mensagem original";
#elif L==10	// tr
	"Original message";	// eviri lazim!
#endif

const char *Txt_Other_actions =
#if   L==1	// ca
	"Altres accions";
#elif L==2	// de
	"Andere Handlungen";
#elif L==3	// en
	"Other actions";
#elif L==4	// es
	"Otras acciones";
#elif L==5	// fr
	"Autres actions";
#elif L==6	// gn
	"Otras acciones";	// Okoteve traduccin
#elif L==7	// it
	"Altri azioni";
#elif L==8	// pl
	"Inne dzia&lstrok;ania";
#elif L==9	// pt
	"Outras a&ccedil;&otilde;es";
#elif L==10	// tr
	"Other actions";	// eviri lazim!
#endif

const char *Txt_Other_countries =
#if   L==1	// ca
	"Altres pa&iuml;sos";
#elif L==2	// de
	"Andere L&auml;nder";
#elif L==3	// en
	"Other countries";
#elif L==4	// es
	"Otros pa&iacute;ses";
#elif L==5	// fr
	"Autres pays";
#elif L==6	// gn
	"Otros pa&iacute;ses";	// Okoteve traduccin
#elif L==7	// it
	"Altri paesi";
#elif L==8	// pl
	"Inne kraje";
#elif L==9	// pt
	"Outros pa&iacute;ses";
#elif L==10	// tr
	"Other countries";	// eviri lazim!
#endif

const char *Txt_Other_departments =
#if   L==1	// ca
	"Altres departaments";
#elif L==2	// de
	"Andere Abteilung";
#elif L==3	// en
	"Other departments";
#elif L==4	// es
	"Otros departamentos";
#elif L==5	// fr
	"Autres d&eacute;partements";
#elif L==6	// gn
	"Otros departamentos";	// Okoteve traduccin
#elif L==7	// it
	"Altri dipartimenti";
#elif L==8	// pl
	"Inne wydzialy";
#elif L==9	// pt
	"Outros departamentos";
#elif L==10	// tr
	"Other departments";	// eviri lazim!
#endif

const char *Txt_Other_emails =
#if   L==1	// ca
	"Altres correus";
#elif L==2	// de
	"Andere E-Mails";
#elif L==3	// en
	"Other emails";
#elif L==4	// es
	"Otros correos";
#elif L==5	// fr
	"Autres emails";
#elif L==6	// gn
	"Otros correos";	// Okoteve traduccin
#elif L==7	// it
	"Altre email";
#elif L==8	// pl
	"Inne email";
#elif L==9	// pt
	"Outros emails";
#elif L==10	// tr
	"Other emails";		// eviri lazim!
#endif

const char *Txt_Other_nicknames =
#if   L==1	// ca
	"Altres sobrenoms";
#elif L==2	// de
	"Andere Benutzername";
#elif L==3	// en
	"Other nicknames";
#elif L==4	// es
	"Otros apodos";
#elif L==5	// fr
	"Autres surnoms";
#elif L==6	// gn
	"Otros apodos";		// Okoteve traduccin
#elif L==7	// it
	"Altri nomi utenti";
#elif L==8	// pl
	"Inne przydomki";
#elif L==9	// pt
	"Outras alcunhas";
#elif L==10	// tr
	"Other nicknames";	// eviri lazim!
#endif

const char *Txt_Other_places =
#if   L==1	// ca
	"Altres llocs";
#elif L==2	// de
	"Andere Standorte";
#elif L==3	// en
	"Other places";
#elif L==4	// es
	"Otros lugares";
#elif L==5	// fr
	"Autres emplacements";
#elif L==6	// gn
	"Otros lugares";	// Okoteve traduccin
#elif L==7	// it
	"Altre localit&agrave;";
#elif L==8	// pl
	"Inne miejsca";
#elif L==9	// pt
	"Outras localiza&ccedil;oes";
#elif L==10	// tr
	"Other places";		// eviri lazim!
#endif

const char *Txt_Other_recipients =
#if   L==1	// ca
	"Altres destinataris";
#elif L==2	// de
	"Andere Empf&auml;nger";
#elif L==3	// en
	"Other recipients";
#elif L==4	// es
	"Otros destinatarios";
#elif L==5	// fr
	"Autres destinataires";
#elif L==6	// gn
	"Otros destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Altri destinatari";
#elif L==8	// pl
	"Inne odbiorc&oacute;w";
#elif L==9	// pt
	"Outros destinat&aacute;rios";
#elif L==10	// tr
	"Other recipients";	// eviri lazim!
#endif

const char *Txt_page =
#if   L==1	// ca
	"p&agrave;g.";
#elif L==2	// de
	"Page";
#elif L==3	// en
	"page";
#elif L==4	// es
	"p&aacute;g.";
#elif L==5	// fr
	"page";
#elif L==6	// gn
	"kuatiarogue";
#elif L==7	// it
	"pag.";
#elif L==8	// pl
	"strona";
#elif L==9	// pt
	"p&aacute;g.";
#elif L==10	// tr
	"page";	// eviri lazim!
#endif

const char *Txt_Page_X_of_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"P&agrave;gina %u de %u";
#elif L==2	// de
	"Seite %u von %u";
#elif L==3	// en
	"Page %u of %u";
#elif L==4	// es
	"P&aacute;gina %u de %u";
#elif L==5	// fr
	"Page %u de %u";
#elif L==6	// gn
	"P&aacute;gina %u de %u";	// Okoteve traduccin
#elif L==7	// it
	"Pagina %u di %u";
#elif L==8	// pl
	"Strona %u z %u";
#elif L==9	// pt
	"P&aacute;gina %u de %u";
#elif L==10	// tr
	"Page %u of %u";		// eviri lazim!
#endif

const char *Txt_PAGE1_Page_generated_in =
#if   L==1	// ca
	"P&agrave;gina generada en";
#elif L==2	// de
	"Page generated in";
#elif L==3	// en
	"Page generated in";
#elif L==4	// es
	"P&aacute;gina generada en";
#elif L==5	// fr
	"Page g&eacute;n&eacute;r&eacute;e en";
#elif L==6	// gn
	"P&aacute;gina generada en";	// Okoteve traduccin
#elif L==7	// it
	"Pagina generata in";
#elif L==8	// pl
	"Strona wygenerowana w";
#elif L==9	// pt
	"P&aacute;gina gerada em";
#elif L==10	// tr
	"Page generated in";		// eviri lazim!
#endif

const char *Txt_PAGE2_and_sent_in =
#if   L==1	// ca
	"i enviat en";
#elif L==2	// de
	"gesendet und in";
#elif L==3	// en
	"and sent in";
#elif L==4	// es
	"y enviada en";
#elif L==5	// fr
	"et envoy&eacute;e en";
#elif L==6	// gn
	"y enviada en";	// Okoteve traduccin
#elif L==7	// it
	"e inviata in";
#elif L==8	// pl
	"i wysylane w";
#elif L==9	// pt
	"e enviada em";
#elif L==10	// tr
	"and sent in";	// eviri lazim!
#endif

const char *Txt_PAGES_Previous =
#if   L==1	// ca
	"Anteriors";
#elif L==2	// de
	"Zur&uuml;ck";
#elif L==3	// en
	"Previous";
#elif L==4	// es
	"Anteriores";
#elif L==5	// fr
	"Pr&eacute;c&eacute;dentes";
#elif L==6	// gn
	"Anteriores";	// Okoteve traduccin
#elif L==7	// it
	"Precedenti";
#elif L==8	// pl
	"Poprzedni";
#elif L==9	// pt
	"Anteriores";
#elif L==10	// tr
	"Previous";	// eviri lazim!
#endif

const char *Txt_PAGES_Next =
#if   L==1	// ca
	"Seg&uuml;ents";
#elif L==2	// de
	"N&auml;chste";
#elif L==3	// en
	"Next";
#elif L==4	// es
	"Siguientes";
#elif L==5	// fr
	"Suivantes";
#elif L==6	// gn
	"Siguientes";	// Okoteve traduccin
#elif L==7	// it
	"Seguenti";
#elif L==8	// pl
	"Nastepny";
#elif L==9	// pt
	"Seguintes";
#elif L==10	// tr
	"Next";		// eviri lazim!
#endif

const char *Txt_Password =
#if   L==1	// ca
	"Contrasenya";
#elif L==2	// de
	"Passwort";
#elif L==3	// en
	"Password";
#elif L==4	// es
	"Contrase&ntilde;a";
#elif L==5	// fr
	"Mot de passe";
#elif L==6	// gn
	"Tera&ntilde;emi";
#elif L==7	// it
	"Password";
#elif L==8	// pl
	"Has&lstrok;o";
#elif L==9	// pt
	"Senha";
#elif L==10	// tr
	"&Scedil;ifre";
#endif

const char *Txt_password =
#if   L==1	// ca
	"contrasenya";
#elif L==2	// de
	"Passwort";
#elif L==3	// en
	"password";
#elif L==4	// es
	"contrase&ntilde;a";
#elif L==5	// fr
	"mot de passe";
#elif L==6	// gn
	"tera&ntilde;emi";
#elif L==7	// it
	"password";
#elif L==8	// pl
	"has&lstrok;o";
#elif L==9	// pt
	"senha";
#elif L==10	// tr
	"&scedil;ifre";
#endif

const char *Txt_Paste =
#if   L==1	// ca
	"Enganxar";
#elif L==2	// de
	"Einf&uuml;gen";
#elif L==3	// en
	"Paste";
#elif L==4	// es
	"Pegar";
#elif L==5	// fr
	"Coller";
#elif L==6	// gn
	"Pegar";	// Okoteve traduccin
#elif L==7	// it
	"Incolla";
#elif L==8	// pl
	"Wklej";
#elif L==9	// pt
	"Colar";
#elif L==10	// tr
	"Paste";	// eviri lazim!
#endif

const char *Txt_Pause =
#if   L==1	// ca
	"Fer una pausa";
#elif L==2	// de
	"Wiedergabe anzuhalten";
#elif L==3	// en
	"Pause";
#elif L==4	// es
	"Hacer una pausa";
#elif L==5	// fr
	"Faire une pause";
#elif L==6	// gn
	"Hacer una pausa";	// Okoteve traduccin
#elif L==7	// it
	"Mettere in pausa";
#elif L==8	// pl
	"Wstrzyma&cacute;";
#elif L==9	// pt
	"Pausar";
#elif L==10	// tr
	"Pause";		// eviri lazim!
#endif

const char *Txt_PERCENT_of_users =
#if   L==1	// ca
	"% d'usuaris";
#elif L==2	// de
	"% der Benutzer";
#elif L==3	// en
	"% of users";
#elif L==4	// es
	"% de usuarios";
#elif L==5	// fr
	"% d'utilisateurs";
#elif L==6	// gn
	"% de usuarios";	// Okoteve traduccin
#elif L==7	// it
	"% di utenti";
#elif L==8	// pl
	"% uzytkownik&oacute;w";
#elif L==9	// pt
	"% de utilizadores";
#elif L==10	// tr
	"% of users";		// eviri lazim!
#endif

const char *Txt_Permalink =
#if   L==1	// ca
	"Enlla&ccedil; permanent";
#elif L==2	// de
	"Permalink";
#elif L==3	// en
	"Permalink";
#elif L==4	// es
	"Permalink";
#elif L==5	// fr
	"Permalien";
#elif L==6	// gn
	"Permalink";	// Okoteve traduccin
#elif L==7	// it
	"Permalink";
#elif L==8	// pl
	"Odno&sacute;nik bezpo&sacute;redni";
#elif L==9	// pt
	"Liga&ccedil;&atilde;o permanente";
#elif L==10	// tr
	"Permalink";	// eviri lazim!
#endif

const char *Txt_Personal_information =
#if   L==1	// ca
	"Dades personals";
#elif L==2	// de
	"Pers&ouml;nliche Angaben";
#elif L==3	// en
	"Personal information";
#elif L==4	// es
	"Datos personales";
#elif L==5	// fr
	"Informations personnelles";
#elif L==6	// gn
	"Datos personales";	// Okoteve traduccin
#elif L==7	// it
	"Informazione personale";
#elif L==8	// pl
	"Informacje osobiste";
#elif L==9	// pt
	"Informa&ccedil;&atilde;o pessoal";
#elif L==10	// tr
	"Personal information";	// eviri lazim!
#endif

const char *Txt_Phone =
#if   L==1	// ca
	"Tel&egrave;fon";
#elif L==2	// de
	"Telefon";
#elif L==3	// en
	"Phone";
#elif L==4	// es
	"Tel&eacute;fono";
#elif L==5	// fr
	"T&eacute;l&eacute;phone";
#elif L==6	// gn
	"Pumbyry";
#elif L==7	// it
	"Telefono";
#elif L==8	// pl
	"Telefon";
#elif L==9	// pt
	"Telefone";
#elif L==10	// tr
	"Phone";	// eviri lazim!
#endif

const char *Txt_Photo =
#if   L==1	// ca
	"Foto";
#elif L==2	// de
	"Foto";
#elif L==3	// en
	"Photo";
#elif L==4	// es
	"Foto";
#elif L==5	// fr
	"Photo";
#elif L==6	// gn
	"Ta'anga";
#elif L==7	// it
	"Foto";
#elif L==8	// pl
	"Zdj&eogon;cie";
#elif L==9	// pt
	"Foto";
#elif L==10	// tr
	"Photo";	// eviri lazim!
#endif

const char *Txt_Photo_has_been_updated =
#if   L==1	// ca
	"La fotografia s'ha actualitzat.";
#elif L==2	// de
	"Foto wurde aktualisiert.";
#elif L==3	// en
	"Photo has been updated.";
#elif L==4	// es
	"La fotograf&iacute;a se ha actualizado.";
#elif L==5	// fr
	"La photo a &eacute;t&eacute; mise &agrave; jour.";
#elif L==6	// gn
	"La fotograf&iacute;a se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"La foto &egrave; stata aggiornata.";
#elif L==8	// pl
	"Zdjecie zostalo zaktualizowane.";
#elif L==9	// pt
	"A fotografia foi atualizada.";
#elif L==10	// tr
	"Photo has been updated.";			// eviri lazim!
#endif

const char *Txt_Photo_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"S'ha eliminat la foto de <strong>%s</strong>.";
#elif L==2	// de
	"Foto von <strong>%s</strong> wurde entfernt.";
#elif L==3	// en
	"Photo of <strong>%s</strong> has been removed.";
#elif L==4	// es
	"Se ha eliminado la foto de <strong>%s</strong>.";
#elif L==5	// fr
	"La photo de <strong>%s</strong> a &eacute;t&eacute; supprim&eacute;e.";
#elif L==6	// gn
	"Se ha eliminado la foto de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La foto de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8	// pl
	"Zdjecie <strong>%s</strong> zostalo usuniete.";
#elif L==9	// pt
	"A foto de <strong>%s</strong> foi removida.";
#elif L==10	// tr
	"Photo of <strong>%s</strong> has been removed.";	// eviri lazim!
#endif

const char *Txt_PHOTO_PROCESSING_CAPTIONS[3] =
	{
#if   L==1	// ca
	"Rostre seleccionat"
#elif L==2	// de
	"Gesicht ausgew&auml;hlt"
#elif L==3	// en
	"Face selected"
#elif L==4	// es
	"Rostro seleccionado"
#elif L==5	// fr
	"Visage s&eacute;lectionn&eacute;"
#elif L==6	// gn
	"Rostro seleccionado"	// Okoteve traduccin
#elif L==7	// it
	"Viso selezionato"
#elif L==8	// pl
	"Twarze wybranych"
#elif L==9	// pt
	"Rosto selecionado"
#elif L==10	// tr
	"Face selected"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Contrast i saturaci&oacute; millorats"
#elif L==2	// de
	"Kontrast und S&auml;ttigung verbessert"
#elif L==3	// en
	"Contrast and saturation enhanced"
#elif L==4	// es
	"Contraste y saturaci&oacute;n mejorados"
#elif L==5	// fr
	"Contraste et saturation am&eacute;lior&eacute;s"
#elif L==6	// gn
	"Contraste y saturaci&oacute;n mejorados"	// Okoteve traduccin
#elif L==7	// it
	"Contrasto e saturazione migliorata"
#elif L==8	// pl
	"Zwiekszenie kontrastu i nasycenia"
#elif L==9	// pt
	"Contraste e satura&ccedil;&atilde;o aprimorados"
#elif L==10	// tr
	"Contrast and saturation enhanced"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Balan&ccedil; de blancs realitzat"
#elif L==2	// de
	"Wei&szlig;abgleich gemacht"
#elif L==3	// en
	"White balance made"
#elif L==4	// es
	"Balance de blancos realizado"
#elif L==5	// fr
	"Balance des blancs faite"
#elif L==6	// gn
	"Balance de blancos realizado"	// Okoteve traduccin
#elif L==7	// it
	"Bilanciamento di bianco realizzato"
#elif L==8	// pl
	"Balans bieli sie"
#elif L==9	// pt
	"Balan&ccedil;o de branco realizado"
#elif L==10	// tr
	"White balance made"		// eviri lazim!
#endif
	};

const char *Txt_PHOTO_SHAPES[PhoSha_NUM_SHAPES] =
	{
#if   L==1	// ca
	"Cercle"
#elif L==2	// de
	"Kreis"
#elif L==3	// en
	"Circle"
#elif L==4	// es
	"C&iacute;rculo"
#elif L==5	// fr
	"Cercle"
#elif L==6	// gn
	"C&iacute;rculo"	// Okoteve traduccin
#elif L==7	// it
	"Cerchio"
#elif L==8	// pl
	"Ko&lstrok;o"
#elif L==9	// pt
	"C&iacute;rculo"
#elif L==10	// tr
	"Circle"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"El&middot;lipse"
#elif L==2	// de
	"Ellipse"
#elif L==3	// en
	"Ellipse"
#elif L==4	// es
	"Elipse"
#elif L==5	// fr
	"Ellipse"
#elif L==6	// gn
	"Elipse"	// Okoteve traduccin
#elif L==7	// it
	"Ellisse"
#elif L==8	// pl
	"Elipsa"
#elif L==9	// pt
	"Elipse"
#elif L==10	// tr
	"Ellipse"	// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Oval"
#elif L==2	// de
	"Oval"
#elif L==3	// en
	"Oval"
#elif L==4	// es
	"&Oacute;valo"
#elif L==5	// fr
	"Ovale"
#elif L==6	// gn
	"&Oacute;valo"	// Okoteve traduccin
#elif L==7	// it
	"Ovale"
#elif L==8	// pl
	"Owalny"
#elif L==9	// pt
	"Ovalo"
#elif L==10	// tr
	"Oval"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"Rectangle"
#elif L==2	// de
	"Rechteck"
#elif L==3	// en
	"Rectangle"
#elif L==4	// es
	"Rect&aacute;ngulo"
#elif L==5	// fr
	"Rectangle"
#elif L==6	// gn
	"Rect&aacute;ngulo"	// Okoteve traduccin
#elif L==7	// it
	"Rettangolo"
#elif L==8	// pl
	"Prostok&aogon;t"
#elif L==9	// pt
	"Ret&acirc;ngulo"
#elif L==10	// tr
	"Rectangle"		// eviri lazim!
#endif
	};

const char *Txt_Photo_removed =
#if   L==1	// ca
	"Fotografia eliminada.";
#elif L==2	// de
	"Foto entfernt.";
#elif L==3	// en
	"Photo removed.";
#elif L==4	// es
	"Fotograf&iacute;a eliminada.";
#elif L==5	// fr
	"Photo supprim&eacute;e.";
#elif L==6	// gn
	"Fotograf&iacute;a eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Foto rimossa.";
#elif L==8	// pl
	"Zdjecie usuniete.";
#elif L==9	// pt
	"Fotografia removida.";
#elif L==10	// tr
	"Photo removed.";		// eviri lazim!
#endif

const char *Txt_photos =
#if   L==1	// ca
	"fotos";
#elif L==2	// de
	"Fotos";
#elif L==3	// en
	"photos";
#elif L==4	// es
	"fotos";
#elif L==5	// fr
	"photos";
#elif L==6	// gn
	"fotos";	// Okoteve traduccin
#elif L==7	// it
	"foto";
#elif L==8	// pl
	"zdjecia";
#elif L==9	// pt
	"fotos";
#elif L==10	// tr
	"photos";	// eviri lazim!
#endif

const char *Txt_XxY_pixels_or_higher =
#if   L==1	// ca
	"p&iacute;xels o m&eacute;s gran";
#elif L==2	// de
	"Pixel oder h&ouml;her";
#elif L==3	// en
	"pixels or higher";
#elif L==4	// es
	"p&iacute;xeles o mayor";
#elif L==5	// fr
	"pixels ou plus";
#elif L==6	// gn
	"p&iacute;xeles o mayor";	// Okoteve traduccin
#elif L==7	// it
	"pixel o superiore";
#elif L==8	// pl
	"pikseli lub wy&zdot;szej";
#elif L==9	// pt
	"pixels ou superior";
#elif L==10	// tr
	"pixels or higher";		// eviri lazim!
#endif

const char *Txt_Place =		// A particular spot or area
#if   L==1	// ca
	"Lloc";
#elif L==2	// de
	"Standort";
#elif L==3	// en
	"Place";
#elif L==4	// es
	"Lugar";
#elif L==5	// fr
	"Emplacement";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Luogo";
#elif L==8	// pl
	"Miejsce";
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o";
#elif L==10	// tr
	"Place";	// eviri lazim!
#endif

const char *Txt_Place_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Lloc <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Standort <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Place <strong>%s</strong> removed.";
#elif L==4	// es
	"Lugar <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Emplacement <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Lugar <strong>%s</strong> eliminado.";		// Okoteve traduccin
#elif L==7	// it
	"Luogo <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Miejsce <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Lugar <strong>%s</strong> removido.";
#elif L==10	// tr
	"Place <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Place_unspecified =
#if   L==1	// ca
	"Lloc sense especificar";
#elif L==2	// de
	"Standort nicht spezifiziert";
#elif L==3	// en
	"Place unspecified";
#elif L==4	// es
	"Lugar sin especificar";
#elif L==5	// fr
	"Emplacement non sp&eacute;cifi&eacute;";
#elif L==6	// gn
	"Lugar sin especificar";	// Okoteve traduccin
#elif L==7	// it
	"Luogo non specificato";
#elif L==8	// pl
	"Miejsce nieokreslone";
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o indeterminada";
#elif L==10	// tr
	"Place unspecified";		// eviri lazim!
#endif

const char *Txt_Places =
#if   L==1	// ca
	"Llocs";
#elif L==2	// de
	"Standorte";
#elif L==3	// en
	"Places";
#elif L==4	// es
	"Lugares / sedes";
#elif L==5	// fr
	"Emplacements";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Localit&grave;";
#elif L==8	// pl
	"Miejsca";
#elif L==9	// pt
	"Localiza&ccedil;oes";
#elif L==10	// tr
	"Places";	// eviri lazim!
#endif

const char *Txt_PLACES_HELP_ORDER[Plc_NUM_ORDERS] =
	{
	[Plc_ORDER_BY_PLACE] =
#if   L==1	// ca
	"Ordenar per lloc"
#elif L==2	// de
	"Nach Standort sortieren"
#elif L==3	// en
	"Sort by place"
#elif L==4	// es
	"Ordenar por lugar"
#elif L==5	// fr
	"Trier par emplacement"
#elif L==6	// gn
	"Ordenar por lugar"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per localit&grave;"
#elif L==8	// pl
	"Sortuj wedlug miejsca"
#elif L==9	// pt
	"Classificar por localiza&ccedil;&atilde;o"
#elif L==10	// tr
	"Sort by place"		// eviri lazim!
#endif
	,
	[Plc_ORDER_BY_NUM_CTRS] =
#if   L==1	// ca
	"Ordenar per nombre de titulacions"
#elif L==2	// de
	"Nach Anzahl der Studieng&auml;nge sortieren"
#elif L==3	// en
	"Sort by no. of degrees"
#elif L==4	// es
	"Ordenar por n&ordm; de titulaciones"
#elif L==5	// fr
	"Trier par nombre d'&eacute;tudes"
#elif L==6	// gn
	"Ordenar por n&ordm; de titulaciones"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di lauree"
#elif L==8	// pl
	"Sortuj wedlug stopni"
#elif L==9	// pt
	"Classificar por n&uacute;mero de graus"
#elif L==10	// tr
	"Sort by no. of degrees"		// eviri lazim!
#endif
	};

const char *Txt_PLACES_ORDER[Plc_NUM_ORDERS] =
	{
	[Plc_ORDER_BY_PLACE] =
#if   L==1	// ca
	"Lloc"
#elif L==2	// de
	"Standort"
#elif L==3	// en
	"Place"
#elif L==4	// es
	"Lugar"
#elif L==5	// fr
	"Emplacement"
#elif L==6	// gn
	"Tenda"
#elif L==7	// it
	"Luogo"
#elif L==8	// pl
	"Miejsce"
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o"
#elif L==10	// tr
	"Place"	// eviri lazim!
#endif
	,
	[Plc_ORDER_BY_NUM_CTRS] =
#if   L==1	// ca
	"Centers"
#elif L==2	// de
	"Lehrinstitute"
#elif L==3	// en
	"Centers"
#elif L==4	// es
	"Centros"
#elif L==5	// fr
	"Centers"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centri"
#elif L==8	// pl
	"Centra"
#elif L==9	// pt
	"Centros"
#elif L==10	// tr
	"Merkezler"
#endif
	};

const char *Txt_Play =	// To play a game match
#if   L==1	// ca
	"Jugar";
#elif L==2	// de
	"Spielen";
#elif L==3	// en
	"Play";
#elif L==4	// es
	"Jugar";
#elif L==5	// fr
	"Jouer";
#elif L==6	// gn
	"Jugar";	// Okoteve traduccin
#elif L==7	// it
	"Giocare";
#elif L==8	// pl
	"Gra&cacute;";
#elif L==9	// pt
	"Jogar";
#elif L==10	// tr
	"Play";		// eviri lazim!
#endif

const char *Txt_Players =	// Match players
#if   L==1	// ca
	"Jugadors";
#elif L==2	// de
	"Spieler";
#elif L==3	// en
	"Players";
#elif L==4	// es
	"Jugadores";
#elif L==5	// fr
	"Joueurs";
#elif L==6	// gn
	"Jugadores";	// Okoteve traduccin
#elif L==7	// it
	"Giocatori";
#elif L==8	// pl
	"Gracze";
#elif L==9	// pt
	"Jogadores";
#elif L==10	// tr
	"Players";	// eviri lazim!
#endif

const char *Txt_Please_check_your_email_address =
#if   L==1	// ca
	"Si us plau, comproveu la seva adre&ccedil;a de correu.";
#elif L==2	// de
	"Bitte &uuml;berpr&uuml;fen Ihre E-Mail-Adresse.";
#elif L==3	// en
	"Please check your email address.";
#elif L==4	// es
	"Por favor, compruebe su direcci&oacute;n de correo.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, v&eacute;rifiez votre adresse email.";
#elif L==6	// gn
	"Por favor, compruebe su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, verifica il tuo indirizzo email.";
#elif L==8	// pl
	"Prosz&eogon; sprawd&zacute; sw&oacute;j adres email.";
#elif L==9	// pt
	"Por favor, verifique seu endere&ccedil;o de email.";
#elif L==10	// tr
	"L&uuml;tfen e-posta adresinizi kontrol edin.";
#endif

const char *Txt_Please_confirm_your_email_address =
#if   L==1	// ca
	"Si us plau, confirmeu la seva adre&ccedil;a de correu.";
#elif L==2	// de
	"Bitte best&auml;tige Ihre E-Mail-Adressen.";
#elif L==3	// en
	"Please confirm your email address.";
#elif L==4	// es
	"Por favor, confirme su direcci&oacute;n de correo.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, confirmez votre adresse email.";
#elif L==6	// gn
	"Por favor, confirme su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, conferma il tuo indirizzo email.";
#elif L==8	// pl
	"Prosz&eogon; potwierdzi&cacute; sw&oacute;j adres email.";
#elif L==9	// pt
	"Por favor, confirme seu endere&ccedil;o de email.";
#elif L==10	// tr
	"Please confirm your email address.";			// eviri lazim!
#endif

const char *Txt_Please_enter_your_ID =
#if   L==1	// ca
	"Si us plau, escriviu el seu ID (DNI/c&egrave;dula).";
#elif L==2	// de
	"Bitte geben Sie Ihre Ausweis-Nr. &auml;nder.";
#elif L==3	// en
	"Please enter your ID.";
#elif L==4	// es
	"Por favor, escriba su ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"S'il vous pla&iacute;t, entrez votre ID.";
#elif L==6	// gn
	"Por favor, escriba su ID (DNI/c&eacute;dula).";		// Okoteve traduccin
#elif L==7	// it
	"Per favore, inserisci il tuo ID.";
#elif L==8	// pl
	"Prosz&eogon;, poda&cacute; swoje zmie&nacute; ID.";
#elif L==9	// pt
	"Por favor, digite n&ordm; de identifica&ccedil;&atilde;o.";
#elif L==10	// tr
	"L&uuml;tfen kimli&gbreve;inizi girin.";
#endif

const char *Txt_Please_fill_in_your_ID =
#if   L==1	// ca
	"Si us plau, ompli el seu ID (DNI/c&egrave;dula)."
	" &Eacute;s una dada opcional,"
	" per&ograve; &eacute;s recomanable que l'ompli"
        " perqu&egrave; facilita el proc&eacute;s d'inscripci&oacute;"
        " dels usuaris en les assignatures.";
#elif L==2	// de
	"Bitte geben Sie Ihre Ausweis-Nr. &auml;nder."
	" Es ist eine optionale Daten,"
	" aber es wird empfohlen, sie zu f&uuml;llen,"
	" weil sie die Registrierung"
	" der Benutzer in den Kursen erleichtert.";
#elif L==3	// en
	"Please fill in your ID."
	" It is an optional data,"
	" but it is recommended that you fill it"
	" because it facilitates the registration process"
	" of users in the courses.";
#elif L==4	// es
	"Por favor, rellene su ID (DNI/c&eacute;dula)."
	" Es un dato opcional,"
	" pero es recomendable que lo rellene"
	" porque facilita el proceso de inscripci&oacute;n"
	" de los usuarios en las asignaturas.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, indiquez votre ID."
	" C'est une donn&eacute;e facultative,"
	" mais il est recommand&eacute; que vous la remplissez"
	" parce qu'il facilite le processus d'enregistrement"
	" des utilisateurs dans les mati&egrave;res.";
#elif L==6	// gn
	"Por favor, rellene su ID (DNI/c&eacute;dula)."
	" Es un dato opcional,"
	" pero es recomendable que lo rellene"
	" porque facilita el proceso de inscripci&oacute;n"
	" de los usuarios en las asignaturas.";		// Okoteve traduccin
#elif L==7	// it
	"Per favore, inserisci il tuo ID."
	" Si tratta di un dato facoltativo,"
	" ma si raccomanda che si riempie"
	" perch&eacute; facilita il processo di registrazione"
	" degli utenti ai corsi.";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje zmie&nacute; ID."
	" Jest to opcja danych, ale zaleca si&eogon;,"
	" aby wype&lstrok;ni&cacute; go, poniewa&zdot; u&lstrok;atwia proces rejestracji"
	" u&zdot;ytkownik&oacute;w w kursach.";
#elif L==9	// pt
	"Por favor, preencha seu n&ordm; de identifica&ccedil;&atilde;o"
	" &Eacute; uma dada opcional,"
	" mas &eacute; recomend&aacute;vel que voc&ecirc; preench&ecirc;-lo,"
	" pois facilita o processo de registro"
	" de usu&aacute;rios nas disciplinas.";
#elif L==10	// tr
	"Please fill in your ID."
	" It is an optional data,"
	" but it is recommended that you fill it"
	" because it facilitates the registration process"
	" of users in the courses.";			// eviri lazim!
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_country_nationality =
#if   L==1	// ca
	"Si us plau, ompli la seva fitxa,"
	" incloent el seu pais (nacionalitat).";
#elif L==2	// de
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlie&szlig;lich Ihrem Land (Nationalit&auml;t).";
#elif L==3	// en
	"Please fill in your country (nationality).";
#elif L==4	// es
	"Por favor, rellene su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";
#elif L==5	// fr
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre pays (nationalit&eacute;).";
#elif L==6	// gn
	"Por favor, rellene su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, compila la tua scheda,"
	" compreso il tuo paese (nazionalit&agrave;).";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym kraju (obywatelstwo";
#elif L==9	// pt
	"Por favor, preencha o seu cart&atilde;o,"
	" incluindo seu pa&iacute;s (nacionalidade).";
#elif L==10	// tr
	"Please fill in your country (nationality).";	// eviri lazim!
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_name =
#if   L==1	// ca
	"Si us plau, ompli la seva fitxa,"
	" incloent el seu nom i cognom(s).";
#elif L==2	// de
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" mit Ihrem Namen.";
#elif L==3	// en
	"Please fill in your record card,"
	" including your name.";
#elif L==4	// es
	"Por favor, rellene su ficha,"
	" incluyendo su nombre y apellido(s).";
#elif L==5	// fr
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre nom.";
#elif L==6	// gn
	"Por favor, rellene su ficha,"
	" incluyendo su nombre y apellido(s).";		// Okoteve traduccin
#elif L==7	// it
	"Per favore, compila la tua scheda,"
	" compresi il tuo nome e cognome.";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym imie i nazwisko.";
#elif L==9	// pt
	"Por favor, preencha o cart&atilde;o,"
	" incluindo seu nome.";
#elif L==10	// tr
	"Please fill in your record card,"
	" including your name.";			// eviri lazim!
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_sex =
#if   L==1	// ca
	"Si us plau, ompli la seva fitxa,"
	" incloent el sexe.";
#elif L==2	// de
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlie&szlig;lich Ihrer Sex.";
#elif L==3	// en
	"Please fill in your record card,"
	" including your sex.";
#elif L==4	// es
	"Por favor, rellene su ficha,"
	" incluyendo el sexo.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre sexe.";
#elif L==6	// gn
	"Por favor, rellene su ficha,"
	" incluyendo el sexo.";		// Okoteve traduccin
#elif L==7	// it
	"Per favore, compila la tua scheda,"
	" compreso il sesso.";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym ple&cacute;.";
#elif L==9	// pt
	"Por favor, preencha o cart&atilde;o,"
	" incluindo o seu sexo.";
#elif L==10	// tr
	"Please fill in your record card,"
	" including your sex.";		// eviri lazim!
#endif

const char *Txt_Please_check_your_privacy_settings =
#if   L==1	// ca
	"Si us plau, comproveu la seva configuraci&oacute; de privacitat.";
#elif L==2	// de
	"Bitte &uuml;berpr&uuml;fen Sie Ihre Datenschutzeinstellungen.";
#elif L==3	// en
	"Please review your privacy settings.";
#elif L==4	// es
	"Por favor, revise sus ajustes de privacidad.";
#elif L==5	// fr
	"S'il vous pla&icirc;t, v&eacute;rifiez vos param&egrave;tres de privacit&eacute;.";
#elif L==6	// gn
	"Por favor, revise sus ajustes de privacidad.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, controlla le tue impostazioni sulla privacy.";
#elif L==8	// pl
	"Sprawd&zacute; ustawienia prywatno&sacute;ci.";
#elif L==9	// pt
	"Por favor, verifique suas configura&ccedil;&otilde;es de privacidade.";
#elif L==10	// tr
	"Please review your privacy settings.";		// eviri lazim!
#endif

const char *Txt_Please_review_your_answers_before_submitting_the_exam =
#if   L==1	// ca
	"Si us plau, revisi les seves respostes abans d'enviar l'examen.";
#elif L==2	// de
	"Bitte &uuml;berpr&uuml;fen Sie Ihre Antworten, bevor Sie die Pr&uuml;fung einreichen.";
#elif L==3	// en
	"Please review your answers before submitting the exam.";
#elif L==4	// es
	"Por favor, revise sus respuestas antes de enviar el examen.";
#elif L==5	// fr
	"S'il vous pla&icirc;t, veuillez revoir vos r&eacute;ponses avant de soumettre l'examen.";
#elif L==6	// gn
	"Por favor, revise sus respuestas antes de enviar el examen.";	// Okoteve traduccin
#elif L==7	// it
	"Si prega di rivedere le risposte prima di inviare l'esame.";
#elif L==8	// pl
	"Przejrzyj swoje odpowiedzi przed przes&lstrok;aniem egzaminu.";
#elif L==9	// pt
	"Por favor, revise suas respostas antes de enviar o exame.";
#elif L==10	// tr
	"Please review your answers before submitting the exam.";	// eviri lazim!
#endif

const char *Txt_Please_select_the_country_of_your_institution =
#if   L==1	// ca
	"Si us plau, seleccioneu el pa&iacute;s de la seva instituci&oacute;.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie das Land Ihrer Hochschule aus";
#elif L==3	// en
	"Please, select the country of your institution.";
#elif L==4	// es
	"Por favor, seleccione el pa&iacute;s de su instituci&oacute;n.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez le pays de votre &eacute;tablissement.";
#elif L==6	// gn
	"Por favor, seleccione el pa&iacute;s de su instituci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, selezionare il paese di la tua istituzione accademica.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; kraj swojej instytucji.";
#elif L==9	// pt
	"Por favor, selecione o pa&iacute;s da sua institui&ccedil;&atilde;o.";
#elif L==10	// tr
	"Please, select the country of your institution.";			// eviri lazim!
#endif

const char *Txt_Please_select_your_center =
#if   L==1	// ca
	"Si us plau, seleccioneu el seu center.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie Ihre Zentrum aus.";
#elif L==3	// en
	"Please, select your center.";
#elif L==4	// es
	"Por favor, seleccione su centro.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez votre center.";
#elif L==6	// gn
	"Por favor, seleccione su centro.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, seleziona il tuo centro.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; swoje tym centrum.";
#elif L==9	// pt
	"Por favor, selecione seu centro.";
#elif L==10	// tr
	"Please, select your center.";		// eviri lazim!
#endif

const char *Txt_Please_select_your_department =
#if   L==1	// ca
	"Si us plau, seleccioneu el seu department.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie Ihre Abteilung aus.";
#elif L==3	// en
	"Please, select your department.";
#elif L==4	// es
	"Por favor, seleccione su departamento.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez votre d&eacute;partement.";
#elif L==6	// gn
	"Por favor, seleccione su departamento.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, seleziona il tuo dipartimento.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; swoje tym dzia&lstrok;u.";
#elif L==9	// pt
	"Por favor, selecione seu departamento.";
#elif L==10	// tr
	"Please, select your department.";		// eviri lazim!
#endif

const char *Txt_Please_select_your_institution =
#if   L==1	// ca
	"Si us plau, seleccioneu la seva instituci&oacute;.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie Ihre Hochschule aus.";
#elif L==3	// en
	"Please, select your institution.";
#elif L==4	// es
	"Por favor, seleccione su instituci&oacute;n.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez votre &eacute;tablissement.";
#elif L==6	// gn
	"Por favor, seleccione su instituci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, seleziona la tua istituzione accademica.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; swoj&aogon; instytucj&eogon;.";
#elif L==9	// pt
	"Por favor, selecione sua institui&ccedil;&atilde;o.";
#elif L==10	// tr
	"Please, select your institution.";		// eviri lazim!
#endif

const char *Txt_Please_specify_if_you_allow_downloading_the_question_bank_from_other_applications =
#if   L==1	// ca
	"Si us plau, especifiqui si permet la desc&agrave;rrega de el banc de preguntes des d'altres aplicacions.";
#elif L==2	// de
	"Bitte geben Sie an, ob Sie das Herunterladen der Fragenbank aus anderen Anwendungen zulassen.";
#elif L==3	// en
	"Please specify if you allow downloading the question bank from other applications.";
#elif L==4	// es
	"Por favor, especifique si permite la descarga del banco de preguntas desde otras aplicaciones.";
#elif L==5	// fr
	"Veuillez indiquer si vous autorisez le t&eacute;l&eacute;chargement de la banque de questions &agrave; partir d'autres applications.";
#elif L==6	// gn
	"Por favor, especifique si permite la descarga del banco de preguntas desde otras aplicaciones.";	// Okoteve traduccin
#elif L==7	// it
	"Si prega di specificare se si consente il download della banca delle domande da altre applicazioni.";
#elif L==8	// pl
	"Okre&sacute;l, czy zezwalasz na pobieranie banku pyta&nacute; z innych aplikacji.";
#elif L==9	// pt
	"Por favor, especifique se voc&ecirc; deseja baixar o banco de perguntas de outros aplicativos.";
#elif L==10	// tr
	"Please specify if you allow downloading the question bank from other applications.";			// eviri lazim!
#endif

const char *Txt_Please_wait_ =
#if   L==1	// ca
	"Espereu, si us plau&hellip;";
#elif L==2	// de
	"Warten Sie bitte&hellip;";
#elif L==3	// en
	"Please wait&hellip;";
#elif L==4	// es
	"Espere, por favor&hellip;";
#elif L==5	// fr
	"Attendez s'il vous pla&icirc;t&hellip;";
#elif L==6	// gn
	"Espere, por favor&hellip;";	// Okoteve traduccin
#elif L==7	// it
	"Aspetta, per favore&hellip;";
#elif L==8	// pl
	"Prosz&eogon; czeka&cacute;&hellip;";
#elif L==9	// pt
	"Espere, por favor&hellip;";
#elif L==10	// tr
	"Please wait&hellip;";		// eviri lazim!
#endif

const char *Txt_Plugin =
#if   L==1	// ca
	"Complement";
#elif L==2	// de
	"Plugin";
#elif L==3	// en
	"Plugin";
#elif L==4	// es
	"Complemento";
#elif L==5	// fr
	"Plugin";
#elif L==6	// gn
	"Complemento";	// Okoteve traduccin
#elif L==7	// it
	"Plugin";
#elif L==8	// pl
	"Wtyczka";
#elif L==9	// pt
	"Plugin";
#elif L==10	// tr
	"Plugin";	// eviri lazim!
#endif

const char *Txt_Plugin_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Complement <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Plugin <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Plugin <strong>%s</strong> removed.";
#elif L==4	// es
	"Complemento <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Plugin <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Complemento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Plugin <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Wtyczka <strong>%s</strong> usunieta.";
#elif L==9	// pt
	"Plugin <strong>%s</strong> removido.";
#elif L==10	// tr
	"Plugin <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Plugins =
#if   L==1	// ca
	"Complements";
#elif L==2	// de
	"Plugins";
#elif L==3	// en
	"Plugins";
#elif L==4	// es
	"Complementos";
#elif L==5	// fr
	"Plugins";
#elif L==6	// gn
	"Complementos";
#elif L==7	// it
	"Plugin";
#elif L==8	// pl
	"wtyczki";
#elif L==9	// pt
	"Plugins";
#elif L==10	// tr
	"Eklentiler";
#endif

const char *Txt_Possibly_duplicate_users =
#if   L==1	// ca
	"Usuaris possiblement duplicats";
#elif L==2	// de
	"M&ouml;gliche doppelte Benutzer";
#elif L==3	// en
	"Possibly duplicate users";
#elif L==4	// es
	"Usuarios posiblemente duplicados";
#elif L==5	// fr
	"Possibles utilisateurs en double";
#elif L==6	// gn
	"Usuarios posiblemente duplicados";	// Okoteve traduccin
#elif L==7	// it
	"Possibili utenti duplicati";
#elif L==8	// pl
	"Mo&zdot;liwe duplikaty u&zdot;ytkownik&oacute;w";
#elif L==9	// pt
	"Usu&aacute;rios possivelmente duplicados";
#elif L==10	// tr
	"Possibly duplicate users";		// eviri lazim!
#endif


const char *Txt_Post =	// Publish (verb)
#if   L==1	// ca
	"Publicar";
#elif L==2	// de
	"Ver&ouml;ffentlichen";
#elif L==3	// en
	"Post";
#elif L==4	// es
	"Publicar";
#elif L==5	// fr
	"Publier";
#elif L==6	// gn
	"Publicar";	// Okoteve traduccin
#elif L==7	// it
	"Pubblicare";
#elif L==8	// pl
	"Publikowa&cacute;";
#elif L==9	// pt
	"Publicar";
#elif L==10	// tr
	"Post";		// eviri lazim!
#endif

const char *Txt_Present =
#if   L==1	// ca
	"Present";
#elif L==2	// de
	"Vorhanden";
#elif L==3	// en
	"Present";
#elif L==4	// es
	"Presente";
#elif L==5	// fr
	"Pr&eacute;sent";
#elif L==6	// gn
	"Presente";	// Okoteve traduccin
#elif L==7	// it
	"Presente";
#elif L==8	// pl
	"Obecny";
#elif L==9	// pt
	"Presente";
#elif L==10	// tr
	"Present";	// eviri lazim!
#endif

const char *Txt_Presents =
#if   L==1	// ca
	"Presents";
#elif L==2	// de
	"Vorhanden";
#elif L==3	// en
	"Presents";
#elif L==4	// es
	"Presentes";
#elif L==5	// fr
	"Pr&eacute;sents";
#elif L==6	// gn
	"Presentes";	// Okoteve traduccin
#elif L==7	// it
	"Presenti";
#elif L==8	// pl
	"Obecny";
#elif L==9	// pt
	"Presentes";
#elif L==10	// tr
	"Presents";	// eviri lazim!
#endif

const char *Txt_Print =
#if   L==1	// ca
	"Imprimir";
#elif L==2	// de
	"Ausdrucken";
#elif L==3	// en
	"Print";
#elif L==4	// es
	"Imprimir";
#elif L==5	// fr
	"Imprimer";
#elif L==6	// gn
	"Imprimir";	// Okoteve traduccin
#elif L==7	// it
	"Stampare";
#elif L==8	// pl
	"Wydrukuj";
#elif L==9	// pt
	"Imprimir";
#elif L==10	// tr
	"Print";	// eviri lazim!
#endif

const char *Txt_Privacy =
#if   L==1	// ca
	"Privacitat";
#elif L==2	// de
	"Geheimhaltung";
#elif L==3	// en
	"Privacy";
#elif L==4	// es
	"Privacidad";
#elif L==5	// fr
	"Privacit&eacute;";
#elif L==6	// gn
	"Privacidad";	// Okoteve traduccin
#elif L==7	// it
	"Privatezza";
#elif L==8	// pl
	"Prywatno&sacute;&cacute;";
#elif L==9	// pt
	"Privacidade";
#elif L==10	// tr
	"Mahremiyet";
#endif

const char *Txt_PRIVACY_OPTIONS[Pri_NUM_OPTIONS_PRIVACY] =
	{
	 [Pri_VISIBILITY_UNKNOWN] =
#if   L==1	// ca
	"Desconeguda"
#elif L==2	// de
	"Unbekannt"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocida"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocida"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuta"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecida"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	 [Pri_VISIBILITY_USER] =
#if   L==1	// ca
	"Visible per mi i els meus professors / els meus estudiants"
#elif L==2	// de
	"Sichtbar f&uuml;r mich und meine Lehrer / meine Sch&uuml;ler"
#elif L==3	// en
	"Visible by me and my teachers / my students"
#elif L==4	// es
	"Visible por m&iacute; y mis profesores / mis estudiantes"
#elif L==5	// fr
	"Visible par moi et mes professeurs / mes &eacute;tudiants"
#elif L==6	// gn
	"Visible por m&iacute; y mis profesores / mis estudiantes"	// Okoteve traduccin
#elif L==7	// it
	"Visibile da me e dai miei insegnanti / miei studenti"
#elif L==8	// pl
	"Widoczne przeze mnie i moich nauczycieli / moich uczni&oacute;w"
#elif L==9	// pt
	"Vis&iacute;vel por mim e meus professores / meus estudantes"
#elif L==10	// tr
	"Visible by me and my teachers / my students"			// eviri lazim!
#endif
	,
	 [Pri_VISIBILITY_COURSE] =
#if   L==1	// ca
	"Visible per usuaris amb qui comparteixo assignatures"
#elif L==2	// de
	"Sichtbar f&uuml;r Benutzer, mit denen ich Kurse teile"
#elif L==3	// en
	"Visible by users with whom I share courses"
#elif L==4	// es
	"Visible por usuarios con los que comparto asignaturas"
#elif L==5	// fr
	"Visible par les utilisateurs avec qui je partage des mati&egrave;res"
#elif L==6	// gn
	"Visible por usuarios con los que comparto asignaturas"		// Okoteve traduccin
#elif L==7	// it
	"Visibile dagli utenti con cui condivido i corsi"
#elif L==8	// pl
	"Widoczne dla u&zdot;ytkownik&oacute;w, z kt&oacute;rymi dziel&eogon; si&eogon; kursami"
#elif L==9	// pt
	"Vis&iacute;vel por usu&aacute;rios com quem compartilho disciplinas"
#elif L==10	// tr
	"Visible by users with whom I share courses"			// eviri lazim!
#endif
	,
	 [Pri_VISIBILITY_SYSTEM] =
#if   L==1	// ca
	"Visible per qualsevol usuari identificat"
#elif L==2	// de
	"Sichtbar f&uuml;r jeden eingeloggten Benutzer"
#elif L==3	// en
	"Visible by any logged user"
#elif L==4	// es
	"Visible por cualquier usuario identificado"
#elif L==5	// fr
	"Visible par tout utilisateur connect&eacute;"
#elif L==6	// gn
	"Visible por cualquier usuario identificado"	// Okoteve traduccin
#elif L==7	// it
	"Visibile da qualsiasi utente registrato"
#elif L==8	// pl
	"Widoczne dla ka&zdot;dego zalogowanego u&zdot;ytkownika"
#elif L==9	// pt
	"Vis&iacute;vel por qualquer usu&aacute;rio logado"
#elif L==10	// tr
	"Visible by any logged user"			// eviri lazim!
#endif
	,
	 [Pri_VISIBILITY_WORLD] =
#if   L==1	// ca
	"Visible per tot el m&oacute;n"
#elif L==2	// de
	"F&uuml;r alle sichtbar"
#elif L==3	// en
	"Visible by everyone"
#elif L==4	// es
	"Visible por todo el mundo"
#elif L==5	// fr
	"Visible par tous"
#elif L==6	// gn
	"Visible por todo el mundo"	// Okoteve traduccin
#elif L==7	// it
	"Visibile da tutti"
#elif L==8	// pl
	"Widoczne dla wszystkich"
#elif L==9	// pt
	"Vis&iacute;vel em todo o mundo"
#elif L==10	// tr
	"Visible by everyone"		// eviri lazim!
#endif
	};

const char *Txt_Private_available_to_certain_users_identified =
#if   L==1	// ca
	"Privat, disponible per a certs usuaris identificats";
#elif L==2	// de
	"Private, verf&uuml;gbar auf bestimmte Benutzer identifiziert";
#elif L==3	// en
	"Private, available to certain users identified";
#elif L==4	// es
	"Privado, disponible para ciertos usuarios identificados";
#elif L==5	// fr
	"Priv&eacute;, &agrave; la disposition de certains utilisateurs identifi&eacute;s";
#elif L==6	// gn
	"Privado, disponible para ciertos usuarios identificados";	// Okoteve traduccin
#elif L==7	// it
	"Privato, a disposizione degli utenti individuati alcuni";
#elif L==8	// pl
	"Prywatne, dost&eogon;pne dla okre&sacute;lonych u&zdot;ytkownik&oacute;w zidentyfikowanych";
#elif L==9	// pt
	"Privado, dispon&iacute;vel para alguns usu&aacute;rios identificados";
#elif L==10	// tr
	"Private, available to certain users identified";		// eviri lazim!
#endif

const char *Txt_Project =
#if   L==1	// ca
	"Projecte";
#elif L==2	// de
	"Projekt";
#elif L==3	// en
	"Project";
#elif L==4	// es
	"Proyecto";
#elif L==5	// fr
	"Projet";
#elif L==6	// gn
	"Apopyr&atilde;";
#elif L==7	// it
	"Progetto";
#elif L==8	// pl
	"Projekt";
#elif L==9	// pt
	"Projeto";
#elif L==10	// tr
	"Proje";
#endif

const char *Txt_project =
#if   L==1	// ca
	"projecte";
#elif L==2	// de
	"Projekt";
#elif L==3	// en
	"project";
#elif L==4	// es
	"proyecto";
#elif L==5	// fr
	"project";
#elif L==6	// gn
	"apopyr&atilde;";
#elif L==7	// it
	"progetto";
#elif L==8	// pl
	"projekt";
#elif L==9	// pt
	"projeto";
#elif L==10	// tr
	"proje";
#endif

const char *Txt_Project_assessment =
#if   L==1	// ca
	"Avaluaci&oacute; del projecte";
#elif L==2	// de
	"Projektbewertung";
#elif L==3	// en
	"Project assessment";
#elif L==4	// es
	"Evaluaci&oacute;n del proyecto";
#elif L==5	// fr
	"&Eacute;valuation du project";
#elif L==6	// gn
	"Apopyr&atilde; jehechauka";
#elif L==7	// it
	"Valutazione di progetto";
#elif L==8	// pl
	"Ocena projektowe";
#elif L==9	// pt
	"Avalia&ccedil;&atilde;o do projeto";
#elif L==10	// tr
	"Projenin de&gbreve;erlendirilmesi";
#endif

const char *Txt_project_assessment =
#if   L==1	// ca
	"avaluaci&oacute; del projecte";
#elif L==2	// de
	"Projektbewertung";
#elif L==3	// en
	"project assessment";
#elif L==4	// es
	"evaluaci&oacute;n del proyecto";
#elif L==5	// fr
	"&eacute;valuation du project";
#elif L==6	// gn
	"evaluaci&oacute;n del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"valutazione di progetto";
#elif L==8	// pl
	"ocena projektowe";
#elif L==9	// pt
	"avalia&ccedil;&atilde;o do projeto";
#elif L==10	// tr
	"project assessment";			// eviri lazim!
#endif

const char *Txt_Project_documents =
#if   L==1	// ca
	"Documents del projecte";
#elif L==2	// de
	"Projektdokumente";
#elif L==3	// en
	"Project documents";
#elif L==4	// es
	"Documentos del proyecto";
#elif L==5	// fr
	"Documents du project";
#elif L==6	// gn
	"Documentos del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Documenti di progetto";
#elif L==8	// pl
	"Dokumenty projektowe";
#elif L==9	// pt
	"Documentos do projeto";
#elif L==10	// tr
	"Project documents";		// eviri lazim!
#endif

const char *Txt_project_documents =
#if   L==1	// ca
	"documents del projecte";
#elif L==2	// de
	"Projektdokumente";
#elif L==3	// en
	"project documents";
#elif L==4	// es
	"documentos del proyecto";
#elif L==5	// fr
	"documents du project";
#elif L==6	// gn
	"documentos del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"documenti di progetto";
#elif L==8	// pl
	"dokumenty projektowe";
#elif L==9	// pt
	"documentos do projeto";
#elif L==10	// tr
	"project documents";		// eviri lazim!
#endif

const char *Txt_PROJECT_ORDER[Prj_NUM_ORDERS] =
	{
	[Prj_ORDER_START_TIME] =
#if   L==1	// ca
	"Data de creaci&oacute;"
#elif L==2	// de
	"Datum der Erstellung"
#elif L==3	// en
	"Date of creation"
#elif L==4	// es
	"Fecha de creaci&oacute;n"
#elif L==5	// fr
	"Date de cr&eacute;ation"
#elif L==6	// gn
	"Fecha de creaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Data di creazione"
#elif L==8	// pl
	"Data utworzenia"
#elif L==9	// pt
	"Data da cria&ccedil;&atilde;o"
#elif L==10	// tr
	"Date of creation"		// eviri lazim!
#endif
	,
	[Prj_ORDER_END_TIME] =
#if   L==1	// ca
	"&Uacute;ltima edici&oacute;"
#elif L==2	// de
	"Letzte Ausgabe"
#elif L==3	// en
	"Last edition"
#elif L==4	// es
	"&Uacute;ltima edici&oacute;n"
#elif L==5	// fr
	"Derni&egrave;re &eacute;dition"
#elif L==6	// gn
	"&Uacute;ltima edici&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ultima edizione"
#elif L==8	// pl
	"Ostatnie wydanie"
#elif L==9	// pt
	"&Uacute;ltima edi&ccedil;&atilde;o"
#elif L==10	// tr
	"Last edition"			// eviri lazim!
#endif
	,
	[Prj_ORDER_TITLE] =
#if   L==1	// ca
	"T&iacute;tol"
#elif L==2	// de
	"Titel"
#elif L==3	// en
	"Title"
#elif L==4	// es
	"T&iacute;tulo"
#elif L==5	// fr
	"Titre"
#elif L==6	// gn
	"Moak&atilde;ha"
#elif L==7	// it
	"Titolo"
#elif L==8	// pl
	"Tytu&lstrok;"
#elif L==9	// pt
	"T&iacute;tulo"
#elif L==10	// tr
	"Title"	// eviri lazim!
#endif
	,
	[Prj_ORDER_DEPARTMENT] =
#if   L==1	// ca
	"Departament"
#elif L==2	// de
	"Abteilung"
#elif L==3	// en
	"Department"
#elif L==4	// es
	"Departamento"
#elif L==5	// fr
	"D&eacute;partement"
#elif L==6	// gn
	"Peh&#7869;"
#elif L==7	// it
	"Dipartimento"
#elif L==8	// pl
	"Wydzia&lstrok;"
#elif L==9	// pt
	"Departamento"
#elif L==10	// tr
	"Department"	// eviri lazim!
#endif
	};

const char *Txt_PROJECT_ORDER_HELP[Prj_NUM_ORDERS] =
	{
	[Prj_ORDER_START_TIME] =
#if   L==1	// ca
	"Ordenar per data de creaci&oacute;"
#elif L==2	// de
	"Sortieren nach Erstellungsdatum"
#elif L==3	// en
	"Sort by date of creation"
#elif L==4	// es
	"Ordenar por fecha de creaci&oacute;n"
#elif L==5	// fr
	"Trier par date de cr&eacute;ation"
#elif L==6	// gn
	"Ordenar por fecha de creaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data di creazione"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug daty utworzenia"
#elif L==9	// pt
	"Classificar por data de cria&ccedil;&atilde;o"
#elif L==10	// tr
	"Sort by date of creation"		// eviri lazim!
#endif
	,
	[Prj_ORDER_END_TIME] =
#if   L==1	// ca
	"Ordenar per data de l'&uacute;ltima edici&oacute;"
#elif L==2	// de
	"Sortieren nach letzten Ausgabe"
#elif L==3	// en
	"Sort by date of last edition"
#elif L==4	// es
	"Ordenar por fecha de &uacute;ltima edici&oacute;n"
#elif L==5	// fr
	"Trier par date de derni&egrave;re &eacute;dition"
#elif L==6	// gn
	"Ordenar por fecha de &uacute;ltima edici&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per data dell'ultima edizione"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug daty ostatniej edycji"
#elif L==9	// pt
	"Classificar por data da &uacute;ltima edi&ccedil;&atilde;o"
#elif L==10	// tr
	"Sort by date of last edition"				// eviri lazim!
#endif
	,
	[Prj_ORDER_TITLE] =
#if   L==1	// ca
	"Ordenar per t&iacute;tol"
#elif L==2	// de
	"Sortieren nach Titel"
#elif L==3	// en
	"Sort by title"
#elif L==4	// es
	"Ordenar por t&iacute;tulo"
#elif L==5	// fr
	"Trier par titre"
#elif L==6	// gn
	"Ordenar por t&iacute;tulo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per titolo"
#elif L==8	// pl
	"Sortuj wedlug tytu&lstrok;"
#elif L==9	// pt
	"Classificar por t&iacute;tulo"
#elif L==10	// tr
	"Sort by title"			// eviri lazim!
#endif
	,
	[Prj_ORDER_DEPARTMENT] =
#if   L==1	// ca
	"Ordenar per departament"
#elif L==2	// de
	"Sortieren nach Abteilung"
#elif L==3	// en
	"Sort by department"
#elif L==4	// es
	"Ordenar por departamento"
#elif L==5	// fr
	"Trier par d&eacute;partement"
#elif L==6	// gn
	"Ordenar por departamento"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per dipartimento"
#elif L==8	// pl
	"Sortuj wedlug wydzia&lstrok;"
#elif L==9	// pt
	"Classificar por departamento"
#elif L==10	// tr
	"Sort by department"		// eviri lazim!
#endif
	};

const char *Txt_Project_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Projecte <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Projekt <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Project <strong>%s</strong> removed.";
#elif L==4	// es
	"Proyecto <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Projet <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Proyecto <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Progetto <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Projekt <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Projeto <strong>%s</strong> removido.";
#elif L==10	// tr
	"<strong>%s</strong> projesi kald&inodot;r&inodot;ld&inodot;.";
#endif

const char *Txt_PROJECT_ROLES_SINGUL_Abc[Prj_NUM_ROLES_IN_PROJECT] =
	{
	[Prj_ROLE_UNK] =
#if   L==1	// ca
	"Desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido/a"
#elif L==5	// fr
	"Inconnu/e"
#elif L==6	// gn
	"Desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto/a"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido/a"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Prj_ROLE_STD] =
#if   L==1	// ca
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant/e"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Student"
#elif L==9	// pt
	"Estudante"
#elif L==10	// tr
	"Student"	// eviri lazim!
#endif
	,
	[Prj_ROLE_TUT] =
#if   L==1	// ca
	"Tutor/a"
#elif L==2	// de
	"Tutor"
#elif L==3	// en
	"Tutor"
#elif L==4	// es
	"Tutor/a"
#elif L==5	// fr
	"Tuteur"
#elif L==6	// gn
	"Tutor/a"	// Okoteve traduccin
#elif L==7	// it
	"Precettore"
#elif L==8	// pl
	"Nauczyciel"
#elif L==9	// pt
	"Tutor/a"
#elif L==10	// tr
	"Tutor"		// eviri lazim!
#endif
	,
	[Prj_ROLE_EVL] =
#if   L==1	// ca
	"Avaluador/a"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"Evaluator"
#elif L==4	// es
	"Evaluador/a"
#elif L==5	// fr
	"&Eacute;valuateur/trice"
#elif L==6	// gn
	"Evaluador/a"	// Okoteve traduccin
#elif L==7	// it
	"Valutatore"
#elif L==8	// pl
	"Ewaluator"
#elif L==9	// pt
	"Avaliador/a"
#elif L==10	// tr
	"Evaluator"	// eviri lazim!
#endif
	};

const char *Txt_PROJECT_ROLES_SINGUL_abc[Prj_NUM_ROLES_IN_PROJECT][Usr_NUM_SEXS] =
	{
	[Prj_ROLE_UNK][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido/a"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto/a"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido/a"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Prj_ROLE_UNK][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"desconeguda"
#elif L==2	// de
	"Unbekanntere"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocida"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocida"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuta"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecida"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Prj_ROLE_UNK][Usr_SEX_MALE] =
#if   L==1	// ca
	"desconegut"
#elif L==2	// de
	"Unbekannter"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido"
#elif L==5	// fr
	"inconnu"
#elif L==6	// gn
	"desconocido"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Prj_ROLE_UNK][Usr_SEX_ALL] =
#if   L==1	// ca
	"desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido/a"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto/a"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido/a"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Prj_ROLE_STD][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant/e"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Prj_ROLE_STD][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiante"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Prj_ROLE_STD][Usr_SEX_MALE] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Prj_ROLE_STD][Usr_SEX_ALL] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant/e"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Prj_ROLE_TUT][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"tutor/a"
#elif L==2	// de
	"Tutor"
#elif L==3	// en
	"tutor"
#elif L==4	// es
	"tutor/a"
#elif L==5	// fr
	"tuteur/trice"
#elif L==6	// gn
	"tutor/a"	// Okoteve traduccin
#elif L==7	// it
	"precettore"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"tutor/a"
#elif L==10	// tr
	"tutor"		// eviri lazim!
#endif
	,
	[Prj_ROLE_TUT][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"tutora"
#elif L==2	// de
	"Tutor"
#elif L==3	// en
	"tutor"
#elif L==4	// es
	"tutora"
#elif L==5	// fr
	"tutrice"
#elif L==6	// gn
	"tutora"	// Okoteve traduccin
#elif L==7	// it
	"precettore"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"tutora"
#elif L==10	// tr
	"tutor"		// eviri lazim!
#endif
	,
	[Prj_ROLE_TUT][Usr_SEX_MALE] =
#if   L==1	// ca
	"tutor"
#elif L==2	// de
	"Tutor"
#elif L==3	// en
	"tutor"
#elif L==4	// es
	"tutor"
#elif L==5	// fr
	"tuteur"
#elif L==6	// gn
	"tutor"	// Okoteve traduccin
#elif L==7	// it
	"precettore"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"tutor"
#elif L==10	// tr
	"tutor"	// eviri lazim!
#endif
	,
	[Prj_ROLE_TUT][Usr_SEX_ALL] =
#if   L==1	// ca
	"tutor/a"
#elif L==2	// de
	"Tutor"
#elif L==3	// en
	"tutor"
#elif L==4	// es
	"tutor/a"
#elif L==5	// fr
	"tuteur/trice"
#elif L==6	// gn
	"tutor/a"	// Okoteve traduccin
#elif L==7	// it
	"precettore"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"tutor/a"
#elif L==10	// tr
	"tutor"		// eviri lazim!
#endif
	,
	[Prj_ROLE_EVL][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"avaluador/a"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"evaluator"
#elif L==4	// es
	"evaluador/a"
#elif L==5	// fr
	"&eacute;valuateur/trice"
#elif L==6	// gn
	"evaluador/a"	// Okoteve traduccin
#elif L==7	// it
	"valutatore"
#elif L==8	// pl
	"ewaluator"
#elif L==9	// pt
	"avaliador/a"
#elif L==10	// tr
	"evaluator"	// eviri lazim!
#endif
	,
	[Prj_ROLE_EVL][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"avaluadora"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"evaluator"
#elif L==4	// es
	"evaluadora"
#elif L==5	// fr
	"&eacute;valuatrice"
#elif L==6	// gn
	"evaluadora"	// Okoteve traduccin
#elif L==7	// it
	"valutatore"
#elif L==8	// pl
	"ewaluator"
#elif L==9	// pt
	"avaliadora"
#elif L==10	// tr
	"evaluator"	// eviri lazim!
#endif
	,
	[Prj_ROLE_EVL][Usr_SEX_MALE] =
#if   L==1	// ca
	"avaluador"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"evaluator"
#elif L==4	// es
	"evaluador"
#elif L==5	// fr
	"&eacute;valuateur"
#elif L==6	// gn
	"evaluador"	// Okoteve traduccin
#elif L==7	// it
	"valutatore"
#elif L==8	// pl
	"ewaluator"
#elif L==9	// pt
	"avaliador"
#elif L==10	// tr
	"evaluator"	// eviri lazim!
#endif
	,
	[Prj_ROLE_EVL][Usr_SEX_ALL] =
#if   L==1	// ca
	"avaluador/a"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"evaluator"
#elif L==4	// es
	"evaluador/a"
#elif L==5	// fr
	"&eacute;valuateur/trice"
#elif L==6	// gn
	"evaluador/a"	// Okoteve traduccin
#elif L==7	// it
	"valutatore"
#elif L==8	// pl
	"ewaluator"
#elif L==9	// pt
	"avaliador/a"
#elif L==10	// tr
	"evaluator"	// eviri lazim!
#endif
	};

const char *Txt_PROJECT_ROLES_PLURAL_Abc[Prj_NUM_ROLES_IN_PROJECT] =
	{
	[Prj_ROLE_UNK] =
#if   L==1	// ca
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecidos"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Prj_ROLE_STD] =
#if   L==1	// ca
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'eku&eacute;ra"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#elif L==10	// tr
	"&Ouml;&gbreve;renciler"
#endif
	,
	[Prj_ROLE_TUT] =
#if   L==1	// ca
	"Tutors"
#elif L==2	// de
	"Tutoren"
#elif L==3	// en
	"Tutors"
#elif L==4	// es
	"Tutores"
#elif L==5	// fr
	"Tuteurs"
#elif L==6	// gn
	"Tutor-ku&eacute;ra"
#elif L==7	// it
	"Precettori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Tutores"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	,
	[Prj_ROLE_EVL] =
#if   L==1	// ca
	"Avaluadors"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"Evaluators"
#elif L==4	// es
	"Evaluadores"
#elif L==5	// fr
	"&Eacute;valuateurs"
#elif L==6	// gn
	"Evaluador-ku&eacute;ra"
#elif L==7	// it
	"Valutatori"
#elif L==8	// pl
	"Ewaluator&oacute;w"
#elif L==9	// pt
	"Avaliadores"
#elif L==10	// tr
	"De&gbreve;erlendiriciler"
#endif
	};

const char *Txt_PROJECT_ROLES_PLURAL_abc[Prj_NUM_ROLES_IN_PROJECT] =
	{
	[Prj_ROLE_UNK] =
#if   L==1	// ca
	"desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Prj_ROLE_STD] =
#if   L==1	// ca
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'eku&eacute;ra"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studenci"
#elif L==9	// pt
	"estudantes"
#elif L==10	// tr
	"&ouml;&gbreve;renciler"
#endif
	,
	[Prj_ROLE_TUT] =
#if   L==1	// ca
	"tutors"
#elif L==2	// de
	"Tutoren"
#elif L==3	// en
	"tutors"
#elif L==4	// es
	"tutores"
#elif L==5	// fr
	"tuteurs"
#elif L==6	// gn
	"tutor-ku&eacute;ra"
#elif L==7	// it
	"precettori"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"tutores"
#elif L==10	// tr
	"&ouml;&gbreve;retmenler"
#endif
	,
	[Prj_ROLE_EVL] =
#if   L==1	// ca
	"avaluadors"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"evaluators"
#elif L==4	// es
	"evaluadores"
#elif L==5	// fr
	"&eacute;valuateurs"
#elif L==6	// gn
	"evaluador-ku&eacute;ra"
#elif L==7	// it
	"valutatori"
#elif L==8	// pl
	"ewaluator&oacute;w"
#elif L==9	// pt
	"avaliadores"
#elif L==10	// tr
	"de&gbreve;erlendiriciler"
#endif
	};

const char *Txt_PROJECT_RUBRIC[PrjCfg_NUM_RUBRIC_TYPES] =
	{
	[PrjCfg_RUBRIC_TUT] =
#if   L==1	// ca
	"Tutors"
#elif L==2	// de
	"Tutoren"
#elif L==3	// en
	"Tutors"
#elif L==4	// es
	"Tutores"
#elif L==5	// fr
	"Tuteurs"
#elif L==6	// gn
	"Tutor"
#elif L==7	// it
	"Precettori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Tutores"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	,
	[PrjCfg_RUBRIC_EVL] =
#if   L==1	// ca
	"Avaluadors"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"Evaluators"
#elif L==4	// es
	"Evaluadores"
#elif L==5	// fr
	"&Eacute;valuateurs"
#elif L==6	// gn
	"Evaluador"
#elif L==7	// it
	"Valutatori"
#elif L==8	// pl
	"Ewaluator&oacute;w"
#elif L==9	// pt
	"Avaliadores"
#elif L==10	// tr
	"De&gbreve;erlendiriciler"
#endif
	,
	[PrjCfg_RUBRIC_GBL] =
#if   L==1	// ca
	"Global"
#elif L==2	// de
	"Gesamt"
#elif L==3	// en
	"Globale"
#elif L==4	// es
	"Global"
#elif L==5	// fr
	"Globale"
#elif L==6	// gn
	"General"
#elif L==7	// it
	"Complessiva"
#elif L==8	// pl
	"Globalna"
#elif L==9	// pt
	"Geral"
#elif L==10	// tr
	"Genel"
#endif
	};

const char *Txt_PROJECT_STATUS[Prj_NUM_PROPOSAL_TYPES] =
	{
	[Prj_PROPOSAL_NEW] =
#if   L==1	// ca
	"Projecte nou"
#elif L==2	// de
	"Neues Projekt"
#elif L==3	// en
	"New project"
#elif L==4	// es
	"Proyecto nuevo"
#elif L==5	// fr
	"Projet nouveau"
#elif L==6	// gn
	"Proyecto nuevo"	// Okoteve traduccin
#elif L==7	// it
	"Progetto nuovo"
#elif L==8	// pl
	"Nowy projekt"
#elif L==9	// pt
	"Projeto novo"
#elif L==10	// tr
	"New project"		// eviri lazim!
#endif
	,
	[Prj_PROPOSAL_MODIFIED] =
#if   L==1	// ca
	"Projecte ja proposat en convocat&ograve;ries anteriors, amb modificacions"
#elif L==2	// de
	"Projekt in fr&uuml;heren Anrufen vorgeschlagen, mit &Auml;nderungen"
#elif L==3	// en
	"Project proposed in previous calls, with modifications"
#elif L==4	// es
	"Proyecto propuesto en convocatorias anteriores, con modificaciones"
#elif L==5	// fr
	"Projet propos&eacute; dans les appels pr&eacute;c&eacute;dents, avec modifications"
#elif L==6	// gn
	"Proyecto propuesto en convocatorias anteriores, con modificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Progetto proposto nelle chiamate precedenti, con modifiche"
#elif L==8	// pl
	"Proponowany projekt we wcze&sacute;niejszych rozmowach z modyfikacjami"
#elif L==9	// pt
	"Projeto proposto em chamadas anteriores, com modifica&ccedil;&otilde;es"
#elif L==10	// tr
	"Project proposed in previous calls, with modifications"		// eviri lazim!
#endif
	,
	[Prj_PROPOSAL_UNMODIFIED] =
#if   L==1	// ca
	"Projecte ja proposat en convocat&ograve;ries anteriors, sense modificacions"
#elif L==2	// de
	"Projekt in fr&uuml;heren Anrufen vorgeschlagen, ohne &Auml;nderungen"
#elif L==3	// en
	"Project proposed in previous calls, without modifications"
#elif L==4	// es
	"Proyecto propuesto en convocatorias anteriores, sin modificaciones"
#elif L==5	// fr
	"Projet propos&eacute; dans les appels pr&eacute;c&eacute;dents, sans modifications"
#elif L==6	// gn
	"Proyecto propuesto en convocatorias anteriores, sin modificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Progetto proposto nelle chiamate precedenti, senza modifiche"
#elif L==8	// pl
	"Proponowany projekt we wcze&sacute;niejszych rozmowach bez modyfikacji"
#elif L==9	// pt
	"Projeto proposto em chamadas anteriores, sem modifica&ccedil;&otilde;es"
#elif L==10	// tr
	"Project proposed in previous calls, without modifications"		// eviri lazim!
#endif
	};
const char *Txt_PROJECT_FAULTY_FAULTLESS_PROJECTS[Prj_NUM_FAULTINESS] =
	{
	[Prj_FAULTY] =
#if   L==1	// ca
	"Projectes defectuosos"
#elif L==2	// de
	"Fehlerhafte Projekte"
#elif L==3	// en
	"Faulty projects"
#elif L==4	// es
	"Proyectos defectuosos"
#elif L==5	// fr
	"Projets d&eacute;fectueux"
#elif L==6	// gn
	"Proyectos defectuosos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti difettosi"
#elif L==8	// pl
	"Wadliwe projekty"
#elif L==9	// pt
	"Projetos defeituosos"
#elif L==10	// tr
	"Hatal&inodot; projeler"
#endif
	,
	[Prj_FAULTLESS] =
#if   L==1	// ca
	"Projectes correctos"
#elif L==2	// de
	"Fehlerfreie Projekte"
#elif L==3	// en
	"Faultless projects"
#elif L==4	// es
	"Proyectos correctos"
#elif L==5	// fr
	"Projets sans faille"
#elif L==6	// gn
	"Proyectos correctos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti corretti"
#elif L==8	// pl
	"Bezb&lstrok;&eogon;dne projekty"
#elif L==9	// pt
	"Projetos sem falhas"
#elif L==10	// tr
	"Kusursuz projeler"
#endif
	};

const char *Txt_PROJECT_REVIEW_PLURAL[Prj_NUM_REVIEW_STATUS] =
	{
	[Prj_UNREVIEWED] =
#if   L==1	// ca
	"Projectes sense revisar"
#elif L==2	// de
	"Ungepr&uuml;fte Projekte"
#elif L==3	// en
	"Unreviewed projects"
#elif L==4	// es
	"Proyectos sin revisar"
#elif L==5	// fr
	"Projets non revus"
#elif L==6	// gn
	"Proyectos sin revisar"	// Okoteve traduccin
#elif L==7	// it
	"Progetti non recensiti"
#elif L==8	// pl
	"Niesprawdzone projekty"
#elif L==9	// pt
	"Projetos n&atilde;o revisados"
#elif L==10	// tr
	"&Idot;ncelenmemi&scedil; projeler"
#endif
	,
	[Prj_UNAPPROVED] =
#if   L==1	// ca
	"Projectes pendents d'esmena"
#elif L==2	// de
	"Projekte mit ausstehender Korrektur"
#elif L==3	// en
	"Projects pending correction"
#elif L==4	// es
	"Proyectos pendientes de subsanaci&oacute;n"
#elif L==5	// fr
	"Projets en attente de correction"
#elif L==6	// gn
	"Proyectos pendientes de subsanaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Progetti in attesa di correzione"
#elif L==8	// pl
	"Projekty oczekuj&aogon;ce na korekt&eogon;"
#elif L==9	// pt
	"Projetos pendentes de corre&ccedil;&atilde;o"
#elif L==10	// tr
	"D&uuml;zeltilmeyi bekleyen projeler"
#endif
	,
	[Prj_APPROVED] =
#if   L==1	// ca
	"Projectes aprovats"
#elif L==2	// de
	"Genehmigte Projekte"
#elif L==3	// en
	"Approved projects"
#elif L==4	// es
	"Proyectos aprobados"
#elif L==5	// fr
	"Projets approuv&eacute;s"
#elif L==6	// gn
	"Proyectos aprobados"	// Okoteve traduccin
#elif L==7	// it
	"Progetti approvati"
#elif L==8	// pl
	"Zatwierdzone projekty"
#elif L==9	// pt
	"Projetos aprovados"
#elif L==10	// tr
	"Onaylanm&inodot;&scedil; projeler"
#endif
	};

const char *Txt_PROJECT_REVIEW_SINGUL[Prj_NUM_REVIEW_STATUS] =
	{
	[Prj_UNREVIEWED] =
#if   L==1	// ca
	"Sense revisar"
#elif L==2	// de
	"Ungepr&uuml;ftes"
#elif L==3	// en
	"Unreviewed"
#elif L==4	// es
	"Sin revisar"
#elif L==5	// fr
	"Non revu"
#elif L==6	// gn
	"Sin revisar"	// Okoteve traduccin
#elif L==7	// it
	"Non recensito"
#elif L==8	// pl
	"Niesprawdzony"
#elif L==9	// pt
	"N&atilde;o revisado"
#elif L==10	// tr
	"&Idot;ncelenmemi&scedil;"
#endif
	,
	[Prj_UNAPPROVED] =
#if   L==1	// ca
	"Pendent d'esmena"
#elif L==2	// de
	"Ausstehende Korrektur"
#elif L==3	// en
	"Pending correction"
#elif L==4	// es
	"Pendiente de subsanacin"
#elif L==5	// fr
	"En attente de correction"
#elif L==6	// gn
	"Pendiente de subsanacin"	// Okoteve traduccin
#elif L==7	// it
	"In attesa di correzione"
#elif L==8	// pl
	"Oczekuje na korekt&eogon;"
#elif L==9	// pt
	"Corre&ccedil;&atilde;o pendente"
#elif L==10	// tr
	"D&uuml;zeltme bekleniyor"
#endif
	,
	[Prj_APPROVED] =
#if   L==1	// ca
	"Aprovat"
#elif L==2	// de
	"Genehmigtes"
#elif L==3	// en
	"Approved"
#elif L==4	// es
	"Aprobado"
#elif L==5	// fr
	"Approuv&eacute;"
#elif L==6	// gn
	"Aprobado"	// Okoteve traduccin
#elif L==7	// it
	"Approvato"
#elif L==8	// pl
	"Zatwierdzony"
#elif L==9	// pt
	"Aprovado"
#elif L==10	// tr
	"Onaylanm&inodot;&scedil;"
#endif
	};

const char *Txt_PROJECT_ASSIGNED_NONASSIGNED_PLURAL[Prj_NUM_ASSIGNED_NONASSIG] =
	{
	[Prj_ASSIGNED] =
#if   L==1	// ca
	"Projectes assignats"
#elif L==2	// de
	"Vorbelegt Projekte"
#elif L==3	// en
	"Assigned projects"
#elif L==4	// es
	"Proyectos asignados"
#elif L==5	// fr
	"Projets assign&eacute;s"
#elif L==6	// gn
	"Proyectos asignados"	// Okoteve traduccin
#elif L==7	// it
	"Progetti assegnati"
#elif L==8	// pl
	"Zadany projekty"
#elif L==9	// pt
	"Projetos atribu&iacute;dos"
#elif L==10	// tr
	"Assigned projects"	// eviri lazim!
#endif
	,
	[Prj_NONASSIG] =
#if   L==1	// ca
	"Projectes no assignats"
#elif L==2	// de
	"Nicht vorbelegt Projekte"
#elif L==3	// en
	"Non-assigned projects"
#elif L==4	// es
	"Proyectos no asignados"
#elif L==5	// fr
	"Projets non assign&eacute;s"
#elif L==6	// gn
	"Proyectos no asignados"	// Okoteve traduccin
#elif L==7	// it
	"Progetti non-assegnati"
#elif L==8	// pl
	"Nieprzypisane projekty"
#elif L==9	// pt
	"Projetos n&atilde;o atribu&iacute;dos"
#elif L==10	// tr
	"Non-assigned projects"		// eviri lazim!
#endif
	};

const char *Txt_PROJECT_ASSIGNED_NONASSIGNED_SINGUL[Prj_NUM_ASSIGNED_NONASSIG] =
	{
	[Prj_ASSIGNED] =
#if   L==1	// ca
	"Projecte assignat"
#elif L==2	// de
	"Vorbelegt Projekt"
#elif L==3	// en
	"Assigned project"
#elif L==4	// es
	"Proyecto asignado"
#elif L==5	// fr
	"Projet assign&eacute;"
#elif L==6	// gn
	"Proyecto asignado"	// Okoteve traduccin
#elif L==7	// it
	"Progetto assegnato"
#elif L==8	// pl
	"Zadany projekt"
#elif L==9	// pt
	"Projeto atribu&iacute;do"
#elif L==10	// tr
	"Assigned project"	// eviri lazim!
#endif
	,
	[Prj_NONASSIG] =
#if   L==1	// ca
	"Projecte no assignat"
#elif L==2	// de
	"Nicht vorbelegt Projekt"
#elif L==3	// en
	"Non-assigned project"
#elif L==4	// es
	"Proyecto no asignado"
#elif L==5	// fr
	"Projet non assign&eacute;"
#elif L==6	// gn
	"Proyecto no asignado"	// Okoteve traduccin
#elif L==7	// it
	"Progetto non-assegnato"
#elif L==8	// pl
	"Nieprzypisane projekt"
#elif L==9	// pt
	"Projeto n&atilde;o atribu&iacute;do"
#elif L==10	// tr
	"Non-assigned project"	// eviri lazim!
#endif
	};

const char *Txt_PROJECT_HIDDEN_VISIBL_PROJECTS[HidVis_NUM_HIDDEN_VISIBLE] =
	{
	[HidVis_HIDDEN] =
#if   L==1	// ca
	"Projectes ocults"
#elif L==2	// de
	"Unsichtbare Projekte"
#elif L==3	// en
	"Hidden projects"
#elif L==4	// es
	"Proyectos ocultos"
#elif L==5	// fr
	"Projets cach&eacute;s"
#elif L==6	// gn
	"Proyectos ocultos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti nascosti"
#elif L==8	// pl
	"Ukryte projekty"
#elif L==9	// pt
	"Projetos ocultos"
#elif L==10	// tr
	"Hidden projects"	// eviri lazim!
#endif
	,
	[HidVis_VISIBLE] =
#if   L==1	// ca
	"Projectes visibles"
#elif L==2	// de
	"Sichtbar Projekte"
#elif L==3	// en
	"Visible projects"
#elif L==4	// es
	"Proyectos visibles"
#elif L==5	// fr
	"Projets visibles"
#elif L==6	// gn
	"Proyectos visibles"	// Okoteve traduccin
#elif L==7	// it
	"Progetti visibili"
#elif L==8	// pl
	"Widoczne projekty"
#elif L==9	// pt
	"Projetos vis&iacute;veis"
#elif L==10	// tr
	"Visible projects"	// eviri lazim!
#endif
	};

const char *Txt_Projects =
#if   L==1	// ca
	"Projectes (treballs fi de grau/m&agrave;ster, tesi&hellip;)";
#elif L==2	// de
	"Projekte (Abschlussarbeiten, Diplomarbeiten&hellip;)";
#elif L==3	// en
	"Projects (end-of-degree projects, thesis&hellip;)";
#elif L==4	// es
	"Proyectos (trabajos fin de grado/m&aacute;ster, tesis&hellip;)";
#elif L==5	// fr
	"Projets (projets de fin de cours, th&egrave;se&hellip;)";
#elif L==6	// gn
	"Proyectos (trabajos fin de grado/m&aacute;ster, tesis&hellip;)";	// Okoteve traduccin
#elif L==7	// it
	"Progetti (progetti di fine corso, tesi&hellip;)";
#elif L==8	// pl
	"Projekty (projekty ko&nacute;cowe, praca dyplomowa&hellip;)";
#elif L==9	// pt
	"Projetos (projetos de fim de grau, tese&hellip;)";
#elif L==10	// tr
	"Projects (end-of-degree projects, thesis&hellip;)";			// eviri lazim!
#endif

const char *Txt_Proposal =
#if   L==1	// ca
	"Proposta";
#elif L==2	// de
	"Vorschlag";
#elif L==3	// en
	"Proposal";
#elif L==4	// es
	"Propuesta";
#elif L==5	// fr
	"Proposition";
#elif L==6	// gn
	"Propuesta";	// Okoteve traduccin
#elif L==7	// it
	"Proposta";
#elif L==8	// pl
	"Wniosek";
#elif L==9	// pt
	"Proposta";
#elif L==10	// tr
	"Proposal";	// eviri lazim!
#endif

const char *Txt_Public_agenda_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Agenda p&uacute;blica de %s";
#elif L==2	// de
	"&Ouml;ffentliche Agenda von %s";
#elif L==3	// en
	"%s's public agenda";
#elif L==4	// es
	"Agenda p&uacute;blica de %s";
#elif L==5	// fr
	"Agenda publique de %s";
#elif L==6	// gn
	"Agenda p&uacute;blica de %s";	// Okoteve traduccin
#elif L==7	// it
	"Agenda pubblica di %s";
#elif L==8	// pl
	"Pami&eogon;tnik publiczny %s";
#elif L==9	// pt
	"Agenda p&uacute;blica de %s";
#elif L==10	// tr
	"%s's public agenda";		// eviri lazim!
#endif

const char *Txt_public_FILES =
#if   L==1	// ca
	"p&uacute;blics";
#elif L==2	// de
	"&ouml;ffentliche";
#elif L==3	// en
	"public";
#elif L==4	// es
	"p&uacute;blicos";
#elif L==5	// fr
	"publics";
#elif L==6	// gn
	"p&uacute;blicos";	// Okoteve traduccin
#elif L==7	// it
	"pubblici";
#elif L==8	// pl
	"publicznych";
#elif L==9	// pt
	"p&uacute;blicos";
#elif L==10	// tr
	"public";		// eviri lazim!
#endif

const char *Txt_Public_open_educational_resource_OER_for_everyone =
#if   L==1	// ca
	"P&uacute;blic, recurs educatiu obert (OER) per a tothom";
#elif L==2	// de
	"&Ouml;ffentliche, Open Educational Resource (OER) f&uuml;r jedermann";
#elif L==3	// en
	"Public, open educational resource for everyone";
#elif L==4	// es
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";
#elif L==5	// fr
	"Public, ressource &eacute;ducatif libre (REL) pour tout le monde";
#elif L==6	// gn
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";	// Okoteve traduccin
#elif L==7	// it
	"Pubblico, risorse didattici aperto (OER) per tutti";
#elif L==8	// pl
	"Publiczne, otwarte zasoby edukacyjne (OER) dla ka&zdot;dego";
#elif L==9	// pt
	"P&uacute;blico, recurso educacional aberto (OER) para todos";
#elif L==10	// tr
	"Public, open educational resource for everyone";			// eviri lazim!
#endif

const char *Txt_Public_views =
#if   L==1	// ca
	"Accessos p&uacute;blics";
#elif L==2	// de
	"&Ouml;ffentliche Ansichten";
#elif L==3	// en
	"Public views";
#elif L==4	// es
	"Accesos p&uacute;blicos";
#elif L==5	// fr
	"Vues publics";
#elif L==6	// gn
	"Accesos p&uacute;blicos";	// Okoteve traduccin
#elif L==7	// it
	"Visualizzazioni pubblici";
#elif L==8	// pl
	"Publiczne ods&lstrok;on";
#elif L==9	// pt
	"Acessos p&uacute;blicos";
#elif L==10	// tr
	"Public views";			// eviri lazim!
#endif

const char *Txt_Publish_call_FOR_EXAM =
#if   L==1	// ca
	"Publicar convocat&ograve;ria";
#elif L==2	// de
	"Ver&ouml;ffentlichen Aufruf";
#elif L==3	// en
	"Publish call";
#elif L==4	// es
	"Publicar convocatoria";
#elif L==5	// fr
	"Publier convocation";
#elif L==6	// gn
	"Publicar convocatoria";	// Okoteve traduccin
#elif L==7	// it
	"Pubblicare sessione";
#elif L==8	// pl
	"Publikuje zaproszenie";
#elif L==9	// pt
	"Publicar chamada";
#elif L==10	// tr
	"Publish call";			// eviri lazim!
#endif

const char *Txt_QR_code =
#if   L==1	// ca
	"Codi QR";
#elif L==2	// de
	"QR-Code";
#elif L==3	// en
	"QR code";
#elif L==4	// es
	"C&oacute;digo QR";
#elif L==5	// fr
	"QR Code";
#elif L==6	// gn
	"C&oacute;digo QR";	// Okoteve traduccin
#elif L==7	// it
	"Codice QR";
#elif L==8	// pl
	"QR Code";
#elif L==9	// pt
	"C&oacute;digo QR";
#elif L==10	// tr
	"QR code";		// eviri lazim!
#endif

const char *Txt_question =
#if   L==1	// ca
	"pregunta";
#elif L==2	// de
	"Frage";
#elif L==3	// en
	"question";
#elif L==4	// es
	"pregunta";
#elif L==5	// fr
	"question";
#elif L==6	// gn
	"pregunta";	// Okoteve traduccin
#elif L==7	// it
	"domande";
#elif L==8	// pl
	"pytanie";
#elif L==9	// pt
	"pergunta";
#elif L==10	// tr
	"question";	// eviri lazim!
#endif

const char *Txt_Question =
#if   L==1	// ca
	"Pregunta";
#elif L==2	// de
	"Frage";
#elif L==3	// en
	"Question";
#elif L==4	// es
	"Pregunta";
#elif L==5	// fr
	"Question";
#elif L==6	// gn
	"Pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Domanda";
#elif L==8	// pl
	"Pytanie";
#elif L==9	// pt
	"Pergunta";
#elif L==10	// tr
	"Question";	// eviri lazim!
#endif

const char *Txt_Question_bank =
#if   L==1	// ca
	"Banc de preguntes";
#elif L==2	// de
	"Fragenbank";
#elif L==3	// en
	"Question bank";
#elif L==4	// es
	"Banco de preguntas";
#elif L==5	// fr
	"Banque de questions";
#elif L==6	// gn
	"Banco de preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Banca di domande";
#elif L==8	// pl
	"Bank pyta&nacute;";
#elif L==9	// pt
	"Banco de perguntas";
#elif L==10	// tr
	"Question bank";	// eviri lazim!
#endif

const char *Txt_Question_code_X =	// Warning: it is very important to include %ld in the following sentences
#if   L==1	// ca
	"Pregunta amb codi %ld";
#elif L==2	// de
	"Frage Code %ld";
#elif L==3	// en
	"Code %ld";
#elif L==4	// es
	"Pregunta con c&oacute;digo %ld";
#elif L==5	// fr
	"Code de question %ld";
#elif L==6	// gn
	"Pregunta con c&oacute;digo %ld";	// Okoteve traduccin
#elif L==7	// it
	"Codice di domanda %ld";
#elif L==8	// pl
	"Kodeks pytanie %ld";
#elif L==9	// pt
	"Pergunta com c&oacute;digo %ld";
#elif L==10	// tr
	"Code %ld";				// eviri lazim!
#endif

const char *Txt_Question_modified =
#if   L==1	// ca
	"Pregunta modificada.";
#elif L==2	// de
	"Frage modifiziert.";
#elif L==3	// en
	"Question modified.";
#elif L==4	// es
	"Pregunta modificada.";
#elif L==5	// fr
	"Question modifi&eacute;e.";
#elif L==6	// gn
	"Pregunta modificada.";		// Okoteve traduccin
#elif L==7	// it
	"Domanda modificata.";
#elif L==8	// pl
	"Pytanie zmodyfikowane.";
#elif L==9	// pt
	"Pergunta modificada.";
#elif L==10	// tr
	"Question modified.";		// eviri lazim!
#endif

const char *Txt_Question_removed =
#if   L==1	// ca
	"Pregunta eliminada.";
#elif L==2	// de
	"Frage entfernt.";
#elif L==3	// en
	"Question removed.";
#elif L==4	// es
	"Pregunta eliminada.";
#elif L==5	// fr
	"Question supprim&eacute;e.";
#elif L==6	// gn
	"Pregunta eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Domanda rimossa.";
#elif L==8	// pl
	"Pytanie usuniete.";
#elif L==9	// pt
	"Pergunta removida.";
#elif L==10	// tr
	"Question removed.";	// eviri lazim!
#endif

const char *Txt_questions =
#if   L==1	// ca
	"preguntes";
#elif L==2	// de
	"Fragen";
#elif L==3	// en
	"questions";
#elif L==4	// es
	"preguntas";
#elif L==5	// fr
	"questions";
#elif L==6	// gn
	"preguntas";	// Okoteve traduccin
#elif L==7	// it
	"domande";
#elif L==8	// pl
	"pytania";
#elif L==9	// pt
	"perguntas";
#elif L==10	// tr
	"questions";	// eviri lazim!
#endif

const char *Txt_Questions =
#if   L==1	// ca
	"Preguntes";
#elif L==2	// de
	"Fragen";
#elif L==3	// en
	"Questions";
#elif L==4	// es
	"Preguntas";
#elif L==5	// fr
	"Questions";
#elif L==6	// gn
	"Preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Domande";
#elif L==8	// pl
	"Pyta&nacute;";
#elif L==9	// pt
	"Perguntas";
#elif L==10	// tr
	"Sorular";
#endif

const char *Txt_Questions_and_problems =
#if   L==1	// ca
	"Consultes i problemes";
#elif L==2	// de
	"Fragen und Probleme";
#elif L==3	// en
	"Questions and problems";
#elif L==4	// es
	"Consultas y problemas";
#elif L==5	// fr
	"Questions et probl&egrave;mes";
#elif L==6	// gn
	"Consultas y problemas";	// Okoteve traduccin
#elif L==7	// it
	"Domande e problemi";
#elif L==8	// pl
	"Pytania i problemy";
#elif L==9	// pt
	"Perguntas e problemas";
#elif L==10	// tr
	"Questions and problems";	// eviri lazim!
#endif

const char *Txt_QUESTIONS_invalid =
#if   L==1	// ca
	"anul&middot;lades";
#elif L==2	// de
	"storniert";
#elif L==3	// en
	"invalid";
#elif L==4	// es
	"anuladas";
#elif L==5	// fr
	"annul&eacute;es";
#elif L==6	// gn
	"anuladas";	// Okoteve traduccin
#elif L==7	// it
	"cancellate";
#elif L==8	// pl
	"anulowane";
#elif L==9	// pt
	"canceladas";
#elif L==10	// tr
	"invalid";	// eviri lazim!
#endif

const char *Txt_Questions_removed_X =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Preguntas eliminadas: %u.";
#elif L==2	// de
	"Fragen entfernt: %u.";
#elif L==3	// en
	"Questions removed: %u.";
#elif L==4	// es
	"Preguntas eliminadas: %u.";
#elif L==5	// fr
	"Questions supprim&eacute;es: %u.";
#elif L==6	// gn
	"Preguntas eliminadas: %u.";	// Okoteve traduccin
#elif L==7	// it
	"Domande rimosse: %u.";
#elif L==8	// pl
	"Pytania usuniete: %u.";
#elif L==9	// pt
	"Perguntas removidas: %u.";
#elif L==10	// tr
	"Questions removed: %u.";	// eviri lazim!
#endif

const char *Txt_QUESTIONS_valid =
#if   L==1	// ca
	"v&agrave;lides";
#elif L==2	// de
	"g&uuml;ltige";
#elif L==3	// en
	"valid";
#elif L==4	// es
	"v&aacute;lidas";
#elif L==5	// fr
	"valides";
#elif L==6	// gn
	"v&aacute;lidas";	// Okoteve traduccin
#elif L==7	// it
	"valide";
#elif L==8	// pl
	"wa&zdot;ne";
#elif L==9	// pt
	"v&aacute;lidas";
#elif L==10	// tr
	"valid";		// eviri lazim!
#endif

const char *Txt_Quota_exceeded =
#if   L==1	// ca
	"Quota excedida!";
#elif L==2	// de
	"Kontingent &uuml;berschritten!";
#elif L==3	// en
	"Quota exceeded!";
#elif L==4	// es
	"&iexcl;Cuota excedida!";
#elif L==5	// fr
	"Quota d&eacute;pass&eacute;!";
#elif L==6	// gn
	"&iexcl;Cuota excedida!";	// Okoteve traduccin
#elif L==7	// it
	"Quota superata!";
#elif L==8	// pl
	"Przekroczono przydzial!";
#elif L==9	// pt
	"Quota excedida!";
#elif L==10	// tr
	"Quota exceeded!";		// eviri lazim!
#endif

const char *Txt_ReaBRders =
#if   L==1	// ca
	"Lec-<br />tores";	// Necessita traducci
#elif L==2	// de
	"Leser";
#elif L==3	// en
	"Rea-<br />ders";
#elif L==4	// es
	"Lec-<br />tores";
#elif L==5	// fr
	"Lec-<br />teurs";
#elif L==6	// gn
	"Lec-<br />tores";	// Okoteve traduccin
#elif L==7	// it
	"Le-<br />ttori";
#elif L==8	// pl
	"Czytel-<br />nik&oacute;w";
#elif L==9	// pt
	"Lei-<br />tores";
#elif L==10	// tr
	"Rea-<br />ders";	// eviri lazim!
#endif

const char *Txt_Real_number_between_A_and_B_1 =
#if   L==1	// ca
	"Nombre real entre";
#elif L==2	// de
	"Reelle Zahl zwischen";
#elif L==3	// en
	"Real number between";
#elif L==4	// es
	"N.&ordm; real entre";
#elif L==5	// fr
	"Nombre r&eacute;el entre";
#elif L==6	// gn
	"N.&ordm; real entre";	// Okoteve traduccin
#elif L==7	// it
	"Numero reale compreso tra";
#elif L==8	// pl
	"Prawdziwa liczbe miedzy";
#elif L==9	// pt
	"N.&ordm; real entre";
#elif L==10	// tr
	"Real number between";	// eviri lazim!
#endif

const char *Txt_Real_number_between_A_and_B_2 =	// Must start by space
#if   L==1	// ca
	" i";
#elif L==2	// de
	" und";
#elif L==3	// en
	" and";
#elif L==4	// es
	" y";
#elif L==5	// fr
	" et";
#elif L==6	// gn
	" y";	// Okoteve traduccin
#elif L==7	// it
	" e";
#elif L==8	// pl
	" i";
#elif L==9	// pt
	" e";
#elif L==10	// tr
	" and";	// eviri lazim!
#endif

const char *Txt_Recommended_aspect_ratio =
#if   L==1	// ca
	"Relaci&oacute; d'aspecte recomanada";
#elif L==2	// de
	"Empfohlene Seitenverh&auml;ltnisr";
#elif L==3	// en
	"Recommended aspect ratio";
#elif L==4	// es
	"Relaci&oacute;n de aspecto recomendada";
#elif L==5	// fr
	"Rapport d'aspect recommand&eacute;";
#elif L==6	// gn
	"Relaci&oacute;n de aspecto recomendada";	// Okoteve traduccin
#elif L==7	// it
	"Rapporto aspetto consigliato";
#elif L==8	// pl
	"Zalecane proporcje obrazu";
#elif L==9	// pt
	"Propor&ccedil;&atilde;o recomendada";
#elif L==10	// tr
	"Recommended aspect ratio";			// eviri lazim!
#endif

const char *Txt_Recommended_resolution =
#if   L==1	// ca
	"Resoluci&oacute; recomanada";
#elif L==2	// de
	"Empfohlene Aufl&ouml;sung";
#elif L==3	// en
	"Recommended resolution";
#elif L==4	// es
	"Resoluci&oacute;n recomendada";
#elif L==5	// fr
	"R&eacute;solution recommand&eacute;e";
#elif L==6	// gn
	"Resoluci&oacute;n recomendada";	// Okoteve traduccin
#elif L==7	// it
	"Risoluzione consigliata";
#elif L==8	// pl
	"Zalecana rozdzielczo&sacute;&cacute;";
#elif L==9	// pt
	"Resolu&ccedil;&atilde;o recomendada";
#elif L==10	// tr
	"Recommended resolution";		// eviri lazim!
#endif

const char *Txt_Record_card_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"S'ha suprimit la fitxa <strong>%s</strong>.";
#elif L==2	// de
	"Karteikarte von <strong>%s</strong> wurde entfernt.";
#elif L==3	// en
	"Record card of <strong>%s</strong> has been removed.";
#elif L==4	// es
	"Se ha eliminado la ficha de <strong>%s</strong>.";
#elif L==5	// fr
	"La carte d'enregistrement de <strong>%s</strong> a &eacute;t&eacute; supprim&eacute;e.";
#elif L==6	// gn
	"Se ha eliminado la ficha de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La scheda de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8	// pl
	"Karta Rekord <strong>%s</strong> zostala usunieta.";
#elif L==9	// pt
	"Cart&atilde;o de registro de <strong>%s</strong> foi removido.";
#elif L==10	// tr
	"Record card of <strong>%s</strong> has been removed.";	// eviri lazim!
#endif

const char *Txt_record_cards_per_page =
#if   L==1	// ca
	"fichas por p&aacute;gina";	// Necessita traducci
#elif L==2	// de
	"Karteikarten pro Seite";
#elif L==3	// en
	"record cards per page";
#elif L==4	// es
	"fichas por p&aacute;gina";
#elif L==5	// fr
	"cartes par page";
#elif L==6	// gn
	"fichas por p&aacute;gina";	// Okoteve traduccin
#elif L==7	// it
	"scheda per pagina";
#elif L==8	// pl
	"karty zapisu na stronie";
#elif L==9	// pt
	"cart&otilde;es por p&aacute;gina";
#elif L==10	// tr
	"record cards per page";	// eviri lazim!
#endif

const char *Txt_Record_field_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Camp de fitxa <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Datenfeld <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Record field <strong>%s</strong> removed.";
#elif L==4	// es
	"Campo de ficha <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Record field <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Campo de ficha <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Campo scheda <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Pole rekordu <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Campo de cart&atilde;o <strong>%s</strong> removido.";
#elif L==10	// tr
	"Record field <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_RECORD_FIELD_VISIBILITY_MENU[Rec_NUM_TYPES_VISIBILITY] =
	{
	[Rec_HIDDEN_FIELD] =
#if   L==1	// ca
	"Ocult"
#elif L==2	// de
	"Versteckt"
#elif L==3	// en
	"Hidden"
#elif L==4	// es
	"Oculto"
#elif L==5	// fr
	"Cach&eacute;"
#elif L==6	// gn
	"Oculto"	// Okoteve traduccin
#elif L==7	// it
	"Nascosto"
#elif L==8	// pl
	"Ukryty"
#elif L==9	// pt
	"Oculto"
#elif L==10	// tr
	"Hidden"	// eviri lazim!
#endif
	,
	[Rec_VISIBLE_FIELD] =
#if   L==1	// ca
	"Visible per&ograve; no editable"
#elif L==2	// de
	"Sichtbar, aber nicht bearbeitet werden"
#elif L==3	// en
	"Visible but not editable"
#elif L==4	// es
	"Visible pero no editable"
#elif L==5	// fr
	"Visible mais non modifiable"
#elif L==6	// gn
	"Visible pero no editable"	// Okoteve traduccin
#elif L==7	// it
	"Visibile ma non modificabile"
#elif L==8	// pl
	"Widoczne, ale nie mozna edytowa&cacute;"
#elif L==9	// pt
	"Vis&iacute;vel, mas n&atilde;o edit&aacute;vel"
#elif L==10	// tr
	"Visible but not editable"	// eviri lazim!
#endif
	,
	[Rec_EDITABLE_FIELD] =
#if   L==1	// ca
	"Visible i editable"
#elif L==2	// de
	"Sichtbar und bearbeitet werde"
#elif L==3	// en
	"Visible and editable"
#elif L==4	// es
	"Visible y editable"
#elif L==5	// fr
	"Visible et modifiable"
#elif L==6	// gn
	"Visible y editable"	// Okoteve traduccin
#elif L==7	// it
	"Visibile e modificabile"
#elif L==8	// pl
	"Widzialne i mozna edytowa&cacute;"
#elif L==9	// pt
	"Vis&iacute;vel e edit&aacute;vel"
#elif L==10	// tr
	"Visible and editable"	// eviri lazim!
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_MSG[Rec_NUM_TYPES_VISIBILITY] =	// Warning: it is very important to include %s in the following sentences
	{
	[Rec_HIDDEN_FIELD] =
#if   L==1	// ca
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Necessita traducci
#elif L==2	// de
	"Von nun an k&ouml;nnen die Studenten"
	" nicht das Feld <strong>%s</strong> der Profile."
#elif L==3	// en
	"From now on students"
	" can not see the field <strong>%s</strong> of the records."
#elif L==4	// es
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."
#elif L==5	// fr
	"A partir de maintenant, les &eacute;tudiants"
	" ne peuvent pas voir le champ <strong>%s</strong> de les cartes."
#elif L==6	// gn
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7	// it
	"Da ora in poi gli studenti"
	" non possono visualizzare il campo <strong>%s</strong> delle schede."
#elif L==8	// pl
	"Od teraz studenci"
	" nie widac pola <strong>%s</strong> zapis&oacute;w."
#elif L==9	// pt
	"De agora em diante,"
	" os estudantes n&atilde;o podem ver o campo <strong>%s</strong> dos cart&otilde;es."
#elif L==10	// tr
	"From now on students"
	" can not see the field <strong>%s</strong> of the records."	// eviri lazim!
#endif
	,
	[Rec_VISIBLE_FIELD] =
#if   L==1	// ca
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Necessita traducci
#elif L==2	// de
	"Von nun an k&ouml;nnen sich Studenten,"
	" aber nicht bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==3	// en
	"From now on students"
	" can view but not edit the field <strong>%s</strong> of the records."
#elif L==4	// es
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."
#elif L==5	// fr
	"A partir de maintenant, les &eacute;tudiants"
	" peuvent consulter mais pas modifier le champ <strong>%s</strong> de les cartes."
#elif L==6	// gn
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7	// it
	"Da ora in poi gli studenti"
	" possono visualizzare ma non modificare il campo <strong>%s</strong> delle schede."
#elif L==8	// pl
	"Od teraz studenci"
	" moga przegladac, ale nie edytowa&cacute; pole <strong>%s</strong> zapis&oacute;w."
#elif L==9	// pt
	"De agora em diante,"
	" os estudantes podem visualizar, mas n&atilde;o editar o campo <strong>%s</strong> dos cart&otilde;es."
#elif L==10	// tr
	"From now on students"
	" can view but not edit the field <strong>%s</strong> of the records."		// eviri lazim!
#endif
	,
	[Rec_EDITABLE_FIELD] =
#if   L==1	// ca
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Necessita traducci
#elif L==2	// de
	"Von nun an k&ouml;nnen die Studenten"
	" anzuzeigen und zu bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==3	// en
	"From now on students"
	" can view and edit the field <strong>%s</strong> of the records."
#elif L==4	// es
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."
#elif L==5	// fr
	"A partir de maintenant,"
	" les &eacute;tudiants peuvent consulter et modifier le champ <strong>%s</strong> de les cartes."
#elif L==6	// gn
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7	// it
	"Da ora in poi"
	" gli studenti possono visualizzare e modificare il campo <strong>%s</strong> delle schede."
#elif L==8	// pl
	"Od teraz studenci"
	" moga ogladac i edytowa&cacute; pola <strong>%s</strong> zapis&oacute;w."
#elif L==9	// pt
	"De agora em diante,"
	" os estudantes podem visualizar e editar o campo <strong>%s</strong> dos cart&otilde;es."
#elif L==10	// tr
	"From now on students"
	" can view and edit the field <strong>%s</strong> of the records."	// eviri lazim!
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_RECORD[Rec_NUM_TYPES_VISIBILITY] =
	{
	[Rec_HIDDEN_FIELD] =
#if   L==1	// ca
	"ocult"
#elif L==2	// de
	"versteckt"
#elif L==3	// en
	"hidden"
#elif L==4	// es
	"oculto"
#elif L==5	// fr
	"cach&eacute;"
#elif L==6	// gn
	"oculto"	// Okoteve traduccin
#elif L==7	// it
	"nascosto"
#elif L==8	// pl
	"ukryty"
#elif L==9	// pt
	"oculto"
#elif L==10	// tr
	"hidden"	// eviri lazim!
#endif
	,
	[Rec_VISIBLE_FIELD] =
#if   L==1	// ca
	"visible"
#elif L==2	// de
	"sichtbar"
#elif L==3	// en
	"visible"
#elif L==4	// es
	"visible"
#elif L==5	// fr
	"visible"
#elif L==6	// gn
	"visible"	// Okoteve traduccin
#elif L==7	// it
	"visibile"
#elif L==8	// pl
	"widoczny"
#elif L==9	// pt
	"vis&iacute;vel"
#elif L==10	// tr
	"visible"	// eviri lazim!
#endif
	,
	[Rec_EDITABLE_FIELD] =
#if   L==1	// ca
	"editable"
#elif L==2	// de
	"bearbeitet werde"
#elif L==3	// en
	"editable"
#elif L==4	// es
	"editable"
#elif L==5	// fr
	"modifiable"
#elif L==6	// gn
	"editable"	// Okoteve traduccin
#elif L==7	// it
	"modificabile"
#elif L==8	// pl
	"edytowalne"
#elif L==9	// pt
	"edit&aacute;vel"
#elif L==10	// tr
	"editable"	// eviri lazim!
#endif
	};

const char *Txt_Record_fields =
#if   L==1	// ca
	"Camps de fitxes";
#elif L==2	// de
	"Aufnahmefelder";
#elif L==3	// en
	"Record fields";
#elif L==4	// es
	"Campos de fichas";
#elif L==5	// fr
	"Champs d'enregistrement";
#elif L==6	// gn
	"Campos de fichas";	// Okoteve traduccin
#elif L==7	// it
	"Campo schede";
#elif L==8	// pl
	"Rekord w dziedzinie";
#elif L==9	// pt
	"Campos de cart&otilde;es";
#elif L==10	// tr
	"Record fields";	// eviri lazim!
#endif

const char *Txt_Recipients =
#if   L==1	// ca
	"Destinataris";
#elif L==2	// de
	"Empf&auml;nger";
#elif L==3	// en
	"Recipients";
#elif L==4	// es
	"Destinatarios";
#elif L==5	// fr
	"Destinataires";
#elif L==6	// gn
	"Destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Destinatari";
#elif L==8	// pl
	"Odbiorcy";
#elif L==9	// pt
	"Destinat&aacute;rios";
#elif L==10	// tr
	"Recipients";		// eviri lazim!
#endif

const char *Txt_Recursive_rubric =
#if   L==1	// ca
	"R&uacute;brica recursiva";
#elif L==2	// de
	"Rekursive Rubrik";
#elif L==3	// en
	"Recursive rubric";
#elif L==4	// es
	"R&uacute;brica recursiva";
#elif L==5	// fr
	"Rubrique r&eacute;cursive";
#elif L==6	// gn
	"R&uacute;brica recursiva";
#elif L==7	// it
	"Rubrica ricorsiva";
#elif L==8	// pl
	"Rubryka rekurencyjna";
#elif L==9	// pt
	"Rubrica recursiva";
#elif L==10	// tr
	"&Ouml;zyinelemeli de&gbreve;erlendirme listesi";
#endif

const char *Txt_Register =
#if   L==1	// ca
	"Inscriure";
#elif L==2	// de
	"Anmeldung";
#elif L==3	// en
	"Register";
#elif L==4	// es
	"Inscribir";
#elif L==5	// fr
	"Inscrire";
#elif L==6	// gn
	"Inscribir";	// Okoteve traduccin
#elif L==7	// it
	"Registrare";
#elif L==8	// pl
	"Zarejestruj sie";
#elif L==9	// pt
	"Registrar";
#elif L==10	// tr
	"Register";	// eviri lazim!
#endif

const char *Txt_Register_students =
#if   L==1	// ca
	"Inscriure estudiants";
#elif L==2	// de
	"Anmeldung Studenten";
#elif L==3	// en
	"Register students";
#elif L==4	// es
	"Inscribir estudiantes";
#elif L==5	// fr
	"Inscrire &eacute;tudiants";
#elif L==6	// gn
	"Inscribir estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Registrare studenti";
#elif L==8	// pl
	"Rejestracja student&oacute;w";
#elif L==9	// pt
	"Registrar estudantes";
#elif L==10	// tr
	"Register students";		// eviri lazim!
#endif

const char *Txt_Register_students_in_COURSE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure estudiants en <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung Studenten im <strong>%s</strong>";
#elif L==3	// en
	"Register students in <strong>%s</strong>";
#elif L==4	// es
	"Inscribir estudiantes en <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire &eacute;tudiants dans <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir estudiantes en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare studenti in <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja student&oacute;w w <strong>%s</strong>";
#elif L==9	// pt
	"Registrar estudantes em <strong>%s</strong>";
#elif L==10	// tr
	"Register students in <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Register_the_users_indicated_in_step_1 =
#if   L==1	// ca
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Necessita traducci
#elif L==2	// de
	"Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==3	// en
	"Add (register the users indicated in step 1)";
#elif L==4	// es
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";
#elif L==5	// fr
	"Inscrire les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Okoteve traduccin
#elif L==7	// it
	"Registrare gli utenti che sono in lista";
#elif L==8	// pl
	"Rejestracja uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Registrar os utilizadores que est&atilde;o na lista";
#elif L==10	// tr
	"Add (register the users indicated in step 1)";				// eviri lazim!
#endif

const char *Txt_Register_the_users_indicated_in_step_1_and_remove_the_users_not_indicated =
#if   L==1	// ca
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";		// Necessita traducci
#elif L==2	// de
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste,"
	" und Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==3	// en
	"Update (register the users indicated in step 1"
	" and remove the users <strong>not</strong> indicated)";
#elif L==4	// es
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";
#elif L==5	// fr
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste,"
	" et inscrire les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";		// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista,"
	" e registrare gli utenti che invece lo sono";
#elif L==8	// pl
	"Usun u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie,"
	"i  zarejestrowac uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista,"
	" e registrar os utilizadores que est&atilde;o na lista";
#elif L==10	// tr
	"Update (register the users indicated in step 1"
	" and remove the users <strong>not</strong> indicated)";	// eviri lazim!
#endif

const char *Txt_Register_teacher =
#if   L==1	// ca
	"Inscriure professor/a";
#elif L==2	// de
	"Anmeldung Lehrkraft";
#elif L==3	// en
	"Register teacher";
#elif L==4	// es
	"Inscribir profesor/a";
#elif L==5	// fr
	"Inscrire enseignant/e";
#elif L==6	// gn
	"Inscribir profesor/a";	// Okoteve traduccin
#elif L==7	// it
	"Registrare professore/ssa";
#elif L==8	// pl
	"Rejestracja nauczyciel";
#elif L==9	// pt
	"Registrar professor/a";
#elif L==10	// tr
	"Register teacher";	// eviri lazim!
#endif

const char *Txt_Register_user_IN_A_COURSE_OR_DEGREE =
#if   L==1	// ca
	"Inscriure usuari/a";
#elif L==2	// de
	"Anmeldung Benutzer";
#elif L==3	// en
	"Register user";
#elif L==4	// es
	"Inscribir usuario/a";
#elif L==5	// fr
	"Inscrire utilisateur/se";
#elif L==6	// gn
	"Inscribir usuario/a";	// Okoteve traduccin
#elif L==7	// it
	"Registrare utente";
#elif L==8	// pl
	"Rejestracja u&zdot;ytkownika";
#elif L==9	// pt
	"Registrar utilizador/a";
#elif L==10	// tr
	"Register user";	// eviri lazim!
#endif

const char *Txt_Register_me_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriureme en <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung Sie mir im <strong>%s</strong>";
#elif L==3	// en
	"Register me in <strong>%s</strong>";
#elif L==4	// es
	"Inscribirme en <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire-moi dans <strong>%s</strong>";
#elif L==6	// gn
	"Inscribirme en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrami in <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja mnie w <strong>%s</strong>";
#elif L==9	// pt
	"Registrar me em <strong>%s</strong>";
#elif L==10	// tr
	"Register me in <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Register_USER_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure en l'assignatura <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung im Kurs <strong>%s</strong>";
#elif L==3	// en
	"Register in the course <strong>%s</strong>";
#elif L==4	// es
	"Inscribir en la asignatura <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire dans la mati&egrave;re <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare nel corso <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja w kurs <strong>%s</strong>";
#elif L==9	// pt
	"Registrar na disciplina <strong>%s</strong>";
#elif L==10	// tr
	"Register in the course <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_center_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure com a administrador/a"
	" del center <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung als Administrator"
	" des Lehrinstitut <strong>%s</strong>";
#elif L==3	// en
	"Register as an administrator"
	" of the center <strong>%s</strong>";
#elif L==4	// es
	"Inscribir como administrador/a"
	" del centro <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire en tant qu'administrateur"
	" du center <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir como administrador/a"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9	// pt
	"Registrar como administrador"
	" do centro <strong>%s</strong>";
#elif L==10	// tr
	"Register as an administrator"
	" of the center <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure com a administrador/a"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3	// en
	"Register as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4	// es
	"Inscribir como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9	// pt
	"Registrar como administrador"
	" do grau <strong>%s</strong>";
#elif L==10	// tr
	"Register as an administrator"
	" of the degree <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure com a administrador/a"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3	// en
	"Register as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4	// es
	"Inscribir como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire en tant qu'administrateur"
	" de l'&eacute;tablissement <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja jako administrator"
	" instytucje <strong>%s</strong>";
#elif L==9	// pt
	"Registrar como administrador"
	" da institui&ccedil;&atilde;o <strong>%s</strong>";
#elif L==10	// tr
	"Register as an administrator"
	" of the institution <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Reject =
#if   L==1	// ca
	"Rebutjar";
#elif L==2	// de
	"Ablehnen";
#elif L==3	// en
	"Reject";
#elif L==4	// es
	"Rechazar";
#elif L==5	// fr
	"Rejeter";
#elif L==6	// gn
	"Rechazar";	// Okoteve traduccin
#elif L==7	// it
	"Rifiutare";
#elif L==8	// pl
	"Odrzuca&cacute;";
#elif L==9	// pt
	"Rejeitar";
#elif L==10	// tr
	"Reject";	// eviri lazim!
#endif

const char *Txt_Removal_not_allowed =
#if   L==1	// ca
	"Eliminaci&oacute; no perm&egrave;s";
#elif L==2	// de
	"Entfernen nicht erlaubt";
#elif L==3	// en
	"Removal not allowed";
#elif L==4	// es
	"Eliminaci&oacute;n no permitida";
#elif L==5	// fr
	"Suppression pas autoris&eacute;";
#elif L==6	// gn
	"Eliminaci&oacute;n no permitida";	// Okoteve traduccin
#elif L==7	// it
	"Rimozione non ammessi";
#elif L==8	// pl
	"Usuwanie nie wolno";
#elif L==9	// pt
	"Remo&ccedil;&atilde;o n&atilde;o permitida";
#elif L==10	// tr
	"Removal not allowed";			// eviri lazim!
#endif

const char *Txt_Remove =
#if   L==1	// ca
	"Eliminar";
#elif L==2	// de
	"Entfernen";
#elif L==3	// en
	"Remove";
#elif L==4	// es
	"Eliminar";
#elif L==5	// fr
	"Supprimer";
#elif L==6	// gn
	"Pe'a";
#elif L==7	// it
	"Rimuovere";
#elif L==8	// pl
	"Usu&nacute;";
#elif L==9	// pt
	"Remover";
#elif L==10	// tr
	"Elemek";
#endif

const char *Txt_Remove_all_students =
#if   L==1	// ca
	"Eliminar tots estudiants";
#elif L==2	// de
	"Alle Studenten entfernen";
#elif L==3	// en
	"Remove all students";
#elif L==4	// es
	"Eliminar todos los estudiantes";
#elif L==5	// fr
	"Enlever tous &eacute;tudiants";
#elif L==6	// gn
	"Eliminar todos los estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere tutti gli stud.";
#elif L==8	// pl
	"Usu&nacute; wszystkich student&oacute;w";
#elif L==9	// pt
	"Tirar todos estudantes";
#elif L==10	// tr
	"Remove all students";			// eviri lazim!
#endif

const char *Txt_Remove_files_older_than_PART_1_OF_2 =
#if   L==1	// ca
	"Eliminar els arxius amb m&eacute;s de";
#elif L==2	// de
	"Dateien zu entfernen, die &auml;lter als";
#elif L==3	// en
	"Remove files older than";
#elif L==4	// es
	"Eliminar los archivos con m&aacute;s de";
#elif L==5	// fr
	"Supprimer les fichiers de plus de";
#elif L==6	// gn
	"Eliminar los archivos con m&aacute;s de";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere i file pi&ugrave; vecchi di";
#elif L==8	// pl
	"Remove files older than";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Remover arquivos com mais de";
#elif L==10	// tr
	"Remove files older than";			// eviri lazim!
#endif

const char *Txt_Remove_files_older_than_PART_2_OF_2 =
#if   L==1	// ca
	"mesos d'antiguitat.";
#elif L==2	// de
	"Monate.";
#elif L==3	// en
	"months.";
#elif L==4	// es
	"meses de antig&uuml;edad.";
#elif L==5	// fr
	"mois.";
#elif L==6	// gn
	"meses de antig&uuml;edad.";	// Okoteve traduccin
#elif L==7	// it
	"mesi.";
#elif L==8	// pl
	"months.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"meses.";
#elif L==10	// tr
	"months.";			// eviri lazim!
#endif

const char *Txt_Remove_me =
#if   L==1	// ca
	"Eliminarme";
#elif L==2	// de
	"Entfernen Sie";
#elif L==3	// en
	"Remove me";
#elif L==4	// es
	"Eliminarme";
#elif L==5	// fr
	"Enlever-moi";
#elif L==6	// gn
	"Eliminarme";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi";
#elif L==8	// pl
	"Usu&ntilde;";
#elif L==9	// pt
	"Tirar me";
#elif L==10	// tr
	"Remove me";	// eviri lazim!
#endif

const char *Txt_Remove_me_as_an_administrator =
#if   L==1	// ca
	"Eliminarme com a administrador/a";
#elif L==2	// de
	"Entfernen Sie mir als Administrator";
#elif L==3	// en
	"Remove me as an administrator";
#elif L==4	// es
	"Eliminarme como administrador/a";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur";
#elif L==6	// gn
	"Eliminarme como administrador/a";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator";
#elif L==9	// pt
	"Tirar me como administrador";
#elif L==10	// tr
	"Remove me as an administrator";	// eviri lazim!
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_center_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme com a administrador/a"
	" del center <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir als Administrator"
	" des Lehrinstitute <strong>%s</strong>";
#elif L==3	// en
	"Remove me as an administrator"
	" of the center <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme como administrador/a"
	" del centro <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur"
	" du center <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme como administrador/a"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me como administrador"
	" do centro <strong>%s</strong>";
#elif L==10	// tr
	"Remove me as an administrator"
	" of the center <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme com a administrador/a"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3	// en
	"Remove me as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me como administrador"
	" do grau <strong>%s</strong>";
#elif L==10	// tr
	"Remove me as an administrator"
	" of the degree <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme com a administrador/a"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3	// en
	"Remove me as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur"
	" de l'&eacute;tablissement <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator"
	" instytucji <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me como administrador"
	" da institu&ccedil;&atilde;o <strong>%s</strong>";
#elif L==10	// tr
	"Remove me as an administrator"
	" of the institution <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Remove_me_from_this_course =
#if   L==1	// ca
	"Eliminarme d'aquesta assignatura";
#elif L==2	// de
	"Entfernen Sie diesen Kurs";
#elif L==3	// en
	"Remove me from this course";
#elif L==4	// es
	"Eliminarme de esta asignatura";
#elif L==5	// fr
	"Enlever-moi de cette mati&egrave;re";
#elif L==6	// gn
	"Eliminarme de esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi da questo corso";
#elif L==8	// pl
	"Usu&ntilde; z kursu";
#elif L==9	// pt
	"Tirar me desta disciplina";
#elif L==10	// tr
	"Remove me from this course";		// eviri lazim!
#endif

const char *Txt_Remove_me_from_THE_COURSE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme de <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir aus <strong>%s</strong>";
#elif L==3	// en
	"Remove me from <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme de <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi de <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme de <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi di <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie w <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me de <strong>%s</strong>";
#elif L==10	// tr
	"Remove me from <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Remove_old_files =
#if   L==1	// ca
	"Eliminar arxius antics";
#elif L==2	// de
	"Sie alte Dateien entfernen";
#elif L==3	// en
	"Remove old files";
#elif L==4	// es
	"Eliminar archivos antiguos";
#elif L==5	// fr
	"Supprimer anciens fichiers";
#elif L==6	// gn
	"Eliminar archivos antiguos";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere vecchi file";
#elif L==8	// pl
	"Usu&nacute; stare pliki";
#elif L==9	// pt
	"Remover arquivos antigos";
#elif L==10	// tr
	"Remove old files";		// eviri lazim!
#endif

const char *Txt_Remove_the_users_not_indicated_in_step_1 =
#if   L==1	// ca
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Necessita traducci
#elif L==2	// de
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste";
#elif L==3	// en
	"Remove the users <strong>not</strong> indicated in step 1";
#elif L==4	// es
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";
#elif L==5	// fr
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste";
#elif L==6	// gn
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista";
#elif L==8	// pl
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie";
#elif L==9	// pt
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista";
#elif L==10	// tr
	"Remove the users <strong>not</strong> indicated in step 1";		// eviri lazim!
#endif

const char *Txt_Remove_the_users_indicated_in_step_1 =
#if   L==1	// ca
	"Eliminar los usuarios indicados en el paso 1";	// Necessita traducci
#elif L==2	// de
	"Entfernen Sie die Benutzer, die auf der Liste";
#elif L==3	// en
	"Remove the users indicated in step 1";
#elif L==4	// es
	"Eliminar los usuarios indicados en el paso 1";
#elif L==5	// fr
	"Enlever les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"Eliminar los usuarios indicados en el paso 1";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gli utenti che sono in lista";
#elif L==8	// pl
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Tirar os utilizadores que est&atilde;o na lista";
#elif L==10	// tr
	"Remove the users indicated in step 1";		// eviri lazim!
#endif

const char *Txt_Remove_user_from_this_course =
#if   L==1	// ca
	"Eliminar usuari d'aquesta assignatura";
#elif L==2	// de
	"Entfernen Benutzer Sie diesen Kurs";
#elif L==3	// en
	"Remove user from this course";
#elif L==4	// es
	"Eliminar usuario de esta asignatura";
#elif L==5	// fr
	"Enlever utilisateur de cette mati&egrave;re";
#elif L==6	// gn
	"Eliminar usuario de esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi utente da questo corso";
#elif L==8	// pl
	"Usu&ntilde; u&zdot;ytkownika z kursu";
#elif L==9	// pt
	"Tirar utilizador desta disciplina";
#elif L==10	// tr
	"Remove user from this course";		// eviri lazim!
#endif

const char *Txt_Remove_USER_from_this_project =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar %s d'aquest projecte";
#elif L==2	// de
	"Entfernen %s Sie diesen Projekt";
#elif L==3	// en
	"Remove %s from this project";
#elif L==4	// es
	"Eliminar %s de este proyecto";
#elif L==5	// fr
	"Enlever %s de ce projet";
#elif L==6	// gn
	"Eliminar %s de este proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi %s da questo progetto";
#elif L==8	// pl
	"Usu&ntilde; %s z projekt";
#elif L==9	// pt
	"Tirar %s deste projeto";
#elif L==10	// tr
	"Remove %s from this project";	// eviri lazim!
#endif

const char *Txt_Remove_USER_as_an_administrator =
#if   L==1	// ca
	"Eliminar com a administrador/a";
#elif L==2	// de
	"Entfernen als Administrator";
#elif L==3	// en
	"Remove as an administrator";
#elif L==4	// es
	"Eliminar como administrador/a";
#elif L==5	// fr
	"Enlever en tant qu'administrateur";
#elif L==6	// gn
	"Eliminar como administrador/a";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore";
#elif L==8	// pl
	"Usu&nacute; jako administrator";
#elif L==9	// pt
	"Tirar como administrador";
#elif L==10	// tr
	"Remove as an administrator";		// eviri lazim!
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_center_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar com a administrador/a"
	" del center <strong>%s</strong>";
#elif L==2	// de
	"Entfernen als Administrator"
	" des Lehrinstitute <strong>%s</strong>";
#elif L==3	// en
	"Remove as an administrator"
	" of the center <strong>%s</strong>";
#elif L==4	// es
	"Eliminar como administrador/a"
	" del centro <strong>%s</strong>";
#elif L==5	// fr
	"Enlever en tant qu'administrateur"
	" du center <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar como administrador/a"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9	// pt
	"Tirar como administrador"
	" do centro <strong>%s</strong>";
#elif L==10	// tr
	"Remove as an administrator"
	" of the center <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar com a administrador/a"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3	// en
	"Remove as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4	// es
	"Eliminar como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9	// pt
	"Tirar como administrador"
	" do grau <strong>%s</strong>";
#elif L==10	// tr
	"Remove as an administrator"
	" of the degree <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar com a administrador/a"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3	// en
	"Remove as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4	// es
	"Eliminar como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever en tant qu'administrateur"
	" de l'&eacute;tablissement <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; jako administrator"
	" instytucji <strong>%s</strong>";
#elif L==9	// pt
	"Tirar como administrador"
	" da institu&ccedil;&atilde;o <strong>%s</strong>";
#elif L==10	// tr
	"Remove as an administrator"
	" of the institution <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Remove_USER_from_THE_COURSE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar de <strong>%s</strong>";
#elif L==2	// de
	"Entfernen aus <strong>%s</strong>";
#elif L==3	// en
	"Remove from <strong>%s</strong>";
#elif L==4	// es
	"Eliminar de <strong>%s</strong>";
#elif L==5	// fr
	"Enlever de <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar de <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi di <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; w <strong>%s</strong>";
#elif L==9	// pt
	"Tirar de <strong>%s</strong>";
#elif L==10	// tr
	"Remove from <strong>%s</strong>";	// eviri lazim!
#endif

const char *Txt_Report =
#if   L==1	// ca
	"Informe";
#elif L==2	// de
	"Nutzungsbericht";
#elif L==3	// en
	"Report";
#elif L==4	// es
	"Informe";
#elif L==5	// fr
	"Rapport";
#elif L==6	// gn
	"Informe";	// Okoteve traduccin
#elif L==7	// it
	"Rapporto";
#elif L==8	// pl
	"Raport";
#elif L==9	// pt
	"Relat&oacute;rio";
#elif L==10	// tr
	"Bildiri";
#endif

const char *Txt_Report_of_use_of_PLATFORM =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Informe d'&uacute;s de %s";
#elif L==2	// de
	"Bericht der Nutzung der %s";
#elif L==3	// en
	"Report of use of %s";
#elif L==4	// es
	"Informe de uso de %s";
#elif L==5	// fr
	"Rapport d'utilisation de %s";
#elif L==6	// gn
	"Informe de uso de %s";	// Okoteve traduccin
#elif L==7	// it
	"Rapporto di utilizzo de %s";
#elif L==8	// pl
	"Sprawozdanie z wykorzystaniem %s";
#elif L==9	// pt
	"Relat&oacute;rio de utiliza&ccedil;&atilde;o de %s";
#elif L==10	// tr
	"Report of use of %s";	// eviri lazim!
#endif

const char *Txt_Report_possible_duplicate_user =
#if   L==1	// ca
	"Informar possible usuari duplicat";
#elif L==2	// de
	"Melden Sie die doppelte Benutzer";
#elif L==3	// en
	"Report possible duplicate user";
#elif L==4	// es
	"Informar de posible usuario duplicado";
#elif L==5	// fr
	"Signaler possible utilisateur en double";
#elif L==6	// gn
	"Informar de posible usuario duplicado";	// Okoteve traduccin
#elif L==7	// it
	"Segnalare eventuali utente duplicato";
#elif L==8	// pl
	"Zg&lstrok;o&sacute; mo&zdot;liwy duplikat u&zdot;ytkownika";
#elif L==9	// pt
	"Denunciar poss&iacute;vel usu&aacute;rio duplicado";
#elif L==10	// tr
	"Report possible duplicate user";		// eviri lazim!
#endif

const char *Txt_Requester =
#if   L==1	// ca
	"Sol&middot;licitant";
#elif L==2	// de
	"Requester";
#elif L==3	// en
	"Requester";
#elif L==4	// es
	"Solicitante";
#elif L==5	// fr
	"Demandeur";
#elif L==6	// gn
	"Solicitante";	// Okoteve traduccin
#elif L==7	// it
	"Richiedente";
#elif L==8	// pl
	"&Zdot;&aogon;daj&aogon;cego";
#elif L==9	// pt
	"Requerente";
#elif L==10	// tr
	"Requester";	// eviri lazim!
#endif

const char *Txt_Required_knowledge =
#if   L==1	// ca
	"Coneixements necessaris";
#elif L==2	// de
	"Notwendiges Wissen";
#elif L==3	// en
	"Required knowledge";
#elif L==4	// es
	"Conocimientos necesarios";
#elif L==5	// fr
	"Connaissances n&eacute;cessaires";
#elif L==6	// gn
	"Conocimientos necesarios";	// Okoteve traduccin
#elif L==7	// it
	"Nozioni necessarie";
#elif L==8	// pl
	"Niezb&eogon;dna wiedza";
#elif L==9	// pt
	"Conhecimento necess&aacute;rio";
#elif L==10	// tr
	"Required knowledge";		// eviri lazim!
#endif

const char *Txt_Required_materials =
#if   L==1	// ca
	"Materials necessaris";
#elif L==2	// de
	"Ben&ouml;tigte Materialien";
#elif L==3	// en
	"Required materials";
#elif L==4	// es
	"Materiales necesarios";
#elif L==5	// fr
	"Mat&eacute;riel n&eacute;cessaire";
#elif L==6	// gn
	"Materiales necesarios";	// Okoteve traduccin
#elif L==7	// it
	"Materiali necessari";
#elif L==8	// pl
	"Potrzebne materia&lstrok;y";
#elif L==9	// pt
	"Materiais necess&aacute;rios";
#elif L==10	// tr
	"Required materials";		// eviri lazim!
#endif

const char *Txt_Required_reading =
#if   L==1	// ca
	"Lectura obligat&ograve;ria";
#elif L==2	// de
	"Erforderliches Lesen";
#elif L==3	// en
	"Required reading";
#elif L==4	// es
	"Lectura obligatoria";
#elif L==5	// fr
	"Lecture obligatoire";
#elif L==6	// gn
	"Lectura obligatoria";	// Okoteve traduccin
#elif L==7	// it
	"Lettura obbligatoria";
#elif L==8	// pl
	"Obowi&aogon;zkowa lektura";
#elif L==9	// pt
	"Leitura obrigat&oacute;ria";
#elif L==10	// tr
	"Required reading";	// eviri lazim!
#endif

const char *Txt_Reset_survey =
#if   L==1	// ca
	"Posar a zero";
#elif L==2	// de
	"Reset";
#elif L==3	// en
	"Reset";
#elif L==4	// es
	"Poner a cero";
#elif L==5	// fr
	"Reset";
#elif L==6	// gn
	"Poner a cero";		// Okoteve traduccin
#elif L==7	// it
	"Resettare";
#elif L==8	// pl
	"Reset";
#elif L==9	// pt
	"Reiniciar";
#elif L==10	// tr
	"S&inodot;f&inodot;rla";
#endif

const char *Txt_Resource_clipboard =
#if   L==1	// ca
	"Portapapers de recursos";
#elif L==2	// de
	"Ressourcen-Zwischenablage";
#elif L==3	// en
	"Resource clipboard";
#elif L==4	// es
	"Portapapeles de recursos";
#elif L==5	// fr
	"Presse-papiers de ressources";
#elif L==6	// gn
	"Recurso portapapeles rehegua";
#elif L==7	// it
	"Appunti delle risorse";
#elif L==8	// pl
	"Schowek zasob&oacute;w";
#elif L==9	// pt
	"&Aacute;rea de transfer&ecirc;ncia de recursos";
#elif L==10	// tr
	"Kaynak panosu";
#endif

const char *Txt_Resource_clipboard_removed =
#if   L==1	// ca
	"Portapapers de recursos eliminat.";
#elif L==2	// de
	"Ressourcen-Zwischenablage entfernt.";
#elif L==3	// en
	"Resource clipboard removed.";
#elif L==4	// es
	"Portapapeles de recursos eliminado.";
#elif L==5	// fr
	"Presse-papiers de ressources supprim&eacute;.";
#elif L==6	// gn
	"Recurso portapapeles ojeipe'a.";
#elif L==7	// it
	"Appunti delle risorse rimossi.";
#elif L==8	// pl
	"Usuni&eogon;to schowek zasob&oacute;w.";
#elif L==9	// pt
	"&Aacute;rea de transfer&ecirc;ncia de recursos removida.";
#elif L==10	// tr
	"Kaynak panosu kald&inodot;r&inodot;ld&inodot;.";
#endif

const char *Txt_Resource_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Recurs <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Ressource <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Resource <strong>%s</strong> removed.";
#elif L==4	// es
	"Recurso <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Ressource <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"Recurso <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Risorsa <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Usuni&eogon;to zas&oacute;b <strong>%s</strong>.";
#elif L==9	// pt
	"Recurso <strong>%s</strong> removido.";
#elif L==10	// tr
	"Kaynak <strong>s</strong> kald&inodot;r&inodot;ld&inodot;.";
#endif

const char *Txt_RESOURCE_TYPES[Rsc_NUM_TYPES] =
	{
	[Rsc_NONE] =
#if   L==1	// ca
	"Sense enlla&ccedil;"
#elif L==2	// de
	"Keine Verbindung"
#elif L==3	// en
	"No link"
#elif L==4	// es
	"Sin enlace"
#elif L==5	// fr
	"Pas de lien"
#elif L==6	// gn
	"Sin enlace"	// Okoteve traduccin
#elif L==7	// it
	"Nessun collegamento"
#elif L==8	// pl
	"Brak &lstrok;&aogon;cza"
#elif L==9	// pt
	"Sem link"
#elif L==10	// tr
	"Ba&gbreve;lant&inodot; yok"
#endif
	,
	[Rsc_ASSIGNMENT] =
#if   L==1	// ca
	"Activitat"
#elif L==2	// de
	"Aufgabe"
#elif L==3	// en
	"Assignment"
#elif L==4	// es
	"Actividad"
#elif L==5	// fr
	"Activit&eacute;"
#elif L==6	// gn
	"Actividad"	// Okoteve traduccin
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Zadanie"
#elif L==9	// pt
	"Atividade"
#elif L==10	// tr
	"Atama"
#endif
	,
	[Rsc_PROJECT] =
#if   L==1	// ca
	"Projecte"
#elif L==2	// de
	"Projekt"
#elif L==3	// en
	"Project"
#elif L==4	// es
	"Proyecto"
#elif L==5	// fr
	"Projet"
#elif L==6	// gn
	"Apopyr&atilde;"
#elif L==7	// it
	"Progetto"
#elif L==8	// pl
	"Projekt"
#elif L==9	// pt
	"Projeto"
#elif L==10	// tr
	"Proje"
#endif
	,
	[Rsc_CALL_FOR_EXAM] =
#if   L==1	// ca
	"Convocat&ograve;ria d'examen"
#elif L==2	// de
	"Aufrufe fr Pr&uuml;fung"
#elif L==3	// en
	"Call for exam"
#elif L==4	// es
	"Convocatoria de examen"
#elif L==5	// fr
	"Convocation &agrave; un examen"
#elif L==6	// gn
	"Convocatoria de examen"	// Okoteve traduccin
#elif L==7	// it
	"Appello d'esame"
#elif L==8	// pl
	"Ogloszenie egzamin"
#elif L==9	// pt
	"Chamada para exame"
#elif L==10	// tr
	"S&inodot;nav i&ccedil;in aray&inodot;n"
#endif
	,
	[Rsc_EXAM] =
#if   L==1	// ca
	"Ex&agrave;men"
#elif L==2	// de
	"Pr&uuml;fung"
#elif L==3	// en
	"Exam"
#elif L==4	// es
	"Examen"
#elif L==5	// fr
	"Examen"
#elif L==6	// gn
	"Aranduchauka"
#elif L==7	// it
	"Esame"
#elif L==8	// pl
	"Egzamin"
#elif L==9	// pt
	"Exame"
#elif L==10	// tr
	"S&inodot;nav"
#endif
	,
	[Rsc_GAME] =
#if   L==1	// ca
	"Joc"
#elif L==2	// de
	"Spiel"
#elif L==3	// en
	"Game"
#elif L==4	// es
	"Juego"
#elif L==5	// fr
	"Jeu"
#elif L==6	// gn
	"Juego"	// Okoteve traduccin
#elif L==7	// it
	"Gioco"
#elif L==8	// pl
	"Gra"
#elif L==9	// pt
	"Jogo"
#elif L==10	// tr
	"Oyun"
#endif
	,
	[Rsc_RUBRIC] =
#if   L==1	// ca
	"R&uacute;brica"
#elif L==2	// de
	"Rubrik"
#elif L==3	// en
	"Rubric"
#elif L==4	// es
	"R&uacute;brica"
#elif L==5	// fr
	"Rubrique"
#elif L==6	// gn
	"R&uacute;brica"	// Okoteve traduccin
#elif L==7	// it
	"Rubriche"
#elif L==8	// pl
	"Rubryki"
#elif L==9	// pt
	"Rubrica"
#elif L==10	// tr
	"De&gbreve;erlendirme"
#endif
	,
	[Rsc_DOCUMENT] =
#if   L==1	// ca
	"Document"
#elif L==2	// de
	"Dokument"
#elif L==3	// en
	"Document"
#elif L==4	// es
	"Documento"
#elif L==5	// fr
	"Document"
#elif L==6	// gn
	"Terakuatia"
#elif L==7	// it
	"Documento"
#elif L==8	// pl
	"Dokument"
#elif L==9	// pt
	"Documento"
#elif L==10	// tr
	"Belge"
#endif
	,
	[Rsc_MARKS] =
#if   L==1	// ca
	"Qualificacions"	// Necessita traducci
#elif L==2	// de
	"Bewertungen"
#elif L==3	// en
	"Marks"
#elif L==4	// es
	"Calificaciones"
#elif L==5	// fr
	"Notes"
#elif L==6	// gn
	"Calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Risultati"
#elif L==8	// pl
	"Marks"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Notas"
#elif L==10	// tr
	"Marks"			// eviri lazim!
#endif
	,
	[Rsc_ATTENDANCE_EVENT] =
#if   L==1	// ca
	"Assist&egrave;ncia"
#elif L==2	// de
	"Anwesenheit"
#elif L==3	// en
	"Attendance"
#elif L==4	// es
	"Asistencia"
#elif L==5	// fr
	"Pr&eacute;sence"
#elif L==6	// gn
	"Asistencia"	// Okoteve traduccin
#elif L==7	// it
	"Presenza"
#elif L==8	// pl
	"Bytno&sacute;&cacute;"
#elif L==9	// pt
	"Presen&ccedil;a"
#elif L==10	// tr
	"Kat&inodot;l&inodot;m"
#endif
	,
	[Rsc_FORUM_THREAD] =
#if   L==1	// ca
	"Discussi&oacute; de f&ograve;rum"
#elif L==2	// de
	"Forum-Thread"
#elif L==3	// en
	"Forum thread"
#elif L==4	// es
	"Discusi&oacute;n de foro"
#elif L==5	// fr
	"Fil de forum"
#elif L==6	// gn
	"Discusi&oacute;n de foro"	// Okoteve traduccin
#elif L==7	// it
	"Discussione del forum"
#elif L==8	// pl
	"W&aogon;tek na forum"
#elif L==9	// pt
	"T&oacute;pico do f&oacute;rum"
#elif L==10	// tr
	"Forum ba&scedil;l&inodot;&gbreve;&inodot;"
#endif
	,
	[Rsc_SURVEY] =
#if   L==1	// ca
	"Enquesta"
#elif L==2	// de
	"Umfrage"
#elif L==3	// en
	"Survey"
#elif L==4	// es
	"Encuesta"
#elif L==5	// fr
	"Sondage"
#elif L==6	// gn
	"Encuesta"	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio"
#elif L==8	// pl
	"Ankieta"
#elif L==9	// pt
	"Inqu&eacute;rito"
#elif L==10	// tr
	"Anket"
#endif
	};

const char *Txt_Resources_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Recursos de %s";
#elif L==2	// de
	"%s-Ressourcen";
#elif L==3	// en
	"Resources of %s";
#elif L==4	// es
	"Recursos de %s";
#elif L==5	// fr
	"Ressources de %s";
#elif L==6	// gn
	"%s recursos rehegua";
#elif L==7	// it
	"Risorse di %s";
#elif L==8	// pl
	"Zasoby %s";
#elif L==9	// pt
	"Recursos de %s";
#elif L==10	// tr
	"%s kaynaklar&inodot;";
#endif

const char *Txt_Result =
#if   L==1	// ca
	"Resultat";
#elif L==2	// de
	"Ergebnis";
#elif L==3	// en
	"Result";
#elif L==4	// es
	"Resultado";
#elif L==5	// fr
	"R&eacute;sultat";
#elif L==6	// gn
	"Resultado";	// Okoteve traduccin
#elif L==7	// it
	"Risultato";
#elif L==8	// pl
	"Wynik";
#elif L==9	// pt
	"Resultado";
#elif L==10	// tr
	"Result";	// eviri lazim!
#endif

const char *Txt_Result_visibility =
#if   L==1	// ca
	"Visibilitat de resultats";
#elif L==2	// de
	"Ergebnis Sichtbarkeit";
#elif L==3	// en
	"Result visibility";
#elif L==4	// es
	"Visibilidad de resultados";
#elif L==5	// fr
	"Visibilit&eacute; des r&eacute;sultats";
#elif L==6	// gn
	"Visibilidad de resultados";	// Okoteve traduccin
#elif L==7	// it
	"Visibilit&agrave; dei risultati";
#elif L==8	// pl
	"Widoczno&sacute;&cacute; wynik&oacute;w";
#elif L==9	// pt
	"Visibilidade dos resultados";
#elif L==10	// tr
	"Sonu&ccedil;lar&inodot;n g&ouml;r&uuml;n&uuml;rl&uuml;&gbreve;&uuml;";
#endif

const char *Txt_Results =
#if   L==1	// ca
	"Resultats";
#elif L==2	// de
	"Ergebnisse";
#elif L==3	// en
	"Results";
#elif L==4	// es
	"Resultados";
#elif L==5	// fr
	"R&eacute;sultats";
#elif L==6	// gn
	"Resultados";	// Okoteve traduccin
#elif L==7	// it
	"Risultati";
#elif L==8	// pl
	"Wyniki";
#elif L==9	// pt
	"Resultados";
#elif L==10	// tr
	"Results";	// eviri lazim!
#endif

const char *Txt_Results_of_game_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Resultats del joc &quot;%s&quot;";
#elif L==2	// de
	"Ergebnisse von Spiel &quot;%s&quot;";
#elif L==3	// en
	"Results of game &quot;%s&quot;";
#elif L==4	// es
	"Resultados del juego &quot;%s&quot;";
#elif L==5	// fr
	"R&eacute;sultats du jeu &quot;%s&quot;";
#elif L==6	// gn
	"Resultados del juego &quot;%s&quot;";	// Okoteve traduccin
#elif L==7	// it
	"Risultati del gioco &quot;%s&quot;";
#elif L==8	// pl
	"Wyniki gry &quot;%s&quot;";
#elif L==9	// pt
	"Resultados do jogo &quot;%s&quot;";
#elif L==10	// tr
	"Results of game &quot;%s&quot;";	// eviri lazim!
#endif

const char *Txt_Results_of_exam_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Resultats del examen &quot;%s&quot;";
#elif L==2	// de
	"Ergebnisse von Pr&uuml;fung &quot;%s&quot;";
#elif L==3	// en
	"Results of exam &quot;%s&quot;";
#elif L==4	// es
	"Resultados del examen &quot;%s&quot;";
#elif L==5	// fr
	"R&eacute;sultats de l'exam &quot;%s&quot;";
#elif L==6	// gn
	"Resultados del examen &quot;%s&quot;";	// Okoteve traduccin
#elif L==7	// it
	"Risultati del esame &quot;%s&quot;";
#elif L==8	// pl
	"Wyniki egzamin &quot;%s&quot;";
#elif L==9	// pt
	"Resultados do exame &quot;%s&quot;";
#elif L==10	// tr
	"Results of exam &quot;%s&quot;";	// eviri lazim!
#endif

const char *Txt_Results_of_match_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Resultats de la partida &quot;%s&quot;";
#elif L==2	// de
	"Ergebnisse von Spiel &quot;%s&quot;";
#elif L==3	// en
	"Results of match &quot;%s&quot;";
#elif L==4	// es
	"Resultados de la partida &quot;%s&quot;";
#elif L==5	// fr
	"R&eacute;sultats du match &quot;%s&quot;";
#elif L==6	// gn
	"Resultados de la partida &quot;%s&quot;";	// Okoteve traduccin
#elif L==7	// it
	"Risultati della partita &quot;%s&quot;";
#elif L==8	// pl
	"Wyniki mecz &quot;%s&quot;";
#elif L==9	// pt
	"Resultados do jogo &quot;%s&quot;";
#elif L==10	// tr
	"Results of match &quot;%s&quot;";		// eviri lazim!
#endif

const char *Txt_Results_of_session_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Resultats de la sessi&oacute; &quot;%s&quot;";
#elif L==2	// de
	"Ergebnisse von Sitzung &quot;%s&quot;";
#elif L==3	// en
	"Results of session &quot;%s&quot;";
#elif L==4	// es
	"Resultados de la sesi&oacute;n &quot;%s&quot;";
#elif L==5	// fr
	"R&eacute;sultats de la session &quot;%s&quot;";
#elif L==6	// gn
	"Resultados de la sesi&oacute;n &quot;%s&quot;";	// Okoteve traduccin
#elif L==7	// it
	"Risultati della sessione &quot;%s&quot;";
#elif L==8	// pl
	"Wyniki sesji &quot;%s&quot;";
#elif L==9	// pt
	"Resultados da sess&atilde;o &quot;%s&quot;";
#elif L==10	// tr
	"Results of session &quot;%s&quot;";			// eviri lazim!
#endif

const char *Txt_results_per_page =
#if   L==1	// ca
	"resultats per p&agrave;gina";
#elif L==2	// de
	"Ergebnisse pro Seite";
#elif L==3	// en
	"results per page";
#elif L==4	// es
	"resultados por p&aacute;gina";
#elif L==5	// fr
	"r&eacute;sultats par page";
#elif L==6	// gn
	"resultados por p&aacute;gina";	// Okoteve traduccin
#elif L==7	// it
	"risultati per pagina";
#elif L==8	// pl
	"wynik&oacute;w na stronie";
#elif L==9	// pt
	"resultados por p&aacute;gina";
#elif L==10	// tr
	"results per page";		// eviri lazim!
#endif

const char *Txt_Resume =
#if   L==1	// ca
	"Reprendre";
#elif L==2	// de
	"Fortsetzen";
#elif L==3	// en
	"Resume";
#elif L==4	// es
	"Reanudar";
#elif L==5	// fr
	"Reprendre";
#elif L==6	// gn
	"Reanudar";	// Okoteve traduccin
#elif L==7	// it
	"Riprendi";
#elif L==8	// pl
	"Wzn&oacute;w";
#elif L==9	// pt
	"Retomar";
#elif L==10	// tr
	"Resume";	// eviri lazim!
#endif

const char *Txt_Retype_new_password =
#if   L==1	// ca
	"Reescriviu la nova contrasenya";
#elif L==2	// de
	"Wiederholung neues Passwort";
#elif L==3	// en
	"Retype new password";
#elif L==4	// es
	"Reescriba la nueva contrase&ntilde;a";
#elif L==5	// fr
	"Confirmez votre nouveau mot de passe";
#elif L==6	// gn
	"Reescriba la nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Riscrivi la nuova password";
#elif L==8	// pl
	"Powt&oacute;rz nowe has&lstrok;o";
#elif L==9	// pt
	"Redigite a nova senha";
#elif L==10	// tr
	"Yeni &scedil;ifrenizi tekrar yaz&inodot;n&inodot;z";
#endif

const char *Txt_Review =
#if   L==1	// ca
	"Revisi&oacute;";
#elif L==2	// de
	"&Uuml;berpr&uuml;fungs";
#elif L==3	// en
	"Review";
#elif L==4	// es
	"Revisi&oacute;n";
#elif L==5	// fr
	"Examen";
#elif L==6	// gn
	"Revisi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Revisione";
#elif L==8	// pl
	"Recenzja";
#elif L==9	// pt
	"Revis&atilde;o";
#elif L==10	// tr
	"G&ouml;zden ge&ccedil;ir";
#endif

const char *Txt_Role =
#if   L==1	// ca
	"Rol";
#elif L==2	// de
	"Rolle";
#elif L==3	// en
	"Role";
#elif L==4	// es
	"Rol";
#elif L==5	// fr
	"R&ocirc;le";
#elif L==6	// gn
	"Rol";	// Okoteve traduccin
#elif L==7	// it
	"Ruolo";
#elif L==8	// pl
	"Rol&eogon;";
#elif L==9	// pt
	"Papel";
#elif L==10	// tr
	"Role";	// eviri lazim!
#endif

const char *Txt_ROLE_STATS[Sta_NUM_ROLES_STAT] =
	{
	[Sta_ROLE_IDENTIFIED_USRS] =
#if   L==1	// ca
	"identificats"
#elif L==2	// de
	"Identifiziert"
#elif L==3	// en
	"identified"
#elif L==4	// es
	"identificados"
#elif L==5	// fr
	"identifi&eacute;s"
#elif L==6	// gn
	"identificados"	// Okoteve traduccin
#elif L==7	// it
	"identificati"
#elif L==8	// pl
	"zidentyfikowane"
#elif L==9	// pt
	"identificados"
#elif L==10	// tr
	"identified"	// eviri lazim!
#endif
	,
	[Sta_ROLE_ALL_USRS] =
#if   L==1	// ca
	"identificats o desconeguts"
#elif L==2	// de
	"Identifiziert oder Unbekannte"
#elif L==3	// en
	"identified or unknown"
#elif L==4	// es
	"identificados o desconocidos"
#elif L==5	// fr
	"identifi&eacute;s ou inconnus"
#elif L==6	// gn
	"identificados o desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"identificati o sconosciuti"
#elif L==8	// pl
	"zidentyfikowanej lub nieznane"
#elif L==9	// pt
	"identificados ou desconhecidos"
#elif L==10	// tr
	"identified or unknown"		// eviri lazim!
#endif
	,
	[Sta_ROLE_INS_ADMINS] =
#if   L==1	// ca
	"administradors d'institucions"
#elif L==2	// de
	"Administratoren von Hochschulen"
#elif L==3	// en
	"administrators of institutions"
#elif L==4	// es
	"administradores de instituciones"
#elif L==5	// fr
	"administrateurs d'&eacute;tablissements"
#elif L==6	// gn
	"administradores de instituciones"	// Okoteve traduccin
#elif L==7	// it
	"amministratori di istituzioni"
#elif L==8	// pl
	"Administratorzy instytucje"
#elif L==9	// pt
	"administradores de institu&ccedil;&otilde;es"
#elif L==10	// tr
	"administrators of institutions"	// eviri lazim!
#endif
	,
	[Sta_ROLE_CTR_ADMINS] =
#if   L==1	// ca
	"administradors de centers"
#elif L==2	// de
	"Administratoren von Lehrinstitute"
#elif L==3	// en
	"administrators of centers"
#elif L==4	// es
	"administradores de centros"
#elif L==5	// fr
	"administrateurs de centers"
#elif L==6	// gn
	"administradores de centros"	// Okoteve traduccin
#elif L==7	// it
	"amministratori di centri"
#elif L==8	// pl
	"Administratorzy centra"
#elif L==9	// pt
	"administradores de centros"
#elif L==10	// tr
	"administrators of centers"	// eviri lazim!
#endif
	,
	[Sta_ROLE_DEG_ADMINS] =
#if   L==1	// ca
	"administradors de titulacions"
#elif L==2	// de
	"Administratoren von Studieng&auml;nge"
#elif L==3	// en
	"administrators of degrees"
#elif L==4	// es
	"administradores de titulaciones"
#elif L==5	// fr
	"administrateurs d'&eacute;tudes"
#elif L==6	// gn
	"administradores de titulaciones"	// Okoteve traduccin
#elif L==7	// it
	"amministratori di laurea"
#elif L==8	// pl
	"Administratorzy stopni"
#elif L==9	// pt
	"administradores de graus"
#elif L==10	// tr
	"administrators of degrees"		// eviri lazim!
#endif
	,
	[Sta_ROLE_TEACHERS] =
#if   L==1	// ca
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczycieli"
#elif L==9	// pt
	"professores"
#elif L==10	// tr
	"teachers"	// eviri lazim!
#endif
	,
	[Sta_ROLE_NON_EDITING_TEACHERS] =
#if   L==1	// ca
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#elif L==10	// tr
	"non-editing teachers"		// eviri lazim!
#endif
	,
	[Sta_ROLE_STUDENTS] =
#if   L==1	// ca
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'eku&eacute;ra"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studenci"
#elif L==9	// pt
	"estudantes"
#elif L==10	// tr
	"&ouml;&gbreve;renciler"
#endif
	,
	[Sta_ROLE_USERS] =
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	,
	[Sta_ROLE_GUESTS] =
#if   L==1	// ca
	"convidats (sense assignatures)"
#elif L==2	// de
	"Eingeladen (ohne Kursen)"
#elif L==3	// en
	"guests (without courses)"
#elif L==4	// es
	"invitados (sin asignaturas)"
#elif L==5	// fr
	"invit&eacute;s (sans mati&egrave;res)"
#elif L==6	// gn
	"invitados (sin asignaturas)"	// Okoteve traduccin
#elif L==7	// it
	"ospiti (senza corsi)"
#elif L==8	// pl
	"zaproszeny (bez kurs&oacute;w)"
#elif L==9	// pt
	"convidados (sem disciplinas)"
#elif L==10	// tr
	"guests (without courses)"	// eviri lazim!
#endif
	,
	[Sta_ROLE_UNKNOWN_USRS] =
#if   L==1	// ca
	"desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Sta_ROLE_ME] =
#if   L==1	// ca
	"jo"
#elif L==2	// de
	"Mich"
#elif L==3	// en
	"me"
#elif L==4	// es
	"yo"
#elif L==5	// fr
	"moi"
#elif L==6	// gn
	"che"
#elif L==7	// it
	"me"
#elif L==8	// pl
	"ja"
#elif L==9	// pt
	"eu"
#elif L==10	// tr
	"me"	// eviri lazim!
#endif
	};

const char *Txt_ROLES_SINGUL_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
	{
	[Rol_UNK][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido/a"
#elif L==5	// fr
	"Inconnu/e"
#elif L==6	// gn
	"Desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto/a"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido/a"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Desconeguda"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocida"
#elif L==5	// fr
	"Inconnue"
#elif L==6	// gn
	"Desconocida"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuta"
#elif L==8	// pl
	"Nieznana"
#elif L==9	// pt
	"Desconhecida"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_MALE] =
#if   L==1	// ca
	"Desconegut"
#elif L==2	// de
	"Unbekannter"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_ALL] =
#if   L==1	// ca
	"Desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido/a"
#elif L==5	// fr
	"Inconnu/e"
#elif L==6	// gn
	"Desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto/a"
#elif L==8	// pl
	"Nieznani"
#elif L==9	// pt
	"Desconhecido/a"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Convidat/da"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitado/a"
#elif L==5	// fr
	"Invit&eacute;/e"
#elif L==6	// gn
	"Invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Go&sacute;&cacute;"
#elif L==9	// pt
	"Convidado/a"
#elif L==10	// tr
	"Guest"		// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Convidada"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitada"
#elif L==5	// fr
	"Invit&eacute;e"
#elif L==6	// gn
	"Invitada"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Opinia"
#elif L==9	// pt
	"Convidada"
#elif L==10	// tr
	"Guest"		// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_MALE] =
#if   L==1	// ca
	"Convidat"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitado"
#elif L==5	// fr
	"Invit&eacute;"
#elif L==6	// gn
	"Invitado"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Opinia"
#elif L==9	// pt
	"Convidado"
#elif L==10	// tr
	"Guest"		// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_ALL] =
#if   L==1	// ca
	"Convidat/da"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitado/a"
#elif L==5	// fr
	"Invit&eacute;/e"
#elif L==6	// gn
	"Invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Opinie"
#elif L==9	// pt
	"Convidado/a"
#elif L==10	// tr
	"Guest"		// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuaria"
#elif L==5	// fr
	"Utilisateuse"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizadora"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_MALE] =
#if   L==1	// ca
	"Usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario"
#elif L==5	// fr
	"Utilisateur"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_ALL] =
#if   L==1	// ca
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant/e"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Student"
#elif L==9	// pt
	"Estudante"
#elif L==10	// tr
	"Student"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiante"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studentessa"
#elif L==8	// pl
	"Studentka"
#elif L==9	// pt
	"Estudante"
#elif L==10	// tr
	"Student"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_MALE] =
#if   L==1	// ca
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Student"
#elif L==9	// pt
	"Estudante"
#elif L==10	// tr
	"Student"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_ALL] =
#if   L==1	// ca
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant/e"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudante"
#elif L==10	// tr
	"Student"	// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Professor/a no editor/a"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesor/a no editor/a"
#elif L==5	// fr
	"Enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"Professore/ssa non-editing"
#elif L==8	// pl
	"Nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professor/a n&atilde;o editor/a"
#elif L==10	// tr
	"Non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Professora no editora"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesora no editora"
#elif L==5	// fr
	"Enseignante non-&eacute;ditrice"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editora"	// Okoteve traduccin
#elif L==7	// it
	"Professoressa non-editing"
#elif L==8	// pl
	"Nauczycielka nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professora n&atilde;o editora"
#elif L==10	// tr
	"Non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_MALE] =
#if   L==1	// ca
	"Professor no editor"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesor no editor"
#elif L==5	// fr
	"Enseignant non-&eacute;diteur"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editor"	// Okoteve traduccin
#elif L==7	// it
	"Professore non-editing"
#elif L==8	// pl
	"Nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professor n&atilde;o editor"
#elif L==10	// tr
	"Non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_ALL] =
#if   L==1	// ca
	"Professor/a no editor/a"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesor/a no editor/a"
#elif L==5	// fr
	"Enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"Professore/ssa non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professor/a n&atilde;o editor/a"
#elif L==10	// tr
	"Non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesor/a"
#elif L==5	// fr
	"Enseignant/e"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore/ssa"
#elif L==8	// pl
	"Nauczyciel"
#elif L==9	// pt
	"Professor/a"
#elif L==10	// tr
	"&Ouml;&gbreve;retmen"
#endif
	,
	[Rol_TCH][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Professora"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesora"
#elif L==5	// fr
	"Enseignante"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professoressa"
#elif L==8	// pl
	"Nauczycielka"
#elif L==9	// pt
	"Professora"
#elif L==10	// tr
	"&Ouml;&gbreve;retmen"
#endif
	,
	[Rol_TCH][Usr_SEX_MALE] =
#if   L==1	// ca
	"Professor"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesor"
#elif L==5	// fr
	"Enseignant"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore"
#elif L==8	// pl
	"Nauczyciel"
#elif L==9	// pt
	"Professor"
#elif L==10	// tr
	"&Ouml;&gbreve;retmen"
#endif
	,
	[Rol_TCH][Usr_SEX_ALL] =
#if   L==1	// ca
	"Professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesor/a"
#elif L==5	// fr
	"Enseignant/e"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore/ssa"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professor/a"
#elif L==10	// tr
	"&Ouml;&gbreve;retmen"
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#elif L==10	// tr
	"Admin. degree"			// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#elif L==10	// tr
	"Admin. degree"			// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#elif L==10	// tr
	"Admin. degree"			// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#elif L==10	// tr
	"Admin. degree"			// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. center"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. center"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#elif L==10	// tr
	"Admin. center"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. center"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. center"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#elif L==10	// tr
	"Admin. center"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. center"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. center"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#elif L==10	// tr
	"Admin. center"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. center"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. center"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#elif L==10	// tr
	"Admin. center"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"Admin. institution"		// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"Admin. institution"		// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"Admin. institution"		// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"Admin. institution"		// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Superusuari/a"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuario/a"
#elif L==5	// fr
	"Superutilisateur/se"
#elif L==6	// gn
	"Superusuario/a"	// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizador/a"
#elif L==10	// tr
	"Superuser"		// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Superusuaria"	// Necessita traducci
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuaria"
#elif L==5	// fr
	"Superutilisateuse"
#elif L==6	// gn
	"Superusuaria"	// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizadora"
#elif L==10	// tr
	"Superuser"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Superusuari"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuario"
#elif L==5	// fr
	"Superutilisateur"
#elif L==6	// gn
	"Superusuario"	// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizador"
#elif L==10	// tr
	"Superuser"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Superusuari/a"	// Necessita traducci
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuario/a"
#elif L==5	// fr
	"Superutilisateur/se"
#elif L==6	// gn
	"Superusuario/a"	// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownicy"
#elif L==9	// pt
	"Superutilizador/a"
#elif L==10	// tr
	"Superuser"		// eviri lazim!
#endif
	};

const char *Txt_ROLES_SINGUL_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
	{
	[Rol_UNK][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"desconocido/a"	// Necessita traducci
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido/a"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto/a"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido/a"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"desconocida"	// Necessita traducci
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocida"
#elif L==5	// fr
	"inconnue"
#elif L==6	// gn
	"desconocida"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuta"
#elif L==8	// pl
	"nieznana"
#elif L==9	// pt
	"desconhecida"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_MALE] =
#if   L==1	// ca
	"desconocido"	// Necessita traducci
#elif L==2	// de
	"Unbekannter"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido"
#elif L==5	// fr
	"inconnu"
#elif L==6	// gn
	"desconocido"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_ALL] =
#if   L==1	// ca
	"desconocido/a"	// Necessita traducci
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido/a"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto/a"
#elif L==8	// pl
	"nieznani"
#elif L==9	// pt
	"desconhecido/a"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"invitado/a"	// Necessita traducci
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitado/a"
#elif L==5	// fr
	"invit&eacute;/e"
#elif L==6	// gn
	"invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"go&sacute;&cacute;"
#elif L==9	// pt
	"convidado/a"
#elif L==10	// tr
	"guest"		// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"invitada"	// Necessita traducci
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitada"
#elif L==5	// fr
	"invit&eacute;e"
#elif L==6	// gn
	"invitada"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"goscinna"
#elif L==9	// pt
	"convidada"
#elif L==10	// tr
	"guest"		// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_MALE] =
#if   L==1	// ca
	"invitado"	// Necessita traducci
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitado"
#elif L==5	// fr
	"invit&eacute;"
#elif L==6	// gn
	"invitado"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"goscinny"
#elif L==9	// pt
	"convidado"
#elif L==10	// tr
	"guest"		// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_ALL] =
#if   L==1	// ca
	"invitado/a"	// Necessita traducci
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitado/a"
#elif L==5	// fr
	"invit&eacute;/e"
#elif L==6	// gn
	"invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"goscinni"
#elif L==9	// pt
	"convidado/a"
#elif L==10	// tr
	"guest"		// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuaria"
#elif L==5	// fr
	"utilisateuse"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizadora"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_MALE] =
#if   L==1	// ca
	"usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario"
#elif L==5	// fr
	"utilisateur"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_ALL] =
#if   L==1	// ca
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant/e"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiante"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studentessa"
#elif L==8	// pl
	"studentka"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_MALE] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_ALL] =
#if   L==1	// ca
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant/e"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente/ssa"
#elif L==8	// pl
	"studenci"
#elif L==9	// pt
	"estudante"
#elif L==10	// tr
	"student"	// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"professor/a no editor/a"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesor/a no editor/a"
#elif L==5	// fr
	"enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"professore/ssa non-editing"
#elif L==8	// pl
	"nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"professor/a n&atilde;o editor/a"
#elif L==10	// tr
	"non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"professora no editora"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesora no editora"
#elif L==5	// fr
	"enseignante non-&eacute;ditrice"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editora"	// Okoteve traduccin
#elif L==7	// it
	"professoressa non-editing"
#elif L==8	// pl
	"nauczycielka nie edytuj&aogon;cy"
#elif L==9	// pt
	"professora n&atilde;o editora"
#elif L==10	// tr
	"non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_MALE] =
#if   L==1	// ca
	"professor no editor"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesor no editor"
#elif L==5	// fr
	"enseignant non-&eacute;diteur"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editor"	// Okoteve traduccin
#elif L==7	// it
	"professore non-editing"
#elif L==8	// pl
	"nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"professor n&atilde;o editor"
#elif L==10	// tr
	"non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_ALL] =
#if   L==1	// ca
	"professor/a no editor/a"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesor/a no editor/a"
#elif L==5	// fr
	"enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"professore/ssa non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professor/a n&atilde;o editor/a"
#elif L==10	// tr
	"non-editing teacher"		// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesor/a"
#elif L==5	// fr
	"enseignant/e"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore/ssa"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"professor/a"
#elif L==10	// tr
	"teacher"	// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"professora"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesora"
#elif L==5	// fr
	"enseignante"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"Professoressa"
#elif L==8	// pl
	"nauczycielka"
#elif L==9	// pt
	"professora"
#elif L==10	// tr
	"teacher"	// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_MALE] =
#if   L==1	// ca
	"professor"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesor"
#elif L==5	// fr
	"enseignant"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professore"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"professor"
#elif L==10	// tr
	"teacher"	// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_ALL] =
#if   L==1	// ca
	"professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesor/a"
#elif L==5	// fr
	"enseignant/e"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professore/ssa"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professor/a"
#elif L==10	// tr
	"teacher"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"admin. titulaci&oacute;n"	// Necessita traducci
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#elif L==10	// tr
	"admin. degree"			// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"admin. titulaci&oacute;n"	// Necessita traducci
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#elif L==10	// tr
	"admin. degree"			// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"admin. titulaci&oacute;n"	// Necessita traducci
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#elif L==10	// tr
	"admin. degree"			// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"admin. titulaci&oacute;n"	// Necessita traducci
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#elif L==10	// tr
	"admin. degree"			// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. center"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. center"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#elif L==10	// tr
	"admin. center"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. center"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. center"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#elif L==10	// tr
	"admin. center"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. center"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. center"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#elif L==10	// tr
	"admin. center"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"admin. center"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. center"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. center"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#elif L==10	// tr
	"admin. center"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"admin. center"			// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"admin. institution"		// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"admin. institution"		// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#elif L==10	// tr
	"admin. institution"		// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"superusuari/a"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuario/a"
#elif L==5	// fr
	"superutilisateur/se"
#elif L==6	// gn
	"superusuario/a"	// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superuzytkownik"
#elif L==9	// pt
	"superutilizador/a"
#elif L==10	// tr
	"superuser"		// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"superusuaria"		// Necessita traducci
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuaria"
#elif L==5	// fr
	"superutilisateuse"
#elif L==6	// gn
	"superusuaria"	// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superuzytkowniczka"
#elif L==9	// pt
	"superutilizadora"
#elif L==10	// tr
	"superuser"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"superusuari"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuario"
#elif L==5	// fr
	"superutilisateur"
#elif L==6	// gn
	"superusuario"	// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superu&zdot;ytkownika"
#elif L==9	// pt
	"superutilizador"
#elif L==10	// tr
	"superuser"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"superusuari/a"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuario/a"
#elif L==5	// fr
	"superutilisateur/se"
#elif L==6	// gn
	"superusuario/a"	// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superuzytkownicy"
#elif L==9	// pt
	"superutilizador/a"
#elif L==10	// tr
	"superuser"		// eviri lazim!
#endif
	};

const char *Txt_ROLES_PLURAL_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
	{
	[Rol_UNK][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecidos"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Desconegudes"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidas"
#elif L==5	// fr
	"Inconnues"
#elif L==6	// gn
	"Desconocidas"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznana"
#elif L==9	// pt
	"Desconhecidas"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_MALE] =
#if   L==1	// ca
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecidos"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_ALL] =
#if   L==1	// ca
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznani"
#elif L==9	// pt
	"Desconhecidos"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitados"
#elif L==5	// fr
	"Invit&eacute;s"
#elif L==6	// gn
	"Invitados"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszenie"
#elif L==9	// pt
	"Convidados"
#elif L==10	// tr
	"Guests"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Convidadas"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitadas"
#elif L==5	// fr
	"Invit&eacute;es"
#elif L==6	// gn
	"Invitadas"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszona"
#elif L==9	// pt
	"Convidadas"
#elif L==10	// tr
	"Guests"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_MALE] =
#if   L==1	// ca
	"Convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitados"
#elif L==5	// fr
	"Invit&eacute;s"
#elif L==6	// gn
	"Invitados"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszony"
#elif L==9	// pt
	"Convidados"
#elif L==10	// tr
	"Guests"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_ALL] =
#if   L==1	// ca
	"Convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitados"
#elif L==5	// fr
	"Invit&eacute;s"
#elif L==6	// gn
	"Invitados"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszeni"
#elif L==9	// pt
	"Convidados"
#elif L==10	// tr
	"Guests"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarios"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Puruh&aacute;ra"
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"U&zdot;ytkownicy"
#elif L==9	// pt
	"Utilizadores"
#elif L==10	// tr
	"Users"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Usu&agrave;ries"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarias"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Puruh&aacute;ra"
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"U&zdot;ytkownicy"
#elif L==9	// pt
	"Utilizadoras"
#elif L==10	// tr
	"Users"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_MALE] =
#if   L==1	// ca
	"Usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarios"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Puruh&aacute;ra"
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"U&zdot;ytkownicy"
#elif L==9	// pt
	"Utilizadores"
#elif L==10	// tr
	"Users"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_ALL] =
#if   L==1	// ca
	"Usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarios"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Puruh&aacute;ra"
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"U&zdot;ytkownicy"
#elif L==9	// pt
	"Utilizadores"
#elif L==10	// tr
	"Users"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'eku&eacute;ra"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#elif L==10	// tr
	"&Ouml;&gbreve;renciler"
#endif
	,
	[Rol_STD][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiantes"
#elif L==6	// gn
	"Temimbo'eku&eacute;ra"
#elif L==7	// it
	"Studentesse"
#elif L==8	// pl
	"Studentki"
#elif L==9	// pt
	"Estudantes"
#elif L==10	// tr
	"&Ouml;&gbreve;renciler"
#endif
	,
	[Rol_STD][Usr_SEX_MALE] =
#if   L==1	// ca
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'eku&eacute;ra"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#elif L==10	// tr
	"&Ouml;&gbreve;renciler"
#endif
	,
	[Rol_STD][Usr_SEX_ALL] =
#if   L==1	// ca
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'eku&eacute;ra"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#elif L==10	// tr
	"&Ouml;&gbreve;renciler"
#endif
	,
	[Rol_NET][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Professors no editors"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesores no editores"
#elif L==5	// fr
	"Enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"Professori non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professores n&atilde;o editores"
#elif L==10	// tr
	"Non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Professores no editores"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesoras no editoras"
#elif L==5	// fr
	"Enseignantes non-&eacute;ditrices"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editoras"	// Okoteve traduccin
#elif L==7	// it
	"Professoresse non-editing"
#elif L==8	// pl
	"Nauczycielki nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professoras n&atilde;o editoras"
#elif L==10	// tr
	"Non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_MALE] =
#if   L==1	// ca
	"Professors no editors"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesores no editores"
#elif L==5	// fr
	"Enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"Professori non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professores n&atilde;o editores"
#elif L==10	// tr
	"Non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_ALL] =
#if   L==1	// ca
	"Professors no editors"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesores no editores"
#elif L==5	// fr
	"Enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"Professori non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professores n&atilde;o editores"
#elif L==10	// tr
	"Non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	,
	[Rol_TCH][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Professores"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesoras"
#elif L==5	// fr
	"Enseignantes"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professoresse"
#elif L==8	// pl
	"Nauczycielki"
#elif L==9	// pt
	"Professoras"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	,
	[Rol_TCH][Usr_SEX_MALE] =
#if   L==1	// ca
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	,
	[Rol_TCH][Usr_SEX_ALL] =
#if   L==1	// ca
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#elif L==10	// tr
	"&Ouml;&gbreve;retmenler"
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Administradoras"	// Necessita traducci
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradoras"
#elif L==5	// fr
	"Administrateuses"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorki"
#elif L==9	// pt
	"Administradoras"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Administradoras"	// Necessita traducci
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradoras"
#elif L==5	// fr
	"Administrateuses"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorki"
#elif L==9	// pt
	"Administradoras"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Administradoras"	// Necessita traducci
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradoras"
#elif L==5	// fr
	"Administrateuses"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorki"
#elif L==9	// pt
	"Administradoras"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#elif L==10	// tr
	"Administrators"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarios"
#elif L==5	// fr
	"Superutilisateurs"
#elif L==6	// gn
	"Superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superu&zdot;ytkownika"
#elif L==9	// pt
	"Superutilizadores"
#elif L==10	// tr
	"Superusers"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Superusuarias"	// Necessita traducci
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarias"
#elif L==5	// fr
	"Superutilisateuses"
#elif L==6	// gn
	"Superusuarias"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superuzytkowniczka"
#elif L==9	// pt
	"Superutilizadoras"
#elif L==10	// tr
	"Superusers"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"Superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarios"
#elif L==5	// fr
	"Superutilisateurs"
#elif L==6	// gn
	"Superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizadores"
#elif L==10	// tr
	"Superusers"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"Superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarios"
#elif L==5	// fr
	"Superutilisateurs"
#elif L==6	// gn
	"Superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superuzytkownicy"
#elif L==9	// pt
	"Superutilizadores"
#elif L==10	// tr
	"Superusers"	// eviri lazim!
#endif
	};

const char *Txt_ROLES_PLURAL_BRIEF_Abc[Rol_NUM_ROLES] =
	{
	[Rol_UNK] =
#if   L==1	// ca
	"Desc."
#elif L==2	// de
	"Unbe."
#elif L==3	// en
	"Unkn."
#elif L==4	// es
	"Desc."
#elif L==5	// fr
	"Inco."
#elif L==6	// gn
	"Desc."	// Okoteve traduccin
#elif L==7	// it
	"Scon."
#elif L==8	// pl
	"Niez."
#elif L==9	// pt
	"Desc."
#elif L==10	// tr
	"Unkn."	// eviri lazim!
#endif
	,
	[Rol_GST] =
#if   L==1	// ca
	"Conv."
#elif L==2	// de
	"Eing."
#elif L==3	// en
	"Gues."
#elif L==4	// es
	"Invi."
#elif L==5	// fr
	"Invi."
#elif L==6	// gn
	"Invi."	// Okoteve traduccin
#elif L==7	// it
	"Ospi."
#elif L==8	// pl
	"Zapr."
#elif L==9	// pt
	"Conv."
#elif L==10	// tr
	"Gues."	// eviri lazim!
#endif
	,
	[Rol_USR] =
#if   L==1	// ca
	"Usua."
#elif L==2	// de
	"Benu."
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usua."
#elif L==5	// fr
	"Util."
#elif L==6	// gn
	"Puru."
#elif L==7	// it
	"Uten."
#elif L==8	// pl
	"U&zdot;yt."
#elif L==9	// pt
	"Util."
#elif L==10	// tr
	"Users"	// eviri lazim!
#endif
	,
	[Rol_STD] =
#if   L==1	// ca
	"Estu."
#elif L==2	// de
	"Stud."
#elif L==3	// en
	"Stud."
#elif L==4	// es
	"Estu."
#elif L==5	// fr
	"&Eacute;tu."
#elif L==6	// gn
	"Temi."
#elif L==7	// it
	"Stud."
#elif L==8	// pl
	"Stud."
#elif L==9	// pt
	"Estu."
#elif L==10	// tr
	"Stud."	// eviri lazim!
#endif
	,
	[Rol_NET] =
#if   L==1	// ca
	"Prof.n.e."
#elif L==2	// de
	"N.b.Lehr."
#elif L==3	// en
	"N.e.teac."
#elif L==4	// es
	"Prof.n.e."
#elif L==5	// fr
	"Ense.n.&eacute;."
#elif L==6	// gn
	"Mbo'e.n.e."	// Okoteve traduccin
#elif L==7	// it
	"Prof.n.e."
#elif L==8	// pl
	"Nauc.n.e."
#elif L==9	// pt
	"Prof.n.e."
#elif L==10	// tr
	"N.e.teac."	// eviri lazim!
#endif
	,
	[Rol_TCH] =
#if   L==1	// ca
	"Prof."
#elif L==2	// de
	"Lehr."
#elif L==3	// en
	"Teac."
#elif L==4	// es
	"Prof."
#elif L==5	// fr
	"Ense."
#elif L==6	// gn
	"Mbo'e."
#elif L==7	// it
	"Prof."
#elif L==8	// pl
	"Nauc."
#elif L==9	// pt
	"Prof."
#elif L==10	// tr
	"Teac."	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM] =
#if   L==1	// ca
	"Admi."
#elif L==2	// de
	"Admi."
#elif L==3	// en
	"Admi."
#elif L==4	// es
	"Admi."
#elif L==5	// fr
	"Admi."
#elif L==6	// gn
	"&Ntilde;ang."
#elif L==7	// it
	"Ammi."
#elif L==8	// pl
	"Admi."
#elif L==9	// pt
	"Admi."
#elif L==10	// tr
	"Admi."	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM] =
#if   L==1	// ca
	"Admi."
#elif L==2	// de
	"Admi."
#elif L==3	// en
	"Admi."
#elif L==4	// es
	"Admi."
#elif L==5	// fr
	"Admi."
#elif L==6	// gn
	"&Ntilde;ang."
#elif L==7	// it
	"Ammi."
#elif L==8	// pl
	"Admi."
#elif L==9	// pt
	"Admi."
#elif L==10	// tr
	"Admi."	// eviri lazim!
#endif
	,
	[Rol_INS_ADM] =
#if   L==1	// ca
	"Admi."
#elif L==2	// de
	"Admi."
#elif L==3	// en
	"Admi."
#elif L==4	// es
	"Admi."
#elif L==5	// fr
	"Admi."
#elif L==6	// gn
	"&Ntilde;ang."
#elif L==7	// it
	"Ammi."
#elif L==8	// pl
	"Admi."
#elif L==9	// pt
	"Admi."
#elif L==10	// tr
	"Admi."	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM] =
#if   L==1	// ca
	"Supe."
#elif L==2	// de
	"Supe."
#elif L==3	// en
	"Supe."
#elif L==4	// es
	"Supe."
#elif L==5	// fr
	"Supe."
#elif L==6	// gn
	"Supe."	// Okoteve traduccin
#elif L==7	// it
	"Supe."
#elif L==8	// pl
	"Supe."
#elif L==9	// pt
	"Supe."
#elif L==10	// tr
	"Supe."	// eviri lazim!
#endif
	};

const char *Txt_ROLES_PLURAL_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
	{
	[Rol_UNK][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"desconocidos"	// Necessita traducci
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"desconocidas"	// Necessita traducci
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidas"
#elif L==5	// fr
	"inconnues"
#elif L==6	// gn
	"desconocidas"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznana"
#elif L==9	// pt
	"desconhecidas"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_MALE] =
#if   L==1	// ca
	"desconocidos"	// Necessita traducci
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_UNK][Usr_SEX_ALL] =
#if   L==1	// ca
	"desconocidos"	// Necessita traducci
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznani"
#elif L==9	// pt
	"desconhecidos"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitados"
#elif L==5	// fr
	"invit&eacute;s"
#elif L==6	// gn
	"invitados"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszenie"
#elif L==9	// pt
	"convidados"
#elif L==10	// tr
	"guests"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"invitadas"	// Necessita traducci
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitadas"
#elif L==5	// fr
	"invit&eacute;es"
#elif L==6	// gn
	"invitadas"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszona"
#elif L==9	// pt
	"convidadas"
#elif L==10	// tr
	"guests"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_MALE] =
#if   L==1	// ca
	"convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitados"
#elif L==5	// fr
	"invit&eacute;s"
#elif L==6	// gn
	"invitados"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszony"
#elif L==9	// pt
	"convidados"
#elif L==10	// tr
	"guests"	// eviri lazim!
#endif
	,
	[Rol_GST][Usr_SEX_ALL] =
#if   L==1	// ca
	"convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitados"
#elif L==5	// fr
	"invit&eacute;s"
#elif L==6	// gn
	"invitados"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszeni"
#elif L==9	// pt
	"convidados"
#elif L==10	// tr
	"guests"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"usu&agrave;ries"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarias"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadoras"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_MALE] =
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	,
	[Rol_USR][Usr_SEX_ALL] =
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'eku&eacute;ra"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studenci"
#elif L==9	// pt
	"estudantes"
#elif L==10	// tr
	"students"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiantes"
#elif L==6	// gn
	"temimbo'eku&eacute;ra"
#elif L==7	// it
	"studentesse"
#elif L==8	// pl
	"studenka"
#elif L==9	// pt
	"estudantes"
#elif L==10	// tr
	"students"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_MALE] =
#if   L==1	// ca
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'eku&eacute;ra"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studentci"
#elif L==9	// pt
	"estudantes"
#elif L==10	// tr
	"students"	// eviri lazim!
#endif
	,
	[Rol_STD][Usr_SEX_ALL] =
#if   L==1	// ca
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'eku&eacute;ra"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studentci"
#elif L==9	// pt
	"estudantes"
#elif L==10	// tr
	"students"	// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#elif L==10	// tr
	"non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"professores no editores"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesoras no editoras"
#elif L==5	// fr
	"enseignantes non-&eacute;ditrices"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editoras"	// Okoteve traduccin
#elif L==7	// it
	"professoresse non-editing"
#elif L==8	// pl
	"nauczycielki nie edytuj&aogon;cy"
#elif L==9	// pt
	"professoras n&atilde;o editoras"
#elif L==10	// tr
	"non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_MALE] =
#if   L==1	// ca
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#elif L==10	// tr
	"non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_NET][Usr_SEX_ALL] =
#if   L==1	// ca
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#elif L==10	// tr
	"non-editing teachers"		// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professores"
#elif L==10	// tr
	"teachers"	// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"professores"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesoras"
#elif L==5	// fr
	"enseignantes"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professoresse"
#elif L==8	// pl
	"nauczycielki"
#elif L==9	// pt
	"professoras"
#elif L==10	// tr
	"teachers"	// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_MALE] =
#if   L==1	// ca
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professores"
#elif L==10	// tr
	"teachers"	// eviri lazim!
#endif
	,
	[Rol_TCH][Usr_SEX_ALL] =
#if   L==1	// ca
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professores"
#elif L==10	// tr
	"teachers"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"administradoras"	// Necessita traducci
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradoras"
#elif L==5	// fr
	"administrateuses"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorki"
#elif L==9	// pt
	"administradoras"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_DEG_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"administradoras"	// Necessita traducci
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradoras"
#elif L==5	// fr
	"administrateuses"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorki"
#elif L==9	// pt
	"administradoras"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_CTR_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"administradoras"	// Necessita traducci
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradoras"
#elif L==5	// fr
	"administrateuses"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorki"
#elif L==9	// pt
	"administradoras"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_INS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#elif L==10	// tr
	"administrators"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarios"
#elif L==5	// fr
	"superutilisateurs"
#elif L==6	// gn
	"superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadores"
#elif L==10	// tr
	"superusers"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_FEMALE] =
#if   L==1	// ca
	"superusuarias"	// Necessita traducci
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarias"
#elif L==5	// fr
	"superutilisateuses"
#elif L==6	// gn
	"superusuarias"	// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadoras"
#elif L==10	// tr
	"superusers"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_MALE] =
#if   L==1	// ca
	"superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarios"
#elif L==5	// fr
	"superutilisateurs"
#elif L==6	// gn
	"superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadores"
#elif L==10	// tr
	"superusers"	// eviri lazim!
#endif
	,
	[Rol_SYS_ADM][Usr_SEX_ALL] =
#if   L==1	// ca
	"superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarios"
#elif L==5	// fr
	"superutilisateurs"
#elif L==6	// gn
	"superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadores"
#elif L==10	// tr
	"superusers"	// eviri lazim!
#endif
	};

const char *Txt_Room =
#if   L==1	// ca
	"Sala";
#elif L==2	// de
	"Raum";
#elif L==3	// en
	"Room";
#elif L==4	// es
	"Sala";
#elif L==5	// fr
	"Salle";
#elif L==6	// gn
	"Mbo'ehakoty";
#elif L==7	// it
	"Aula";
#elif L==8	// pl
	"Klasa";
#elif L==9	// pt
	"Sala";
#elif L==10	// tr
	"Room";	// eviri lazim!
#endif

const char *Txt_Room_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Sala <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Raum <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Room <strong>%s</strong> removed.";
#elif L==4	// es
	"Sala <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Salle <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"Sala <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Aula <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Klasa <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Sala <strong>%s</strong> removida.";
#elif L==10	// tr
	"Room <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_ROOMS_HELP_ORDER[Roo_NUM_ORDERS] =
	{
	[Roo_ORDER_BY_BUILDING] =
#if   L==1	// ca
	"Ordenar per edifici"
#elif L==2	// de
	"Nach Geb&auml;ude sortieren"
#elif L==3	// en
	"Sort by building"
#elif L==4	// es
	"Ordenar por edificio"
#elif L==5	// fr
	"Trier par b&acirc;timent"
#elif L==6	// gn
	"Ordenar por edificio"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per edificio"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug budynek"
#elif L==9	// pt
	"Classificar por edif&iacute;cio"
#elif L==10	// tr
	"Sort by building"	// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_FLOOR] =
#if   L==1	// ca
	"Ordenar per planta"
#elif L==2	// de
	"Nach Stock sortieren"
#elif L==3	// en
	"Sort by floor"
#elif L==4	// es
	"Ordenar por planta"
#elif L==5	// fr
	"Trier par &eacute;tage"
#elif L==6	// gn
	"Ordenar por planta"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per piano"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug pi&eogon;trze"
#elif L==9	// pt
	"Classificar por andar"
#elif L==10	// tr
	"Sort by floor"		// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_TYPE] =
#if   L==1	// ca
	"Ordenar per tipus"
#elif L==2	// de
	"Nach Art sortieren"
#elif L==3	// en
	"Sort by type"
#elif L==4	// es
	"Ordenar por tipo"
#elif L==5	// fr
	"Trier par type"
#elif L==6	// gn
	"Ordenar por tipo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per tipo"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug typ"
#elif L==9	// pt
	"Classificar por tipo"
#elif L==10	// tr
	"Sort by type"		// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_SHRT_NAME] =
#if   L==1	// ca
	"Ordenar per nom breu"
#elif L==2	// de
	"Nach Kurzname sortieren"
#elif L==3	// en
	"Sort by short name"
#elif L==4	// es
	"Ordenar por nombre breve"
#elif L==5	// fr
	"Trier par nom abr&eacute;g&eacute;"
#elif L==6	// gn
	"Ordenar por nombre breve"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per nome breve"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug kr&oacute;tkiej nazwy"
#elif L==9	// pt
	"Classificar por nome abreviado"
#elif L==10	// tr
	"Sort by short name"		// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_FULL_NAME] =
#if   L==1	// ca
	"Ordenar per nom complet"
#elif L==2	// de
	"Nach vollst&auml;ndiger Name sortieren"
#elif L==3	// en
	"Sort by full name"
#elif L==4	// es
	"Ordenar por nombre completo"
#elif L==5	// fr
	"Trier par nom complet"
#elif L==6	// gn
	"Ordenar por nombre completo"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per nome completo"
#elif L==8	// pl
	"Sortuj wed&lstrok;ug pe&lstrok;na nazwa"
#elif L==9	// pt
	"Classificar por nome completo"
#elif L==10	// tr
	"Sort by full name"		// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_CAPACITY] =
#if   L==1	// ca
	"Ordenar per aforo"
#elif L==2	// de
	"Sortieren nach Sitzplatzkapazit&auml;t"
#elif L==3	// en
	"Sort by seating capacity"
#elif L==4	// es
	"Ordenar por aforo"
#elif L==5	// fr
	"Trier par nombre de places"
#elif L==6	// gn
	"Ordenar por aforo"		// Okoteve traduccin
#elif L==7	// it
	"Ordina per capacit&agrave; di posti"
#elif L==8	// pl
	"Sortuj wed&lsgtrok;ug pojemno&sacute;ci miejsc"
#elif L==9	// pt
	"Ordenar por capacidade"
#elif L==10	// tr
	"Sort by seating capacity"	// eviri lazim!
#endif
  };

const char *Txt_ROOMS_ORDER[Roo_NUM_ORDERS] =
	{
	[Roo_ORDER_BY_BUILDING] =
#if   L==1	// ca
	"Edifici"
#elif L==2	// de
	"Geb&auml;ude"
#elif L==3	// en
	"Building"
#elif L==4	// es
	"Edificio"
#elif L==5	// fr
	"B&acirc;timent"
#elif L==6	// gn
	"Edificio"	// Okoteve traduccin
#elif L==7	// it
	"Edificio"
#elif L==8	// pl
	"Budynek"
#elif L==9	// pt
	"Edif&iacute;cio"
#elif L==10	// tr
	"Building"	// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_FLOOR] =
#if   L==1	// ca
	"Planta"
#elif L==2	// de
	"Stock"
#elif L==3	// en
	"Floor"
#elif L==4	// es
	"Planta"
#elif L==5	// fr
	"&Eacute;tage"
#elif L==6	// gn
	"Planta"	// Okoteve traduccin
#elif L==7	// it
	"Piano"
#elif L==8	// pl
	"Pi&eogon;trze"
#elif L==9	// pt
	"Andar"
#elif L==10	// tr
	"Floor"		// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_TYPE] =
#if   L==1	// ca
	"Tipus"
#elif L==2	// de
	"Art"
#elif L==3	// en
	"Type"
#elif L==4	// es
	"Tipo"
#elif L==5	// fr
	"Type"
#elif L==6	// gn
	"Teko"
#elif L==7	// it
	"Tipo"
#elif L==8	// pl
	"Typ"
#elif L==9	// pt
	"Tipo"
#elif L==10	// tr
	"Type"	// eviri lazim!
#endif
	,
	[Roo_ORDER_BY_SHRT_NAME] =
#if   L==1	// ca
	"Nom breu"
#elif L==2	// de
	"Kurzname"
#elif L==3	// en
	"Short name"
#elif L==4	// es
	"Nombre breve"
#elif L==5	// fr
	"Nom abr&eacute;g&eacute;"
#elif L==6	// gn
	"Nombre breve"	// Okoteve traduccin
#elif L==7	// it
	"Nome breve"
#elif L==8	// pl
	"Kr&oacute;tkiej nazwy"
#elif L==9	// pt
	"Nome abreviado"
#elif L==10	// tr
	"K&inodot;sa isim"
#endif
	,
	[Roo_ORDER_BY_FULL_NAME] =
#if   L==1	// ca
	"Nom complet"
#elif L==2	// de
	"Vollst&auml;ndiger Name"
#elif L==3	// en
	"Full name"
#elif L==4	// es
	"Nombre completo"
#elif L==5	// fr
	"Nom complet"
#elif L==6	// gn
	"Nombre completo"	// Okoteve traduccin
#elif L==7	// it
	"Nome completo"
#elif L==8	// pl
	"Pe&lstrok;na nazwa"
#elif L==9	// pt
	"Nome completo"
#elif L==10	// tr
	"Tam isim"
#endif
	,
	[Roo_ORDER_BY_CAPACITY] =
#if   L==1	// ca
	"Aforo"
#elif L==2	// de
	"Kapazit&auml;t"
#elif L==3	// en
	"Capacity"
#elif L==4	// es
	"Aforo"
#elif L==5	// fr
	"Capacit&eacute;"
#elif L==6	// gn
	"Aforo"		// Okoteve traduccin
#elif L==7	// it
	"Capacit&agrave;"
#elif L==8	// pl
	"Pojemno&sacute;&cacute;"
#elif L==9	// pt
	"Capacidade"
#elif L==10	// tr
	"Capacity"	// eviri lazim!
#endif
	};

const char *Txt_ROOM_TYPES[Roo_NUM_TYPES] =
	{
	[Roo_NO_TYPE] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"No type"
#elif L==4	// es
	"Sin tipo"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Sin tipo"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"No type"	// eviri lazim!
#endif
	,
	[Roo_ADMINISTRATION] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Administration/Management"
#elif L==4	// es
	"Administraci&oacute;n/Direcci&oacute;n"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Administraci&oacute;n/Direcci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Administration/Management"	// eviri lazim!
#endif
	,
	[Roo_AUDITORIUM] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Auditorium"
#elif L==4	// es
	"Auditorio"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Auditorio"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Auditorium"	// eviri lazim!
#endif
	,
	[Roo_CAFETERIA] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Cafeteria"
#elif L==4	// es
	"Cafeter&iacute;a"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Cafeter&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Cafeteria"		// eviri lazim!
#endif
	,
	[Roo_CANTEEN] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Canteen"
#elif L==4	// es
	"Comedor"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Comedor"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Canteen"	// eviri lazim!
#endif
	,
	[Roo_CLASSROOM] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Classroom"
#elif L==4	// es
	"Aula"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Aula"		// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Classroom"	// eviri lazim!
#endif
	,
	[Roo_CONCIERGE] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Concierge"
#elif L==4	// es
	"Conserjer&iacute;a/Porter&iacute;a"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Conserjer&iacute;a/Porter&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Concierge"				// eviri lazim!
#endif
	,
	[Roo_CORRIDOR] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Corridor"
#elif L==4	// es
	"Pasillo"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Pasillo"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Corridor"	// eviri lazim!
#endif
	,
	[Roo_GYM] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Gym"
#elif L==4	// es
	"Gimnasio"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Gimnasio"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Gym"		// eviri lazim!
#endif
	,
	[Roo_HALL] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Hall"
#elif L==4	// es
	"Vest&iacute;bulo"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Vest&iacute;bulo"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Hall"			// eviri lazim!
#endif
	,
	[Roo_KINDERGARTEN] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Kindergarten"
#elif L==4	// es
	"Guarder&iacute;a"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Guarder&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Kindergarten"		// eviri lazim!
#endif
	,
	[Roo_LABORATORY] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Laboratory"
#elif L==4	// es
	"Laboratorio"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Laboratorio"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Laboratory"	// eviri lazim!
#endif
	,
	[Roo_LIBRARY] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Library"
#elif L==4	// es
	"Biblioteca"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Biblioteca"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Library"	// eviri lazim!
#endif
	,
	[Roo_OFFICE] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Office"
#elif L==4	// es
	"Despacho/Oficina"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Despacho/Oficina"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Office"		// eviri lazim!
#endif
	,
	[Roo_OUTDOORS] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Outdoors"
#elif L==4	// es
	"Exteriores"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Exteriores"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Outdoors"	// eviri lazim!
#endif
	,
	[Roo_PARKING] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Parking"
#elif L==4	// es
	"Aparcamiento"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Aparcamiento"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Parking"	// eviri lazim!
#endif
	,
	[Roo_PAVILION] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Pavilion"
#elif L==4	// es
	"Pabell&oacute;n"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Pabell&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Pavilion	"	// eviri lazim!
#endif
	,
	[Roo_ROOM] =
#if   L==1	// ca
	"Sala"
#elif L==2	// de
	"Raum"
#elif L==3	// en
	"Room"
#elif L==4	// es
	"Sala"
#elif L==5	// fr
	"Salle"
#elif L==6	// gn
	"Mbo'ehakoty"
#elif L==7	// it
	"Aula"
#elif L==8	// pl
	"Klasa"
#elif L==9	// pt
	"Sala"
#elif L==10	// tr
	"Room"	// eviri lazim!
#endif
	,
	[Roo_SECRETARIAT] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Secretariat"
#elif L==4	// es
	"Secretar&iacute;a"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Secretar&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Secretariat"		// eviri lazim!
#endif
	,
	[Roo_SEMINAR] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Seminar"
#elif L==4	// es
	"Seminario"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Seminario"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Seminar"	// eviri lazim!
#endif
	,
	[Roo_SHOP] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Shop"
#elif L==4	// es
	"Tienda"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Tienda"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Shop"		// eviri lazim!
#endif
	,
	[Roo_STORE] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Store"
#elif L==4	// es
	"Almac&eacute;n"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Almac&eacute;n"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Store"			// eviri lazim!
#endif
	,
	[Roo_TOILETS] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Toilets"
#elif L==4	// es
	"Aseos"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Aseos"		// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Toilets"	// eviri lazim!
#endif
	,
	[Roo_VIRTUAL] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Virtual"
#elif L==4	// es
	"Virtual"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Virtual"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Virtual"	// eviri lazim!
#endif
	,
	[Roo_YARD] =
#if   L==1	// ca
	""
#elif L==2	// de
	""
#elif L==3	// en
	"Yard"
#elif L==4	// es
	"Patio/Jard&iacute;n"
#elif L==5	// fr
	""
#elif L==6	// gn
	"Patio/Jard&iacute;n"	// Okoteve traduccin
#elif L==7	// it
	""
#elif L==8	// pl
	""
#elif L==9	// pt
	""
#elif L==10	// tr
	"Yard"			// eviri lazim!
#endif
	};

const char *Txt_Rooms =
#if   L==1	// ca
	"Sales";
#elif L==2	// de
	"R&auml;ume";
#elif L==3	// en
	"Rooms";
#elif L==4	// es
	"Salas";
#elif L==5	// fr
	"Salles";
#elif L==6	// gn
	"Mbo'ehakoty";
#elif L==7	// it
	"Aule";
#elif L==8	// pl
	"Klasy";
#elif L==9	// pt
	"Salas";
#elif L==10	// tr
	"Odalar";
#endif

const char *Txt_Rooms_with_users =
#if   L==1	// ca
	"Sales amb usuaris";
#elif L==2	// de
	"R&auml;ume mit Benutzern";
#elif L==3	// en
	"Rooms with users";
#elif L==4	// es
	"Salas con usuarios";
#elif L==5	// fr
	"Salles avec utilisateurs";
#elif L==6	// gn
	"Salas con usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Rooms con utenti";
#elif L==8	// pl
	"Rooms with users";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Salas com usu&aacute;rios";
#elif L==10	// tr
	"Rooms with users";	// eviri lazim!
#endif

const char *Txt_ROOT_FOLDER_EXTERNAL_NAMES[Brw_NUM_TYPES_FILE_BROWSER] =
	{
	[Brw_UNKNOWN] =
	"",
	[Brw_SHOW_DOC_CRS] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_SHOW_MRK_CRS] =
#if   L==1	// ca
	"qualificacions"
#elif L==2	// de
	"Bewertungen"
#elif L==3	// en
	"marks"
#elif L==4	// es
	"calificaciones"
#elif L==5	// fr
	"notes"
#elif L==6	// gn
	"calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"risultati"
#elif L==8	// pl
	"marks"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"notas"
#elif L==10	// tr
	"marks"			// eviri lazim!
#endif
	,
	[Brw_ADMI_DOC_CRS] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_ADMI_SHR_CRS] =
#if   L==1	// ca
	"compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"shared"
#elif L==4	// es
	"compartidos"
#elif L==5	// fr
	"partages"
#elif L==6	// gn
	"compartidos"	// Okoteve traduccin
#elif L==7	// it
	"condivisi"
#elif L==8	// pl
	"shared"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"comum"
#elif L==10	// tr
	"shared"	// eviri lazim!
#endif
	,
	[Brw_ADMI_SHR_GRP] =
#if   L==1	// ca
	"compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"shared"
#elif L==4	// es
	"compartidos"
#elif L==5	// fr
	"partages"
#elif L==6	// gn
	"compartidos"	// Okoteve traduccin
#elif L==7	// it
	"condivisi"
#elif L==8	// pl
	"shared"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"comum"
#elif L==10	// tr
	"shared"	// eviri lazim!
#endif
	,
	[Brw_ADMI_WRK_USR] =
#if   L==1	// ca
	"treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"homework"
#elif L==4	// es
	"trabajos"
#elif L==5	// fr
	"travaux"
#elif L==6	// gn
	"tembiapo"
#elif L==7	// it
	"lavori"
#elif L==8	// pl
	"homework"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"trabalhos"
#elif L==10	// tr
	"homework"	// eviri lazim!
#endif
	,
	[Brw_ADMI_WRK_CRS] =
#if   L==1	// ca
	"treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"homework"
#elif L==4	// es
	"trabajos"
#elif L==5	// fr
	"travaux"
#elif L==6	// gn
	"tembiapo"
#elif L==7	// it
	"lavori"
#elif L==8	// pl
	"homework"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"trabalhos"
#elif L==10	// tr
	"homework"	// eviri lazim!
#endif
	,
	[Brw_ADMI_MRK_CRS] =
#if   L==1	// ca
	"qualificacions"
#elif L==2	// de
	"Bewertungen"
#elif L==3	// en
	"marks"
#elif L==4	// es
	"calificaciones"
#elif L==5	// fr
	"notes"
#elif L==6	// gn
	"calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"risultati"
#elif L==8	// pl
	"marks"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"notas"
#elif L==10	// tr
	"marks"			// eviri lazim!
#endif
	,
	[Brw_ADMI_BRF_USR] =
#if   L==1	// ca
	"maleti"
#elif L==2	// de
	"Aktentasche"
#elif L==3	// en
	"briefcase"
#elif L==4	// es
	"maletin"
#elif L==5	// fr
	"porte-documents"
#elif L==6	// gn
	"maletin"	// Okoteve traduccin
#elif L==7	// it
	"cartella"
#elif L==8	// pl
	"briefcase"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"pasta"
#elif L==10	// tr
	"briefcase"	// eviri lazim!
#endif
	,
	[Brw_SHOW_DOC_GRP] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_ADMI_DOC_GRP] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_SHOW_MRK_GRP] =
#if   L==1	// ca
	"qualificacions"	// Necessita traducci
#elif L==2	// de
	"Bewertungen"
#elif L==3	// en
	"marks"
#elif L==4	// es
	"calificaciones"
#elif L==5	// fr
	"notes"
#elif L==6	// gn
	"calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"risultati"
#elif L==8	// pl
	"marks"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"notas"
#elif L==10	// tr
	"marks"			// eviri lazim!
#endif
	,
	[Brw_ADMI_MRK_GRP] =
#if   L==1	// ca
	"qualificacions"
#elif L==2	// de
	"Bewertungen"
#elif L==3	// en
	"marks"
#elif L==4	// es
	"calificaciones"
#elif L==5	// fr
	"notes"
#elif L==6	// gn
	"calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"risultati"
#elif L==8	// pl
	"marks"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"notas"
#elif L==10	// tr
	"marks"			// eviri lazim!
#endif
	,
	[Brw_ADMI_ASG_USR] =
#if   L==1	// ca
	"activitats"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"assignments"
#elif L==4	// es
	"actividades"
#elif L==5	// fr
	"activites"
#elif L==6	// gn
	"tar&eacute;a"
#elif L==7	// it
	"attivit&agrave;"
#elif L==8	// pl
	"assignments"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"atividades"
#elif L==10	// tr
	"assignments"	// eviri lazim!
#endif
	,
	[Brw_ADMI_ASG_CRS] =
#if   L==1	// ca
	"activitats"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"assignments"
#elif L==4	// es
	"actividades"
#elif L==5	// fr
	"activites"
#elif L==6	// gn
	"tar&eacute;a"
#elif L==7	// it
	"attivit&agrave;"
#elif L==8	// pl
	"assignments"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"atividades"
#elif L==10	// tr
	"assignments"	// eviri lazim!
#endif
	,
	[Brw_SHOW_DOC_DEG] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_ADMI_DOC_DEG] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"belgeler"
#endif
	,
	[Brw_SHOW_DOC_CTR] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"belgeler"
#endif
	,
	[Brw_ADMI_DOC_CTR] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_SHOW_DOC_INS] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_ADMI_DOC_INS] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_ADMI_SHR_DEG] =
#if   L==1	// ca
	"compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"shared"
#elif L==4	// es
	"compartidos"
#elif L==5	// fr
	"partages"
#elif L==6	// gn
	"compartidos"	// Okoteve traduccin
#elif L==7	// it
	"condivisi"
#elif L==8	// pl
	"shared"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"comum"
#elif L==10	// tr
	"shared"	// eviri lazim!
#endif
	,
	[Brw_ADMI_SHR_CTR] =
#if   L==1	// ca
	"compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"shared"
#elif L==4	// es
	"compartidos"
#elif L==5	// fr
	"partages"
#elif L==6	// gn
	"compartidos"	// Okoteve traduccin
#elif L==7	// it
	"condivisi"
#elif L==8	// pl
	"shared"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"comum"
#elif L==10	// tr
	"shared"	// eviri lazim!
#endif
	,
	[Brw_ADMI_SHR_INS] =
#if   L==1	// ca
	"compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"shared"
#elif L==4	// es
	"compartidos"
#elif L==5	// fr
	"partages"
#elif L==6	// gn
	"compartidos"	// Okoteve traduccin
#elif L==7	// it
	"condivisi"
#elif L==8	// pl
	"shared"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"comum"
#elif L==10	// tr
	"shared"	// eviri lazim!
#endif
	,
	[Brw_ADMI_TCH_CRS] =
#if   L==1	// ca
	"privats"
#elif L==2	// de
	"Private"
#elif L==3	// en
	"private"
#elif L==4	// es
	"privados"
#elif L==5	// fr
	"priv&eacute;s"
#elif L==6	// gn
	"privados"	// Okoteve traduccin
#elif L==7	// it
	"privati"
#elif L==8	// pl
	"prywatne"
#elif L==9	// pt
	"privados"
#elif L==10	// tr
	"private"	// eviri lazim!
#endif
	,
	[Brw_ADMI_TCH_GRP] =
#if   L==1	// ca
	"privats"
#elif L==2	// de
	"Private"
#elif L==3	// en
	"private"
#elif L==4	// es
	"privados"
#elif L==5	// fr
	"priv&eacute;s"
#elif L==6	// gn
	"privados"	// Okoteve traduccin
#elif L==7	// it
	"privati"
#elif L==8	// pl
	"prywatne"
#elif L==9	// pt
	"privados"
#elif L==10	// tr
	"private"	// eviri lazim!
#endif
	,
	[Brw_ADMI_DOC_PRJ] =
#if   L==1	// ca
	"documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"documents"
#elif L==4	// es
	"documentos"
#elif L==5	// fr
	"documents"
#elif L==6	// gn
	"kuatia"
#elif L==7	// it
	"documenti"
#elif L==8	// pl
	"dokumenty"
#elif L==9	// pt
	"documentos"
#elif L==10	// tr
	"documents"	// eviri lazim!
#endif
	,
	[Brw_ADMI_ASS_PRJ] =
#if   L==1	// ca
	"avaluacio"
#elif L==2	// de
	"Bewertung"
#elif L==3	// en
	"assessment"
#elif L==4	// es
	"evaluacion"
#elif L==5	// fr
	"evaluation"
#elif L==6	// gn
	"mbohepy"
#elif L==7	// it
	"valutazione"
#elif L==8	// pl
	"ocena"
#elif L==9	// pt
	"avaliacao"
#elif L==10	// tr
	"assessment"	// eviri lazim!
#endif
	};

const char *Txt_Rubric =
#if   L==1	// ca
	"R&uacute;brica";
#elif L==2	// de
	"Rubrik";
#elif L==3	// en
	"Rubric";
#elif L==4	// es
	"R&uacute;brica";
#elif L==5	// fr
	"Rubrique";
#elif L==6	// gn
	"R&uacute;brica";	// Okoteve traduccin
#elif L==7	// it
	"Rubriche";
#elif L==8	// pl
	"Rubryki";
#elif L==9	// pt
	"Rubrica";
#elif L==10	// tr
	"De&gbreve;erlendirme";
#endif

const char *Txt_Rubric_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"R&uacute;brica <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Rubrik <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Rubric <strong>%s</strong> removed.";
#elif L==4	// es
	"R&uacute;brica <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Rubrique <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"R&uacute;brica <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Rubrica <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Rubryki <strong>%s</strong> usuni&eogon;te.";
#elif L==9	// pt
	"Rubrica <strong>%s</strong> removida.";
#elif L==10	// tr
	"De&gbreve;erlendirme listesi <strong>%s</strong> kald&inodot;r&inodot;ld&inodot;.";
#endif

const char *Txt_Rubrics =
#if   L==1	// ca
	"R&uacute;briques";
#elif L==2	// de
	"Rubriken";
#elif L==3	// en
	"Rubrics";
#elif L==4	// es
	"R&uacute;bricas";
#elif L==5	// fr
	"Rubriques";
#elif L==6	// gn
	"R&uacute;bricas";	// Okoteve traduccin
#elif L==7	// it
	"Rubriche";
#elif L==8	// pl
	"Rubryki";
#elif L==9	// pt
	"Rubricas";
#elif L==10	// tr
	"De&gbreve;erlendirme listesi";
#endif

const char *Txt_Save_as =
#if   L==1	// ca
	"Guardar com";
#elif L==2	// de
	"Speichern unter";
#elif L==3	// en
	"Save as";
#elif L==4	// es
	"Guardar como";
#elif L==5	// fr
	"Enregistrer sous";
#elif L==6	// gn
	"Guardar como";	// Okoteve traduccin
#elif L==7	// it
	"Salva come";
#elif L==8	// pl
	"Zapisz jako";
#elif L==9	// pt
	"Salvar como";
#elif L==10	// tr
	"Save as";	// eviri lazim!
#endif

const char *Txt_Save_changes =
#if   L==1	// ca
	"Desar canvis";
#elif L==2	// de
	"&Auml;nderungen speichern";
#elif L==3	// en
	"Save changes";
#elif L==4	// es
	"Guardar cambios";
#elif L==5	// fr
	"Enregistrer les modifications";
#elif L==6	// gn
	"&Ntilde;ongatu";
#elif L==7	// it
	"Salva modifiche";
#elif L==8	// pl
	"Zapisz zmiany";
#elif L==9	// pt
	"Salvar altera&ccedil;&otilde;es";
#elif L==10	// tr
	"De&gbreve;i&scedil;iklikleri kaydet";
#endif

const char *Txt_Save_file_properties =
#if   L==1	// ca
	"Guardar les propietats del fitxer";
#elif L==2	// de
	"Speichern Dateieigenschaften";
#elif L==3	// en
	"Save file properties";
#elif L==4	// es
	"Guardar propiedades del archivo";
#elif L==5	// fr
	"Enregistrer les propri&eacute;t&eacute;s du fichier";
#elif L==6	// gn
	"Guardar propiedades del archivo";	// Okoteve traduccin
#elif L==7	// it
	"Salva le propriet&agrave; dei file";
#elif L==8	// pl
	"Zapisz w&lstrok;a&sacute;ciwo&sacute;ci pliku";
#elif L==9	// pt
	"Salvar as propriedades do arquivo";
#elif L==10	// tr
	"Save file properties";			// eviri lazim!
#endif

const char *Txt_Scope =
#if   L==1	// ca
	"&Agrave;mbit";
#elif L==2	// de
	"Geltungsbereich";
#elif L==3	// en
	"Scope";
#elif L==4	// es
	"&Aacute;mbito";
#elif L==5	// fr
	"Port&eacute;e";
#elif L==6	// gn
	"&Aacute;mbito";	// Okoteve traduccin
#elif L==7	// it
	"Ambito";
#elif L==8	// pl
	"Scope";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"&Acirc;mbito";
#elif L==10	// tr
	"Scope";		// eviri lazim!
#endif

const char *Txt_Score =
#if   L==1	// ca
	"Puntuaci&oacute;n";	// Necessita traducci
#elif L==2	// de
	"Ergebnis";
#elif L==3	// en
	"Score";
#elif L==4	// es
	"Puntuaci&oacute;n";
#elif L==5	// fr
	"Note";
#elif L==6	// gn
	"Puntuaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Punteggio";
#elif L==8	// pl
	"Score";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o";
#elif L==10	// tr
	"Score";		// eviri lazim!
#endif

const char *Txt_Search =
#if   L==1	// ca
	"Cercar";
#elif L==2	// de
	"Suche";
#elif L==3	// en
	"Search";
#elif L==4	// es
	"Buscar";
#elif L==5	// fr
	"Chercher";
#elif L==6	// gn
	"Heka";
#elif L==7	// it
	"Cercare";
#elif L==8	// pl
	"Szuka&cacute;";
#elif L==9	// pt
	"Procurar";
#elif L==10	// tr
	"Aramak";
#endif

const char *Txt_See_full_notice =
#if   L==1	// ca
	"Veure avs complet";
#elif L==2	// de
	"Siehe voll Ank&uuml;ndigung";
#elif L==3	// en
	"See full notice";
#elif L==4	// es
	"Ver aviso completo";
#elif L==5	// fr
	"Voir l'avis complet";
#elif L==6	// gn
	"Ver aviso completo";	// Okoteve traduccin
#elif L==7	// it
	"Vedi avviso completo ";
#elif L==8	// pl
	"Wiadomo&sacute;ci ca&lstrok;y obwieszczenie";
#elif L==9	// pt
	"Ver aviso completo";
#elif L==10	// tr
	"See full notice";	// eviri lazim!
#endif

const char *Txt_See_less =
#if   L==1	// ca
	"Veure menys";
#elif L==2	// de
	"Zeige weniger";
#elif L==3	// en
	"See less";
#elif L==4	// es
	"Ver menos";
#elif L==5	// fr
	"Voir moins";
#elif L==6	// gn
	"Ver menos";	// Okoteve traduccin
#elif L==7	// it
	"Vedi meno";
#elif L==8	// pl
	"Zobacz mniej";
#elif L==9	// pt
	"Ver menos";
#elif L==10	// tr
	"See less";	// eviri lazim!
#endif

const char *Txt_See_message =
#if   L==1	// ca
	"Veure missatge";
#elif L==2	// de
	"Siehe Nachricht";
#elif L==3	// en
	"See message";
#elif L==4	// es
	"Ver mensaje";
#elif L==5	// fr
	"Voir message";
#elif L==6	// gn
	"Ver mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Vedi messaggio";
#elif L==8	// pl
	"See message";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver mensagem";
#elif L==10	// tr
	"See message";	// eviri lazim!
#endif

const char *Txt_See_messages_received =
#if   L==1	// ca
	"Veure missatges rebuts";
#elif L==2	// de
	"Siehe Empfangene Nachrichten";
#elif L==3	// en
	"See messages received";
#elif L==4	// es
	"Ver mensajes recibidos";
#elif L==5	// fr
	"Voir messages re&ccedil;us";
#elif L==6	// gn
	"Ver mensajes recibidos";	// Okoteve traduccin
#elif L==7	// it
	"Vedi messaggi ricevuti";
#elif L==8	// pl
	"Wiadomo&sacute;ci odebrane";
#elif L==9	// pt
	"Ver mensagens recebidas";
#elif L==10	// tr
	"See messages received";	// eviri lazim!
#endif

const char *Txt_See_more =
#if   L==1	// ca
	"Veure m&eacute;s";
#elif L==2	// de
	"Siehe mehr";
#elif L==3	// en
	"See more";
#elif L==4	// es
	"Ver m&aacute;s";
#elif L==5	// fr
	"Voir plus";
#elif L==6	// gn
	"Ver m&aacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Vedi pi&ugrave;";
#elif L==8	// pl
	"Zobacz wi&eogon;cej";
#elif L==9	// pt
	"Ver mais";
#elif L==10	// tr
	"See more";		// eviri lazim!
#endif

const char *Txt_See_new_activity =
#if   L==1	// ca
	"Veure nova activitat";
#elif L==2	// de
	"Siehe neue Aktivit&auml;t";
#elif L==3	// en
	"See new activity";
#elif L==4	// es
	"Ver nueva actividad";
#elif L==5	// fr
	"Voir nouvelle activit&eacute;";
#elif L==6	// gn
	"Ver nueva actividad";	// Okoteve traduccin
#elif L==7	// it
	"Vedi nuova attivit&agrave;";
#elif L==8	// pl
	"Zobacz now&aogon; aktywno&sacute;&cacute;";
#elif L==9	// pt
	"Ver nova atividade";
#elif L==10	// tr
	"See new activity";	// eviri lazim!
#endif

const char *Txt_See_notifications =
#if   L==1	// ca
	"Ver notificaciones";	// Necessita traducci
#elif L==2	// de
	"Siehe Anmeldungen";
#elif L==3	// en
	"See notifications";
#elif L==4	// es
	"Ver notificaciones";
#elif L==5	// fr
	"Voir notifications";
#elif L==6	// gn
	"Ver notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Vedi notifiche";
#elif L==8	// pl
	"See notifications";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver notifica&ccedil;&otilde;es";
#elif L==10	// tr
	"See notifications";	// eviri lazim!
#endif

const char *Txt_See_only_the_latest_X_COMMENTS =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Veure nom&eacute;s els %u &uacute;ltims";
#elif L==2	// de
	"Nur die neuesten %u anzeigen";
#elif L==3	// en
	"See only the %u latest";
#elif L==4	// es
	"Ver s&oacute;lo los %u &uacute;ltimos";
#elif L==5	// fr
	"Voir seulement les %u derni&egrave;res";
#elif L==6	// gn
	"Ver s&oacute;lo los %u &uacute;ltimos";	// Okoteve traduccin
#elif L==7	// it
	"Vedi solo le ultime %u";
#elif L==8	// pl
	"Zobacz tylko najnowsze %u";
#elif L==9	// pt
	"Ver apenas os %u &uacute;ltimos";
#elif L==10	// tr
	"See only the %u latest";			// eviri lazim!
#endif

const char *Txt_See_the_previous_X_COMMENTS =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Veure els %u anteriors";
#elif L==2	// de
	"Siehe die vorherigen %u";
#elif L==3	// en
	"See the previous %u";
#elif L==4	// es
	"Ver los %u anteriores";
#elif L==5	// fr
	"Voir les %u pr&eacute;c&eacute;dents";
#elif L==6	// gn
	"Ver los %u anteriores";	// Okoteve traduccin
#elif L==7	// it
	"Vedi i precedente %u";
#elif L==8	// pl
	"Zobacz poprzednie %u";
#elif L==9	// pt
	"Ver os %u anteriores";
#elif L==10	// tr
	"See the previous %u";		// eviri lazim!
#endif

const char *Txt_Select_one_country_in_X =
#if   L==1	// ca
	"Seleccionar un pa&iacute;s en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen Sie ein Land von <strong>%s</strong>";
#elif L==3	// en
	"Select one country in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar un pa&iacute;s en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez un pays dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar un pa&iacute;s en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare un paese di <strong>%s</strong>";
#elif L==8	// pl
	"Wybierz jeden kraj w <strong>%s</strong>";
#elif L==9	// pt
	"Selecionar um pa&iacute;s de <strong>%s</strong>";
#elif L==10	// tr
	"Select one country in <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Select_or_create_one_center_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear un center en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen Lehrinstitut von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one center in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear un centro en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tude dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear un centro en <strong>%s</strong>";		// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare un centro di <strong>%s</strong>";
#elif L==8	// pl
	"Wybierz lub utw&oacute;rz jeden stopie&nacute; w <strong>%s</strong>.";
#elif L==9	// pt
	"Selecionar ou criar um centro de <strong>%s</strong>";
#elif L==10	// tr
	"Select or create one center in <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Select_or_create_one_course_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una assignatura en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie eine Kurs von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one course in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear una asignatura en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez une mati&egrave;re dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear una asignatura en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare un corso di <strong>%s</strong>";
#elif L==8	// pl
	"Wybierz lub utw&oacute;rz jeden kurs w <strong>%s</strong>";
#elif L==9	// pt
	"Selecionar ou criar uma disciplina de <strong>%s</strong>";
#elif L==10	// tr
	"Select or create one course in <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Select_or_create_one_degree_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una titulaci&oacute; en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen Studiengang von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one degree in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear una titulaci&oacute;n en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tude dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear una titulaci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare una laurea di <strong>%s</strong>";
#elif L==8	// pl
	"Wybierz lub utw&oacute;rz jeden stopie&nacute; w <strong>%s</strong>";
#elif L==9	// pt
	"Selecionar ou criar um grau de <strong>%s</strong>";
#elif L==10	// tr
	"Select or create one degree in <strong>%s</strong>";			// eviri lazim!
#endif

const char *Txt_Select_or_create_one_institution_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una instituci&oacute; en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen Hochschule von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one institution in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear una instituci&oacute;n en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tablissement de <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear una instituci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare una istituzione di <strong>%s</strong>";
#elif L==8	// pl
	"Wybierz lub utw&oacute;rz jedn&aogon; instytucj&eogon; w <strong>%s</strong>";
#elif L==9	// pt
	"Selecionar ou criar uma institu&ccedil;&atilde;o de <strong>%s</strong>";
#elif L==10	// tr
	"Select or create one institution in <strong>%s</strong>";		// eviri lazim!
#endif

const char *Txt_Select_one_or_more_files_from_your_computer_or_drag_and_drop_here =
#if   L==1	// ca
	"Seleccioneu un o m&eacute;s arxius del seu ordinador<br />"
	"o arrossegueu i deixeu-los anar aqu&iacute;";
#elif L==2	// de
	"W&auml;hlen Sie eine oder mehrere Dateien von Ihrem Computer<br />"
	"oder per Drag & Drop hier";
#elif L==3	// en
	"Select one or more files from your computer<br />"
	"or drag and drop here";
#elif L==4	// es
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";
#elif L==5	// fr
	"S&eacute;lectionnez un ou plusieurs fichiers &agrave; partir de votre ordinateur<br />"
	"ou glisser-d&eacute;poser ici";
#elif L==6	// gn
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare uno o pi&ugrave; file dal tuo computer<br />"
	"o trascinare e rilasciare qui";
#elif L==8	// pl
	"Wybierz jeden lub wi&eogon;cej plik&oacute;w z komputera<br />"
	"lub przeci&aogon;gnij i upu&sacute;&cacute; tutaj";
#elif L==9	// pt
	"Selecione um ou mais arquivos de seu computador<br />"
	"ou arrastar e soltar aqui";
#elif L==10	// tr
	"Select one or more files from your computer<br />"
	"or drag and drop here";				// eviri lazim!
#endif

const char *Txt_Select_questions =
#if   L==1	// ca
	"Seleccionar preguntes";
#elif L==2	// de
	"W&auml;hlen Sie Fragen";
#elif L==3	// en
	"Select questions";
#elif L==4	// es
	"Seleccionar preguntas";
#elif L==5	// fr
	"Choisir questions";
#elif L==6	// gn
	"Seleccionar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare domande";
#elif L==8	// pl
	"Wybierz pytania";
#elif L==9	// pt
	"Selecionar perguntas";
#elif L==10	// tr
	"Select questions";		// eviri lazim!
#endif

const char *Txt_Select_the_groups_in_from_which_you_want_to_register_remove_users_ =
#if   L==1	// ca
	"Seleccioneu els grups en/des dels quals voleu registrar/eliminar usuaris.<br />"
	"Si seleccioneu algun grup, els usuaris s'inscriuran a l'assignatura i als grups seleccionats, o s'eliminaran dels grups seleccionats per&ograve; no de l'assignatura.<br />"
	"Si no seleccioneu cap grup, els usuaris seran inscrits a l'assignatura per&ograve; no a grups especfics,"
	" o s'eliminaran de l'assignatura i de tots els grups.";
#elif L==2	// de
	"W&auml;hlen Sie die Gruppen aus, in/aus denen Sie Benutzer registrieren/entfernen m&ouml;chten.<br />"
	"Wenn Sie Gruppen ausw&auml;hlen, werden Benutzer in den Kurs und die ausgew&auml;hlten Gruppen eingeschrieben oder aus den ausgew&auml;hlten Gruppen,"
	" aber nicht aus dem Kurs entfernt.<br />"
	"Wenn Sie keine Gruppen ausw&auml;hlen, werden Benutzer in den Kurs eingeschrieben, aber nicht in bestimmte Gruppen,"
	" oder aus dem Kurs und aus allen Gruppen entfernt.";
#elif L==3	// en
	"Select the groups in/from which you want to register/remove users.<br />"
	"If you select any groups, users will be enroled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enroled in the course but not in specific groups,"
	" or removed from the course and from all groups.";
#elif L==4	// es
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";
#elif L==5	// fr
	"S&eacute;lectionnez les groupes dans/dont vous souhaitez enregistrer/supprimer des utilisateurs.<br />"
	"Si vous s&eacute;lectionnez des groupes, les utilisateurs seront inscrits a la mati&egrave;re et aux groupes s&eacute;lectionn&eacute;s,"
	" ou supprim&eacute;s des groupes s&eacute;lectionn&eacute;s mais pas de la mati&egrave;re.<br />"
	"Si vous ne s&eacute;lectionnez aucun groupe, les utilisateurs seront inscrits au cours mais pas &agrave; des groupes sp&eacute;cifiques,"
	" ou seront supprim&eacute;s de la mati&egrave;re et de tous les groupes.";
#elif L==6	// gn
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";	// Okoteve traduccin
#elif L==7	// it
	"Seleziona i gruppi nel/dal quale vuoi registrare/rimuovere utenti.<br />"
	"Se selezioni qualche gruppo, gli utenti saranno registrati nel corso e nel gruppo selezionato,"
	" o rimossi dai gruppi selezionati ma non dal corso.<br />"
	"Se non selezioni alcun gruppo, gli utenti saranno registrati nel corso ma non in gruppi specifici,"
	" o rimossi dal corso e da tutti i gruppi.";
#elif L==8	// pl
	"Wybierz grupy, w kt&oacute;rych/z kt&oacute;rych chcesz zarejestrowa&cacute;/usun&aogon;&cacute; u&zdot;ytkownik&oacute;w.<br />"
	"Je&sacute;li wybierzesz jak&aogon;kolwiek grup&eogon;, u&zdot;ytkownicy zostan&aogon; zapisani do kursu i wybranych grup"
	" lub zostan&aogon; usuni&eogon;ci z wybranych grup, ale nie z kursu.<br />"
	"Je&sacute;li nie wybierzesz &zdot;adnej grupy, u&zdot;ytkownicy zostan&aogon; zapisani do kursu, ale nie do okre&sacute;lonych grup,"
	" lub usuni&eogon;ci z kursu i ze wszystkich grup.";
#elif L==9	// pt
	"Selecione os grupos em/dos quais voc&ecirc; deseja registrar/remover usu&aacute;rios.<br />"
	"Se voc&ecirc; selecionar grupos, os usu&aacute;rios ser&atilde;o inscritos na disciplina e nos grupos selecionados"
	" ou removidos dos grupos selecionados, mas n&atilde;o da disciplina.<br />"
	"Se voc&ecirc; n&atilde;o selecionar nenhum grupo, os usu&aacute;rios ser&atilde;o inscritos na disciplina, mas n&atilde;o em grupos espec&iacute;ficos,"
	" ou ser&atilde;o removidos da disciplina e de todos os grupos.";
#elif L==10	// tr
	"Select the groups in/from which you want to register/remove users.<br />"
	"If you select any groups, users will be enroled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enroled in the course but not in specific groups,"
	" or removed from the course and from all groups.";		// eviri lazim!
#endif

const char *Txt_Selected_users_X_Followed_Y_Not_followed_Z =	// Warning: it is very important to include three %u in the following sentences
#if   L==1	// ca
	"Usuaris seleccionats: %u<br />"
	"&bull; Seguits: %u<br />"
	"&bull; No seguits: %u";
#elif L==2	// de
	"Ausgew&auml;hlte Benutzer: %u<br />"
	"&bull; Gefolgt: %u<br />"
	"&bull; Nicht gefolgt: %u";
#elif L==3	// en
	"Selected users: %u<br />"
	"&bull; Followed: %u<br />"
	"&bull; Not followed: %u";
#elif L==4	// es
	"Usuarios seleccionados: %u<br />"
	"&bull; Seguidos: %u<br />"
	"&bull; No seguidos: %u";
#elif L==5	// fr
	"Utilisateurs s&eacute;lectionn&eacute;s: %u<br />"
	"&bull; Suivis: %u<br />"
	"&bull; Non suivis: %u";
#elif L==6	// gn
	"Usuarios seleccionados: %u<br />"
	"&bull; Seguidos: %u<br />"
	"&bull; No seguidos: %u";	// Okoteve traduccin
#elif L==7	// it
	"Utenti selezionati: %u<br />"
	"&bull; Seguiti: %u<br />"
	"&bull; Non seguiti: %u";
#elif L==8	// pl
	"Wybrani u&zdot;ytkownicy: %u<br />"
	"&bull; Nast&eogon;pnie: %u<br />"
	"&bull; Nie przestrzegano: %u";
#elif L==9	// pt
	"Usu&aacute;rios selecionados: %u<br />"
	"&bull; Seguidos: %u<br />"
	"&bull; N&atilde;o seguidos: %u";
#elif L==10	// tr
	"Selected users: %u<br />"
	"&bull; Followed: %u<br />"
	"&bull; Not followed: %u";	// eviri lazim!
#endif

const char *Txt_Send =
#if   L==1	// ca
	"Enviar";
#elif L==2	// de
	"Senden";
#elif L==3	// en
	"Send";
#elif L==4	// es
	"Enviar";
#elif L==5	// fr
	"Envoyer";
#elif L==6	// gn
	"Mondo";
#elif L==7	// it
	"Invia";
#elif L==8	// pl
	"Wy&sacute;lij";
#elif L==9	// pt
	"Enviar";
#elif L==10	// tr
	"Send";	// eviri lazim!
#endif

const char *Txt_Send_message =
#if   L==1	// ca
	"Enviar missatge";
#elif L==2	// de
	"Senden Nachricht";
#elif L==3	// en
	"Send message";
#elif L==4	// es
	"Enviar mensaje";
#elif L==5	// fr
	"Envoyer message";
#elif L==6	// gn
	"Enviar mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Invia messaggio";
#elif L==8	// pl
	"Wy&sacute;lij wiadomo&sacute;&cacute;";
#elif L==9	// pt
	"Enviar mensagem";
#elif L==10	// tr
	"Send message";		// eviri lazim!
#endif

const char *Txt_Send_survey =
#if   L==1	// ca
	"Enviar enquesta";
#elif L==2	// de
	"Senden Umfrage";
#elif L==3	// en
	"Send survey";
#elif L==4	// es
	"Enviar encuesta";
#elif L==5	// fr
	"Envoyer sondage";
#elif L==6	// gn
	"Enviar encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Invia sondaggio";
#elif L==8	// pl
	"Send survey";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Enviar inqu&eacute;rito";
#elif L==10	// tr
	"Send survey";		// eviri lazim!
#endif

const char *Txt_Send_URL =
#if   L==1	// ca
	"Enviar URL";
#elif L==2	// de
	"Senden URL";
#elif L==3	// en
	"Send URL";
#elif L==4	// es
	"Enviar URL";
#elif L==5	// fr
	"Envoyer URL";
#elif L==6	// gn
	"Enviar URL";	// Okoteve traduccin
#elif L==7	// it
	"Invia URL";
#elif L==8	// pl
	"Wy&sacute;lij URL";
#elif L==9	// pt
	"Enviar URL";
#elif L==10	// tr
	"Send URL";	// eviri lazim!
#endif

const char *Txt_Session =
#if   L==1	// ca
	"Sessi&oacute;";
#elif L==2	// de
	"Sitzung";
#elif L==3	// en
	"Session";
#elif L==4	// es
	"Sesi&oacute;n";
#elif L==5	// fr
	"Session";
#elif L==6	// gn
	"Sesi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Sessione";
#elif L==8	// pl
	"Sesja";
#elif L==9	// pt
	"Sess&atilde;o";
#elif L==10	// tr
	"Oturum";
#endif

const char *Txt_session =
#if   L==1	// ca
	"sessi&oacute;";
#elif L==2	// de
	"Sitzung";
#elif L==3	// en
	"session";
#elif L==4	// es
	"sesi&oacute;n";
#elif L==5	// fr
	"session";
#elif L==6	// gn
	"sesi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"sessione";
#elif L==8	// pl
	"sesja";
#elif L==9	// pt
	"sess&atilde;o";
#elif L==10	// tr
	"session";		// eviri lazim!
#endif

const char *Txt_Session_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Sessi&oacute; <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Sitzung <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Session <strong>%s</strong> removed.";
#elif L==4	// es
	"Sesi&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Session <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Sesi&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Sessione <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Sesja <strong>%s</strong> usuni&eogon;te.";
#elif L==9	// pt
	"Sess&atilde;o <strong>%s</strong> removida.";
#elif L==10	// tr
	"Session <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Sessions =
#if   L==1	// ca
	"Sessions";
#elif L==2	// de
	"Sitzungen";
#elif L==3	// en
	"Sessions";
#elif L==4	// es
	"Sesiones";
#elif L==5	// fr
	"Sessions";
#elif L==6	// gn
	"Sesiones";	// Okoteve traduccin
#elif L==7	// it
	"Sessioni";
#elif L==8	// pl
	"Sesje";
#elif L==9	// pt
	"Sess&otilde;es";
#elif L==10	// tr
	"Sessions";	// eviri lazim!
#endif

const char *Txt_sessions =
#if   L==1	// ca
	"sessions";
#elif L==2	// de
	"Sitzungen";
#elif L==3	// en
	"sessions";
#elif L==4	// es
	"sesiones";
#elif L==5	// fr
	"sessions";
#elif L==6	// gn
	"sesiones";	// Okoteve traduccin
#elif L==7	// it
	"sessioni";
#elif L==8	// pl
	"sesje";
#elif L==9	// pt
	"sess&otilde;es";
#elif L==10	// tr
	"sessions";	// eviri lazim!
#endif

const char *Txt_Set_of_questions =
#if   L==1	// ca
	"Conjunt de preguntes";
#elif L==2	// de
	"Satz von Fragen";
#elif L==3	// en
	"Set of questions";
#elif L==4	// es
	"Conjunto de preguntas";
#elif L==5	// fr
	"Ensemble de questions";
#elif L==6	// gn
	"Conjunto de preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Set di domande";
#elif L==8	// pl
	"Zestaw pyta&nacute;";
#elif L==9	// pt
	"Conjunto de perguntas";
#elif L==10	// tr
	"Set of questions";		// eviri lazim!
#endif

const char *Txt_Set_of_questions_removed =
#if   L==1	// ca
	"Conjunt de preguntes eliminat.";
#elif L==2	// de
	"Satz von Fragen entfernt.";
#elif L==3	// en
	"Set of questions removed.";
#elif L==4	// es
	"Conjunto de preguntas eliminado.";
#elif L==5	// fr
	"Ensemble de questions supprim&eacute;.";
#elif L==6	// gn
	"Conjunto de preguntas eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Set di domande rimosso.";
#elif L==8	// pl
	"Zestaw pyta&nacute; usuniete.";
#elif L==9	// pt
	"Conjunto de perguntas removido.";
#elif L==10	// tr
	"Set of questions removed.";		// eviri lazim!
#endif

const char *Txt_Set_password =
#if   L==1	// ca
	"Establir contrasenya";
#elif L==2	// de
	"Passwort festlegen";
#elif L==3	// en
	"Set password";
#elif L==4	// es
	"Establecer contrase&ntilde;a";
#elif L==5	// fr
	"D&eacute;finir mot de passe";
#elif L==6	// gn
	"Establecer contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Impostare password";
#elif L==8	// pl
	"Ustaw has&lstrok;o";
#elif L==9	// pt
	"Definir senha";
#elif L==10	// tr
	"&Scedil;ifreyi belirle";
#endif

const char *Txt_Set_up =
#if   L==1	// ca
	"Instal&centerdot;lar";
#elif L==2	// de
	"Einrichten";
#elif L==3	// en
	"Set up";
#elif L==4	// es
	"Instalar";
#elif L==5	// fr
	"Installer";
#elif L==6	// gn
	"Instalar";	// Okoteve traduccin
#elif L==7	// it
	"Setup";
#elif L==8	// pl
	"Set up";
#elif L==9	// pt
	"Instalar";
#elif L==10	// tr
	"Set up";	// eviri lazim!
#endif

const char *Txt_Sets_of_questions =
#if   L==1	// ca
	"Conjunts de preguntes";
#elif L==2	// de
	"S&auml;tze von Fragen";
#elif L==3	// en
	"Sets of questions";
#elif L==4	// es
	"Conjuntos de preguntas";
#elif L==5	// fr
	"Ensembles de questions";
#elif L==6	// gn
	"Porandu &ntilde;emohenda";
#elif L==7	// it
	"Set di domande";
#elif L==8	// pl
	"Zestaw&oacute;w pyta&nacute;";
#elif L==9	// pt
	"Conjuntos de perguntas";
#elif L==10	// tr
	"Soru setleri";
#endif

const char *Txt_Settings =
#if   L==1	// ca
	"Configuraci&oacute;";
#elif L==2	// de
	"Einstellungen";
#elif L==3	// en
	"Settings";
#elif L==4	// es
	"Ajustes";
#elif L==5	// fr
	"R&eacute;glages";
#elif L==6	// gn
	"Mbohoryha";
#elif L==7	// it
	"Impostazioni";
#elif L==8	// pl
	"Ustawienia";
#elif L==9	// pt
	"Configura&ccedil;&atilde;o";
#elif L==10	// tr
	"Ayarlar";
#endif

const char *Txt_Sex =
#if   L==1	// ca
	"Sexe";
#elif L==2	// de
	"Sex";
#elif L==3	// en
	"Sex";
#elif L==4	// es
	"Sexo";
#elif L==5	// fr
	"Sexe";
#elif L==6	// gn
	"Me&ntilde;a";
#elif L==7	// it
	"Sesso";
#elif L==8	// pl
	"Ple&cacute;";
#elif L==9	// pt
	"Sexo";
#elif L==10	// tr
	"Sex";	// eviri lazim!
#endif

const char *Txt_SEX_PLURAL_Abc[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Desconeguts"
#elif L==2	// de
	"Unbekannt"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecidos"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Dones"	// Necessita traducci
#elif L==2	// de
	"Frauen"
#elif L==3	// en
	"Women"
#elif L==4	// es
	"Mujeres"
#elif L==5	// fr
	"Femmes"
#elif L==6	// gn
	"Mujeres"	// Okoteve traduccin
#elif L==7	// it
	"Donne"
#elif L==8	// pl
	"Kobiety"
#elif L==9	// pt
	"Mulheres"
#elif L==10	// tr
	"Women"		// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"Homes"
#elif L==2	// de
	"M&auml;nner"
#elif L==3	// en
	"Men"
#elif L==4	// es
	"Hombres"
#elif L==5	// fr
	"Hommes"
#elif L==6	// gn
	"Hombres"	// Okoteve traduccin
#elif L==7	// it
	"Uomini"
#elif L==8	// pl
	"Mezczyzni"
#elif L==9	// pt
	"Homens"
#elif L==10	// tr
	"Men"		// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"Todos"	// Necessita traducci
#elif L==2	// de
	"Alle"
#elif L==3	// en
	"All"
#elif L==4	// es
	"Todos"
#elif L==5	// fr
	"Tous"
#elif L==6	// gn
	"Todos"	// Okoteve traduccin
#elif L==7	// it
	"Tutti"
#elif L==8	// pl
	"Wszyscy"
#elif L==9	// pt
	"Todos"
#elif L==10	// tr
	"All"	// eviri lazim!
#endif
	};

const char *Txt_SEX_PLURAL_abc[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"desconeguts"
#elif L==2	// de
	"unbekannt"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#elif L==10	// tr
	"unknown"	// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"dones"
#elif L==2	// de
	"Frauen"
#elif L==3	// en
	"women"
#elif L==4	// es
	"mujeres"
#elif L==5	// fr
	"femmes"
#elif L==6	// gn
	"mujeres"	// Okoteve traduccin
#elif L==7	// it
	"donne"
#elif L==8	// pl
	"kobiety"
#elif L==9	// pt
	"mulheres"
#elif L==10	// tr
	"women"		// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"homes"
#elif L==2	// de
	"M&auml;nner"
#elif L==3	// en
	"men"
#elif L==4	// es
	"hombres"
#elif L==5	// fr
	"hommes"
#elif L==6	// gn
	"hombres"	// Okoteve traduccin
#elif L==7	// it
	"uomini"
#elif L==8	// pl
	"mezczyzni"
#elif L==9	// pt
	"homens"
#elif L==10	// tr
	"men"		// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"todos"	// Necessita traducci
#elif L==2	// de
	"alle"
#elif L==3	// en
	"all"
#elif L==4	// es
	"todos"
#elif L==5	// fr
	"tous"
#elif L==6	// gn
	"todos"	// Okoteve traduccin
#elif L==7	// it
	"tutti"
#elif L==8	// pl
	"wszyscy"
#elif L==9	// pt
	"todos"
#elif L==10	// tr
	"all"	// eviri lazim!
#endif
	};

const char *Txt_SEX_SINGULAR_Abc[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Desconegut"
#elif L==2	// de
	"Unbekannt"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido"
#elif L==10	// tr
	"Unknown"	// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Dona"
#elif L==2	// de
	"Frau"
#elif L==3	// en
	"Woman"
#elif L==4	// es
	"Mujer"
#elif L==5	// fr
	"Femme"
#elif L==6	// gn
	"Ku&ntilde;a"
#elif L==7	// it
	"Donna"
#elif L==8	// pl
	"Kobieta"
#elif L==9	// pt
	"Mulher"
#elif L==10	// tr
	"Woman"	// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"Home"
#elif L==2	// de
	"Mann"
#elif L==3	// en
	"Man"
#elif L==4	// es
	"Hombre"
#elif L==5	// fr
	"Homme"
#elif L==6	// gn
	"Kuimba'e"
#elif L==7	// it
	"Uomo"
#elif L==8	// pl
	"M&eogon;&zdot;czyzna"
#elif L==9	// pt
	"Homem"
#elif L==10	// tr
	"Man"	// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"Tots"
#elif L==2	// de
	"Alle"
#elif L==3	// en
	"All"
#elif L==4	// es
	"Todos"
#elif L==5	// fr
	"Tous"
#elif L==6	// gn
	"Todos"		// Okoteve traduccin
#elif L==7	// it
	"Tutti"
#elif L==8	// pl
	"Wwszyscy"
#elif L==9	// pt
	"Todos"
#elif L==10	// tr
	"All"		// eviri lazim!
#endif
	};

const char *Txt_Shared_files_area =
#if   L==1	// ca
	"Zona d'arxius compartits";
#elif L==2	// de
	"Freigegebene Dateien-Bereich";
#elif L==3	// en
	"Shared files area";
#elif L==4	// es
	"Zona de archivos compartidos";
#elif L==5	// fr
	"Zone de fichiers partag&eacute;s";
#elif L==6	// gn
	"Zona de archivos compartidos";	// Okoteve traduccin
#elif L==7	// it
	"Area comune";
#elif L==8	// pl
	"Wsp&oacute;lnej dla";
#elif L==9	// pt
	"Zona comum";
#elif L==10	// tr
	"Shared files area";		// eviri lazim!
#endif

const char *Txt_shared_files_area =
#if   L==1	// ca
	"zona d'arxius compartits";
#elif L==2	// de
	"Freigegebene Dateien-Bereich";
#elif L==3	// en
	"shared files area";
#elif L==4	// es
	"zona de archivos compartidos";
#elif L==5	// fr
	"zone de fichiers partag&eacute;s";
#elif L==6	// gn
	"zona de archivos compartidos";	// Okoteve traduccin
#elif L==7	// it
	"zona di file condivisi";
#elif L==8	// pl
	"Powierzchnia udost&eogon;pnionych plik&oacute;w";
#elif L==9	// pt
	"zona de arquivos compartilhados";
#elif L==10	// tr
	"shared files area";		// eviri lazim!
#endif

const char *Txt_Short_name =
#if   L==1	// ca
	"Nom breu";
#elif L==2	// de
	"Kurzname";
#elif L==3	// en
	"Short name";
#elif L==4	// es
	"Nombre breve";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute;";
#elif L==6	// gn
	"Nombre breve";		// Okoteve traduccin
#elif L==7	// it
	"Nome breve";
#elif L==8	// pl
	"Nazwa skr&oacute;cona";
#elif L==9	// pt
	"Nome abreviado";
#elif L==10	// tr
	"K&inodot;sa isim";
#endif

const char *Txt_Short_name_of_the_degree =
#if   L==1	// ca
	"Nom breu de la titulaci&oacute;";
#elif L==2	// de
	"Kurzname der Studiengang";
#elif L==3	// en
	"Short name of the degree";
#elif L==4	// es
	"Nombre breve de la titulaci&oacute;n";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; du &eacute;tude";
#elif L==6	// gn
	"Nombre breve de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve della laurea";
#elif L==8	// pl
	"Skr&oacute;cona nazwa stopie&nacute;";
#elif L==9	// pt
	"Nome abreviado do grau";
#elif L==10	// tr
	"Short name of the degree";		// eviri lazim!
#endif

const char *Txt_Short_name_of_the_center =
#if   L==1	// ca
	"Nom breu del center";
#elif L==2	// de
	"Kurzname der Lehrinstitut";
#elif L==3	// en
	"Short name of the center";
#elif L==4	// es
	"Nombre breve del centro";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; du center";
#elif L==6	// gn
	"Nombre breve del centro";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve del centro";
#elif L==8	// pl
	"Skr&oacute;cona nazwa centrum";
#elif L==9	// pt
	"Nome abreviado do centro";
#elif L==10	// tr
	"Short name of the center";	// eviri lazim!
#endif

const char *Txt_Short_name_of_the_course =
#if   L==1	// ca
	"Nom breu de l'assignatura";
#elif L==2	// de
	"Kurzname der Kurs";
#elif L==3	// en
	"Short name of the course";
#elif L==4	// es
	"Nombre breve de la asignatura";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; de la mati&egrave;re";
#elif L==6	// gn
	"Nombre breve de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve del corso";
#elif L==8	// pl
	"Skr&oacute;cona nazwa kurs";
#elif L==9	// pt
	"Nome abreviado da disciplina";
#elif L==10	// tr
	"Short name of the course";		// eviri lazim!
#endif

const char *Txt_Short_name_of_the_institution =
#if   L==1	// ca
	"Nom breu de la instituci&oacute;";
#elif L==2	// de
	"Kurzname der Institution";
#elif L==3	// en
	"Short name of the institution";
#elif L==4	// es
	"Nombre breve de la instituci&oacute;n";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; de l'institution";
#elif L==6	// gn
	"Nombre breve de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve dell'istituzione";
#elif L==8	// pl
	"Skr&oacute;cona nazwa instytucji";
#elif L==9	// pt
	"Nome abreviado da institui&ccedil;&atilde;o";
#elif L==10	// tr
	"Short name of the institution";		// eviri lazim!
#endif

const char *Txt_Shortcut =
#if   L==1	// ca
	"Acc&eacute;s directe";
#elif L==2	// de
	"Direktzugriff";
#elif L==3	// en
	"Shortcut";
#elif L==4	// es
	"Acceso directo";
#elif L==5	// fr
	"Acc&egrave;s direct";
#elif L==6	// gn
	"Acceso directo";	// Okoteve traduccin
#elif L==7	// it
	"Accesso diretto";
#elif L==8	// pl
	"Bezpo&sacute;redni dost&eogon;p";
#elif L==9	// pt
	"Acesso directo";
#elif L==10	// tr
	"Shortcut";		// eviri lazim!
#endif

const char *Txt_Show =
#if   L==1	// ca
	"Mostrar";
#elif L==2	// de
	"Zeige";
#elif L==3	// en
	"Show";
#elif L==4	// es
	"Mostrar";
#elif L==5	// fr
	"Voir";
#elif L==6	// gn
	"Mostrar";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza";
#elif L==8	// pl
	"Pokaza&cacute;";
#elif L==9	// pt
	"Mostrar";
#elif L==10	// tr
	"Show";		// eviri lazim!
#endif

const char *Txt_Show_all_files =
#if   L==1	// ca
	"Mostrar tots els arxius";
#elif L==2	// de
	"Alle Dateien anzeigen";
#elif L==3	// en
	"Show all files";
#elif L==4	// es
	"Mostrar todos los archivos";
#elif L==5	// fr
	"Afficher tous les fichiers";
#elif L==6	// gn
	"Mostrar todos los archivos";	// Okoteve traduccin
#elif L==7	// it
	"Mostra tutti i file";
#elif L==8	// pl
	"Poka&zdot; wszystkie pliki";
#elif L==9	// pt
	"Mostrar todos os arquivos";
#elif L==10	// tr
	"Show all files";		// eviri lazim!
#endif

const char *Txt_Show_all_NOTIFICATIONS =
#if   L==1	// ca
	"Mostrar tots";
#elif L==2	// de
	"Alle anzeigen";
#elif L==3	// en
	"Show all";
#elif L==4	// es
	"Mostrar todas";
#elif L==5	// fr
	"Afficher tous";
#elif L==6	// gn
	"Mostrar todas";	// Okoteve traduccin
#elif L==7	// it
	"Mostra tutti";
#elif L==8	// pl
	"Poka&zdot; wszystkie";
#elif L==9	// pt
	"Mostrar todas";
#elif L==10	// tr
	"Show all";		// eviri lazim!
#endif

const char *Txt_Show_all_notifications =
#if   L==1	// ca
	"Mostrar tots les notificacions";
#elif L==2	// de
	"Alle Anmeldungen anzeigen";
#elif L==3	// en
	"Show all notifications";
#elif L==4	// es
	"Mostrar todas las notificaciones";
#elif L==5	// fr
	"Afficher tous les notifications";
#elif L==6	// gn
	"Mostrar todas las notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Mostra tutti le notifiche";
#elif L==8	// pl
	"Poka&zdot; wszystkie powiadomienia";
#elif L==9	// pt
	"Mostrar todas as notifica&ccedil;&otilde;es";
#elif L==10	// tr
	"Show all notifications";		// eviri lazim!
#endif

const char *Txt_Show_agenda =
#if   L==1	// ca
	"Mostrar agenda";
#elif L==2	// de
	"Organizer anzeigen";
#elif L==3	// en
	"Show agenda";
#elif L==4	// es
	"Ver agenda";
#elif L==5	// fr
	"Afficher agenda";
#elif L==6	// gn
	"Ver agenda";	// Okoteve traduccin
#elif L==7	// it
	"Mostra agenda";
#elif L==8	// pl
	"Poka&zdot; pami&eogon;tnik";
#elif L==9	// pt
	"Mostrar agenda";
#elif L==10	// tr
	"Show agenda";	// eviri lazim!
#endif

const char *Txt_Show_anyway =
#if   L==1	// ca
	"Mostrar de totes maneres";
#elif L==2	// de
	"Zeigesowieso";
#elif L==3	// en
	"Show anyway";
#elif L==4	// es
	"Mostrar de todos modos";
#elif L==5	// fr
	"Voir de toute faon";
#elif L==6	// gn
	"Mostrar de todos modos";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza comunque";
#elif L==8	// pl
	"Show anyway";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar de qualquer maneira";
#elif L==10	// tr
	"Show anyway";			// eviri lazim!
#endif

const char *Txt_Show_calendar =
#if   L==1	// ca
	"Mostrar calendari";
#elif L==2	// de
	"Kalender anzeigen";
#elif L==3	// en
	"Show calendar";
#elif L==4	// es
	"Ver calendario";
#elif L==5	// fr
	"Afficher calendrier";
#elif L==6	// gn
	"Ver calendario";	// Okoteve traduccin
#elif L==7	// it
	"Mostra calendario";
#elif L==8	// pl
	"Poka&zdot; kalendarz";
#elif L==9	// pt
	"Mostrar calend&aacute;rio";
#elif L==10	// tr
	"Show calendar";	// eviri lazim!
#endif

const char *Txt_Show_hits =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Veure accessos";
#elif L==2	// de
	"Zeige Anmeldungen";
#elif L==3	// en
	"Show hits";
#elif L==4	// es
	"Ver accesos";
#elif L==5	// fr
	"Voir visites";
#elif L==6	// gn
	"Ver accesos";		// Okoteve traduccin
#elif L==7	// it
	"Visualizza visite";
#elif L==8	// pl
	"Show hits";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar visitas";
#elif L==10	// tr
	"Show hits";		// eviri lazim!
#endif

const char *Txt_Show_more_details =
#if   L==1	// ca
	"Mostrar m&eacute;s detalls";
#elif L==2	// de
	"Mehr Details";
#elif L==3	// en
	"Show more details";
#elif L==4	// es
	"Mostrar m&aacute;s detalles";
#elif L==5	// fr
	"Montrer plus de d&eacute;tails";
#elif L==6	// gn
	"Ehechauka hetave detalle";
#elif L==7	// it
	"Mostra pi&ugrave; dettagli";
#elif L==8	// pl
	"Poka&zdot; wi&eogon;cej szczeg&oacute;&lstrok;&oacute;w";
#elif L==9	// pt
	"Mostrar mais detalhes";
#elif L==10	// tr
	"Daha fazla ayr&inodot;nt&inodot; g&ouml;ster";
#endif

const char *Txt_Show_more_recipients =
#if   L==1	// ca
	"Veure m&eacute;s recipients";
#elif L==2	// de
	"Mehrere Empf&auml;nger anzeigen";
#elif L==3	// en
	"Show more recipients";
#elif L==4	// es
	"Ver m&aacute;s destinatarios";
#elif L==5	// fr
	"Montrer plus de destinataires";
#elif L==6	// gn
	"Ver m&aacute;s destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Mostra pi&ugrave; destinatari";
#elif L==8	// pl
	"Poka&zdot; wi&eogon;cej odbiorc&oacute;w";
#elif L==9	// pt
	"Ver mais destinat&aacute;rios";
#elif L==10	// tr
	"Show more recipients";		// eviri lazim!
#endif

const char *Txt_Show_next_X_clicks =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Mostrar els %u clics seg&uuml;ents";
#elif L==2	// de
	"N&auml;chste %u Klicks anzeigen";
#elif L==3	// en
	"Show next %u clicks";
#elif L==4	// es
	"Mostrar %u clics posteriores";
#elif L==5	// fr
	"Afficher les %u clics suivants";
#elif L==6	// gn
	"Mostrar %u clics posteriores";	// Okoteve traduccin
#elif L==7	// it
	"Mostra %u scatti seguenti";
#elif L==8	// pl
	"Poka&zdot; nast&eogon;pne %u klikni&eogon;&cacute;";
#elif L==9	// pt
	"Mostrar %u cliques subsequentes";
#elif L==10	// tr
	"Show next %u clicks";		// eviri lazim!
#endif

const char *Txt_Show_previous_X_clicks =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Mostrar %u clics anteriors";
#elif L==2	// de
	"Vorherige %u Klicks anzeigen";
#elif L==3	// en
	"Show previous %u clicks";
#elif L==4	// es
	"Mostrar %u clics anteriores";
#elif L==5	// fr
	"Afficher les %u clics pr&eacute;c&eacute;dents";
#elif L==6	// gn
	"Mostrar %u clics anteriores";	// Okoteve traduccin
#elif L==7	// it
	"Mostra %u scatti precedenti";
#elif L==8	// pl
	"Poka&zdot; poprzednie %u klikni&eogon;cia";
#elif L==9	// pt
	"Mostrar %u cliques anteriores";
#elif L==10	// tr
	"Show previous %u clicks";	// eviri lazim!
#endif

const char *Txt_Show_questions =
#if   L==1	// ca
	"Mostrar preguntes";
#elif L==2	// de
	"Zeige Testfragen";
#elif L==3	// en
	"Show questions";
#elif L==4	// es
	"Mostrar preguntas";
#elif L==5	// fr
	"Voir les questions";
#elif L==6	// gn
	"Mostrar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza le domande";
#elif L==8	// pl
	"Poka&zdot; pytania";
#elif L==9	// pt
	"Mostrar perguntas";
#elif L==10	// tr
	"Show questions";	// eviri lazim!
#endif

const char *Txt_Show_statistic =
#if   L==1	// ca
	"Mostrar estad&iacute;stica";
#elif L==2	// de
	"Zeige Statistik";
#elif L==3	// en
	"Show statistic";
#elif L==4	// es
	"Mostrar estad&iacute;stica";
#elif L==5	// fr
	"Voir statistique";
#elif L==6	// gn
	"Mostrar estad&iacute;stica";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza statistica";
#elif L==8	// pl
	"Poka&zdot; statystyk&eogon;";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar estat&iacute;stica";
#elif L==10	// tr
	"Show statistic";		// eviri lazim!
#endif

const char *Txt_Show_tutoring_hours =
#if   L==1	// ca
	"Mostrar horaris tutories";
#elif L==2	// de
	"Gesch&auml;ftszeiten anzeigen";
#elif L==3	// en
	"Show tutoring hours";
#elif L==4	// es
	"Ver horarios tutor&iacute;as";
#elif L==5	// fr
	"Voir horaires tuteur";
#elif L==6	// gn
	"Ver horarios tutor&iacute;as";	// Okoteve traduccin
#elif L==7	// it
	"Mostra orari tutoraggi";
#elif L==8	// pl
	"Poka&zdot; godzin pracy biura";
#elif L==9	// pt
	"Mostrar hor&aacute;rios tutor";
#elif L==10	// tr
	"Show tutoring hours";		// eviri lazim!
#endif

const char *Txt_Shuffle =
#if   L==1	// ca
	"Barrejar";
#elif L==2	// de
	"Shuffle";
#elif L==3	// en
	"Shuffle";
#elif L==4	// es
	"Barajar";
#elif L==5	// fr
	"M&eacute;langer";
#elif L==6	// gn
	"Barajar";	// Okoteve traduccin
#elif L==7	// it
	"Mischiare";
#elif L==8	// pl
	"Cz&lstrok;apa&cacute;";
#elif L==9	// pt
	"Baralhar";
#elif L==10	// tr
	"Shuffle";	// eviri lazim!
#endif

const char *Txt_Sign_up =
#if   L==1	// ca
	"Sol&middot;licitar inscripci&oacute;";
#elif L==2	// de
	"Registrierung beantragen";
#elif L==3	// en
	"Sign up";
#elif L==4	// es
	"Solicitar inscripci&oacute;n";
#elif L==5	// fr
	"Demander inscription";
#elif L==6	// gn
	"Solicitar inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Richiedere iscrizione";
#elif L==8	// pl
	"Wniosku rejestracj&eogon;";
#elif L==9	// pt
	"Solicitar inscri&ccedil;&atilde;o";
#elif L==10	// tr
	"Sign up";			// eviri lazim!
#endif

const char *Txt_Similar_users =
#if   L==1	// ca
	"Usuaris similars";
#elif L==2	// de
	"&Auml;hnliche Benutzer";
#elif L==3	// en
	"Similar users";
#elif L==4	// es
	"Usuarios similares";
#elif L==5	// fr
	"Utilisateurs similaires";
#elif L==6	// gn
	"Usuarios similares";	// Okoteve traduccin
#elif L==7	// it
	"Utenti simili";
#elif L==8	// pl
	"Podobne u&zdot;ytkownik&oacute;w";
#elif L==9	// pt
	"Usu&aacute;rios similares";
#elif L==10	// tr
	"Similar users";	// eviri lazim!
#endif

const char *Txt_Size =		// ...of a file zone
#if   L==1	// ca
	"Mida";
#elif L==2	// de
	"Gr&ouml;&szlig;e";
#elif L==3	// en
	"Size";
#elif L==4	// es
	"Tama&ntilde;o";
#elif L==5	// fr
	"Taille";
#elif L==6	// gn
	"Tama&ntilde;o";	// Okoteve traduccin
#elif L==7	// it
	"Dimensione";
#elif L==8	// pl
	"Rozmiar";
#elif L==9	// pt
	"Tamanho";
#elif L==10	// tr
	"Size";			// eviri lazim!
#endif

const char *Txt_Size_of_photos =
#if   L==1	// ca
	"Mida de les fotos";
#elif L==2	// de
	"Gr&ouml;&szlig;e des Fotos";
#elif L==3	// en
	"Size of photos";
#elif L==4	// es
	"Tama&ntilde;o de las fotos";
#elif L==5	// fr
	"Taille des photos";
#elif L==6	// gn
	"Tama&ntilde;o de las fotos";	// Okoteve traduccin
#elif L==7	// it
	"Dimensioni delle foto";
#elif L==8	// pl
	"Rozmiar zdj&eogon;&cacute;";
#elif L==9	// pt
	"Tamanho das fotos";
#elif L==10	// tr
	"Size of photos";		// eviri lazim!
#endif

const char *Txt_Skip_this_step =
#if   L==1	// ca
	"Saltar aquest pas";
#elif L==2	// de
	"&Uuml;berspringen Sie diesen Schritt";
#elif L==3	// en
	"Skip this step";
#elif L==4	// es
	"Saltar este paso";
#elif L==5	// fr
	"Passer cette &eacute;tape";
#elif L==6	// gn
	"Saltar este paso";	// Okoteve traduccin
#elif L==7	// it
	"Salta questo passaggio";
#elif L==8	// pl
	"Pomi&nacute; ten krok";
#elif L==9	// pt
	"Pule esta etapa";
#elif L==10	// tr
	"Bu ad&inodot;m&inodot; atla";
#endif

const char *Txt_Sort_degrees_by =
#if   L==1	// ca
	"Ordenar titulacions per";
#elif L==2	// de
	"Sortieren Studieng&auml;nge nach";
#elif L==3	// en
	"Sort degrees by";
#elif L==4	// es
	"Ordenar titulaciones por";
#elif L==5	// fr
	"Trier &eacute;tudes par";
#elif L==6	// gn
	"Ordenar titulaciones por";	// Okoteve traduccin
#elif L==7	// it
	"Ordina laurea per";
#elif L==8	// pl
	"Sortuj stopni wed&lstrok;ug";
#elif L==9	// pt
	"Classificar graus por";
#elif L==10	// tr
	"Sort degrees by";		// eviri lazim!
#endif

const char *Txt_Source =
#if   L==1	// ca
	"Font";
#elif L==2	// de
	"Quelle";
#elif L==3	// en
	"Source";
#elif L==4	// es
	"Fuente";
#elif L==5	// fr
	"Source";
#elif L==6	// gn
	"Fuente";	// Okoteve traduccin
#elif L==7	// it
	"Fonte";
#elif L==8	// pl
	"&Zacute;r&oacute;d&lstrok;o";
#elif L==9	// pt
	"Fonte";
#elif L==10	// tr
	"Source";	// eviri lazim!
#endif

const char *Txt_Source_of_information =
#if   L==1	// ca
	"Font d'informaci&oacute;";
#elif L==2	// de
	"Quelle von Informationen";
#elif L==3	// en
	"Source of information";
#elif L==4	// es
	"Fuente de informaci&oacute;n";
#elif L==5	// fr
	"Source d'information";
#elif L==6	// gn
	"Fuente de informaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Fonte delle informazioni";
#elif L==8	// pl
	"&Zacute;r&oacute;d&lstrok;o informacji";
#elif L==9	// pt
	"Fonte de informa&ccedil;&atilde;o";
#elif L==10	// tr
	"Source of information";	// eviri lazim!
#endif

const char *Txt_STAT_CLICKS_GROUPED_BY[Sta_NUM_CLICKS_GROUPED_BY] =
	{
	[Sta_CLICKS_CRS_DETAILED_LIST] =
#if   L==1	// ca
	"Clics detallats"
#elif L==2	// de
	"Detaillierte Klicks"
#elif L==3	// en
	"Detailed clicks"
#elif L==4	// es
	"Clics detallados"
#elif L==5	// fr
	"Clics d&eacute;taill&eacute;es"
#elif L==6	// gn
	"Clics detallados"	// Okoteve traduccin
#elif L==7	// it
	"Dettagli di scatto"
#elif L==8	// pl
	"Szczeg&oacute;&lstrok;owe klikni&eogon;&cacute;"
#elif L==9	// pt
	"Detalhadamente cliques"
#elif L==10	// tr
	"Detailed clicks"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_USR] =
#if   L==1	// ca
	"usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario"
#elif L==5	// fr
	"utilisateur"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownik"
#elif L==9	// pt
	"utilizador"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_DAY] =
#if   L==1	// ca
	"dia"
#elif L==2	// de
	"Tag"
#elif L==3	// en
	"day"
#elif L==4	// es
	"d&iacute;a"
#elif L==5	// fr
	"jour"
#elif L==6	// gn
	"&aacute;ra"
#elif L==7	// it
	"giorno"
#elif L==8	// pl
	"dzie&nacute;"
#elif L==9	// pt
	"dia"
#elif L==10	// tr
	"day"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_DAY_AND_HOUR] =
#if   L==1	// ca
	"dia i hora"
#elif L==2	// de
	"Tag und Stunde"
#elif L==3	// en
	"day and hour"
#elif L==4	// es
	"d&iacute;a y hora"
#elif L==5	// fr
	"jour et heure"
#elif L==6	// gn
	"&aacute;ra ha aravo"
#elif L==7	// it
	"giorno e ora"
#elif L==8	// pl
	"dzie&nacute; i godzin&eogon;"
#elif L==9	// pt
	"dia e hora"
#elif L==10	// tr
	"day and hour"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_WEEK] =
#if   L==1	// ca
	"setmana"
#elif L==2	// de
	"Woche"
#elif L==3	// en
	"week"
#elif L==4	// es
	"semana"
#elif L==5	// fr
	"semaine"
#elif L==6	// gn
	"arapok&otilde;indy"
#elif L==7	// it
	"settimana"
#elif L==8	// pl
	"tydzie&nacute;"
#elif L==9	// pt
	"semana"
#elif L==10	// tr
	"week"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_MONTH] =
#if   L==1	// ca
	"mes"
#elif L==2	// de
	"Monat"
#elif L==3	// en
	"month"
#elif L==4	// es
	"mes"
#elif L==5	// fr
	"mois"
#elif L==6	// gn
	"jasy"
#elif L==7	// it
	"mese"
#elif L==8	// pl
	"miesi&aogon;c"
#elif L==9	// pt
	"m&ecirc;s"
#elif L==10	// tr
	"month"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_YEAR] =
#if   L==1	// ca
	"any"
#elif L==2	// de
	"Jahr"
#elif L==3	// en
	"year"
#elif L==4	// es
	"a&ntilde;o"
#elif L==5	// fr
	"ann&eacute;e"
#elif L==6	// gn
	"ary"
#elif L==7	// it
	"anno"
#elif L==8	// pl
	"rok"
#elif L==9	// pt
	"ano"
#elif L==10	// tr
	"year"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_HOUR] =
#if   L==1	// ca
	"hora"
#elif L==2	// de
	"Stunde"
#elif L==3	// en
	"hour"
#elif L==4	// es
	"hora"
#elif L==5	// fr
	"heure"
#elif L==6	// gn
	"aravo"
#elif L==7	// it
	"ora"
#elif L==8	// pl
	"godzina"
#elif L==9	// pt
	"hora"
#elif L==10	// tr
	"hour"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_MINUTE] =
#if   L==1	// ca
	"minut"
#elif L==2	// de
	"Minute"
#elif L==3	// en
	"minute"
#elif L==4	// es
	"minuto"
#elif L==5	// fr
	"minute"
#elif L==6	// gn
	"aravo'i"
#elif L==7	// it
	"minuto"
#elif L==8	// pl
	"minut&eogon;"
#elif L==9	// pt
	"minuto"
#elif L==10	// tr
	"minute"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_CRS_PER_ACTION] =
#if   L==1	// ca
	"acci&oacute;"
#elif L==2	// de
	"Handlung"
#elif L==3	// en
	"action"
#elif L==4	// es
	"acci&oacute;n"
#elif L==5	// fr
	"action"
#elif L==6	// gn
	"tembiapo"
#elif L==7	// it
	"azione"
#elif L==8	// pl
	"dzia&lstrok;anie"
#elif L==9	// pt
	"a&ccedil;&atilde;o"
#elif L==10	// tr
	"action"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_DAY] =
#if   L==1	// ca
	"dia"
#elif L==2	// de
	"Tag"
#elif L==3	// en
	"day"
#elif L==4	// es
	"d&iacute;a"
#elif L==5	// fr
	"jour"
#elif L==6	// gn
	"&aacute;ra"
#elif L==7	// it
	"giorno"
#elif L==8	// pl
	"dzie&nacute;"
#elif L==9	// pt
	"dia"
#elif L==10	// tr
	"day"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_DAY_AND_HOUR] =
#if   L==1	// ca
	"dia i hora"
#elif L==2	// de
	"Tag und Stunde"
#elif L==3	// en
	"day and hour"
#elif L==4	// es
	"d&iacute;a y hora"
#elif L==5	// fr
	"jour et heure"
#elif L==6	// gn
	"&aacute;ra ha aravo"
#elif L==7	// it
	"giorno e ora"
#elif L==8	// pl
	"dzie&nacute; i godzin&eogon;"
#elif L==9	// pt
	"dia e hora"
#elif L==10	// tr
	"day and hour"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_WEEK] =
#if   L==1	// ca
	"setmana"
#elif L==2	// de
	"Woche"
#elif L==3	// en
	"week"
#elif L==4	// es
	"semana"
#elif L==5	// fr
	"semaine"
#elif L==6	// gn
	"arapok&otilde;indy"
#elif L==7	// it
	"settimana"
#elif L==8	// pl
	"tydzie&nacute;"
#elif L==9	// pt
	"semana"
#elif L==10	// tr
	"week"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_MONTH] =
#if   L==1	// ca
	"mes"
#elif L==2	// de
	"Monat"
#elif L==3	// en
	"month"
#elif L==4	// es
	"mes"
#elif L==5	// fr
	"mois"
#elif L==6	// gn
	"jasy"
#elif L==7	// it
	"mese"
#elif L==8	// pl
	"miesi&aogon;c"
#elif L==9	// pt
	"m&ecirc;s"
#elif L==10	// tr
	"month"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_YEAR] =
#if   L==1	// ca
	"any"
#elif L==2	// de
	"Jahr"
#elif L==3	// en
	"year"
#elif L==4	// es
	"a&ntilde;o"
#elif L==5	// fr
	"ann&eacute;e"
#elif L==6	// gn
	"ary"
#elif L==7	// it
	"anno"
#elif L==8	// pl
	"rok"
#elif L==9	// pt
	"ano"
#elif L==10	// tr
	"year"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_HOUR] =
#if   L==1	// ca
	"hora"
#elif L==2	// de
	"Stunde"
#elif L==3	// en
	"hour"
#elif L==4	// es
	"hora"
#elif L==5	// fr
	"heure"
#elif L==6	// gn
	"aravo"
#elif L==7	// it
	"ora"
#elif L==8	// pl
	"godzina"
#elif L==9	// pt
	"hora"
#elif L==10	// tr
	"hour"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_MINUTE] =
#if   L==1	// ca
	"minut"
#elif L==2	// de
	"Minute"
#elif L==3	// en
	"minute"
#elif L==4	// es
	"minuto"
#elif L==5	// fr
	"minute"
#elif L==6	// gn
	"aravo'i"
#elif L==7	// it
	"minuto"
#elif L==8	// pl
	"minut&eogon;"
#elif L==9	// pt
	"minuto"
#elif L==10	// tr
	"minute"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_ACTION] =
#if   L==1	// ca
	"acci&oacute;"
#elif L==2	// de
	"Handlung"
#elif L==3	// en
	"action"
#elif L==4	// es
	"acci&oacute;n"
#elif L==5	// fr
	"action"
#elif L==6	// gn
	"tembiapo"
#elif L==7	// it
	"azione"
#elif L==8	// pl
	"dzia&lstrok;anie"
#elif L==9	// pt
	"a&ccedil;&atilde;o"
#elif L==10	// tr
	"action"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_PLUGIN] =
#if   L==1	// ca
	"complement"
#elif L==2	// de
	"Plugin"
#elif L==3	// en
	"plugin"
#elif L==4	// es
	"complemento"
#elif L==5	// fr
	"plugin"
#elif L==6	// gn
	"complemento"	// Okoteve traduccin
#elif L==7	// it
	"plugin"
#elif L==8	// pl
	"wtyczka"
#elif L==9	// pt
	"plugin"
#elif L==10	// tr
	"plugin"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_API_FUNCTION] =
#if   L==1	// ca
	"funci&oacute;"
#elif L==2	// de
	"Funktion"
#elif L==3	// en
	"function"
#elif L==4	// es
	"funci&oacute;n"
#elif L==5	// fr
	"fonction"
#elif L==6	// gn
	"funci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"funzione"
#elif L==8	// pl
	"funkcja"
#elif L==9	// pt
	"fun&ccedil;&atilde;o"
#elif L==10	// tr
	"function"		// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_BANNER] =
#if   L==1	// ca
	"banner"
#elif L==2	// de
	"Werbebanner"
#elif L==3	// en
	"banner"
#elif L==4	// es
	"banner"
#elif L==5	// fr
	"banni&eacute;re"
#elif L==6	// gn
	"banner"	// Okoteve traduccin
#elif L==7	// it
	"banner"
#elif L==8	// pl
	"banner"
#elif L==9	// pt
	"banner"
#elif L==10	// tr
	"banner"	// eviri lazim!
#endif
	,
	[Sta_CLICKS_GBL_PER_COUNTRY] =
#if   L==1	// ca
	"pa&iacute;s"
#elif L==2	// de
	"Land"
#elif L==3	// en
	"country"
#elif L==4	// es
	"pa&iacute;s"
#elif L==5	// fr
	"pays;"
#elif L==6	// gn
	"tet&atilde;"
#elif L==7	// it
	"paese"
#elif L==8	// pl
	"kraj"
#elif L==9	// pt
	"pa&iacute;s"
#elif L==10	// tr
	"&uuml;lke"
#endif
	,
	[Sta_CLICKS_GBL_PER_INSTITUTION] =
#if   L==1	// ca
	"instituci&oacute;"
#elif L==2	// de
	"Hochschule"
#elif L==3	// en
	"institution"
#elif L==4	// es
	"instituci&oacute;n"
#elif L==5	// fr
	"&eacute;tablissement"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"istituzione"
#elif L==8	// pl
	"instytucji"
#elif L==9	// pt
	"institu&ccedil;&atilde;o"
#elif L==10	// tr
	"kurum"
#endif
	,
	[Sta_CLICKS_GBL_PER_CENTER] =
#if   L==1	// ca
	"center"
#elif L==2	// de
	"Lehrinstitut"
#elif L==3	// en
	"center"
#elif L==4	// es
	"centro"
#elif L==5	// fr
	"center"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"centro"
#elif L==8	// pl
	"centrum"
#elif L==9	// pt
	"centro"
#elif L==10	// tr
	"merkez"
#endif
	,
	[Sta_CLICKS_GBL_PER_DEGREE] =
#if   L==1	// ca
	"titulaci&oacute;"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"degree"
#elif L==4	// es
	"titulaci&oacute;n"
#elif L==5	// fr
	"&eacute;tude"
#elif L==6	// gn
	"arandur&atilde;"
#elif L==7	// it
	"laurea"
#elif L==8	// pl
	"stopie&nacute;"
#elif L==9	// pt
	"grau"
#elif L==10	// tr
	"derece"
#endif
	,
	[Sta_CLICKS_GBL_PER_COURSE] =
#if   L==1	// ca
	"assignatura"
#elif L==2	// de
	"Kurs"
#elif L==3	// en
	"course"
#elif L==4	// es
	"asignatura"
#elif L==5	// fr
	"mati&egrave;re"
#elif L==6	// gn
	"mbo'esyry"
#elif L==7	// it
	"corso"
#elif L==8	// pl
	"kurs"
#elif L==9	// pt
	"disciplina"
#elif L==10	// tr
	"kurs"
#endif
	};

const char *Txt_STAT_COLOR_TYPES[Sta_NUM_COLOR_TYPES] =
	{
	[Sta_COLOR] =
#if   L==1	// ca
	"color"
#elif L==2	// de
	"Farbe"
#elif L==3	// en
	"color"
#elif L==4	// es
	"color"
#elif L==5	// fr
	"couleur"
#elif L==6	// gn
	"sa'y"
#elif L==7	// it
	"colore"
#elif L==8	// pl
	"kolor"
#elif L==9	// pt
	"cor"
#elif L==10	// tr
	"color"	// eviri lazim!
#endif
	,
	[Sta_BLACK_TO_WHITE] =
#if   L==1	// ca
	"de negre a blanc"
#elif L==2	// de
	"schwarz zu wei&szlig;"
#elif L==3	// en
	"black to white"
#elif L==4	// es
	"de negro a blanco"
#elif L==5	// fr
	"du noir au blanc"
#elif L==6	// gn
	"de negro a blanco"	// Okoteve traduccin
#elif L==7	// it
	"dal nero al bianco"
#elif L==8	// pl
	"czarny na bia&lstrok;y"
#elif L==9	// pt
	"do preto ao branco"
#elif L==10	// tr
	"black to white"	// eviri lazim!
#endif
	,
	[Sta_WHITE_TO_BLACK] =
#if   L==1	// ca
	"de blanc a negre"
#elif L==2	// de
	"wei&szlig; zu schwarz"
#elif L==3	// en
	"white to black"
#elif L==4	// es
	"de blanco a negro"
#elif L==5	// fr
	"du blanc au noir"
#elif L==6	// gn
	"de blanco a negro"	// Okoteve traduccin
#elif L==7	// it
	"dal bianco al nero"
#elif L==8	// pl
	"bia&lstrok;y do czarny"
#elif L==9	// pt
	"do branco ao preto"
#elif L==10	// tr
	"white to black"	// eviri lazim!
#endif
	};

const char *Txt_STAT_COURSE_FILE_ZONES[Fig_NUM_STAT_CRS_FILE_ZONES] =	// Use &nbsp; instead of space
	{
	[0] =
#if   L==1	// ca
	"Documents&nbsp;(ass.)"
#elif L==2	// de
	"Dokumente&nbsp;(Ver.)"
#elif L==3	// en
	"Documents&nbsp;(crs.)"
#elif L==4	// es
	"Documentos&nbsp;(asg.)"
#elif L==5	// fr
	"Documents&nbsp;(mat.)"
#elif L==6	// gn
	"Documentos&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7	// it
	"Documenti&nbsp;(crs.)"
#elif L==8	// pl
	"Dokumenty&nbsp;(kurs)"
#elif L==9	// pt
	"Documentos&nbsp;(crs.)"
#elif L==10	// tr
	"Documents&nbsp;(crs.)"		// eviri lazim!
#endif
	,
	[1] =
#if   L==1	// ca
	"Documents&nbsp;(grp.)"
#elif L==2	// de
	"Dokumente&nbsp;(Grp.)"
#elif L==3	// en
	"Documents&nbsp;(grp.)"
#elif L==4	// es
	"Documentos&nbsp;(grp.)"
#elif L==5	// fr
	"Documents&nbsp;(grp.)"
#elif L==6	// gn
	"Documentos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7	// it
	"Documenti&nbsp;(grp.)"
#elif L==8	// pl
	"Dokumenty&nbsp;(grp.)"
#elif L==9	// pt
	"Documentos&nbsp;(grp.)"
#elif L==10	// tr
	"Documents&nbsp;(grp.)"		// eviri lazim!
#endif
	,
	[2] =
#if   L==1	// ca
	"Privats&nbsp;(ass.)"
#elif L==2	// de
	"Private&nbsp;(Ver.)"
#elif L==3	// en
	"Private&nbsp;(crs.)"
#elif L==4	// es
	"Privados&nbsp;(asg.)"
#elif L==5	// fr
	"Priv&eacute;s&nbsp;(mat.)"
#elif L==6	// gn
	"Privados&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7	// it
	"Privati&nbsp;(crs.)"
#elif L==8	// pl
	"Prywatne&nbsp;(kurs)"
#elif L==9	// pt
	"Privadoss&nbsp;(crs.)"
#elif L==10	// tr
	"Private&nbsp;(crs.)"	// eviri lazim!
#endif
	,
	[3] =
#if   L==1	// ca
	"Privats&nbsp;(grp.)"
#elif L==2	// de
	"Private&nbsp;(Grp.)"
#elif L==3	// en
	"Private&nbsp;(grp.)"
#elif L==4	// es
	"Privados&nbsp;(grp.)"
#elif L==5	// fr
	"Priv&eacute;s&nbsp;(grp.)"
#elif L==6	// gn
	"Privados&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7	// it
	"Privati&nbsp;(grp.)"
#elif L==8	// pl
	"Prywatne&nbsp;(grp)"
#elif L==9	// pt
	"Privadoss&nbsp;(grp.)"
#elif L==10	// tr
	"Private&nbsp;(grp.)"	// eviri lazim!
#endif
	,
	[4] =
#if   L==1	// ca
	"Compartits&nbsp;(asg.)"
#elif L==2	// de
	"Freigegebene&nbsp;(Ver.)"
#elif L==3	// en
	"Shared&nbsp;(crs.)"
#elif L==4	// es
	"Compartidos&nbsp;(asg.)"
#elif L==5	// fr
	"Partag&eacute;s&nbsp;(mat.)"
#elif L==6	// gn
	"Compartidos&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi&nbsp;(crs.)"
#elif L==8	// pl
	"Wsp&oacute;lna&nbsp;(kurs)"
#elif L==9	// pt
	"Compartilhados&nbsp;(crs.)"
#elif L==10	// tr
	"Shared&nbsp;(crs.)"		// eviri lazim!
#endif
	,
	[5] =
#if   L==1	// ca
	"Compartits&nbsp;(grp.)"
#elif L==2	// de
	"Freigegebene&nbsp;(Grp.)"
#elif L==3	// en
	"Shared&nbsp;(grp.)"
#elif L==4	// es
	"Compartidos&nbsp;(grp.)"
#elif L==5	// fr
	"Partag&eacute;s&nbsp;(grp.)"
#elif L==6	// gn
	"Compartidos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi&nbsp;(grp.)"
#elif L==8	// pl
	"Wsp&oacute;lna&nbsp;(grp.)"
#elif L==9	// pt
	"Compartilhados&nbsp;(grp.)"
#elif L==10	// tr
	"Shared&nbsp;(grp.)"		// eviri lazim!
#endif
	,
	[6] =
#if   L==1	// ca
	"Qualificacions&nbsp;(ass.)"
#elif L==2	// de
	"Bewertungen&nbsp;(Ver.)"
#elif L==3	// en
	"Marks&nbsp;(crs.)"
#elif L==4	// es
	"Calificaciones&nbsp;(asg.)"
#elif L==5	// fr
	"Notes&nbsp;(mat.)"
#elif L==6	// gn
	"Calificaciones&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7	// it
	"Risultati&nbsp;(crs.)"
#elif L==8	// pl
	"Marks&nbsp;(crs.)"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Notas&nbsp;(crs.)"
#elif L==10	// tr
	"Marks&nbsp;(crs.)"		// eviri lazim!
#endif
	,
	[7] =
#if   L==1	// ca
	"Qualificacions&nbsp;(grp.)"
#elif L==2	// de
	"Bewertungen&nbsp;(Grp.)"
#elif L==3	// en
	"Marks&nbsp;(grp.)"
#elif L==4	// es
	"Calificaciones&nbsp;(grp.)"
#elif L==5	// fr
	"Notes&nbsp;(grp.)"
#elif L==6	// gn
	"Calificaciones&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7	// it
	"Risultati&nbsp;(grp.)"
#elif L==8	// pl
	"Marks&nbsp;(grp.)"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Notas&nbsp;(grp.)"
#elif L==10	// tr
	"Marks&nbsp;(grp.)"		// eviri lazim!
#endif
	,
	[8] =
#if   L==1	// ca
	"Activitats"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"Assignments"
#elif L==4	// es
	"Actividades"
#elif L==5	// fr
	"Activit&eacute;s"
#elif L==6	// gn
	"Tar&eacute;a"
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Zadania"
#elif L==9	// pt
	"Atividades"
#elif L==10	// tr
	"&Ouml;devler"
#endif
	,
	[9] =
#if   L==1	// ca
	"Treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"Homework"
#elif L==4	// es
	"Trabajos"
#elif L==5	// fr
	"Travails"
#elif L==6	// gn
	"Tembiapo"
#elif L==7	// it
	"Lavori"
#elif L==8	// pl
	"Praca domowa"
#elif L==9	// pt
	"Trabalhos"
#elif L==10	// tr
	"Ev &ouml;devi"
#endif
	,
	[10] =
#if   L==1	// ca
	"Total"
#elif L==2	// de
	"Gesamtzahl"
#elif L==3	// en
	"Total"
#elif L==4	// es
	"Total"
#elif L==5	// fr
	"Total"
#elif L==6	// gn
	"Total"		// Okoteve traduccin
#elif L==7	// it
	"Totale"
#elif L==8	// pl
	"Ca&lstrok;kowity"
#elif L==9	// pt
	"Total"
#elif L==10	// tr
	"Total"		// eviri lazim!
#endif
	,
	[11] =
#if   L==1	// ca
	"Maletins"
#elif L==2	// de
	"Aktentasche"
#elif L==3	// en
	"Briefcases"
#elif L==4	// es
	"Maletines"
#elif L==5	// fr
	"Porte-documents"
#elif L==6	// gn
	"Maletines"	// Okoteve traduccin
#elif L==7	// it
	"Penna USB"
#elif L==8	// pl
	"Akt&oacute;wki"
#elif L==9	// pt
	"Pasta"
#elif L==10	// tr
	"Briefcases"	// eviri lazim!
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_SIZE[Pho_NUM_HOW_COMPUTE_PHOTO_SIZES] =
	{
	[Pho_PROPORTIONAL_TO_NUMBER_OF_STUDENTS] =
#if   L==1	// ca
	"proporcional al nombre d'estudiants"
#elif L==2	// de
	"proportional zur Sch&uuml;lerzahl"
#elif L==3	// en
	"proportional to the number of students"
#elif L==4	// es
	"proporcional al n&ordm; de estudiantes"
#elif L==5	// fr
	"proportionnel au nombre d'&eacute;tudiants"
#elif L==6	// gn
	"proporcional al n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==7	// it
	"proporzionale al numero di studenti"
#elif L==8	// pl
	"proporcjonalna do liczby student&oacute;w"
#elif L==9	// pt
	"proporcional ao n&uacute;mero de estudantes"
#elif L==10	// tr
	"proportional to the number of students"	// eviri lazim!
#endif
	,
	[Pho_PROPORTIONAL_TO_NUMBER_OF_PHOTOS] =
#if   L==1	// ca
	"proporcional al nombre d'estudiants amb foto"
#elif L==2	// de
	"proportional zur Anzahl der Sch&uuml;ler mit Foto"
#elif L==3	// en
	"proportional to the number of students with photo"
#elif L==4	// es
	"proporcional al n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	"proportionnel au nombre d'&eacute;tudiants avec photo"
#elif L==6	// gn
	"proporcional al n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"proporzionale al numero di studenti con foto"
#elif L==8	// pl
	"proporcjonalna do liczby uczni&oacute;w ze zdj&eogon;ciem"
#elif L==9	// pt
	"proporcional ao n&uacute;mero de estudantes com foto"
#elif L==10	// tr
	"proportional to the number of students with photo"		// eviri lazim!
#endif
	,
	[Pho_PROPORTIONAL_TO_PERCENT] =
#if   L==1	// ca
	"proporcional al percentatge d'estudiants amb fotografia"
#elif L==2	// de
	"proportional zum Prozentsatz der Sch&uuml;ler mit Foto"
#elif L==3	// en
	"proportional to the percentage of students with photo"
#elif L==4	// es
	"proporcional al porcentaje de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	"proportionnel au pourcentage d'&eacute;tudiants avec photo"
#elif L==6	// gn
	"proporcional al porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"proporzionale alla percentuale di studenti con foto"
#elif L==8	// pl
	"proporcjonalna do odsetka uczni&oacute;w ze zdj&eogon;ciem"
#elif L==9	// pt
	"proporcional ao porcentagem de estudantes com foto"
#elif L==10	// tr
	"proportional to the percentage of students with photo"			// eviri lazim!
#endif
	,
	[Pho_UNIQUE_SIZE] =
#if   L==1	// ca
	"igual per a totes les titulacions"
#elif L==2	// de
	"f&uuml;r alle Abschl&uuml;sse gleich"
#elif L==3	// en
	"same for all degrees"
#elif L==4	// es
	"igual para todas las titulaciones"
#elif L==5	// fr
	"identique pour tous les &eacute;tudes"
#elif L==6	// gn
	"igual para todas las titulaciones"	// Okoteve traduccin
#elif L==7	// it
	"uguale per tutte le lauree"
#elif L==8	// pl
	"to samo dla wszystkich stopni"
#elif L==9	// pt
	"mesmo para todos os graus"
#elif L==10	// tr
	"same for all degrees"			// eviri lazim!
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_ORDER[Pho_NUM_HOW_ORDER_DEGREES] =
	{
	[Pho_PROPORTIONAL_TO_NUMBER_OF_STUDENTS] =
#if   L==1	// ca
	"nombre d'estudiants"
#elif L==2	// de
	"Anzahl der Studenten"
#elif L==3	// en
	"number of students"
#elif L==4	// es
	"n&ordm; de estudiantes"
#elif L==5	// fr
	"nombre d'&eacute;tudiants"
#elif L==6	// gn
	"n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==7	// it
	"numero di studenti"
#elif L==8	// pl
	"liczba student&oacute;w"
#elif L==9	// pt
	"n&uacute;mero de estudantes"
#elif L==10	// tr
	"number of students"		// eviri lazim!
#endif
	,
	[Pho_PROPORTIONAL_TO_NUMBER_OF_PHOTOS] =
#if   L==1	// ca
	"nombre d'estudiants amb fotografia"
#elif L==2	// de
	"Anzahl der Studenten mit Foto"
#elif L==3	// en
	"number of students with photo"
#elif L==4	// es
	"n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	"nombre d'&eacute;tudiants avec photo"
#elif L==6	// gn
	"n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"numero di studenti con foto"
#elif L==8	// pl
	"liczba student&oacute;w ze zdj&eogon;ciem"
#elif L==9	// pt
	"n&uacute;mero de estudantes com foto"
#elif L==10	// tr
	"number of students with photo"			// eviri lazim!
#endif
	,
	[Pho_PROPORTIONAL_TO_PERCENT] =
#if   L==1	// ca
	"percentatge d'estudiants amb fotografia"
#elif L==2	// de
	"Prozentsatz der Sch&uuml;ler mit Foto"
#elif L==3	// en
	"percentage of students with photo"
#elif L==4	// es
	"porcentaje de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	"pourcentage d'&eacute;tudiants avec photo"
#elif L==6	// gn
	"porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"percentuale di studenti con foto"
#elif L==8	// pl
	"odsetek uczni&oacute;w ze zdj&eogon;ciem"
#elif L==9	// pt
	"porcentagem de estudantes com foto"
#elif L==10	// tr
	"percentage of students with photo"			// eviri lazim!
#endif
	,
	[Pho_UNIQUE_SIZE] =
#if   L==1	// ca
	"nom de titulaci&oacute;"
#elif L==2	// de
	"Grad Name"
#elif L==3	// en
	"degree name"
#elif L==4	// es
	"nombre de titulaci&oacute;n"
#elif L==5	// fr
	"nom d'&eacute;tude"
#elif L==6	// gn
	"nombre de titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"nome laurea"
#elif L==8	// pl
	"nazwa stopnia"
#elif L==9	// pt
	"nome do grau"
#elif L==10	// tr
	"degree name"			// eviri lazim!
#endif
	};

const char *Txt_STAT_COUNT_TYPE[Sta_NUM_COUNT_TYPES] =
	{
	[Sta_TOTAL_CLICKS] =
#if   L==1	// ca
	"Nombre total de clics"
#elif L==2	// de
	"Gesamtzahl der Klicks"
#elif L==3	// en
	"Total number of clicks"
#elif L==4	// es
	"N.&ordm; total de clics"
#elif L==5	// fr
	"Nombre total de clics"
#elif L==6	// gn
	"N.&ordm; total de clics"	// Okoteve traduccin
#elif L==7	// it
	"Numero totale di scatti"
#elif L==8	// pl
	"Ca&lstrok;kowita liczba klikni&eogon;&cacute;"
#elif L==9	// pt
	"N&uacute;mero total de cliques"
#elif L==10	// tr
	"Total number of clicks"	// eviri lazim!
#endif
	,
	[Sta_DISTINCT_USRS] =
#if   L==1	// ca
	"Nombre d'usuaris diferents"
#elif L==2	// de
	"Anzahl der unterschiedlichen Benutzer"
#elif L==3	// en
	"Number of distinct users"
#elif L==4	// es
	"N.&ordm; de usuarios distintos"
#elif L==5	// fr
	"Nombre des diff&eacute;rents utilisateurs"
#elif L==6	// gn
	"N.&ordm; de usuarios distintos"	// Okoteve traduccin
#elif L==7	// it
	"Numero di utenti distinti"
#elif L==8	// pl
	"Liczba r&oacute;&zdot;nych u&zdot;ytkownik&oacute;w"
#elif L==9	// pt
	"N&uacute;mero de utilizadores diferentes"
#elif L==10	// tr
	"Number of distinct users"		// eviri lazim!
#endif
	,
	[Sta_CLICKS_PER_USR] =
#if   L==1	// ca
	"Nombre de clics per usuari"
#elif L==2	// de
	"Anzahl der Klicks pro Benutzer"
#elif L==3	// en
	"Number of clicks per user"
#elif L==4	// es
	"N.&ordm; de clics por usuario"
#elif L==5	// fr
	"Nombre de clics par utilisateur"
#elif L==6	// gn
	"N.&ordm; de clics por usuario"		// Okoteve traduccin
#elif L==7	// it
	"Numero di scatti per utente"
#elif L==8	// pl
	"Liczba klikni&eogon;&cacute; na u&zdot;ytkownika"
#elif L==9	// pt
	"N&uacute;mero de cliques por utilizador"
#elif L==10	// tr
	"Number of clicks per user"		// eviri lazim!
#endif
	,
	[Sta_GENERATION_TIME] =
#if   L==1	// ca
	"Temps de generaci&oacute; de p&agrave;gina"
#elif L==2	// de
	"Zeit zur Erzeugung der Seite"
#elif L==3	// en
	"Time to generate page"
#elif L==4	// es
	"Tiempo de generaci&oacute;n de p&aacute;gina"
#elif L==5	// fr
	"Temps pour g&eacute;n&eacute;rer la page"
#elif L==6	// gn
	"Tiempo de generaci&oacute;n de p&aacute;gina"	// Okoteve traduccin
#elif L==7	// it
	"Tempo per generare pagina"
#elif L==8	// pl
	"Czas wygenerowa&cacute; stron&eogon;"
#elif L==9	// pt
	"Tempo para gerar p&aacute;gina"
#elif L==10	// tr
	"Time to generate page"				// eviri lazim!
#endif
	,
	[Sta_SEND_TIME] =
#if   L==1	// ca
	"Temps d'enviament de p&agrave;gina"
#elif L==2	// de
	"Zeit, um die Seite zu senden"
#elif L==3	// en
	"Time to send page"
#elif L==4	// es
	"Tiempo de env&iacute;o de p&aacute;gina"
#elif L==5	// fr
	"Temps pour envoyer la page"
#elif L==6	// gn
	"Tiempo de env&iacute;o de p&aacute;gina"	// Okoteve traduccin
#elif L==7	// it
	"Tempo d'invio della pagina"
#elif L==8	// pl
	"Czas wys&lstrok;a&cacute; stron&eogon;"
#elif L==9	// pt
	"Tempo para enviar a p&aacute;gina"
#elif L==10	// tr
	"Time to send page"				// eviri lazim!
#endif
	};

const char *Txt_STAT_TYPE_COUNT_SMALL[Sta_NUM_COUNT_TYPES] =
	{
	[Sta_TOTAL_CLICKS] =
#if   L==1	// ca
	"nombre total de clics"
#elif L==2	// de
	"Gesamtzahl der Klicks"
#elif L==3	// en
	"total number of clicks"
#elif L==4	// es
	"n&ordm; total de clics"
#elif L==5	// fr
	"nombre total de clics"
#elif L==6	// gn
	"n&ordm; total de clics"	// Okoteve traduccin
#elif L==7	// it
	"numero totale di scatti"
#elif L==8	// pl
	"ca&lstrok;kowita liczba klikni&eogon;&cacute;"
#elif L==9	// pt
	"n&uacute;mero total de cliques"
#elif L==10	// tr
	"total number of clicks"	// eviri lazim!
#endif
	,
	[Sta_DISTINCT_USRS] =
#if   L==1	// ca
	"nombre d'usuaris diferents"
#elif L==2	// de
	"Anzahl der unterschiedliche Benutzer"
#elif L==3	// en
	"number of distinct users"
#elif L==4	// es
	"n&ordm; de usuarios distintos"
#elif L==5	// fr
	"nombre des diff&eacute;rents utilisateurs"
#elif L==6	// gn
	"n&ordm; de usuarios distintos"		// Okoteve traduccin
#elif L==7	// it
	"numero di utenti distinti"
#elif L==8	// pl
	"liczba r&oacute;&zdot;nych u&zdot;ytkownik&oacute;w"
#elif L==9	// pt
	"n&uacute;mero de utilizadores diferentes"
#elif L==10	// tr
	"number of distinct users"		// eviri lazim!
#endif
	,
 	[Sta_CLICKS_PER_USR] =
#if   L==1	// ca
	"nombre de clics per usuari"
#elif L==2	// de
	"Anzahl der Klicks pro Benutzer"
#elif L==3	// en
	"number of clicks per user"
#elif L==4	// es
	"n&ordm; de clics por usuario"
#elif L==5	// fr
	"nombre de clics par utilisateur"
#elif L==6	// gn
	"n&ordm; de clics por usuario"	// Okoteve traduccin
#elif L==7	// it
	"numero di scatti per utente"
#elif L==8	// pl
	"liczba klikni&eogon;&cacute; na u&zdot;ytkownika"
#elif L==9	// pt
	"n&uacute;mero de cliques por utilizador"
#elif L==10	// tr
	"number of clicks per user"	// eviri lazim!
#endif
	,
	[Sta_GENERATION_TIME] =
#if   L==1	// ca
	"temps de generaci&oacute;"
#elif L==2	// de
	"Zeit zur Erzeugung"
#elif L==3	// en
	"time to generate"
#elif L==4	// es
	"tiempo de generaci&oacute;n"
#elif L==5	// fr
	"temps pour g&eacute;n&eacute;rer"
#elif L==6	// gn
	"tiempo de generaci&oacute;n"		// Okoteve traduccin
#elif L==7	// it
	"tempo per generare"
#elif L==8	// pl
	"czas na wygenerowanie"
#elif L==9	// pt
	"tempo para gerar"
#elif L==10	// tr
	"time to generate"			// eviri lazim!
#endif
	,
	[Sta_SEND_TIME] =
#if   L==1	// ca
	"temps d'enviament"
#elif L==2	// de
	"Zeit, um Seite"
#elif L==3	// en
	"time to send"
#elif L==4	// es
	"tiempo de env&iacute;o"
#elif L==5	// fr
	"temps pour envoyer"
#elif L==6	// gn
	"tiempo de env&iacute;o"	// Okoteve traduccin
#elif L==7	// it
	"tempo d'invio"
#elif L==8	// pl
	"czas wys&lstrok;a&cacute;"
#elif L==9	// pt
	"Tempo para enviar"
#elif L==10	// tr
	"time to send"			// eviri lazim!
#endif
	};

const char *Txt_FIGURE_TYPES[Fig_NUM_FIGURES] =
	{
	[Fig_USERS] =
#if   L==1	// ca
	"Usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarios"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Puruh&aacute;ra"
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"U&zdot;ytkownicy"
#elif L==9	// pt
	"Utilizadores"
#elif L==10	// tr
	"Users"	// eviri lazim!
#endif
	,
	[Fig_USERS_RANKING] =
#if   L==1	// ca
	"R&agrave;nquing"
#elif L==2	// de
	"Rang"
#elif L==3	// en
	"Ranking"
#elif L==4	// es
	"Ranking"
#elif L==5	// fr
	"Classement"
#elif L==6	// gn
	"Ranking"	// Okoteve traduccin
#elif L==7	// it
	"Posto"
#elif L==8	// pl
	"Ranking"
#elif L==9	// pt
	"Posi&ccedil;&atilde;o"
#elif L==10	// tr
	"Ranking"	// eviri lazim!
#endif
	,
	[Fig_HIERARCHY] =
#if   L==1	// ca
	"Jerarquia"
#elif L==2	// de
	"Hierarchie"
#elif L==3	// en
	"Hierarchy"
#elif L==4	// es
	"Jerarqu&iacute;a"
#elif L==5	// fr
	"Hi&eacute;rarchie"
#elif L==6	// gn
	"S&atilde;mbyhyha reko"
#elif L==7	// it
	"Gerarchia"
#elif L==8	// pl
	"Hierarchia"
#elif L==9	// pt
	"Hierarquia"
#elif L==10	// tr
	"Hierarchy"	// eviri lazim!
#endif
	,
	[Fig_INSTITS] =
#if   L==1	// ca
	"Institucions"
#elif L==2	// de
	"Hochschulen"
#elif L==3	// en
	"Institutions"
#elif L==4	// es
	"Instituciones"
#elif L==5	// fr
	"&Eacute;tablissements"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzioni"
#elif L==8	// pl
	"Instytucje"
#elif L==9	// pt
	"Institu&ccedil;&otilde;es"
#elif L==10	// tr
	"Institutions"	// eviri lazim!
#endif
	,
	[Fig_DEGREE_TYPES] =
#if   L==1	// ca
	"Tipus de titulaci&oacute;"
#elif L==2	// de
	"Abschlussart"
#elif L==3	// en
	"Types of degree"
#elif L==4	// es
	"Tipos de titulaci&oacute;n"
#elif L==5	// fr
	"Types d'&eacute;tude"
#elif L==6	// gn
	"Arandur&atilde; l&aacute;ja"
#elif L==7	// it
	"Tipi di laurea"
#elif L==8	// pl
	"Rodzaje stopni"
#elif L==9	// pt
	"Tipos de grau"
#elif L==10	// tr
	"Types of degree"	// eviri lazim!
#endif
	,
	[Fig_FOLDERS_AND_FILES] =
#if   L==1	// ca
	"Carpetes i arxius"
#elif L==2	// de
	"Verzeichnisse und Dateien"
#elif L==3	// en
	"Folders and files"
#elif L==4	// es
	"Carpetas y archivos"
#elif L==5	// fr
	"R&eacute;pertoires et fichiers"
#elif L==6	// gn
	"Carpetas y archivos"	// Okoteve traduccin
#elif L==7	// it
	"Cartelle e file"
#elif L==8	// pl
	"Foldery i pliki"
#elif L==9	// pt
	"Diret&oacute;rios e arquivos"
#elif L==10	// tr
	"Folders and files"	// eviri lazim!
#endif
	,
	[Fig_OER] =
#if   L==1	// ca
	"Recursos Educatius Oberts (OER)"
#elif L==2	// de
	"Open Educational Resources (OER)"
#elif L==3	// en
	"Open Educational Resources (OER)"
#elif L==4	// es
	"Recursos Educativos Abiertos (OER)"
#elif L==5	// fr
	"Ressources &Eacute;ducatives Libres (REL)"
#elif L==6	// gn
	"Recursos Educativos Abiertos (OER)"	// Okoteve traduccin
#elif L==7	// it
	"Risorse Didattiche Aperte (OER)"
#elif L==8	// pl
	"Otwarte Zasoby Edukacyjne (OER)"
#elif L==9	// pt
	"Recursos Educacionais Abertos (OER)"
#elif L==10	// tr
	"Open Educational Resources (OER)"	// eviri lazim!
#endif
	,
	[Fig_COURSE_PROGRAMS] =
#if   L==1	// ca
	"Programa"
#elif L==2	// de
	"Programm"
#elif L==3	// en
	"Program"
#elif L==4	// es
	"Programa"
#elif L==5	// fr
	"Programme"
#elif L==6	// gn
	"Programa"	// Okoteve traduccin
#elif L==7	// it
	"Programma"
#elif L==8	// pl
	"Program"
#elif L==9	// pt
	"Programa"
#elif L==10	// tr
	"Program&inodot;"
#endif
	,
	[Fig_ASSIGNMENTS] =
#if   L==1	// ca
	"Activitats"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"Assignments"
#elif L==4	// es
	"Actividades"
#elif L==5	// fr
	"Activit&eacute;s"
#elif L==6	// gn
	"Tar&eacute;a"
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Zadania"
#elif L==9	// pt
	"Atividades"
#elif L==10	// tr
	"&Ouml;devler"
#endif
	,
	[Fig_PROJECTS] =
#if   L==1	// ca
	"Projectes"
#elif L==2	// de
	"Projekte"
#elif L==3	// en
	"Projects"
#elif L==4	// es
	"Proyectos"
#elif L==5	// fr
	"Projets"
#elif L==6	// gn
	"Proyectos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti"
#elif L==8	// pl
	"Projekty"
#elif L==9	// pt
	"Projetos"
#elif L==10	// tr
	"Projeler"
#endif
	,
	[Fig_TESTS] =
#if   L==1	// ca
	"Tests"
#elif L==2	// de
	"Tests"
#elif L==3	// en
	"Tests"
#elif L==4	// es
	"Test"
#elif L==5	// fr
	"Tests"
#elif L==6	// gn
	"Test"		// Okoteve traduccin
#elif L==7	// it
	"Test"
#elif L==8	// pl
	"Testy"
#elif L==9	// pt
	"Testes"
#elif L==10	// tr
	"Testler"
#endif
	,
	[Fig_EXAMS] =
#if   L==1	// ca
	"Ex&agrave;mens"
#elif L==2	// de
	"Pr&uuml;fungen"
#elif L==3	// en
	"Exams"
#elif L==4	// es
	"Ex&aacute;menes"
#elif L==5	// fr
	"Examens"
#elif L==6	// gn
	"Ex&aacute;menes"	// Okoteve traduccin
#elif L==7	// it
	"Esami"
#elif L==8	// pl
	"Egzaminy"
#elif L==9	// pt
	"Exames"
#elif L==10	// tr
	"S&inodot;navlar"
#endif
	,
	[Fig_GAMES] =
#if   L==1	// ca
	"Jocs"
#elif L==2	// de
	"Spiele"
#elif L==3	// en
	"Games"
#elif L==4	// es
	"Juegos"
#elif L==5	// fr
	"Jeux"
#elif L==6	// gn
	"Juegos"	// Okoteve traduccin
#elif L==7	// it
	"Giochi"
#elif L==8	// pl
	"Gry"
#elif L==9	// pt
	"Jogos"
#elif L==10	// tr
	"Oyunlar"
#endif
	,
	[Fig_RUBRICS] =
#if   L==1	// ca
	"R&uacute;briques"
#elif L==2	// de
	"Rubriken"
#elif L==3	// en
	"Rubrics"
#elif L==4	// es
	"R&uacute;bricas"
#elif L==5	// fr
	"Rubriques"
#elif L==6	// gn
	"R&uacute;bricas"	// Okoteve traduccin
#elif L==7	// it
	"Rubriche"
#elif L==8	// pl
	"Rubryki"
#elif L==9	// pt
	"Rubricas"
#elif L==10	// tr
	"De&gbreve;erlendirme"
#endif
	,
	[Fig_TIMELINE] =
#if   L==1	// ca
	"Timeline"
#elif L==2	// de
	"Timeline"
#elif L==3	// en
	"Timeline"
#elif L==4	// es
	"Timeline"
#elif L==5	// fr
	"Chronologie"
#elif L==6	// gn
	"Timeline"	// Okoteve traduccin
#elif L==7	// it
	"Timeline"
#elif L==8	// pl
	"Timeline"
#elif L==9	// pt
	"Timeline"
#elif L==10	// tr
	"Timeline"	// eviri lazim!
#endif
	,
	[Fig_FOLLOW] =
#if   L==1	// ca
	"Seguits / Seguidors"
#elif L==2	// de
	"Gefolgt / Anh&auml;nger"
#elif L==3	// en
	"Followed / Followers"
#elif L==4	// es
	"Seguidos / Seguidores"
#elif L==5	// fr
	"Suivis / Suiveurs"
#elif L==6	// gn
	"Seguidos / Seguidores"	// Okoteve traduccin
#elif L==7	// it
	"Seguiti / Seguaci"
#elif L==8	// pl
	"Obserwowani / Obserwuj&aogon;"
#elif L==9	// pt
	"Seguidos / Seguidores"
#elif L==10	// tr
	"Followed / Followers"	// eviri lazim!
#endif
	,
	[Fig_FORUMS] =
#if   L==1	// ca
	"F&ograve;rums"
#elif L==2	// de
	"Foren"
#elif L==3	// en
	"Forums"
#elif L==4	// es
	"Foros"
#elif L==5	// fr
	"Forums"
#elif L==6	// gn
	"Foros"		// Okoteve traduccin
#elif L==7	// it
	"Forum"
#elif L==8	// pl
	"Forum"
#elif L==9	// pt
	"F&oacute;runs"
#elif L==10	// tr
	"Forumlar"
#endif
	,
	[Fig_NOTIFY_EVENTS] =
#if   L==1	// ca
	"Notificacions enviades per correu"
#elif L==2	// de
	"Anmeldungen per E-Mail gesendet"
#elif L==3	// en
	"Notifications sent by email"
#elif L==4	// es
	"Notificaciones enviadas por correo"
#elif L==5	// fr
	"Notifications envoy&eacute;es par courriel"
#elif L==6	// gn
	"Notificaciones enviadas por correo"	// Okoteve traduccin
#elif L==7	// it
	"Notifiche inviate via email"
#elif L==8	// pl
	"Powiadomienia wysy&lstrok;ane przez email"
#elif L==9	// pt
	"Notifica&ccedil;&otilde;es enviadas por email"
#elif L==10	// tr
	"Notifications sent by email"		// eviri lazim!
#endif
	,
	[Fig_NOTICES] =
#if   L==1	// ca
	"Avisos"
#elif L==2	// de
	"Ank&uuml;ndigungen"
#elif L==3	// en
	"Notices"
#elif L==4	// es
	"Avisos"
#elif L==5	// fr
	"Avis"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Avvisi"
#elif L==8	// pl
	"Ogloszenia"
#elif L==9	// pt
	"Avisos"
#elif L==10	// tr
	"Bildirimler"
#endif
	,
	[Fig_MESSAGES] =
#if   L==1	// ca
	"Missatges"
#elif L==2	// de
	"Nachrichten"
#elif L==3	// en
	"Messages"
#elif L==4	// es
	"Mensajes"
#elif L==5	// fr
	"Messages"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Messaggi"
#elif L==8	// pl
	"Wiadomo&sacute;ci"
#elif L==9	// pt
	"Mensagens"
#elif L==10	// tr
	"Mesajlar"
#endif
	,
	[Fig_SURVEYS] =
#if   L==1	// ca
	"Enquestes"
#elif L==2	// de
	"Umfragen"
#elif L==3	// en
	"Surveys"
#elif L==4	// es
	"Encuestas"
#elif L==5	// fr
	"Sondages"
#elif L==6	// gn
	"Encuestas"	// Okoteve traduccin
#elif L==7	// it
	"Sondaggi"
#elif L==8	// pl
	"Ankieta"
#elif L==9	// pt
	"Inqu&eacute;ritos"
#elif L==10	// tr
	"Anketler"
#endif
	,
	[Fig_AGENDAS] =
#if   L==1	// ca
	"Agendes"
#elif L==2	// de
	"Agenden"
#elif L==3	// en
	"Agendas"
#elif L==4	// es
	"Agendas"
#elif L==5	// fr
	"Agendas"
#elif L==6	// gn
	"Aporyr&atilde;"
#elif L==7	// it
	"Agende"
#elif L==8	// pl
	"Pami&eogon;tnik"
#elif L==9	// pt
	"Agendas"
#elif L==10	// tr
	"Agendas"	// eviri lazim!
#endif
	,
	[Fig_SOCIAL_NETWORKS] =
#if   L==1	// ca
	"Webs / xarxes socials"
#elif L==2	// de
	"Webs / Social Networks"
#elif L==3	// en
	"Webs / social networks"
#elif L==4	// es
	"Webs / redes sociales"
#elif L==5	// fr
	"Webs / r&eacute;seaux sociaux"
#elif L==6	// gn
	"Webs / redes sociales"		// Okoteve traduccin
#elif L==7	// it
	"Webs / reti sociali"
#elif L==8	// pl
	"Webs / sieci spo&lstrok;eczne"
#elif L==9	// pt
	"Webs / redes sociais"
#elif L==10	// tr
	"Webs / social networks"	// eviri lazim!
#endif
	,
	[Fig_LANGUAGES] =
#if   L==1	// ca
	"Idioma"
#elif L==2	// de
	"Sprache"
#elif L==3	// en
	"Language"
#elif L==4	// es
	"Idioma"
#elif L==5	// fr
	"Langue"
#elif L==6	// gn
	"&Ntilde;e'&#7869;"
#elif L==7	// it
	"Lingua"
#elif L==8	// pl
	"Language"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Lingua"
#elif L==10	// tr
	"Dil"
#endif
	,
	[Fig_FIRST_DAY_OF_WEEK] =
#if   L==1	// ca
	"Calendari"
#elif L==2	// de
	"Kalender"
#elif L==3	// en
	"Calendar"
#elif L==4	// es
	"Calendario"
#elif L==5	// fr
	"Calendrier"
#elif L==6	// gn
	"Arapapaha"
#elif L==7	// it
	"Calendario"
#elif L==8	// pl
	"Kalendarz"
#elif L==9	// pt
	"Calend&aacute;rio"
#elif L==10	// tr
	"Takvim"
#endif
	,
	[Fig_DATE_FORMAT] =
#if   L==1	// ca
	"Dates"
#elif L==2	// de
	"Datum"
#elif L==3	// en
	"Dates"
#elif L==4	// es
	"Fechas"
#elif L==5	// fr
	"Dates"
#elif L==6	// gn
	"Arange"
#elif L==7	// it
	"Date"
#elif L==8	// pl
	"Daty"
#elif L==9	// pt
	"Datas"
#elif L==10	// tr
	"Tarihleri"
#endif
	,
	[Fig_ICON_SETS] =
#if   L==1	// ca
	"Icones"
#elif L==2	// de
	"Symbole"
#elif L==3	// en
	"Icons"
#elif L==4	// es
	"Iconos"
#elif L==5	// fr
	"Ic&ocirc;nes"
#elif L==6	// gn
	"Iconos"		// Okoteve traduccin
#elif L==7	// it
	"Icone"
#elif L==8	// pl
	"Ikony"
#elif L==9	// pt
	"&Iacute;cones"
#elif L==10	// tr
	"Simgeler"
#endif
	,
	[Fig_MENUS] =
#if   L==1	// ca
	"Men&uacute;"
#elif L==2	// de
	"Men&uuml;"
#elif L==3	// en
	"Menu"
#elif L==4	// es
	"Men&uacute;"
#elif L==5	// fr
	"Menu"
#elif L==6	// gn
	"Men&uacute;"	// Okoteve traduccin
#elif L==7	// it
	"Menu"
#elif L==8	// pl
	"Menu"
#elif L==9	// pt
	"Menu"
#elif L==10	// tr
	"Men&uuml;"
#endif
	,
	[Fig_THEMES] =
#if   L==1	// ca
	"Tema (colors)"
#elif L==2	// de
	"Skin"
#elif L==3	// en
	"Theme"
#elif L==4	// es
	"Tema (colores)"
#elif L==5	// fr
	"Th&egrave;me"
#elif L==6	// gn
	"Tema (colores)"	// Okoteve traduccin
#elif L==7	// it
	"Tema"
#elif L==8	// pl
	"Temat"
#elif L==9	// pt
	"Skin"
#elif L==10	// tr
	"Tema"
#endif
	,
	[Fig_SIDE_COLUMNS] =
#if   L==1	// ca
	"Columnes"
#elif L==2	// de
	"Spalten"
#elif L==3	// en
	"Columns"
#elif L==4	// es
	"Columnas"
#elif L==5	// fr
	"Colonnes"
#elif L==6	// gn
	"Columnas"	// Okoteve traduccin
#elif L==7	// it
	"Colonne"
#elif L==8	// pl
	"Kolumny"
#elif L==9	// pt
	"Colunas"
#elif L==10	// tr
	"S&uuml;tunlar"
#endif
	,
	[Fig_PHOTO_SHAPES] =
#if   L==1	// ca
	"Fotos d'usuaris"
#elif L==2	// de
	"Benutzerfotos"
#elif L==3	// en
	"User photos"
#elif L==4	// es
	"Fotos de usuarios"
#elif L==5	// fr
	"Photos des utilisateurs"
#elif L==6	// gn
	"Fotos de usuarios"		// Okoteve traduccin
#elif L==7	// it
	"Foto utente"
#elif L==8	// pl
	"Zdj&eogon;cia u&zdot;ytkownika "
#elif L==9	// pt
	"Fotos de utilizadores"
#elif L==10	// tr
	"Kullan&inodot;c&inodot; foto&gbreve;raflar&inodot;"
#endif
	,
	[Fig_PRIVACY] =
#if   L==1	// ca
	"Privacitat"
#elif L==2	// de
	"Geheimhaltung"
#elif L==3	// en
	"Privacy"
#elif L==4	// es
	"Privacidad"
#elif L==5	// fr
	"Privacit&eacute;"
#elif L==6	// gn
	"Privacidad"	// Okoteve traduccin
#elif L==7	// it
	"Privatezza"
#elif L==8	// pl
	"Prywatno&sacute;&cacute;"
#elif L==9	// pt
	"Privacidade"
#elif L==10	// tr
	"Mahremiyet"
#endif
	,
	[Fig_COOKIES] =
#if   L==1	// ca
	"Cookies"
#elif L==2	// de
	"Cookies"
#elif L==3	// en
	"Cookies"
#elif L==4	// es
	"Cookies"
#elif L==5	// fr
	"Cookies"
#elif L==6	// gn
	"Cookies"
#elif L==7	// it
	"Cookies"
#elif L==8	// pl
	"Cookies"
#elif L==9	// pt
	"Cookies"
#elif L==10	// tr
	"&Ccedil;erezleri"
#endif
	};

const char *Txt_Start =	// As a verb
#if   L==1	// ca
	"Comen&ccedil;ar";
#elif L==2	// de
	"Starten";
#elif L==3	// en
	"Start";
#elif L==4	// es
	"Comenzar";
#elif L==5	// fr
	"Commencer";
#elif L==6	// gn
	"&Ntilde;epyr&utilde;";
#elif L==7	// it
	"Inizia";
#elif L==8	// pl
	"Rozpocznij";
#elif L==9	// pt
	"Come&ccedil;ar";
#elif L==10	// tr
	"Ba&scedil;lama";
#endif

const char *Txt_Statistic =
#if   L==1	// ca
	"Estad&iacute;stique";
#elif L==2	// de
	"Statistik";
#elif L==3	// en
	"Statistic";
#elif L==4	// es
	"Estad&iacute;stica";
#elif L==5	// fr
	"Statistique";
#elif L==6	// gn
	"Papy'aporeko";
#elif L==7	// it
	"Statistica";
#elif L==8	// pl
	"Statystyki";
#elif L==9	// pt
	"Estat&iacute;sticas";
#elif L==10	// tr
	"Statistic";	// eviri lazim!
#endif

const char *Txt_Statistics_of_all_visits =
#if   L==1	// ca
	"Estad&iacute;stica de tots els accessos";
#elif L==2	// de
	"Statistik aller Besuche";
#elif L==3	// en
	"Statistics of all visits";
#elif L==4	// es
	"Estad&iacute;stica de todos los accesos";
#elif L==5	// fr
	"Statistiques de toutes les visites";
#elif L==6	// gn
	"Estad&iacute;stica opaite jeike rehegua";
#elif L==7	// it
	"Statistiche di tutte gli accessi";
#elif L==8	// pl
	"Statystyki wszystkich wizyt";
#elif L==9	// pt
	"Estat&iacute;sticas de todas as visitas";
#elif L==10	// tr
	"T&uuml;m ziyaretlerin istatistikleri";
#endif

const char *Txt_Statistics_of_visits_to_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Estad&iacute;stica d'accessos a l'assignatura %s";
#elif L==2	// de
	"Statistik der Besuche des Kurses %s";
#elif L==3	// en
	"Statistics of visits to the course %s";
#elif L==4	// es
	"Estad&iacute;stica de accesos a la asignatura %s";
#elif L==5	// fr
	"Statistiques de visites a la mati&egrave;re %s";
#elif L==6	// gn
	"Estad&iacute;stica de accesos a la asignatura %s";	// Okoteve traduccin
#elif L==7	// it
	"Statistiche delle visite al corso %s";
#elif L==8	// pl
	"Statystyki odwiedzin kursu %s";
#elif L==9	// pt
	"Estat&iacute;sticas de visitas &agrave; disciplina %s";
#elif L==10	// tr
	"Statistics of visits to the course %s";		// eviri lazim!
#endif

const char *Txt_Status =
#if   L==1	// ca
	"Estat";
#elif L==2	// de
	"Status";
#elif L==3	// en
	"Status";
#elif L==4	// es
	"Estado";
#elif L==5	// fr
	"Statut";
#elif L==6	// gn
	"O&#297; h&aacute;icha";
#elif L==7	// it
	"Stato";
#elif L==8	// pl
	"Status";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Status";
#elif L==10	// tr
	"Status";	// eviri lazim!
#endif

const char *Txt_Stay_connected_with_SWADroid =
#if   L==1	// ca
	"Seguiu connectat/da amb <strong>SWADroid</strong>.";
#elif L==2	// de
	"Bleiben Sie mit <strong>SWADroid</strong> in Verbindung.";
#elif L==3	// en
	"Stay connected with <strong>SWADroid</strong>.";
#elif L==4	// es
	"Siga conectado/a con <strong>SWADroid</strong>.";
#elif L==5	// fr
	"Restez connect&eacute; avec <strong>SWADroid</strong>.";
#elif L==6	// gn
	"Siga conectado/a con <strong>SWADroid</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Resta in contatto con <strong>SWADroid</strong>.";
#elif L==8	// pl
	"Pozosta&nacute; w kontakcie z <strong>SWADroid</strong>.";
#elif L==9	// pt
	"Fique ligado/a com <strong>SWADroid</strong>.";
#elif L==10	// tr
	"Stay connected with <strong>SWADroid</strong>.";	// eviri lazim!
#endif

const char *Txt_Step_1_Provide_a_list_of_users =
#if   L==1	// ca
	"Pas 1: Proporcioneu una llista d'usuaris";
#elif L==2	// de
	"Schritt 1: Geben Sie eine Liste der Benutzer an";
#elif L==3	// en
	"Step 1: Provide a list of users";
#elif L==4	// es
	"Paso 1: Proporcione una lista de usuarios";
#elif L==5	// fr
	"&Eacute;tape 1: Fournissez une liste d'utilisateurs";
#elif L==6	// gn
	"Paso 1: Proporcione una lista de usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Passo 1: Fornisci una lista di utenti";
#elif L==8	// pl
	"Krok 1: Podaj list&eogon; u&zdot;ytkownik&oacute;w";
#elif L==9	// pt
	"Etapa 1: Forne&ccedil;a uma lista de usu&aacute;rios";
#elif L==10	// tr
	"Step 1: Provide a list of users";		// eviri lazim!
#endif

const char *Txt_Step_2_Select_the_desired_action =
#if   L==1	// ca
	"Pas 2: Seleccioneu l'acci&oacute; que voleu realitzar";
#elif L==2	// de
	"Schritt 2: W&auml;hlen Sie die gew&uuml;nschte Aktion aus";
#elif L==3	// en
	"Step 2: Select the desired action";
#elif L==4	// es
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";
#elif L==5	// fr
	"&Eacute;tape 2 : S&eacute;lectionnez l'action souhait&eacute;e";
#elif L==6	// gn
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";	// Okoteve traduccin
#elif L==7	// it
	"Passo 2: Seleziona l'azione desiderata";
#elif L==8	// pl
	"Krok 2: Wybierz &zdot;&aogon;dan&aogon; akcj&eogon;";
#elif L==9	// pt
	"Etapa 2: Selecione a a&ccedil;&atilde;o desejada";
#elif L==10	// tr
	"Step 2: Select the desired action";				// eviri lazim!
#endif

const char *Txt_Step_3_Optionally_select_groups =
#if   L==1	// ca
	"Pas 3: Opcionalment, seleccioneu grups";
#elif L==2	// de
	"Schritt 3: W&auml;hlen Sie optional Gruppen aus";
#elif L==3	// en
	"Step 3: Optionally, select groups";
#elif L==4	// es
	"Paso 3: Opcionalmente, seleccione grupos";
#elif L==5	// fr
	"&Eacute;tape 3 : S&eacute;lectionnez &eacute;ventuellement des groupes";
#elif L==6	// gn
	"Paso 3: Opcionalmente, seleccione grupos";	// Okoteve traduccin
#elif L==7	// it
	"Passo 3: A scelta, seleziona i gruppi";
#elif L==8	// pl
	"Krok 3: Opcjonalnie wybierz grupy";
#elif L==9	// pt
	"Etapa 3: Opcionalmente, selecione grupos";
#elif L==10	// tr
	"Step 3: Optionally, select groups";		// eviri lazim!
#endif

const char *Txt_Step_4_Confirm_the_enrolment_removing =
#if   L==1	// ca
	"Pas 4: Confirmeu la inscripci&oacute; / eliminaci&oacute;";
#elif L==2	// de
	"Schritt 4: Best&auml;tigen Sie die Registrierung / Entfernung";
#elif L==3	// en
	"Step 4: Confirm the enrolment / removing";
#elif L==4	// es
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";
#elif L==5	// fr
	"&Eacute;tape 4 : Confirmer l'inscription / la suppression";
#elif L==6	// gn
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Passo 4: Conferma la registrazione / rimozione";
#elif L==8	// pl
	"Krok 4: Potwierd&zacute; rejestracj&eogon; / usuni&eogon;cie";
#elif L==9	// pt
	"Etapa 4: Confirmar a inscri&ccedil;&atilde;o / remo&ccedil;&atilde;o";
#elif L==10	// tr
	"Step 4: Confirm the enrolment / removing";			// eviri lazim!
#endif

const char *Txt_Student_comment =
#if   L==1	// ca
	"Comentari de l'estudiant";
#elif L==2	// de
	"Student-Kommentar";
#elif L==3	// en
	"Student's comment";
#elif L==4	// es
	"Comentario del estudiante";
#elif L==5	// fr
	"Commentaire de l'&eacute;tudiant";
#elif L==6	// gn
	"Temimbo'e he'&iacute;va";
#elif L==7	// it
	"Commento di student";
#elif L==8	// pl
	"Komentarz studenta";
#elif L==9	// pt
	"Coment&aacute;rio do estudante";
#elif L==10	// tr
	"&Ouml;&gbreve;rencinin yorumu";
#endif

const char *Txt_students_ABBREVIATION =	// Abbreviation of "students"
#if   L==1	// ca
	"estu.";
#elif L==2	// de
	"Stud.";
#elif L==3	// en
	"stud.";
#elif L==4	// es
	"estu.";
#elif L==5	// fr
	"&eacute;tu.";
#elif L==6	// gn
	"Temi.";
#elif L==7	// it
	"stud.";
#elif L==8	// pl
	"stud.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"estu.";
#elif L==10	// tr
	"stud.";	// eviri lazim!
#endif

const char *Txt_Student_record_card_in_this_course_has_been_updated =
#if   L==1	// ca
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Necessita traducci
#elif L==2	// de
	"Studentenkartei in diesem Kurs wurde aktualisiert.";
#elif L==3	// en
	"Student record card in this course has been updated.";
#elif L==4	// es
	"La ficha del estudiante en la asignatura se ha actualizado.";
#elif L==5	// fr
	"La fiche du &eacute;tudiant de cette mati&egrave;re a &eacute;t&eacute; mise &agrave; jour.";
#elif L==6	// gn
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"La scheda dello/a studente/ssa in questo corso &egrave; stata aggiornata.";
#elif L==8	// pl
	"Karta ewidencyjna ucznia na tym kursie zosta&lstrok;a zaktualizowana.";
#elif L==9	// pt
	"O cart&atilde;o do estudante nesta disciplina foi atualizado.";
#elif L==10	// tr
	"Student record card in this course has been updated.";		// eviri lazim!
#endif

const char *Txt_Students_are_no_longer_obliged_to_read_this_information =
#if   L==1	// ca
	"Els estudiants ja no estan obligats a llegir aquesta informaci&oacute;.";
#elif L==2	// de
	"Studierende sind nicht mehr verpflichtet, diese Informationen zu lesen.";
#elif L==3	// en
	"Students are no longer obliged to read this information.";
#elif L==4	// es
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==5	// fr
	"Les &eacute;tudiants ne sont plus oblig&eacute;s de lire ces informations.";
#elif L==6	// gn
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Gli studenti non sono obbligati a leggere questa informazione.";
#elif L==8	// pl
	"Studenci nie s&aogon; ju&zdot; zobowi&aogon;zani do czytania tych informacji.";
#elif L==9	// pt
	"Os estudantes n&atilde;o precisam mais ler essas informa&ccedil;&otilde;es.";
#elif L==10	// tr
	"Students are no longer obliged to read this information.";			// eviri lazim!
#endif

const char *Txt_Students_now_are_required_to_read_this_information =
#if   L==1	// ca
	"Ara els estudiants han de llegir aquesta informaci&oacute;.";
#elif L==2	// de
	"Die Sch&uuml;ler m&uuml;ssen diese Informationen jetzt lesen.";
#elif L==3	// en
	"Students now are required to read this information.";
#elif L==4	// es
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==5	// fr
	"Les &eacute;tudiants sont maintenant tenus de lire ces informations.";
#elif L==6	// gn
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Gli studenti ora sono obbligati a leggere questa informazione.";
#elif L==8	// pl
	"Uczniowie s&aogon; teraz zobowi&aogon;zani do przeczytania tych informacji.";
#elif L==9	// pt
	"Os estudantes agora precisam ler essas informa&ccedil;&otilde;es.";
#elif L==10	// tr
	"Students now are required to read this information.";				// eviri lazim!
#endif

const char *Txt_Surname_1 =
#if   L==1	// ca
	"Primer cognom";
#elif L==2	// de
	"Nachname 1";
#elif L==3	// en
	"Surname 1";
#elif L==4	// es
	"Primer apellido";
#elif L==5	// fr
	"Nom de famille 1";
#elif L==6	// gn
	"Terajoapy 1";
#elif L==7	// it
	"Primo cognome";
#elif L==8	// pl
	"Nazwisko 1";
#elif L==9	// pt
	"Primeiro apelido";
#elif L==10	// tr
	"Surname 1";	// eviri lazim!
#endif

const char *Txt_Surname_2 =
#if   L==1	// ca
	"Segon cognom";
#elif L==2	// de
	"Nachname 2";
#elif L==3	// en
	"Surname 2";
#elif L==4	// es
	"Segundo apellido";
#elif L==5	// fr
	"Nom de famille 2";
#elif L==6	// gn
	"Terajoapy 2";
#elif L==7	// it
	"Secondo cognome";
#elif L==8	// pl
	"Nazwisko 2";
#elif L==9	// pt
	"Segundo apelido";
#elif L==10	// tr
	"Surname 2";	// eviri lazim!
#endif

const char *Txt_Survey =
#if   L==1	// ca
	"Enquesta";
#elif L==2	// de
	"Umfrage";
#elif L==3	// en
	"Survey";
#elif L==4	// es
	"Encuesta";
#elif L==5	// fr
	"Sondage";
#elif L==6	// gn
	"Encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio";
#elif L==8	// pl
	"Ankieta";
#elif L==9	// pt
	"Inqu&eacute;rito";
#elif L==10	// tr
	"Anket";
#endif

const char *Txt_SURVEY_You_belong_to_the_scope_of_the_survey =
#if   L==1	// ca
	"Pertany a l'&agrave;mbit de l'enquesta";
#elif L==2	// de
	"Sie geh&ouml;ren zum Umfang der Umfrage";
#elif L==3	// en
	"You belong to the scope of the survey";
#elif L==4	// es
	"Pertenece al &aacute;mbito de la encuesta";
#elif L==5	// fr
	"Vous appartenez &agrave; la port&eacute;e de l'enqu&ecirc;te";
#elif L==6	// gn
	"Pertenece al &aacute;mbito de la encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Voi appartenete al campo d'indagine";
#elif L==8	// pl
	"Nale&zdot;ysz do zakresu badania";
#elif L==9	// pt
	"Voc&ecirc; pertence ao &acirc;mbito do inqu&eacute;rito";
#elif L==10	// tr
	"You belong to the scope of the survey";	// eviri lazim!
#endif

const char *Txt_SURVEY_You_dont_belong_to_the_scope_of_the_survey =
#if   L==1	// ca
	"No pertany a l'&agrave;mbit de l'enquesta";
#elif L==2	// de
	"Sie geh&ouml;ren nicht zum Umfang der Befragung";
#elif L==3	// en
	"You don't belong to the scope of the survey";
#elif L==4	// es
	"No pertenece al &aacute;mbito de la encuesta";
#elif L==5	// fr
	"Vous n'appartenez pas &agrave; la port&eacute;e de l'enqu&ecirc;te";
#elif L==6	// gn
	"No pertenece al &aacute;mbito de la encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Voi non appartieni al campo d'indagine";
#elif L==8	// pl
	"Nie nale&zdot;&aogon; do zakresu badania";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pertence ao &acirc;mbito do inqu&eacute;rito";
#elif L==10	// tr
	"You don't belong to the scope of the survey";	// eviri lazim!
#endif

const char *Txt_SURVEY_You_have_already_answered =
#if   L==1	// ca
	"Ja ha respost";
#elif L==2	// de
	"Sie haben bereits geantwortet";
#elif L==3	// en
	"You have already answered";
#elif L==4	// es
	"Ya ha respondido";
#elif L==5	// fr
	"Vous avez d&eacute;j&agrave; r&eacute;pondu";
#elif L==6	// gn
	"Ya ha respondido";		// Okoteve traduccin
#elif L==7	// it
	"Hai gi&agrave; risposto";
#elif L==8	// pl
	"Ju&zdot; odpowiedzia&lstrok;e&sacute;";
#elif L==9	// pt
	"Voc&ecirc; j&aacute; respondeu";
#elif L==10	// tr
	"You have already answered";	// eviri lazim!
#endif

const char *Txt_SURVEY_You_have_not_answered =
#if   L==1	// ca
	"No ha respost";
#elif L==2	// de
	"Sie haben nicht geantwortet";
#elif L==3	// en
	"You have not answered";
#elif L==4	// es
	"No ha respondido";
#elif L==5	// fr
	"Vous n'avez pas r&eacute;pondu";
#elif L==6	// gn
	"No ha respondido";		// Okoteve traduccin
#elif L==7	// it
	"Non hai risposto";
#elif L==8	// pl
	"Nie odpowiedzia&lstrok;e&sacute;";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o respondeu";
#elif L==10	// tr
	"You have not answered";	// eviri lazim!
#endif

const char *Txt_SURVEY_Type_of_user_allowed =
#if   L==1	// ca
	"Tipus d'usuari perm&egrave;s";
#elif L==2	// de
	"Zul&auml;ssiger Benutzertyp";
#elif L==3	// en
	"Type of user allowed";
#elif L==4	// es
	"Tipo de usuario permitido";
#elif L==5	// fr
	"Type d'utilisateur autoris&eacute;";
#elif L==6	// gn
	"Tipo de usuario permitido";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di utente autorizzato";
#elif L==8	// pl
	"Dozwolony typ u&zdot;ytkownika";
#elif L==9	// pt
	"Tipo de usu&aacute;rio permitido";
#elif L==10	// tr
	"Type of user allowed";		// eviri lazim!
#endif

const char *Txt_SURVEY_Type_of_user_not_allowed =
#if   L==1	// ca
	"Tipus d'usuari no perm&egrave;s";
#elif L==2	// de
	"Benutzertyp nicht zul&auml;ssig";
#elif L==3	// en
	"Type of user not allowed";
#elif L==4	// es
	"Tipo de usuario no permitido";
#elif L==5	// fr
	"Type d'utilisateur non autoris&eacute;";
#elif L==6	// gn
	"Tipo de usuario no permitido";		// Okoteve traduccin
#elif L==7	// it
	"Tipo di utente non autorizzato";
#elif L==8	// pl
	"Niedozwolony typ u&zdot;ytkownika";
#elif L==9	// pt
	"Tipo de usu&aacute;rio n&atilde;o permitido";
#elif L==10	// tr
	"Type of user not allowed";		// eviri lazim!
#endif

const char *Txt_Survey_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Enquesta <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Umfrage <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Survey <strong>%s</strong> removed.";
#elif L==4	// es
	"Encuesta <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Sondage <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Encuesta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Ankieta <strong>%s</strong> usuni&eogon;te.";
#elif L==9	// pt
	"Inqu&eacute;rito <strong>%s</strong> removido.";
#elif L==10	// tr
	"Survey <strong>%s</strong> removed.";		// eviri lazim!
#endif

const char *Txt_Survey_X_reset =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Enquesta <strong>%s</strong> posada a zero..";
#elif L==2	// de
	"Umfrage <strong>%s</strong> zur&uuml;ckgesetzt.";
#elif L==3	// en
	"Survey <strong>%s</strong> reset.";
#elif L==4	// es
	"Encuesta <strong>%s</strong> puesta a cero.";
#elif L==5	// fr
	"Sondage <strong>%s</strong> remis &agrave; z&eacute;ro.";
#elif L==6	// gn
	"Encuesta <strong>%s</strong> puesta a cero.";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio <strong>%s</strong> resettato.";
#elif L==8	// pl
	"Ankieta <strong>%s</strong> zresetowane.";
#elif L==9	// pt
	"Inqu&eacute;rito <strong>%s</strong> fixado em zero.";
#elif L==10	// tr
	"Survey <strong>%s</strong> reset.";		// eviri lazim!
#endif

const char *Txt_SURVEY_STR_ANSWER_TYPES[Svy_NUM_ANS_TYPES] =
	{
	[Svy_ANS_UNIQUE_CHOICE] =
#if   L==1	// ca
	"Elecci&oacute; &uacute;nica"
#elif L==2	// de
	"Unique-Choice"
#elif L==3	// en
	"Unique choice"
#elif L==4	// es
	"Elecci&oacute;n &uacute;nica"
#elif L==5	// fr
	"Choix unique"
#elif L==6	// gn
	"Elecci&oacute;n &uacute;nica"	// Okoteve traduccin
#elif L==7	// it
	"Scelta unica"
#elif L==8	// pl
	"Unikalny wyb&oacute;r"
#elif L==9	// pt
	"&Uacute;nica escolha"
#elif L==10	// tr
	"Unique choice"			// eviri lazim!
#endif
	,
	[Svy_ANS_MULTIPLE_CHOICE] =
#if   L==1	// ca
	"Elecci&oacute; m&uacute;ltiple"
#elif L==2	// de
	"Multiple-Choice"
#elif L==3	// en
	"Multiple choice"
#elif L==4	// es
	"Elecci&oacute;n m&uacute;ltiple"
#elif L==5	// fr
	"Choix multiple"
#elif L==6	// gn
	"Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==7	// it
	"Scelta multipla"
#elif L==8	// pl
	"Wielokrotny wyb&oacute;r"
#elif L==9	// pt
	"M&uacute;ltipla escolha"
#elif L==10	// tr
	"Multiple choice"			// eviri lazim!
#endif
	};

const char *Txt_Surveys =
#if   L==1	// ca
	"Enquestes";
#elif L==2	// de
	"Umfragen";
#elif L==3	// en
	"Surveys";
#elif L==4	// es
	"Encuestas";
#elif L==5	// fr
	"Sondages";
#elif L==6	// gn
	"Encuestas";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggi";
#elif L==8	// pl
	"Ankiety";
#elif L==9	// pt
	"Inqu&eacute;ritos";
#elif L==10	// tr
	"Anketler";
#endif

const char *Txt_Syllabus =
#if   L==1	// ca
	"Programa";
#elif L==2	// de
	"Studienplan";
#elif L==3	// en
	"Syllabus";
#elif L==4	// es
	"Programa";
#elif L==5	// fr
	"Plan";
#elif L==6	// gn
	"Programa";	// Okoteve traduccin
#elif L==7	// it
	"Programma";
#elif L==8	// pl
	"Program";
#elif L==9	// pt
	"Programa";
#elif L==10	// tr
	"M&uuml;fredat";
#endif

const char *Txt_Syllabus_of_the_course =
#if   L==1	// ca
	"Programa de l'assignatura";
#elif L==2	// de
	"Studienplan der Kurs";
#elif L==3	// en
	"Syllabus of the course";
#elif L==4	// es
	"Programa de la asignatura";
#elif L==5	// fr
	"Plan de la mati&egrave;re";
#elif L==6	// gn
	"Programa de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Programma del corso";
#elif L==8	// pl
	"Program kursu";
#elif L==9	// pt
	"Programa da disciplina";
#elif L==10	// tr
	"Syllabus of the course";	// eviri lazim!
#endif

const char *Txt_SYLLABUS_WHICH_SYLLABUS[Syl_NUM_WHICH_SYLLABUS] =
	{
	[Syl_NONE] =
	""
	,
	[Syl_LECTURES] =
#if   L==1	// ca
	"Teoria"
#elif L==2	// de
	"Vorlesung"
#elif L==3	// en
	"Lectures"
#elif L==4	// es
	"Teor&iacute;a"
#elif L==5	// fr
	"Th&eacute;orie"
#elif L==6	// gn
	"Teor&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"Teoria"
#elif L==8	// pl
	"Wyklady"
#elif L==9	// pt
	"Teoria"
#elif L==10	// tr
	"Lectures"	// eviri lazim!
#endif
	,
	[Syl_PRACTICALS] =
#if   L==1	// ca
	"Pr&agrave;ctiques"
#elif L==2	// de
	"&Uuml;bung"
#elif L==3	// en
	"Practicals"
#elif L==4	// es
	"Pr&aacute;cticas"
#elif L==5	// fr
	"Pratique"
#elif L==6	// gn
	"Pr&aacute;cticas"	// Okoteve traduccin
#elif L==7	// it
	"Pratica"
#elif L==8	// pl
	"Praktyki"
#elif L==9	// pt
	"Pr&aacute;tica"
#elif L==10	// tr
	"Practicals"		// eviri lazim!
#endif
	};

/*
const char *Txt_Platform =
#if   L==1	// ca
	"Plataforma";
#elif L==2	// de
	"Plattform";
#elif L==3	// en
	"Platform";
#elif L==4	// es
	"Plataforma";
#elif L==5	// fr
	"Plate-forme";
#elif L==6	// gn
	"Plataforma";	// Okoteve traduccin
#elif L==7	// it
	"Piattaforma";
#elif L==8	// pl
	"Platforma";
#elif L==9	// pt
	"Plataforma";
#elif L==10	// tr
	"Platformu";
#endif
*/
const char *Txt_TABLE_Footer =
#if   L==1	// ca
	"Peu";
#elif L==2	// de
	"Fu&szlig;";
#elif L==3	// en
	"Footer";
#elif L==4	// es
	"Pie";
#elif L==5	// fr
	"Pied";
#elif L==6	// gn
	"Pie";		// Okoteve traduccin
#elif L==7	// it
	"Piede";
#elif L==8	// pl
	"Stopka";
#elif L==9	// pt
	"P&eacute;";
#elif L==10	// tr
	"Footer";	// eviri lazim!
#endif

const char *Txt_TABLE_Header =
#if   L==1	// ca
	"Cap&ccedil;alera";
#elif L==2	// de
	"Header";
#elif L==3	// en
	"Header";
#elif L==4	// es
	"Cabecera";
#elif L==5	// fr
	"T&ecirc;te";
#elif L==6	// gn
	"Cabecera";	// Okoteve traduccin
#elif L==7	// it
	"Intestazione";
#elif L==8	// pl
	"Nag&lstrok;&oacute;wek";
#elif L==9	// pt
	"Cabe&ccedil;alho";
#elif L==10	// tr
	"Header";	// eviri lazim!
#endif

const char *Txt_Table_not_found_in_the_file_of_marks =
#if   L==1	// ca
	"No hi ha cap taula al arxiu de qualificacions.";
#elif L==2	// de
	"Tabelle nicht in der Notendatei gefunden.";
#elif L==3	// en
	"Table not found in the file of marks.";
#elif L==4	// es
	"No hay ninguna tabla en el archivo de calificaciones.";
#elif L==5	// fr
	"Tableau introuvable dans le fichier des notes.";
#elif L==6	// gn
	"No hay ninguna tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"Tabella non trovata nel file dei risultati.";
#elif L==8	// pl
	"Nie znaleziono tabeli w teczce znak&oacute;w.";
#elif L==9	// pt
	"N&atilde;o h&aacute; tabela no arquivo de classifica&ccedil;&otilde;es.";
#elif L==10	// tr
	"Table not found in the file of marks.";			// eviri lazim!
#endif

const char *Txt_TABS_TXT[Tab_NUM_TABS] =
	{
	[TabUnk] =
	""
	,
	[TabStr] =
#if   L==1	// ca
	"Inici"
#elif L==2	// de
	"Start"
#elif L==3	// en
	"Start"
#elif L==4	// es
	"Inicio"
#elif L==5	// fr
	"D&eacute;marrer"
#elif L==6	// gn
	"&Ntilde;epyr&utilde;"
#elif L==7	// it
	"Inizio"
#elif L==8	// pl
	"Start"
#elif L==9	// pt
	"Inicio"
#elif L==10	// tr
	"Ba&scedil;lama"
#endif
	,
	[TabSys] =
#if   L==1	// ca
	"Plataforma"
#elif L==2	// de
	"Plattform"
#elif L==3	// en
	"System"
#elif L==4	// es
	"Plataforma"
#elif L==5	// fr
	"Plate-forme"
#elif L==6	// gn
	"Ypykatu"
#elif L==7	// it
	"Piattaforma"
#elif L==8	// pl
	"System"
#elif L==9	// pt
	"Sistema"
#elif L==10	// tr
	"Platformu"
#endif
	,
	[TabCty] =
#if   L==1	// ca
	"Pa&iacute;s"
#elif L==2	// de
	"Land"
#elif L==3	// en
	"Country"
#elif L==4	// es
	"Pa&iacute;s"
#elif L==5	// fr
	"Pays"
#elif L==6	// gn
	"Tet&atilde;"
#elif L==7	// it
	"Paese"
#elif L==8	// pl
	"Kraj"
#elif L==9	// pt
	"Pa&iacute;s"
#elif L==10	// tr
	"&Uuml;lke"
#endif
	,
	[TabIns] =
#if   L==1	// ca
	"Instituci&oacute;"
#elif L==2	// de
	"Hochschule"
#elif L==3	// en
	"Institution"
#elif L==4	// es
	"Instituci&oacute;n"
#elif L==5	// fr
	"&Eacute;tablissement"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzione"
#elif L==8	// pl
	"Instytucji"
#elif L==9	// pt
	"Institu&ccedil;&atilde;o"
#elif L==10	// tr
	"Kurum"
#endif
	,
	[TabCtr] =
#if   L==1	// ca
	"Center"
#elif L==2	// de
	"Lehrinstitut"
#elif L==3	// en
	"Center"
#elif L==4	// es
	"Centro"
#elif L==5	// fr
	"Center"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centro"
#elif L==8	// pl
	"Centrum"
#elif L==9	// pt
	"Centro"
#elif L==10	// tr
	"Merkez"
#endif
	,
	[TabDeg] =
#if   L==1	// ca
	"Titulaci&oacute;"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degree"
#elif L==4	// es
	"Titulaci&oacute;n"
#elif L==5	// fr
	"&Eacute;tude"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Laurea"
#elif L==8	// pl
	"Stopien"
#elif L==9	// pt
	"Grau"
#elif L==10	// tr
	"Derece"
#endif
	,
	[TabCrs] =
#if   L==1	// ca
	"Assignatura"
#elif L==2	// de
	"Kurs"
#elif L==3	// en
	"Course"
#elif L==4	// es
	"Asignatura"
#elif L==5	// fr
	"Mati&egrave;re"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corso"
#elif L==8	// pl
	"Kurs"
#elif L==9	// pt
	"Disciplina"
#elif L==10	// tr
	"Kurs"
#endif
	,
	[TabAss] =
#if   L==1	// ca
	"Avaluaci&oacute;"
#elif L==2	// de
	"Bewertung"
#elif L==3	// en
	"Assessment"
#elif L==4	// es
	"Evaluaci&oacute;n"
#elif L==5	// fr
	"&Eacute;valuation"
#elif L==6	// gn
	"Mbohepy"
#elif L==7	// it
	"Valutazione"
#elif L==8	// pl
	"Ocena"
#elif L==9	// pt
	"Avalia&ccedil;&atilde;o"
#elif L==10	// tr
	"De&gbreve; erlendirme"
#endif
	,
	[TabFil] =
#if   L==1	// ca
	"Fitxers"
#elif L==2	// de
	"Dateien"
#elif L==3	// en
	"Files"
#elif L==4	// es
	"Archivos"
#elif L==5	// fr
	"Fichiers"
#elif L==6	// gn
	"Archivos"	// Okoteve traduccin
#elif L==7	// it
	"File"
#elif L==8	// pl
	"Pliki"
#elif L==9	// pt
	"Arquivos"
#elif L==10	// tr
	"Dosyalar&inodot;"
#endif
	,
	[TabUsr] =
#if   L==1	// ca
	"Usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarios"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Puruh&aacute;ra"
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"U&zdot;ytkownicy"
#elif L==9	// pt
	"Utilizadores"
#elif L==10	// tr
	"Kullan&inodot;c&inodot;lar"
#endif
	,
	[TabMsg] =
#if   L==1	// ca
	"Comunicaci&oacute;"
#elif L==2	// de
	"Kommunikation"
#elif L==3	// en
	"Communication"
#elif L==4	// es
	"Comunicaci&oacute;n"
#elif L==5	// fr
	"Communication"
#elif L==6	// gn
	"Mombe'u"
#elif L==7	// it
	"Comunicazione"
#elif L==8	// pl
	"Komunikacja"
#elif L==9	// pt
	"Comunica&ccedil;&atilde;o"
#elif L==10	// tr
	"&Idot;leti&scedil;im"
#endif
	,
	[TabAna] =
#if   L==1	// ca
	"An&agrave;lisi"
#elif L==2	// de
	"Analytik"
#elif L==3	// en
	"Analytics"
#elif L==4	// es
	"An&aacute;lisis"
#elif L==5	// fr
	"Analytiques"
#elif L==6	// gn
	"&Ntilde;ehesa'&yuml;ijo"
#elif L==7	// it
	"Analitiche"
#elif L==8	// pl
	"Analizy"
#elif L==9	// pt
	"An&aacute;lises"
#elif L==10	// tr
	"Analitik"
#endif
	,
	[TabPrf] =
#if   L==1	// ca
	"Perfil"
#elif L==2	// de
	"Profil"
#elif L==3	// en
	"Profile"
#elif L==4	// es
	"Perfil"
#elif L==5	// fr
	"Profil"
#elif L==6	// gn
	"Che"
#elif L==7	// it
	"Profilo"
#elif L==8	// pl
	"Profil"
#elif L==9	// pt
	"Perfil"
#elif L==10	// tr
	"Profil"
#endif
	};

const char *Txt_Tag_allowed =
#if   L==1	// ca
	"Descriptor perm&egrave;s";
#elif L==2	// de
	"Tag erlaubt";
#elif L==3	// en
	"Tag allowed";
#elif L==4	// es
	"Descriptor permitido";
#elif L==5	// fr
	"Descripteur admis";
#elif L==6	// gn
	"Descriptor permitido";	// Okoteve traduccin
#elif L==7	// it
	"Etichetta ammessa";
#elif L==8	// pl
	"Tag allowed";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Descritor permitido";
#elif L==10	// tr
	"Tag allowed";		// eviri lazim!
#endif

const char *Txt_Tag_not_allowed =
#if   L==1	// ca
	"Descriptor no perm&egrave;s";
#elif L==2	// de
	"Tag nicht erlaubt";
#elif L==3	// en
	"Tag not allowed";
#elif L==4	// es
	"Descriptor no permitido";
#elif L==5	// fr
	"Descripteur non admis";
#elif L==6	// gn
	"Descriptor no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Etichetta non autorizzata";
#elif L==8	// pl
	"Tag niedozwolony";
#elif L==9	// pt
	"Descritor n&atilde;o permitido";
#elif L==10	// tr
	"Tag not allowed";		// eviri lazim!
#endif

#if defined LOCALHOST_UBUNTU		// localhost (Ubuntu) *****************

const char *Txt_TAGLINE =
#if   L==1
	"plataforma educativa";
#elif L==2
	"Bildungsplattform";
#elif L==3
	"social learning platform";
#elif L==4
	"plataforma educativa";
#elif L==5
	"plate-forme &eacute;ducative";
#elif L==6
	"plataforma educativa";	// Okoteve traduccin
#elif L==7
	"piattaforma educativa";
#elif L==8
	"platforma edukacyjna";
#elif L==9
	"plataforma educacional";
#elif L==10	// tr
	"e&gbreve;itim platformu";
#endif

const char *Txt_TAGLINE_BR =
#if   L==1
	"plataforma<br />educativa";
#elif L==2
	"Bildungs-<br />plattform";
#elif L==3
	"social learning<br />platform";
#elif L==4
	"plataforma<br />educativa";
#elif L==5
	"plate-forme<br />&eacute;ducative";
#elif L==6
	"plataforma<br />educativa";	// Okoteve traduccin
#elif L==7
	"piattaforma<br />educativa";
#elif L==8
	"platforma<br />edukacyjna";
#elif L==9
	"plataforma<br />educacional";
#elif L==10	// tr
	"e&gbreve;itim<br />platformu";
#endif

#elif defined OPENSWAD_ORG		// openswad.org ***********************

const char *Txt_TAGLINE =
#if   L==1
	"plataforma educativa";
#elif L==2
	"Bildungsplattform";
#elif L==3
	"social learning platform";
#elif L==4
	"plataforma educativa";
#elif L==5
	"plate-forme &eacute;ducative";
#elif L==6
	"plataforma educativa";		// Okoteve traduccin
#elif L==7
	"piattaforma educativa";
#elif L==8
	"platforma edukacyjna";
#elif L==9
	"plataforma educacional";
#elif L==10	// tr
	"social learning platform";	// eviri lazim!
#endif

const char *Txt_TAGLINE_BR =
#if   L==1
	"plataforma<br />educativa";
#elif L==2
	"Bildungs-<br />plattform";
#elif L==3
	"social learning<br />platform";
#elif L==4
	"plataforma<br />educativa";
#elif L==5
	"plate-forme<br />&eacute;ducative";
#elif L==6
	"plataforma<br />educativa";		// Okoteve traduccin
#elif L==7
	"piattaforma<br />educativa";
#elif L==8
	"platforma<br />edukacyjna";
#elif L==9
	"plataforma<br />educacional";
#elif L==10	// tr
	"social learning<br />platform";	// eviri lazim!
#endif

#elif defined SWAD_UGR_ES		// swad.ugr.es ************************

const char *Txt_TAGLINE =
#if   L==1
	"plataforma de suport a la doc&egrave;ncia / UGR /";
#elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre / UGR /";
#elif L==3
	"platform to support teaching / UGR /";
#elif L==4
	"plataforma de apoyo a la docencia / UGR /";
#elif L==5
	"plate-forme pour appuyer l'enseignement / UGR /";
#elif L==6
	"plataforma de apoyo a la docencia / UGR /";	// Okoteve traduccin
#elif L==7
	"piattaforma per sostenere l'insegnamento / UGR /";
#elif L==8
	"platforma do wspierania nauczania / UGR /";
#elif L==9
	"plataforma de apoio ao ensino / UGR /";
#elif L==10	// tr
	"platform to support teaching / UGR /";		// eviri lazim!
#endif

const char *Txt_TAGLINE_BR =
#if   L==1
	"/ UGR / plataforma de<br />suport a la doc&egrave;ncia";
#elif L==2
	"/ UGR / Plattform zur<br />Unterst&uuml;tzung der Lehre";
#elif L==3
	"/ UGR / platform to<br />support teaching";
#elif L==4
	"/ UGR / plataforma de<br />apoyo a la docencia";
#elif L==5
	"/ UGR / plate-forme pour<br />appuyer l'enseignement";
#elif L==6
	"/ UGR / plataforma de<br />apoyo a la docencia";	// Okoteve traduccin
#elif L==7
	"/ UGR / piattaforma per<br />sostenere l'insegnamento";
#elif L==8
	"/ UGR / platforma do<br />wspierania nauczania";
#elif L==9
	"/ UGR / plataforma<br />de apoio ao ensino";
#elif L==10	// tr
	"/ UGR / platform to<br />support teaching";		// eviri lazim!
#endif

#elif defined SWADBERRY_UGR_ES		// swadberry.ugr.es *******************

const char *Txt_TAGLINE =
#if   L==1
	"plataforma de suport a la doc&egrave;ncia (Raspberry Pi)";
#elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre (Raspberry Pi)";
#elif L==3
	"platform to support teaching (Raspberry Pi)";
#elif L==4
	"plataforma de apoyo a la docencia (Raspberry Pi)";
#elif L==5
	"plate-forme pour appuyer l'enseignement (Raspberry Pi)";
#elif L==6
	"plataforma de apoyo a la docencia (Raspberry Pi)";	// Okoteve traduccin
#elif L==7
	"piattaforma per sostenere l'insegnamento (Raspberry Pi)";
#elif L==8
	"platforma do wspierania nauczania (Raspberry Pi)";
#elif L==9
	"plataforma de apoio ao ensino (Raspberry Pi)";
#elif L==10	// tr
	"platform to support teaching (Raspberry Pi)";		// eviri lazim!
#endif

const char *Txt_TAGLINE_BR =
#if   L==1
	"plataforma de suport a la<br />doc&egrave;ncia (Raspberry Pi)";
#elif L==2
	"Plattform zur Unterst&uuml;tzung<br />der Lehre (Raspberry Pi)";
#elif L==3
	"platform to support<br />teaching (Raspberry Pi)";
#elif L==4
	"plataforma de apoyo a la<br />docencia (Raspberry Pi)";
#elif L==5
	"plate-forme pour appuyer<br />l'enseignement (Raspberry Pi)";
#elif L==6
	"plataforma de apoyo a la<br />docencia (Raspberry Pi)";	// Okoteve traduccin
#elif L==7
	"piattaforma per sostenere<br />l'insegnamento (Raspberry Pi)";
#elif L==8
	"platforma do wspierania<br />nauczania (Raspberry Pi)";
#elif L==9
	"plataforma de apoio<br />ao ensino (Raspberry Pi)";
#elif L==10	// tr
	"platform to support<br />teaching (Raspberry Pi)";		// eviri lazim!
#endif

#endif

const char *Txt_Tags =		// Tags used in test
#if   L==1	// ca
	"Descriptors";
#elif L==2	// de
	"Tags";
#elif L==3	// en
	"Tags";
#elif L==4	// es
	"Descriptores";
#elif L==5	// fr
	"Descripteurs";
#elif L==6	// gn
	"Descriptores";	// Okoteve traduccin
#elif L==7	// it
	"Etichette";
#elif L==8	// pl
	"Tagi";
#elif L==9	// pt
	"Descritores";
#elif L==10	// tr
	"Etiketler";
#endif

const char *Txt_teachers_ABBREVIATION =	// Abbreviation of "Teachers"
#if   L==1	// ca
	"prof.";
#elif L==2	// de
	"Lehr.";
#elif L==3	// en
	"teac.";
#elif L==4	// es
	"prof.";
#elif L==5	// fr
	"ense.";
#elif L==6	// gn
	"prof.";	// Okoteve traduccin
#elif L==7	// it
	"prof.";
#elif L==8	// pl
	"nauc.";
#elif L==9	// pt
	"prof.";
#elif L==10	// tr
	"&ouml;&gbreve;re.";
#endif

const char *Txt_Teachers_comment =
#if   L==1	// ca
	"Comentari de professors";
#elif L==2	// de
	"Lehrern-Kommentar";
#elif L==3	// en
	"Teachers' comment";
#elif L==4	// es
	"Comentario de profesores";
#elif L==5	// fr
	"Commentaire des enseignants";
#elif L==6	// gn
	"Comentario de profesores";	// Okoteve traduccin
#elif L==7	// it
	"Commento degli professori";
#elif L==8	// pl
	"Komentarz nauczycieli";
#elif L==9	// pt
	"Coment&aacute;rio dos professores";
#elif L==10	// tr
	"Teachers' comment";		// eviri lazim!
#endif

const char *Txt_Teachers_files_area =
#if   L==1	// ca
	"Zona d'arxius de professors";
#elif L==2	// de
	"Lehrer Dateien-Bereich";
#elif L==3	// en
	"Teachers' files area";
#elif L==4	// es
	"Zona de archivos de profesores";
#elif L==5	// fr
	"Zone de fichiers d'enseignants";
#elif L==6	// gn
	"Zona de archivos de profesores";	// Okoteve traduccin
#elif L==7	// it
	"Zona di file di professori";
#elif L==8	// pl
	"Nauczyciela udost&eogon;pnionych plik&oacute;w";
#elif L==9	// pt
	"Zona de arquivos dos professores";
#elif L==10	// tr
	"Teachers' files area";			// eviri lazim!
#endif

const char *Txt_teachers_files_area =
#if   L==1	// ca
	"zona d'arxius de professors";
#elif L==2	// de
	"Lehrer Dateien-Bereich";
#elif L==3	// en
	"teachers' files area";
#elif L==4	// es
	"zona de archivos de profesores";
#elif L==5	// fr
	"zone de fichiers d'enseignants";
#elif L==6	// gn
	"zona de archivos de profesores";	// Okoteve traduccin
#elif L==7	// it
	"zona di file di professori";
#elif L==8	// pl
	"nauczyciela udost&eogon;pnionych plik&oacute;w";
#elif L==9	// pt
	"zona de arquivos dos professores";
#elif L==10	// tr
	"teachers' files area";			// eviri lazim!
#endif

const char *Txt_Teaching_platform =
#if   L==1	// ca
	"Plataforma docent";
#elif L==2	// de
	"Lernplattform ";
#elif L==3	// en
	"Teaching platform";
#elif L==4	// es
	"Plataforma docente";
#elif L==5	// fr
	"Plate-forme d'enseignement";
#elif L==6	// gn
	"Plataforma docente";	// Okoteve traduccin
#elif L==7	// it
	"Piattaforma didattica";
#elif L==8	// pl
	"Platforma nauczania";
#elif L==9	// pt
	"Plataforma de ensino";
#elif L==10	// tr
	"Teaching platform";	// eviri lazim!
#endif

const char *Txt_Temporary_private_storage_area =
#if   L==1	// ca
	"Zona d'emmagatzematge privada";
#elif L==2	// de
	"Privater Speicherbereich";
#elif L==3	// en
	"Temporary private storage area";
#elif L==4	// es
	"Zona de almacenamiento privada temporal";
#elif L==5	// fr
	"Zone de stockage priv&eacute;e";
#elif L==6	// gn
	"Zona de almacenamiento privada temporal";	// Okoteve traduccin
#elif L==7	// it
	"Area di archiviazione privata";
#elif L==8	// pl
	"Prywatny magazyn";
#elif L==9	// pt
	"Zona privada de armazenamento";
#elif L==10	// tr
	"Temporary private storage area";		// eviri lazim!
#endif

const char *Txt_temporary_private_storage_area =
#if   L==1	// ca
	"zona d'emmagatzematge privada";
#elif L==2	// de
	"privater Speicherbereich";
#elif L==3	// en
	"temporary private storage area";
#elif L==4	// es
	"zona de almacenamiento privada temporal";
#elif L==5	// fr
	"zone de stockage priv&eacute;e";
#elif L==6	// gn
	"zona de almacenamiento privada temporal";	// Okoteve traduccin
#elif L==7	// it
	"area di archiviazione privata";
#elif L==8	// pl
	"obszar prywatnego przechowywania";
#elif L==9	// pt
	"zona privada de armazenamento";
#elif L==10	// tr
	"temporary private storage area";		// eviri lazim!
#endif

const char *Txt_Test =
#if   L==1	// ca
	"Test";
#elif L==2	// de
	"Test";
#elif L==3	// en
	"Test";
#elif L==4	// es
	"Test";
#elif L==5	// fr
	"Test";
#elif L==6	// gn
	"Test";	// Okoteve traduccin
#elif L==7	// it
	"Test";
#elif L==8	// pl
	"Test";
#elif L==9	// pt
	"Teste";
#elif L==10	// tr
	"Test";	// eviri lazim!
#endif

const char *Txt_Test_No_X_that_you_make_in_this_course =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Test n&ordm; %u que realitza vost&egrave; en aquesta assignatura";
#elif L==2	// de
	"Der %u. Test, den Sie in dieser Kurs machen";
#elif L==3	// en
	"Test No. %u that you make in this course";
#elif L==4	// es
	"Test n&ordm; %u que realiza usted en esta asignatura";
#elif L==5	// fr
	"Test n&ordm; %u que vous faites dans cette mati&egrave;re";
#elif L==6	// gn
	"Test n&ordm; %u que realiza usted en esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Test n. %u che fai in questo corso";
#elif L==8	// pl
	"Test nr %u kt&oacute;ry przeprowadzasz na ten kurs";
#elif L==9	// pt
	"Teste n&ordm; %u que voc&ecirc; faz nesta disciplina";
#elif L==10	// tr
	"Test No. %u that you make in this course";		// eviri lazim!
#endif

const char *Txt_Text =
#if   L==1	// ca
	"Text";
#elif L==2	// de
	"Text";
#elif L==3	// en
	"Text";
#elif L==4	// es
	"Texto";
#elif L==5	// fr
	"Texte";
#elif L==6	// gn
	"Texto";	// Okoteve traduccin
#elif L==7	// it
	"Testo";
#elif L==8	// pl
	"Tekst";
#elif L==9	// pt
	"Texto";
#elif L==10	// tr
	"Text";		// eviri lazim!
#endif

const char *Txt_TEXT_plain =
#if   L==1	// ca
	"pla";
#elif L==2	// de
	"Plain";
#elif L==3	// en
	"plain";
#elif L==4	// es
	"plano";
#elif L==5	// fr
	"plain";
#elif L==6	// gn
	"plano";	// Okoteve traduccin
#elif L==7	// it
	"normale";
#elif L==8	// pl
	"zwykly";
#elif L==9	// pt
	"plano";
#elif L==10	// tr
	"plain";	// eviri lazim!
#endif

const char *Txt_Thank_you_for_reporting_a_possible_duplicate_user =
#if   L==1	// ca
	"Gr&agrave;cies per informar d'un usuari possiblement duplicat.";
#elif L==2	// de
	"Vielen Dank fr eine m&ouml;gliche doppelte Benutzer berichten.";
#elif L==3	// en
	"Thank you for reporting a possible duplicate user.";
#elif L==4	// es
	"Gracias por informar de un usuario posiblemente duplicado.";
#elif L==5	// fr
	"Merci d'avoir signal&eacute; un &eacute;ventuel utilisateur en double.";
#elif L==6	// gn
	"Gracias por informar de un usuario posiblemente duplicado.";	// Okoteve traduccin
#elif L==7	// it
	"Grazie per aver segnalato un possibile utente duplicato.";
#elif L==8	// pl
	"Dzi&eogon;kujemy za zg&lstrok;oszenie ewentualnego duplikat u&zdot;ytkownika.";
#elif L==9	// pt
	"Obrigado por relatar um possvel usu&aacute;rio duplicado.";
#elif L==10	// tr
	"Thank you for reporting a possible duplicate user.";		// eviri lazim!
#endif

const char *Txt_Thanks_for_answering_the_survey =
#if   L==1	// ca
	"Gr&agrave;cies per respondre l'enquesta.";
#elif L==2	// de
	"Vielen Dank f&uuml;r die Beantwortung der Umfrage.";
#elif L==3	// en
	"Thanks for answering the survey.";
#elif L==4	// es
	"Gracias por responder la encuesta.";
#elif L==5	// fr
	"Merci de r&eacute;pondre au sondage.";
#elif L==6	// gn
	"Gracias por responder la encuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Grazie per la risposta al sondaggio.";
#elif L==8	// pl
	"Thanks for answering the survey.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Obrigado por responder o inqu&eacute;rito.";
#elif L==10	// tr
	"Thanks for answering the survey.";	// eviri lazim!
#endif

const char *Txt_The_alphabetical_code_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El codi alfab&egrave;tic ISO 3166-1 %s ja existeix.";
#elif L==2	// de
	"Der alphabetische ISO 3166-1-Code %s existiert bereits.";
#elif L==3	// en
	"The ISO 3166-1 alphabetical code %s already exists.";
#elif L==4	// es
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";
#elif L==5	// fr
	"Le code alphab&eacute;tique ISO 3166-1 %s existe d&eacute;j&agrave;.";
#elif L==6	// gn
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice alfabetico ISO 3166-1 %s gi&agrave; esiste.";
#elif L==8	// pl
	"Kod alfabetyczny ISO 3166-1 %s ju&zdot; istnieje.";
#elif L==9	// pt
	"O c&oacute;digo alfab&eacute;tico ISO 3166-1 %s j&aacute; existe.";
#elif L==10	// tr
	"The ISO 3166-1 alphabetical code %s already exists.";		// eviri lazim!
#endif

const char *Txt_The_alphabetical_code_X_is_not_correct =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El codi alfab&egrave;tic ISO 3166-1 %s no &eacute;s correcte.";
#elif L==2	// de
	"Der alphabetische ISO 3166-1-Code %s ist nicht korrekt.";
#elif L==3	// en
	"The ISO 3166-1 alphabetical code %s is not correct.";
#elif L==4	// es
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";
#elif L==5	// fr
	"Le code alphab&eacute;tique ISO 3166-1 %s n'est pas correct.";
#elif L==6	// gn
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice alfabetico ISO 3166-1 %s non &egrave; corretto.";
#elif L==8	// pl
	"Kod alfabetyczny ISO 3166-1 %s jest nieprawid&lstrok;owy.";
#elif L==9	// pt
	"O c&oacute;digo alfab&eacute;tico ISO 3166-1 %s n&atilde;o est&aacute; correto.";
#elif L==10	// tr
	"The ISO 3166-1 alphabetical code %s is not correct.";			// eviri lazim!
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_shuffled =	// Warning: it is very important to include %ld in the following sentences
#if   L==1	// ca
	"Les respostes de la pregunta amb el codi <strong>%ld</strong>"
	" apareixeran remenades.";
#elif L==2	// de
	"Die Antworten der Frage mit dem Code <strong>%ld</strong>"
	" erscheinen gemischt.";
#elif L==3	// en
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";
#elif L==4	// es
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";
#elif L==5	// fr
	"Les r&eacute;ponses &agrave; la question avec le code <strong>%ld</strong>"
	" appara&icirc;tront m&eacute;lang&eacute;es.";
#elif L==6	// gn
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";	// Okoteve traduccin
#elif L==7	// it
	"Le risposte alla domande con il codice <strong>%ld</strong>"
	" appariranno mischiate.";
#elif L==8	// pl
	"Odpowiedzi na pytanie z kodem <strong>%ld</strong>"
	" pojawi&aogon; si&eogon; potasowane.";
#elif L==9	// pt
	"As respostas da pergunta com o c&oacute;digo <strong>%ld</strong>"
	" aparecer&atilde;o embaralhadas.";
#elif L==10	// tr
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// eviri lazim!
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_without_shuffling =	// Warning: it is very important to include %ld in the following sentences
#if   L==1	// ca
	"Les respostes de la pregunta amb el codi <strong>%ld</strong>"
	" apareixeran sense remenar.";
#elif L==2	// de
	"Die Antworten der Frage mit Code <strong>%ld</strong>"
	" erscheinen ohne Mischen.";
#elif L==3	// en
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";
#elif L==4	// es
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";
#elif L==5	// fr
	"Les r&eacute;ponses de la question avec le code <strong>%ld</strong>"
	" appara&icirc;tront sans m&eacute;lange.";
#elif L==6	// gn
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";	// Okoteve traduccin
#elif L==7	// it
	"Le risposte della domanda con il codice <strong>%ld</strong>"
	" appariranno senza essere mischiate.";
#elif L==8	// pl
	"Odpowiedzi na pytanie z kodem <strong>%ld</strong>"
	" pojawi&aogon; si&eogon; bez tasowania.";
#elif L==9	// pt
	"As respostas da pergunta com o c&oacute;digo <strong>%ld</strong>"
	" aparecer&atilde;o sem embaralhar.";
#elif L==10	// tr
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// eviri lazim!
#endif

const char *Txt_The_assignment_has_been_modified =
#if   L==1	// ca
	"L'activitat ha estat modificada.";
#elif L==2	// de
	"Die Aufgabe wurde ge&auml;ndert.";
#elif L==3	// en
	"The assignment has been modified.";
#elif L==4	// es
	"La actividad ha sido modificada.";
#elif L==5	// fr
	"L'activit&eacute; a &eacute;t&eacute; modifi&eacute;e.";
#elif L==6	// gn
	"La actividad ha sido modificada.";	// Okoteve traduccin
#elif L==7	// it
	"L'attivit&agrave; &egrave; stata modificata.";
#elif L==8	// pl
	"Dzia&lstrok;anie zosta&lstrok;o zmodyfikowane.";
#elif L==9	// pt
	"A atividade foi modificada.";
#elif L==10	// tr
	"&Ouml;dev de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_banner_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El banner <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Banner <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"La banni&egrave;re <strong>%s</strong> a &eacute;t&eacute; renomm&eacute;e en <strong>%s</strong>.";
#elif L==6	// gn
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il banner <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Baner <strong>%s</strong> zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"O banner <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_building_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"L'edifici <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Geb&auml;ude <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The building <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El edificio <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le b&acirc;timent <strong>%s</strong> a &eacute;t&eacute; renomm&eacute;e en <strong>%s</strong>.";
#elif L==6	// gn
	"El edificio <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'edificio <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"Budynek <strong>%s</strong> zosta&lstrok;o przemianowane na <strong>%s</strong>.";
#elif L==9	// pt
	"O edif&iacute;cio <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The building <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_building_of_room_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'edifici de la sala <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Das Geb&auml;ude des Raum <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The building of room <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El edificio de la sala <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le b&acirc;timent de la salle <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El edificio de la sala <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'edificio della aula <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"Budynek klasy <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O edif&iacute;cio da sala <strong>%s</strong> n&atilde;o mudou.";
#elif L==10	// tr
	"The building of room <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_building_of_room_X_is_now_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"L'edifici de la sala <strong>%s</strong> ara &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Das Geb&auml;ude des Raum <strong>%s</strong> betr&auml;gt jetzt <strong>%s</strong>.";
#elif L==3	// en
	"The building of room <strong>%s</strong> is now <strong>%s</strong>.";
#elif L==4	// es
	"El edificio de la sala <strong>%s</strong> ahora es <strong>%s</strong>.";
#elif L==5	// fr
	"Le b&acirc;timent de la salle <strong>%s</strong> est maintenant de <strong>%s</strong>.";
#elif L==6	// gn
	"El edificio de la sala <strong>%s</strong> ahora es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'edificio della aula <strong>%s</strong> &egrave; ora <strong>%s</strong>.";
#elif L==8	// pl
	"Budynek klasy <strong>%s</strong> wynosi teraz <strong>%s</strong>.";
#elif L==9	// pt
	"O edif&iacute;cio da sala <strong>%s</strong> &eacute; agora <strong>%s</strong>.";
#elif L==10	// tr
	"The building of room <strong>%s</strong> is now <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_call_for_exam_has_been_successfully_updated =
#if   L==1	// ca
	"La convocat&ograve;ria d'examen s'ha actualitzat correctament.";
#elif L==2	// de
	"Der Aufruf zur Pr&uuml;fung wurde erfolgreich aktualisiert.";
#elif L==3	// en
	"The call for exam has been successfully updated.";
#elif L==4	// es
	"La convocatoria de examen se ha actualizado correctamente.";
#elif L==5	// fr
	"La convocation pour l'examen a &eacute;t&eacute; mis &agrave; jour avec succ&egrave;s.";
#elif L==6	// gn
	"La convocatoria de examen se ha actualizado correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"L'apello d'esame &egrave; stata aggiornata con successo.";
#elif L==8	// pl
	"Ogloszenie egzamin zosta&lstrok; pomy&sacute;lnie zaktualizowany.";
#elif L==9	// pt
	"A chamada para exame foi atualizada com sucesso.";
#elif L==10	// tr
	"The call for exam has been successfully updated.";		// eviri lazim!
#endif

const char *Txt_The_capacity_of_room_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'aforament de la sala <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die Kapazit&auml;t des Raum <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The capacity of room <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El aforo de la sala <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"La capacit&eacute; de la salle <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El aforo de la sala <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"La capacit&agrave; della aula <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"Pojemno&sacute;&cacute; klasy <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"A capacidade da sala <strong>%s</strong> n&atilde;o mudou.";
#elif L==10	// tr
	"The capacity of room <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_capacity_of_room_X_is_now_Y =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"L'aforament de la sala <strong>%s</strong> ara &eacute;s <strong>%u</strong>.";
#elif L==2	// de
	"Die Kapazit&auml;t des Raum <strong>%s</strong> betr&auml;gt jetzt <strong>%u</strong>.";
#elif L==3	// en
	"The capacity of room <strong>%s</strong> is now <strong>%u</strong>.";
#elif L==4	// es
	"El aforo de la sala <strong>%s</strong> ahora es <strong>%u</strong>.";
#elif L==5	// fr
	"La capacit&eacute; de la salle <strong>%s</strong> est maintenant de <strong>%u</strong>.";
#elif L==6	// gn
	"El aforo de la sala <strong>%s</strong> ahora es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La capacit&agrave; della aula <strong>%s</strong> &egrave; ora <strong>%u</strong>.";
#elif L==8	// pl
	"Pojemno&sacute;&cacute; klasy <strong>%s</strong> wynosi teraz <strong>%u</strong>.";
#elif L==9	// pt
	"A capacidade da sala <strong>%s</strong> &eacute; agora <strong>%u</strong>.";
#elif L==10	// tr
	"The capacity of room <strong>%s</strong> is now <strong>%u</strong>.";		// eviri lazim!
#endif

const char *Txt_The_center_X_has_been_moved_to_the_institution_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El centre <strong>%s</strong> s'ha traslladat"
	" a la instituci&oacute; <strong>%s</strong>.";
#elif L==2	// de
	"Das Zentrum <strong>%s</strong> wurde"
	" in die Institution <strong>%s</strong> verlegt.";
#elif L==3	// en
	"The center <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";
#elif L==4	// es
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Le centre <strong>%s</strong> a &eacute;t&eacute; transf&eacute;r&eacute;"
	" dans l'&eacute;tablissement <strong>%s</strong>.";
#elif L==6	// gn
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il center <strong>%s</strong> &egrave; stato spostato"
	" alla istituzione <strong>%s</strong>.";
#elif L==8	// pl
	"Centrum <strong>%s</strong> zosta&lstrok;o przeniesione"
	" do instytucji <strong>%s</strong>.";
#elif L==9	// pt
	"O centro <strong>%s</strong> foi transferido"
	" para a institui&ccedil;&atilde;o <strong>%s</strong>.";
#elif L==10	// tr
	"The center <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_center_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El centre <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Zentrum <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The center <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le centre <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il centro <strong>%s</strong> &egrave; stato rinominato <strong>%s</strong>.";
#elif L==8	// pl
	"Centrum <strong>%s</strong> zosta&lstrok;o przemianowane na <strong>%s</strong>.";
#elif L==9	// pt
	"O centro <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The center <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_configuration_of_the_projects_has_been_updated =
#if   L==1	// ca
	"La configuraci&oacute; dels projectes s'ha actualitzat.";
#elif L==2	// de
	"Die Konfiguration der Projekte wurde aktualisiert.";
#elif L==3	// en
	"The configuration of the projects has been updated.";
#elif L==4	// es
	"La configuraci&oacute;n de los proyectos se ha actualizado.";
#elif L==5	// fr
	"La configuration des projets a &eacute;t&eacute; mise &agrave; jour.";
#elif L==6	// gn
	"La configuraci&oacute;n de los proyectos se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"La configurazione dei progetti &egrave; stata aggiornata.";
#elif L==8	// pl
	"Konfiguracja projekt&oacute;w zosta&lstrok;a zaktualizowana.";
#elif L==9	// pt
	"A configura&ccedil;&atilde;o dos projetos foi atualizada.";
#elif L==10	// tr
	"The configuration of the projects has been updated.";		// eviri lazim!
#endif

const char *Txt_The_contents_of_the_folder_are_too_big =
#if   L==1	// ca
	"El contingut de la carpeta &eacute;s massa gran.";
#elif L==2	// de
	"Der Inhalt der Ordner sind zu gro&szlig;.";
#elif L==3	// en
	"The contents of the folder are too big.";
#elif L==4	// es
	"El contenido de la carpeta es demasiado grande.";
#elif L==5	// fr
	"Le contenu du dossier est trop gros.";
#elif L==6	// gn
	"El contenido de la carpeta es demasiado grande.";	// Okoteve traduccin
#elif L==7	// it
	"Il contenuto della cartella &egrave; troppo grande.";
#elif L==8	// pl
	"Zawarto&sacute;&cacute; folderu s&aogon; zbyt du&zdot;e.";
#elif L==9	// pt
	"O conte&uacute;do do diret&oacute;rio &eacute; muito grande.";
#elif L==10	// tr
	"The contents of the folder are too big.";		// eviri lazim!
#endif

const char *Txt_The_copy_has_been_successful =
#if   L==1	// ca
	"La c&ograve;pia s'ha fet amb &egrave;xit.";
#elif L==2	// de
	"Die Kopie wurde erfolgreich.";
#elif L==3	// en
	"The copy has been successful.";
#elif L==4	// es
	"La copia se ha realizado con &eacute;xito.";
#elif L==5	// fr
	"La copie a &eacute;t&eacute; un succ&egrave;s.";
#elif L==6	// gn
	"La copia se ha realizado con &eacute;xito.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; avvenuta con successo.";
#elif L==8	// pl
	"The copy has been successful.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia foi feita com sucesso.";
#elif L==10	// tr
	"The copy has been successful.";		// eviri lazim!
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traducci
#elif L==2	// de
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare <strong>%s</strong>"
	" perch&egrave; esso superava la quota di gestione del disco.";
#elif L==8	// pl
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar <strong>%s</strong>"
	" porque ele excederia a cota de disco.";
#elif L==10	// tr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the disk quota.";		// eviri lazim!
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traducci
#elif L==2	// de
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==5	// fr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare <strong>%s</strong>"
	" perch&egrave; esso superava il massimo numero di livelli permesso.";
#elif L==8	// pl
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar <strong>%s</strong>"
	" porque ele excederia o n&uacute;mero m&aacute;ximo permitido de n&iacute;veis.";
#elif L==10	// tr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// eviri lazim!
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_X_because_there_is_already_an_object_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";	// Necessita traducci
#elif L==2	// de
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, <strong>%s</strong>"
	" einzuf&uuml;gen. Der Objektname wird bereits verwendet.";
#elif L==3	// en
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because there is already an object with that name.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";
#elif L==5	// fr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because there is already an object with that name.";	// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque ya existe otro objecto con ese nombre.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare <strong>%s</strong>"
	" perch&egrave; c'era gi&agrave; un oggetto con quel nome.";
#elif L==8	// pl
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because there is already an object with that name.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar <strong>%s</strong>"
	" porque j&aacute; existe um objeto com esse nome.";
#elif L==10	// tr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because there is already an object with that name.";	// eviri lazim!
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_X_because_you_can_not_paste_a_file_here_of_a_type_other_than_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Necessita traducci
#elif L==2	// de
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, <strong>%s</strong>"
	" einzuf&uuml;gen. Es d&uuml;rfen nur HTML-Daten eingef&uumlgt werden.";
#elif L==3	// en
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";
#elif L==5	// fr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare <strong>%s</strong>"
	" perch&egrave; non puoi incollare qui un file di un tipo diverso da HTML.";
#elif L==8	// pl
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar <strong>%s</strong>"
	" porque voc&ecirc; n&atilde;o pode colar um arquivo aqui de um tipo diferente de HTML.";
#elif L==10	// tr
	"The copy has stopped when trying to paste <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// eviri lazim!
#endif

const char *Txt_The_country_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Necessita traducci
#elif L==2	// de
	"Die Land <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The country <strong>%s</strong> already exists.";
#elif L==4	// es
	"El pa&iacute;s <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The country <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il paese <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The country <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O pa&iacute;s <strong>%s</strong> j&aacute; existe.";
#elif L==10	// tr
	"The country <strong>%s</strong> already exists.";	// eviri lazim!
#endif

const char *Txt_The_country_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El pa&iacute;s <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Land <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le pays <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il paese <strong>%s</strong> &egrave; stato rimosso come <strong>%s</strong>.";
#elif L==8	// pl
	"Kraj <strong>%s</strong> zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"O pa&iacute;s <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_country_of_the_institution_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Necessita traducci
#elif L==2	// de
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==3	// en
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==6	// gn
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il paese dell'istituzione <strong>%s</strong>"
	" &egrave; cambiato in <strong>%s</strong>.";
#elif L==8	// pl
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==10	// tr
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_course_X_has_been_moved_to_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"L'assignatura <strong>%s</strong>"
	" s'ha traslladat al grau <strong>%s</strong>.";
#elif L==2	// de
	"Der Studiengang <strong>%s</strong>"
	" wurde in den Studiengang <strong>%s</strong> verschoben.";
#elif L==3	// en
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";
#elif L==4	// es
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"La mati&egrave;re <strong>%s</strong> a &eacute;t&eacute; d&eacute;plac&eacute;e"
	" &agrave; l'&eacute;tude <strong>%s</strong>.";
#elif L==6	// gn
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il corso <strong>%s</strong> &egrave; stato spostato"
	" dalla laurea <strong>%s</strong>.";
#elif L==8	// pl
	"Kurs <strong>%s</strong> zosta&lstrok; przeniesiony"
	" na stopie&nacute; <strong>%s</strong>.";
#elif L==9	// pt
	"A disciplina curso <strong>%s</strong> foi movida"
	" para o grau <strong>%s</strong>.";
#elif L==10	// tr
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_course_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"L'assignatura <strong>%s</strong>"
	" ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Kurs <strong>%s</strong>"
	" wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The course <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"La asignatura <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"La mati&egrave;re <strong>%s</strong>"
	" a &eacute;t&eacute; renomm&eacute;e en <strong>%s</strong>.";
#elif L==6	// gn
	"La asignatura <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il corso <strong>%s</strong>"
	" &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"Kurs <strong>%s</strong>"
	" zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"A disciplina <strong>%s</strong>"
	" foi renomeada como <strong>%s</strong>.";
#elif L==10	// tr
	"The course <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_date_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La data de la festivitat <strong>%s</strong> ha canviat a <strong>%s</strong>.";
#elif L==2	// de
	"Das Datum des Feiertags <strong>%s</strong> hat sich auf <strong>%s</strong> ge&auml;ndert.";
#elif L==3	// en
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"La date de la f&ecirc;te <strong>%s</strong> est pass&eacute;e &agrave; <strong>%s</strong>.";
#elif L==6	// gn
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La data delle festivit&agrave; <strong>%s</strong> &egrave; cambiata a <strong>%s</strong>.";
#elif L==8	// pl
	"Data &sacute;wi&eogon;ta <strong>%s</strong> zmieni&lstrok;a si&eogon; na <strong>%s</strong>.";
#elif L==9	// pt
	"A data do feriado <strong>%s</strong> mudou para <strong>%s</strong>.";
#elif L==10	// tr
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_date_range_must_be_less_than_or_equal_to_X_days =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"L'interval de dates ha de ser inferior o igual a %u dies.";
#elif L==2	// de
	"Der Datumsbereich muss kleiner oder gleich %u Tage sein.";
#elif L==3	// en
	"The date range must be less than or equal to %u days.";
#elif L==4	// es
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";
#elif L==5	// fr
	"La plage de dates doit &ecirc;tre inf&eacute;rieure ou &eacute;gale &agrave; %u jours.";
#elif L==6	// gn
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";	// Okoteve traduccin
#elif L==7	// it
	"Il range della data deve essere minore o uguale a %u giorni.";
#elif L==8	// pl
	"Zakres dat musi by&cacute; kr&oacute;tszy lub r&oacute;wny %u dniom.";
#elif L==9	// pt
	"O intervalo de datas deve ser menor ou igual a %u dias.";
#elif L==10	// tr
	"The date range must be less than or equal to %u days.";	// eviri lazim!
#endif

const char *Txt_The_date_time_of_opening_of_groups_has_changed =
#if   L==1	// ca
	"La data/hora d'obertura de grups s'ha modificat.";
#elif L==2	// de
	"Das Datum/Uhrzeit der Er&ouml;ffnung von Gruppen hat sich ge&auml;ndert.";
#elif L==3	// en
	"The date/time of opening of groups has changed.";
#elif L==4	// es
	"La fecha/hora de apertura ha cambiado.";
#elif L==5	// fr
	"La date/heure d'ouverture des groupes a chang&eacute;.";
#elif L==6	// gn
	"La fecha/hora de apertura ha cambiado.";		// Okoteve traduccin
#elif L==7	// it
	"La data/ora di apertura dei gruppi &egrave; cambiata.";
#elif L==8	// pl
	"The date/time of opening of groups has changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A data/hora de abertura dos grupos foi alterada.";
#elif L==10	// tr
	"The date/time of opening of groups has changed.";	// eviri lazim!
#endif

const char *Txt_The_degree_X_has_been_moved_to_the_center_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La titulaci&oacute; <strong>%s</strong> s'ha traslladat"
	" al centre <strong>%s</strong>.";
#elif L==2	// de
	"Der Grad <strong>%s</strong> wurde"
	" in die Mitte <strong>%s</strong> verschoben.";
#elif L==3	// en
	"The degree <strong>%s</strong> has been moved"
	" to the center <strong>%s</strong>.";
#elif L==4	// es
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";
#elif L==5	// fr
	"L'&eacute;tude <strong>%s</strong> a &eacute;t&eacute; d&eacute;plac&eacute;"
	" vers le centre <strong>%s</strong>.";
#elif L==6	// gn
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"La laurea <strong>%s</strong> &egrave; stata spostata"
	" al centro <strong>%s</strong>.";
#elif L==8	// pl
	"Stopie&nacute; <strong>%s</strong> zosta&lstrok; przesuni&eogon;ty"
	" do &sacute;rodka <strong>%s</strong>.";
#elif L==9	// pt
	"O grau <strong>%s</strong> foi movido"
	" para o centro <strong>%s</strong>.";
#elif L==10	// tr
	"The degree <strong>%s</strong> has been moved"
	" to the center <strong>%s</strong>."	;	// eviri lazim!
#endif

const char *Txt_The_degree_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La titulaci&oacute; <strong>%s</strong>"
	" ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Grad <strong>%s</strong>"
	" wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The degree <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"La titulaci&oacute;n <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"L'&eacute;tude <strong>%s</strong>"
	" a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"La titulaci&oacute;n <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La laurea <strong>%s</strong>"
	" &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"Stopie&nacute; <strong>%s</strong>"
	" zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"O grau <strong>%s</strong>"
	" foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The degree <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_department_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El departament <strong>%s</strong>"
	" ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Die Abteilung <strong>%s</strong>"
	" wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The department <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El departamento <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le d&eacute;partement <strong>%s</strong>"
	" a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El departamento <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il dipartimento <strong>%s</strong>"
	" &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Dzia&lstrok; <strong>%s</strong>"
	" zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"O departamento <strong>%s</strong>"
	" foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The department <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_email_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El correu <strong>%s</strong> ha estat confirmat.";
#elif L==2	// de
	"Die E-Mail <strong>%s</strong> wurde best&auml;tigt.";
#elif L==3	// en
	"The email <strong>%s</strong> has been confirmed.";
#elif L==4	// es
	"El correo <strong>%s</strong> ha sido confirmado.";
#elif L==5	// fr
	"L'e-mail <strong>%s</strong> a &eacute;t&eacute; confirm&eacute;.";
#elif L==6	// gn
	"El correo <strong>%s</strong> ha sido confirmado.";	// Okoteve traduccin
#elif L==7	// it
	"L'email <strong>%s</strong> &egrave; stata confermata.";
#elif L==8	// pl
	"E-mail <strong>%s</strong> zosta&lstrok; potwierdzony.";
#elif L==9	// pt
	"O email <strong>%s</strong> foi confirmado.";
#elif L==10	// tr
	"The email <strong>%s</strong> has been confirmed.";	// eviri lazim!
#endif

const char *Txt_The_email_address_entered_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'adre&ccedil;a electr&ograve;nica introdu&iuml;da"
	" <strong>%s</strong> no &eacute;s v&agrave;lida.";
#elif L==2	// de
	"Die eingegebene E-Mail-Adresse"
	" <strong>%s</strong> ist ung&uuml;ltig.";
#elif L==3	// en
	"The email address entered"
	" <strong>%s</strong> is not valid.";
#elif L==4	// es
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";
#elif L==5	// fr
	"L'adresse e-mail saisie"
	" <strong>%s</strong> n'est pas valide.";
#elif L==6	// gn
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";	// Okoteve traduccin
#elif L==7	// it
	"Il email immesso"
	" <strong>%s</strong> non &egrave; valido.";
#elif L==8	// pl
	"Wpisany adres e-mail"
	" <strong>%s</strong> jest nieprawid&lstrok;owy.";
#elif L==9	// pt
	"O endere&ccedil;o de email inserido"
	" <strong>%s</strong> n&atilde;o &eacute; v&aacute;lido.";
#elif L==10	// tr
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// eviri lazim!
#endif

const char *Txt_The_email_address_X_had_been_registered_by_another_user =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'adre&ccedil;a de correu electr&ograve;nic <strong>%s</strong>"
	" ha estat registrada per un altre usuari.";
#elif L==2	// de
	"Die E-Mail-Adresse <strong>%s</strong>"
	" wurde von einem anderen Benutzer registriert.";
#elif L==3	// en
	"The email address <strong>%s</strong>"
	" had been registered by another user.";
#elif L==4	// es
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";
#elif L==5	// fr
	"L'adresse e-mail <strong>%s</strong>"
	" a &eacute;t&eacute; inscrite par un autre utilisateur.";
#elif L==6	// gn
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Il email <strong>%s</strong>"
	" &egrave; stato registrato da un altro utente.";
#elif L==8	// pl
	"Adres e-mail <strong>%s</strong>"
	" zosta&lstrok; zarejestrowany przez innego u&zdot;ytkownika.";
#elif L==9	// pt
	"O endere&ccedil;o de email <strong>%s</strong>"
	" foi registrado por outro usu&aacute;rio.";
#elif L==10	// tr
	"The email address <strong>%s</strong>"
	" had been registered by another user.";	// eviri lazim!
#endif

const char *Txt_The_email_address_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'adre&ccedil;a de correu electr&ograve;nic <strong>%s</strong>"
	" s'ha registrat correctament.";
#elif L==2	// de
	"Die E-Mail-Adresse <strong>%s</strong>"
	" wurde erfolgreich registriert.";
#elif L==3	// en
	"The email address <strong>%s</strong>"
	" has been registered successfully.";
#elif L==4	// es
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==5	// fr
	"L'adresse e-mail <strong>%s</strong>"
	" a &eacute;t&eacute; enregistr&eacute;e avec succ&egrave;s.";
#elif L==6	// gn
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il email <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8	// pl
	"Adres e-mail <strong>%s</strong>"
	" zosta&lstrok; pomy&sacute;lnie zarejestrowany.";
#elif L==9	// pt
	"O endere&ccedil;o de email <strong>%s</strong>"
	" foi registrado com sucesso.";
#elif L==10	// tr
	"The email address <strong>%s</strong>"
	" has been registered successfully.";	// eviri lazim!
#endif

const char *Txt_The_email_address_X_matches_one_previously_registered =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'adre&ccedil;a de correu electr&ograve;nic <strong>%s</strong>"
	" coincideix amb una registrada anteriorment.";
#elif L==2	// de
	"Die E-Mail-Adresse <strong>%s</strong>"
	" stimmt mit einer zuvor registrierten &uuml;berein.";
#elif L==3	// en
	"The email address <strong>%s</strong>"
	" matches one previously registered.";
#elif L==4	// es
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";
#elif L==5	// fr
	"L'adresse e-mail <strong>%s</strong>"
	" correspond &agrave; une adresse pr&eacute;c&eacute;demment enregistr&eacute;e.";
#elif L==6	// gn
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il email <strong>%s</strong>"
	" coincide con uno precedentemente registrato.";
#elif L==8	// pl
	"Adres e-mail <strong>%s</strong>"
	" odpowiada poprzednio zarejestrowanemu.";
#elif L==9	// pt
	"O endere&ccedil;o de email <strong>%s</strong>"
	" corresponde a um registrado anteriormente.";
#elif L==10	// tr
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// eviri lazim!
#endif

const char *Txt_The_email_domain_X_already_exists =
#if   L==1	// ca
	"El domini de correu ja existeix.";
#elif L==2	// de
	"Die E-Mail-Dom&auml;ne wurde bereits eingegeben.";
#elif L==3	// en
	"The email domain already exists.";
#elif L==4	// es
	"El dominio de correo ya existe.";
#elif L==5	// fr
	"Le domaine de messagerie existe d&eacute;j&grave;.";
#elif L==6	// gn
	"El dominio de correo ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo mail gi&agrave; esiste.";
#elif L==8	// pl
	"Domena poczty e-mail ju&zdot; istnieje.";
#elif L==9	// pt
	"O dom&iacute;nio de email j&aacute; existe.";
#elif L==10	// tr
	"The email domain already exists.";	// eviri lazim!
#endif

const char *Txt_The_email_domain_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El domini de correu <strong>%s</strong>"
	" ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Die E-Mail-Dom&auml;ne <strong>%s</strong>"
	" wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The email domain <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El dominio de correo <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le domaine de messagerie <strong>%s</strong>"
	" a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El dominio de correo <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo mail <strong>%s</strong>"
	" &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Domena poczty e-mail <strong>%s</strong>"
	" zosta&lstrok;a zmieniona na <strong>%s</strong>.";
#elif L==9	// pt
	"O dom&iacute;nio de email <strong>%s</strong>"
	" foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The email domain <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_email_domain_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El domini de correu <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die E-Mail-Dom&auml;ne <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The email domain <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El dominio de correo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le domaine de messagerie <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El dominio de correo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo mail <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Domena poczty e-mail <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O dom&iacute;nio de email <strong>%s</strong> n&atilde;o foi alterado.";
#elif L==10	// tr
	"The email domain <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_enrolment_of_students_into_groups_of_type_X_is_now_mandatory =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'adscripci&oacute; d'estudiants a grups del tipus <strong>%s</strong> ara &eacute;s obligat&ograve;ria.";
#elif L==2	// de
	"Die Einschreibung von Studierenden in Gruppen des Typs <strong>%s</strong> ist nun verpflichtend.";
#elif L==3	// en
	"The enrolment of students into groups of type <strong>%s</strong> is now mandatory.";
#elif L==4	// es
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";
#elif L==5	// fr
	"L'inscription des &eacute;tudiants dans les groupes de type <strong>%s</strong> est d&eacute;sormais obligatoire.";	// Besoin de traduction
#elif L==6	// gn
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";			// Okoteve traduccin
#elif L==7	// it
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora obbligatoria.";
#elif L==8	// pl
	"Zapis uczni&oacute;w do grup typu <strong>%s</strong> jest teraz obowi&aogon;zkowy.";
#elif L==9	// pt
	"A inscri&ccedil;&atilde;o de alunos em grupos do tipo <strong>%s</strong> &eacute; agora obrigat&oacute;ria.";
#elif L==10	// tr
	"The enrolment of students into groups of type <strong>%s</strong> is now mandatory.";					// eviri lazim!
#endif

const char *Txt_The_enrolment_of_students_into_groups_of_type_X_is_now_voluntary =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'adscripci&oacute; d'estudiants a grups del tipus <strong>%s</strong> ara &eacute;s volunt&agrave;ria.";
#elif L==2	// de
	"Die Einschreibung von Studierenden in Gruppen des Typs <strong>%s</strong> ist nun freiwillig.";
#elif L==3	// en
	"The enrolment of students into groups of type <strong>%s</strong> is now voluntary.";
#elif L==4	// es
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";
#elif L==5	// fr
	"L'inscription des &eacute;tudiants dans les groupes de type <strong>%s</strong> est d&eacute;sormais volontaire.";
#elif L==6	// gn
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";	// Okoteve traduccin
#elif L==7	// it
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora volontaria.";
#elif L==8	// pl
	"Zapisy uczni&oacute;w do grup typu <strong>%s</strong> s&aogon; teraz dobrowolne.";
#elif L==9	// pt
	"A inscri&ccedil;&atilde;o de estudantes em grupos do tipo <strong>%s</strong> &eacute; agora volunt&aacute;ria.";
#elif L==10	// tr
	"The enrolment of students into groups of type <strong>%s</strong> is now voluntary.";			// eviri lazim!
#endif

const char *Txt_The_event_has_been_modified =
#if   L==1	// ca
	"L'esdeveniment ha estat modificat.";
#elif L==2	// de
	"Der Ereignis wurde ge&auml;ndert.";
#elif L==3	// en
	"The event has been modified.";
#elif L==4	// es
	"El evento ha sido modificado.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El evento ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento &egrave; stata modificato.";
#elif L==8	// pl
	"Wydarzenie zosta&lstrok;o zmodyfikowane.";
#elif L==9	// pt
	"O evento foi modificado.";
#elif L==10	// tr
	"Etkinlik de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_exam_has_been_modified =
#if   L==1	// ca
	"L'examen ha estat modificat.";
#elif L==2	// de
	"Der Ereignis wurde ge&auml;ndert.";
#elif L==3	// en
	"The exam has been modified.";
#elif L==4	// es
	"El examen ha sido modificado.";
#elif L==5	// fr
	"L'examen a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El examen ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"L'esame &egrave; stata modificato.";
#elif L==8	// pl
	"Egzamin zosta&lstrok; zmodyfikowany.";
#elif L==9	// pt
	"O exame foi modificado.";
#elif L==10	// tr
	"S&inodot;nav de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_file_could_not_be_processed_successfully =
#if   L==1	// ca
	"El fitxer no s'ha pogut processar correctament.";
#elif L==2	// de
	"Die Datei konnte nicht erfolgreich verarbeitet werden.";
#elif L==3	// en
	"The file could not be processed successfully.";
#elif L==4	// es
	"El archivo no ha podido procesarse correctamente.";
#elif L==5	// fr
	"Le fichier n'a pas pu &ecirc;tre trait&eacute; avec succ&egrave;s.";
#elif L==6	// gn
	"El archivo no ha podido procesarse correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il file non pu&ograve; essere elaborato con successo.";
#elif L==8	// pl
	"Plik nie m&oacute;g&lstrok; zosta&cacute; pomy&sacute;lnie przetworzony.";
#elif L==9	// pt
	"O arquivo n&atilde;o p&ocirc;de ser processado com sucesso.";
#elif L==10	// tr
	"The file could not be processed successfully.";	// eviri lazim!
#endif

const char *Txt_The_file_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traducci
#elif L==2	// de
	"Die Datei <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==3	// en
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==4	// es
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Le fichier <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Il file <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==8	// pl
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O arquivo <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#elif L==10	// tr
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_file_is_not_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'arxiu no &eacute;s <em>%s</em>.";
#elif L==2	// de
	"Die Datei ist kein <em>%s</em>.";
#elif L==3	// en
	"The file is not <em>%s</em>.";
#elif L==4	// es
	"El archivo no es <em>%s</em>.";
#elif L==5	// fr
	"Le fichier n'est pas <em>%s</em>.";
#elif L==6	// gn
	"El archivo no es <em>%s</em>.";	// Okoteve traduccin
#elif L==7	// it
	"IL file non &egrave; <em>%s</em>.";
#elif L==8	// pl
	"Plik nie jest w formacie <em>%s</em>.";
#elif L==9	// pt
	"O arquivo n&atilde;o &eacute; <em>%s</em>.";
#elif L==10	// tr
	"The file is not <em>%s</em>.";		// eviri lazim!
#endif

const char *Txt_The_file_of_folder_no_longer_exists_or_is_now_hidden =
#if   L==1	// ca
	"L'arxiu o carpeta ja no existeix o est&agrave; ara ocult.";
#elif L==2	// de
	"Die Datei oder Ordner existiert nicht mehr oder ist jetzt versteckt.";
#elif L==3	// en
	"The file or folder no longer exists or is now hidden.";
#elif L==4	// es
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";
#elif L==5	// fr
	"Le fichier ou r&eacute;pertoire n'existe plus ou est maintenant cach&eacute;.";
#elif L==6	// gn
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";	// Okoteve traduccin
#elif L==7	// it
	"Il file o cartella non esiste pi&ugrave; o &egrave; nascosta.";
#elif L==8	// pl
	"Plik lub folder ju&zdot; nie istnieje lub jest ukryte.";
#elif L==9	// pt
	"O arquivo ou diret&oacute;rio n&atilde;o existe mais ou est&aacute; escondido.";
#elif L==10	// tr
	"The file or folder no longer exists or is now hidden.";		// eviri lazim!
#endif

const char *Txt_The_file_type_is_X_and_should_be_HTML_or_ZIP =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de fitxer &eacute;s %s i hauria de ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==2	// de
	"Der Dateityp ist %s und sollte <em>HTML</em> oder <em>ZIP</em> sein.";
#elif L==3	// en
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==4	// es
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==5	// fr
	"Le type de fichier est %s et doit &ecirc;tre <em>HTML</em> ou <em>ZIP</em>.";
#elif L==6	// gn
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di file &egrave; %s e dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==8	// pl
	"Typ pliku to %s i powinien to by&cacute; <em>HTML</em> lub <em>ZIP</em>.";
#elif L==9	// pt
	"O tipo de arquivo &eacute; %s e deve ser <em>HTML</em> ou <em>ZIP</em>.";
#elif L==10	// tr
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// eviri lazim!
#endif

const char *Txt_The_file_type_should_be_HTML_or_ZIP =
#if   L==1	// ca
	"El tipus d'arxiu hauria de ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==2	// de
	"Der Dateityp sollte <em>HTML</em> oder <em>ZIP</em> sein.";
#elif L==3	// en
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==4	// es
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==5	// fr
	"Le type de fichier doit &ecirc;tre <em>HTML</em> ou <em>ZIP</em>.";
#elif L==6	// gn
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di file dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==8	// pl
	"Typ pliku powinien by&cacute; <em>HTML</em> lub <em>ZIP</em>.";
#elif L==9	// pt
	"O tipo de arquivo deve ser <em>HTML</em> ou <em>ZIP</em>.";
#elif L==10	// tr
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// eviri lazim!
#endif

const char *Txt_The_floor_of_room_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La planta de la sala <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Der Stock des Raum <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The floor of room <strong>%s</strong> has not changed.";
#elif L==4	// es
	"La planta de la sala <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"L'&eacute;tage de la salle <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"La planta de la sala <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il piano della aula <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"Pi&eogon;trze klasy <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O andar da sala <strong>%s</strong> n&atilde;o mudou.";
#elif L==10	// tr
	"The floor of room <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_floor_of_room_X_is_now_Y =	// Warning: it is very important to include %s and %d in the following sentences
#if   L==1	// ca
	"La planta de la sala <strong>%s</strong> ara &eacute;s <strong>%d</strong>.";
#elif L==2	// de
	"Der Stock des Raum <strong>%s</strong> betr&auml;gt jetzt <strong>%d</strong>.";
#elif L==3	// en
	"The floor of room <strong>%s</strong> is now <strong>%d</strong>.";
#elif L==4	// es
	"La planta de la sala <strong>%s</strong> ahora es <strong>%d</strong>.";
#elif L==5	// fr
	"L'&eacute;tage de la salle <strong>%s</strong> est maintenant de <strong>%d</strong>.";
#elif L==6	// gn
	"La planta de la sala <strong>%s</strong> ahora es <strong>%d</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il piano della aula <strong>%s</strong> &egrave; ora <strong>%d</strong>.";
#elif L==8	// pl
	"Pi&eogon;trze klasy <strong>%s</strong> wynosi teraz <strong>%d</strong>.";
#elif L==9	// pt
	"O andar da sala <strong>%s</strong> &eacute; agora <strong>%d</strong>.";
#elif L==10	// tr
	"The floor of room <strong>%s</strong> is now <strong>%d</strong>.";		// eviri lazim!
#endif

const char *Txt_The_folder_X_has_been_created_inside_the_folder_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Necessita traducci
#elif L==2	// de
	"Das Verzeichnis <strong>%s</strong> wurde im Verzeichnis <strong>%s</strong> erstellt.";
#elif L==3	// en
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";
#elif L==4	// es
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Le r&eacute;pertoire <strong>%s</strong> a &eacute;t&eacute; cr&eacute;&eacute; dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La cartella <strong>%s</strong> &egrave; stata creata all'interno della cartella <strong>%s</strong>.";
#elif L==8	// pl
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O diret&oacute;rio <strong>%s</strong> foi criado dentro do diret&oacute;rio <strong>%s</strong>.";
#elif L==10	// tr
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_folder_is_empty =
#if   L==1	// ca
	"La carpeta est&agrave; buida.";
#elif L==2	// de
	"Der Verzeichnis is leer.";
#elif L==3	// en
	"The folder is empty.";
#elif L==4	// es
	"La carpeta est&aacute; vac&iacute;a.";
#elif L==5	// fr
	"Le r&eacute;pertoire est vide.";
#elif L==6	// gn
	"La carpeta est&aacute; vac&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"La cartella &egrave; vuota.";
#elif L==8	// pl
	"Folder jest pusty.";
#elif L==9	// pt
	"O diret&oacute;rio est&aacute; vazio.";
#elif L==10	// tr
	"The folder is empty.";			// eviri lazim!
#endif

const char *Txt_The_folder_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La carpeta <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Verzeichnis <strong>%s</strong> wurde umbenannt in <strong>%s</strong>.";
#elif L==3	// en
	"The folder <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"La carpeta <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The folder <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"La carpeta <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"La cartella <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"The folder <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A pasta <strong>%s</strong> foi renomeada como <strong>%s</strong>.";
#elif L==10	// tr
	"The folder <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_folder_name_X_has_not_changed_because_there_is_already_a_folder_or_a_file_with_the_name_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Necessita traducci
#elif L==2	// de
	"Der Name des Verzeichnisses <strong>%s</strong> wurde nicht ge&auml;ndert,"
	" weil bereits ein Verzeichnis oder eine Datei mit Namen <strong>%s</strong> besteht.";
#elif L==3	// en
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";
#elif L==4	// es
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della cartella <strong>%s</strong> non &egrave; cambiato"
	" perch&egrave; c'&egrave; gi&agrave; una cartella o un file con il nome <strong>%s</strong>.";
#elif L==8	// pl
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome da pasta <strong>%s</strong> n&atilde;o foi alterado"
	" porque j&aacute; existe uma pasta ou um arquivo com o nome <strong>%s</strong>.";
#elif L==10	// tr
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_game_has_been_modified =
#if   L==1	// ca
	"El joc ha estat modificat.";
#elif L==2	// de
	"Das Spiel wurde ge&auml;ndert.";
#elif L==3	// en
	"The game has been modified.";
#elif L==4	// es
	"El juego ha sido modificado.";
#elif L==5	// fr
	"Le jeu a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El juego ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"Il gioco &egrave; stato modificato.";
#elif L==8	// pl
	"Gra zosta&lstrok;o zmodyfikowane.";
#elif L==9	// pt
	"O jogo foi modificado.";
#elif L==10	// tr
	"Oyun de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grup <strong>%s</strong> ja existeix.";
#elif L==2	// de
	"Die Gruppe <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The group <strong>%s</strong> already exists.";
#elif L==4	// es
	"El grupo <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"Le groupe <strong>%s</strong> existe d&eacute;j&agrave;.";
#elif L==6	// gn
	"El grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"Grupa <strong>%s</strong> ju&zdot; istnieje.";
#elif L==9	// pt
	"O grupo <strong>%s</strong> j&aacute; existe.";
#elif L==10	// tr
	"The group <strong>%s</strong> already exists.";	// eviri lazim!
#endif

const char *Txt_The_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El grup <strong>%s</strong>"
	" ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Die Gruppe <strong>%s</strong>"
	" wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The group <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El grupo <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le groupe <strong>%s</strong>"
	" a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El grupo <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong>"
	" &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Grupa <strong>%s</strong>"
	" zosta&lstrok;a przemianowana na <strong>%s</strong>.";
#elif L==9	// pt
	"O grupo <strong>%s</strong>"
	" foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The group <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_group_X_is_now_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grup <strong>%s</strong> ara est&agrave; tancat.";
#elif L==2	// de
	"Die Gruppe <strong>%s</strong> ist jetzt geschlossen.";
#elif L==3	// en
	"The group <strong>%s</strong> is now closed.";
#elif L==4	// es
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";
#elif L==5	// fr
	"Le groupe <strong>%s</strong> est maintenant ferm&eacute;.";
#elif L==6	// gn
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> &egrave; ora chiuso.";
#elif L==8	// pl
	"Grupa <strong>%s</strong> jest teraz zamkni&eogon;ta.";
#elif L==9	// pt
	"O grupo <strong>%s</strong> est&aacute; agora fechado.";
#elif L==10	// tr
	"The group <strong>%s</strong> is now closed.";			// eviri lazim!
#endif

const char *Txt_The_group_X_is_now_open =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grup <strong>%s</strong> ara est&agrave; obert.";
#elif L==2	// de
	"Die Gruppe <strong>%s</strong> ist jetzt ge&ouml;ffnet.";
#elif L==3	// en
	"The group <strong>%s</strong> is now open.";
#elif L==4	// es
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";
#elif L==5	// fr
	"Le groupe <strong>%s</strong> est maintenant ouvert.";
#elif L==6	// gn
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> &egrave; ora aperto.";
#elif L==8	// pl
	"Grupa <strong>%s</strong> jest teraz otwarta.";
#elif L==9	// pt
	"O grupo <strong>%s</strong> est&aacute; agora aberto.";
#elif L==10	// tr
	"The group <strong>%s</strong> is now open.";			// eviri lazim!
#endif

const char *Txt_The_group_X_does_not_have_a_student_limit_now =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grup <strong>%s</strong> ara no t&eacute; cap l&iacute;mit d'estudiants.";
#elif L==2	// de
	"Die Gruppe <strong>%s</strong> hat jetzt kein Studentenlimit.";
#elif L==3	// en
	"The group <strong>%s</strong> does not have a student limit now.";
#elif L==4	// es
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";
#elif L==5	// fr
	"Le groupe <strong>%s</strong> n'a plus de limite d'&eacute;tudiants.";
#elif L==6	// gn
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> ora non ha limite di studenti.";
#elif L==8	// pl
	"Grupa <strong>%s</strong> nie ma teraz &zdot;adnych limit&oacute;w uczni&oacute;w.";
#elif L==9	// pt
	"O grupo <strong>%s</strong> n&atilde;o tem limite de estudantes agora.";
#elif L==10	// tr
	"The group <strong>%s</strong> does not have a student limit now.";		// eviri lazim!
#endif

const char *Txt_The_groups_will_automatically_open =
#if   L==1	// ca
	"Els grups s'obriran autom&agrave;ticament.";
#elif L==2	// de
	"Die Gruppen werden automatisch ge&ouml;ffnet.";
#elif L==3	// en
	"The groups will automatically open.";
#elif L==4	// es
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";
#elif L==5	// fr
	"Les groupes s'ouvriront automatiquement.";
#elif L==6	// gn
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==7	// it
	"I gruppi si aprir&agrave; automaticamente.";
#elif L==8	// pl
	"Grupy otworz&aogon; si&eogon; automatycznie.";
#elif L==9	// pt
	"Os grupos ser&atilde;o abertos automaticamente.";
#elif L==10	// tr
	"The groups will automatically open.";			// eviri lazim!
#endif

const char *Txt_The_groups_will_not_automatically_open =
#if   L==1	// ca
	"Els grups no s'obriran autom&agrave;ticament.";
#elif L==2	// de
	"Die Gruppen werden nicht automatisch ge&ouml;ffnet.";
#elif L==3	// en
	"The groups will not automatically open.";
#elif L==4	// es
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";
#elif L==5	// fr
	"Les groupes ne s'ouvriront pas automatiquement.";
#elif L==6	// gn
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==7	// it
	"I gruppi non si aprir&agrave; automaticamente.";
#elif L==8	// pl
	"Grupy nie otworz&aogon; si&eogon; automatycznie.";
#elif L==9	// pt
	"Os grupos n&atilde;o ser&atilde;o abertos automaticamente.";
#elif L==10	// tr
	"The groups will not automatically open.";			// eviri lazim!
#endif

const char *Txt_The_holiday_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La festivitat <strong>%s</strong>"
	" ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Feiertag <strong>%s</strong>"
	" wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The holiday <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"La festividad <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"La f&ecirc;te <strong>%s</strong>"
	" a &eacute;t&eacute; renomm&eacute;e en <strong>%s</strong>.";
#elif L==6	// gn
	"La festividad <strong>%s</strong>"
	" ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La festivit&agrave; <strong>%s</strong>"
	" &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"&Sacute;wi&eogon;to <strong>%s</strong>"
	" zosta&lstrok;o przemianowane na <strong>%s</strong>.";
#elif L==9	// pt
	"O feriado <strong>%s</strong>"
	" foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The holiday <strong>%s</strong>"
	" has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_HTML_file_has_been_received_successfully =
#if   L==1	// ca
	"L'arxiu <em>HTML</em> s'ha rebut correctament.";
#elif L==2	// de
	"Die <em>HTML</em>-Datei wurde erfolgreich empfangen.";
#elif L==3	// en
	"The <em>HTML</em> file has been received successfully.";
#elif L==4	// es
	"El archivo <em>HTML</em> se ha recibido correctamente.";
#elif L==5	// fr
	"Le fichier <em>HTML</em> a bien &eacute;t&eacute; re&ccedil;u.";
#elif L==6	// gn
	"El archivo <em>HTML</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il <em>HTML</em> file &egrave; stato ricevuto con successo.";
#elif L==8	// pl
	"Plik <em>HTML</em> zosta&lstrok; pomy&sacute;lnie odebrany.";
#elif L==9	// pt
	"O arquivo <em>HTML</em> foi recebido com sucesso.";
#elif L==10	// tr
	"The <em>HTML</em> file has been received successfully.";	// eviri lazim!
#endif

const char *Txt_The_ID_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"S'ha confirmat l'ID (DNI/c&egrave;dula) <strong>%s</strong>.";
#elif L==2	// de
	"Die ID <strong>%s</strong>"
	" wurde best&auml;tigt.";
#elif L==3	// en
	"The ID <strong>%s</strong>"
	" has been confirmed.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";
#elif L==5	// fr
	"L'ID <strong>%s</strong>"
	" a &eacute;t&eacute; confirm&eacute;.";
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";		// Okoteve traduccin
#elif L==7	// it
	"Il ID <strong>%s</strong>"
	" &egrave; stato confermato.";
#elif L==8	// pl
	"Identyfikator <strong>%s</strong>"
	" zosta&lstrok; potwierdzony.";
#elif L==9	// pt
	"O ID <strong>%s</strong>"
	" foi confirmado.";
#elif L==10	// tr
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// eviri lazim!
#endif

const char *Txt_The_ID_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'ID (DNI/c&egrave;dula) <strong>%s</strong>"
	" s'ha registrat correctament.";
#elif L==2	// de
	"Die ID <strong>%s</strong>"
	" wurde korrekt registriert.";
#elif L==3	// en
	"The ID <strong>%s</strong>"
	" has been registered successfully.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==5	// fr
	"L'ID <strong>%s</strong>"
	" a &eacute;t&eacute; enregistr&eacute; correctement.";
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7	// it
	"Il ID <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8	// pl
	"Identyfikator <strong>%s</strong>"
	" zosta&lstrok; poprawnie zarejestrowany.";
#elif L==9	// pt
	"O ID <strong>%s</strong>"
	" foi registrado com sucesso.";
#elif L==10	// tr
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// eviri lazim!
#endif

const char *Txt_The_ID_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'ID (DNI/c&egrave;dula) <strong>%s</strong> no &eacute;s v&agrave;lid."
	" Nom&eacute;s pot contenir n&uacute;meros (almenys un) i lletres.";
#elif L==2	// de
	"Die ID <strong>%s</strong> ist ung&uuml;ltig."
	" Es darf nur Zahlen (mindestens eine) und Buchstaben enthalten.";
#elif L==3	// en
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";
#elif L==5	// fr
	"L'ID <strong>%s</strong> n'est pas valide."
	" Il ne peut contenir que des chiffres (au moins un) et des lettres.";
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";		// Okoteve traduccin
#elif L==7	// it
	"L'ID <strong>%s</strong> non &egrave; valido."
	" Pu&ograve; contenere solo numeri (almeno uno) e lettere.";
#elif L==8	// pl
	"ID <strong>%s</strong> jest nieprawid&lstrok;owy."
	" Mo&zdot;e zawiera&cacute; tylko cyfry (co najmniej jedn&aogon;) i litery.";
#elif L==9	// pt
	"O ID <strong>%s</strong> n&atilde;o &eacute; v&aacute;lido."
	" S&oacute; pode conter n&uacute;meros (pelo menos um) e letras.";
#elif L==10	// tr
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// eviri lazim!
#endif

const char *Txt_The_ID_is_used_in_order_to_facilitate_ =
#if   L==1	// ca
	"L'ID s'utilitza per tal de facilitar"
	" la inscripci&oacute; en les assignatures"
	" i la consulta de qualificacions."
	" Si hi ha diverses versions de la ID"
	" (per ex. DNI amb o sense lletra),"
	" indiqueu totes les versions.";
#elif L==2	// de
	"Die ID wird zu erleichtern,"
	" um die Einschreibung in Kurse"
	" und Beratungskompetenz eingesetzt."
	" Wenn es mehrere Versionen der ID"
	" (zB. mit oder ohne Buchstabe),"
	" schreiben alle Versionen.";
#elif L==3	// en
	"The ID is used in order to facilitate"
	" enrolment in courses"
	" and consulting marks."
	" If there are multiple versions of the ID"
	" (eg. with or without letter),"
	" write out all versions.";
#elif L==4	// es
	"El ID se utiliza para facilitar"
	" la inscripci&oacute;n en las asignaturas"
	" y la consulta de calificaciones."
	" Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra),"
	" indique todas las versiones.";
#elif L==5	// fr
	"L'ID est utilis&eacute; afin de faciliter"
	" l'inscription &agrave; des mati&egrave;res"
	" et la consulte des notes."
	" S'il existe plusieurs versions du ID"
	" (par example avec ou sans lettre),"
	" &eacute;crire toutes les versions.";
#elif L==6	// gn
	"El ID se utiliza para facilitar"
	" la inscripci&oacute;n en las asignaturas"
	" y la consulta de calificaciones."
	" Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra),"
	" indique todas las versiones.";	// Okoteve traduccin
#elif L==7	// it
	"L'ID &egrave; utilizzato per agevolare"
	" l'iscrizione ai corsi"
	" e consultare dei risultati."
	" Se ci sono pi&ugrave; versioni del ID"
	" (ad es. con o senza lettera),"
	" scrivere tutte le versioni.";
#elif L==8	// pl
	"Identyfikator jest u&zdot;ywany w celu u&lstrok;atwienia"
	" rejestracji w kursach"
	" i skonsultuje znak&oacute;w."
	" Je&sacute;li istnieje wiele wersji ID"
	" (np. Z lub bez litery),"
	" wypisa&cacute; wszystkie wersje.";
#elif L==9	// pt
	"O n&ordm; de identifica&ccedil;&atilde;o &eacute; usado para facilitar"
	" a inscri&ccedil;&atilde;o em disciplinas"
	" e consultas de notas."
	" Se houver v&aacute;rias vers&otilde;es do n&ordm; de identifica&ccedil;&atilde;o"
	" (eg., com ou sem letra),"
	" escreva todas as vers&otilde;es.";
#elif L==10	// tr
	"The ID is used in order to facilitate"
	" enrolment in courses"
	" and consulting marks."
	" If there are multiple versions of the ID"
	" (eg. with or without letter),"
	" write out all versions.";		// eviri lazim!
#endif

const char *Txt_The_ID_nickname_or_email_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El @nick, el correu electr&ograve;nic o l'ID (DNI/c&egrave;dula) <strong>%s</strong> no &eacute;s v&agrave;lid.";
#elif L==2	// de
	"Der @nick, die E-Mail-Adresse oder die ID <strong>%s</strong> ist ung&uuml;ltig.";
#elif L==3	// en
	"The @nick, email or ID <strong>%s</strong> is not valid.";
#elif L==4	// es
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";
#elif L==5	// fr
	"Le @nick, email ou ID <strong>%s</strong> n'est pas valide.";
#elif L==6	// gn
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"Il @nome-utente, email o ID <strong>%s</strong> non &egrave; valido.";
#elif L==8	// pl
	"@nick, e-mail lub ID <strong>%s</strong> jest nieprawid&lstrok;owy.";
#elif L==9	// pt
	"O @nick, email ou ID <strong>%s</strong> n&atilde;o &eacute; v&aacute;lido.";
#elif L==10	// tr
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// eviri lazim!
#endif

const char *Txt_The_ID_X_matches_one_of_the_existing =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'ID (DNI/c&egrave;dula) <strong>%s</strong>"
	" coincideix amb un dels existents.";
#elif L==2	// de
	"Die ID <strong>%s</strong>"
	" stimmt mit einer der vorhandenen &uuml;berein.";
#elif L==3	// en
	"The ID <strong>%s</strong>"
	" matches one of the existing.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";
#elif L==5	// fr
	"L'ID <strong>%s</strong>"
	" correspond &agrave; l'un des existants.";
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";		// Okoteve traduccin
#elif L==7	// it
	"L'ID <strong>%s</strong>"
	" corrisponde a uno degli esistenti.";
#elif L==8	// pl
	"ID <strong>%s</strong>"
	" pasuje do jednego z istniej&aogon;cych.";
#elif L==9	// pt
	"O ID <strong>%s</strong>"
	" corresponde a um dos existentes.";
#elif L==10	// tr
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// eviri lazim!
#endif

const char *Txt_The_institution_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La instituci&oacute; <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Die Institution <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"L'institution <strong>%s</strong> a &eacute;t&eacute; renomm&eacute;e <strong>%s</strong>.";
#elif L==6	// gn
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'istituzione <strong>%s</strong> &egrave; stata rinominata <strong>%s</strong>.";
#elif L==8	// pl
	"Instytucja <strong>%s</strong> zosta&lstrok;a przemianowana na <strong>%s</strong>.";
#elif L==9	// pt
	"A institui&ccedil;&atilde;o <strong>%s</strong> foi renomeada como <strong>%s</strong>.";
#elif L==10	// tr
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_The_institution_of_the_center_has_changed =
#if   L==1	// ca
	"La instituci&oacute; del centre ha canviat.";
#elif L==2	// de
	"Die Institution des Zentrums hat sich ge&auml;ndert.";
#elif L==3	// en
	"The institution of the center has changed.";
#elif L==4	// es
	"La instituci&oacute;n del centro ha cambiado.";
#elif L==5	// fr
	"L'institution du centre a chang&eacute;.";
#elif L==6	// gn
	"La instituci&oacute;n del centro ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'istituzione del centro &egrave; cambiata.";
#elif L==8	// pl
	"Zmieni&lstrok;a si&eogon; instytucja o&sacute;rodka.";
#elif L==9	// pt
	"A institui&ccedil;&atilde;o do centro mudou.";
#elif L==10	// tr
	"The institution of the center has changed.";		// eviri lazim!
#endif

const char *Txt_The_institution_of_the_department_has_changed =
#if   L==1	// ca
	"La instituci&oacute; del departament ha canviat.";
#elif L==2	// de
	"Die Institution des Fachbereichs hat sich ge&auml;ndert.";
#elif L==3	// en
	"The institution of the department has changed.";
#elif L==4	// es
	"La instituci&oacute;n del departamento ha cambiado.";
#elif L==5	// fr
	"L'institution du d&eacute;partement a chang&eacute;.";
#elif L==6	// gn
	"La instituci&oacute;n del departamento ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'istituzione del dipartimento &egrave; cambiata.";
#elif L==8	// pl
	"Zmieni&lstrok;a si&eogon; instytucja wydzia&lstrok;u.";
#elif L==9	// pt
	"A institui&ccedil;&atilde;o do departamento mudou.";
#elif L==10	// tr
	"The institution of the department has changed.";	// eviri lazim!
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El codi institucional de l'assignatura <strong>%s</strong>"
	" ha canviat a <strong>%s</strong>.";
#elif L==2	// de
	"Der Institutionscode des Studiengangs <strong>%s</strong>"
	" hat sich in <strong>%s</strong> ge&auml;ndert.";
#elif L==3	// en
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"Le code institutionnel de la mati&egrave;re <strong>%s</strong>"
	" est devenu <strong>%s</strong>.";
#elif L==6	// gn
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";			// Okoteve traduccin
#elif L==7	// it
	"Il codice istituzionale del corso <strong>%s</strong>"
	" &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"Kod instytucjonalny kursu <strong>%s</strong>"
	" zosta&lstrok; zmieniony na <strong>%s</strong>.";
#elif L==9	// pt
	"O c&oacute;digo institucional da disciplina <strong>%s</strong>"
	" mudou para <strong>%s</strong>.";
#elif L==10	// tr
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El codi institucional de l'assignatura <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Der Hochschulcode des Studiengangs <strong>%s</strong> hat sich nicht ge&aauml;ndert.";
#elif L==3	// en
	"The institutional code of the course <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le code institutionnel de la mati&egrave;re <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice istituzionale del corso <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Kod instytucjonalny kursu <strong>%s</strong> nie uleg&lstrok; zmianie.";
#elif L==9	// pt
	"O c&oacute;digo institucional da disciplina <strong>%s</strong> n&atilde;o mudou.";
#elif L==10	// tr
	"The institutional code of the course <strong>%s</strong> has not changed.";		// eviri lazim!
#endif

const char *Txt_The_integrated_editor_is_not_yet_available =
#if   L==1	// ca
	"L'editor integrat encara no est&agrave; disponible.";
#elif L==2	// de
	"Der integrierte Editor ist noch nicht verf&uuml;gbar.";
#elif L==3	// en
	"The integrated editor is not yet available.";
#elif L==4	// es
	"El editor integrado a&uacute;n no est&aacute; disponible.";
#elif L==5	// fr
	"L'&eacute;diteur int&eacute;gr&eacute; n'est pas encore disponible.";
#elif L==6	// gn
	"El editor integrado a&uacute;n no est&aacute; disponible.";	// Okoteve traduccin
#elif L==7	// it
	"L'editor integrato non &egrave; ancora disponibile.";
#elif L==8	// pl
	"Zintegrowany edytor nie jest jeszcze dost&eogon;pny.";
#elif L==9	// pt
	"O editor integrado ainda n&atilde;o est&aacute; dispon&iacute;vel.";
#elif L==10	// tr
	"The integrated editor is not yet available.";			// eviri lazim!
#endif

const char *Txt_The_link_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traducci
#elif L==2	// de
	"Die Link <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==3	// en
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==4	// es
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Le lien <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Il link <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==8	// pl
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A liga&ccedil;&atilde;o <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#elif L==10	// tr
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_link_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"L'enlla&ccedil; <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Link <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le lien <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il link <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Link <strong>%s</strong> zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"O link <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_list_of_X_courses_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La llista de %u assignatures &eacute;s massa gran per mostrar-se.";
#elif L==2	// de
	"Die Liste mit %u Kursen ist zu gro&szlig;, um angezeigt zu werden.";
#elif L==3	// en
	"The list of %u courses is too large to be displayed.";
#elif L==4	// es
	"La lista de %u asignaturas es demasiado grande para mostrarla.";
#elif L==5	// fr
	"La liste des %u mati&egrave;res est trop longue pour &ecirc;tre affich&eacute;e.";
#elif L==6	// gn
	"La lista de %u asignaturas es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==7	// it
	"La lista di %u corsi &egrave; troppo grande per essere mostrata.";
#elif L==8	// pl
	"Lista %u kurs&oacute;w jest za du&zdot;a, aby mo&zdot;na j&aogon; by&lstrok;o wy&sacute;wietli&cacute;.";
#elif L==9	// pt
	"A lista de %u disciplinas &eacute; muito grande para ser exibida.";
#elif L==10	// tr
	"The list of %u courses is too large to be displayed.";			// eviri lazim!
#endif

const char *Txt_The_list_of_X_users_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La llista de %u usuaris &eacute;s massa gran per mostrar-se.";
#elif L==2	// de
	"Die Liste mit %u Benutzern ist zu gro&szlig;, um angezeigt zu werden.";
#elif L==3	// en
	"The list of %u users is too large to be displayed.";
#elif L==4	// es
	"La lista de %u usuarios es demasiado grande para mostrarla.";
#elif L==5	// fr
	"La liste des %u utilisateurs est trop longue pour &ecirc;tre affich&eacute;e.";
#elif L==6	// gn
	"La lista de %u usuarios es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==7	// it
	"La lista di %u utenti &egrave; troppo grande per essere mostrata.";
#elif L==8	// pl
	"Lista %u u&zdot;ytkownik&oacute;w jest za du&zdot;a, aby mo&zdot;na j&aogon; by&lstrok;o wy&sacute;wietli&cacute;.";
#elif L==9	// pt
	"A lista de %u usu&aacute;rios &eacute; muito grande para ser exibida.";
#elif L==10	// tr
	"The list of %u users is too large to be displayed.";		// eviri lazim!
#endif

const char *Txt_The_location_of_the_building_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La ubicaci&oacute; de l'edifici <strong>%s</strong>"
	" ha canviat a <strong>%s</strong>.";
#elif L==2	// de
	"Der Standort des Geb&auml;udes <strong>%s</strong>"
	" wurde in <strong>%s</strong> ge&auml;ndert.";
#elif L==3	// en
	"The location of the building <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4	// es
	"La ubicaci&oacute;n del edificio <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"L'emplacement du b&acirc;timent <strong>%s</strong>"
	" a &eacute;t&eacute; remplac&eacute; par <strong>%s</strong>.";
#elif L==6	// gn
	"La ubicaci&oacute;n del edificio <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La posizione dell'edificio <strong>%s</strong>"
	" &egrave; cambiata in <strong>%s</strong>.";
#elif L==8	// pl
	"Lokalizacja budynek <strong>%s</strong>"
	" zmieni&lstrok;a si&eogon; na <strong>%s</strong>.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o do edif&iacute;cio <strong>%s</strong>"
	" mudou para <strong>%s</strong>.";
#elif L==10	// tr
	"The location of the building <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_location_of_the_building_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La ubicaci&oacute; de l'edifici <strong>%s</strong>"
	" no ha canviat.";
#elif L==2	// de
	"Der Standort des Geb&auml;udes <strong>%s</strong>"
	" hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The location of the building <strong>%s</strong>"
	" has not changed.";
#elif L==4	// es
	"La ubicaci&oacute;n del edificio <strong>%s</strong>"
	" no ha cambiado.";
#elif L==5	// fr
	"L'emplacement du b&acirc;timent <strong>%s</strong>"
	" n'a pas chang&eacute;.";
#elif L==6	// gn
	"La ubicaci&oacute;n del edificio <strong>%s</strong>"
	" no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"La posizione dell'edificio <strong>%s</strong>"
	" non &egrave; cambiata.";
#elif L==8	// pl
	"Lokalizacja budynek <strong>%s</strong>"
	" nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o do edif&iacute;cio <strong>%s</strong>"
	" n&atilde;o foi alterada.";
#elif L==10	// tr
	"The location of the building <strong>%s</strong>"
	" has not changed.";	// eviri lazim!
#endif

const char *Txt_The_lower_limit_of_correct_answers_must_be_less_than_or_equal_to_the_upper_limit =
#if   L==1	// ca
	"El l&iacute;mit inferior de respostes correctes"
	" ha de ser inferior o igual al l&iacute;mit superior.";
#elif L==2	// de
	"Die Untergrenze richtiger Antworten"
	" muss kleiner oder gleich der Obergrenze sein.";
#elif L==3	// en
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";
#elif L==4	// es
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";
#elif L==5	// fr
	"La limite inf&eacute;rieure des bonnes r&eacute;ponses"
	" doit &ecirc;tre inf&eacute;rieure ou &eacute;gale &agrave; la limite sup&eacute;rieure.";
#elif L==6	// gn
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";	// Okoteve traduccin
#elif L==7	// it
	"Il limite inferiore di risposte corrette"
	" deve essere minore o uguale al limite superiore.";
#elif L==8	// pl
	"Dolna granica poprawnych odpowiedzi"
	" musi by&cacute; mniejsza lub r&oacute;wna g&oacute;rnej granicy.";
#elif L==9	// pt
	"O limite inferior de respostas corretas"
	" deve ser menor ou igual ao limite superior.";
#elif L==10	// tr
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// eviri lazim!
#endif

const char *Txt_The_maximum_number_of_students_in_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre m&agrave;xim d'estudiants"
	" del grup <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die maximale Anzahl der Studenten"
	" in Gruppe <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The maximum number of students"
	" in group <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le nombre maximum d'&eacute;tudiants"
	" du groupe <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Maksymalna liczba uczni&oacute;w"
	" w grupie <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O n&uacute;mero m&aacute;ximo de estudantes"
	" no grupo <strong>%s</strong> n&atilde;o foi alterado.";
#elif L==10	// tr
	"The maximum number of students"
	" in group <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_maximum_number_of_students_in_group_X_is_now_Y =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"El nombre m&agrave;xim d'estudiants"
	" del grup <strong>%s</strong> ara &eacute;s <strong>%u</strong>.";
#elif L==2	// de
	"Die maximale Anzahl der Studenten"
	" in Gruppe <strong>%s</strong> betr&auml;gt jetzt <strong>%u</strong>.";
#elif L==3	// en
	"The maximum number of students"
	" in group <strong>%s</strong> is now <strong>%u</strong>.";
#elif L==4	// es
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> ahora es <strong>%u</strong>.";
#elif L==5	// fr
	"Le nombre maximum d'&eacute;tudiants"
	" du groupe <strong>%s</strong> est maintenant de <strong>%u</strong>.";
#elif L==6	// gn
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> ahora es <strong>%u</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> &egrave; ora <strong>%u</strong>.";
#elif L==8	// pl
	"Maksymalna liczba student&oacute;w"
	" w grupie <strong>%s</strong> wynosi teraz <strong>%u</strong>.";
#elif L==9	// pt
	"O n&uacute;mero m&aacute;ximo de estudantes"
	" no grupo <strong>%s</strong> &eacute; agora <strong>%u</strong>.";
#elif L==10	// tr
	"The maximum number of students"
	" in group <strong>%s</strong> is now <strong>%u</strong>.";		// eviri lazim!
#endif

const char *Txt_The_message_has_not_been_sent_to_any_recipient =
#if   L==1	// ca
	"El missatge no ha estat enviat a cap destinatari.";
#elif L==2	// de
	"Die Nachricht wurde an keinen Empf&auml;nger gesendet.";
#elif L==3	// en
	"The message has not been sent to any recipient.";
#elif L==4	// es
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";
#elif L==5	// fr
	"Le message n'a &eacute;t&eacute; envoy&eacute; &agrave; aucun destinataire.";
#elif L==6	// gn
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";	// Okoteve traduccin
#elif L==7	// it
	"Il messaggio non &egrave; stato inviato a nessun destinatario.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; nie zosta&lstrok;a wys&lstrok;ana do &zdot;adnego adresata.";
#elif L==9	// pt
	"A mensagem n&atilde;o foi enviada para nenhum destinat&aacute;rio.";
#elif L==10	// tr
	"The message has not been sent to any recipient.";		// eviri lazim!
#endif

const char *Txt_The_message_has_been_sent_to_1_recipient =
#if   L==1	// ca
	"El missatge s'ha enviat a 1 destinatari.";
#elif L==2	// de
	"Die Nachricht wurde an 1 Empf&auml;nger gesendet.";
#elif L==3	// en
	"The message has been sent to 1 recipient.";
#elif L==4	// es
	"El mensaje se ha enviado a 1 destinatario.";
#elif L==5	// fr
	"Le message a &eacute;t&eacute; envoy&eacute; &agrave; 1 destinataire.";
#elif L==6	// gn
	"El mensaje se ha enviado a 1 destinatario.";	// Okoteve traduccin
#elif L==7	// it
	"Il messaggio &egrave; stato inviato a 1 destinatario.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; zosta&lstrok;a wys&lstrok;ana do 1 odbiorc&oacute;w.";
#elif L==9	// pt
	"A mensagem foi enviada para 1 destinat&aacute;rio.";
#elif L==10	// tr
	"The message has been sent to 1 recipient.";	// eviri lazim!
#endif

const char *Txt_The_message_has_been_sent_to_X_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"El missatge s'ha enviat a %u destinataris.";
#elif L==2	// de
	"Die Nachricht wurde an %u Empf&auml;nger gesendet.";
#elif L==3	// en
	"The message has been sent to %u recipients.";
#elif L==4	// es
	"El mensaje se ha enviado a %u destinatarios.";
#elif L==5	// fr
	"Le message a &eacute;t&eacute; envoy&eacute; &agrave; %u destinataires.";
#elif L==6	// gn
	"El mensaje se ha enviado a %u destinatarios.";	// Okoteve traduccin
#elif L==7	// it
	"Il messaggio &egrave; stato inviato a %u destinatari.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; zosta&lstrok;a wys&lstrok;ana do %u odbiorc&oacute;w.";
#elif L==9	// pt
	"A mensagem foi enviada para %u destinat&aacute;rios.";
#elif L==10	// tr
	"The message has been sent to %u recipients.";	// eviri lazim!
#endif

const char *Txt_The_name_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nom <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Der Name <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The name <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le nom <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El nombre <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Nazwa <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O nome <strong>%s</strong> n&atilde;o foi alterado.";
#elif L==10	// tr
	"The name <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_new_application_key_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nova clau d'aplicaci&oacute; &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Der neue Anwendungsschl&uuml;ssel ist <strong>%s</strong>.";
#elif L==3	// en
	"The new application key is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva clave de aplicaci&oacute;n es <strong>%s</strong>.";
#elif L==5	// fr
	"La nouvelle cl&eacute; d'application est <strong>%s</strong>.";
#elif L==6	// gn
	"La nueva clave de aplicaci&oacute;n es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La nuova chiave dell'applicazione &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Nowy klucz aplikacji to <strong>%s</strong>.";
#elif L==9	// pt
	"A nova chave do aplicativo &eacute; <strong>%s</strong>.";
#elif L==10	// tr
	"The new application key is <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_new_IP_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nova adre&ccedil;a IP &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Die neue IP-Adresse ist <strong>%s</strong>.";
#elif L==3	// en
	"The new IP address is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";
#elif L==5	// fr
	"La nouvelle adresse IP est <strong>%s</strong>.";
#elif L==6	// gn
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nuovo indirizzo IP &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Nowy adres IP to <strong>%s</strong>.";
#elif L==9	// pt
	"O novo endere&ccedil;o IP &eacute; <strong>%s</strong>.";
#elif L==10	// tr
	"The new IP address is <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_new_logo_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nou logotip &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Das neue Logo ist <strong>%s</strong>.";
#elif L==3	// en
	"The new logo is <strong>%s</strong>.";
#elif L==4	// es
	"El nuevo logo es <strong>%s</strong>.";
#elif L==5	// fr
	"Le nouveau logo est <strong>%s</strong>.";
#elif L==6	// gn
	"El nuevo logo es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nuovo logo &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Nowe logo to <strong>%s</strong>.";
#elif L==9	// pt
	"O novo logotipo &eacute; <strong>%s</strong>.";
#elif L==10	// tr
	"The new logo is <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_nickname_has_been_registered_successfully =
#if   L==1	// ca
	"El sobrenom s'ha registrat correctament.";
#elif L==2	// de
	"Der Spitzname wurde erfolgreich registriert.";
#elif L==3	// en
	"The nickname has been registered successfully.";
#elif L==4	// es
	"El apodo se ha registrado correctamente.";
#elif L==5	// fr
	"Le surnom a &eacute;t&eacute; enregistr&eacute; avec succ&egrave;s.";
#elif L==6	// gn
	"El apodo se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7	// it
	"Il nome utente &egrave; stato registrato con successo.";
#elif L==8	// pl
	"Pseudonim zosta&lstrok; pomy&sacute;lnie zarejestrowany.";
#elif L==9	// pt
	"A alcunha foi registrada com sucesso.";
#elif L==10	// tr
	"The nickname has been registered successfully.";	// eviri lazim!
#endif

const char *Txt_The_nickname_is_not_valid_ =
#if   L==1	// ca
	"El sobrenom no &eacute;s v&agrave;lid.<br />"
	"El vostre &agrave;lies ha de tenir una longitud d'entre %u i %u car&agrave;cters (sense incloure la @ inicial).<br />"
	"Cada car&agrave;cter despr&eacute;s de la inicial @ pot ser una lletra (a-z, A-Z),"
	" un dgit (0-9) o &quot;_&quot;.";
#elif L==2	// de
	"Der Spitzname ist ung&uuml;ltig.<br />"
	"Ihr Spitzname muss zwischen %u und %u Zeichen lang sein (ohne den Anfangsbuchstaben @).<br />"
	"Jedes Zeichen nach dem Anfangsbuchstaben @ kann ein Buchstabe (a-z, A-Z), eine Ziffer (0-9) oder &quot;_&quot; sein.";
#elif L==3	// en
	"The nickname is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";
#elif L==4	// es
	"El apodo no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";
#elif L==5	// fr
	"Le surnom n'est pas valide.<br />"
	"Votre pseudo doit avoir une longueur comprise entre %u et %u caract&egrave;res (n'incluant pas l'initiale @).<br />"
	"Chaque caract&egrave;re apr&egrave;s l'initiale @ peut &ecirc;tre une lettre (a-z, A-Z), un chiffre (0-9) ou &quot;_&quot;.";
#elif L==6	// gn
	"El apodo no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";	// Okoteve traduccin
#elif L==7	// it
	"Il soprannome non &egrave; valido.<br />"
	"Il tuo nickname deve avere una lunghezza compresa tra %u e %u caratteri (esclusa l'iniziale @).<br />"
	"Ciascun carattere dopo l'iniziale @ pu&ograve; essere una lettera (a-z, A-Z), una cifra (0-9) o &quot;_&quot;.";
#elif L==8	// pl
	"Pseudonim jest nieprawid&lstrok;owy.<br />"
	"Tw&oacute;j pseudonim musi mie&cacute; d&lstrok;ugo&sacute;&cacute; od %u do %u znak&oacute;w (nie licz&aogon;c pocz&aogon;tkowego @).<br />"
	"Ka&zdot;dy znak po pocz&aogon;tkowym @ mo&zdot;e by&cacute; liter&aogon; (a-z, A-Z), cyfr&aogon; (0-9) lub &quot;_&quot;.";
#elif L==9	// pt
	"A alcunha n&atilde;o &eacute; v&aacute;lido.<br />"
	"Sua alcunha deve ter um tamanho entre %u e %u caracteres (sem incluir o inicial @).<br />"
	"Cada caractere ap&oacute;s inicial @ pode ser uma letra (a-z, A-Z),"
	" um dgito (0-9) ou &quot;_&quot;.";
#elif L==10	// tr
	"The nickname is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// eviri lazim!
#endif

const char *Txt_The_notice_will_appear_as_a_yellow_note_ =
#if   L==1	// ca
	"L'av&iacute;s apareixer&agrave; en forma de nota groga"
	" a l'esquerra de la p&agrave;gina"
	" i ser&agrave; <strong>visible per tot el m&oacute;n</strong>"
	" (no nom&eacute;s pels usuaris de l'assignatura).<br />"
	"<strong>El missatge ha de ser breu.</strong>"
	" Per crear un av&iacute;s extens"
	" o d'acc&eacute;s exclusiu per a estudiants,"
	" pot utilitzar el f&ograve;rum de l'assignatura,"
	" enviar un missatge a tots,"
	" o pujar un arxiu amb el text a la zona de documents.";
#elif L==2	// de
	"Die Ank&uuml;ndigung wird als ein gelber Hinweis"
	" auf der linken Seite der Seite erscheinen"
	" und wird <strong>weltweit</strong>"
	" (nicht nur von den Benutzern des Kurses) sichtbar sein.<br />"
	"<strong>Die Nachricht sollte kurz sein</strong>."
	" Wenn Sie eine lange Ank&uuml;ndigung"
	" oder einen Hinweis mit exklusivem Zugang f&uuml;r Studenten erstellen m&ouml;chten,"
	" k&ouml;nnen Sie das Forum des Kurses nutzen,"
	" eine Nachricht an alle senden"
	" oder eine Datei mit dem Text in den Dokumentenbereich hochladen.";
#elif L==3	// en
	"The notice will appear as a yellow note"
	" in the left of the page"
	" and will be <strong>visible worldwide</strong>"
	" (not just by users of the course).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice"
	" or a notice with exclusive access for students,"
	" you can use the forum of the course,"
	" send a message to all,"
	" or upload a file with the text to the documents area.";
#elif L==4	// es
	"El aviso aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina"
	" y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no solo por los usuarios de la asignatura).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso"
	" o de acceso exclusivo para estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos,"
	" o subir un archivo con el texto a la zona de documentos.";
#elif L==5	// fr
	"L'avis appara&icirc;tra comme une note jaune"
	" &agrave; gauche de la page"
	" et sera <strong>visible dans le monde entier</strong>"
	" (pas seulement par les utilisateurs du cours).<br />"
	"<strong>Le message doit &ecirc;tre court</strong>."
	" Si vous souhaitez cr&eacute;er un avis long"
	" ou un avis avec un acc&egrave;s exclusif pour les &eacute;tudiants,"
	" vous pouvez utiliser le forum du cours,"
	" envoyer un message &agrave; tous"
	" ou t&eacute;l&eacute;charger un fichier avec le texte dans la zone des documents.";
#elif L==6	// gn
	"El aviso aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina"
	" y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no solo por los usuarios de la asignatura).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso"
	" o de acceso exclusivo para estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos,"
	" o subir un archivo con el texto a la zona de documentos.";	// Okoteve traduccin
#elif L==7	// it
	"L'avviso verr&agrave; visualizzato come una nota di colore giallo"
	" nella parte sinistra della pagina"
	" e sar&agrave; <strong>visibile in tutto il mondo</strong>"
	" (non solo dagli utenti del corso).<br />"
	"<strong>Il messaggio deve essere breve</strong>."
	" Se si desidera creare un lungo avviso"
	" o una comunicazione con accesso esclusivo per gli studenti,"
	" &egrave; possibile utilizzare il forum del corso,"
	" inviare un messaggio a tutti,"
	" o caricare un file con il testo all'area documenti.";
#elif L==8	// pl
	"Og&lstrok;oszenie pojawi si&eogon; jako &zdot;&oacute;&lstrok;ty notatki"
	" w lewej strony"
	" i b&eogon;dzie <strong>widoczny na ca&lstrok;ym &sacute;wiecie</strong>"
	" (nie tylko przez u&zdot;ytkownik&oacute;w oczywi&sacute;cie).<br />"
	"<strong>Wiadomo&sacute;&cacute; powinna by&cacute; kr&oacute;tka</strong>."
	" Je&sacute;li chcesz stworzy&cacute; d&lstrok;ug&aogon; zawiadomienie"
	" lub zawiadomienie z wy&lstrok;&aogon;cznego dost&eogon;pu dla student&oacute;w,"
	" mo&zdot;na korzysta&cacute; z forum kursu,"
	" wys&lstrok;a&cacute; wiadomo&sacute;&cacute; do wszystkich,"
	" lub przes&lstrok;a&cacute; plik z tekstem do obszaru dokument&oacute;w.";
#elif L==9	// pt
	"O aviso aparecer&aacute; como uma nota amarela"
	" &agrave; esquerda da p&aacute;gina"
	" e ser&aacute; <strong>vis&iacute;vel em todo o mundo</strong>"
	" (n&atilde;o apenas pelos usu&aacute;rios do curso).<br />"
	"<strong>A mensagem deve ser curta</strong>."
	" Se voc&ecirc; quiser criar um aviso longo"
	" ou um aviso com acesso exclusivo para os estudantes,"
	" voc&ecirc; pode usar o f&oacute;rum do curso,"
	" enviar uma mensagem para todos"
	" ou enviar um arquivo com o texto para a &aacute;rea de documentos.";
#elif L==10	// tr
	"The notice will appear as a yellow note"
	" in the left of the page"
	" and will be <strong>visible worldwide</strong>"
	" (not just by users of the course).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice"
	" or a notice with exclusive access for students,"
	" you can use the forum of the course,"
	" send a message to all,"
	" or upload a file with the text to the documents area.";	// eviri lazim!
#endif

const char *Txt_The_new_web_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nova adre&ccedil;a web &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Die neue Webadresse ist <strong>%s</strong>.";
#elif L==3	// en
	"The new web address is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";
#elif L==5	// fr
	"La nouvelle adresse Web est <strong>%s</strong>.";
#elif L==6	// gn
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nuovo indirizzo web &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Nowy adres internetowy to <strong>%s</strong>.";
#elif L==9	// pt
	"O novo endere&ccedil;o web &eacute; <strong>%s</strong>.";
#elif L==10	// tr
	"The new web address is <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_The_new_description_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nova descripci&oacute; &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Die neue Beschreibung ist <strong>%s</strong>.";
#elif L==3	// en
	"The new description is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva descripci&oacute;n es <strong>%s</strong>.";
#elif L==5	// fr
	"La nouvelle description est <strong>%s</strong>.";
#elif L==6	// gn
	"La nueva descripci&oacute;n es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La nuova descrizione &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Nowy opis to <strong>%s</strong>.";
#elif L==9	// pt
	"A nova descri&ccedil;&atilde;o &eacute; <strong>%s</strong>.";
#elif L==10	// tr
	"The new description is <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_new_image_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nova imatge &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Das neue Bild ist <strong>%s</strong>.";
#elif L==3	// en
	"The new image is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva imagen es <strong>%s</strong>.";
#elif L==5	// fr
	"La nouvelle image est <strong>%s</strong>.";
#elif L==6	// gn
	"La nueva imagen es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La nuova image &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Nowy obraz to <strong>%s</strong>.";
#elif L==9	// pt
	"A nova imagem &eacute; <strong>%s</strong>.";
#elif L==10	// tr
	"The new image is <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_new_URL_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nou URL &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Die neue URL ist <strong>%s</strong>.";
#elif L==3	// en
	"The new URL is <strong>%s</strong>.";
#elif L==4	// es
	"El nuevo URL es <strong>%s</strong>.";
#elif L==5	// fr
	"La nouvelle URL est <strong>%s</strong>.";
#elif L==6	// gn
	"El nuevo URL es <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"La nuova URL &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Nowy adres URL to <strong>%s</strong>.";
#elif L==9	// pt
	"O novo URL &eacute; <strong>%s</strong>.";
#elif L==10	// tr
	"The new URL is <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_nickname_had_been_registered_by_another_user =
#if   L==1	// ca
	"L'&agrave;lies havia estat registrat per un altre usuari.";
#elif L==2	// de
	"Der Spitzname wurde von einem anderen Benutzer registriert.";
#elif L==3	// en
	"The nickname had been registered by another user.";
#elif L==4	// es
	"El apodo ya hab&iacute;a sido registrado por otro usuario.";
#elif L==5	// fr
	"Le surnom avait &eacute;t&eacute; enregistr&eacute; par un autre utilisateur.";
#elif L==6	// gn
	"El apodo ya hab&iacute;a sido registrado por otro usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome utente &egrave; stato registrato da un altro utente.";
#elif L==8	// pl
	"Pseudonim zosta&lstrok; zarejestrowany przez innego u&zdot;ytkownika.";
#elif L==9	// pt
	"A alcunha foi registrada por outro usu&aacute;rio.";
#elif L==10	// tr
	"The nickname had been registered by another user.";		// eviri lazim!
#endif

const char *Txt_The_nickname_matches_the_one_you_had_previously_registered =
#if   L==1	// ca
	"El sobrenom coincideix amb el que havies registrat anteriorment.";
#elif L==2	// de
	"Der Spitzname stimmt mit demjenigen &uuml;berein, den Sie zuvor registriert haben.";
#elif L==3	// en
	"The nickname matches the one you had previously registered.";
#elif L==4	// es
	"El apodo coincide con el que ya ten&iacute;a registrado previamente.";
#elif L==5	// fr
	"Le pseudo correspond &agrave; celui que vous aviez pr&eacute;c&eacute;demment enregistr&eacute;.";
#elif L==6	// gn
	"El apodo coincide con el que ya ten&iacute;a registrado previamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome utente coincide con quello che hai precedentemente registrato.";
#elif L==8	// pl
	"Pseudonim jest zgodny z tym, kt&oacute;ry wcze&sacute;niej zarejestrowa&lstrok;e&sacute;.";
#elif L==9	// pt
	"A alcunha corresponde &agrave;quela que voc&ecirc; registrou anteriormente.";
#elif L==10	// tr
	"The nickname matches the one you had previously registered.";		// eviri lazim!
#endif

const char *Txt_The_number_of_editing_lines_in_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre de l&iacute;nies d'edici&oacute;"
	" al camp de fitxa <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die Anzahl der Bearbeitungszeilen"
	" im Satzfeld <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The number of editing lines"
	" in the record field <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El n&ordm; de l&iacute;neas de edici&oacute;n"
	" del campo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le nombre de lignes d'&eacute;dition"
	" dans le champ de carte <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El n&ordm; de l&iacute;neas de edici&oacute;n"
	" del campo <strong>%s</strong> no ha cambiado.";		// Okoteve traduccin
#elif L==7	// it
	"Il numero di linee editate"
	" nel campo scheda <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Liczba wierszy edycji"
	" w polu rekordu <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O n&uacute;mero de linhas de edi&ccedil;&atilde;o"
	" no campo de cart&atilde;o <strong>%s</strong> n&atilde;o foi alterado.";
#elif L==10	// tr
	"The number of editing lines"
	" in the record field <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_number_of_questions_must_be_in_the_interval_X =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Necessita traducci
#elif L==2	// de
	"Die Anzahl der Fragen muss sich im Intervall [%u,%u] befinden.";
#elif L==3	// en
	"The number of questions must be in the interval [%u,%u].";
#elif L==4	// es
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";
#elif L==5	// fr
	"The number of questions must be in the interval [%u,%u].";		// Besoin de traduction
#elif L==6	// gn
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Okoteve traduccin
#elif L==7	// it
	"Il numero di domande dovrebbe stare nell'intervallo [%u,%u].";
#elif L==8	// pl
	"The number of questions must be in the interval [%u,%u].";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O n&uacute;mero de perguntas deve estar no intervalo [%u,%u].";
#elif L==10	// tr
	"The number of questions must be in the interval [%u,%u].";		// eviri lazim!
#endif

const char *Txt_The_number_of_rows_is_now_X =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";	// Necessita traducci
#elif L==2	// de
	"Die Anzahl der Zeilen ist jetzt <strong>%u</strong>.";
#elif L==3	// en
	"The number of rows is now <strong>%u</strong>.";
#elif L==4	// es
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";
#elif L==5	// fr
	"The number of rows is now <strong>%u</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il numero di righe &egrave; ora <strong>%u</strong>.";
#elif L==8	// pl
	"The number of rows is now <strong>%u</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O n&uacute;mero de linhas agora &eacute; <strong>%u</strong>.";
#elif L==10	// tr
	"The number of rows is now <strong>%u</strong>.";		// eviri lazim!
#endif

const char *Txt_The_numerical_code_X_already_exists =	// Warning: it is very important to include %03lu in the following sentences
#if   L==1	// ca
	"El codi num&egrave;ric ISO 3166-1 %03ld ja existeix.";
#elif L==2	// de
	"Der ISO 3166-1 Zahlencode %03ld existiert bereits.";
#elif L==3	// en
	"The ISO 3166-1 numerical code %03ld already exists.";
#elif L==4	// es
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";
#elif L==5	// fr
	"Le code num&eacute;rique %03ld ISO 3166-1 existe d&eacute;j&agrave;.";
#elif L==6	// gn
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice numerico ISO 3166-1 %03ld gi&agrave; esiste.";
#elif L==8	// pl
	"Kod numeryczny %03ld ISO 3166-1 ju&zdot; istnieje.";
#elif L==9	// pt
	"O c&oacute;digo num&eacute;rico %03ld da ISO 3166-1 j&aacute; existe.";
#elif L==10	// tr
	"The ISO 3166-1 numerical code %03ld already exists.";		// eviri lazim!
#endif

const char *Txt_The_photo_no_longer_exists =
#if   L==1	// ca
	"La foto ja no existeix.";
#elif L==2	// de
	"Das Foto nicht mehr existiert.";
#elif L==3	// en
	"The photo no longer exists.";
#elif L==4	// es
	"La foto ya no existe.";
#elif L==5	// fr
	"La photo a disparu.";
#elif L==6	// gn
	"La foto ya no existe.";	// Okoteve traduccin
#elif L==7	// it
	"La foto non esiste pi&ugrave;.";
#elif L==8	// pl
	"Zdj&eogon;cie ju&zdot; nie istnieje.";
#elif L==9	// pt
	"La foto n&atilde;o existe mais.";
#elif L==10	// tr
	"The photo no longer exists.";	// eviri lazim!
#endif

const char *Txt_The_post_no_longer_exists =
#if   L==1	// ca
	"La publicaci&oacute; ja no existeix.";
#elif L==2	// de
	"Der Post nicht mehr existiert.";
#elif L==3	// en
	"The post no longer exists.";
#elif L==4	// es
	"La publicaci&oacute;n ya no existe.";
#elif L==5	// fr
	"Le post a disparu.";
#elif L==6	// gn
	"La publicaci&oacute;n ya no existe.";	// Okoteve traduccin
#elif L==7	// it
	"La pubblicazione non esiste pi&ugrave;.";
#elif L==8	// pl
	"Publikacja ju&zdot; nie istnieje.";
#elif L==9	// pt
	"A publica&ccedil;&atilde;o n&atilde;o existe mais.";
#elif L==10	// tr
	"The post no longer exists.";		// eviri lazim!
#endif

const char *Txt_The_password_can_not_consist_only_of_digits =
#if   L==1	// ca
	"La contrasenya no pot constar"
	" nom&eacute;s de d&iacute;gits.";
#elif L==2	// de
	"Das Passwort darf nicht"
	" nur aus Ziffern bestehen.";
#elif L==3	// en
	"The password can not consist"
	" only of digits.";
#elif L==4	// es
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";
#elif L==5	// fr
	"Le mot de passe ne peut pas &ecirc;tre compos&eacute;"
	" uniquement de chiffres.";
#elif L==6	// gn
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";		// Okoteve traduccin
#elif L==7	// it
	"La password non pu&ograve; consistere"
	" solo di numeri.";
#elif L==8	// pl
	"Has&lstrok;o nie mo&zdot;e sk&lstrok;ada&cacute;"
	" si&eogon; wy&lstrok;&aogon;cznie z cyfr.";
#elif L==9	// pt
	"A senha n&atilde;o pode consistir"
	" apenas em d&iacute;gitos.";
#elif L==10	// tr
	"The password can not consist"
	" only of digits.";			// eviri lazim!
#endif

const char *Txt_The_password_can_not_contain_spaces =
#if   L==1	// ca
	"La contrasenya no pot contenir espais.";
#elif L==2	// de
	"Das Passwort darf keine Leerzeichen enthalten.";
#elif L==3	// en
	"The password can not contain spaces.";
#elif L==4	// es
	"La contrase&ntilde;a no puede tener espacios.";
#elif L==5	// fr
	"Le mot de passe ne peut pas contenir d'espaces.";
#elif L==6	// gn
	"La contrase&ntilde;a no puede tener espacios.";	// Okoteve traduccin
#elif L==7	// it
	"La password non pu&ograve; contenere spazi.";
#elif L==8	// pl
	"Has&lstrok;o nie mo&zdot;e zawiera&cacute; spacji.";
#elif L==9	// pt
	"A senha n&atilde;o pode conter espa&ccedil;os.";
#elif L==10	// tr
	"The password can not contain spaces.";			// eviri lazim!
#endif

const char *Txt_The_password_is_too_trivial_ =
#if   L==1	// ca
	"La contrasenya &eacute;s massa f&agrave;cil."
	" Feu servir una altra m&eacute;s dif&iacute;cil d'esbrinar.";
#elif L==2	// de
	"Das Passwort ist zu trivial."
	" Verwenden Sie andere schwerer zu erraten.";
#elif L==3	// en
	"The password is too trivial."
	" Use other harder to guess.";
#elif L==4	// es
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";
#elif L==5	// fr
	"Le mot de passe est trop trivial."
	" Utilisez d'autres plus difficiles &agrave; deviner.";
#elif L==6	// gn
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";	// Okoteve traduccin
#elif L==7	// it
	"La password &egrave; troppo semplice."
	" Utilizza un'altra pi&ugrave; difficile da individuare.";
#elif L==8	// pl
	"Has&lstrok;o jest zbyt trywialne."
	" U&zdot;yj innych, trudniejszych do odgadni&eogon;cia.";
#elif L==9	// pt
	"A senha &eacute; muito trivial."
	" Use outro mais difcil de adivinhar.";
#elif L==10	// tr
	"The password is too trivial."
	" Use other harder to guess.";				// eviri lazim!
#endif

const char *Txt_The_password_must_be_at_least_X_characters =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La contrasenya ha de tenir com a m&iacute;nim %u car&agrave;cters.";
#elif L==2	// de
	"Das Passwort muss mindestens %u Zeichen lang sein.";
#elif L==3	// en
	"The password must be at least %u characters.";
#elif L==4	// es
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";
#elif L==5	// fr
	"Le mot de passe doit comporter au moins %u caract&egrave;res.";
#elif L==6	// gn
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";	// Okoteve traduccin
#elif L==7	// it
	"La password dovrebbe essere di almeno %u caratteri.";
#elif L==8	// pl
	"Has&lstrok;o musi mie&cacute; co najmniej %u znak&oacute;w.";
#elif L==9	// pt
	"A senha deve ter pelo menos %u caracteres.";
#elif L==10	// tr
	"The password must be at least %u characters.";			// eviri lazim!
#endif

const char *Txt_The_password_has_been_changed_successfully =
#if   L==1	// ca
	"La contrasenya s'ha canviat correctament.";
#elif L==2	// de
	"Das Passwort wurde erfolgreich ge&auml;ndert.";
#elif L==3	// en
	"The password has been changed successfully.";
#elif L==4	// es
	"La contrase&ntilde;a se ha modificado con &eacute;xito.";
#elif L==5	// fr
	"Le mot de passe a &eacute;t&eacute; chang&eacute; avec succ&egrave;s.";
#elif L==6	// gn
	"La contrase&ntilde;a se ha modificado con &eacute;xito.";	// Okoteve traduccin
#elif L==7	// it
	"La password &egrave; stata cambiata con successo.";
#elif L==8	// pl
	"Has&lstrok;o zosta&lstrok;o pomy&sacute;lnie zmienione.";
#elif L==9	// pt
	"A senha foi alterada com sucesso.";
#elif L==10	// tr
	"The password has been changed successfully.";			// eviri lazim!
#endif

const char *Txt_The_place_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El lloc <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Standort <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"L'emplacement <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il luogo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Miejsce <strong>%s</strong> zosta&lstrok;o przemianowane na <strong>%s</strong>.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o <strong>%s</strong> foi renomeada como <strong>%s</strong>.";
#elif L==10	// tr
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_place_of_the_center_has_changed =
#if   L==1	// ca
	"El lloc del centre ha canviat.";
#elif L==2	// de
	"Der Ort des Zentrums hat sich ge&auml;ndert.";
#elif L==3	// en
	"The place of the center has changed.";
#elif L==4	// es
	"El lugar del centro ha cambiado.";
#elif L==5	// fr
	"L'emplacement du centre a chang&eacute;.";
#elif L==6	// gn
	"El lugar del centro ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il luogo del centro &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok;o si&eogon; miejsce centrum.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o do centro mudou.";
#elif L==10	// tr
	"The place of the center has changed.";	// eviri lazim!
#endif

const char *Txt_The_place_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El lloc de les vacances <strong>%s</strong> ha canviat a <strong>%s</strong>.";
#elif L==2	// de
	"Der Ort des Feiertags <strong>%s</strong> hat sich zu <strong>%s</strong> ge&auml;ndert.";
#elif L==3	// en
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"Le lieu de la f&ecirc;te <strong>%s</strong> a chang&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il luogo della festivit&agrave; <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"Miejsce &sacute;wi&eogon;ta <strong>%s</strong> zmieni&lstrok;o si&eogon; na <strong>%s</strong>.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o do feriado <strong>%s</strong> mudou para <strong>%s</strong>.";
#elif L==10	// tr
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_plugin_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El complement <strong>%s</strong> ja existeix.";
#elif L==2	// de
	"Der Plugin <strong>%s</strong> wurde bereits installiert.";
#elif L==3	// en
	"The plugin <strong>%s</strong> already exists.";
#elif L==4	// es
	"El complemento <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The plugin <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El complemento <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il plugin <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The plugin <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O plugin <strong>%s</strong> j&aacute; existe.";
#elif L==10	// tr
	"The plugin <strong>%s</strong> already exists.";	// eviri lazim!
#endif

const char *Txt_The_plugin_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El complement <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Plugin <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le plugin <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il plugin <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Wtyczka <strong>%s</strong> zosta&lstrok;a przemianowana na <strong>%s</strong>.";
#elif L==9	// pt
	"O plugin <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_project_has_been_modified =
#if   L==1	// ca
	"El projecte ha estat modificada.";
#elif L==2	// de
	"Die Projekt wurde ge&auml;ndert.";
#elif L==3	// en
	"The project has been modified.";
#elif L==4	// es
	"El proyecto ha sido modificado.";
#elif L==5	// fr
	"Le projet a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El proyecto ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"Il progetto &egrave; stato modificato.";
#elif L==8	// pl
	"Projekt zosta&lstrok; zmodyfikowany.";
#elif L==9	// pt
	"O projeto foi modificado.";
#elif L==10	// tr
	"Proje de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_properties_of_file_X_have_been_saved =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Les propietats del arxiu %s s'han desat.";
#elif L==2	// de
	"Die Eigenschaften der Datei %s wurden gespeichert.";
#elif L==3	// en
	"The properties of file %s have been saved.";
#elif L==4	// es
	"Las propiedades del archivo %s se han guardado.";
#elif L==5	// fr
	"Les propri&eacute;t&eacute;s du fichier %s ont &eacute;t&eacute; enregistr&eacute;es.";
#elif L==6	// gn
	"Las propiedades del archivo %s se han guardado.";	// Okoteve traduccin
#elif L==7	// it
	"Le propriet&agrave; del file %s sono state salvate.";
#elif L==8	// pl
	"W&lstrok;a&sacute;ciwo&sacute;ci pliku %s zosta&lstrok;y zapisane.";
#elif L==9	// pt
	"As propriedades do arquivo %s foram salvas.";
#elif L==10	// tr
	"The properties of file %s have been saved.";		// eviri lazim!
#endif

const char *Txt_The_record_field_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El camp de fitxa <strong>%s</strong> ja existeix.";
#elif L==2	// de
	"Das Datenfeld <strong>%s</strong> wurde bereits erstellt.";
#elif L==3	// en
	"The record field <strong>%s</strong> already exists.";
#elif L==4	// es
	"El campo de ficha <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The record field <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El campo de ficha <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo di scheda <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The record field <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O campo de cart&atilde;o <strong>%s</strong> j&aacute; existe.";
#elif L==10	// tr
	"The record field <strong>%s</strong> already exists.";	// eviri lazim!
#endif

const char *Txt_The_record_field_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El camp de fitxa <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Datensatzfeld <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le champ de carte <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo di scheda <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Nazwa pola rekordu <strong>%s</strong> zosta&lstrok;a zmieniona na <strong>%s</strong>.";
#elif L==9	// pt
	"O campo de cart&atilde;o <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_requested_group_changes_were_successful =
#if   L==1	// ca
	"Els canvis de grup sol&middot;licitats s'han realitzat correctament.";
#elif L==2	// de
	"Die angeforderten Gruppen&auml;nderungen waren erfolgreich.";
#elif L==3	// en
	"The requested group changes were successful.";
#elif L==4	// es
	"Los cambios de grupo solicitados se han realizado correctamente.";
#elif L==5	// fr
	"Les changements de groupe demand&eacute;es ont r&eacute;ussi.";
#elif L==6	// gn
	"Los cambios de grupo solicitados se han realizado correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"I cambiamenti di gruppi richieste hanno avuto successo.";
#elif L==8	// pl
	"&Zdot;&aogon;dane zmiany grupy powiod&lstrok;y si&eogon;.";
#elif L==9	// pt
	"As altera&ccedil;&otilde;es do grupo solicitadas foram bem sucedidas.";
#elif L==10	// tr
	"The requested group changes were successful.";				// eviri lazim!
#endif

const char *Txt_The_role_of_THE_USER_X_in_the_course_Y_has_changed_from_A_to_B =	// Warning: it is very important to include four %s in the following sentences
#if   L==1	// ca
	"El paper de <strong>%s</strong>"
	" a l'assignatura <strong>%s</strong>"
	" ha canviat de %s a %s.";
#elif L==2	// de
	"Die Rolle von <strong>%s</strong>"
	" im Kurs <strong>%s</strong>"
	" hat sich von %s nach %s ge&auml;ndert.";
#elif L==3	// en
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";
#elif L==4	// es
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";
#elif L==5	// fr
	"Le r&ocirc;le de <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong>"
	" est pass&eacute; de %s &agrave; %s.";
#elif L==6	// gn
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";		// Okoteve traduccin
#elif L==7	// it
	"Il ruolo de <strong>%s</strong>"
	" nel corso <strong>%s</strong>"
	" &egrave; cambiato da %s a %s.";
#elif L==8	// pl
	"Rola <strong>%s</strong>"
	" w kursie <strong>%s</strong>"
	" zmieni&lstrok;a si&eogon; z %s na %s.";
#elif L==9	// pt
	"O papel de <strong>%s</strong>"
	" na disciplina <strong>%s</strong>"
	" mudou de %s para %s.";
#elif L==10	// tr
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// eviri lazim!
#endif

const char *Txt_The_room_X_does_not_have_a_limited_capacity_now =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La sala <strong>%s</strong> ja no t&eacute; un aforament limitat.";
#elif L==2	// de
	"Das Raum <strong>%s</strong> hat jetzt keine begrenzte Kapazit&auml;t.";
#elif L==3	// en
	"The room <strong>%s</strong> does not have a limited capacity now.";
#elif L==4	// es
	"La sala <strong>%s</strong> ya no tiene un aforo limitado.";
#elif L==5	// fr
	"La salle <strong>%s</strong> n'a plus de capacit&eacute; limit&eacute;e.";
#elif L==6	// gn
	"La sala <strong>%s</strong> ya no tiene un aforo limitado.";		// Okoteve traduccin
#elif L==7	// it
	"L'aula <strong>%s</strong> non ha una capacit&agrave; limitata ora.";
#elif L==8	// pl
	"Klasa <strong>%s</strong> nie ma teraz ograniczonej pojemno&sacute;ci.";
#elif L==9	// pt
	"A sala <strong>%s</strong> n&atilde;o tem capacidade limitada agora.";
#elif L==10	// tr
	"The room <strong>%s</strong> does not have a limited capacity now.";	// eviri lazim!
#endif

const char *Txt_The_room_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La sala <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Raum <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The room <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"La sala <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"La salle <strong>%s</strong> a &eacute;t&eacute; renomm&eacute;e en <strong>%s</strong>.";
#elif L==6	// gn
	"La sala <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'aula <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"Klasa <strong>%s</strong> zosta&lstrok;o przemianowane na <strong>%s</strong>.";
#elif L==9	// pt
	"A sala <strong>%s</strong> foi renomeada como <strong>%s</strong>.";
#elif L==10	// tr
	"The room <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_room_assigned_to_the_group_X_has_changed =
#if   L==1	// ca
	"La sala assignada al grup <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Das der Gruppe <strong>%s</strong> zugeordnete Raum hat sich ge&auml;ndert.";
#elif L==3	// en
	"The room assigned to the group <strong>%s</strong> has changed.";
#elif L==4	// es
	"La sala asignada al grupo <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"La salle affect&eacute;e au groupe <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"La sala asignada al grupo <strong>%s</strong> ha cambiado.";		// Okoteve traduccin
#elif L==7	// it
	"L'aula assegnata al gruppo <strong>%s</strong> &egrave; cambiata.";
#elif L==8	// pl
	"Klasa przypisana do grupy <strong>%s</strong> uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"A sala atribu&iacute;da ao grupo <strong>%s</strong> mudou.";
#elif L==10	// tr
	"The room assigned to the group <strong>%s</strong> has changed.";	// eviri lazim!
#endif

const char *Txt_The_rubric_has_been_modified =
#if   L==1	// ca
	"La r&uacute;brica ha estat modificada.";
#elif L==2	// de
	"Die Rubrik wurde ge&auml;ndert.";
#elif L==3	// en
	"The rubric has been modified.";
#elif L==4	// es
	"La r&uacute;brica ha sido modificada.";
#elif L==5	// fr
	"La rubrique a &eacute;t&eacute; modifi&eacute;e.";
#elif L==6	// gn
	"O&ntilde;emoambue pe r&uacute;brica.";
#elif L==7	// it
	"La rubrica &egrave; stata modificata.";
#elif L==8	// pl
	"Rubryka zosta&lstrok;a zmieniona.";
#elif L==9	// pt
	"A rubrica foi modificada.";
#elif L==10	// tr
	"Puan anahtar&inodot;n&inodot; de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_X_students_who_belonged_to_the_course_Y_have_been_removed_from_it =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1	// ca
	"Els %u estudiants que pertanyien a l'assignatura <strong>%s</strong>"
	" n'han estat eliminats.";
#elif L==2	// de
	"Die %u Sch&uuml;ler des Kurses <strong>%s</strong>"
	" wurden daraus entfernt.";
#elif L==3	// en
	"The %u students who belonged"
	" to the course <strong>%s</strong> have been removed from it.";
#elif L==4	// es
	"Se han eliminado los %u estudiantes que hab&iacute;a"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Les %u &eacute;tudiants qui appartenaient a la mati&egrave;re <strong>%s</strong>"
	" en ont &eacute;t&eacute; retir&eacute;s.";
#elif L==6	// gn
	"Se han eliminado los %u estudiantes que hab&iacute;a"
	" en la asignatura <strong>%s</strong>.";				// Okoteve traduccin
#elif L==7	// it
	"Gli %u studenti che appartengono"
	" al corso <strong>%s</strong> sono stati rimossi da esso.";
#elif L==8	// pl
	"%u uczni&oacute;w, kt&oacute;rzy nale&zdot;eli do kursu <strong>%s</strong>,"
	" zosta&lstrok;o z niego usuni&eogon;tych.";
#elif L==9	// pt
	"Os %u estudantes que pertenceram"
	" &agrave; disciplina <strong>%s</strong> foram removidos dela.";
#elif L==10	// tr
	"The %u students who belonged"
	" to the course <strong>%s</strong> have been removed from it.";	// eviri lazim!
#endif

const char *Txt_The_search_text_must_be_longer =
#if   L==1	// ca
	"El text a cercar ha de ser m&eacute;s llarg.";
#elif L==2	// de
	"Der Suchtext muss l&auml;nger sein.";
#elif L==3	// en
	"The search text must be longer.";
#elif L==4	// es
	"El texto a buscar debe ser m&aacute;s largo.";
#elif L==5	// fr
	"Le texte de recherche doit &ecirc;tre plus longue.";
#elif L==6	// gn
	"El texto a buscar debe ser m&aacute;s largo.";	// Okoteve traduccin
#elif L==7	// it
	"Il testo di ricerca deve essere pi&ugrave; lungo.";
#elif L==8	// pl
	"Tekst wyszukiwania powinien by&cacute; d&lstrok;u&zdot;szy.";
#elif L==9	// pt
	"O texto de pesquisa deve ser mais longo.";
#elif L==10	// tr
	"The search text must be longer.";		// eviri lazim!
#endif

const char *Txt_The_session_has_been_modified =
#if   L==1	// ca
	"La sessi&oacute; ha estat modificada.";
#elif L==2	// de
	"Die Sitzung wurde ge&auml;ndert.";
#elif L==3	// en
	"The session has been modified.";
#elif L==4	// es
	"La sesi&oacute;n ha sido modificada.";
#elif L==5	// fr
	"La session a &eacute;t&eacute; modifi&eacute;e.";
#elif L==6	// gn
	"O&ntilde;emoambue pe sesi&oacute;n.";
#elif L==7	// it
	"La sessione &egrave; stata modificata.";
#elif L==8	// pl
	"Sesja zosta&lstrok;a zmieniona.";
#elif L==9	// pt
	"A sess&atilde;o foi modificada.";
#elif L==10	// tr
	"Oturum de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_session_has_expired =
#if   L==1	// ca
	"La sessi&oacute; ha expirat.";
#elif L==2	// de
	"Die Sitzung wurde.";
#elif L==3	// en
	"The session has expired.";
#elif L==4	// es
	"La sesi&oacute;n ha expirado.";
#elif L==5	// fr
	"La session a expir&eacute;.";
#elif L==6	// gn
	"La sesi&oacute;n ha expirado.";	// Okoteve traduccin
#elif L==7	// it
	"La sessione &egrave; scaduta.";
#elif L==8	// pl
	"Sesja wygas&lstrok;a.";
#elif L==9	// pt
	"A sess&atilde;o expirou.";
#elif L==10	// tr
	"Oturumun s&uuml;resi doldu.";		// eviri lazim!
#endif
/*
const char *Txt_The_set_of_questions_has_been_modified =
#if   L==1	// ca
	"El conjunt de preguntes ha estat modificat.";
#elif L==2	// de
	"Der Satz von Fragen wurde ge&auml;ndert.";
#elif L==3	// en
	"The set of questions has been modified.";
#elif L==4	// es
	"El conjunto de preguntas ha sido modificado.";
#elif L==5	// fr
	"L'ensemble de questions a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El conjunto de preguntas ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"Il set di domande &egrave; stato modificato.";
#elif L==8	// pl
	"Zestaw pyta&nacute; zosta&lstrok; zmodyfikowany.";
#elif L==9	// pt
	"O conjunto de perguntas foi modificado.";
#elif L==10	// tr
	"Soru seti de&gbreve;i&scedil;tirildi.";
#endif
*/
const char *Txt_The_size_of_the_file_exceeds_the_maximum_allowed_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La mida del fitxer"
	" supera el m&agrave;xim perm&egrave;s (%s).";
#elif L==2	// de
	"Die Gr&ouml;&szlig;e der Datei"
	" &uuml;berschreitet den zul&auml;ssigen H&ouml;chstwert (%s).";
#elif L==3	// en
	"The size of the file"
	" exceeds the maximum allowed (%s).";
#elif L==4	// es
	"El tama&ntilde;o del archivo"
	" excede el m&aacute;ximo permitido (%s).";
#elif L==5	// fr
	"La taille du fichier"
	" d&eacute;passe le maximum autoris&eacute; (%s).";
#elif L==6	// gn
	"El tama&ntilde;o del archivo"
	" excede el m&aacute;ximo permitido (%s).";	// Okoteve traduccin
#elif L==7	// it
	"La dimensione del file"
	" supera il massimo consentito (%s).";
#elif L==8	// pl
	"Rozmiar pliku"
	" przekracza maksymalne dozwolone (%s).";
#elif L==9	// pt
	"O tamanho do arquivo"
	" excede o m&aacute;ximo permitido (%s).";
#elif L==10	// tr
	"The size of the file"
	" exceeds the maximum allowed (%s).";		// eviri lazim!
#endif

const char *Txt_The_status_of_the_center_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'estat del centre <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Der Status des Zentrums <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The status of the center <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado del centro <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"Le statut du centre <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El estado del centro <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato del centro <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; status centrum <strong>%s</strong>.";
#elif L==9	// pt
	"O status do centro <strong>%s</strong> foi alterado.";
#elif L==10	// tr
	"The status of the center <strong>%s</strong> has changed.";	// eviri lazim!
#endif

const char *Txt_The_status_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'estat de la assignatura <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Der Status des Kurses <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The status of the course <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"Le statut de la mati&egrave;re <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; status kursu <strong>%s</strong>.";
#elif L==9	// pt
	"O status do curso <strong>%s</strong> foi alterado.";
#elif L==10	// tr
	"The status of the course <strong>%s</strong> has changed.";	// eviri lazim!
#endif

const char *Txt_The_status_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'estat de la titulaci&oacute; <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Der Status des Studiengangs <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The status of the degree <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"Le statut de l'&eacute;tude <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato della laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; status stopnia <strong>%s</strong>.";
#elif L==9	// pt
	"O status do grau <strong>%s</strong> foi alterado.";
#elif L==10	// tr
	"The status of the degree <strong>%s</strong> has changed.";		// eviri lazim!
#endif

const char *Txt_The_status_of_the_institution_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'estat de la instituci&oacute; <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Der Status des Institution <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The status of the institution <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"Le statut de l'institution <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato della istituzione <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; status instytucji <strong>%s</strong>.";
#elif L==9	// pt
	"O status da institui&ccedil;&atilde;o <strong>%s</strong> mudou.";
#elif L==10	// tr
	"<strong>%s</strong> kurumunun stat&uuml;s&uuml; de&gbreve;i&scedil;ti.";
#endif

const char *Txt_The_survey_has_been_modified =
#if   L==1	// ca
	"L'enquesta ha estat modificada.";
#elif L==2	// de
	"Die Umfrage wurde ge&auml;ndert.";
#elif L==3	// en
	"The survey has been modified.";
#elif L==4	// es
	"La encuesta ha sido modificada.";
#elif L==5	// fr
	"Le sondage a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"La encuesta ha sido modificada.";	// Okoteve traduccin
#elif L==7	// it
	"Il sondaggio &egrave; stato modificato.";
#elif L==8	// pl
	"Badanie zosta&lstrok;o zmodyfikowane.";
#elif L==9	// pt
	"O inqu&eacute;rito foi modificado.";
#elif L==10	// tr
	"Anket de&gbreve;i&scedil;tirildi.";
#endif

const char *Txt_The_tag_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El descriptor <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Das Tag <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le descripteur <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'etichetta <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"Nazwa tagu <strong>%s</strong> zosta&lstrok;a zmieniona na <strong>%s</strong>.";
#elif L==9	// pt
	"O descritor <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_The_tag_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El descriptor <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Das Tag <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The tag <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El descriptor <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le descripteur <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El descriptor <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'etichetta <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"Tag <strong>%s</strong> nie uleg&lstrok; zmianie.";
#elif L==9	// pt
	"O descritor <strong>%s</strong> n&atilde;o foi alterado.";
#elif L==10	// tr
	"The tag <strong>%s</strong> has not changed.";		// eviri lazim!
#endif

const char *Txt_The_test_X_has_already_been_assessed_previously =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La prova %u ja s'ha avaluat pr&egrave;viament.";
#elif L==2	// de
	"Test %u zosta&lstrok; ju&zdot; wcze&sacute;niej oceniony.";
#elif L==3	// en
	"The test %u has already been assessed previously.";
#elif L==4	// es
	"El test %u ya ha sido evaluado anteriormente.";
#elif L==5	// fr
	"Le test %u a d&eacute;j&agrave; &eacute;t&eacute; &eacute;valu&eacute; pr&eacute;c&eacute;demment.";
#elif L==6	// gn
	"El test %u ya ha sido evaluado anteriormente.";	// Okoteve traduccin
#elif L==7	// it
	"Il test %u &egrave; stato valutato precedentemente.";
#elif L==8	// pl
	"Test %u zosta&lstrok; ju&zdot; wcze&sacute;niej oceniony.";
#elif L==9	// pt
	"O teste %u j&aacute; foi avaliado anteriormente.";
#elif L==10	// tr
	"The test %u has already been assessed previously.";	// eviri lazim!
#endif

const char *Txt_The_test_configuration_has_been_updated =
#if   L==1	// ca
	"La configuraci&oacute; dels test s'ha actualitzat.";
#elif L==2	// de
	"Die Testkonfiguration wurde aktualisiert.";
#elif L==3	// en
	"The test configuration has been updated.";
#elif L==4	// es
	"La configuraci&oacute;n de los test se ha actualizado.";
#elif L==5	// fr
	"La configuration des tests a &eacute;t&eacute; mise &agrave; jour.";
#elif L==6	// gn
	"La configuraci&oacute;n de los test se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"La configurazione di test &egrave; stata aggiornata.";
#elif L==8	// pl
	"Konfiguracja testowa zosta&lstrok;a zaktualizowana.";
#elif L==9	// pt
	"A configura&ccedil;&atilde;o de teste foi atualizada.";
#elif L==10	// tr
	"The test configuration has been updated.";			// eviri lazim!
#endif

const char *Txt_The_thread_is_already_in_this_forum =
#if   L==1	// ca
	"La discussi&oacute; ja &eacute;s en aquest f&ograve;rum.";
#elif L==2	// de
	"Der Thread ist bereits in diesem Forum.";
#elif L==3	// en
	"The thread is already in this forum.";
#elif L==4	// es
	"La discusi&oacute;n ya est&aacute; en este foro.";
#elif L==5	// fr
	"Le fil est d&eacute;j&agrave; dans ce forum.";
#elif L==6	// gn
	"La discusi&oacute;n ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==7	// it
	"La discussione &egrave; gi&agrave; in questo forum.";
#elif L==8	// pl
	"W&aogon;tek jest ju&zdot; na tym forum.";
#elif L==9	// pt
	"A discuss&atilde;o j&aacute; est&aacute; neste f&oacute;rum.";
#elif L==10	// tr
	"The thread is already in this forum.";			// eviri lazim!
#endif

const char *Txt_The_thread_X_is_already_in_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La discussi&oacute; <strong>%s</strong> ja &eacute;s en aquest f&ograve;rum.";
#elif L==2	// de
	"Der Thread <strong>%s</strong> ist bereits in diesem Forum.";
#elif L==3	// en
	"The thread <strong>%s</strong> is already in this forum.";
#elif L==4	// es
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";
#elif L==5	// fr
	"Le fil <strong>%s</strong> est d&eacute;j&agrave; dans ce forum.";
#elif L==6	// gn
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==7	// it
	"La discussione <strong>%s</strong> &egrave; gi&agrave; in questo forum.";
#elif L==8	// pl
	"W&aogon;tek <strong>%s</strong> jest ju&dot; na tym forum.";
#elif L==9	// pt
	"A discuss&atilde;o <strong>%s</strong> j&aacute; est&aacute; neste f&oacute;rum.";
#elif L==10	// tr
	"The thread <strong>%s</strong> is already in this forum.";		// eviri lazim!
#endif

const char *Txt_The_timetable_is_empty =
#if   L==1	// ca
	"L'horari est&agrave; buit.";
#elif L==2	// de
	"Der Fahrplan ist leer.";
#elif L==3	// en
	"The timetable is empty.";
#elif L==4	// es
	"El horario est&aacute; vac&iacute;o.";
#elif L==5	// fr
	"L'horaire est vide.";
#elif L==6	// gn
	"El horario est&aacute; vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Il orario &egrave; vuoto.";
#elif L==8	// pl
	"Harmonogram jest pusty.";
#elif L==9	// pt
	"O hor&aacute;rio est&aacute; vazio.";
#elif L==10	// tr
	"The timetable is empty.";		// eviri lazim!
#endif

const char *Txt_The_type_of_degree_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traducci
#elif L==2	// de
	"Die Abschlussart <strong>%s</strong> wurde bereits definiert.";
#elif L==3	// en
	"The type of degree <strong>%s</strong> already exists.";
#elif L==4	// es
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The type of degree <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di laurea <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The type of degree <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grau <strong>%s</strong> j&aacute; existe.";
#elif L==10	// tr
	"The type of degree <strong>%s</strong> already exists.";	// eviri lazim!
#endif

const char *Txt_The_type_of_degree_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El tipus de titulaci&oacute; <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Die Abschlussart <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le type de dipl&ocirc;me <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di laurea <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Rodzaj stopnia <strong>%s</strong> zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"O tipo de grau <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_The_type_of_degree_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de titulaci&oacute; de la titulaci&oacute; <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Die Abschlussart des Abschlusses <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The type of degree of the degree <strong>%s</strong> has changed.";
#elif L==4	// es
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"Le type d'&eacute;tude de l'&eacute;tude <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; rodzaj stopnia <strong>%s</strong> stopnia.";
#elif L==9	// pt
	"O tipo de grau do grau <strong>%s</strong> mudou.";
#elif L==10	// tr
	"The type of degree of the degree <strong>%s</strong> has changed.";				// eviri lazim!
#endif

const char *Txt_The_type_of_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de grup <strong>%s</strong> ja existeix.";
#elif L==2	// de
	"Der Gruppen-Typ <strong>%s</strong> wurde bereits definiert.";
#elif L==3	// en
	"The type of group <strong>%s</strong> already exists.";
#elif L==4	// es
	"El tipo de grupo <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The type of group <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il tipo di gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The type of group <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grupo <strong>%s</strong> j&aacute; existe.";
#elif L==10	// tr
	"The type of group <strong>%s</strong> already exists.";	// eviri lazim!
#endif

const char *Txt_The_type_of_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El tipus de grup <strong>%s</strong> ha passat a anomenar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Typ der Gruppe <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"Le type de groupe <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di gruppo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Typ grupy <strong>%s</strong> zosta&lstrok; przemianowany na <strong>%s</strong>.";
#elif L==9	// pt
	"O tipo de grupo <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#elif L==10	// tr
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_type_of_group_of_the_group_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de grup del grup <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Der Gruppentyp der Gruppe <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The type of group of the group <strong>%s</strong> has changed.";
#elif L==4	// es
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"Le type de groupe du groupe <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";		// Okoteve traduccin
#elif L==7	// it
	"Il tipo di gruppo del gruppo <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; typ grupy grupy <strong>%s</strong>.";
#elif L==9	// pt
	"O tipo de grupo do grupo <strong>%s</strong> foi alterado.";
#elif L==10	// tr
	"The type of group of the group <strong>%s</strong> has changed.";	// eviri lazim!
#endif

const char *Txt_The_type_of_enrolment_of_the_type_of_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de matr&iacute;cula del tipus de grup <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die Einschreibungsart der Gruppe <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The type of enrolment of the type of group <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le type d'inscription du type de groupe <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di registrazione del tipo di gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Rodzaj rejestracji typu grupy <strong>%s</strong> nie uleg&lstrok; zmianie.";
#elif L==9	// pt
	"O tipo de inscri&ccedil;&atilde;o do tipo de grupo <strong>%s</strong> n&atilde;o foi alterado.";
#elif L==10	// tr
	"The type of enrolment of the type of group <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_type_of_room_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de la sala <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Der Art des Raum <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The type of room <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El tipo de la sala <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le type de la salle <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El tipo de la sala <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo della aula <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"Typ klasy <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O tipo da sala <strong>%s</strong> n&atilde;o mudou.";
#elif L==10	// tr
	"The type of room <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_type_of_room_X_is_now_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El tipus de la sala <strong>%s</strong> ara &eacute;s <strong>%s</strong>.";
#elif L==2	// de
	"Der Art des Raum <strong>%s</strong> betr&auml;gt jetzt <strong>%s</strong>.";
#elif L==3	// en
	"The type of room <strong>%s</strong> is now <strong>%s</strong>.";
#elif L==4	// es
	"El tipo de la sala <strong>%s</strong> ahora es <strong>%s</strong>.";
#elif L==5	// fr
	"Le type de la salle <strong>%s</strong> est maintenant de <strong>%s</strong>.";
#elif L==6	// gn
	"El tipo de la sala <strong>%s</strong> ahora es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo della aula <strong>%s</strong> &egrave; ora <strong>%s</strong>.";
#elif L==8	// pl
	"Typ klasy <strong>%s</strong> wynosi teraz <strong>%s</strong>.";
#elif L==9	// pt
	"O tipo da sala <strong>%s</strong> &eacute; agora <strong>%s</strong>.";
#elif L==10	// tr
	"The type of room <strong>%s</strong> is now <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_The_type_of_the_holiday_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de vacances <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Die Art des Urlaubs <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The type of the holiday <strong>%s</strong> has changed.";
#elif L==4	// es
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"Le type de f&ecirc;te <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di festivit&agrave; <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; rodzaj &sacute;wi&eogon;ta <strong>%s</strong>.";
#elif L==9	// pt
	"O tipo do feriado <strong>%s</strong> mudou.";
#elif L==10	// tr
	"The type of the holiday <strong>%s</strong> has changed.";	// eviri lazim!
#endif

const char *Txt_The_URL_X_has_been_updated =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'URL <strong>%s</strong> s'ha actualitzat.";
#elif L==2	// de
	"Die URL <strong>%s</strong> wurde aktualisiert.";
#elif L==3	// en
	"The URL <strong>%s</strong> has been updated.";
#elif L==4	// es
	"El URL <strong>%s</strong> se ha actualizado.";
#elif L==5	// fr
	"L'URL <strong>%s</strong> a &eacute;t&eacute; mise &agrave; jour.";
#elif L==6	// gn
	"El URL <strong>%s</strong> se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"L'URL <strong>%s</strong> &egrave; stata archiviata.";
#elif L==8	// pl
	"Zaktualizowano adres URL <strong>%s</strong>.";
#elif L==9	// pt
	"O URL <strong>%s</strong> foi atualizado.";
#elif L==10	// tr
	"The URL <strong>%s</strong> has been updated.";	// eviri lazim!
#endif

const char *Txt_The_user_does_not_exist =
#if   L==1	// ca
	"El usuario/a no existe.";	// Necessita traducci
#elif L==2	// de
	"Die Benutzer ist nicht vorhanden.";
#elif L==3	// en
	"The user does not exist.";
#elif L==4	// es
	"El usuario/a no existe.";
#elif L==5	// fr
	"L'utilisateur n'existe pas.";
#elif L==6	// gn
	"El usuario/a no existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il utente non esiste.";
#elif L==8	// pl
	"The user does not exist.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O utilizador n&atilde;o existe.";
#elif L==10	// tr
	"The user does not exist.";	// eviri lazim!
#endif

const char *Txt_The_user_does_not_exist_or_password_is_incorrect =
#if   L==1	// ca
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Necessita traducci
#elif L==2	// de
	"Die Benutzer ist nicht vorhanden"
	" oder Passwort falsch ist.";
#elif L==3	// en
	"The user does not exist"
	" or password is incorrect.";
#elif L==4	// es
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";
#elif L==5	// fr
	"L'utilisateur n'existe pas"
	" ou le mot de passe est incorrect.";
#elif L==6	// gn
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Okoteve traduccin
#elif L==7	// it
	"Il utente non esiste"
	" o la password non &egrave; corretta.";
#elif L==8	// pl
	"The user does not exist"
	" or password is incorrect.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O utilizador n&atilde;o existe"
	" ou a senha est&aacute; incorreta.";
#elif L==10	// tr
	"The user does not exist"
	" or password is incorrect.";			// eviri lazim!
#endif

const char *Txt_THE_USER_X_already_exists_in_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ja existeix a %s.";
#elif L==2	// de
	"<strong>%s</strong> existiert bereits in %s.";
#elif L==3	// en
	"<strong>%s</strong> already exists in %s.";
#elif L==4	// es
	"<strong>%s</strong> ya existe en %s.";
#elif L==5	// fr
	"<strong>%s</strong> existe d&eacute;j&agrave; dans %s.";
#elif L==6	// gn
	"<strong>%s</strong> ya existe en %s.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> esiste gi&agrave; in %s.";
#elif L==8	// pl
	"<strong>%s</strong> ju&zdot; istnieje w %s.";
#elif L==9	// pt
	"<strong>%s</strong> j&aacute; existe em %s.";
#elif L==10	// tr
	"<strong>%s</strong> already exists in %s.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_exists_in_Y_but_is_not_enroled_in_the_course_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> existeix a %s,"
	" per&ograve; <strong>no</strong> est&agrave; matriculat"
	" a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"<strong>%s</strong> existiert in %s,"
	" ist aber <strong>nicht</strong>"
	" im Kurs <strong>%s</strong> eingeschrieben.";
#elif L==3	// en
	"<strong>%s</strong> exists in %s,"
	" but is <strong>not</strong> enroled"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> existe en %s,"
	" pero <strong>no</strong> est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> existe en %s,"
	" mais <strong>n'est pas</strong> inscrit"
	" a la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"<strong>%s</strong> existe en %s,"
	" pero <strong>no</strong> est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> esiste in %s,"
	" ma <strong>non</strong> &egrave; registrato"
	" al corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> istnieje w %s,"
	" ale <strong>nie</strong> jest zapisany"
	" na kurs <strong>%s</strong>.";
#elif L==9	// pt
	"<strong>%s</strong> existe em %s,"
	" mas <strong>n&atilde;o</strong> est&aacute; inscrito"
	" na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> exists in %s,"
	" but is <strong>not</strong> enroled"
	" in the course <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_enroled_as_a_Y_in_the_project =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat afegit/da"
	" com a %s al projecte.";
#elif L==2	// de
	"<strong>%s</strong> hat sich"
	" als %s im Projekt eingeschrieben.";
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" as a %s in the project.";
#elif L==4	// es
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" como %s al proyecto.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; inscrit"
	" en tant que %s dans le projet.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" como %s al proyecto.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato/a"
	" como %s nel progetto.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; zapisany"
	" jako %s do projektu.";
#elif L==9	// pt
	"<strong>%s</strong> foi matriculado/a"
	" como %s no projeto.";
#elif L==10	// tr
	"<strong>%s</strong> has been enroled"
	" as a %s in the project.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_enroled_as_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido inscrito/a"
	" como administrador/a de <strong>%s</strong>.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido inscrito/a"
	" como administrador/a de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido inscrito/a"
	" como administrador/a de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato"
	" come amministratore de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi inscrito"
	" como administrador de <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_enroled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat afegit/da"
	" a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"<strong>%s</strong> hat sich"
	" im Kurs <strong>%s</strong> eingeschrieben.";
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; inscrit"
	" dans la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato/a"
	" nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; zapisany"
	"  do kursu <strong>%s</strong>.";
#elif L==9	// pt
	"<strong>%s</strong> foi matriculado/a"
	" na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> has been enroled"
	" in the course <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_enroled_in_the_group_of_type_Y_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato"
	" nel gruppo di tipo %s <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi inscrito"
	" no grupo do tipo %s <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat eliminat/da com a administrador/a.";
#elif L==2	// de
	"<strong>%s</strong> wurde als Administrator entfernt.";
#elif L==3	// en
	"<strong>%s</strong> has been removed as administrator.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; supprim&eacute; en tant qu'administrateur.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso come amministratore.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; usuni&eogon;ty jako administrator.";
#elif L==9	// pt
	"<strong>%s</strong> foi removido como administrador.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed as administrator.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a como administrador/a"
	" de <strong>%s</strong>.";		// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a como administrador/a"
	" de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a como administrador/a"
	" de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso come amministratore"
	" de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido como administrador"
	" de <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_his_her_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has been removed from all her/his courses.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed from all her/his courses.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed from all her/his courses.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso da tutti i suoi corsi.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed from all her/his courses.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido de todas as suas disciplinas.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed from all her/his courses.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_not_been_removed_from_any_group =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has not been removed"
	" from any group.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has not been removed"
	" from any group.";
#elif L==4	// es
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";
#elif L==5	// fr
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> non &egrave; stato rimosso"
	" da alcun gruppo.";
#elif L==8	// pl
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> n&atilde;o foi removido"
	" de nenhum grupo.";
#elif L==10	// tr
	"<strong>%s</strong> has not been removed"
	" from any group.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_from_Y_groups =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has been removed from %u groups.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed from %u groups.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed from %u groups.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso da %u gruppi.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed from %u groups.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido de %u grupos.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed from %u groups.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_from_one_group =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has been removed from one group.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed from one group.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed from one group.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso da un gruppo.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed from one group.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido de um grupo.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed from one group.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat eliminat"
	" de l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"<strong>%s</strong> wurde aus"
	" dem Kurs <strong>%s</strong> entfernt.";
#elif L==3	// en
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; retir&eacute;"
	" de la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; usuni&eogon;ty"
	" z trasy <strong>%s</strong>.";
#elif L==9	// pt
	"<strong>%s</strong> foi removido"
	" da disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_group_of_type_Y_to_which_it_belonged =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal gruppo di tipo %s al quale apparteneva.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido"
	" do grupo do tipo %s ao qual pertencia.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_has_been_removed_as_a_Y_from_the_project_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat eliminat"
	" como a %s del projecte <strong>%s</strong>.";
#elif L==2	// de
	"<strong>%s</strong> wurde"
	" als %s aus dem Projekt <strong>%s</strong> entfernt.";
#elif L==3	// en
	"<strong>%s</strong> has been removed"
	" as a %s from the project <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a"
	" como %s del proyecto <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; retir&eacute;"
	" en tant que %s du projet <strong>%s</strong>.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a"
	" como %s del proyecto <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso"
	" come %s dal progetto <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; wyeliminowany"
	" jako %s z projektu <strong>%s</strong>.";
#elif L==9	// pt
	"<strong>%s</strong> foi removido"
	" como %s do projeto <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> has been removed"
	" as a %s from the project <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_is_already_an_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";		// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; gi&agrave; un amministratore de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> j&aacute; &eacute; um administrador de <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_THE_USER_X_is_in_the_course_Y_but_has_not_yet_accepted_the_enrolment =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";			// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";
#elif L==4	// es
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";
#elif L==5	// fr
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";			// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; registrato/a"
	" nel corso <strong>%s</strong>,"
	" ma non ha ancora accettato la registrazione.";
#elif L==8	// pl
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> est&aacute; inscrito/a"
	" na disciplina <strong>%s</strong>,"
	" mas ainda n&atilde;o aceitou a inscri&ccedil;&atilde;o.";
#elif L==10	// tr
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";			// eviri lazim!
#endif

const char *Txt_THE_USER_X_is_already_enroled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; registrato/a"
	" nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> est&aacute; inscrito/a"
	" na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_The_user_is_new_not_yet_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'usuari &eacute;s nou, encara no existeix en %s.";
#elif L==2	// de
	"Der Benutzer ist neu, noch nicht in %s.";
#elif L==3	// en
	"The user is new, not yet in %s.";
#elif L==4	// es
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";
#elif L==5	// fr
	"L'utilisateur est nouveau, pas encore dans %s.";
#elif L==6	// gn
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";	// Okoteve traduccin
#elif L==7	// it
	"L'utente &egrave; nuovo, non esiste ancora in %s.";
#elif L==8	// pl
	"U&zdot;ytkownik jest nowa, jeszcze nie w %s.";
#elif L==9	// pt
	"O usu&aacute;rio &eacute; novo, ainda n&atilde;o existe o %s.";
#elif L==10	// tr
	"The user is new, not yet in %s.";			// eviri lazim!
#endif

const char *Txt_THE_USER_X_is_not_an_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";		// Necessita traducci
#elif L==2	// de
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> non &egrave; un amministratore de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> n&atilde;o &eacute; um administrador de <strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_THE_USER_X_is_not_found_in_the_file_of_marks =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> no es troba a l'arxiu de notes.";
#elif L==2	// de
	"<strong>%s</strong> wird in der Markendatei nicht gefunden.";
#elif L==3	// en
	"<strong>%s</strong> is not found in the file of marks.";
#elif L==4	// es
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";
#elif L==5	// fr
	"<strong>%s</strong> n'est pas trouv&eacute; dans le fichier de marques.";
#elif L==6	// gn
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> non &egrave; stato trovato nel file dei risultati.";
#elif L==8	// pl
	"<strong>%s</strong> nie znajduje si&eogon; w aktach znak&oacute;w.";
#elif L==9	// pt
	"<strong>%s</strong> n&atilde;o &eacute; encontrado no arquivo de notas.";
#elif L==10	// tr
	"<strong>%s</strong> is not found in the file of marks.";			// eviri lazim!
#endif

const char *Txt_The_visibility_of_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La visibilitat del camp <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die Sichtbarkeit des Datensatzfeldes <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The visibility of the record field <strong>%s</strong> has not changed.";
#elif L==4	// es
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"La visibilit&eacute; du champ de carte <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";			// Okoteve traduccin
#elif L==7	// it
	"La visibilit&agrave; del campo di scheda <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"Widoczno&sacute;&cacute; pola rekordu <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"A visibilidade do campo de cart&atilde;o <strong>%s</strong> n&atilde;o foi alterada.";
#elif L==10	// tr
	"The visibility of the record field <strong>%s</strong> has not changed.";	// eviri lazim!
#endif

const char *Txt_The_whole_course =
#if   L==1	// ca
	"Toda la asignatura";		// Necessita traducci
#elif L==2	// de
	"Die gesamte Kurs";
#elif L==3	// en
	"The whole course";
#elif L==4	// es
	"Toda la asignatura";
#elif L==5	// fr
	"L'ensemble de la mati&egrave;re";
#elif L==6	// gn
	"Toda la asignatura";		// Okoteve traduccin
#elif L==7	// it
	"L'intero corso";
#elif L==8	// pl
	"The whole course";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Toda a disciplina";
#elif L==10	// tr
	"The whole course";		// eviri lazim!
#endif

const char *Txt_The_year_X_is_not_allowed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"L'any/semestre <strong>%u</strong> no est&agrave; perm&egrave;s.";
#elif L==2	// de
	"Das Jahr/Semester <strong>%u</strong> ist nicht erlaubt.";
#elif L==3	// en
	"The year/semester <strong>%u</strong> is not allowed.";
#elif L==4	// es
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";
#elif L==5	// fr
	"L'ann&eacute;e/semestre <strong>%u</strong> n'est pas autoris&eacute;.";
#elif L==6	// gn
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";	// Okoteve traduccin
#elif L==7	// it
	"L'anno/semester <strong>%u</strong> non &egrave; consentito.";
#elif L==8	// pl
	"Rok/semestr <strong>%u</strong> jest niedozwolony.";
#elif L==9	// pt
	"O ano/semestre <strong>%u</strong> n&atilde;o &eacute; permitido.";
#elif L==10	// tr
	"The year/semester <strong>%u</strong> is not allowed.";		// eviri lazim!
#endif

const char *Txt_The_year_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'any/semestre de l'assignatura <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Das Jahr/Semester des Kurses <strong>%s</strong> hat sich ge&auml;ndert.";
#elif L==3	// en
	"The year/semester of the couse <strong>%s</strong> has changed.";
#elif L==4	// es
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"L'anne/le semestre de la mati&egrave;re <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'anno/semestre del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"Zmieni&lstrok; si&eogon; rok/semestr przypadku <strong>%s</strong>.";
#elif L==9	// pt
	"O ano/semestre da disciplina <strong>%s</strong> mudou.";
#elif L==10	// tr
	"The year/semester of the couse <strong>%s</strong> has changed.";		// eviri lazim!
#endif

const char *Txt_The_ZIP_file_has_been_received_successfully =
#if   L==1	// ca
	"L'arxiu <em>ZIP</em> s'ha rebut correctament.";
#elif L==2	// de
	"Die <em>ZIP</em>-Datei wurde erfolgreich empfangen.";
#elif L==3	// en
	"The <em>ZIP</em> file has been received successfully.";
#elif L==4	// es
	"El archivo <em>ZIP</em> se ha recibido correctamente.";
#elif L==5	// fr
	"Le fichier <em>ZIP</em> a bien &eacute;t&eacute; re&ccedil;u.";
#elif L==6	// gn
	"El archivo <em>ZIP</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il <em>ZIP</em> file &egrave; stato ricevuto con successo.";
#elif L==8	// pl
	"Plik <em>ZIP</em> zosta&lstrok; pomy&sacute;lnie odebrany.";
#elif L==9	// pt
	"O arquivo <em>ZIP</em> foi recebido com sucesso.";
#elif L==10	// tr
	"The <em>ZIP</em> file has been received successfully.";	// eviri lazim!
#endif

const char *Txt_The_ZIP_file_has_been_unzipped_successfully =
#if   L==1	// ca
	"L'arxiu <em>ZIP</em> s'ha descomprimit correctament.";
#elif L==2	// de
	"Die <em>ZIP</em>-Datei wurde erfolgreich entpackt.";
#elif L==3	// en
	"The <em>ZIP</em> file has been unzipped successfully.";
#elif L==4	// es
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";
#elif L==5	// fr
	"Le fichier <em>ZIP</em> a &eacute;t&eacute; d&eacute;compress&eacute; avec succ&egrave;s.";
#elif L==6	// gn
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il <em>ZIP</em> file &egrave; stato decompresso con successo.";
#elif L==8	// pl
	"Plik <em>ZIP</em> zosta&lstrok; rozpakowany pomy&sacute;lnie.";
#elif L==9	// pt
	"O arquivo <em>ZIP</em> foi descompactado com sucesso.";
#elif L==10	// tr
	"The <em>ZIP</em> file has been unzipped successfully.";	// eviri lazim!
#endif

const char *Txt_Theme_SKIN =
#if   L==1	// ca
	"Tema";
#elif L==2	// de
	"Skin";
#elif L==3	// en
	"Theme";
#elif L==4	// es
	"Tema";
#elif L==5	// fr
	"Th&egrave;me";
#elif L==6	// gn
	"Tema";	// Okoteve traduccin
#elif L==7	// it
	"Tema";
#elif L==8	// pl
	"Temat";
#elif L==9	// pt
	"Skin";
#elif L==10	// tr
	"Tema";
#endif

const char *Txt_There_are_more_than_one_table_in_the_file_of_marks =
#if   L==1	// ca
	"Hi ha m&eacute;s d'una taula a l'arxiu de notes.";
#elif L==2	// de
	"Es gibt mehr als eine Tabelle in der Markendatei.";
#elif L==3	// en
	"There are more than one table in the file of marks.";
#elif L==4	// es
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";
#elif L==5	// fr
	"Il y a plus d'un tableau dans le fichier des notes.";
#elif L==6	// gn
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"C'&egrave; pi&ugrave; di una tabella nel file dei risultati.";
#elif L==8	// pl
	"W teczce znak&oacute;w znajduje si&eogon; wi&eogon;cej ni&zdot; jedna tabela.";
#elif L==9	// pt
	"Existem mais de uma tabela no arquivo de notas.";
#elif L==10	// tr
	"There are more than one table in the file of marks.";		// eviri lazim!
#endif

const char *Txt_There_are_more_than_one_user_with_the_ID_X_Please_type_a_nick_or_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Hi ha m&eacute;s d'un usuari amb l'ID <strong>%s</strong>."
	" Escriviu un @nick o correu electr&ograve;nic.";
#elif L==2	// de
	"Es gibt mehr als einen Benutzer mit der ID <strong>%s</strong>."
	" Bitte geben Sie einen @nick oder eine E-Mail ein.";
#elif L==3	// en
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";
#elif L==4	// es
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";
#elif L==5	// fr
	"Il y a plus d'un utilisateur avec l'ID <strong>%s</strong>."
	" Veuillez saisir un @nick ou un e-mail.";
#elif L==6	// gn
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";	// Okoteve traduccin
#elif L==7	// it
	"Ci sono pi&ugrave; di un utente con l'ID <strong>%s</strong>."
	" Si prega di digitare un @nome-utente o email.";
#elif L==8	// pl
	"Istnieje wi&eogon;cej ni&zdot; jeden u&zdot;ytkownik o ID <strong>%s</strong>."
	" Wpisz @nick lub e-mail.";
#elif L==9	// pt
	"H&aacute; mais de um usu&aacute;rio com o ID <strong>%s</strong>."
	" Por favor, digite uma @alcunha ou um email.";
#elif L==10	// tr
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// eviri lazim!
#endif

const char *Txt_There_are_new_posts =
#if   L==1	// ca
	"Hi ha missatges nous";
#elif L==2	// de
	"Es gibt neue Nachrichten";
#elif L==3	// en
	"There are new posts";
#elif L==4	// es
	"Hay mensajes nuevos";
#elif L==5	// fr
	"Il y a des nouveaux messages";
#elif L==6	// gn
	"Hay mensajes nuevos";	// Okoteve traduccin
#elif L==7	// it
	"Ci sono nuovi post";
#elif L==8	// pl
	"Istnieje nowych post&oacute;w";
#elif L==9	// pt
	"H&aacute; novas posts";
#elif L==10	// tr
	"There are new posts";	// eviri lazim!
#endif

const char *Txt_There_are_no_accesses_with_the_selected_search_criteria =
#if   L==1	// ca
	"No hi ha accessos amb els criteris de cerca seleccionats.";
#elif L==2	// de
	"Es gibt keine Zugriffe mit den gew&auml;hlten Suchkriterien.";
#elif L==3	// en
	"There are no accesses with the selected search criteria.";
#elif L==4	// es
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";
#elif L==5	// fr
	"Il n'y a pas d'acc&egrave;s avec les crit&egrave;res de recherche s&eacute;lectionn&eacute;s.";
#elif L==6	// gn
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono accessi con il criterio di ricerca selezionato.";
#elif L==8	// pl
	"Brak dost&eogon;pu z wybranymi kryteriami wyszukiwania.";
#elif L==9	// pt
	"N&atilde;o h&aacute; acessos com os crit&eacute;rios de pesquisa selecionados.";
#elif L==10	// tr
	"There are no accesses with the selected search criteria.";		// eviri lazim!
#endif

const char *Txt_There_are_no_centers_with_requests_for_degrees_to_be_confirmed =
#if   L==1	// ca
	"No hi ha centers"
	" amb sol&middot;licituds de titulacions"
	" pendents de confirmar.";
#elif L==2	// de
	"Es gibt keine Zentren"
	" mit Antr&auml;gen auf Best&auml;tigung"
	" von Abschl&uuml;ssen.";
#elif L==3	// en
	"There are no centers"
	" with requests for degrees"
	" to be confirmed.";
#elif L==4	// es
	"No hay centros"
	" con solicitudes de titulaciones"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas de centers"
	" avec demandes d'&eacute;tudes"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay centros"
	" con solicitudes de titulaciones"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono centri"
	" con richieste di gradi"
	" pendenti de confermare.";
#elif L==8	// pl
	"Nie ma o&sacute;rodk&oacute;w"
	" z pro&sacute;bami o potwierdzenie"
	" stopni naukowych.";
#elif L==9	// pt
	"N&atilde;o h&aacute; centros"
	" com solicita&ccedil;&otilde;es de graus"
	" pendentes de confirma&ccedil;&atilde;o.";
#elif L==10	// tr
	"There are no centers"
	" with requests for degrees"
	" to be confirmed.";		// eviri lazim!
#endif

const char *Txt_There_are_no_countries_with_requests_for_institutions_to_be_confirmed =
#if   L==1	// ca
	"No hi ha pa&iuml;sos"
	" amb sol&middot;licituds de institucions"
	" pendents de confirmar.";
#elif L==2	// de
	"W &zdot;adnym kraju"
	" nie ma pr&oacute;&sacute;b"
	" o potwierdzenie instytucji.";
#elif L==3	// en
	"There are no countries"
	" with requests for institutions"
	" to be confirmed.";
#elif L==4	// es
	"No hay pa&iacute;ses"
	" con solicitudes de instituciones"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas de pays"
	" avec demandes d'&eacute;tablissements"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay pa&iacute;ses"
	" con solicitudes de instituciones"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono paesi"
	" con richieste di istituzioni"
	" pendenti de confermare.";
#elif L==8	// pl
	"W &zdot;adnym kraju"
	" nie ma pr&oacute;&sacute;b"
	" o potwierdzenie instytucji.";
#elif L==9	// pt
	"N&atilde;o h&aacute; pa&iacute;ses"
	" com solicita&ccedil;&otilde;es de institui&ccedil;&otilde;es"
	" pendentes de confirma&ccedil;&atilde;o.";
#elif L==10	// tr
	"There are no countries"
	" with requests for institutions"
	" to be confirmed.";		// eviri lazim!
#endif

const char *Txt_There_are_no_degrees_with_requests_for_courses_to_be_confirmed =
#if   L==1	// ca
	"No hi ha titulacions"
	" amb sol&middot;licituds d'assignatures"
	" pendents de confirmar.";
#elif L==2	// de
	"Es liegen keine Abschl&uuml;sse"
	" mit Antr&auml;gen auf Best&auml;tigung"
	" von Lehrveranstaltungen vor.";
#elif L==3	// en
	"There are no degrees"
	" with requests for courses"
	" to be confirmed.";
#elif L==4	// es
	"No hay titulaciones"
	" con solicitudes de asignaturas"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;tudes"
	" avec demandes de cours"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay titulaciones"
	" con solicitudes de asignaturas"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono gradi"
	" con richieste di corsi"
	" pendenti de confermare.";
#elif L==8	// pl
	"Nie ma stopni z pro&sacute;bami"
	" o potwierdzenie kurs&oacute;w.";
#elif L==9	// pt
	"N&atilde;o h&aacute; graus"
	" com solicita&ccedil;&otilde;es de disciplinas"
	" pendentes de confirma&ccedil;&atilde;o.";
#elif L==10	// tr
	"There are no degrees"
	" with requests for courses"
	" to be confirmed.";		// eviri lazim!
#endif

const char *Txt_There_are_no_institutions_with_requests_for_centers_to_be_confirmed =
#if   L==1	// ca
	"No hi ha institucions"
	" amb sol&middot;licituds de centers"
	" pendents de confirmar.";
#elif L==2	// de
	"Anfragen f&uuml;r"
	" zu best&auml;tigende Zentren.";
#elif L==3	// en
	"There are no institutions"
	" with requests for centers"
	" to be confirmed.";
#elif L==4	// es
	"No hay instituciones"
	" con solicitudes de centros"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;tablissements"
	" avec demandes de centers"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay instituciones"
	" con solicitudes de centros"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono istituzioni"
	" con richieste di centri"
	" pendenti de confermare.";
#elif L==8	// pl
	"Nie ma instytucji"
	" z pro&sacute;bami"
	" o potwierdzenie o&sacute;rodk&oacute;w.";
#elif L==9	// pt
	"N&atilde;o h&aacute; institui&ccedil;&otilde;es"
	" com solicita&ccedil;&otilde;es de centros"
	" pendentes de confirma&ccedil;&atilde;o.";
#elif L==10	// tr
	"There are no institutions"
	" with requests for centers"
	" to be confirmed.";		// eviri lazim!
#endif

const char *Txt_There_are_no_record_fields_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No hi ha camps de fitxes a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt keine Datensatzfelder im Kurs <strong>%s</strong>.";
#elif L==3	// en
	"There are no record fields in the course <strong>%s</strong>.";
#elif L==4	// es
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Il n'y a pas de champs de cartes dans la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono campi di schede nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"W kursie <strong>%s</strong> nie ma p&oacute;l rekord&oacute;w.";
#elif L==9	// pt
	"N&atilde;o h&atilde; campos de cart&atilde;o na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"There are no record fields in the course <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_There_are_no_types_of_group_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No hi ha cap tipus de grup a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"Im Kurs <strong>%s</strong> gibt es keine Gruppentypen.";
#elif L==3	// en
	"There are no types of group in the course <strong>%s</strong>.";
#elif L==4	// es
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Il n'y a aucun type de groupe dans la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono tipi di gruppi nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"W kursie <strong>%s</strong> nie ma typ&oacute;w grup.";
#elif L==9	// pt
	"N&atilde;o h&aacute; tipos de grupo na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"There are no types of group in the course <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_There_are_X_users_with_the_ID_Y =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1	// ca
	"Hi ha %u usuaris amb el ID (DNI/c&egrave;dula) <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt %u Benutzer mit der ID <strong>%s</strong>.";
#elif L==3	// en
	"There are %u users with the ID <strong>%s</strong>.";
#elif L==4	// es
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";
#elif L==5	// fr
	"Il y a %u utilisateurs avec le ID <strong>%s</strong>.";
#elif L==6	// gn
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Ci sono %u utenti con il ID <strong>%s</strong>.";
#elif L==8	// pl
	"Jest %u u&zdot;ytkownik&oacute;w o ID <strong>%s</strong>.";
#elif L==9	// pt
	"Existem %u usu&aacute;rios com o ID <strong>%s</strong>.";
#elif L==10	// tr
	"There are %u users with the ID <strong>%s</strong>.";			// eviri lazim!
#endif

const char *Txt_There_has_been_no_change_in_groups =
#if   L==1	// ca
	"No hi ha hagut cap canvi en els grups.";
#elif L==2	// de
	"An den Gruppen hat sich nichts ge&auml;ndert";
#elif L==3	// en
	"There has been no change in groups.";
#elif L==4	// es
	"No se ha realizado ning&uacute;n cambio de grupos.";
#elif L==5	// fr
	"Il n'y a pas eu de changement dans les groupes.";
#elif L==6	// gn
	"No se ha realizado ning&uacute;n cambio de grupos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono stati cambiamenti in gruppi.";
#elif L==8	// pl
	"Nie by&lstrok;o zmian w grupach.";
#elif L==9	// pt
	"Nenhuma altera&ccedil;&atilde;o de grupo foi feita.";
#elif L==10	// tr
	"There has been no change in groups.";			// eviri lazim!
#endif

const char *Txt_There_have_been_X_errors_in_sending_the_message =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han produ&iuml;t %u errors en enviar el missatge.";
#elif L==2	// de
	"Beim Senden der Nachricht sind %u Fehler aufgetreten.";
#elif L==3	// en
	"There have been %u errors in sending the message.";
#elif L==4	// es
	"Ha habido %u errores enviando el mensaje.";
#elif L==5	// fr
	"Il y a eu %u erreurs dans l'envoi du message.";
#elif L==6	// gn
	"Ha habido %u errores enviando el mensaje.";		// Okoteve traduccin
#elif L==7	// it
	"Ci sono stati %u errori inviando il messaggio.";
#elif L==8	// pl
	"Wyst&aogon;pi&lstrok;o %u b&lstrok;&eogon;d&oacute;w podczas wysy&lstrok;ania wiadomo&sacute;ci.";
#elif L==9	// pt
	"Houve %u erros ao enviar a mensagem.";
#elif L==10	// tr
	"There have been %u errors in sending the message.";	// eviri lazim!
#endif

const char *Txt_There_is_already_a_file_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja hi ha un arxiu amb el nom <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits eine Datei mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"There is already a file named <strong>%s</strong>.";
#elif L==4	// es
	"Ya existe un archivo con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un fichier nomm&eacute; <strong>%s</strong>.";
#elif L==6	// gn
	"O&itilde;ma pete&itilde; vore h&eacute;rava <strong>%s</strong>.";
#elif L==7	// it
	"C'&egrave; gi&agrave; un file chiamato <strong>%s</strong>.";
#elif L==8	// pl
	"Istnieje ju&zdot; plik o nazwie <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um arquivo chamado <strong>%s</strong>.";
#elif L==10	// tr
	"Zaten <strong>%s</strong> ad&inodot;nda bir dosya var.";
#endif

const char *Txt_There_is_already_a_non_empty_folder_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja hi ha una carpeta no buida amb el nom <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen nicht leeren Ordner mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"There is already a non-empty folder named <strong>%s</strong>.";
#elif L==4	// es
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un dossier non vide nomm&eacute; <strong>%s</strong>.";
#elif L==6	// gn
	"O&itilde;ma pete&itilde; kundaha nandi'&ytilde;va h&eacute;rava <strong>%s</strong>.";
#elif L==7	// it
	"C'&egrave; gi&agrave; una cartella non vuota chiamata <strong>%s</strong>.";
#elif L==8	// pl
	"Istnieje ju&zdot; niepusty folder o nazwie <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe uma pasta n&atilde;o vazia chamada strong>%s</strong>.";
#elif L==10	// tr
	"<strong>%s</strong> ad&inodot;nda bo&scedil; olmayan bir klas&ouml;r zaten var.";
#endif

const char *Txt_There_is_no_empty_account_associated_with_your_ID_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No hi ha cap compte buida (encara no usada)"
	" associada al seu ID <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt keine leeren Benutzerkonto (noch nicht verwendet)"
	" im Zusammenhang mit Ihrer ID <strong>%s</strong>.";
#elif L==3	// en
	"There is no empty (not yet used) account"
	" associated with your ID <strong>%s</strong>.";
#elif L==4	// es
	"No existe ninguna cuenta vac&iacute;a (a&uacute;n no usada)"
	" asociada a su ID <strong>%s</strong>.";
#elif L==5	// fr
	"Il n'y a pas de compte vide (pas encore utilis&eacute;)"
	" associ&eacute; &agrave; votre ID <strong>%s</strong>.";
#elif L==6	// gn
	"No existe ninguna cuenta vac&iacute;a (a&uacute;n no usada)"
	" asociada a su ID <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non vi &egrave; alcun conto vuoto (non ancora utilizzato)"
	" associato con il tuo ID <strong>%s</strong>.";
#elif L==8	// pl
	"Nie ma pusta konta (jeszcze nie u&zdot;ywany)"
	" powi&aogon;zany z ID <strong>%s</strong>.";
#elif L==9	// pt
	"N&atilde;o existe nenhuma conta vazio (ainda n&atilde;o utilizado)"
	" associado ao seu ID <strong>%s</strong>.";
#elif L==10	// tr
	"Kimli&gbreve;iniz <strong>%s</strong> ile ili&scedil;kili bo&scedil;"
	" (hen&uuml;z kullan&inodot;lmam&inodot;&scedil;) hesap yok.";
#endif

const char *Txt_There_is_no_knowing_how_many_users_not_logged_have_accessed =
#if   L==1	// ca
	"No se puede saber cu&aacute;ntos usuarios distintos"
	" han accedido sin identificarse.";	// Necessita traducci
#elif L==2	// de
	"There is no knowing how many users"
	" not logged have accessed.";		// bersetzung notwendig
#elif L==3	// en
	"There is no knowing how many users"
	" not logged have accessed.";
#elif L==4	// es
	"No se puede saber cu&aacute;ntos usuarios distintos"
	" han accedido sin identificarse.";
#elif L==5	// fr
	"There is no knowing how many users"
	" not logged have accessed.";		// Besoin de traduction
#elif L==6	// gn
	"No se puede saber cu&aacute;ntos usuarios distintos"
	" han accedido sin identificarse.";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; sapere quanti utenti distinti"
	" hanno avuto accesso senza identificarsi.";
#elif L==8	// pl
	"There is no knowing how many users"
	" not logged have accessed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; como saber quantos usu&aacute;rios distintos"
	" n&atilde;o logados acessaram.";
#elif L==10	// tr
	"There is no knowing how many users"
	" not logged have accessed.";		// eviri lazim!
#endif

const char *Txt_There_is_no_user_with_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Necessita traducci
#elif L==2	// de
	"There is no user"
	" with email <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3	// en
	"There is no user"
	" with email <strong>%s</strong>.";
#elif L==4	// es
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";
#elif L==5	// fr
	"There is no user"
	" with email <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non c'&egrave; alcun utente"
	" con email <strong>%s</strong>.";
#elif L==8	// pl
	"There is no user"
	" with email <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; usu&aacute;rio"
	" com email <strong>%s</strong>.";
#elif L==10	// tr
	"There is no user"
	" with email <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_There_is_no_user_with_ID_nick_or_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Necessita traducci
#elif L==2	// de
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";				// bersetzung notwendig
#elif L==3	// en
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";
#elif L==4	// es
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";
#elif L==5	// fr
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";				// Besoin de traduction
#elif L==6	// gn
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non c'&egrave; alcun utente"
	" con ID, nome utente o email <strong>%s</strong>.";
#elif L==8	// pl
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; usu&aacute;rio"
	" com ID, alcunha ou email <strong>%s</strong>.";
#elif L==10	// tr
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";				// eviri lazim!
#endif

const char *Txt_There_is_no_user_with_nickname_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";		// Necessita traducci
#elif L==2	// de
	"There is no user"
	" with nickname <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"There is no user"
	" with nickname <strong>%s</strong>.";
#elif L==4	// es
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";
#elif L==5	// fr
	"There is no user"
	" with nickname <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non c'&egrave; alcun utente"
	" con nome utente <strong>%s</strong>.";
#elif L==8	// pl
	"There is no user"
	" with nickname <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; usu&aacute;rio"
	" com alcunha <strong>%s</strong>.";
#elif L==10	// tr
	"There is no user"
	" with nickname <strong>%s</strong>.";	// eviri lazim!
#endif

const char *Txt_There_was_a_problem_sending_an_email_automatically =
#if   L==1	// ca
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";	// Necessita traducci
#elif L==2	// de
	"There was a problem sending an email automatically.";			// bersetzung notwendig
#elif L==3	// en
	"There was a problem sending an email automatically.";
#elif L==4	// es
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";
#elif L==5	// fr
	"There was a problem sending an email automatically.";			// Besoin de traduction
#elif L==6	// gn
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";	// Okoteve traduccin
#elif L==7	// it
	"C'&egrave; stato un problema inviando automaticamente un'email.";
#elif L==8	// pl
	"There was a problem sending an email automatically.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ocorreu um problema ao enviar um email automaticamente.";
#elif L==10	// tr
	"There was a problem sending an email automatically.";			// eviri lazim!
#endif

const char *Txt_This_exam_has_no_questions =
#if   L==1	// ca
	"Aquest examen no t&eacute; preguntes.";
#elif L==2	// de
	"Dieses Pr&uuml;fung hat keine Fragen.";
#elif L==3	// en
	"This exam has no questions.";
#elif L==4	// es
	"Este examen no tiene preguntas.";
#elif L==5	// fr
	"Cet examen n'a pas de questions.";
#elif L==6	// gn
	"Este examen no tiene preguntas.";	// Okoteve traduccin
#elif L==7	// it
	"Questo esame non ha domande.";
#elif L==8	// pl
	"Ten egzamin nie ma pytania.";
#elif L==9	// pt
	"Este exame n&atilde;o tem perguntas.";
#elif L==10	// tr
	"This exam has no questions.";		// eviri lazim!
#endif

const char *Txt_This_link_will_remain_active_as_long_as_your_user_s_account_exists =
#if   L==1	// ca
	"Aquest enlla&ccedil; romandr&agrave; actiu mentre existeixi el seu compte de l'usuari.";
#elif L==2	// de
	"Diese Verbindung bleibt aktiv, solange das Konto Ihres Benutzers vorhanden.";
#elif L==3	// en
	"This link will remain active as long as your user's account exists.";
#elif L==4	// es
	"Este enlace permanecer&aacute; activo mientras exista su cuenta del usuario.";
#elif L==5	// fr
	"Ce lien restera actif aussi longtemps que votre compte de l'utilisateur existe.";
#elif L==6	// gn
	"Este enlace permanecer&aacute; activo mientras exista su cuenta del usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Questo link rimarr&agrave; attiva finch&eacute; esiste il tuo account utente.";
#elif L==8	// pl
	"Ten link pozostanie aktywna tak d&lstrok;ugo, jak istnieje konto u&zdot;ytkownika.";
#elif L==9	// pt
	"Este link permanecer&aacute; ativo enquanto existir a sua conta do usu&aacute;rio.";
#elif L==10	// tr
	"This link will remain active as long as your user's account exists.";		// eviri lazim!
#endif

const char *Txt_This_post_has_been_banned_probably_for_not_satisfy_the_rules_of_the_forums =
#if   L==1	// ca
	"Aquesta publicaci&oacute; ha estat prohibida,"
	" probablement per no satisfer les regles dels f&ograve;rums.";
#elif L==2	// de
	"Dieser Beitrag wurde gesperrt,"
	" wahrscheinlich weil er nicht den Regeln des Forums entspricht.";
#elif L==3	// en
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";
#elif L==4	// es
	"Este comentario ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";
#elif L==5	// fr
	"Ce post a &eacute;t&eacute; banni,"
	" probablement pour ne pas respecter les r&egrave;gles des forums.";
#elif L==6	// gn
	"Este comentario ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";		// Okoteve traduccin
#elif L==7	// it
	"Questo post &egrave; stato censurato,"
	" probabilmente perch&egrave; non soddisfa le regole del forum.";
#elif L==8	// pl
	"Ten post zosta&lstrok; zbanowany,"
	" prawdopodobnie z powodu niespe&lstrok;nienia regulaminu forum.";
#elif L==9	// pt
	"Este post foi censurado,"
	" provavelmente por n&atilde;o satisfazer as regras dos f&oacute;runs.";
#elif L==10	// tr
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// eviri lazim!
#endif

const char *Txt_This_survey_has_no_questions =
#if   L==1	// ca
	"Aquesta enquesta no t&eacute; preguntes.";
#elif L==2	// de
	"Diese Umfrage hat keine Fragen.";
#elif L==3	// en
	"This survey has no questions.";
#elif L==4	// es
	"Esta encuesta no tiene preguntas.";
#elif L==5	// fr
	"Ce sondage n'a pas de questions.";
#elif L==6	// gn
	"Esta encuesta no tiene preguntas.";	// Okoteve traduccin
#elif L==7	// it
	"Questo sondaggio non ha domande.";
#elif L==8	// pl
	"Badanie to nie ma pytania.";
#elif L==9	// pt
	"Este inqu&eacute;rito n&atilde;o tem perguntas.";
#elif L==10	// tr
	"This survey has no questions.";	// eviri lazim!
#endif

const char *Txt_Thread =
#if   L==1	// ca
	"Discussi&oacute;";
#elif L==2	// de
	"Thread";
#elif L==3	// en
	"Thread";
#elif L==4	// es
	"Discusi&oacute;n";
#elif L==5	// fr
	"Fil";
#elif L==6	// gn
	"Discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Discussione";
#elif L==8	// pl
	"W&aogon;tek";
#elif L==9	// pt
	"Discuss&atilde;o";
#elif L==10	// tr
	"Thread";		// eviri lazim!
#endif

const char *Txt_thread =
#if   L==1	// ca
	"discussi&oacute;";
#elif L==2	// de
	"Thread";
#elif L==3	// en
	"thread";
#elif L==4	// es
	"discusi&oacute;n";
#elif L==5	// fr
	"fil";
#elif L==6	// gn
	"discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"discussione";
#elif L==8	// pl
	"w&aogon;tek";
#elif L==9	// pt
	"discuss&atilde;o";
#elif L==10	// tr
	"thread";		// eviri lazim!
#endif

const char *Txt_Thread_marked_to_be_moved =
#if   L==1	// ca
	"Discussi&oacute; marcada per moure-la.";
#elif L==2	// de
	"Thread zum Verschieben markiert.";
#elif L==3	// en
	"Thread marked to be moved.";
#elif L==4	// es
	"Discusi&oacute;n marcada para moverla.";
#elif L==5	// fr
	"Fil marqu&eacute; pour &ecirc;tre d&eacute;plac&eacute;.";
#elif L==6	// gn
	"Discusi&oacute;n marcada para moverla.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione segnata per spostarla.";
#elif L==8	// pl
	"W&aogon;tek oznaczony do przeniesienia.";
#elif L==9	// pt
	"Discuss&atilde;o marcada para ser movida.";
#elif L==10	// tr
	"Thread marked to be moved.";			// eviri lazim!
#endif

const char *Txt_Thread_X_marked_to_be_moved =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discussi&oacute; <strong>%s</strong> marcada per moure-la.";
#elif L==2	// de
	"Thread <strong>%s</strong> zum Verschieben markiert.";
#elif L==3	// en
	"Thread <strong>%s</strong> marked to be moved.";
#elif L==4	// es
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";
#elif L==5	// fr
	"Fil <strong>%s</strong> marqu&eacute; pour &ecirc;tre d&eacute;plac&eacute;.";
#elif L==6	// gn
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione <strong>%s</strong> segnata per spostarla.";
#elif L==8	// pl
	"W&aogon;tek <strong>%s</strong> oznaczony do przeniesienia.";
#elif L==9	// pt
	"Discuss&atilde;o <strong>%s</strong> marcada para ser movida.";
#elif L==10	// tr
	"Thread <strong>%s</strong> marked to be moved.";		// eviri lazim!
#endif

const char *Txt_Thread_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discussi&oacute; moguda a aquest f&ograve;rum.";
#elif L==2	// de
	"Thread wurde in dieses Forum verschoben.";
#elif L==3	// en
	"Thread moved to this forum.";
#elif L==4	// es
	"Discusi&oacute;n movida a este foro.";
#elif L==5	// fr
	"Fil d&eacute;plac&eacute; dans ce forum.";
#elif L==6	// gn
	"Discusi&oacute;n movida a este foro.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione spostata in questo forum.";
#elif L==8	// pl
	"W&aogon;tek zosta&lstrok; przeniesiony na to forum.";
#elif L==9	// pt
	"Discuss&atilde;o movida para este f&oacute;rum.";
#elif L==10	// tr
	"Thread moved to this forum.";		// eviri lazim!
#endif

const char *Txt_Thread_X_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discussi&oacute; <strong>%s</strong> moguda a aquest f&ograve;rum.";
#elif L==2	// de
	"Thread <strong>%s</strong> wurde in dieses Forum verschoben.";
#elif L==3	// en
	"Thread <strong>%s</strong> moved to this forum.";
#elif L==4	// es
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";
#elif L==5	// fr
	"Fil <strong>%s</strong> d&eacute;plac&eacute; dans ce forum.";
#elif L==6	// gn
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione <strong>%s</strong> spostata in questo forum.";
#elif L==8	// pl
	"W&aogon;tek <strong>%s</strong> zosta&lstrok; przeniesiony na to forum.";
#elif L==9	// pt
	"Discuss&atilde;o <strong>%s</strong> movida para este f&oacute;rum.";
#elif L==10	// tr
	"Thread <strong>%s</strong> moved to this forum.";		// eviri lazim!
#endif

const char *Txt_Thread_removed =
#if   L==1	// ca
	"Discussi&oacute; eliminada.";
#elif L==2	// de
	"Faden entfernt.";
#elif L==3	// en
	"Thread removed.";
#elif L==4	// es
	"Discusi&oacute;n eliminada.";
#elif L==5	// fr
	"Fil supprim&eacute;.";
#elif L==6	// gn
	"Discusi&oacute;n eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione rimossa.";
#elif L==8	// pl
	"Usuni&eogon;to w&aogon;tek.";
#elif L==9	// pt
	"Discuss&atilde;o removida.";
#elif L==10	// tr
	"Thread removed.";		// eviri lazim!
#endif

const char *Txt_Thread_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discussi&oacute; <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Faden <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Thread <strong>%s</strong> removed.";
#elif L==4	// es
	"Discusi&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Fil <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Discusi&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Usuni&eogon;to w&aogon;tek <strong>%s</strong>.";
#elif L==9	// pt
	"Discuss&atilde;o <strong>%s</strong> removida.";
#elif L==10	// tr
	"Thread <strong>%s</strong> removed.";			// eviri lazim!
#endif

const char *Txt_Thread_with_posts_from_you =
#if   L==1	// ca
	"Discussi&oacute; amb comentaris de vost&egrave;";
#elif L==2	// de
	"Thread mit Beitr&auml;gen von Ihnen";
#elif L==3	// en
	"Thread with posts from you";
#elif L==4	// es
	"Discusi&oacute;n con comentarios de usted";
#elif L==5	// fr
	"Fil avec des messages de votre part";
#elif L==6	// gn
	"Discusi&oacute;n con comentarios de usted";	// Okoteve traduccin
#elif L==7	// it
	"Discussione con post per te";
#elif L==8	// pl
	"W&aogon;tek z wpisami od Ciebie";
#elif L==9	// pt
	"Discuss&atilde;o com mensagens de voc&ecirc;";
#elif L==10	// tr
	"Thread with posts from you";			// eviri lazim!
#endif

const char *Txt_threads =
#if   L==1	// ca
	"discussions";
#elif L==2	// de
	"Threads";
#elif L==3	// en
	"threads";
#elif L==4	// es
	"discusiones";
#elif L==5	// fr
	"fils";
#elif L==6	// gn
	"discusiones";	// Okoteve traduccin
#elif L==7	// it
	"discussione";
#elif L==8	// pl
	"w&aogon;tki";
#elif L==9	// pt
	"discuss&otilde;es";
#elif L==10	// tr
	"threads";	// eviri lazim!
#endif

const char *Txt_TIME_Since =
#if   L==1	// ca
	"Des de";
#elif L==2	// de
	"Seit dem";
#elif L==3	// en
	"Since";
#elif L==4	// es
	"Desde";
#elif L==5	// fr
	"Depuis";
#elif L==6	// gn
	"Desde";	// Okoteve traduccin
#elif L==7	// it
	"Dal";
#elif L==8	// pl
	"Od";
#elif L==9	// pt
	"Desde";
#elif L==10	// tr
	"Since";	// eviri lazim!
#endif

const char *Txt_TIME_until =
#if   L==1	// ca
	"fins al";
#elif L==2	// de
	"bis zum";
#elif L==3	// en
	"until";
#elif L==4	// es
	"hasta";
#elif L==5	// fr
	"jusqu'au";
#elif L==6	// gn
	"hasta";	// Okoteve traduccin
#elif L==7	// it
	"al";
#elif L==8	// pl
	"do";
#elif L==9	// pt
	"at&eacute;";
#elif L==10	// tr
	"until";	// eviri lazim!
#endif

const char *Txt_time =
#if   L==1	// ca
	"temps";
#elif L==2	// de
	"Zeit";
#elif L==3	// en
	"time";
#elif L==4	// es
	"tiempo";
#elif L==5	// fr
	"temps";
#elif L==6	// gn
	"aravo";
#elif L==7	// it
	"tempo";
#elif L==8	// pl
	"czas";
#elif L==9	// pt
	"tempo";
#elif L==10	// tr
	"zaman";
#endif

const char *Txt_Time_zone_used_in_the_calculation_of_these_statistics =
#if   L==1	// ca
	"Zona hor&agrave;ria usada en el c&agrave;lcul d'aquesta estad&iacute;stica";
#elif L==2	// de
	"Zeitzone in die Berechnung dieser Statistik verwendet";
#elif L==3	// en
	"Time zone used in the calculation of these statistics";
#elif L==4	// es
	"Zona horaria usada en el c&aacute;lculo de esta estad&iacute;stica";
#elif L==5	// fr
	"Fuseau horaire utilis&eacute; dans le calcul de ces statistiques";
#elif L==6	// gn
	"Zona horaria usada en el c&aacute;lculo de esta estad&iacute;stica";	// Okoteve traduccin
#elif L==7	// it
	"Fuso orario utilizzato per il calcolo di queste statistiche";
#elif L==8	// pl
	"Strefa czasowa u&zdot;ywana w obliczeniach tych statystyk";
#elif L==9	// pt
	"Fuso hor&aacute;rio utilizado no c&aacute;lculo dessas estat&iacute;sticas";
#elif L==10	// tr
	"Time zone used in the calculation of these statistics";		// eviri lazim!
#endif

const char *Txt_Timeline =
#if   L==1	// ca
	"Timeline";
#elif L==2	// de
	"Timeline";
#elif L==3	// en
	"Timeline";
#elif L==4	// es
	"Timeline";
#elif L==5	// fr
	"Chronologie";
#elif L==6	// gn
	"Timeline";
#elif L==7	// it
	"Timeline";
#elif L==8	// pl
	"Timeline";
#elif L==9	// pt
	"Timeline";
#elif L==10	// tr
	"Zaman &ccedil;izelgesi";
#endif

const char *Txt_Timeline_OF_A_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Timeline de %s";
#elif L==2	// de
	"%s timeline";
#elif L==3	// en
	"%s timeline";
#elif L==4	// es
	"Timeline de %s";
#elif L==5	// fr
	"Chronologie de %s";
#elif L==6	// gn
	"Timeline de %s";	// Okoteve traduccin
#elif L==7	// it
	"Timeline di %s";
#elif L==8	// pl
	"Timeline %s";
#elif L==9	// pt
	"Timeline de %s";
#elif L==10	// tr
	"%s timeline";		// eviri lazim!
#endif

const char *Txt_TIMELINE_Favourited_by_X_USERS =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Marcat com favorit per %u";
#elif L==2	// de
	"Von %u vorgemerkt";
#elif L==3	// en
	"Favourited by %u";
#elif L==4	// es
	"Marcado como favorito por %u";
#elif L==5	// fr
	"Ajout&eacute; aux favoris par %u";
#elif L==6	// gn
	"Marcado como favorito por %u";	// Okoteve traduccin
#elif L==7	// it
	"Preferiti da %u";
#elif L==8	// pl
	"Dodane do ulubionych przez %u";
#elif L==9	// pt
	"Marcado como favorito por %u";
#elif L==10	// tr
	"Favourited by %u";		// eviri lazim!
#endif

const char *Txt_TIMELINE_Not_favourited_by_anyone =	// No favourited
#if   L==1	// ca
	"No marcat com favorit per ning&uacute;";
#elif L==2	// de
	"Niemand hat sie als Favorit markiert";
#elif L==3	// en
	"Not favourited by anyone";
#elif L==4	// es
	"No marcado como favorito por nadie";
#elif L==5	// fr
	"Personne n'a marqu&eacute; comme favori";
#elif L==6	// gn
	"No marcado como favorito por nadie";	// Okoteve traduccin
#elif L==7	// it
	"Non preferiti da chiunque";
#elif L==8	// pl
	"Nie przez nikogo ulubionych";
#elif L==9	// pt
	"N&atilde;o marcado como favorito por qualquer pessoa";
#elif L==10	// tr
	"Not favourited by anyone";		// eviri lazim!
#endif

const char *Txt_TIMELINE_Not_shared_by_anyone =	// No shared
#if   L==1	// ca
	"No compartit per ning&uacute;";
#elif L==2	// de
	"Nicht durch Dritte freigegebenen";
#elif L==3	// en
	"Not shared by anyone";
#elif L==4	// es
	"No compartido por nadie";
#elif L==5	// fr
	"Personne n'a partag&eacute;";
#elif L==6	// gn
	"No compartido por nadie";	// Okoteve traduccin
#elif L==7	// it
	"Non condiviso da chiunque";
#elif L==8	// pl
	"Nie jest dzielona przez nikogo";
#elif L==9	// pt
	"N&atilde;o compartilhado por qualquer pessoa";
#elif L==10	// tr
	"Not shared by anyone";		// eviri lazim!
#endif

const char *Txt_TIMELINE_NOTE[Tml_NOT_NUM_NOTE_TYPES] =
	{
	[TmlNot_UNKNOWN] =
#if   L==1	// ca
	"Esdeveniment desconegut"
#elif L==2	// de
	"Unbekannt Ereignis"
#elif L==3	// en
	"Unknown event"
#elif L==4	// es
	"Evento desconocido"
#elif L==5	// fr
	"&Eacute;v&eacute;nement inconnu"
#elif L==6	// gn
	"Evento desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Evento sconosciuto"
#elif L==8	// pl
	"Nieznane zdarzenia"
#elif L==9	// pt
	"Evento desconhecido"
#elif L==10	// tr
	"Unknown event"		// eviri lazim!
#endif
	,
	[TmlNot_INS_DOC_PUB_FILE] =
#if   L==1	// ca
	"Document p&uacute;blic (instituci&oacute;)"
#elif L==2	// de
	"&Ouml;ffentliche Dokumentdatei (Hochschule)"
#elif L==3	// en
	"Public document file (institution)"
#elif L==4	// es
	"Documento p&uacute;blico (instituci&oacute;n)"
#elif L==5	// fr
	"Fichier de document public (&eacute;tablissement)"
#elif L==6	// gn
	"Documento p&uacute;blico (instituci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	"Documento pubblico (istituzione)"
#elif L==8	// pl
	"Plik dokumentu publiczne (instytucji)"
#elif L==9	// pt
	"Arquivo de documento p&uacute;blico (institu&ccedil;&atilde;o)"
#elif L==10	// tr
	"Public document file (institution)"		// eviri lazim!
#endif
	,
	[TmlNot_INS_SHA_PUB_FILE] =
#if   L==1	// ca
	"Arxiu compartit p&uacute;blic (instituci&oacute;)"
#elif L==2	// de
	"&Ouml;ffentliche Freigegebene Datei (Hochschule)"
#elif L==3	// en
	"Public shared file (institution)"
#elif L==4	// es
	"Archivo compartido p&uacute;blico (instituci&oacute;n)"
#elif L==5	// fr
	"Fichier partag&eacute; public (&eacute;tablissement)"
#elif L==6	// gn
	"Archivo compartido p&uacute;blico (instituci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	"File condiviso pubblico (istituzione)"
#elif L==8	// pl
	"Udost&eogon;pniony plik publiczne (instytucji)"
#elif L==9	// pt
	"Arquivo compartilhado p&uacute;blico (institu&ccedil;&atilde;o)"
#elif L==10	// tr
	"Public shared file (institution)"				// eviri lazim!
#endif
	,
	[TmlNot_CTR_DOC_PUB_FILE] =
#if   L==1	// ca
	"Document p&uacute;blic (center)"
#elif L==2	// de
	"&Ouml;ffentliche Dokumentdatei (Lehrinstitut)"
#elif L==3	// en
	"Public document file (center)"
#elif L==4	// es
	"Documento p&uacute;blico (centro)"
#elif L==5	// fr
	"Fichier de document public (center)"
#elif L==6	// gn
	"Documento p&uacute;blico (centro)"	// Okoteve traduccin
#elif L==7	// it
	"Documento pubblico (centro)"
#elif L==8	// pl
	"Plik dokumentu publiczne (centrum)"
#elif L==9	// pt
	"Arquivo de documento p&uacute;blico (centro)"
#elif L==10	// tr
	"Public document file (center)"		// eviri lazim!
#endif
	,
	[TmlNot_CTR_SHA_PUB_FILE] =
#if   L==1	// ca
	"Arxiu compartit p&uacute;blic (center)"
#elif L==2	// de
	"&Ouml;ffentliche Freigegebene Datei (Lehrinstitut)"
#elif L==3	// en
	"Public shared file (center)"
#elif L==4	// es
	"Archivo compartido p&uacute;blico (centro)"
#elif L==5	// fr
	"Fichier partag&eacute; public (center)"
#elif L==6	// gn
	"Archivo compartido p&uacute;blico (centro)"	// Okoteve traduccin
#elif L==7	// it
	"File condiviso pubblico (centro)"
#elif L==8	// pl
	"Udost&eogon;pniony plik publiczne (centrum)"
#elif L==9	// pt
	"Arquivo compartilhado p&uacute;blico (centro)"
#elif L==10	// tr
	"Public shared file (center)"			// eviri lazim!
#endif
	,
	[TmlNot_DEG_DOC_PUB_FILE] =
#if   L==1	// ca
	"Document p&uacute;blic (titulaci&oacute;)"
#elif L==2	// de
	"&Ouml;ffentliche Dokumentdatei (Studiengang)"
#elif L==3	// en
	"Public document file (degree)"
#elif L==4	// es
	"Documento p&uacute;blico (titulaci&oacute;n)"
#elif L==5	// fr
	"Fichier de document public (&eacute;tude)"
#elif L==6	// gn
	"Documento p&uacute;blico (titulaci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	"Documento pubblico (laurea)"
#elif L==8	// pl
	"Plik dokumentu publiczne (stopie&nacute;)"
#elif L==9	// pt
	"Arquivo de documento p&uacute;blico (grau)"
#elif L==10	// tr
	"Public document file (degree)"			// eviri lazim!
#endif
	,
	[TmlNot_DEG_SHA_PUB_FILE] =
#if   L==1	// ca
	"Arxiu compartit p&uacute;blic (titulaci&oacute;)"
#elif L==2	// de
	"&Ouml;ffentliche Freigegebene Datei (Studiengang)"
#elif L==3	// en
	"Public shared file (degree)"
#elif L==4	// es
	"Archivo compartido p&uacute;blico (titulaci&oacute;n)"
#elif L==5	// fr
	"Fichier partag&eacute; public (&eacute;tude)"
#elif L==6	// gn
	"Archivo compartido p&uacute;blico (titulaci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	"File condiviso pubblico (laurea)"
#elif L==8	// pl
	"Udost&eogon;pniony plik publiczne (stopie&nacute;)"
#elif L==9	// pt
	"Arquivo compartilhado p&uacute;blico (grau)"
#elif L==10	// tr
	"Public shared file (degree)"				// eviri lazim!
#endif
	,
	[TmlNot_CRS_DOC_PUB_FILE] =
#if   L==1	// ca
	"Document p&uacute;blic (assignatura)"
#elif L==2	// de
	"&Ouml;ffentliche Dokumentdatei (Kurs)"
#elif L==3	// en
	"Public document file (course)"
#elif L==4	// es
	"Documento p&uacute;blico (asignatura)"
#elif L==5	// fr
	"Fichier de document public (mati&egrave;re)"
#elif L==6	// gn
	"Documento p&uacute;blico (asignatura)"	// Okoteve traduccin
#elif L==7	// it
	"Documento pubblico (corso)"
#elif L==8	// pl
	"Plik dokumentu publiczne (kurs)"
#elif L==9	// pt
	"Arquivo de documento p&uacute;blico (disciplina)"
#elif L==10	// tr
	"Public document file (course)"		// eviri lazim!
#endif
	,
	[TmlNot_CRS_SHA_PUB_FILE] =
#if   L==1	// ca
	"Arxiu compartit p&uacute;blic (assignatura)"
#elif L==2	// de
	"&Ouml;ffentliche Freigegebene Datei (Kurs)"
#elif L==3	// en
	"Public shared file (course)"
#elif L==4	// es
	"Archivo compartido p&uacute;blico (asignatura)"
#elif L==5	// fr
	"Fichier partag&eacute; public (mati&egrave;re)"
#elif L==6	// gn
	"Archivo compartido p&uacute;blico (asignatura)"	// Okoteve traduccin
#elif L==7	// it
	"File condiviso pubblico (corso)"
#elif L==8	// pl
	"Udost&eogon;pniony plik publiczne (kurs)"
#elif L==9	// pt
	"Arquivo compartilhado p&uacute;blico (disciplina)"
#elif L==10	// tr
	"Public shared file (course)"				// eviri lazim!
#endif
	,
	[TmlNot_CALL_FOR_EXAM] =
#if   L==1	// ca
	"Convocat&ograve;ria d'examen"
#elif L==2	// de
	"Aufrufe fr Pr&uuml;fung"
#elif L==3	// en
	"Call for exam"
#elif L==4	// es
	"Convocatoria de examen"
#elif L==5	// fr
	"Convocation &agrave; un examen"
#elif L==6	// gn
	"Convocatoria de examen"	// Okoteve traduccin
#elif L==7	// it
	"Appello d'esame"
#elif L==8	// pl
	"Ogloszenie egzamin"
#elif L==9	// pt
	"Chamada para exame"
#elif L==10	// tr
	"S&inodot;nav i&ccedil;in aray&inodot;n"
#endif
	,
	[TmlNot_POST] =
#if   L==1	// ca
	"Missatge social"
#elif L==2	// de
	"Social Beitrag"
#elif L==3	// en
	"Social post"
#elif L==4	// es
	"Mensaje social"
#elif L==5	// fr
	"Message social"
#elif L==6	// gn
	"Mensaje social"	// Okoteve traduccin
#elif L==7	// it
	"Post sociale"
#elif L==8	// pl
	"Post spo&lstrok;ecznej"
#elif L==9	// pt
	"Post sociais"
#elif L==10	// tr
	"Social post"		// eviri lazim!
#endif
	,
	[TmlNot_FORUM_POST] =
#if   L==1	// ca
	"Comentari en un f&ograve;rum"
#elif L==2	// de
	"Beitr&auml;ge in einem Forum"
#elif L==3	// en
	"Post in a forum"
#elif L==4	// es
	"Comentario en un foro"
#elif L==5	// fr
	"Post dans un forum"
#elif L==6	// gn
	"Comentario en un foro"	// Okoteve traduccin
#elif L==7	// it
	"Post in un forum"
#elif L==8	// pl
	"Post na forum"
#elif L==9	// pt
	"Post em um f&oacute;rum"
#elif L==10	// tr
	"Post in a forum"	// eviri lazim!
#endif
	,
	[TmlNot_NOTICE] =
#if   L==1	// ca
	"Av&iacute;s"
#elif L==2	// de
	"Ank&uuml;ndigungen"
#elif L==3	// en
	"Notice"
#elif L==4	// es
	"Aviso"
#elif L==5	// fr
	"Avis"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Avviso"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"An&uacute;ncio"
#elif L==10	// tr
	"Notice"	// eviri lazim!
#endif
	};

const char *Txt_TIMELINE_NOTE_TOP_MESSAGES[Tml_NUM_TOP_MESSAGES] =
	{
	[Tml_TOP_MESSAGE_NONE] =
	""
	,
	[Tml_TOP_MESSAGE_COMMENTED] =
#if   L==1	// ca
	"ha comentat"
#elif L==2	// de
	"hat kommentiert"
#elif L==3	// en
	"has commented"
#elif L==4	// es
	"ha comentado"
#elif L==5	// fr
	"a comment&eacute;"
#elif L==6	// gn
	"ha comentado"	// Okoteve traduccin
#elif L==7	// it
	"ha commentato"
#elif L==8	// pl
	"skomentowa&lstrok;"
#elif L==9	// pt
	"comentou"
#elif L==10	// tr
	"has commented"	// eviri lazim!
#endif
	,
	[Tml_TOP_MESSAGE_FAVED] =
#if   L==1	// ca
	"ha marcat com favorit"
#elif L==2	// de
	"hat als Favorit markiert"
#elif L==3	// en
	"has marked as favourite"
#elif L==4	// es
	"ha marcado como favorito"
#elif L==5	// fr
	"a marqu&eacute; en tant que favori"
#elif L==6	// gn
	"ha marcado como favorito"	// Okoteve traduccin
#elif L==7	// it
	"ha segnato come preferito"
#elif L==8	// pl
	"oznaczone jako ulubiony"
#elif L==9	// pt
	"marcou como favorito"
#elif L==10	// tr
	"has marked as favourite"	// eviri lazim!
#endif
	,
	[Tml_TOP_MESSAGE_SHARED] =
#if   L==1	// ca
	"ha compartit"
#elif L==2	// de
	"hat sich geteilt"
#elif L==3	// en
	"has shared"
#elif L==4	// es
	"ha compartido"
#elif L==5	// fr
	"a partag&eacute;"
#elif L==6	// gn
	"ha compartido"	// Okoteve traduccin
#elif L==7	// it
	"ha condiviso"
#elif L==8	// pl
	"podzieli&lstrok;"
#elif L==9	// pt
	"compartilhou"
#elif L==10	// tr
	"has shared"	// eviri lazim!
#endif
	,
	[Tml_TOP_MESSAGE_MENTIONED] =
#if   L==1	// ca
	"li ha esmentat"
#elif L==2	// de
	"hat Sie erw&auml;hnt"
#elif L==3	// en
	"has mentioned you"
#elif L==4	// es
	"le ha mencionado"
#elif L==5	// fr
	"vous a mentionn&eacute;"
#elif L==6	// gn
	"le ha mencionado"	// Okoteve traduccin
#elif L==7	// it
	"ti ha menzionato"
#elif L==8	// pl
	"wspomnia&lstrok; o Tobie"
#elif L==9	// pt
	"mencionou voc&ecirc;"
#elif L==10	// tr
	"has mentioned you"	// eviri lazim!
#endif
	};

const char *Txt_TIMELINE_post =
#if   L==1	// ca
	"publicaci&oacute;";
#elif L==2	// de
	"Publikation";
#elif L==3	// en
	"post";
#elif L==4	// es
	"publicaci&oacute;n";
#elif L==5	// fr
	"publication";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"pubblicazione";
#elif L==8	// pl
	"publikacja";
#elif L==9	// pt
	"publica&ccedil;&atilde;o";
#elif L==10	// tr
	"post";	// eviri lazim!
#endif

const char *Txt_TIMELINE_Post_removed =
#if   L==1	// ca
	"Publicaci&oacute; eliminada.";
#elif L==2	// de
	"Publikation entfernt.";
#elif L==3	// en
	"Post removed.";
#elif L==4	// es
	"Publicaci&oacute;n eliminada.";
#elif L==5	// fr
	"Publication supprim&eacute;e.";
#elif L==6	// gn
	"Publicaci&oacute;n eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Pubblicazione rimossa.";
#elif L==8	// pl
	"Publikacja usuni&eogon;ta.";
#elif L==9	// pt
	"Publica&ccedil;&atilde;o removida.";
#elif L==10	// tr
	"Post removed.";			// eviri lazim!
#endif

const char *Txt_TIMELINE_posts =
#if   L==1	// ca
	"publicacions";
#elif L==2	// de
	"Publikationen";
#elif L==3	// en
	"posts";
#elif L==4	// es
	"publicaciones";
#elif L==5	// fr
	"publications";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"pubblicazioni";
#elif L==8	// pl
	"publikacje";
#elif L==9	// pt
	"publica&ccedil;&otilde;es";
#elif L==10	// tr
	"posts";	// eviri lazim!
#endif

const char *Txt_TIMETABLE_TYPES[Tmt_NUM_TIMETABLE_TYPES] =
	{
	[Tmt_COURSE_TIMETABLE] =
#if   L==1	// ca
	"Horari de classes de l'assignatura"
#elif L==2	// de
	"Stundenplan des Kurses"
#elif L==3	// en
	"Class timetable of the course"
#elif L==4	// es
	"Horario de clases de la asignatura"
#elif L==5	// fr
	"Horaire de classe de la mati&egrave;re"
#elif L==6	// gn
	"Horario de clases de la asignatura"	// Okoteve traduccin
#elif L==7	// it
	"Orario delle lezioni del corso"
#elif L==8	// pl
	"Plan zaj&eogon;&cacute; kursu"
#elif L==9	// pt
	"Hor&aacute;rio das aulas da disciplina"
#elif L==10	// tr
	"Dersin ders program&inodot;"
#endif
	,
	[Tmt_MY_TIMETABLE] =
#if   L==1	// ca
	"Horari de les meves assignatures"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"My timetable"
#elif L==4	// es
	"Horario de mis asignaturas"
#elif L==5	// fr
	"Horaire mes mati&egrave;res"
#elif L==6	// gn
	"Horario de mis asignaturas"	// Okoteve traduccin
#elif L==7	// it
	"Orario dei miei corsi"
#elif L==8	// pl
	"M&oacute;j plan lekcji"
#elif L==9	// pt
	"Hor&aacute;rio minha disciplinas"
#elif L==10	// tr
	"My timetable"			// eviri lazim!
#endif
	,
	[Tmt_TUTORING_TIMETABLE] =
#if   L==1	// ca
	"Horari de tutories"
#elif L==2	// de
	"Gesch&auml;ftszeiten"
#elif L==3	// en
	"Tutoring hours"
#elif L==4	// es
	"Horario de tutor&iacute;as"
#elif L==5	// fr
	"Heures de tuteur"
#elif L==6	// gn
	"Horario de tutor&iacute;as"	// Okoteve traduccin
#elif L==7	// it
	"Orario di tutoraggi"
#elif L==8	// pl
	"Godziny pracy"
#elif L==9	// pt
	"Hor&aacute;rio de tutor"
#elif L==10	// tr
	"Tutoring hours"		// eviri lazim!
#endif
	};

const char *Txt_TIMELINE_Shared_by_X_USERS =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Compartit per %u";
#elif L==2	// de
	"Von %u geteilt";
#elif L==3	// en
	"Shared by %u";
#elif L==4	// es
	"Compartido por %u";
#elif L==5	// fr
	"Partag&eacute; par %u";
#elif L==6	// gn
	"Compartido por %u";	// Okoteve traduccin
#elif L==7	// it
	"Condiviso da %u";
#elif L==8	// pl
	"Udost&eogon;pnione przez %u";
#elif L==9	// pt
	"Compartilhado por %u";
#elif L==10	// tr
	"Shared by %u";		// eviri lazim!
#endif

const char *Txt_TIMETABLE_CLASS_TYPES[Tmt_NUM_CLASS_TYPES] =
	{
	[Tmt_FREE] =
#if   L==1	// ca
	"Lliure"
#elif L==2	// de
	"Keine Klasse"
#elif L==3	// en
	"No class"
#elif L==4	// es
	"Libre"
#elif L==5	// fr
	"Pas de classe"
#elif L==6	// gn
	"Libre"		// Okoteve traduccin
#elif L==7	// it
	"Libero"
#elif L==8	// pl
	"Bez klasy"
#elif L==9	// pt
	"Livre"
#elif L==10	// tr
	"No class"	// eviri lazim!
#endif
	,
	[Tmt_LECTURE] =
#if   L==1	// ca
	"Teoria"
#elif L==2	// de
	"Vorlesung"
#elif L==3	// en
	"Lectures"
#elif L==4	// es
	"Teor&iacute;a"
#elif L==5	// fr
	"Th&eacute;orie"
#elif L==6	// gn
	"Teor&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"Teoria"
#elif L==8	// pl
	"Wyklady"
#elif L==9	// pt
	"Teoria"
#elif L==10	// tr
	"Lectures"	// eviri lazim!
#endif
	,
	[Tmt_PRACTICAL] =
#if   L==1	// ca
	"Pr&agrave;ct."
#elif L==2	// de
	"&Uuml;bung"
#elif L==3	// en
	"Practicals"
#elif L==4	// es
	"Pr&aacute;ct."
#elif L==5	// fr
	"Pratique"
#elif L==6	// gn
	"Pr&aacute;ct."	// Okoteve traduccin
#elif L==7	// it
	"Pratica"
#elif L==8	// pl
	"praktyki"
#elif L==9	// pt
	"Pr&aacute;ct."
#elif L==10	// tr
	"Practicals"	// eviri lazim!
#endif
	,
	[Tmt_TUTORING] =
#if   L==1	// ca
	"Tutories"
#elif L==2	// de
	"Anleitungen"
#elif L==3	// en
	"Tutoring"
#elif L==4	// es
	"Tutor&iacute;as"
#elif L==5	// fr
	"Tutorat"
#elif L==6	// gn
	"Tutor&iacute;as"	// Okoteve traduccin
#elif L==7	// it
	"Tutoraggio"
#elif L==8	// pl
	"Korepetycje"
#elif L==9	// pt
	"Tutoria"
#elif L==10	// tr
	"Tutoring"		// eviri lazim!
#endif
	};

const char *Txt_Title =
#if   L==1	// ca
	"T&iacute;tol";
#elif L==2	// de
	"Titel";
#elif L==3	// en
	"Title";
#elif L==4	// es
	"T&iacute;tulo";
#elif L==5	// fr
	"Titre";
#elif L==6	// gn
	"Moak&atilde;ha";
#elif L==7	// it
	"Titolo";
#elif L==8	// pl
	"Tytu&lstrok;";
#elif L==9	// pt
	"T&iacute;tulo";
#elif L==10	// tr
	"Title";	// eviri lazim!
#endif

const char *Txt_Title_attribution =
#if   L==1	// ca
	"T&iacute;tol/atribuci&oacute;";
#elif L==2	// de
	"Titel / Namensnennung";
#elif L==3	// en
	"Title/attribution";
#elif L==4	// es
	"T&iacute;tulo/atribuci&oacute;n";
#elif L==5	// fr
	"Titre/attribution";
#elif L==6	// gn
	"T&iacute;tulo/atribuci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Titolo/attribuzione";
#elif L==8	// pl
	"Tytu&lstrok;/przypisanie";
#elif L==9	// pt
	"T&iacute;tulo/atribui&ccedil;&atilde;o";
#elif L==10	// tr
	"Title/attribution";			// eviri lazim!
#endif

const char *Txt_To_remove_a_center_you_must_first_remove_all_degrees_and_teachers_in_the_center =
#if   L==1	// ca
	"Per eliminar un centre, primer heu d'eliminar"
	" tots els graus i professors del centre.";
#elif L==2	// de
	"Um ein Zentrum zu entfernen, m&uuml;ssen Sie zuerst"
	" alle Abschlsse und Lehrer im Zentrum entfernen.";
#elif L==3	// en
	"To remove a center, you must first remove"
	" all degrees and teachers in the center.";
#elif L==4	// es
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";
#elif L==5	// fr
	"Pour supprimer un centre, vous devez d'abord supprimer"
	" tous les dipl&ocirc;mes et enseignants du centre.";
#elif L==6	// gn
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un centro, devi prima rimuovere"
	" tutte le lauree e professori di quel centro.";
#elif L==8	// pl
	"Aby usun&aogon;&cacute; &sacute;rodek, musisz najpierw usun&aogon;&cacute;"
	" wszystkie stopnie i nauczycieli ze &sacute;rodka.";
#elif L==9	// pt
	"Para remover um centro, voc&ecirc; deve primeiro remover"
	" todos os graus e professores no centro.";
#elif L==10	// tr
	"To remove a center, you must first remove"
	" all degrees and teachers in the center.";				// eviri lazim!
#endif

const char *Txt_To_remove_a_course_you_must_first_remove_all_users_in_the_course =
#if   L==1	// ca
	"Per eliminar una assignatura, primer heu d'eliminar"
	" tots els usuaris de la assignatura.";
#elif L==2	// de
	"Um einen Kurs zu entfernen, m&uuml;ssen Sie zuerst"
	" alle Benutzer im Kurs entfernen.";
#elif L==3	// en
	"To remove a course, you must first remove"
	" all users in the course.";
#elif L==4	// es
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";
#elif L==5	// fr
	"Pour supprimer une mati&egrave;re, vous devez d'abord supprimer"
	" tous les utilisateurs de la mati&egrave;re.";
#elif L==6	// gn
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";		// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un corso, devi prima rimuovere"
	" tutti gli utenti di quel corso.";
#elif L==8	// pl
	"Aby usun&aogon;&cacute; kurs, musisz najpierw usun&aogon;&cacute;"
	" wszystkich u&zdot;ytkownik&oacute;w kursu.";
#elif L==9	// pt
	"Para remover uma disciplina, voc&ecirc; deve primeiro remover"
	" todos os usu&aacute;rios da disciplina.";
#elif L==10	// tr
	"To remove a course, you must first remove"
	" all users in the course.";				// eviri lazim!
#endif

const char *Txt_To_remove_a_degree_you_must_first_remove_all_courses_in_the_degree =
#if   L==1	// ca
	"Per eliminar una titulaci&oacute;, primer heu d'eliminar"
	" tots els cursos de la titulaci&oacute;.";
#elif L==2	// de
	"Um einen Abschluss zu entfernen, m&uuml;ssen Sie zuerst"
	" alle Kurse des Abschlusses entfernen.";
#elif L==3	// en
	"To remove a degree, you must first remove"
	" all courses in the degree.";
#elif L==4	// es
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";
#elif L==5	// fr
	"Pour supprimer un &eacute;tude, vous devez d'abord supprimer"
	" tous les mati&egrave;res de l'&eacute;tude.";
#elif L==6	// gn
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere una laurea, devi prima rimuovere"
	" tutti i corsi della laurea.";
#elif L==8	// pl
	"Aby usun&aogon;&cacute; stopie&nacute;, musisz najpierw usun&aogon;&cacute;"
	" wszystkie przedmioty z tego stopnia.";
#elif L==9	// pt
	"Para remover um grau, voc&ecirc; deve primeiro remover"
	" todas as disciplinas do grau.";
#elif L==10	// tr
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// eviri lazim!
#endif

const char *Txt_To_remove_a_department_you_must_first_remove_all_teachers_in_the_department =
#if   L==1	// ca
	"Per eliminar un departament, primer heu d'eliminar"
	" tots els professors del departament.";
#elif L==2	// de
	"Um eine Abteilung zu entfernen, m&uuml;ssen Sie zuerst"
	" alle Lehrer in der Abteilung entfernen.";
#elif L==3	// en
	"To remove a department, you must first remove"
	" all teachers in the department.";
#elif L==4	// es
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";
#elif L==5	// fr
	"Pour supprimer un d&eacute;partement, vous devez d'abord supprimer"
	" tous les enseignants du d&eacute;partement.";
#elif L==6	// gn
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un dipartimento, devi prima rimuovere"
	" tutti i professori di quel dipartimento.";
#elif L==8	// pl
	"Aby usun&aogon;&cacute; dzia&lstrok;, musisz najpierw usun&aogon;&cacute;"
	" wszystkich nauczycieli z dzia&lstrok;u.";
#elif L==9	// pt
	"Para remover um departamento, voc&ecirc; deve primeiro remover"
	" todos os professores do departamento.";
#elif L==10	// tr
	"To remove a department, you must first remove"
	" all teachers in the department.";		// eviri lazim!
#endif

const char *Txt_To_remove_a_place_you_must_first_remove_all_centers_of_that_place =
#if   L==1	// ca
	"Per eliminar un lloc, primer heu d'eliminar"
	" tots els centres d'aquest lloc.";
#elif L==2	// de
	"Um einen Ort zu entfernen, m&uuml;ssen Sie zuerst"
	" alle Zentren dieses Ortes entfernen.";
#elif L==3	// en
	"To remove a place, you must first remove"
	" all centers of that place.";
#elif L==4	// es
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";
#elif L==5	// fr
	"Pour supprimer un lieu, vous devez d'abord supprimer"
	" tous les centres de ce lieu.";
#elif L==6	// gn
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";		// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un luogo, devi prima rimuovere"
	" tutte le centri di quel luogo.";
#elif L==8	// pl
	"Aby usun&aogon;&cacute; miejsce, musisz najpierw usun&aogon;&cacute;"
	" wszystkie centra tego miejsca.";
#elif L==9	// pt
	"Para remover uma localiza&ccedil;&atilde;o, voc&ecirc; deve primeiro remover"
	" todos os centros da localiza&ccedil;&atilde;o.";
#elif L==10	// tr
	"To remove a place, you must first remove"
	" all centers of that place.";			// eviri lazim!
#endif

const char *Txt_To_remove_a_type_of_degree_you_must_first_remove_all_degrees_of_that_type =
#if   L==1	// ca
	"Per eliminar un tipus de grau,"
	" primer heu d'eliminar"
	" tots els graus d'aquest tipus.";
#elif L==2	// de
	"Um einen Abschlusstyp zu entfernen,"
	" m&uuml;ssen Sie zuerst"
	" alle Abschl&uuml;sse dieses Typs entfernen.";
#elif L==3	// en
	"To remove a type of degree,"
	" you must first remove"
	" all degrees of that type.";
#elif L==4	// es
	"Para eliminar un tipo de titulaci&oacute;n,"
	" primero debe eliminar"
	" todas las titulaciones de ese tipo.";
#elif L==5	// fr
	"Pour supprimer un type d'&eacute;tude,"
	" vous devez d'abord supprimer"
	" tous les &eacute;tudes de ce type.";
#elif L==6	// gn
	"Para eliminar un tipo de titulaci&oacute;n,"
	" primero debe eliminar"
	" todas las titulaciones de ese tipo.";		// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un tipo di laurea,"
	" devi prima rimuovere"
	" tutte le lauree di quel tipo.";
#elif L==8	// pl
	"Aby usun&aogon;&cacute; rodzaj stopnia,"
	" musisz najpierw usun&aogon;&cacute;"
	" wszystkie stopnie tego typu.";
#elif L==9	// pt
	"Para remover um tipo de grau,"
	" voc&ecirc; deve primeiro remover"
	" todos os graus desse tipo.";
#elif L==10	// tr
	"To remove a type of degree,"
	" you must first remove"
	" all degrees of that type.";			// eviri lazim!
#endif

const char *Txt_To_remove_an_institution_you_must_first_remove_all_centers_and_users_in_the_institution =
#if   L==1	// ca
	"Per eliminar una instituci&oacute;, primer heu d'eliminar"
	" tots els centres i usuaris de la instituci&oacute;.";
#elif L==2	// de
	"Um eine Institution zu entfernen, m&uuml;ssen Sie zuerst"
	" alle Zentren und Benutzer in der Institution entfernen.";
#elif L==3	// en
	"To remove an institution, you must first remove"
	" all centers and users in the institution.";
#elif L==4	// es
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos los centros y todos los usuarios de esa instituci&oacute;n.";
#elif L==5	// fr
	"Pour supprimer un &eacute;tablissement, vous devez d'abord supprimer"
	" tous les centres et utilisateurs de l'&eacute;tablissement.";
#elif L==6	// gn
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos centros y todos los usuarios de esa instituci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un'istituzione, devi prima rimuovere"
	" tutti gli centri e utenti di quell'istituzione.";
#elif L==8	// pl
	"Aby usun&aogon;&cacute; instytucj&eogon;, musisz najpierw"
	" usun&aogon;&cacute; wszystkie centra i u&zdot;ytkownik&oacute;w w instytucji.";
#elif L==9	// pt
	"Para remover uma institui&ccedil;&atilde;o, voc&ecirc; deve primeiro remover"
	" todos os centros e usu&aacute;rios da institui&ccedil;&atilde;o.";
#elif L==10	// tr
	"To remove an institution, you must first remove"
	" all centers and users in the institution.";				// eviri lazim!
#endif

const char *Txt_To_use_chat_you_must_have_installed_the_software_X_and_add_Y_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"Per utilitzar el xat cal tenir instal lat el programari"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" i afegir <strong>https://%s</strong>"
	" a la llista de llocs permesos en el panell de control de Java.";
#elif L==2	// de
	"So verwenden Sie chatten, m&uuml;ssen Sie die Software installiert haben"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" und f&uuml;gen Sie <strong>https://%s</strong>"
	" in die Liste der erlaubten Seiten in der Java-Bedienfeld.";
#elif L==3	// en
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";
#elif L==4	// es
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";
#elif L==5	// fr
	"Pour utiliser le chat, vous devez avoir install&eacute; le logiciel"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" et ajouter <strong>https://%s</strong>"
	" &agrave; la liste des sites autoris&eacute;s dans le panneau de contr&ocirc;le Java.";
#elif L==6	// gn
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";	// Okoteve traduccin
#elif L==7	// it
	"Per utilizzare la chat devi aver installato il software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e aggiungere <strong>https://%s</strong>"
	" alla lista dei siti consentiti nel pannello di controllo Java.";
#elif L==8	// pl
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para usar o chat &eacute; necess&aacute;rio ter instalado o software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e adicionar <strong>https://%s</strong>"
	" &agrave; lista de sites permitidos no painel de controle Java.";
#elif L==10	// tr
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";		// eviri lazim!
#endif

const char *Txt_To_watch_multimedia_content_from_another_website_you_have_to_accept_third_party_cookies_in_your_personal_settings =
#if   L==1	// ca
	"Per veure contingut multim&egrave;dia d'un altre lloc web"
	" ha d'acceptar les cookies de tercers"
	" en les seves par&agrave;metres personals.";
#elif L==2	// de
	"Um Multimedia-Inhalte von einer anderen Website anzuzeigen,"
	" m&uuml;ssen Sie in Ihren pers&ouml;nlichen Einstellungen Cookies"
	" von Drittanbietern akzeptieren.";
#elif L==3	// en
	"To watch multimedia content from another website"
	" you have to accept third party cookies"
	" in your personal settings.";
#elif L==4	// es
	"Para ver contenido multimedia de otro sitio web"
	" tiene que aceptar las cookies de terceros"
	" en sus ajustes personales.";
#elif L==5	// fr
	"Pour regarder le contenu multim&eacute;dia d'un autre site Web,"
	" vous devez accepter les cookies de tiers"
	" dans vos r&eacute;glages personnels.";
#elif L==6	// gn
	"Para ver contenido multimedia de otro sitio web"
	" tiene que aceptar las cookies de terceros"
	" en sus ajustes personales.";	// Okoteve traduccin
#elif L==7	// it
	"Per visualizzare i contenuti multimediali di un altro sito web"
	" devi accettare i cookie di terze parti"
	" nelle tue impostazioni personali.";
#elif L==8	// pl
	"Aby przegl&aogon;da&cacute; tre&sacute;ci multimedialne z innej strony,"
	" musisz zaakceptowa&cacute; pliki cookie innych firm"
	" w swoich ustawieniach osobistych.";
#elif L==9	// pt
	"Para visualizar o conte&uacute;do multimdia de outro site,"
	" voc&ecirc; precisa aceitar cookies de terceiros"
	" nas suas configura&ccedil;&otilde;es pessoais.";
#elif L==10	// tr
	"To watch multimedia content from another website"
	" you have to accept third party cookies"
	" in your personal settings.";	// eviri lazim!
#endif

const char *Txt_Today =
#if   L==1	// ca
	"Avui";
#elif L==2	// de
	"Heute";
#elif L==3	// en
	"Today";
#elif L==4	// es
	"Hoy";
#elif L==5	// fr
	"Aujourd'hui";
#elif L==6	// gn
	"Ko &aacute;ra";
#elif L==7	// it
	"Oggi";
#elif L==8	// pl
	"Dzisiaj";
#elif L==9	// pt
	"Hoje";
#elif L==10	// tr
	"Today";	// eviri lazim!
#endif

const char *Txt_Total =
#if   L==1	// ca
	"Total";
#elif L==2	// de
	"Gesamtzahl";
#elif L==3	// en
	"Total";
#elif L==4	// es
	"Total";
#elif L==5	// fr
	"Total";
#elif L==6	// gn
	"Total";	// Okoteve traduccin
#elif L==7	// it
	"Totale";
#elif L==8	// pl
	"Ca&lstrok;kowity";
#elif L==9	// pt
	"Total";
#elif L==10	// tr
	"Total";	// eviri lazim!
#endif

const char *Txt_total =
#if   L==1	// ca
	"total";
#elif L==2	// de
	"Gesamtzahl";
#elif L==3	// en
	"total";
#elif L==4	// es
	"total";
#elif L==5	// fr
	"total";
#elif L==6	// gn
	"total";	// Okoteve traduccin
#elif L==7	// it
	"totale";
#elif L==8	// pl
	"ca&lstrok;kowity";
#elif L==9	// pt
	"total";
#elif L==10	// tr
	"total";	// eviri lazim!
#endif

const char *Txt_TST_Answer_given_by_the_teachers =
#if   L==1	// ca
	"Resposta donada pels professors";
#elif L==2	// de
	"Antwort von den Lehrern";
#elif L==3	// en
	"Answer given by the teachers";
#elif L==4	// es
	"Respuesta dada por los profesores";
#elif L==5	// fr
	"R&eacute;ponse donn&eacute;e par les professeurs";
#elif L==6	// gn
	"Respuesta dada por los profesores";	// Okoteve traduccin
#elif L==7	// it
	"Risposta fornita dagli insegnanti";
#elif L==8	// pl
	"Odpowied&zacute; udzielona przez nauczycieli";
#elif L==9	// pt
	"Resposta dada pelos professores";
#elif L==10	// tr
	"Answer given by the teachers";		// eviri lazim!
#endif

const char *Txt_TST_Answer_given_by_the_user =
#if   L==1	// ca
	"Resposta donada per l'usuari/a";
#elif L==2	// de
	"Antwort vom Benutzer";
#elif L==3	// en
	"Answer given by the user";
#elif L==4	// es
	"Respuesta dada por el usuario/a";
#elif L==5	// fr
	"R&eacute;ponse donn&eacute;e par l'utilisateur/se";
#elif L==6	// gn
	"Respuesta dada por el usuario/a";	// Okoteve traduccin
#elif L==7	// it
	"Risposta fornita dall'utente";
#elif L==8	// pl
	"Odpowied&zacute; udzielona przez u&zdot;ytkownika";
#elif L==9	// pt
	"Resposta dada pelo usu&aacute;rio/a";
#elif L==10	// tr
	"Answer given by the user";		// eviri lazim!
#endif

const char *Txt_TST_HIDDEN_VISIBLE[HidVis_NUM_HIDDEN_VISIBLE] =
	{
	[HidVis_HIDDEN] =
#if   L==1	// ca
	"ocult"
#elif L==2	// de
	"versteckt"
#elif L==3	// en
	"hidden"
#elif L==4	// es
	"oculto"
#elif L==5	// fr
	"cach&eacute;"
#elif L==6	// gn
	"oculto"	// Okoteve traduccin
#elif L==7	// it
	"nascosto"
#elif L==8	// pl
	"ukryty"
#elif L==9	// pt
	"oculto"
#elif L==10	// tr
	"hidden"	// eviri lazim!
#endif
	,
	[HidVis_VISIBLE] =
#if   L==1	// ca
	"visible"
#elif L==2	// de
	"sichtbar"
#elif L==3	// en
	"visible"
#elif L==4	// es
	"visible"
#elif L==5	// fr
	"visible"
#elif L==6	// gn
	"visible"	// Okoteve traduccin
#elif L==7	// it
	"visibile"
#elif L==8	// pl
	"widoczny"
#elif L==9	// pt
	"vis&iacute;vel"
#elif L==10	// tr
	"visible"	// eviri lazim!
#endif
	};

const char *Txt_TST_PLUGGABLE[TstCfg_NUM_OPTIONS_PLUGGABLE] =
	{
	[TstCfg_PLUGGABLE_UNKNOWN] =
	"",
	[TstCfg_PLUGGABLE_NO] =
#if   L==1	// ca
	"Banc de preguntes no descarregable/visible des d'altres aplicacions (SWADroid, iSWAD, TriSWADos)."
#elif L==2	// de
	"Fragenbank nicht herunterladbar/sichtbar von anderen Anwendungen (SWADroid, iSWAD, TriSWADos)."
#elif L==3	// en
	"Question bank not downloadable/visible from other applications (SWADroid, iSWAD, TriSWADos)."
#elif L==4	// es
	"Banco de preguntas no descargable/visible desde otras aplicaciones (SWADroid, iSWAD, TriSWADos)."
#elif L==5	// fr
	"Banque de questions non t&eacute;l&eacute;chargeable/visible depuis d'autres applications (SWADroid, iSWAD, TriSWADos)."
#elif L==6	// gn
	"Banco de preguntas no descargable/visible desde otras aplicaciones (SWADroid, iSWAD, TriSWADos)."// Okoteve traduccin
#elif L==7	// it
	"Banca di domande non scaricabile/visibile da altre applicazioni (SWADroid, iSWAD, TriSWADos)."
#elif L==8	// pl
	"Bank pyta&nacute; nie do pobrania/widoczny z innych aplikacji (SWADroid, iSWAD, TriSWADos)."
#elif L==9	// pt
	"Banco de perguntas n&atilde;o dispon&iacute;vel para download/vis&iacute;vel de outros aplicativos (SWADroid, iSWAD, TriSWADos)."
#elif L==10	// tr
	"Question bank not downloadable/visible from other applications (SWADroid, iSWAD, TriSWADos)."	// eviri lazim!
#endif
	,
	[TstCfg_PLUGGABLE_YES] =
#if   L==1	// ca
	"Banc de preguntes descarregable/visible des d'altres aplicacions (SWADroid, iSWAD, TriSWADos)."
#elif L==2	// de
	"Fragenbank herunterladbar/sichtbar von anderen Anwendungen (SWADroid, iSWAD, TriSWADos)."
#elif L==3	// en
	"Question bank downloadable/visible from other applications (SWADroid, iSWAD, TriSWADos)."
#elif L==4	// es
	"Banco de preguntas descargable/visible desde otras aplicaciones (SWADroid, iSWAD, TriSWADos)."
#elif L==5	// fr
	"Banque de questions t&eacute;l&eacute;chargeable/visible depuis d'autres applications (SWADroid, iSWAD, TriSWADos)."
#elif L==6	// gn
	"Banco de preguntas descargable/visible desde otras aplicaciones (SWADroid, iSWAD, TriSWADos)."// Okoteve traduccin
#elif L==7	// it
	"Banca di domande scaricabile/visibile da altre applicazioni (SWADroid, iSWAD, TriSWADos)."
#elif L==8	// pl
	"Bank pyta&nacute; do pobrania/widoczny z innych aplikacji (SWADroid, iSWAD, TriSWADos)."
#elif L==9	// pt
	"Banco de perguntas dispon&iacute;vel para download/vis&iacute;vel de outros aplicativos (SWADroid, iSWAD, TriSWADos)."
#elif L==10	// tr
	"Question bank downloadable/visible from other applications (SWADroid, iSWAD, TriSWADos)."	// eviri lazim!
#endif
	};

const char *Txt_TST_STR_ANSWER_TYPES[Qst_NUM_ANS_TYPES] =
	{
	[Qst_ANS_INT] =
#if   L==1	// ca
	"Nombre enter"
#elif L==2	// de
	"Ganze Zahl"
#elif L==3	// en
	"Integer number"
#elif L==4	// es
	"N.&ordm; entero"
#elif L==5	// fr
	"Nombre entier"
#elif L==6	// gn
	"N.&ordm; entero"	// Okoteve traduccin
#elif L==7	// it
	"Numero intero"
#elif L==8	// pl
	"Integer number"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N.&ordm; inteiro"
#elif L==10	// tr
	"Integer number"	// eviri lazim!
#endif
	,
	[Qst_ANS_FLOAT] =
#if   L==1	// ca
	"Nombre real"
#elif L==2	// de
	"Reelle Zahl"
#elif L==3	// en
	"Real number"
#elif L==4	// es
	"N.&ordm; real"
#elif L==5	// fr
	"Nombre r&eacute;el"
#elif L==6	// gn
	"N.&ordm; real"	// Okoteve traduccin
#elif L==7	// it
	"Numero reale"
#elif L==8	// pl
	"Liczba rzeczywista"
#elif L==9	// pt
	"N.&ordm; real"
#elif L==10	// tr
	"Real number"	// eviri lazim!
#endif
	,
	[Qst_ANS_TRUE_FALSE] =
#if   L==1	// ca
	"V/F"
#elif L==2	// de
	"R/F"
#elif L==3	// en
	"T/F"
#elif L==4	// es
	"V/F"
#elif L==5	// fr
	"V/F"
#elif L==6	// gn
	"V/F"	// Okoteve traduccin
#elif L==7	// it
	"V/F"
#elif L==8	// pl
	"P/F"
#elif L==9	// pt
	"V/F"
#elif L==10	// tr
	"T/F"	// eviri lazim!
#endif
	,
	[Qst_ANS_UNIQUE_CHOICE] =
#if   L==1	// ca
	"Elecci&oacute; &uacute;nica"
#elif L==2	// de
	"Unique-Choice"
#elif L==3	// en
	"Unique choice"
#elif L==4	// es
	"Elecci&oacute;n &uacute;nica"
#elif L==5	// fr
	"Choix unique"
#elif L==6	// gn
	"Elecci&oacute;n &uacute;nica"	// Okoteve traduccin
#elif L==7	// it
	"Scelta unica"
#elif L==8	// pl
	"Unikalny wyb&oacute;r"
#elif L==9	// pt
	"&Uacute;nica escolha"
#elif L==10	// tr
	"Unique choice"			// eviri lazim!
#endif
	,
	[Qst_ANS_MULTIPLE_CHOICE] =
#if   L==1	// ca
	"Elecci&oacute; m&uacute;ltiple"
#elif L==2	// de
	"Multiple-Choice"
#elif L==3	// en
	"Multiple choice"
#elif L==4	// es
	"Elecci&oacute;n m&uacute;ltiple"
#elif L==5	// fr
	"Choix multiple"
#elif L==6	// gn
	"Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==7	// it
	"Scelta multipla"
#elif L==8	// pl
	"Wielokrotnego wyboru"
#elif L==9	// pt
	"M&uacute;ltipla escolha"
#elif L==10	// tr
	"Multiple choice"			// eviri lazim!
#endif
	,
	[Qst_ANS_TEXT] =
#if   L==1	// ca
	"Text"
#elif L==2	// de
	"Text"
#elif L==3	// en
	"Text"
#elif L==4	// es
	"Texto"
#elif L==5	// fr
	"Texte"
#elif L==6	// gn
	"Mba'ehaipyre"
#elif L==7	// it
	"Testo"
#elif L==8	// pl
	"Tekst"
#elif L==9	// pt
	"Texto"
#elif L==10	// tr
	"Text"	// eviri lazim!
#endif
	};

const char *Txt_TST_STR_ORDER_FULL[Qst_NUM_TYPES_ORDER_QST] =
	{
	[Qst_ORDER_STEM] =
#if   L==1	// ca
	"Ordenar por enunciado de la pregunta"	// Necessita traducci
#elif L==2	// de
	"Sortieren nach Stammzellen in Frage"
#elif L==3	// en
	"Sort by stem of the question"
#elif L==4	// es
	"Ordenar por enunciado de la pregunta"
#elif L==5	// fr
	"Trier par question"
#elif L==6	// gn
	"Ordenar por enunciado de la pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per domanda"
#elif L==8	// pl
	"Sort by stem of the question"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por formula&ccedil;&atilde;o da pergunta"
#elif L==10	// tr
	"Sort by stem of the question"		// eviri lazim!
#endif
	,
	[Qst_ORDER_NUM_HITS] =
#if   L==1	// ca
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Necessita traducci
#elif L==2	// de
	"Sortieren nach Anzahl der mal die Frage beantwortet wurde"
#elif L==3	// en
	"Sort by number of times the question has been answered"
#elif L==4	// es
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"
#elif L==5	// fr
	"Trier par nombre de fois qu'on a r&eacute;pondu &agrave; la question"
#elif L==6	// gn
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di volte che hai risposto alla domanda"
#elif L==8	// pl
	"Sort by number of times the question has been answered"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por n&ordm; de vezes que a pergunta foi respondida"
#elif L==10	// tr
	"Sort by number of times the question has been answered"	// eviri lazim!
#endif
	,
	[Qst_ORDER_AVERAGE_SCORE] =
#if   L==1	// ca
	"Ordenar por puntuaci&oacute;n media"	// Necessita traducci
#elif L==2	// de
	"Sortieren nach Durchschnittsnote"
#elif L==3	// en
	"Sort by average score"
#elif L==4	// es
	"Ordenar por puntuaci&oacute;n media"
#elif L==5	// fr
	"Trier par note moyenne"
#elif L==6	// gn
	"Ordenar por puntuaci&oacute;n media"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per punteggio medio"
#elif L==8	// pl
	"Sort by average score"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por pontua&ccedil;&atilde;o m&eacute;dia"
#elif L==10	// tr
	"Sort by average score"			// eviri lazim!
#endif
	,
	[Qst_ORDER_NUM_HITS_NOT_BLANK] =
#if   L==1	// ca
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"	// Necessita traducci
#elif L==2	// de
	"Sortieren nach Anzahl der mal"
	" die Frage beantwortet wurde (nicht leer)"
#elif L==3	// en
	"Sort by number of times the question"
	" has been answered (excluding blank answers)"
#elif L==4	// es
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"
#elif L==5	// fr
	"Trier par nombre de fois qu'on a r&eacute;pondu"
	" &agrave; la question (non vide)"
#elif L==6	// gn
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di volte che hai risposto"
	" alla domanda (non vuoto)"
#elif L==8	// pl
	"Sort by number of times the question"
	" has been answered (excluding blank answers)"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por n&ordm; de vezes"
	" que a pergunta foi respondida (n&atilde;o em branco)"
#elif L==10	// tr
	"Sort by number of times the question"
	" has been answered (excluding blank answers)"		// eviri lazim!
#endif
	,
	[Qst_ORDER_AVERAGE_SCORE_NOT_BLANK] =
#if   L==1	// ca
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Necessita traducci
#elif L==2	// de
	"Sortieren nach Durchschnittsnote (nicht leer)"
#elif L==3	// en
	"Sort by average score (excluding blank answers)"
#elif L==4	// es
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"
#elif L==5	// fr
	"Trier par note moyenne (non vide)"
#elif L==6	// gn
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per punteggio medio (non vuoto)"
#elif L==8	// pl
	"Sort by average score (excluding blank answers)"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por pontua&ccedil;&atilde;o m&eacute;dia (n&atilde;o em branco)"
#elif L==10	// tr
	"Sort by average score (excluding blank answers)"			// eviri lazim!
#endif
	};

const char *Txt_TST_STR_ORDER_SHORT[Qst_NUM_TYPES_ORDER_QST] =
	{
	[Qst_ORDER_STEM] =
#if   L==1	// ca
	"Pregunta"
#elif L==2	// de
	"Frage"
#elif L==3	// en
	"Question"
#elif L==4	// es
	"Pregunta"
#elif L==5	// fr
	"Question"
#elif L==6	// gn
	"Pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Domanda"
#elif L==8	// pl
	"Pytanie"
#elif L==9	// pt
	"Pergunta"
#elif L==10	// tr
	"Question"	// eviri lazim!
#endif
	,
	[Qst_ORDER_NUM_HITS] =
#if   L==1	// ca
	"N.&ordm;<br />resp."		// Necessita traducci
#elif L==2	// de
	"Anzahl<br />Antw."
#elif L==3	// en
	"No.<br />ans."
#elif L==4	// es
	"N.&ordm;<br />resp."
#elif L==5	// fr
	"Nombre<br />r&eacute;p."
#elif L==6	// gn
	"N.&ordm;<br />resp."		// Okoteve traduccin
#elif L==7	// it
	"Num.<br />risp."
#elif L==8	// pl
	"No.<br />ans."			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N.&ordm;<br />resp."
#elif L==10	// tr
	"No.<br />ans."			// eviri lazim!
#endif
	,
	[Qst_ORDER_AVERAGE_SCORE] =
#if   L==1	// ca
	"Puntuaci&oacute;<br />mitjana"
#elif L==2	// de
	"Durchschnitt"
#elif L==3	// en
	"Average<br />score"
#elif L==4	// es
	"Puntuaci&oacute;n<br />media"
#elif L==5	// fr
	"Note<br />moyenne"
#elif L==6	// gn
	"Puntuaci&oacute;n<br />media"	// Okoteve traduccin
#elif L==7	// it
	"Punteggio<br />medio"
#elif L==8	// pl
	"Average<br />score"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia"
#elif L==10	// tr
	"Average<br />score"		// eviri lazim!
#endif
	,
	[Qst_ORDER_NUM_HITS_NOT_BLANK] =
#if   L==1	// ca
	"N.&ordm;<br />resp.<br />no en<br />blanco"	// Necessita traducci
#elif L==2	// de
	"Anzahl<br />Antw.<br />nicht<br />leer"
#elif L==3	// en
	"No.<br />ans.<br />not<br />blank"
#elif L==4	// es
	"N.&ordm;<br />resp.<br />no en<br />blanco"
#elif L==5	// fr
	"Nombre<br />r&eacute;p.<br />non<br />vide"
#elif L==6	// gn
	"N.&ordm;<br />resp.<br />no en<br />blanco"	// Okoteve traduccin
#elif L==7	// it
	"Num.<br />risp.<br />non<br />vuoto"
#elif L==8	// pl
	"No.<br />ans.<br />not<br />blank"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N.&ordm;<br />resp.<br />n&atilde;o em<br />branco"
#elif L==10	// tr
	"No.<br />ans.<br />not<br />blank"		// eviri lazim!
#endif
	,
	[Qst_ORDER_AVERAGE_SCORE_NOT_BLANK] =
#if   L==1	// ca
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Necessita traducci
#elif L==2	// de
	"Durchschnitt<br />nicht<br />leer"
#elif L==3	// en
	"Average<br />score<br />not<br />blank"
#elif L==4	// es
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"
#elif L==5	// fr
	"Note<br />moyenne<br />non<br />vide"
#elif L==6	// gn
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Okoteve traduccin
#elif L==7	// it
	"Punteggio<br />medio<br />non<br />vuoto"
#elif L==8	// pl
	"Average<br />score<br />not<br />blank"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia<br />n&atilde;o em<br />branco"
#elif L==10	// tr
	"Average<br />score<br />not<br />blank"		// eviri lazim!
#endif
	};

const char *Txt_TST_STR_VISIBILITY[TstVis_NUM_ITEMS_VISIBILITY] =
	{
	[TstVis_VISIBLE_QST_ANS_TXT] =
#if   L==1	// ca
	"Text de preguntes i respostes"
#elif L==2	// de
	"Fragen und Antworten Text"
#elif L==3	// en
	"Questions and answers text"
#elif L==4	// es
	"Texto de preguntas y respuestas"
#elif L==5	// fr
	"Texte de questions et r&eacute;ponses."
#elif L==6	// gn
	"Texto de preguntas y respuestas"	// Okoteve traduccin
#elif L==7	// it
	"Testo di domande e risposte"
#elif L==8	// pl
	"Tekst pyta&nacute; i odpowiedzi"
#elif L==9	// pt
	"Texto de perguntas e respostas"
#elif L==10	// tr
	"Questions and answers text"		// eviri lazim!
#endif
	,
	[TstVis_VISIBLE_FEEDBACK_TXT] =
#if   L==1	// ca
	"Text de realimentaci&oacute;"
#elif L==2	// de
	"Feedback-Text"
#elif L==3	// en
	"Feedback text"
#elif L==4	// es
	"Texto de realimentaci&oacute;n"
#elif L==5	// fr
	"Texte de r&eacute;troaction"
#elif L==6	// gn
	"Texto de realimentaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Testo di feedback"
#elif L==8	// pl
	"Tekst feedback"
#elif L==9	// pt
	"Texto de feedback"
#elif L==10	// tr
	"Feedback text"				// eviri lazim!
#endif
	,
	[TstVis_VISIBLE_CORRECT_ANSWER] =
#if   L==1	// ca
	"Respostes correctes"
#elif L==2	// de
	"Richtige Antworten"
#elif L==3	// en
	"Correct answers"
#elif L==4	// es
	"Respuestas correctas"
#elif L==5	// fr
	"Bonnes r&eacute;ponses"
#elif L==6	// gn
	"Respuestas correctas"	// Okoteve traduccin
#elif L==7	// it
	"Risposte corrette"
#elif L==8	// pl
	"Prawid&lstrok;owe odpowiedzi"
#elif L==9	// pt
	"Respostas corretas"
#elif L==10	// tr
	"Correct answers"	// eviri lazim!
#endif
	,
	[TstVis_VISIBLE_EACH_QST_SCORE] =
#if   L==1	// ca
	"Puntuaci&oacute; de cada pregunta"
#elif L==2	// de
	"Punktzahl jeder Frage"
#elif L==3	// en
	"Score of each question"
#elif L==4	// es
	"Puntuaci&oacute;n de cada pregunta"
#elif L==5	// fr
	"Score de chaque question"
#elif L==6	// gn
	"Puntuaci&oacute;n de cada pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Punteggio di ogni domanda"
#elif L==8	// pl
	"Wynik ka&zdot;dego pytania"
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o de cada pergunta"
#elif L==10	// tr
	"Score of each question"		// eviri lazim!
#endif
	,
	[TstVis_VISIBLE_TOTAL_SCORE] =
#if   L==1	// ca
	"Puntuaci&oacute; total"
#elif L==2	// de
	"Gesamtpunktzahl"
#elif L==3	// en
	"Total score"
#elif L==4	// es
	"Puntuaci&oacute;n total"
#elif L==5	// fr
	"Score total"
#elif L==6	// gn
	"Puntuaci&oacute;n total"	// Okoteve traduccin
#elif L==7	// it
	"Punteggio totale"
#elif L==8	// pl
	"&Lstrok;&aogon;czny wynik"
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o total"
#elif L==10	// tr
	"Total score"			// eviri lazim!
#endif
	};

const char *Txt_TF_QST[2] =
	{
#if   L==1	// ca	// True
	"V"
#elif L==2	// de
	"R"
#elif L==3	// en
	"T"
#elif L==4	// es
	"V"
#elif L==5	// fr
	"V"
#elif L==6	// gn
	"V"	// Okoteve traduccin
#elif L==7	// it
	"V"
#elif L==8	// pl
	"P"
#elif L==9	// pt
	"V"
#elif L==10	// tr
	"T"	// eviri lazim!
#endif
	,
#if   L==1	// ca	// False
	"F"
#elif L==2	// de
	"F"
#elif L==3	// en
	"F"
#elif L==4	// es
	"F"
#elif L==5	// fr
	"F"
#elif L==6	// gn
	"F"	// Okoteve traduccin
#elif L==7	// it
	"F"
#elif L==8	// pl
	"F"
#elif L==9	// pt
	"F"
#elif L==10	// tr
	"F"	// eviri lazim!
#endif
	};

const char *Txt_Type =
#if   L==1	// ca
	"Tipus";
#elif L==2	// de
	"Art";
#elif L==3	// en
	"Type";
#elif L==4	// es
	"Tipo";
#elif L==5	// fr
	"Type";
#elif L==6	// gn
	"Teko";
#elif L==7	// it
	"Tipo";
#elif L==8	// pl
	"Typ";
#elif L==9	// pt
	"Tipo";
#elif L==10	// tr
	"Type";	// eviri lazim!
#endif

const char *Txt_Type_of_BR_answers =		// Answers of a test
#if   L==1	// ca
	"Tipus de<br />resposte";
#elif L==2	// de
	"Antworttype";
#elif L==3	// en
	"Type of<br />answers";
#elif L==4	// es
	"Tipo de<br />respuestas";
#elif L==5	// fr
	"Type de<br />r&eacute;ponses";
#elif L==6	// gn
	"Tipo de<br />respuestas";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di<br />risposte";
#elif L==8	// pl
	"Typu<br />odpowiedzi";
#elif L==9	// pt
	"Tipo de<br />respostas";
#elif L==10	// tr
	"Type of<br />answers";		// eviri lazim!
#endif

const char *Txt_Type_of_degree =
#if   L==1	// ca
	"Tipo de titulaci&oacute;n";	// Necessita traducci
#elif L==2	// de
	"Abschlussart";
#elif L==3	// en
	"Type of degree";
#elif L==4	// es
	"Tipo de titulaci&oacute;n";
#elif L==5	// fr
	"Type d'&eacute;tude";
#elif L==6	// gn
	"Tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di laurea";
#elif L==8	// pl
	"Type of degree";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de grau";
#elif L==10	// tr
	"Type of degree";		// eviri lazim!
#endif

const char *Txt_Type_of_degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Tipus de grau <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Art des Abschlusses <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Type of degree <strong>%s</strong> removed.";
#elif L==4	// es
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Type d'&eacute;tude <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di laurea <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Usuni&eogon;to rodzaj stopnia <strong>%s</strong>.";
#elif L==9	// pt
	"Tipo de grau <strong>%s</strong> removido.";
#elif L==10	// tr
	"Type of degree <strong>%s</strong> removed.";			// eviri lazim!
#endif

const char *Txt_Type_of_group =
#if   L==1	// ca
	"Tipus de grup";
#elif L==2	// de
	"Art der Gruppe";
#elif L==3	// en
	"Type of group";
#elif L==4	// es
	"Tipo de grupo";
#elif L==5	// fr
	"Type de groupe";
#elif L==6	// gn
	"Tipo de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di gruppo";
#elif L==8	// pl
	"Type of group";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de grupo";
#elif L==10	// tr
	"Type of group";	// eviri lazim!
#endif

const char *Txt_Type_BR_of_group =
#if   L==1	// ca
	"Tipus<br />de grup";
#elif L==2	// de
	"Art<br />der Gruppe";
#elif L==3	// en
	"Type<br />of group";
#elif L==4	// es
	"Tipo<br />de grupo";
#elif L==5	// fr
	"Type<br />de groupe";
#elif L==6	// gn
	"Tipo<br />de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Tipo<br />di gruppo";
#elif L==8	// pl
	"Type<br />of group";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo<br />de grupo";
#elif L==10	// tr
	"Type<br />of group";	// eviri lazim!
#endif

const char *Txt_Type_of_group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Tipus de grup <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Gruppen-Typ <strong>%s</strong> wurde entfernt.";
#elif L==3	// en
	"Type of group <strong>%s</strong> removed.";
#elif L==4	// es
	"Tipo de grupo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Type de groupe <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Tipo de grupo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di gruppo <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Usuni&eogon;to typ grupy <strong>%s</strong>.";
#elif L==9	// pt
	"Tipo de grupo <strong>%s</strong> removido.";
#elif L==10	// tr
	"Type of group <strong>%s</strong> removed.";	// eviri lazim!
#endif

const char *Txt_Type_or_paste_a_list_of_IDs_nicks_or_emails_ =
#if   L==1	// ca
	"Escriviu o enganxeu una llista"
	" de @nicks, correus electr&ograve;nics o IDs."
	" Els @nicks, correus electr&ograve;nics o IDs"
	" es poden separar per cadenes arbitr&agrave;ries"
	" (per exemple, noms, llocs, etc.)."
	" Nom&eacute;s es tindran en compte les paraules"
	" que sembli @nicks, correus electr&ograve;nics o IDs;"
	" la resta del text s'ignorar&agrave;.";
#elif L==2	// de
	"Geben oder f&uuml;gen Sie eine Liste"
	" mit @nicks, E-Mails oder IDs ein."
	" Die @nicks, E-Mails oder IDs"
	" k&ouml;nnen durch beliebige Zeichenfolgen"
	" (z. B. Namen, Orte usw.) getrennt werden."
	" Nur die W&ouml;rter, die wie @nicks, E-Mails oder IDs aussehen,"
	" werden ber&uuml;cksichtigt;"
	" der Rest des Textes wird ignoriert.";
#elif L==3	// en
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";
#elif L==4	// es
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";
#elif L==5	// fr
	"Tapez ou collez une liste"
	" de @nicks, e-mails ou identifiants."
	" Les @nicks, emails ou identifiants"
	" peuvent &ecirc;tre s&eacute;par&eacute;s par des chanes arbitraires"
	" (ex. noms, lieux, etc.)."
	" Seuls les mots qui semblent @nicks, emails ou identifiants"
	" seront pris en compte;"
	" le reste du texte sera ignor&eacute;.";
#elif L==6	// gn
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";		// Okoteve traduccin
#elif L==7	// it
	"Scrivi o incolla la lista"
	" di @nome-utenti, emails o IDs."
	" I @nome-utenti, emails o IDs"
	" possono essere separati da caratteri arbitrari"
	" (eg. nomi, luoghi, etc.)."
	" Solo le parole che sembrano"
	" @nome-utenti, emails o IDs"
	" saranno presi in considerazione;"
	" il resto del testo sar&agrave; ignorato.";
#elif L==8	// pl
	"Wpisz lub wklej list&eogon;"
	" @nick&oacute;w, e-maili lub ID."
	" @nicki, e-maile lub ID"
	" mog&aogon; by&cacute; oddzielone dowolnymi ci&aogon;gami"
	" (np. nazwy, miejsca itp.)."
	" Tylko s&lstrok;owa, kt&oacute;re wygl&aogon;daj&aogon; jak @nick, e-maile lub ID,"
	" b&eogon;d&aogon; brane pod uwag&eogon;;"
	" reszta tekstu zostanie zignorowana.";
#elif L==9	// pt
	"Digite ou cole uma lista"
	" de @alcunhas, emails ou IDs."
	" As @alcunhas, os emails ou os IDs"
	" podem ser separados por strings arbitr&aacute;rias"
	" (por exemplo, nomes, lugares, etc.)."
	" Apenas as palavras que parecem @alcunhas, emails ou IDs"
	" ser%atilde;o levadas em conta;"
	" o resto do texto ser&aacute; ignorado.";
#elif L==10	// tr
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// eviri lazim!
#endif

const char *Txt_Types_of_degree =
#if   L==1	// ca
	"Tipus de titulaci&oacute;";
#elif L==2	// de
	"Abschlussart";
#elif L==3	// en
	"Types of degree";
#elif L==4	// es
	"Tipos de titulaci&oacute;n";
#elif L==5	// fr
	"Types d'&eacute;tude";
#elif L==6	// gn
	"Arandur&atilde; l&aacute;ja";
#elif L==7	// it
	"Tipi di laurea";
#elif L==8	// pl
	"Rodzaje stopni";
#elif L==9	// pt
	"Tipos de grau";
#elif L==10	// tr
	"Types of degree";	// eviri lazim!
#endif

const char *Txt_Types_of_answers =		// Answers of a test
#if   L==1	// ca
	"Tipus de respostes";
#elif L==2	// de
	"Antworttypen";
#elif L==3	// en
	"Types of answers";
#elif L==4	// es
	"Tipos de respuestas";
#elif L==5	// fr
	"Types de r&eacute;ponses";
#elif L==6	// gn
	"Tipos de respuestas";	// Okoteve traduccin
#elif L==7	// it
	"Tipi di risposte";
#elif L==8	// pl
	"Typy odpowiedzi";
#elif L==9	// pt
	"Tipos de respostas";
#elif L==10	// tr
	"Types of answers";	// eviri lazim!
#endif

const char *Txt_Types_of_group =
#if   L==1	// ca
	"Tipus de grup";
#elif L==2	// de
	"Gruppen-Typen";
#elif L==3	// en
	"Types of group";
#elif L==4	// es
	"Tipos de grupo";
#elif L==5	// fr
	"Types de groupe";
#elif L==6	// gn
	"Tipos de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Tipi di gruppo";
#elif L==8	// pl
	"Types of group";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipos de grupo";
#elif L==10	// tr
	"Types of group";	// eviri lazim!
#endif

const char *Txt_Undo =
#if   L==1	// ca
	"Desfer";
#elif L==2	// de
	"R&uuml;ckg&auml;ngig";
#elif L==3	// en
	"Undo";
#elif L==4	// es
	"Deshacer";
#elif L==5	// fr
	"Annuler";
#elif L==6	// gn
	"Deshacer";	// Okoteve traduccin
#elif L==7	// it
	"Annulla";
#elif L==8	// pl
	"Anulowa&cacute;";
#elif L==9	// pt
	"Desfazer";
#elif L==10	// tr
	"Undo";		// eviri lazim!
#endif

const char *Txt_Unfollow =
#if   L==1	// ca
	"Deixar de seguir";
#elif L==2	// de
	"Entfolgen";
#elif L==3	// en
	"Unfollow";
#elif L==4	// es
	"Dejar de seguir";
#elif L==5	// fr
	"Se d&eacute;sabonner";
#elif L==6	// gn
	"Dejar de seguir";	// Okoteve traduccin
#elif L==7	// it
	"Smetti";
#elif L==8	// pl
	"Nie obserwuj";
#elif L==9	// pt
	"Deixar de seguir";
#elif L==10	// tr
	"Unfollow";		// eviri lazim!
#endif

const char *Txt_Unfortunately_Firefox_and_Chrome_no_longer_allow_Java_to_run_ =
#if   L==1	// ca
	"Desafortunadament Firefox i Chrome ja no permeten executar Java,"
	" de manera que el xat no estar&agrave; disponible per a aquests navegadors"
	" fins que desenvolupem un nou xat amb una altra tecnologia."
	" Sentim les mol&egrave;sties.";
#elif L==2	// de
	"Leider keinen Firefox und Chrome l&auml;nger erlauben Ihnen Java zu laufen,"
	" so dass der Chat f&uuml;r diesen Browser nicht zur Verf&uuml;gung steht,"
	" bis wir einen neuen Chat mit anderer Technologie zu entwickeln."
	" Es tut uns leid.";
#elif L==3	// en
	"Unfortunately Firefox and Chrome no longer allow Java to run,"
	" so chat will not be available for these browsers"
	" until we develop a new chat with another technology."
	" We apologize for the inconveniences.";
#elif L==4	// es
	"Desafortunadamente Firefox y Chrome ya no permiten ejecutar Java,"
	" de modo que el chat no estar&aacute; disponible para estos navegadores"
	" hasta que desarrollemos un nuevo chat con otra tecnolog&iacute;a."
	" Sentimos las molestias.";
#elif L==5	// fr
	"Malheureusement, pas Firefox et Chrome plus vous permettent d'ex&eacute;cuter Java,"
	" donc le chat n'est pas disponible pour ces navigateurs"
	" jusqu'&agrave; ce que nous d&eacute;veloppons un nouveau chat avec d'autres technologies."
	" D&eacute;sol&eacute; pour la g&ecirc;ne occasionn&eacute;e.";
#elif L==6	// gn
	"Desafortunadamente Firefox y Chrome ya no permiten ejecutar Java,"
	" de modo que el chat no estar&aacute; disponible para estos navegadores"
	" hasta que desarrollemos un nuevo chat con otra tecnolog&iacute;a."
	" Sentimos las molestias.";			// Okoteve traduccin
#elif L==7	// it
	"Purtroppo Firefox e Chrome non consentono pi&ugrave; di eseguire Java,"
	" quindi la chat non &egrave; disponibile per questi browser"
	" finch&eacute; non sviluppiamo una nuova chat con altre tecnologie."
	" Ci scusiamo per l'inconveniente.";
#elif L==8	// pl
	"Niestety Firefox i Chrome nie pozwalaj&aogon; ju&zdot; na uruchamianie Java,"
	" wi&eogon;c rozmowa nie jest dost&eogon;pny dla tych przegl&aogon;darek"
	" a&zdot; opracowa&cacute; nowy czat z innymi technologiami."
	" Przepraszamy za niedogodno&sacute;ci.";
#elif L==9	// pt
	"Infelizmente Firefox e Chrome j&aacute; n&atilde;o permitem que voc&ecirc; execute o Java,"
	" de modo que o chat n&atilde;o est&aacute; dispon&iacute;vel para estes navegadores"
	" at&eacute; que desenvolver um novo chat com outra tecnologia."
	" Desculpe o transtorno.";
#elif L==10	// tr
	"Unfortunately Firefox and Chrome no longer allow Java to run,"
	" so chat will not be available for these browsers"
	" until we develop a new chat with another technology."
	" We apologize for the inconveniences.";	// eviri lazim!
#endif

const char *Txt_unknown_assignment =
#if   L==1	// ca
	"activitat desconeguda";
#elif L==2	// de
	"Aufgabe unbekannt";
#elif L==3	// en
	"unknown assignment";
#elif L==4	// es
	"actividad desconocida";
#elif L==5	// fr
	"activit&eacute; inconnue";
#elif L==6	// gn
	"actividad desconocida";	// Okoteve traduccin
#elif L==7	// it
	"attivit&agrave; sconosciuta";
#elif L==8	// pl
	"nieznany przydzia&lstrok;";
#elif L==9	// pt
	"atividade desconhecida";
#elif L==10	// tr
	"bilinmeyen atama";
#endif

const char *Txt_Unknown_or_without_photo =
#if   L==1
	"Desconegut/da o sense fotografia";
#elif L==2
	"Unbekannt oder ohne Foto";
#elif L==3
	"Unknown or without photo";
#elif L==4
	"Desconocido/a o sin fotograf&iacute;a";
#elif L==5
	"Inconnu ou sans photo";
#elif L==6
	"Desconocido/a o sin fotograf&iacute;a";	// Okoteve traduccin
#elif L==7
	"Sconosciuto/a o senza foto";
#elif L==8
	"Nieznany lub bez zdj&eogon;cia";
#elif L==9
	"Desconhecido/a ou sem foto";
#elif L==10	// tr
	"Unknown or without photo";			// eviri lazim!
#endif

const char *Txt_Unknown_FORUM =
#if   L==1	// ca
	"Desconocido";	// Necessita traducci
#elif L==2	// de
	"Unbekannt";
#elif L==3	// en
	"Unknown";
#elif L==4	// es
	"Desconocido";
#elif L==5	// fr
	"Inconnu";
#elif L==6	// gn
	"Desconocido";	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto";
#elif L==8	// pl
	"Unknown";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Desconhecido";
#elif L==10	// tr
	"Unknown";	// eviri lazim!
#endif

const char *Txt_unknown_recipient =
#if   L==1	// ca
	"destinatario desconocido";	// Necessita traducci
#elif L==2	// de
	"Empf&auml;nger unbekannt";
#elif L==3	// en
	"unknown recipient";
#elif L==4	// es
	"destinatario desconocido";
#elif L==5	// fr
	"destinataire inconnu";
#elif L==6	// gn
	"destinatario desconocido";	// Okoteve traduccin
#elif L==7	// it
	"destinatario sconosciuto";
#elif L==8	// pl
	"unknown recipient";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"destinat&aacute;rio desconhecido";
#elif L==10	// tr
	"unknown recipient";		// eviri lazim!
#endif

const char *Txt_unknown_recipients =
#if   L==1	// ca
	"destinatarios desconocidos";	// Necessita traducci
#elif L==2	// de
	"Empf&auml;nger unbekannt";
#elif L==3	// en
	"unknown recipients";
#elif L==4	// es
	"destinatarios desconocidos";
#elif L==5	// fr
	"destinataires inconnus";
#elif L==6	// gn
	"destinatarios desconocidos";	// Okoteve traduccin
#elif L==7	// it
	"destinatari sconosciuti";
#elif L==8	// pl
	"unknown recipients";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"destinat&aacute;rios desconhecidos";
#elif L==10	// tr
	"unknown recipients";		// eviri lazim!
#endif

const char *Txt_unknown_removed_course =
#if   L==1	// ca
	"assignatura desconeguda / eliminada";
#elif L==2	// de
	"Kurs unbekannt / entfernt";
#elif L==3	// en
	"unknown / removed course";
#elif L==4	// es
	"asignatura desconocida / eliminada";
#elif L==5	// fr
	"mati&egrave;re inconnue / supprim&eacute;e";
#elif L==6	// gn
	"asignatura desconocida / eliminada";	// Okoteve traduccin
#elif L==7	// it
	"corso sconosciuto / rimosso";
#elif L==8	// pl
	"kurs nieznany / usuni&eogon;te";
#elif L==9	// pt
	"disciplina desconhecida / removida";
#elif L==10	// tr
	"unknown / removed course";		// eviri lazim!
#endif

const char *Txt_unknown_TIME =
#if   L==1	// ca
	"desconocido";	// Necessita traducci
#elif L==2	// de
	"unbekannt";
#elif L==3	// en
	"unknown";
#elif L==4	// es
	"desconocido";
#elif L==5	// fr
	"inconnu";
#elif L==6	// gn
	"desconocido";	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto";
#elif L==8	// pl
	"unknown";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"desconhecido";
#elif L==10	// tr
	"unknown";	// eviri lazim!
#endif

const char *Txt_Unlock_editing =
#if   L==1	// ca
	"Desbloquejar edici&oacute;";
#elif L==2	// de
	"Bearbeitung freischalten";
#elif L==3	// en
	"Unlock editing";
#elif L==4	// es
	"Desbloquear edici&oacute;n";
#elif L==5	// fr
	"D&eacute;verrouiller l'&eacute;dition";
#elif L==6	// gn
	"Desbloquear edici&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Sblocca la modifica";
#elif L==8	// pl
	"Odblokuj edycj&eogon;";
#elif L==9	// pt
	"Desbloquear edi&ccedil;&atilde;o";
#elif L==10	// tr
	"Unlock editing";		// eviri lazim!
#endif

const char *Txt_unread_MESSAGE =
#if   L==1	// ca
	"no le&iacute;do";	// Necessita traducci
#elif L==2	// de
	"ungelesene";
#elif L==3	// en
	"unread";
#elif L==4	// es
	"no le&iacute;do";
#elif L==5	// fr
	"non lu";
#elif L==6	// gn
	"no le&iacute;do";	// Okoteve traduccin
#elif L==7	// it
	"non letto";
#elif L==8	// pl
	"unread";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"n&atilde;o lida";
#elif L==10	// tr
	"unread";		// eviri lazim!
#endif

const char *Txt_unread_MESSAGES =
#if   L==1	// ca
	"no le&iacute;dos";	// Necessita traducci
#elif L==2	// de
	"ungelesene";
#elif L==3	// en
	"unread";
#elif L==4	// es
	"no le&iacute;dos";
#elif L==5	// fr
	"non lus";
#elif L==6	// gn
	"no le&iacute;dos";	// Okoteve traduccin
#elif L==7	// it
	"non letti";
#elif L==8	// pl
	"unread";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"n&atilde;o lidas";
#elif L==10	// tr
	"unread";		// eviri lazim!
#endif

const char *Txt_Unread_BR_msgs =
#if   L==1	// ca
	"No<br />le&iacute;dos";	// Necessita traducci
#elif L==2	// de
	"Ungel.<br />Nachr.";
#elif L==3	// en
	"Unread<br />msgs.";
#elif L==4	// es
	"No<br />le&iacute;dos";
#elif L==5	// fr
	"Non<br />lus";
#elif L==6	// gn
	"No<br />le&iacute;dos";	// Okoteve traduccin
#elif L==7	// it
	"Non<br />letti";
#elif L==8	// pl
	"Unread<br />msgs.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o<br />lidas";
#elif L==10	// tr
	"Unread<br />msgs.";		// eviri lazim!
#endif

const char *Txt_Update =
#if   L==1	// ca
	"Actualitzar";
#elif L==2	// de
	"Update";
#elif L==3	// en
	"Update";
#elif L==4	// es
	"Actualizar";
#elif L==5	// fr
	"Mise &agrave; jour";
#elif L==6	// gn
	"Actualizar";	// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento";
#elif L==8	// pl
	"Update";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Atualizar";
#elif L==10	// tr
	"Update";	// eviri lazim!
#endif

const char *Txt_Update_attendance =
#if   L==1	// ca
	"Actualitzar assist&egrave;ncia";
#elif L==2	// de
	"Update Anwesenheit";
#elif L==3	// en
	"Update attendance";
#elif L==4	// es
	"Actualizar asistencia";
#elif L==5	// fr
	"Mise &agrave; jour pr&eacute;sence";
#elif L==6	// gn
	"Actualizar asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento presenza";
#elif L==8	// pl
	"Aktualizacja bytno&sacute;&cacute;";
#elif L==9	// pt
	"Atualizar presen&ccedil;a";
#elif L==10	// tr
	"Update attendance";		// eviri lazim!
#endif

const char *Txt_Update_messages =
#if   L==1	// ca
	"Actualitzar missatges";
#elif L==2	// de
	"Update Nachrichten";
#elif L==3	// en
	"Update messages";
#elif L==4	// es
	"Actualizar mensajes";
#elif L==5	// fr
	"Mise &agrave; jour des messages";
#elif L==6	// gn
	"Actualizar mensajes";		// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento messaggi";
#elif L==8	// pl
	"Wiadomo&sacute;ci aktualizacji";
#elif L==9	// pt
	"Atualizar mensagens";
#elif L==10	// tr
	"Update messages";		// eviri lazim!
#endif

const char *Txt_Update_results =
#if   L==1	// ca
	"Actualitzar resultats";
#elif L==2	// de
	"Update Ergebnisse";
#elif L==3	// en
	"Update results";
#elif L==4	// es
	"Actualizar resultados";
#elif L==5	// fr
	"Mise &agrave; jour des r&eacute;sultats";
#elif L==6	// gn
	"Actualizar resultados";	// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento risultati";
#elif L==8	// pl
	"Zaktualizuj wyniki mecze";
#elif L==9	// pt
	"Atualizar os resultados";
#elif L==10	// tr
	"Update results";		// eviri lazim!
#endif

const char *Txt_Update_users =
#if   L==1	// ca
	"Actualitzar usuaris";
#elif L==2	// de
	"Update Benutzer";
#elif L==3	// en
	"Update users";
#elif L==4	// es
	"Actualizar usuarios";
#elif L==5	// fr
	"Mise &agrave; jour des &eacute;tudiants";
#elif L==6	// gn
	"Actualizar usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento utenti";
#elif L==8	// pl
	"Zaktualizuj u&zdot;ytkownik&oacute;w";
#elif L==9	// pt
	"Atualizar os utilizadores";
#elif L==10	// tr
	"Update users";		// eviri lazim!
#endif

const char *Txt_UPLOAD_FILE_Invalid_link =
#if   L==1	// ca
	"Enlla&ccedil; no v&agrave;lid.";
#elif L==2	// de
	"Ung&uuml;ltiger Link.";
#elif L==3	// en
	"Invalid link.";
#elif L==4	// es
	"Enlace no v&aacute;lido.";
#elif L==5	// fr
	"Lien invalide.";
#elif L==6	// gn
	"Enlace no v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"Link non valido.";
#elif L==8	// pl
	"Niepoprawny link.";
#elif L==9	// pt
	"Link n&atilde;o v&aacute;lido.";
#elif L==10	// tr
	"Invalid link.";		// eviri lazim!
#endif

const char *Txt_UPLOAD_FILE_Invalid_name =
#if   L==1	// ca
	"Nom no v&agrave;lid.";
#elif L==2	// de
	"Ung&uuml;ltiger Name.";
#elif L==3	// en
	"Invalid name.";
#elif L==4	// es
	"Nombre no v&aacute;lido.";
#elif L==5	// fr
	"Nom invalide.";
#elif L==6	// gn
	"Nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"Nome non valido.";
#elif L==8	// pl
	"B&lstrok;&eogon;dna nazwa.";
#elif L==9	// pt
	"Nome n&atilde;o v&aacute;lido.";
#elif L==10	// tr
	"Invalid name.";		// eviri lazim!
#endif

const char *Txt_UPLOAD_FILE_X_invalid_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong>: nom no v&agrave;lid.";
#elif L==2	// de
	"<strong>%s</strong>: ung&uuml;ltiger Name.";
#elif L==3	// en
	"<strong>%s</strong>: invalid name.";
#elif L==4	// es
	"<strong>%s</strong>: nombre no v&aacute;lido.";
#elif L==5	// fr
	"<strong>%s</strong>: nom invalide.";
#elif L==6	// gn
	"<strong>%s</strong>: nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong>: nome non valido.";
#elif L==8	// pl
	"<strong>%s</strong>: b&lstrok;&eogon;dna nazwa.";
#elif L==9	// pt
	"<strong>%s</strong>: nome n&atilde;o v&aacute;lido.";
#elif L==10	// tr
	"<strong>%s</strong>: invalid name.";			// eviri lazim!
#endif

const char *Txt_Upload_file =
#if   L==1	// ca
	"Enviar arxiu";
#elif L==2	// de
	"Upload Datei";
#elif L==3	// en
	"Upload file";
#elif L==4	// es
	"Enviar archivo";
#elif L==5	// fr
	"T&eacute;l&eacute;charger fichier";
#elif L==6	// gn
	"Enviar archivo";	// Okoteve traduccin
#elif L==7	// it
	"Caricare file";
#elif L==8	// pl
	"Przesy&lstrok;anie pliku";
#elif L==9	// pt
	"Fazer upload de arquivo";
#elif L==10	// tr
	"Upload file";		// eviri lazim!
#endif

const char *Txt_Upload_files =
#if   L==1	// ca
	"Enviar arxius";
#elif L==2	// de
	"Upload Dateien";
#elif L==3	// en
	"Upload files";
#elif L==4	// es
	"Subir archivos";
#elif L==5	// fr
	"T&eacute;l&eacute;charger fichiers";
#elif L==6	// gn
	"Subir archivos";	// Okoteve traduccin
#elif L==7	// it
	"Caricare file";
#elif L==8	// pl
	"Prze&sacute;lij pliki";
#elif L==9	// pt
	"Fazer upload de arquivos";
#elif L==10	// tr
	"Upload files";		// eviri lazim!
#endif

const char *Txt_Upload_my_picture =
#if   L==1	// ca
	"Pujar la meva fotografia";
#elif L==2	// de
	"Hochladen Mein Foto";
#elif L==3	// en
	"Upload my picture";
#elif L==4	// es
	"Subir mi fotograf&iacute;a";
#elif L==5	// fr
	"Ajouter ma photo";
#elif L==6	// gn
	"Subir mi fotograf&iacute;a";	// Okoteve traduccin
#elif L==7	// it
	"Caricare una foto";
#elif L==8	// pl
	"Prze&sacute;lij swoje zdj&eogon;cia";
#elif L==9	// pt
	"Carregar minha foto";
#elif L==10	// tr
	"Upload my picture";		// eviri lazim!
#endif

const char *Txt_Upload_photo =
#if   L==1	// ca
	"Enviar foto";
#elif L==2	// de
	"Upload Foto";
#elif L==3	// en
	"Upload photo";
#elif L==4	// es
	"Subir foto";
#elif L==5	// fr
	"T&eacute;l&eacute;charger photo";
#elif L==6	// gn
	"Subir foto";	// Okoteve traduccin
#elif L==7	// it
	"Caricare foto";
#elif L==8	// pl
	"Prze&sacute;lij zdj&eogon;cie";
#elif L==9	// pt
	"Upload foto";
#elif L==10	// tr
	"Upload photo";	// eviri lazim!
#endif

const char *Txt_Uploaded_by =
#if   L==1	// ca
	"Penjat per";
#elif L==2	// de
	"Hochgeladen von";
#elif L==3	// en
	"Uploaded by";
#elif L==4	// es
	"Subido por";
#elif L==5	// fr
	"Post&eacute; par";
#elif L==6	// gn
	"Subido por";	// Okoteve traduccin
#elif L==7	// it
	"Caricato da";
#elif L==8	// pl
	"Wyslane przez";
#elif L==9	// pt
	"Enviado por";
#elif L==10	// tr
	"Uploaded by";	// eviri lazim!
#endif

const char *Txt_URL =		// Uniform Resource Locator
#if   L==1	// ca
	"URL";
#elif L==2	// de
	"URL";
#elif L==3	// en
	"URL";
#elif L==4	// es
	"URL";
#elif L==5	// fr
	"URL";
#elif L==6	// gn
	"URL";	// Okoteve traduccin
#elif L==7	// it
	"URL";
#elif L==8	// pl
	"URL";
#elif L==9	// pt
	"URL";
#elif L==10	// tr
	"URL";	// eviri lazim!
#endif

const char *Txt_Use_this_email =
#if   L==1	// ca
	"Utilitzar aquest correu";
#elif L==2	// de
	"Verwenden Sie diese E-Mail";
#elif L==3	// en
	"Use this email";
#elif L==4	// es
	"Usar este correo";
#elif L==5	// fr
	"Utiliser ce courrier";
#elif L==6	// gn
	"Usar este correo";	// Okoteve traduccin
#elif L==7	// it
	"Utilizzare questa email";
#elif L==8	// pl
	"U&zdot;yj tej email";
#elif L==9	// pt
	"Utilizar este email";
#elif L==10	// tr
	"Use this email";	// eviri lazim!
#endif

const char *Txt_Use_this_nickname =
#if   L==1	// ca
	"Utilitzar aquest sobrenom";
#elif L==2	// de
	"Verwenden Sie diese Benutzername";
#elif L==3	// en
	"Use this nickname";
#elif L==4	// es
	"Usar este apodo";
#elif L==5	// fr
	"Utiliser ce surnom";
#elif L==6	// gn
	"Usar este apodo";	// Okoteve traduccin
#elif L==7	// it
	"Utilizzare questo nome utente";
#elif L==8	// pl
	"U&zdot;yj tej nick";
#elif L==9	// pt
	"Utilizar esta alcunha";
#elif L==10	// tr
	"Use this nickname";	// eviri lazim!
#endif

const char *Txt_User[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"Usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuaria"
#elif L==5	// fr
	"Utilisateuse"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizadora"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"Usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario"
#elif L==5	// fr
	"Utilisateur"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#elif L==10	// tr
	"User"	// eviri lazim!
#endif
	};

const char *Txt_user[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuaria"
#elif L==5	// fr
	"utilisateuse"
#elif L==6	// gn
	"usuaria"	// Okoteve traduccin
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizadora"
#elif L==10	// tr
	"user"		// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario"
#elif L==5	// fr
	"utilisateur"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#elif L==10	// tr
	"user"	// eviri lazim!
#endif
	};

const char *Txt_USER_comments =
#if   L==1	// ca
	"Observacions";
#elif L==2	// de
	"Kommentare";
#elif L==3	// en
	"Comments";
#elif L==4	// es
	"Observaciones";
#elif L==5	// fr
	"Commentaires";
#elif L==6	// gn
	"Observaciones";	// Okoteve traduccin
#elif L==7	// it
	"Commenti";
#elif L==8	// pl
	"Comments";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Coment&aacute;rios";
#elif L==10	// tr
	"Comments";		// eviri lazim!
#endif

const char *Txt_User_ID =
#if   L==1	// ca
	"ID (DNI/c&egrave;dula) de l'usuari";
#elif L==2	// de
	"Ausweis-Nr.";
#elif L==3	// en
	"User ID";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) del usuario";
#elif L==5	// fr
	"Num&eacute;ro d'identit&eacute; d'utilisateur";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) del usuario";		// Okoteve traduccin
#elif L==7	// it
	"Numero carta d'identit&agrave; utente";
#elif L==8	// pl
	"User ID";					// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N.&ordm; de identif. do utilizador";
#elif L==10	// tr
	"User ID";					// eviri lazim!
#endif

const char *Txt_USER_in_COURSE =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"%s en";
#elif L==2	// de
	"%s im";
#elif L==3	// en
	"%s in";
#elif L==4	// es
	"%s en";
#elif L==5	// fr
	"%s dans";
#elif L==6	// gn
	"%s en";	// Okoteve traduccin
#elif L==7	// it
	"%s in";
#elif L==8	// pl
	"%s w";
#elif L==9	// pt
	"%s em";
#elif L==10	// tr
	"%s in";	// eviri lazim!
#endif

const char *Txt_User_not_found_or_you_do_not_have_permission_ =
#if   L==1	// ca
	"Usuari no trobat"
	" o b&eacute; vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==2	// de
	"Benutzer nicht gefunden"
	" oder Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==3	// en
	"User not found"
	" or you do not have permission"
	" to perform this action.";
#elif L==4	// es
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==5	// fr
	"Utilisateur introuvable"
	" ou vous n'avez pas l'autorisation"
	" d'effectuer cette action.";
#elif L==6	// gn
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"IUtente non trovato"
	" o non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==8	// pl
	"User not found"
	" or you do not have permission"
	" to perform this action.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Utilizador n&atilde;o encontrado"
	" ou voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#elif L==10	// tr
	"User not found"
	" or you do not have permission"
	" to perform this action.";	// eviri lazim!
#endif

const char *Txt_User_photos =
#if   L==1	// ca
	"Fotos d'usuaris";
#elif L==2	// de
	"Benutzerfotos";
#elif L==3	// en
	"User photos";
#elif L==4	// es
	"Fotos de usuarios";
#elif L==5	// fr
	"Photos des utilisateurs";
#elif L==6	// gn
	"Fotos de usuarios";		// Okoteve traduccin
#elif L==7	// it
	"Foto utente";
#elif L==8	// pl
	"Zdj&eogon;cia u&zdot;ytkownika ";
#elif L==9	// pt
	"Fotos de utilizadores";
#elif L==10	// tr
	"Kullan&inodot;c&inodot; foto&gbreve;raflar&inodot;";
#endif

const char *Txt_Users =
#if   L==1	// ca
	"Usuaris";
#elif L==2	// de
	"Benutzer";
#elif L==3	// en
	"Users";
#elif L==4	// es
	"Usuarios";
#elif L==5	// fr
	"Utilisateurs";
#elif L==6	// gn
	"Puruh&aacute;ra";
#elif L==7	// it
	"Utenti";
#elif L==8	// pl
	"U&zdot;ytkownicy";
#elif L==9	// pt
	"Utilizadores";
#elif L==10	// tr
	"Users";	// eviri lazim!
#endif

const char *Txt_users[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"usuarias"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarias"
#elif L==5	// fr
	"utilisateuses"
#elif L==6	// gn
	"usuarias"	// Okoteve traduccin
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadoras"
#elif L==10	// tr
	"users"		// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#elif L==10	// tr
	"users"	// eviri lazim!
#endif
	};

const char *Txt_X_users_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'ha(n) eliminat %u usuari(s).";
#elif L==2	// de
	"%u Benutzer wurden eliminiert.";
#elif L==3	// en
	"%u user(s) have been eliminated.";
#elif L==4	// es
	"Se ha(n) eliminado %u usuario(s).";
#elif L==5	// fr
	"%u utilisateur(s) ont &eacute;t&eacute; &eacute;limin&eacute;s.";
#elif L==6	// gn
	"Se ha(n) eliminado %u usuario(s).";	// Okoteve traduccin
#elif L==7	// it
	"%u utenti sono stati rimossi.";
#elif L==8	// pl
	"Wyeliminowano %u u&zdot;ytkownik&oacute;w.";
#elif L==9	// pt
	"%u usu&aacute;rio(s) foram eliminados.";
#elif L==10	// tr
	"%u user(s) have been eliminated.";	// eviri lazim!
#endif

const char *Txt_X_users_have_been_enroled_including_possible_repetitions =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han inscrit %u usuaris"
	" (incloent possibles repeticions).";
#elif L==2	// de
	"Es wurden %u Benutzer eingeschrieben"
	" (einschlie&szlig;lich m&ouml;glicher Wiederholungen).";
#elif L==3	// en
	"%u users have been enroled"
	" (including possible repetitions).";
#elif L==4	// es
	"Se han inscrito %u usuarios"
	" (incluyendo posibles repeticiones).";
#elif L==5	// fr
	"%u utilisateurs ont &eacute;t&eacute; inscrits"
	" (y compris les r&eacute;p&eacute;titions possibles).";
#elif L==6	// gn
	"Se han inscrito %u usuarios"
	" (incluyendo posibles repeticiones).";	// Okoteve traduccin
#elif L==7	// it
	"%u utenti sono stati registrati"
	" (comprese le eventuali ripetizioni).";
#elif L==8	// pl
	"Zarejestrowano %u u&zdot;ytkownik&oacute;w"
	" (wliczaj&aogon;c mo&zdot;liwe powt&oacute;rki).";
#elif L==9	// pt
	"%u usu&aacute;rios foram inscritos"
	" (incluindo poss&iacute;veis repeti&ccedil;&otilde;es).";
#elif L==10	// tr
	"%u users have been enroled"
	" (including possible repetitions).";	// eviri lazim!
#endif

const char *Txt_X_users_have_been_removed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han eliminat %u usuaris.";
#elif L==2	// de
	"%u Benutzer wurden entfernt.";
#elif L==3	// en
	"%u users have been removed.";
#elif L==4	// es
	"Se han eliminado %u usuarios.";
#elif L==5	// fr
	"%u utilisateurs ont &eacute;t&eacute; supprim&eacute;s.";
#elif L==6	// gn
	"Se han eliminado %u usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"%u utenti sono stati rimossi.";
#elif L==8	// pl
	"Usuni&eogon;to %u u&zdot;ytkownik&oacute;w.";
#elif L==9	// pt
	"%u usu&aacute;rios foram removidos.";
#elif L==10	// tr
	"%u users have been removed.";		// eviri lazim!
#endif

const char *Txt_Users_in_courses =
#if   L==1	// ca
	"Usuaris en assignatures";
#elif L==2	// de
	"Benutzer in Kursen";
#elif L==3	// en
	"Users in courses";
#elif L==4	// es
	"Usuarios en asignaturas";
#elif L==5	// fr
	"Utilisateurs dans mati&egrave;res";
#elif L==6	// gn
	"Puruh&aacute;ra en asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Utenti nei corsi";
#elif L==8	// pl
	"U&zdot;ytkownicy kurs&oacute;w";
#elif L==9	// pt
	"Utilizadores em disciplinas";
#elif L==10	// tr
	"Users in courses";			// eviri lazim!
#endif

const char *Txt_Users_of_the_center =
#if   L==1	// ca
	"Usuaris del center";
#elif L==2	// de
	"Benutzer der Lehrinstitut";
#elif L==3	// en
	"Users of the center";
#elif L==4	// es
	"Usuarios del centro";
#elif L==5	// fr
	"Utilisateurs du center";
#elif L==6	// gn
	"Usuarios del centro";	// Okoteve traduccin
#elif L==7	// it
	"Utenti del centro";
#elif L==8	// pl
	"U&zdot;ytkownicy centrum";
#elif L==9	// pt
	"Utilizadores do centro";
#elif L==10	// tr
	"Users of the center";	// eviri lazim!
#endif

const char *Txt_Users_of_the_country =
#if   L==1	// ca
	"Usuaris del pa&iacute;s";
#elif L==2	// de
	"Benutzer des Landes";
#elif L==3	// en
	"Users of the country";
#elif L==4	// es
	"Usuarios del pa&iacute;s";
#elif L==5	// fr
	"Utilisateurs du pays";
#elif L==6	// gn
	"Usuarios del pa&iacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Utenti del paese";
#elif L==8	// pl
	"U&zdot;ytkownicy kraju";
#elif L==9	// pt
	"Utilizadores do pa&iacute;s";
#elif L==10	// tr
	"Users of the country";		// eviri lazim!
#endif

const char *Txt_Users_of_the_institution =
#if   L==1	// ca
	"Usuaris de la instituci&oacute;";
#elif L==2	// de
	"Benutzer der Hochschule";
#elif L==3	// en
	"Users of the institution";
#elif L==4	// es
	"Usuarios de la instituci&oacute;n";
#elif L==5	// fr
	"Utilisateurs de l'&eacute;tablissement";
#elif L==6	// gn
	"Usuarios de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Utenti della istituzione";
#elif L==8	// pl
	"U&zdot;ytkownicy instytucji";
#elif L==9	// pt
	"Utilizadores da institu&ccedil;&atilde;o";
#elif L==10	// tr
	"Users of the institution";		// eviri lazim!
#endif

const char *Txt_users_with_no_group =
#if   L==1	// ca
	"usuaris en cap grup";
#elif L==2	// de
	"Benutzer mit nicht-Gruppe";
#elif L==3	// en
	"users with no group";
#elif L==4	// es
	"usuarios en ning&uacute;n grupo";
#elif L==5	// fr
	"utilisateurs avec aucun groupe";
#elif L==6	// gn
	"usuarios en ning&uacute;n grupo";	// Okoteve traduccin
#elif L==7	// it
	"utenti con nessun gruppo";
#elif L==8	// pl
	"u&zdot;ytkownik&oacute;w w ka&zdot;dej grupie";
#elif L==9	// pt
	"utilizadores com nenhum grupo";
#elif L==10	// tr
	"users with no group";			// eviri lazim!
#endif

const char *Txt_X_users_who_have_accepted_and_who_have_email =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"%u usuaris que han aceptado la inscripci&oacute;n"
	" y que tienen correo";		// Necessita traducci
#elif L==2	// de
	"%u Benutzer die akzeptiert wurden"
	" und die E-Mail";
#elif L==3	// en
	"%u users who have accepted"
	" and who have email";
#elif L==4	// es
	"%u usuarios que han aceptado la inscripci&oacute;n"
	" y que tienen correo";
#elif L==5	// fr
	"%u utilisateurs qui ont accept&eacute;"
	" et qui ont l'email";
#elif L==6	// gn
	"%u usuarios que han aceptado la inscripci&oacute;n"
	" y que tienen correo";		// Okoteve traduccin
#elif L==7	// it
	"%u utenti che hanno aderito"
	" e che hanno email";
#elif L==8	// pl
	"%u u&zdot;ytkownik&oacute;w who have accepted"
	" and who have email";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u utilizadores que aceitaram"
	" e que t&ecirc;m email";
#elif L==10	// tr
	"%u users who have accepted"
	" and who have email";		// eviri lazim!
#endif

const char *Txt_X_users_who_have_email =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"%u usuaris con correo";			// Necessita traducci
#elif L==2	// de
	"%u Benutzer mit E-Mail";
#elif L==3	// en
	"%u users who have email";
#elif L==4	// es
	"%u usuarios con correo";
#elif L==5	// fr
	"%u utilisateurs qui ont l'email";
#elif L==6	// gn
	"%u usuarios con correo";			// Okoteve traduccin
#elif L==7	// it
	"%u utenti che hanno email";
#elif L==8	// pl
	"%u u&zdot;ytkownik&oacute;w who have email";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u utilizadores que t&ecirc;m email";
#elif L==10	// tr
	"%u users who have email";			// eviri lazim!
#endif

const char *Txt_USR_LIST_TYPES[Set_NUM_USR_LIST_TYPES] =
	{
	[Set_USR_LIST_UNKNOWN] =
	""
	,
	[Set_USR_LIST_AS_CLASS_PHOTO] =
#if   L==1	// ca
	"Orla"
#elif L==2	// de
	"Klass Foto"
#elif L==3	// en
	"Class photo"
#elif L==4	// es
	"Orla"
#elif L==5	// fr
	"Photo de classe"
#elif L==6	// gn
	"Orla"		// Okoteve traduccin
#elif L==7	// it
	"Foto di classe"
#elif L==8	// pl
	"Class photo"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Foto de formatura"
#elif L==10	// tr
	"Class photo"	// eviri lazim!
#endif
	,
	[Set_USR_LIST_AS_LISTING] =
#if   L==1	// ca
	"Llista"
#elif L==2	// de
	"Liste"
#elif L==3	// en
	"List"
#elif L==4	// es
	"Lista"
#elif L==5	// fr
	"Liste"
#elif L==6	// gn
	"Lista"	// Okoteve traduccin
#elif L==7	// it
	"Lista"
#elif L==8	// pl
	"List"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Lista"
#elif L==10	// tr
	"List"	// eviri lazim!
#endif
	};

const char *Txt_Vacants =
#if   L==1	// ca
	"Vacants";
#elif L==2	// de
	"Frei";
#elif L==3	// en
	"Vacants";
#elif L==4	// es
	"Vacantes";
#elif L==5	// fr
	"Vacants";
#elif L==6	// gn
	"Vacantes";	// Okoteve traduccin
#elif L==7	// it
	"Liberi";
#elif L==8	// pl
	"Wolne";
#elif L==9	// pt
	"Desocupados";
#elif L==10	// tr
	"Vacants";	// eviri lazim!
#endif

const char *Txt_Valid_answers =
#if   L==1	// ca
	"Respostes v&agrave;lides";
#elif L==2	// de
	"G&uuml;ltige Antworten";
#elif L==3	// en
	"Valid answers";
#elif L==4	// es
	"Respuestas v&aacute;lidas";
#elif L==5	// fr
	"R&eacute;ponses valides";
#elif L==6	// gn
	"Respuestas v&aacute;lidas";	// Okoteve traduccin
#elif L==7	// it
	"Risposte valide";
#elif L==8	// pl
	"Prawid&lstrok;owe odpowiedzi";
#elif L==9	// pt
	"Respostas v&aacute;lidas";
#elif L==10	// tr
	"Valid answers";		// eviri lazim!
#endif

const char *Txt_valid_grade =
#if   L==1	// ca
	"nota v&agrave;lida";
#elif L==2	// de
	"G&uuml;ltige Note";
#elif L==3	// en
	"valid grade";
#elif L==4	// es
	"nota v&aacute;lida";
#elif L==5	// fr
	"note valide";
#elif L==6	// gn
	"nota v&aacute;lida";	// Okoteve traduccin
#elif L==7	// it
	"voto valido";
#elif L==8	// pl
	"ocena wynik";
#elif L==9	// pt
	"nota v&aacute;lida";
#elif L==10	// tr
	"valid grade";		// eviri lazim!
#endif

const char *Txt_Valid_question =
#if   L==1	// ca
	"Pregunta v&agrave;lida";
#elif L==2	// de
	"G&uuml;ltige Frage";
#elif L==3	// en
	"Valid question";
#elif L==4	// es
	"Pregunta v&aacute;lida";
#elif L==5	// fr
	"Question valide";
#elif L==6	// gn
	"Pregunta v&aacute;lida";	// Okoteve traduccin
#elif L==7	// it
	"Domanda valida";
#elif L==8	// pl
	"Wa&zdot;ne pytanie";
#elif L==9	// pt
	"Pergunta v&aacute;lida";
#elif L==10	// tr
	"Valid question";		// eviri lazim!
#endif

const char *Txt_valid_score =
#if   L==1	// ca
	"puntuaci&oacute; v&agrave;lida";
#elif L==2	// de
	"G&uuml;ltige Ergebnis";
#elif L==3	// en
	"valid score";
#elif L==4	// es
	"puntuaci&oacute;n v&aacute;lida";
#elif L==5	// fr
	"note valide";
#elif L==6	// gn
	"puntuaci&oacute;n v&aacute;lida";	// Okoteve traduccin
#elif L==7	// it
	"punteggio valido";
#elif L==8	// pl
	"prawid&lstrok;owy wynik";
#elif L==9	// pt
	"pontua&ccedil;&atilde;o v&aacute;lida";
#elif L==10	// tr
	"valid score";				// eviri lazim!
#endif
/*
const char *Txt_Value =	// Score, quantity
#if   L==1	// ca
	"Valor";
#elif L==2	// de
	"Wert";
#elif L==3	// en
	"Value";
#elif L==4	// es
	"Valor";
#elif L==5	// fr
	"Valeur";
#elif L==6	// gn
	"Hepykue";
#elif L==7	// it
	"Valore";
#elif L==8	// pl
	"Warto&sacute;&cacute;";
#elif L==9	// pt
	"Valor";
#elif L==10	// tr
	"De&gbreve;er";
#endif
*/
const char *Txt_View_all_recipients =
#if   L==1	// ca
	"Veure tots els destinataris";
#elif L==2	// de
	"Anzeigen alle Empf&auml;nger";
#elif L==3	// en
	"View all recipients";
#elif L==4	// es
	"Ver todos los destinatarios";
#elif L==5	// fr
	"Voir tous les destinataires";
#elif L==6	// gn
	"Ver todos los destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Vedi tutti i destinatari";
#elif L==8	// pl
	"Zobacz wszystkich odbiorc&oacute;w";
#elif L==9	// pt
	"Ver todos os estinat&aacute;rios";
#elif L==10	// tr
	"View all recipients";		// eviri lazim!
#endif

const char *Txt_View_attendance =
#if   L==1	// ca
	"Veure assist&egrave;ncia";
#elif L==2	// de
	"Anzeigen Anwesenheit";
#elif L==3	// en
	"View attendance";
#elif L==4	// es
	"Ver asistencia";
#elif L==5	// fr
	"Voir pr&eacute;sence";
#elif L==6	// gn
	"Ver asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Vedi presenza";
#elif L==8	// pl
	"Zobacz obecno&sacute;&cacute;";
#elif L==9	// pt
	"Ver presen&ccedil;a";
#elif L==10	// tr
	"View attendance";	// eviri lazim!
#endif

const char *Txt_View_email_addresses =
#if   L==1	// ca
	"Veure adreces de correu";
#elif L==2	// de
	"E-Mail-Adressen anzeigen";
#elif L==3	// en
	"View email addresses";
#elif L==4	// es
	"Ver direcciones de correo";
#elif L==5	// fr
	"Voir les adresses email";
#elif L==6	// gn
	"Ver asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Vedi gli indirizzi email";
#elif L==8	// pl
	"Zobacz adresy e-mail";
#elif L==9	// pt
	"Ver endere&ccedil;os de email";
#elif L==10	// tr
	"View email addresses";	// eviri lazim!
#endif

const char *Txt_View_event =
#if   L==1	// ca
	"Veure esdeveniment";
#elif L==2	// de
	"Anzeigen Ereignis";
#elif L==3	// en
	"View event";
#elif L==4	// es
	"Ver evento";
#elif L==5	// fr
	"Voir &eacute;v&eacute;nement";
#elif L==6	// gn
	"Ver evento";	// Okoteve traduccin
#elif L==7	// it
	"Vedi evento";
#elif L==8	// pl
	"View event";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver evento";
#elif L==10	// tr
	"View event";	// eviri lazim!
#endif

const char *Txt_View_exam =
#if   L==1	// ca
	"Veure examen";
#elif L==2	// de
	"Anzeigen Pr&uuml;fung";
#elif L==3	// en
	"View exam";
#elif L==4	// es
	"Ver examen";
#elif L==5	// fr
	"Voir examen";
#elif L==6	// gn
	"Ver examen";	// Okoteve traduccin
#elif L==7	// it
	"Vedi esame";
#elif L==8	// pl
	"Zobacz egzaminu";
#elif L==9	// pt
	"Ver exame";
#elif L==10	// tr
	"View exam";	// eviri lazim!
#endif

const char *Txt_View_game =
#if   L==1	// ca
	"Veure joc";
#elif L==2	// de
	"Anzeigen Spiel";
#elif L==3	// en
	"View game";
#elif L==4	// es
	"Ver juego";
#elif L==5	// fr
	"Voir jeu";
#elif L==6	// gn
	"Ver juego";	// Okoteve traduccin
#elif L==7	// it
	"Vedi gioco";
#elif L==8	// pl
	"Zobacz gr&eogon;";
#elif L==9	// pt
	"Ver jogo";
#elif L==10	// tr
	"View game";	// eviri lazim!
#endif

const char *Txt_View_in_a_new_window =
#if   L==1	// ca
	"Mostrar en una finestra nova";
#elif L==2	// de
	"Ansicht in einem neuen Fenster";
#elif L==3	// en
	"View in a new window";
#elif L==4	// es
	"Mostrar en una ventana nueva";
#elif L==5	// fr
	"Voir dans une nouvelle fen&ecirc;tre";
#elif L==6	// gn
	"Mostrar en una ventana nueva";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza in una nuova finestra";
#elif L==8	// pl
	"View in a new window";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver em uma nova janela";
#elif L==10	// tr
	"View in a new window";		// eviri lazim!
#endif

const char *Txt_View_my_answer =
#if   L==1	// ca
	"Veure la meva resposta";
#elif L==2	// de
	"Meine Antwort anzeigen";
#elif L==3	// en
	"View my answer";
#elif L==4	// es
	"Ver mi respuesta";
#elif L==5	// fr
	"Voir ma r&eacute;ponse";
#elif L==6	// gn
	"Ver mi respuesta";	// Okoteve traduccin
#elif L==7	// it
	"Vedi la mia risposta";
#elif L==8	// pl
	"Zobacz moj&aogon; odpowied&zacute;";
#elif L==9	// pt
	"Ver minha resposta";
#elif L==10	// tr
	"View my answer";	// eviri lazim!
#endif

const char *Txt_View_projects =
#if   L==1	// ca
	"Veure projectes";
#elif L==2	// de
	"Anzeigen Projekte";
#elif L==3	// en
	"View projects";
#elif L==4	// es
	"Ver proyectos";
#elif L==5	// fr
	"Voir projets";
#elif L==6	// gn
	"Ver proyectos";	// Okoteve traduccin
#elif L==7	// it
	"Vedi progetti";
#elif L==8	// pl
	"Zobacz projekty";
#elif L==9	// pt
	"Ver projetos";
#elif L==10	// tr
	"View projects";	// eviri lazim!
#endif

const char *Txt_View_record_for_this_course =
#if   L==1	// ca
	"Veure fitxa en aquesta assignatura";
#elif L==2	// de
	"Anzeigen profil bei Kurs";
#elif L==3	// en
	"View record for this course";
#elif L==4	// es
	"Ver ficha en esta asignatura";
#elif L==5	// fr
	"Voir carte dans mati&egrave;re";
#elif L==6	// gn
	"Ver ficha en esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Vedi scheda per questo corso";
#elif L==8	// pl
	"Zobacz rekord na ten kurs";
#elif L==9	// pt
	"Ver cart&atilde;o em disciplina";
#elif L==10	// tr
	"View record for this course";	// eviri lazim!
#endif

const char *Txt_View_records =
#if   L==1	// ca
	"Veure fitxes";
#elif L==2	// de
	"Anzeigen Profile";
#elif L==3	// en
	"View records";
#elif L==4	// es
	"Ver fichas";
#elif L==5	// fr
	"Voir cartes";
#elif L==6	// gn
	"Ver fichas";	// Okoteve traduccin
#elif L==7	// it
	"Vedi schede";
#elif L==8	// pl
	"Zobacz rekordy";
#elif L==9	// pt
	"Ver cart&otilde;es";
#elif L==10	// tr
	"View records";	// eviri lazim!
#endif

const char *Txt_View_results =
#if   L==1	// ca
	"Veure resultats";
#elif L==2	// de
	"Anzeigen Ergebnisse";
#elif L==3	// en
	"View results";
#elif L==4	// es
	"Ver resultados";
#elif L==5	// fr
	"Voir r&eacute;sultats";
#elif L==6	// gn
	"Ver resultados";	// Okoteve traduccin
#elif L==7	// it
	"Vedi risultati";
#elif L==8	// pl
	"Poka&zdot; wyniki";
#elif L==9	// pt
	"Ver resultados";
#elif L==10	// tr
	"View results";		// eviri lazim!
#endif

const char *Txt_View_rubric =
#if   L==1	// ca
	"Veure r&uacute;brica";
#elif L==2	// de
	"Anzeigen Rubrik";
#elif L==3	// en
	"View rubric";
#elif L==4	// es
	"Ver r&uacute;brica";
#elif L==5	// fr
	"Voir rubrique";
#elif L==6	// gn
	"Ver r&uacute;brica";	// Okoteve traduccin
#elif L==7	// it
	"Vedi rubrica";
#elif L==8	// pl
	"Zobacz rubryk&eogon;";
#elif L==9	// pt
	"Ver rubrica";
#elif L==10	// tr
	"Puan anahtar&inodot;n&inodot; g&ouml;r&uuml;nt&uuml;le";
#endif

const char *Txt_View_survey =
#if   L==1	// ca
	"Veure enquesta";
#elif L==2	// de
	"Anzeigen Umfrage";
#elif L==3	// en
	"View survey";
#elif L==4	// es
	"Ver encuesta";
#elif L==5	// fr
	"Voir sondage";
#elif L==6	// gn
	"Ver encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Vedi sondaggio";
#elif L==8	// pl
	"Zobacz ankiet&eogon;";
#elif L==9	// pt
	"Ver inqu&eacute;rito";
#elif L==10	// tr
	"View survey";	// eviri lazim!
#endif

const char *Txt_View_homework =
#if   L==1	// ca
	"Veure treballs";
#elif L==2	// de
	"Anzeigen Hausarbeiten";
#elif L==3	// en
	"View homework";
#elif L==4	// es
	"Ver trabajos";
#elif L==5	// fr
	"Voir travaux";
#elif L==6	// gn
	"Ver trabajos";		// Okoteve traduccin
#elif L==7	// it
	"Vedi lavori";
#elif L==8	// pl
	"Zobacz prac&eogon; domow&aogon;";
#elif L==9	// pt
	"Ver trabalhos";
#elif L==10	// tr
	"View homework";	// eviri lazim!
#endif

const char *Txt_Visible =
#if   L==1	// ca
	"Visible";
#elif L==2	// de
	"Sichtbare";
#elif L==3	// en
	"Visible";
#elif L==4	// es
	"Visible";
#elif L==5	// fr
	"Visible";
#elif L==6	// gn
	"Visible";	// Okoteve traduccin
#elif L==7	// it
	"Visibile";
#elif L==8	// pl
	"Widoczny";
#elif L==9	// pt
	"Vis&iacute;vel";
#elif L==10	// tr
	"Visible";	// eviri lazim!
#endif

const char *Txt_Visible_by_BR_the_student =
#if   L==1	// ca
	"&iquest;Visible por<br />el estudiante?";	// Necessita traducci
#elif L==2	// de
	"Sichtbare durch<br />die Student?";
#elif L==3	// en
	"Visible by<br />the student?";
#elif L==4	// es
	"&iquest;Visible por<br />el estudiante?";
#elif L==5	// fr
	"Visible par<br />l'&eacute;tudiant/e?";
#elif L==6	// gn
	"&iquest;Visible por<br />el estudiante?";	// Okoteve traduccin
#elif L==7	// it
	"Visibili dallo<br />studente?";
#elif L==8	// pl
	"Visible by<br />the student?";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Vis&iacute;vel pelo<br />estudante?";
#elif L==10	// tr
	"Visible by<br />the student?";			// eviri lazim!
#endif

const char *Txt_Visible_results =
#if   L==1	// ca
	"Resultats visibles";
#elif L==2	// de
	"Sichtbar Ergebnisse";
#elif L==3	// en
	"Visible results";
#elif L==4	// es
	"Resultados visibles";
#elif L==5	// fr
	"R&eacute;sultats visible";
#elif L==6	// gn
	"Resultados visibles";	// Okoteve traduccin
#elif L==7	// it
	"Risultati visibili";
#elif L==8	// pl
	"Widoczne wynik&oacute;w";
#elif L==9	// pt
	"Resultados vis&iacute;veis";
#elif L==10	// tr
	"Visible results";	// eviri lazim!
#endif

const char *Txt_Visible_MALE_PLURAL =
#if   L==1	// ca
	"Visibles";
#elif L==2	// de
	"Sichtbar";
#elif L==3	// en
	"Visible";
#elif L==4	// es
	"Visibles";
#elif L==5	// fr
	"Visibles";
#elif L==6	// gn
	"Visibles";	// Okoteve traduccin
#elif L==7	// it
	"Visibili";
#elif L==8	// pl
	"Widoczne";
#elif L==9	// pt
	"Vis&iacute;veis";
#elif L==10	// tr
	"Visible";	// eviri lazim!
#endif

const char *Txt_Visible_tests =
#if   L==1	// ca
	"Tests visibles";
#elif L==2	// de
	"Sichtbare Pr%uuml;fungen";
#elif L==3	// en
	"Visible tests";
#elif L==4	// es
	"Test visibles";
#elif L==5	// fr
	"Tests visibles";
#elif L==6	// gn
	"Test visibles";	// Okoteve traduccin
#elif L==7	// it
	"Test visibili";
#elif L==8	// pl
	"Widoczne testy";
#elif L==9	// pt
	"Testes vis&iacute;veis";
#elif L==10	// tr
	"Visible tests";	// eviri lazim!
#endif

const char *Txt_Visible_survey =
#if   L==1	// ca
	"Enquesta visible";
#elif L==2	// de
	"Sichtbare Umfrage";
#elif L==3	// en
	"Visible survey";
#elif L==4	// es
	"Encuesta visible";
#elif L==5	// fr
	"Sondage visible";
#elif L==6	// gn
	"Encuesta visible";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio visibile";
#elif L==8	// pl
	"Widoczny badanie";
#elif L==9	// pt
	"Inqu&eacute;rito vis&iacute;vel";
#elif L==10	// tr
	"Visible survey";	// eviri lazim!
#endif

const char *Txt_Visits_to_course =
#if   L==1	// ca
	"Accessos assignatura";
#elif L==2	// de
	"Anmeldungen bei Kurs";
#elif L==3	// en
	"Visits to course";
#elif L==4	// es
	"Accesos asignatura";
#elif L==5	// fr
	"Visites mati&egrave;re";
#elif L==6	// gn
	"Accesos asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Accessi al corso";
#elif L==8	// pl
	"Wizyty w trakcie";
#elif L==9	// pt
	"Visitas disciplina";
#elif L==10	// tr
	"Visits to course";	// eviri lazim!
#endif

const char *Txt_Web =
#if   L==1	// ca
	"Web";
#elif L==2	// de
	"Web";
#elif L==3	// en
	"Web";
#elif L==4	// es
	"Web";
#elif L==5	// fr
	"Web";
#elif L==6	// gn
	"Web";	// Okoteve traduccin
#elif L==7	// it
	"Web";
#elif L==8	// pl
	"Web";
#elif L==9	// pt
	"Web";
#elif L==10	// tr
	"Web";	// eviri lazim!
#endif

const char *Txt_Web_browser =
#if   L==1	// ca
	"Navegador web";
#elif L==2	// de
	"Webbrowser";
#elif L==3	// en
	"Web browser";
#elif L==4	// es
	"Navegador web";
#elif L==5	// fr
	"Navigateur web";
#elif L==6	// gn
	"Navegador web";	// Okoteve traduccin
#elif L==7	// it
	"Web browser";
#elif L==8	// pl
	"Przegl&aogon;darka internetowa";
#elif L==9	// pt
	"Navegador web";
#elif L==10	// tr
	"Web browser";		// eviri lazim!
#endif

const char *Txt_Web_page =
#if   L==1	// ca
	"P&agrave;gina web";
#elif L==2	// de
	"Webseite";
#elif L==3	// en
	"Web page";
#elif L==4	// es
	"P&aacute;gina web";
#elif L==5	// fr
	"Page Web";
#elif L==6	// gn
	"P&aacute;gina web";	// Okoteve traduccin
#elif L==7	// it
	"Pagina Web";
#elif L==8	// pl
	"Strona internetowa";
#elif L==9	// pt
	"P&aacute;gina Web";
#elif L==10	// tr
	"Web page";		// eviri lazim!
#endif

const char *Txt_Web_page_of_the_course =
#if   L==1	// ca
	"P&aacute;gina web de la asignatura";	// Necessita traducci
#elif L==2	// de
	"Webseite der Kurs";
#elif L==3	// en
	"Web page of the course";
#elif L==4	// es
	"P&aacute;gina web de la asignatura";
#elif L==5	// fr
	"Page Web de la mati&egrave;re";
#elif L==6	// gn
	"P&aacute;gina web de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Pagina Web del corso";
#elif L==8	// pl
	"Web page of the course";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"P&aacute;gina Web da disciplina";
#elif L==10	// tr
	"Web page of the course";		// eviri lazim!
#endif

const char *Txt_Web_social_network =
#if   L==1	// ca
	"Web / xarxa social";
#elif L==2	// de
	"Web / Soziales Netzwerk";
#elif L==3	// en
	"Web / social network";
#elif L==4	// es
	"Web / red social";
#elif L==5	// fr
	"Web / r&eacute;seau social";
#elif L==6	// gn
	"Web / red social";	// Okoteve traduccin
#elif L==7	// it
	"Web / social network";
#elif L==8	// pl
	"Web / sieci spo&lstrok;eczne";
#elif L==9	// pt
	"Web / rede social";
#elif L==10	// tr
	"Web / social network";	// eviri lazim!
#endif

const char *Txt_Webs_social_networks =
#if   L==1	// ca
	"Webs / xarxes socials";
#elif L==2	// de
	"Webs / Soziale Netzwerke";
#elif L==3	// en
	"Webs / social networks";
#elif L==4	// es
	"Webs / redes sociales";
#elif L==5	// fr
	"Webs / r&eacute;seaux sociaux";
#elif L==6	// gn
	"Webs / redes sociales";	// Okoteve traduccin
#elif L==7	// it
	"Webs / reti sociali";
#elif L==8	// pl
	"Webs / sieci spo&lstrok;eczne";
#elif L==9	// pt
	"Webs / redes sociais";
#elif L==10	// tr
	"Webs / social networks";	// eviri lazim!
#endif

const char *Txt_Week =
#if   L==1	// ca
	"Semana";	// Necessita traducci
#elif L==2	// de
	"Woche";
#elif L==3	// en
	"Week";
#elif L==4	// es
	"Semana";
#elif L==5	// fr
	"Semaine";
#elif L==6	// gn
	"Arapok&otilde;indy";
#elif L==7	// it
	"Settimana";
#elif L==8	// pl
	"Week";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Semana";
#elif L==10	// tr
	"Week";		// eviri lazim!
#endif

const char *Txt_Weight =
#if   L==1	// ca
	"Pes";
#elif L==2	// de
	"Gewicht";
#elif L==3	// en
	"Weight";
#elif L==4	// es
	"Peso";
#elif L==5	// fr
	"Poids";
#elif L==6	// gn
	"Peso";	// Okoteve traduccin
#elif L==7	// it
	"Peso";
#elif L==8	// pl
	"Waga";
#elif L==9	// pt
	"Peso";
#elif L==10	// tr
	"A&gbreve;&inodot;rl&inodot;k";
#endif

const char *Txt_What_would_you_like_to_do =
#if   L==1	// ca
	"&iquest;Qu&egrave; voleu fer?";
#elif L==2	// de
	"Was w&uuml;rden Sie tun?";
#elif L==3	// en
	"What would you like to do?";
#elif L==4	// es
	"&iquest;Qu&eacute; desea hacer?";
#elif L==5	// fr
	"Que feriez-vous?";
#elif L==6	// gn
	"&iquest;Qu&eacute; desea hacer?";	// Okoteve traduccin
#elif L==7	// it
	"Cosa si dovrebbe fare?";
#elif L==8	// pl
	"Co by&sacute; zrobi&lstrok;?";
#elif L==9	// pt
	"O que voc&ecirc; faria?";
#elif L==10	// tr
	"What would you like to do?";		// eviri lazim!
#endif

const char *Txt_Where_s_USER =
#if   L==1	// ca
	"On &eacute;s %s?";
#elif L==2	// de
	"Wo ist %s?";
#elif L==3	// en
	"Where's %s";
#elif L==4	// es
	"&iquest;D&oacute;nde est&aacute; %s?";
#elif L==5	// fr
	"O&ugrave; est %s?";
#elif L==6	// gn
	"&iquest;D&oacute;nde est&aacute; %s?";	// Okoteve traduccin
#elif L==7	// it
	"Dove &egrave; %s?";
#elif L==8	// pl
	"Gdzie jest %s?";
#elif L==9	// pt
	"Onde est&aacute; %s?";
#elif L==10	// tr
	"Where's %s";				// eviri lazim!
#endif

const char *Txt_WHO[Usr_NUM_WHO] =
	{
	[Usr_WHO_UNKNOWN] =
	NULL
	,
	[Usr_WHO_ME] =
#if   L==1	// ca
	"Jo"
#elif L==2	// de
	"Mich"
#elif L==3	// en
	"Me"
#elif L==4	// es
	"Yo"
#elif L==5	// fr
	"Moi"
#elif L==6	// gn
	"Che"
#elif L==7	// it
	"Me"
#elif L==8	// pl
	"Mnie"
#elif L==9	// pt
	"Eu"
#elif L==10	// tr
	"Me"	// eviri lazim!
#endif
	,
	[Usr_WHO_SELECTED] =
#if   L==1	// ca
	"Usuaris seleccionats"
#elif L==2	// de
	"Ausgew&auml;hlte Benutzer"
#elif L==3	// en
	"Selected users"
#elif L==4	// es
	"Usuarios seleccionados"
#elif L==5	// fr
	"Utilisateurs s&eacute;lectionn&eacute;s"
#elif L==6	// gn
	"Usuarios seleccionados"	// Okoteve traduccin
#elif L==7	// it
	"Utenti selezionati"
#elif L==8	// pl
	"Wybrani u&zdot;ytkownicy"
#elif L==9	// pt
	"Usu&aacute;rios selecionados"
#elif L==10	// tr
	"Selected users"		// eviri lazim!
#endif
	,
	[Usr_WHO_FOLLOWED] =
#if   L==1	// ca
	"Usuaris que segueixo"
#elif L==2	// de
	"Benutzer Ich folge"
#elif L==3	// en
	"Users I'm following"
#elif L==4	// es
	"Usuarios que sigo"
#elif L==5	// fr
	"Utilisateurs qui je suis"
#elif L==6	// gn
	"Usuarios que sigo"	// Okoteve traduccin
#elif L==7	// it
	"Utenti che sto seguenti"
#elif L==8	// pl
	"U&zdot;ytkownicy, kt&oacute;rzy mam nast&eogon;puj&aogon;ce"
#elif L==9	// pt
	"Usu&aacute;rios que estou seguindo"
#elif L==10	// tr
	"Users I'm following"	// eviri lazim!
#endif
	,
	[Usr_WHO_ALL] =
#if   L==1	// ca
	"Tots els usuaris"
#elif L==2	// de
	"Alle Benutzer"
#elif L==3	// en
	"All users"
#elif L==4	// es
	"Todos los usuarios"
#elif L==5	// fr
	"Tous les utilisateurs"
#elif L==6	// gn
	"Todos los usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Tutti gli utenti"
#elif L==8	// pl
	"Wszyscy u&zdot;ytkownicy"
#elif L==9	// pt
	"Todos os usu&aacute;rios"
#elif L==10	// tr
	"All users"		// eviri lazim!
#endif
	};

const char *Txt_Who_to_follow =
#if   L==1	// ca
	"A qui seguir";
#elif L==2	// de
	"Wer zu folgen";
#elif L==3	// en
	"Who to follow";
#elif L==4	// es
	"A qui&eacute;n seguir";
#elif L==5	// fr
	"Qui &agrave; suivre";
#elif L==6	// gn
	"M&aacute;vapepa oseg&iacute;ta";
#elif L==7	// it
	"Chi da seguire";
#elif L==8	// pl
	"Kto do na&sacute;ladowania";
#elif L==9	// pt
	"Quem seguir";
#elif L==10	// tr
	"Kimi takip etmeli";
#endif

const char *Txt_With_ =	// Example: "With students"
#if   L==1	// ca
	"Amb ";
#elif L==2	// de
	"Mit ";
#elif L==3	// en
	"With ";
#elif L==4	// es
	"Con ";
#elif L==5	// fr
	"Avec ";
#elif L==6	// gn
	"Con ";		// Okoteve traduccin
#elif L==7	// it
	"Con ";
#elif L==8	// pl
	"Z ";
#elif L==9	// pt
	"Com ";
#elif L==10	// tr
	"With ";	// eviri lazim!
#endif

const char *Txt_with_new_posts =
#if   L==1	// ca
	"con comentarios nuevos";	// Necessita traducci
#elif L==2	// de
	"mit neuen Beitr&auml;ge";
#elif L==3	// en
	"with new posts";
#elif L==4	// es
	"con comentarios nuevos";
#elif L==5	// fr
	"avec de nouveaux posts";
#elif L==6	// gn
	"con comentarios nuevos";	// Okoteve traduccin
#elif L==7	// it
	"con nuovi post";
#elif L==8	// pl
	"with new posts";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"com novas posts";
#elif L==10	// tr
	"with new posts";		// eviri lazim!
#endif

const char *Txt_works_ZIP_FILE_NAME =	// Name (without extension) of the ZIP file for works. Must be a valid filename in small letters!
#if   L==1	// ca
	"treballs";
#elif L==2	// de
	"Hausarbeiten";
#elif L==3	// en
	"homework";
#elif L==4	// es
	"trabajos";
#elif L==5	// fr
	"travaux";
#elif L==6	// gn
	"tembiapo";
#elif L==7	// it
	"lavori";
#elif L==8	// pl
	"homework";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"trabalhos";
#elif L==10	// tr
	"homework";	// eviri lazim!
#endif

const char *Txt_Wording =	// Statement of a question
#if   L==1	// ca
	"Enunciat";
#elif L==2	// de
	"Aussage";
#elif L==3	// en
	"Wording";
#elif L==4	// es
	"Enunciado";
#elif L==5	// fr
	"&Eacute;nonc&eacute;";
#elif L==6	// gn
	"Enunciado";	// Okoteve traduccin
#elif L==7	// it
	"Formulazione";
#elif L==8	// pl
	"Sformu&lstrok;owanie";
#elif L==9	// pt
	"Reda&ccedil;&atilde;o";
#elif L==10	// tr
	"Wording";	// eviri lazim!
#endif

const char *Txt_Works_area =
#if   L==1	// ca
	"Zona d'altres treballs";
#elif L==2	// de
	"Hausarbeiten-Bereich";
#elif L==3	// en
	"Homework area";
#elif L==4	// es
	"Zona de otros trabajos";
#elif L==5	// fr
	"Zone de travaux";
#elif L==6	// gn
	"Zona de otros trabajos";	// Okoteve traduccin
#elif L==7	// it
	"Area lavori";
#elif L==8	// pl
	"Homework area";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Zona de trabalhos";
#elif L==10	// tr
	"Homework area";		// eviri lazim!
#endif

const char *Txt_works_area =
#if   L==1	// ca
	"zona de trabajos";	// Necessita traducci
#elif L==2	// de
	"Hausarbeiten-Bereich";
#elif L==3	// en
	"homework area";
#elif L==4	// es
	"zona de trabajos";
#elif L==5	// fr
	"zone de travaux";
#elif L==6	// gn
	"zona de trabajos";	// Okoteve traduccin
#elif L==7	// it
	"area lavori";
#elif L==8	// pl
	"homework area";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"zona de trabalhos";
#elif L==10	// tr
	"homework area";	// eviri lazim!
#endif

const char *Txt_WriBRters =
#if   L==1	// ca
	"Escri-<br />tores";	// Necessita traducci
#elif L==2	// de
	"Schrift-<br />steller";
#elif L==3	// en
	"Wri-<br />ters";
#elif L==4	// es
	"Escri-<br />tores";
#elif L==5	// fr
	"&Eacute;cri-<br />vains";
#elif L==6	// gn
	"Escri-<br />tores";	// Okoteve traduccin
#elif L==7	// it
	"Scri-<br />ttori";
#elif L==8	// pl
	"Wri-<br />ters";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Escri-<br />tores";
#elif L==10	// tr
	"Wri-<br />ters";	// eviri lazim!
#endif

const char *Txt_Write_a_message =
#if   L==1	// ca
	"Escriu un missatge";
#elif L==2	// de
	"Schreiben eine Nachricht";
#elif L==3	// en
	"Write a message";
#elif L==4	// es
	"Escribir un mensaje";
#elif L==5	// fr
	"&Eacute;crire un message";
#elif L==6	// gn
	"Escribir un mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Scrivi un messaggio";
#elif L==8	// pl
	"Napisz wiadomo&sacute;&cacute;";
#elif L==9	// pt
	"Escrever uma mensagem";
#elif L==10	// tr
	"Write a message";	// eviri lazim!
#endif

const char *Txt_Wrong_file_type =
#if   L==1	// ca
	"Tipus d'arxiu incorrecte.";
#elif L==2	// de
	"Falsche Dateityp.";
#elif L==3	// en
	"Wrong file type.";
#elif L==4	// es
	"Tipo de archivo incorrecto.";
#elif L==5	// fr
	"Mauvais type de fichier.";
#elif L==6	// gn
	"Tipo de archivo incorrecto.";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di file sbagliato.";
#elif L==8	// pl
	"Nieprawid&lstrok;owy typ pliku.";
#elif L==9	// pt
	"Tipo de arquivo errado.";
#elif L==10	// tr
	"Wrong file type.";		// eviri lazim!
#endif

const char *Txt_WWW =		// World Wide Web
#if   L==1	// ca
	"WWW";
#elif L==2	// de
	"WWW";
#elif L==3	// en
	"WWW";
#elif L==4	// es
	"WWW";
#elif L==5	// fr
	"WWW";
#elif L==6	// gn
	"WWW";
#elif L==7	// it
	"WWW";
#elif L==8	// pl
	"WWW";
#elif L==9	// pt
	"WWW";
#elif L==10	// tr
	"WWW";	// eviri lazim!
#endif

const char *Txt_XML_file =
#if   L==1	// ca
	"Fitxer XML";
#elif L==2	// de
	"XML-Datei";
#elif L==3	// en
	"XML file";
#elif L==4	// es
	"Archivo XML";
#elif L==5	// fr
	"Fichier XML";
#elif L==6	// gn
	"Archivo XML";	// Okoteve traduccin
#elif L==7	// it
	"File XML";
#elif L==8	// pl
	"Plik XML";
#elif L==9	// pt
	"Arquivo XML";
#elif L==10	// tr
	"XML file";	// eviri lazim!
#endif

const char *Txt_XML_file_content =
#if   L==1	// ca
	"Contingut del fitxer XML";
#elif L==2	// de
	"XML-Datei-Inhalt";
#elif L==3	// en
	"XML file content";
#elif L==4	// es
	"Contenido del archivo XML";
#elif L==5	// fr
	"Contenu du fichier XML";
#elif L==6	// gn
	"Contenido del archivo XML";	// Okoteve traduccin
#elif L==7	// it
	"Contenuto di file XML";
#elif L==8	// pl
	"Zawarto&sacute;&cacute; pliku XML";
#elif L==9	// pt
	"Conte&uacute;do do arquivo XML";
#elif L==10	// tr
	"XML file content";		// eviri lazim!
#endif

const char *Txt_Year =	// 1984, 2011...
#if   L==1	// ca
	"Any";
#elif L==2	// de
	"Jahr";
#elif L==3	// en
	"Year";
#elif L==4	// es
	"A&ntilde;o";
#elif L==5	// fr
	"Ann&eacute;e";
#elif L==6	// gn
	"Ary";
#elif L==7	// it
	"Anno";
#elif L==8	// pl
	"Rok";
#elif L==9	// pt
	"Ano";
#elif L==10	// tr
	"Year";	// eviri lazim!
#endif

const char *Txt_year =	// 1984, 2011...
#if   L==1	// ca
	"any";
#elif L==2	// de
	"Jahr";
#elif L==3	// en
	"year";
#elif L==4	// es
	"a&ntilde;o";
#elif L==5	// fr
	"ann&eacute;e";
#elif L==6	// gn
	"ary";
#elif L==7	// it
	"anno";
#elif L==8	// pl
	"rok";
#elif L==9	// pt
	"ano";
#elif L==10	// tr
	"year";	// eviri lazim!
#endif

const char *Txt_Year_OF_A_DEGREE =	// 1, 2, 3..., not 1984, 2011...
#if   L==1	// ca
	"Any o semestre";
#elif L==2	// de
	"Jahr oder Semester";
#elif L==3	// en
	"Year or semester";
#elif L==4	// es
	"A&ntilde;o o semestre";
#elif L==5	// fr
	"Ann&eacute;e ou semestre";
#elif L==6	// gn
	"A&ntilde;o o semestre";	// Okoteve traduccin
#elif L==7	// it
	"Anno o semestre";
#elif L==8	// pl
	"Rok lub semestr";
#elif L==9	// pt
	"Ano ou semestre";
#elif L==10	// tr
	"Year or semester";		// eviri lazim!
#endif

const char *Txt_YEAR_OF_DEGREE[1 + Deg_MAX_YEARS_PER_DEGREE] =
	{
#if   L==1	// ca
	"N.A."
#elif L==2	// de
	"N.A."
#elif L==3	// en
	"N.A."
#elif L==4	// es
	"N.A."
#elif L==5	// fr
	"N.A."
#elif L==6	// gn
	"N.A."		// Okoteve traduccin
#elif L==7	// it
	"N.A."
#elif L==8	// pl
	"N.D."
#elif L==9	// pt
	"N.A."
#elif L==10	// tr
	"N.A."		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"1r"
#elif L==2	// de
	"1"
#elif L==3	// en
	"1st"
#elif L==4	// es
	"1&ordm;"
#elif L==5	// fr
	"1er"
#elif L==6	// gn
	"1&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"1&ordm;"
#elif L==8	// pl
	"1."
#elif L==9	// pt
	"1&ordm;"
#elif L==10	// tr
	"1st"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"2n"
#elif L==2	// de
	"2"
#elif L==3	// en
	"2nd"
#elif L==4	// es
	"2&ordm;"
#elif L==5	// fr
	"2&egrave;me"
#elif L==6	// gn
	"2&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"2&ordm;"
#elif L==8	// pl
	"2."
#elif L==9	// pt
	"2&ordm;"
#elif L==10	// tr
	"2nd"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"3r"
#elif L==2	// de
	"3rd"
#elif L==3	// en
	"3rd"
#elif L==4	// es
	"3&ordm;"
#elif L==5	// fr
	"3e"
#elif L==6	// gn
	"3&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"3&ordm;"
#elif L==8	// pl
	"3."
#elif L==9	// pt
	"3&ordm;"
#elif L==10	// tr
	"3rd"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"4t"
#elif L==2	// de
	"4"
#elif L==3	// en
	"4th"
#elif L==4	// es
	"4&ordm;"
#elif L==5	// fr
	"4&egrave;me"
#elif L==6	// gn
	"4&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"4&ordm;"
#elif L==8	// pl
	"4."
#elif L==9	// pt
	"4&ordm;"
#elif L==10	// tr
	"4th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"5&egrave;"
#elif L==2	// de
	"5"
#elif L==3	// en
	"5th"
#elif L==4	// es
	"5&ordm;"
#elif L==5	// fr
	"5&egrave;me"
#elif L==6	// gn
	"5&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"5&ordm;"
#elif L==8	// pl
	"5."
#elif L==9	// pt
	"5&ordm;"
#elif L==10	// tr
	"5th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"6&egrave;"
#elif L==2	// de
	"6"
#elif L==3	// en
	"6th"
#elif L==4	// es
	"6&ordm;"
#elif L==5	// fr
	"6&egrave;me"
#elif L==6	// gn
	"6&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"6&ordm;"
#elif L==8	// pl
	"6."
#elif L==9	// pt
	"6&ordm;"
#elif L==10	// tr
	"6th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"7&egrave;"
#elif L==2	// de
	"7"
#elif L==3	// en
	"7th"
#elif L==4	// es
	"7&ordm;"
#elif L==5	// fr
	"7&egrave;me"
#elif L==6	// gn
	"7&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"7&ordm;"
#elif L==8	// pl
	"7."
#elif L==9	// pt
	"7&ordm;"
#elif L==10	// tr
	"7th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"8&egrave;"
#elif L==2	// de
	"8"
#elif L==3	// en
	"8th"
#elif L==4	// es
	"8&ordm;"
#elif L==5	// fr
	"8&egrave;me"
#elif L==6	// gn
	"8&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"8&ordm;"
#elif L==8	// pl
	"8."
#elif L==9	// pt
	"8&ordm;"
#elif L==10	// tr
	"8th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"9&egrave;"
#elif L==2	// de
	"9"
#elif L==3	// en
	"9th"
#elif L==4	// es
	"9&ordm;"
#elif L==5	// fr
	"9&egrave;me"
#elif L==6	// gn
	"9&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"9&ordm;"
#elif L==8	// pl
	"9."
#elif L==9	// pt
	"9&ordm;"
#elif L==10	// tr
	"9th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"10&egrave;"
#elif L==2	// de
	"10"
#elif L==3	// en
	"10th"
#elif L==4	// es
	"10&ordm;"
#elif L==5	// fr
	"10e"
#elif L==6	// gn
	"10&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"10&ordm;"
#elif L==8	// pl
	"10."
#elif L==9	// pt
	"10&ordm;"
#elif L==10	// tr
	"10th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"11&egrave;"
#elif L==2	// de
	"11"
#elif L==3	// en
	"11th"
#elif L==4	// es
	"11&ordm;"
#elif L==5	// fr
	"11&egrave;me"
#elif L==6	// gn
	"11&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"11&ordm;"
#elif L==8	// pl
	"11."
#elif L==9	// pt
	"11&ordm;"
#elif L==10	// tr
	"11th"		// eviri lazim!
#endif
	,
#if   L==1	// ca
	"12&egrave;"
#elif L==2	// de
	"12"
#elif L==3	// en
	"12th"
#elif L==4	// es
	"12&ordm;"
#elif L==5	// fr
	"12e"
#elif L==6	// gn
	"12&ordm;"	// Okoteve traduccin
#elif L==7	// it
	"12&ordm;"
#elif L==8	// pl
	"12."
#elif L==9	// pt
	"12&ordm;"
#elif L==10	// tr
	"12th"		// eviri lazim!
#endif
	};

const char *Txt_years =
#if   L==1	// ca
	"anys";
#elif L==2	// de
	"Jahre";
#elif L==3	// en
	"years";
#elif L==4	// es
	"a&ntilde;os";
#elif L==5	// fr
	"ann&eacute;es";
#elif L==6	// gn
	"ary";
#elif L==7	// it
	"anni";
#elif L==8	// pl
	"lat";
#elif L==9	// pt
	"anos";
#elif L==10	// tr
	"years";	// eviri lazim!
#endif

const char *Txt_YES =
#if   L==1	// ca
	"S&Iacute;";
#elif L==2	// de
	"JA";
#elif L==3	// en
	"YES";
#elif L==4	// es
	"S&Iacute;";
#elif L==5	// fr
	"OUI";
#elif L==6	// gn
	"HE&#7868;";
#elif L==7	// it
	"SI";
#elif L==8	// pl
	"TAK";
#elif L==9	// pt
	"SIM";
#elif L==10	// tr
	"YES";	// eviri lazim!
#endif

const char *Txt_Yes =
#if   L==1	// ca
	"S&iacute;";
#elif L==2	// de
	"Ja";
#elif L==3	// en
	"Yes";
#elif L==4	// es
	"S&iacute;";
#elif L==5	// fr
	"Oui";
#elif L==6	// gn
	"He&#7869;";
#elif L==7	// it
	"S&igrave;";
#elif L==8	// pl
	"Tak";
#elif L==9	// pt
	"Sim";
#elif L==10	// tr
	"Yes";	// eviri lazim!
#endif

const char *Txt_Yesterday =
#if   L==1	// ca
	"Ahir";
#elif L==2	// de
	"Gestern";
#elif L==3	// en
	"Yesterday";
#elif L==4	// es
	"Ayer";
#elif L==5	// fr
	"Hier";
#elif L==6	// gn
	"Kuehe";
#elif L==7	// it
	"Ieri";
#elif L==8	// pl
	"Wczorajszy dzie&nacute;";
#elif L==9	// pt
	"Ontem";
#elif L==10	// tr
	"Yesterday";	// eviri lazim!
#endif

const char *Txt_yet_unnamed =
#if   L==1	// ca
	"encara sense nom";
#elif L==2	// de
	"noch unbenannte";
#elif L==3	// en
	"yet unnamed";
#elif L==4	// es
	"a&uacute;n sin nombre";
#elif L==5	// fr
	"encore sans nom";
#elif L==6	// gn
	"a&uacute;n sin nombre";
#elif L==7	// it
	"ancora senza nome";
#elif L==8	// pl
	"jeszcze nienazwany";
#elif L==9	// pt
	"ainda sem nome";
#elif L==10	// tr
	"yet unnamed";	// eviri lazim!
#endif

const char *Txt_You_already_answered_this_survey_before =
#if   L==1	// ca
	"Vost&egrave; ja ha respost a aquesta enquesta abans.";
#elif L==2	// de
	"Sie haben bereits beantwortet dieser Umfrage vor.";
#elif L==3	// en
	"You already answered this survey before.";
#elif L==4	// es
	"Usted ya respondi&oacute; esta encuesta anteriormente.";
#elif L==5	// fr
	"Vous avez d&eacute;j&agrave; r&eacute;pondu &agrave; ce sondage avant.";
#elif L==6	// gn
	"Usted ya respondi&oacute; esta encuesta anteriormente.";	// Okoteve traduccin
#elif L==7	// it
	"Hai gi&agrave; risposto a questo sondaggio prima.";
#elif L==8	// pl
	"Odpowiedzia&lstrok;e&sacute; ju&zdot; wcze&sacute;niej na ten ankiet&eogon;.";
#elif L==9	// pt
	"Voc&ecirc; j&aacute; respondeu a esso inqu&eacute;rito antes.";
#elif L==10	// tr
	"You already answered this survey before.";			// eviri lazim!
#endif

const char *Txt_You_are_now_LOGGED_IN_as_X =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Vost&egrave; est&agrave; ara %s com a <strong>%s</strong>.";
#elif L==2	// de
	"Sie sind nun %s als <strong>%s</strong>.";
#elif L==3	// en
	"You are now %s as <strong>%s</strong>.";
#elif L==4	// es
	"Ahora est&aacute; %s como <strong>%s</strong>.";
#elif L==5	// fr
	"Vous &ecirc;tes maintenant %s en tant que <strong>%s</strong>.";
#elif L==6	// gn
	"Ahora est&aacute; %s como <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Sei ora %s come <strong>%s</strong>.";
#elif L==8	// pl
	"Jeste&sacute; teraz %s jako <strong>%s</strong>.";
#elif L==9	// pt
	"Agora voc&ecirc; est&aacute; %s como <strong>%s</strong>.";
#elif L==10	// tr
	"You are now %s as <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_You_are_not_enroled_in_any_course[Usr_NUM_SEXS] =
	{
	[Usr_SEX_UNKNOWN] =
#if   L==1	// ca
	"No est&agrave; inscrit/a en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrito/a en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritto/a ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#elif L==10	// tr
	"You are not enroled in any course."			// eviri lazim!
#endif
	,
	[Usr_SEX_FEMALE] =
#if   L==1	// ca
	"No est&agrave; inscrita en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrita en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrita en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritta ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculada em nenhuma disciplina."
#elif L==10	// tr
	"You are not enroled in any course."			// eviri lazim!
#endif
	,
	[Usr_SEX_MALE] =
#if   L==1	// ca
	"No est&agrave; inscrit en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrito en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrito en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritto ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculado em nenhuma disciplina."
#elif L==10	// tr
	"You are not enroled in any course."			// eviri lazim!
#endif
	,
	[Usr_SEX_ALL] =
#if   L==1	// ca
	"No est&agrave; inscrit/a en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrito/a en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritto/a ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#elif L==10	// tr
	"You are not enroled in any course."			// eviri lazim!
#endif
	};

const char *Txt_You_can_create_a_new_folder_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Podeu crear una nova carpeta"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==2	// de
	"Sie k&ouml;nnen ein neues Verzeichnis"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==3	// en
	"You can create a new folder"
	" inside the folder <strong>%s</strong>:";
#elif L==4	// es
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"Vous pouvez cr&eacute;er un nouveau r&eacute;pertoire"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==8	// pl
	"Mo&zdot;esz utworzy&cacute; nowy folder"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==9	// pt
	"Voc&ecirc; pode criar um novo diret&oacute;rio"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#elif L==10	// tr
	"You can create a new folder"
	" inside the folder <strong>%s</strong>:";	// eviri lazim!
#endif

const char *Txt_You_can_not_create_folders_files_or_links_here =
#if   L==1	// ca
	"No podeu crear carpetes, arxius o enlla&ccedil;os aqu&iacute;.";
#elif L==2	// de
	"Sie k&ouml;nnen hier keine Ordner, Dateien oder Links erstellen.";
#elif L==3	// en
	"You can not create folders, files or links here.";
#elif L==4	// es
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";
#elif L==5	// fr
	"Vous ne pouvez pas cr&eacute;er de dossiers, de fichiers ou de liens ici.";
#elif L==6	// gn
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare cartelle, file o linki qui.";
#elif L==8	// pl
	"Nie mo&zdot;esz tutaj tworzy&cacute; folder&oacute;w, plik&oacute;w ani link&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode criar pastas, arquivos ou links aqui.";
#elif L==10	// tr
	"You can not create folders, files or links here.";		// eviri lazim!
#endif

const char *Txt_You_can_not_create_folders_here =
#if   L==1	// ca
	"No podeu crear carpetes aqu&iacute;.";
#elif L==2	// de
	"Sie k&ouml;nnen hier keine Ordner erstellen.";
#elif L==3	// en
	"You can not create folders here.";
#elif L==4	// es
	"No puede crear carpetas aqu&iacute;.";
#elif L==5	// fr
	"Vous ne pouvez pas cr&eacute;er de r&eacute;pertoires ici.";
#elif L==6	// gn
	"No puede crear carpetas aqu&iacute;.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare cartelle qui.";
#elif L==8	// pl
	"Nie mo&zdot;esz tutaj tworzy&cacute; folder&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode criar pastas aqui.";
#elif L==10	// tr
	"You can not create folders here.";	// eviri lazim!
#endif

const char *Txt_You_can_not_create_links_here =
#if   L==1	// ca
	"No podeu crear enlla&ccedil;os aqu&iacute;.";
#elif L==2	// de
	"Sie k&ouml;nnen hier keine Links erstellen.";
#elif L==3	// en
	"You can not create links here.";
#elif L==4	// es
	"No puede crear enlaces aqu&iacute;.";
#elif L==5	// fr
	"Vous ne pouvez pas cr&eacute;er de liens ici.";
#elif L==6	// gn
	"No puede crear enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare linki qui.";
#elif L==8	// pl
	"Nie mo&zdot;esz tutaj tworzy&cacute; link&oacute;w..";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode criar links aqui.";
#elif L==10	// tr
	"You can not create links here.";	// eviri lazim!
#endif

const char *Txt_You_can_not_delete_your_current_nickname =
#if   L==1	// ca
	"No podeu suprimir el sobrenom actual.";
#elif L==2	// de
	"Sie k&ouml;nnen Ihren aktuellen Spitznamen nicht l&ouml;schen.";
#elif L==3	// en
	"You can not delete your current nickname.";
#elif L==4	// es
	"No puede eliminar su apodo actual.";
#elif L==5	// fr
	"Vous ne pouvez pas supprimer votre pseudo actuel.";
#elif L==6	// gn
	"No puede eliminar su apodo actual.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi cancellare il tuo nome utente attuale.";
#elif L==8	// pl
	"Nie mo&zdot;esz usun&aogon;&cacute; swojego obecnego pseudonimu.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode excluir sua alcunha atual.";
#elif L==10	// tr
	"You can not delete your current nickname.";	// eviri lazim!
#endif

const char *Txt_You_can_not_delete_this_ID =
#if   L==1	// ca
	"No podeu suprimir aquest ID (DNI/c&egrave;dula).";
#elif L==2	// de
	"Sie k&ouml;nnen diese ID nicht l&ouml;schen.";
#elif L==3	// en
	"You can not delete this ID.";
#elif L==4	// es
	"No puede eliminar este ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"Vous ne pouvez pas supprimer cet ID.";
#elif L==6	// gn
	"No puede eliminar este ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi cancellare questo ID.";
#elif L==8	// pl
	"Nie mo&zdot;esz usun&aogon;&cacute; tego ID.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode excluir este ID.";
#elif L==10	// tr
	"You can not delete this ID.";				// eviri lazim!
#endif

const char *Txt_You_can_not_disable_file_uploading_once_folders_have_been_created =
#if   L==1	// ca
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Necessita traducci
#elif L==2	// de
	"Sie k&ouml;nnen den Upload nicht deaktivieren,"
	" wenn bereits Verzeichnisse angelegt worden sind.";
#elif L==3	// en
	"You can not disable file uploading"
	" once folders have been created.";
#elif L==4	// es
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";
#elif L==5	// fr
	"Vous ne pouvez pas d&eacute;sactiver le t&eacute;l&eacute;chargement de fichiers,"
	" une fois des r&eacute;pertoires ont &eacute;t&eacute; cr&eacute;&eacute;s.";
#elif L==6	// gn
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Okoteve traduccin
#elif L==7	// it
	"Non &egrave; possibile disabilitare il caricamento dei file"
	" una volta che le cartelle sono state create.";
#elif L==8	// pl
	"You can not disable file uploading"
	" once folders have been created.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode desativar o upload de arquivos,"
	" uma vez diret&oacute;rios foram criados.";
#elif L==10	// tr
	"You can not disable file uploading"
	" once folders have been created.";	// eviri lazim!
#endif

const char *Txt_You_can_not_leave_empty_intermediate_answers =
#if   L==1	// ca
	"No podeu deixar buides les respostes interm&egrave;dies.";
#elif L==2	// de
	"Sie k&ouml;nnen keine leeren Zwischenantworten hinterlassen.";
#elif L==3	// en
	"You can not leave empty intermediate answers.";
#elif L==4	// es
	"No puede dejar respuestas intermedias sin rellenar.";
#elif L==5	// fr
	"Vous ne pouvez pas laisser de r&eacute;ponses interm&eacute;diaires vides.";
#elif L==6	// gn
	"No puede dejar respuestas intermedias sin rellenar.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare vuote domande intermedie.";
#elif L==8	// pl
	"Nie mo&zdot;esz pozostawi&cacute; pustych odpowiedzi po&sacute;rednich.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar respostas intermedi&aacute;rias vazias.";
#elif L==10	// tr
	"You can not leave empty intermediate answers.";	// eviri lazim!
#endif

const char *Txt_You_can_not_leave_the_field_empty =
#if   L==1	// ca
	"No podeu deixar el camp buit.";
#elif L==2	// de
	"Sie k&ouml;nnen das Feld nicht leer lassen.";
#elif L==3	// en
	"You can not leave the field empty.";
#elif L==4	// es
	"No puede dejar el campo vac&iacute;o.";
#elif L==5	// fr
	"Vous ne pouvez pas laisser le champ vide.";
#elif L==6	// gn
	"No puede dejar el campo vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il campo vuoto.";
#elif L==8	// pl
	"Nie mo&zdot;esz zostawi&cacute; pola pustego.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o campo vazio.";
#elif L==10	// tr
	"You can not leave the field empty.";		// eviri lazim!
#endif

const char *Txt_You_can_not_paste_file_or_folder_here =
#if   L==1	// ca
	"No podeu enganxar un arxiu o una carpeta aqu&iacute;.";
#elif L==2	// de
	"Sie k&ouml;nnen hier keine Dateien oder Ordner einf&uuml;gen.";
#elif L==3	// en
	"You can not paste file or folder here.";
#elif L==4	// es
	"No puede pegar un archivo o carpeta aqu&iacute;.";
#elif L==5	// fr
	"Vous ne pouvez pas coller de fichier ou de r&eacute;pertoire ici.";
#elif L==6	// gn
	"No puede pegar un archivo o carpeta aqu&iacute;.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi copiare file o cartella qui.";
#elif L==8	// pl
	"Nie mo&zdot;esz tutaj wklei&cacute; pliku lub folderu.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode colar arquivos ou pastas aqui.";
#elif L==10	// tr
	"You can not paste file or folder here.";		// eviri lazim!
#endif

const char *Txt_You_can_not_remove_this_file_or_link =
#if   L==1	// ca
	"No podeu eliminar aquest arxiu o enlla&ccedil;.";
#elif L==2	// de
	"Sie k&ouml;nnen diese Datei oder diesen Link nicht entfernen.";
#elif L==3	// en
	"You can not remove this file or link.";
#elif L==4	// es
	"No puede eliminar este archivo o enlace.";
#elif L==5	// fr
	"Vous ne pouvez pas supprimer ce fichier ou ce lien.";
#elif L==6	// gn
	"No puede eliminar este archivo o enlace.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rimuovere questo file o link.";
#elif L==8	// pl
	"Nie mo&zdot;esz usun&aogon;&cacute; tego pliku lub linku.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode remover este arquivo ou link.";
#elif L==10	// tr
	"You can not remove this file or link.";	// eviri lazim!
#endif

const char *Txt_You_can_not_remove_this_folder =
#if   L==1	// ca
	"No podeu eliminar aquesta carpeta.";
#elif L==2	// de
	"Sie k&ouml;nnen diesen Ordner nicht entfernen.";
#elif L==3	// en
	"You can not remove this folder.";
#elif L==4	// es
	"No puede eliminar esta carpeta.";
#elif L==5	// fr
	"Vous ne pouvez pas supprimer ce r&eacute;pertoire.";
#elif L==6	// gn
	"No puede eliminar esta carpeta.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rimuovere questa cartella.";
#elif L==8	// pl
	"Nie mo&zdot;esz usun&aogon;&cacute; tego folderu.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode remover esta pasta.";
#elif L==10	// tr
	"ou can not remove this folder.";	// eviri lazim!
#endif

const char *Txt_You_can_not_remove_a_country_with_institutions_or_users =
#if   L==1	// ca
	"No pots eliminar un pa&iacute;s amb institucions o usuaris.";
#elif L==2	// de
	"Sie k&ouml;nnen kein Land mit Institutionen oder Benutzern entfernen.";
#elif L==3	// en
	"You can not remove a country with institutions or users.";
#elif L==4	// es
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";
#elif L==5	// fr
	"Vous ne pouvez pas supprimer un pays avec des institutions ou des utilisateurs.";
#elif L==6	// gn
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rimuovere un paese con istituzioni o utenti.";
#elif L==8	// pl
	"Nie mo&zdot;esz usun&aogon;&cacute; kraju z instytucjami lub u&zdot;ytkownikami.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode remover um pa&iacute;s com instituciones ou usu&aacute;rios.";
#elif L==10	// tr
	"You can not remove a country with institutions or users.";		// eviri lazim!
#endif

const char *Txt_You_can_not_rename_this_folder =
#if   L==1	// ca
	"No podeu canviar el nom d'aquesta carpeta.";
#elif L==2	// de
	"Sie k&ouml;nnen diesen Ordner nicht umbenennen.";
#elif L==3	// en
	"You can not rename this folder.";
#elif L==4	// es
	"No puede cambiar el nombre de esta carpeta.";
#elif L==5	// fr
	"Vous ne pouvez pas renommer ce r&eacute;pertoire.";
#elif L==6	// gn
	"No puede cambiar el nombre de esta carpeta.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rinominare questa cartella.";
#elif L==8	// pl
	"Nie mo&zdot;esz zmieni&cacute; nazwy tego folderu.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode renomear esta pasta.";
#elif L==10	// tr
	"You can not rename this folder.";		// eviri lazim!
#endif

const char *Txt_You_can_not_send_a_message_to_so_many_recipients_ =
#if   L==1	// ca
	"No podeu enviar un missatge a tant&iacute;ssims destinataris."
	" &Eacute;s millor utilitzar els f&ograve;rums.";
#elif L==2	// de
	"Sie k&ouml;nnen eine Nachricht nicht an so viele Empf&auml;nger senden."
	" Benutze besser die Foren.";
#elif L==3	// en
	"You can not send a message to so many recipients."
	" Better use the forums.";
#elif L==4	// es
	"No puede enviar un mensaje a tant&iacute;simos destinatarios."
	" Es mejor usar los foros.";
#elif L==5	// fr
	"Vous ne pouvez pas envoyer un message &agrave; autant de destinataires."
	" Mieux vaut utiliser les forums.";
#elif L==6	// gn
	"No puede enviar un mensaje a tant&iacute;simos destinatarios."
	" Es mejor usar los foros.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi inviare un messaggio a cos tanti destinatari."
	" Meglio usare il forum.";
#elif L==8	// pl
	"Nie mo&zdot;esz wys&lstrok;a&cacute; wiadomo&sacute;ci do tylu odbiorc&oacute;w."
	" Lepiej korzystaj z for&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode enviar uma mensagem para tantos destinat&aacute;rios."
	" Melhor usar os f&oacute;runs.";
#elif L==10	// tr
	"You can not send a message to so many recipients."
	" Better use the forums.";	// eviri lazim!
#endif

const char *Txt_You_can_not_take_a_new_test_until =
#if   L==1	// ca
	"No podr&agrave; fer un test fins";
#elif L==2	// de
	"Sie k&ouml;nnen nicht einen neuen Test bis";
#elif L==3	// en
	"You can not take a new test until";
#elif L==4	// es
	"No podr&aacute; hacer un nuevo test hasta";
#elif L==5	// fr
	"Vous ne pouvez pas passer un nouveau test avant";
#elif L==6	// gn
	"No podr&aacute; hacer un nuevo test hasta";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; fare un nuovo test fino";
#elif L==8	// pl
	"Nie mo&zdot;na wykona&cacute; nowy test do";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode fazer um novo teste at&eacute;";
#elif L==10	// tr
	"You can not take a new test until";		// eviri lazim!
#endif

const char *Txt_You_can_only_perform_X_further_actions_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Nom&eacute;s podeu realitzar %u accions m&eacute;s"
	" si no envieu la vostra foto.";
#elif L==2	// de
	"Sie k&ouml;nnen nur %u weitere Aktionen ausf&uuml;hren,"
	" wenn Sie Ihr Foto nicht senden.";
#elif L==3	// en
	"You can only perform %u further actions"
	" if you don't send your photo.";
#elif L==4	// es
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";
#elif L==5	// fr
	"Vous ne pouvez effectuer que %u actions suppl&eacute;mentaires"
	" si vous n'envoyez pas votre photo.";
#elif L==6	// gn
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Puoi solo realizzare %u massime azioni"
	" se non hai inviato la tua foto.";
#elif L==8	// pl
	"Je&sacute;li nie wy&sacute;lesz swojego zdj&eogon;cia,"
	" mo&zdot;esz wykona&cacute; tylko %u dalszych dzia&lstrok;a&nacute;.";
#elif L==9	// pt
	"Voc&ecirc; s&oacute; pode executar %u a&ccedil;&otilde;es adicionais"
	" se n&atilde;o enviar sua foto.";
#elif L==10	// tr
	"You can only perform %u further actions"
	" if you don't send your photo.";		// eviri lazim!
#endif

const char *Txt_You_can_only_receive_email_notifications_if_ =	// Warning: it is very important to include five %s in the following sentences
#if   L==1	// ca
	"Nom&eacute;s podr&agrave; rebre notificacions per correu"
	" si la seva adre&ccedil;a de correu est&agrave; confirmada (%s &gt; %s)"
	" i acaba en un dels dominis permesos (%s &gt; %s &gt; %s).";
#elif L==2	// de
	"Sie k&ouml;nnen nur E-Mail-Anmeldungen zu erhalten,"
	" wenn Sie Ihre E-Mail-Adresse best&uuml;tigt wird (%s &gt; %s),"
	" und endet in einer der erlaubt Domains (%s &gt; %s &gt; %s).";
#elif L==3	// en
	"You can only receive email notifications"
	" if your email address is confirmed (%s &gt; %s)"
	" and ends in one of the allowed domains (%s &gt; %s &gt; %s).";
#elif L==4	// es
	"Solo podr&aacute; recibir notificaciones por correo"
	" si su direcci&oacute;n de correo est&aacute; confirmada (%s &gt; %s)"
	" y termina en uno de los dominios permitidos (%s &gt; %s &gt; %s).";
#elif L==5	// fr
	"Vous ne pouvez recevoir des notifications par email"
	" si votre adresse email est confirm&eacute; (%s &gt; %s)"
	" et se termine dans l'un des domaines autoris&eacute;s (%s &gt; %s &gt; %s).";
#elif L==6	// gn
	"Solo podr&aacute; recibir notificaciones por correo"
	" si su direcci&oacute;n de correo est&aacute; confirmada (%s &gt; %s)"
	" y termina en uno de los dominios permitidos (%s &gt; %s &gt; %s).";	// Okoteve traduccin
#elif L==7	// it
	"Puoi solo ricevere email di notifica"
	" se la tua email &egrave; confermata (%s &gt; %s)"
	" e si termina in uno dei domini consentiti (%s &gt; %s &gt; %s).";
#elif L==8	// pl
	"Mo&zdot;na otrzyma&cacute; tylko informacj&eogon;,"
	" czy adres email jest potwierdzone (%s &gt; %s),"
	" a ko&nacute;czy si&eogon; w jednej z dozwolone domeny (%s &gt; %s &gt; %s).";
#elif L==9	// pt
	"Voc&ecirc; s&oacute; pode receber notifica&ccedil;&otilde;es de email"
	" se o seu endere&ccedil;o de email est&aacute; confirmado (%s &gt; %s)"
	" e termina em um dos dom&iacute;nios permitidos (%s &gt; %s &gt; %s).";
#elif L==10	// tr
	"You can only receive email notifications"
	" if your email address is confirmed (%s &gt; %s)"
	" and ends in one of the allowed domains (%s &gt; %s &gt; %s).";	// eviri lazim!
#endif

const char *Txt_You_can_register_voluntarily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Podeu apuntar-vos volunt&agrave;riament a un grup de tipus %s.";
#elif L==2	// de
	"Sie k&ouml;nnen sich freiwillig in einer Gruppe vom Typ %s anmelden.";
#elif L==3	// en
	"You can register voluntarily in one group of type %s.";
#elif L==4	// es
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";
#elif L==5	// fr
	"Vous pouvez vous inscrire volontairement dans un groupe de type %s.";
#elif L==6	// gn
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7	// it
	"Ti puoi registrare volontariamente in uno dei gruppi di tipo %s.";
#elif L==8	// pl
	"Mo&zdot;esz zarejestrowa&cacute; si&eogon; dobrowolnie w jednej grupie typu %s.";
#elif L==9	// pt
	"Voc&ecirc; pode se inscrever voluntariamente em um grupo do tipo %s.";
#elif L==10	// tr
	"You can register voluntarily in one group of type %s.";	// eviri lazim!
#endif

const char *Txt_You_can_register_voluntarily_in_one_or_more_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Podeu registrar-vos volunt&agrave;riament"
	" en un o m&eacute;s grups de tipus %s.";
#elif L==2	// de
	"Sie k&ouml;nnen sich freiwillig"
	" in einer oder mehreren Gruppen vom Typ %s registrieren.";
#elif L==3	// en
	"You can register voluntarily"
	" in one or more groups of type %s.";
#elif L==4	// es
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";
#elif L==5	// fr
	"Vous pouvez vous inscrire volontairement"
	" dans un ou plusieurs groupes de type %s.";
#elif L==6	// gn
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";		// Okoteve traduccin
#elif L==7	// it
	"Ti puoi registrare volontariamente"
	" in uno o pi&ugrave; gruppi di tipo %s.";
#elif L==8	// pl
	"Mo&zdot;esz zarejestrowa&cacute; si&eogon; dobrowolnie"
	" w jednej lub kilku grupach typu %s.";
#elif L==9	// pt
	"Voc&ecirc; pode se inscrever voluntariamente"
	" em um ou mais grupos do tipo %s.";
#elif L==10	// tr
	"You can register voluntarily"
	" in one or more groups of type %s.";		// eviri lazim!
#endif

const char *Txt_You_can_search_for_courses_select_them_and_request_your_enrolment_in_them =
#if   L==1	// ca
	"Podeu cercar assignatures, seleccionar-les"
	" i sol&middot;licitar la seva inscripci&oacute; en elles.";
#elif L==2	// de
	"Sie k&ouml;nnen f&uuml;r Kurse zu suchen, w&auml;hlen Sie sie aus"
	" und fordern Sie Ihre Anmeldung in ihnen.";
#elif L==3	// en
	"You can search for courses, select them"
	" and request your enrolment in them.";
#elif L==4	// es
	"Puede buscar asignaturas, seleccionarlas"
	" y solicitar su inscripci&oacute;n en ellas.";
#elif L==5	// fr
	"Vous pouvez rechercher des cours, s&eacute;lectionnez-les"
	" et demander votre inscription en eux.";
#elif L==6	// gn
	"Puede buscar asignaturas, seleccionarlas"
	" y solicitar su inscripci&oacute;n en ellas.";	// Okoteve traduccin"
#elif L==7	// it
	"&Egrave; possibile cercare corsi, selezionarli"
	" e richiedere l'iscrizione in loro.";
#elif L==8	// pl
	"Mo&zdot;esz szuka&cacute; kurs&oacute;w, zaznacz je"
	" i poprosi&cacute; o rejestracj&eogon; w nich.";
#elif L==9	// pt
	"Voc&ecirc; pode procurar por cursos, selecion&aacute;-los"
	" e solicitar a sua inscri&ccedil;&atilde;o nos mesmos.";
#elif L==10	// tr
	"You can search for courses, select them"
	" and request your enrolment in them.";		// eviri lazim!
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_PNG_format_transparent_background_and_size_X_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"Podeu enviar un arxiu amb una imatge en format <em>PNG</em>,"
	" fons transparent i mida %u&times;%u p&iacute;xels.";
#elif L==2	// de
	"Sie k&ouml;nnen eine Datei mit einem Bild im <em>PNG</em>-Format,"
	" transparentem Hintergrund und der Gr&ouml;e %u&times;%u Pixel senden.";
#elif L==3	// en
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";
#elif L==4	// es
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";
#elif L==5	// fr
	"Vous pouvez envoyer un fichier avec une image au format <em>PNG</em>,"
	" fond transparent et taille %u&times;%u pixels.";
#elif L==6	// gn
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";	// Okoteve traduccin
#elif L==7	// it
	"Puoi inviare un file con un'immagine in formato <em>PNG</em>,"
	" sfondo trasparente e di dimensione %u&times;%u pixel.";
#elif L==8	// pl
	"Mo&zdot;esz wys&lstrok;a&cacute; plik z obrazem w formacie <em>PNG</em>,"
	" z przezroczystym t&lstrok;em i rozmiarem %u&times;%u pikseli.";
#elif L==9	// pt
	"Voc&ecirc; pode enviar um arquivo com uma imagem no formato <em>PNG</em>,"
	" fundo transparente e tamanho %u&times;%u pixels.";
#elif L==10	// tr
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// eviri lazim!
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_JPEG_format_ =
#if   L==1	// ca
	"Podeu enviar un arxiu amb una imatge en format <em>JPEG</em>.<br />"
	"La imatge ha de contenir almenys un rostre en posici&oacute; vertical i frontal,"
	" i amb el fons blanc o molt clar.";
#elif L==2	// de
	"Sie k&ouml;nnen eine Datei mit einem Bild im <em>JPEG</em>-Format senden.<br />"
	"Das Bild muss mindestens eine Fl&auml;che in vertikaler und frontaler Position enthalten,"
	" und der Hintergrund muss wei&szlig; oder sehr hell sein.";
#elif L==3	// en
	"You can send a file with an image in <em>JPEG</em> format.<br />"
	"The image must contain at least one face in vertical and frontal position,"
	" and the background must be white or very light.";
#elif L==4	// es
	"Puede enviar un archivo con una imagen en formato <em>JPEG</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n vertical y frontal,"
	" y con el fondo blanco o muy claro.";
#elif L==5	// fr
	"Vous pouvez envoyer un fichier avec une image au format JPEG.<br />"
	"L'image doit contenir au moins un visage en position verticale et frontale,"
	" et le fond doit &ecirc;tre blanc ou tr&egrave;s clair.";
#elif L==6	// gn
	"Puede enviar un archivo con una imagen en formato <em>JPEG</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n vertical y frontal,"
	" y con el fondo blanco o muy claro.";			// Okoteve traduccin
#elif L==7	// it
	"&Egrave; possibile inviare un file in un'immagine con formato <em>JPEG</em>.<br />"
	"L'immagine deve contenere almeno una faccia in posizione verticale e frontale,"
	" e lo sfondo deve essere bianco o molto chiaro.";
#elif L==8	// pl
	"Mo&zdot;esz wys&lstrok;a&cacute; plik obrazu w formacie <em>JPEG</em>.<br />"
	"Obraz musi zawiera&cacute; co najmniej jedn&aogon; twarz w pozycji pionowej jak i czo&lstrok;owego,"
	" a t&lstrok;o musi by&cacute; bia&lstrok;y lub bardzo jasnym.";
#elif L==9	// pt
	"Voc&ecirc; pode enviar um arquivo em uma imagem com formato <em>JPEG</em>.<br />"
	"A imagem deve conter pelo menos um rosto na posi&ccedil;&atilde;o vertical e frontal,"
	" eo fundo deve ser branco ou muito claro.";
#elif L==10	// tr
	"You can send a file with an image in <em>JPEG</em> format.<br />"
	"The image must contain at least one face in vertical and frontal position,"
	" and the background must be white or very light.";	// eviri lazim!
#endif

const char *Txt_You_dont_follow_any_user =
#if   L==1	// ca
	"Vost&egrave; no segueix a cap usuari.";
#elif L==2	// de
	"Sie haben noch keine Benutzer folgen.";
#elif L==3	// en
	"You don't follow any user.";
#elif L==4	// es
	"Usted no sigue a ning&uacute;n usuario.";
#elif L==5	// fr
	"Vous ne suivez pas aucun utilisateur.";
#elif L==6	// gn
	"Usted no sigue a ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Non si seguono gli utenti.";
#elif L==8	// pl
	"Nie po &zdot;adnej u&zdot;ytkownika.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o segue nenhum usu&aacute;rio.";
#elif L==10	// tr
	"You don't follow any user.";			// eviri lazim!
#endif

const char *Txt_You_dont_have_access_to_the_exam =
#if   L==1	// ca
	"No t&eacute; acc&eacute;s a l'examen.";
#elif L==2	// de
	"Sie haben keinen Zugang zur Pr&uuml;fung.";
#elif L==3	// en
	"You don't have access to the exam.";
#elif L==4	// es
	"No tiene acceso al examen.";
#elif L==5	// fr
	"Vous n'avez pas acc&egrave;s  l'examen.";
#elif L==6	// gn
	"No tiene acceso al examen.";		// Okoteve traduccin
#elif L==7	// it
	"Non hai accesso all'esame.";
#elif L==8	// pl
	"Nie masz dost&eogon;pu do egzaminu.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o tem acesso ao exame.";
#elif L==10	// tr
	"You don't have access to the exam.";	// eviri lazim!
#endif

const char *Txt_You_dont_have_permission_to_perform_this_action =
#if   L==1	// ca
	"Vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==2	// de
	"Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==3	// en
	"You don't have permission"
	" to perform this action.";
#elif L==4	// es
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==5	// fr
	"Vous n'avez pas la permission"
	" d'effectuer cette action.";
#elif L==6	// gn
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";		// Okoteve traduccin
#elif L==7	// it
	"Non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==8	// pl
	"You don't have permission"
	" to perform this action.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#elif L==10	// tr
	"You don't have permission"
	" to perform this action.";			// eviri lazim!
#endif

const char *Txt_You_have_confirmed_that_you_have_read_this_information =
#if   L==1	// ca
	"Heu confirmat que heu llegit aquesta informaci&oacute;.";
#elif L==2	// de
	"Sie haben best&auml;tigt, dass Sie diese Informationen gelesen haben.";
#elif L==3	// en
	"You have confirmed that you have read this information.";
#elif L==4	// es
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";
#elif L==5	// fr
	"Vous avez confirm&eacute; avoir lu ces informations.";
#elif L==6	// gn
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Hai confermato che hai letto questa informazione.";
#elif L==8	// pl
	"Potwierdzi&lstrok;e&sacute;, &zdot;e zapozna&lstrok;e&sacute; si&eogon; z tymi informacjami.";
#elif L==9	// pt
	"Voc&ecirc; confirmou que leu esta informa&ccedil;&atilde;o.";
#elif L==10	// tr
	"You have confirmed that you have read this information.";	// eviri lazim!
#endif

const char *Txt_You_have_confirmed_your_enrolment_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Heu confirmat la seva inscripci&oacute;"
	" a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"Sie haben Ihre Einschreibung"
	" in den Kurs <strong>%s</strong> best&auml;tigt.";
#elif L==3	// en
	"You have confirmed your enrolment"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Vous avez confirm&eacute; votre inscription"
	" &agrave; la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Hai confermato la tua registrazione"
	" in questo corso <strong>%s</strong>.";
#elif L==8	// pl
	"Potwierdzi&lstrok;e&sacute; zapis"
	" na kurs <strong>%s</strong>.";
#elif L==9	// pt
	"Voc&ecirc; confirmou sua inscri&ccedil;&atilde;o"
	" na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"You have confirmed your enrolment"
	" in the course <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_You_have_eliminated_the_confirmation_that_you_have_read_this_information =
#if   L==1	// ca
	"Heu eliminat la confirmaci&oacute;"
	" que heu llegit aquesta informaci&oacute;.";
#elif L==2	// de
	"Sie haben die Best&auml;tigung,"
	" dass Sie diese Informationen gelesen haben, gel&ouml;scht.";
#elif L==3	// en
	"You have eliminated the confirmation"
	" that you have read this information.";
#elif L==4	// es
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";
#elif L==5	// fr
	"Vous avez &eacute;limin&eacute; la confirmation"
	" que vous avez lu ces informations.";
#elif L==6	// gn
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Hai rimosso la conferma"
	" che hai letto questa informazione.";
#elif L==8	// pl
	"Usun&aogon;&lstrok;e&sacute; potwierdzenie,"
	" &zdot;e przeczyta&lstrok;e&sacute; te informacje.";
#elif L==9	// pt
	"Voc&ecirc; eliminou a confirma&ccedil;&atilde;o"
	" de que leu esta informa&ccedil;&atilde;o.";
#elif L==10	// tr
	"You have eliminated the confirmation"
	" that you have read this information.";	// eviri lazim!
#endif

const char *Txt_You_have_followed_one_user =
#if   L==1	// ca
	"Ha seguit a un usuari.";
#elif L==2	// de
	"Sie haben einen Benutzer verfolgt.";
#elif L==3	// en
	"You have followed one user.";
#elif L==4	// es
	"Ha seguido a un usuario.";
#elif L==5	// fr
	"Vous avez suivi un utilisateur.";
#elif L==6	// gn
	"Ha seguido a un usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Hai seguito un utente.";
#elif L==8	// pl
	"&Sacute;ledzi&lstrok;e&sacute; jednego u&zdot;ytkownika.";
#elif L==9	// pt
	"Voc&ecirc; seguiu um usu&aacute;rio.";
#elif L==10	// tr
	"You have followed one user.";	// eviri lazim!
#endif

const char *Txt_You_have_followed_X_users =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Ha seguit a %u usuaris.";
#elif L==2	// de
	"Sie haben %u Benutzer verfolgt.";
#elif L==3	// en
	"You have followed %u users.";
#elif L==4	// es
	"Ha seguido a %u usuarios.";
#elif L==5	// fr
	"Vous avez suivi %u utilisateurs.";
#elif L==6	// gn
	"Ha seguido a %u usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Hai seguito %u utenti.";
#elif L==8	// pl
	"&Sacute;ledzi&lstrok;e&sacute; %u u&zdot;ytkownik&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; seguiu %u usu&aacute;rios.";
#elif L==10	// tr
	"You have followed %u users.";	// eviri lazim!
#endif

const char *Txt_You_have_no_notifications =
#if   L==1	// ca
	"No tens notificacions.";
#elif L==2	// de
	"Sie haben keine Benachrichtigungen.";
#elif L==3	// en
	"You have no notifications.";
#elif L==4	// es
	"No tiene notificaciones.";
#elif L==5	// fr
	"Vous n'avez aucune notification.";
#elif L==6	// gn
	"No tiene notificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono notifiche.";
#elif L==8	// pl
	"Nie masz &zdot;adnych powiadomie&nacute;.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o tem notifica&ccedil;&otilde;es.";
#elif L==10	// tr
	"You have no notifications.";	// eviri lazim!
#endif

const char *Txt_You_have_no_unread_notifications =
#if   L==1	// ca
	"No tens notificacions sense llegir.";
#elif L==2	// de
	"Sie haben keine ungelesenen Benachrichtigungen.";
#elif L==3	// en
	"You have no unread notifications.";
#elif L==4	// es
	"No tiene notificaciones sin leer.";
#elif L==5	// fr
	"Vous n'avez aucune notification non lue.";
#elif L==6	// gn
	"No tiene notificaciones sin leer.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono notifiche non lette.";
#elif L==8	// pl
	"Nie masz nieprzeczytanych powiadomie&nacute;.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o tem notifica&ccedil;&otilde;es n&atilde;o lidas.";
#elif L==10	// tr
	"You have no unread notifications.";	// eviri lazim!
#endif

const char *Txt_You_have_not_banned_any_sender =
#if   L==1	// ca
	"No heu prohibit cap remitent.";
#elif L==2	// de
	"Sie haben keinen Absender gesperrt.";
#elif L==3	// en
	"You have not banned any sender.";
#elif L==4	// es
	"No ha bloqueado ning&uacute;n remitente.";
#elif L==5	// fr
	"Vous n'avez banni aucun exp&eacute;diteur.";
#elif L==6	// gn
	"No ha bloqueado ning&uacute;n remitente.";	// Okoteve traduccin
#elif L==7	// it
	"Non hai bannato nessun mittente.";
#elif L==8	// pl
	"Nie zbanowa&lstrok;e&sacute; &zdot;adnego nadawcy.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o baniu nenhum remetente.";
#elif L==10	// tr
	"You have not banned any sender.";		// eviri lazim!
#endif

const char *Txt_You_have_not_confirmed_the_action =
#if   L==1	// ca
	"No ha confirmat l'acci&oacute;.";
#elif L==2	// de
	"Sie haben nicht die Aktion best&auml;tigt.";
#elif L==3	// en
	"You have not confirmed the action.";
#elif L==4	// es
	"No ha confirmado la acci&oacute;n.";
#elif L==5	// fr
	"Vous n'avez pas confirm&eacute; l'action.";
#elif L==6	// gn
	"No ha confirmado la acci&oacute;n.";		// Okoteve traduccin
#elif L==7	// it
	"Non hai inserito correttamente la tua password.";
#elif L==8	// pl
	"Nie potwierdzi&lstrok;y dzia&lstrok;anie.";
#elif L==9	// pt
	"Voc&ecirc; ainda n&atilde;o confirmou a a&ccedil;&atilde;o.";
#elif L==10	// tr
	"You have not confirmed the action.";		// eviri lazim!
#endif

const char *Txt_You_have_not_entered_your_password_correctly =
#if   L==1	// ca
	"No ha introdu&iuml;t correctament la seva contrasenya.";
#elif L==2	// de
	"Sie haben Ihr Passwort nicht korrekt eingegeben wurde.";
#elif L==3	// en
	"You have not entered your password correctly.";
#elif L==4	// es
	"No ha introducido correctamente su contrase&ntilde;a.";
#elif L==5	// fr
	"Vous n'avez pas entr&eacute; votre mot de passe correctement.";
#elif L==6	// gn
	"No ha introducido correctamente su contrase&ntilde;a.";	// Okoteve traduccin
#elif L==7	// it
	"Non hai inserito correttamente la tua password.";
#elif L==8	// pl
	"Nie wpisa&lstrok;e&sacute; poprawnie has&lstrok;o.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o digitou sua senha corretamente.";
#elif L==10	// tr
	"You have not entered your password correctly.";		// eviri lazim!
#endif

const char *Txt_You_have_not_written_twice_the_same_new_password =
#if   L==1	// ca
	"No heu escrit dues vegades la mateixa contrasenya nova.";
#elif L==2	// de
	"Sie haben nicht zweimal das gleiche neue Passwort geschrieben.";
#elif L==3	// en
	"You have not written twice the same new password.";
#elif L==4	// es
	"No ha escrito dos veces la misma contrase&ntilde;a nueva.";
#elif L==5	// fr
	"Vous n'avez pas &eacute;crit deux fois le m&ecirc;me nouveau mot de passe.";
#elif L==6	// gn
	"No ha escrito dos veces la misma contrase&ntilde;a nueva.";	// Okoteve traduccin
#elif L==7	// it
	"Non hai scritto due volte la stessa password nuova.";
#elif L==8	// pl
	"Nie wpisa&lstrok;e&sacute; dwa razy tego samego nowego has&lstrok;a.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o escreveu duas vezes a mesma nova senha.";
#elif L==10	// tr
	"You have not written twice the same new password.";		// eviri lazim!
#endif

const char *Txt_You_have_stopped_following_one_user =
#if   L==1	// ca
	"Ha deixat de seguir a un usuari.";
#elif L==2	// de
	"Sie haben aufgeh&ouml;rt, einem Benutzer zu folgen.";
#elif L==3	// en
	"You have stopped following one user.";
#elif L==4	// es
	"Ha dejado de seguir a un usuario.";
#elif L==5	// fr
	"Vous avez arr&ecirc;t&eacute; de suivre un utilisateur.";
#elif L==6	// gn
	"Ha dejado de seguir a un usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Hai smesso di seguire un utente.";
#elif L==8	// pl
	"Przesta&lstrok;e&sacute; obserwowa&cacute; u&zdot;ytkownika.";
#elif L==9	// pt
	"Voc&ecirc; parou de seguir um usu&aacute;rio.";
#elif L==10	// tr
	"You have stopped following one user.";	// eviri lazim!
#endif

const char *Txt_You_have_stopped_following_X_users =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Ha deixat de seguir a %u usuaris.";
#elif L==2	// de
	"Sie haben aufgeh&ouml;rt, %u Benutzern zu folgen.";
#elif L==3	// en
	"You have stopped following %u users.";
#elif L==4	// es
	"Ha dejado de seguir a %u usuarios.";
#elif L==5	// fr
	"Vous avez cess&eacute; de suivre %u utilisateurs.";
#elif L==6	// gn
	"Ha dejado de seguir a %u usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Hai smesso di seguire %u utenti.";
#elif L==8	// pl
	"Zatrzyma&lstrok;e&sacute; &sacute;ledzenie %u u&zdot;ytkownik&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; parou de seguir %u usu&aacute;rios.";
#elif L==10	// tr
	"You have stopped following %u users.";	// eviri lazim!
#endif

const char *Txt_You_have_to_register_compulsorily_at_least_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Cal registrar-se obligat&ograve;riament almenys en un grup de tipus %s.";
#elif L==2	// de
	"Sie m&uuml;ssen sich mindestens in einer Gruppe vom Typ %s verpflichtend anmelden.";
#elif L==3	// en
	"You have to register compulsorily at least in one group of type %s.";
#elif L==4	// es
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";
#elif L==5	// fr
	"Vous devez obligatoirement vous inscrire au moins dans un groupe de type %s.";
#elif L==6	// gn
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7	// it
	"Devi registrarti obbligatoriamente almeno in uno dei gruppi del tipo %s.";
#elif L==8	// pl
	"Musisz obowi&aogon;zkowo zarejestrowa&cacute; si&eogon; przynajmniej w jednej grupie typu %s.";
#elif L==9	// pt
	"Voc&ecirc; tem que se registrar obrigatoriamente pelo menos em um grupo do tipo %s.";
#elif L==10	// tr
	"You have to register compulsorily at least in one group of type %s.";	// eviri lazim!
#endif

const char *Txt_You_have_to_register_compulsorily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Cal inscriure's obligat&ograve;riament en un grup de tipus %s.";
#elif L==2	// de
	"Sie m&uuml;ssen sich zwingend in einer Gruppe vom Typ %s anmelden.";
#elif L==3	// en
	"You have to register compulsorily in one group of type %s.";
#elif L==4	// es
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";
#elif L==5	// fr
	"Vous devez obligatoirement vous inscrire dans un groupe de type %s.";
#elif L==6	// gn
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7	// it
	"Devi registrarti obbligatoriamente in uno dei gruppi del tipo %s.";
#elif L==8	// pl
	"Musisz obowi&aogon;zkowo zarejestrowa&cacute; si&eogon; w jednej grupie typu %s.";
#elif L==9	// pt
	"Voc&ecirc; tem que se registrar obrigatoriamente em um grupo do tipo %s.";
#elif L==10	// tr
	"You have to register compulsorily in one group of type %s.";	// eviri lazim!
#endif

const char *Txt_You_must_enter_an_integer_value_as_the_correct_answer =
#if   L==1	// ca
	"Heu d'introduir un valor enter com a resposta correcta.";
#elif L==2	// de
	"Als richtige Antwort m&uuml;ssen Sie einen ganzzahligen Wert eingeben.";
#elif L==3	// en
	"You must enter an integer value as the correct answer.";
#elif L==4	// es
	"Debe escribir un valor entero como respuesta correcta.";
#elif L==5	// fr
	"Vous devez entrer une valeur enti&egrave;re comme r&eacute;ponse correcte.";
#elif L==6	// gn
	"Debe escribir un valor entero como respuesta correcta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi inserire un valore intero come risposta corretta.";
#elif L==8	// pl
	"Musisz poda&cacute; warto&sacute;&cacute; ca&lstrok;kowit&aogon; jako poprawn&aogon; odpowied&zacute;.";
#elif L==9	// pt
	"Voc&ecirc; deve inserir um valor inteiro como a resposta correta.";
#elif L==10	// tr
	"You must enter an integer value as the correct answer.";	// eviri lazim!
#endif

const char *Txt_You_must_enter_the_range_of_floating_point_values_allowed_as_answer =
#if   L==1	// ca
	"Heu d'introduir l'interval de valors de coma flotant perm&egrave;s com a resposta.";
#elif L==2	// de
	"Als Antwort m&uuml;ssen Sie den Bereich der zul&auml;ssigen Flie&szlig;kommawerte eingeben.";
#elif L==3	// en
	"You must enter the range of floating point values allowed as answer.";
#elif L==4	// es
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";
#elif L==5	// fr
	"Vous devez entrer la plage de valeurs &agrave; virgule flottante autoris&eacute;e comme r&eacute;ponse.";
#elif L==6	// gn
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi inserire il range di valori reali permessi come risposta.";
#elif L==8	// pl
	"Musisz poda&cacute; zakres warto&sacute;ci zmiennoprzecinkowych dozwolonych jako odpowied&zacute;.";
#elif L==9	// pt
	"Voc&ecirc; deve inserir o intervalo de valores de ponto flutuante permitidos como resposta.";
#elif L==10	// tr
	"You must enter the range of floating point values allowed as answer.";		// eviri lazim!
#endif

const char *Txt_You_must_enter_your_nick_email_or_ID =
#if   L==1	// ca
	"Ha d'escriure el seu @sobrenom, correu o ID (DNI/c&egrave;dula).";
#elif L==2	// de
	"Bitte geben Sie Ihre @Benutzername, Email oder Ausweis-Nr.";
#elif L==3	// en
	"You must enter your @nick, email or ID.";
#elif L==4	// es
	"Debe escribir su @apodo, correo o ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"Vous devez entrer votre @surnom, courrier ou num&eacute;ro d'identit&eacute;.";
#elif L==6	// gn
	"Debe escribir su @apodo, correo o ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7	// it
	"Devi inserire il tuo @nome-utente, email o numero carta d'identit&agrave;.";
#elif L==8	// pl
	"Musisz napisa&cacute; @pseudonim, email lub ID.";
#elif L==9	// pt
	"Voc&ecirc; deve digitar seu @alcunha, email ou n&ordm; identif.";
#elif L==10	// tr
	"You must enter your @nick, email or ID.";			// eviri lazim!
#endif

const char *Txt_You_must_mark_an_answer_as_correct =
#if   L==1	// ca
	"Heu de marcar una resposta com a correcta.";
#elif L==2	// de
	"Sie m&uuml;ssen eine Antwort als richtig markieren.";
#elif L==3	// en
	"You must mark an answer as correct.";
#elif L==4	// es
	"Debe marcar alguna respuesta como correcta.";
#elif L==5	// fr
	"Vous devez marquer une r&eacute;ponse comme correcte.";
#elif L==6	// gn
	"Debe marcar alguna respuesta como correcta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi segnare una risposta come corretta.";
#elif L==8	// pl
	"Musisz oznaczy&cacute; odpowied&zacute; jako poprawn&aogon;.";
#elif L==9	// pt
	"Voc&ecirc; deve marcar uma resposta como correta.";
#elif L==10	// tr
	"You must mark an answer as correct.";		// eviri lazim!
#endif

const char *Txt_You_must_select_a_T_F_answer =
#if   L==1	// ca
	"Heu de seleccionar una resposta V o F.";
#elif L==2	// de
	"Sie m&uuml;ssen eine R/F-Antwort ausw&auml;hlen.";
#elif L==3	// en
	"You must select a T/F answer.";
#elif L==4	// es
	"Debe seleccionar una respuesta V o F.";
#elif L==5	// fr
	"Vous devez slectionner une r&eacute;ponse V/F.";
#elif L==6	// gn
	"Debe seleccionar una respuesta V o F.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare una risposta V/F.";
#elif L==8	// pl
	"Musisz wybra&cacute; odpowied&zacute; P/F.";
#elif L==9	// pt
	"Voc&ecirc; deve selecionar uma resposta V ou F.";
#elif L==10	// tr
	"You must select a T/F answer.";		// eviri lazim!
#endif

const char *Txt_You_must_select_one_ore_more_recipients =
#if   L==1	// ca
	"Heu de seleccionar un o m&eacute;s destinataris.";
#elif L==2	// de
	"Es muss mindestens ein Empf&auml;nger angegeben werden.";
#elif L==3	// en
	"You must select one or more recipients.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s destinatarios.";
#elif L==5	// fr
	"Vous devez s&eacute;lectionner un ou plusieurs destinataires.";
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s destinatarios.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare uno o pi&ugrave; destinatari.";
#elif L==8	// pl
	"Musisz wybra&cacute; jednego lub wi&eogon;cej odbiorc&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; deve selecionar um ou mais destinat&aacute;rios.";
#elif L==10	// tr
	"You must select one or more recipients.";		// eviri lazim!
#endif

const char *Txt_You_must_select_one_ore_more_tags =
#if   L==1	// ca
	"Heu de seleccionar una o m&eacute;s descriptors.";
#elif L==2	// de
	"Es muss mindestens ein Tag ausgew&auml;hlt werden.";
#elif L==3	// en
	"You must select one or more tags.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s descriptores.";
#elif L==5	// fr
	"Vous devez s&eacute;lectionner une ou plusieurs descripteurs.";
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s descriptores.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare una o pi&ugrave; etichette.";
#elif L==8	// pl
	"Musisz wybra&cacute; jeden lub wi&eogon;cej tag&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; deve selecionar uma ou mais descritores.";
#elif L==10	// tr
	"You must select one or more tags.";			// eviri lazim!
#endif

const char *Txt_You_must_select_one_ore_more_types_of_answer =
#if   L==1	// ca
	"Heu de seleccionar un o m&eacute;s tipus de resposta.";
#elif L==2	// de
	"Es muss mindestens ein Antworttyp ausgew&auml;hlt werden.";
#elif L==3	// en
	"You must select one or more types of answer.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";
#elif L==5	// fr
	"Vous devez s&eacute;lectionner un ou plusieurs types de r&eacute;ponse.";
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare uno o pi&ugrave; tipi di risposta.";
#elif L==8	// pl
	"Musisz wybra&cacute; jeden lub wi&eogon;cej typ&oacute;w odpowiedzi.";
#elif L==9	// pt
	"Voc&ecirc; deve selecionar um ou mais tipos de resposta.";
#elif L==10	// tr
	"You must select one or more types of answer.";			// eviri lazim!
#endif

const char *Txt_You_must_select_one_ore_more_users =
#if   L==1	// ca
	"Heu de seleccionar un o m&eacute;s usuaris.";
#elif L==2	// de
	"Es muss mindestens ein Benutzer ausgew&auml;hlt werden.";
#elif L==3	// en
	"You must select one or more users.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s usuarios.";
#elif L==5	// fr
	"Vous devez s&eacute;lectionner un ou plusieurs utilisateurs.";
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare uno o pi&ugrave; utenti.";
#elif L==8	// pl
	"Musisz wybra&cacute; jednego lub wi&eogon;cej u&zdot;ytkownik&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; deve selecionar um ou mais usu&aacute;rios.";
#elif L==10	// tr
	"You must select one or more users.";		// eviri lazim!
#endif

const char *Txt_You_must_send_your_photo_because_ =
#if   L==1	// ca
	"Heu d'enviar la seva foto,"
	" perqu&egrave; has esgotat el nombre de clics"
	" permesos sense foto.";
#elif L==2	// de
	"Sie m&uuml;ssen Ihr Foto senden,"
	" da Sie die Anzahl der zul&auml;ssigen Klicks"
	" ohne Foto ausgesch&ouml;pft haben.";
#elif L==3	// en
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";
#elif L==4	// es
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";
#elif L==5	// fr
	"Vous devez envoyer votre photo,"
	" car vous avez &eacute;puis&eacute; le nombre de clics"
	" autoris&eacute;s sans photo.";
#elif L==6	// gn
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Devi inviare la tua foto,"
	" perch&egrave; hai esaurito il numero di accessi"
	" permessi senza foto.";
#elif L==8	// pl
	"Musisz wys&lstrok;a&cacute; swoje zdj&eogon;cie,"
	" poniewa&zdot; wyczerpa&lstrok;e&sacute; dozwolon&aogon; liczb&eogon; klikni&eogon;&cacute;"
	" bez zdj&eogon;cia.";
#elif L==9	// pt
	"Voc&ecirc; deve enviar sua foto,"
	" porque esgotou o n&uacute;mero de cliques"
	" permitidos sem foto.";
#elif L==10	// tr
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// eviri lazim!
#endif

const char *Txt_You_must_specify_the_image_of_the_new_banner =
#if   L==1	// ca
	"Heu d'especificar la imatge del nou banner.";
#elif L==2	// de
	"Sie m&uuml;ssen das Bild des neuen Banners angeben.";
#elif L==3	// en
	"You must specify the image of the new banner.";
#elif L==4	// es
	"Debe especificar la imagen del nuevo banner.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier l'image de la nouvelle banni&egrave;re.";
#elif L==6	// gn
	"Debe especificar la imagen del nuevo banner.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'image del nuovo banner.";
#elif L==8	// pl
	"Musisz okre&sacute;li&cacute; obraz nowego banera.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar a imagem do novo banner.";
#elif L==10	// tr
	"You must specify the image of the new banner.";	// eviri lazim!
#endif

const char *Txt_You_must_specify_the_logo_the_application_key_the_URL_and_the_IP_address_of_the_new_plugin =
#if   L==1	// ca
	"Heu d'especificar el logotip, la clau de l'aplicaci&oacute;,"
	" l'URL i l'adre&ccedil;a IP del nou complement.";
#elif L==2	// de
	"Sie m&uuml;ssen das Logo, den Anwendungsschl&uuml;ssel,"
	" die URL und die IP-Adresse des neuen Plugins angeben.";
#elif L==3	// en
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";
#elif L==4	// es
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le logo, la cl&eacute; d'application,"
	" l'URL et l'adresse IP du nouveau plugin.";
#elif L==6	// gn
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il logo, il codice di applicazione,"
	" l'URL e l'indirizzo IP del nuovo plugin.";
#elif L==8	// pl
	"Musisz poda&cacute; logo, klucz aplikacji,"
	" adres URL i adres IP nowej wtyczki.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o logotipo, a chave do aplicativo,"
	" o URL e o endere&ccedil;o IP do novo plugin.";
#elif L==10	// tr
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// eviri lazim!
#endif

const char *Txt_You_must_specify_the_numerical_code_of_the_new_country =
#if   L==1	// ca
	"Heu d'especificar el codi num&egrave;ric ISO 3166-1 del nou pa&iacute;s.";
#elif L==2	// de
	"Sie m&uuml;ssen den ISO 3166-1-Zahlencode des neuen Landes angeben.";
#elif L==3	// en
	"You must specify the ISO 3166-1 numerical code of the new country.";
#elif L==4	// es
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le code num&eacute;rique ISO 3166-1 du nouveau pays.";
#elif L==6	// gn
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il codice numerico ISO 3166-1 del nuovo paese.";
#elif L==8	// pl
	"Musisz poda&cacute; kod numeryczny ISO 3166-1 nowego kraju.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o c&oacute;digo num&eacute;rico ISO 3166-1 do novo pa&iacute;s.";
#elif L==10	// tr
	"You must specify the ISO 3166-1 numerical code of the new country.";			// eviri lazim!
#endif

const char *Txt_You_must_specify_the_name =
#if   L==1	// ca
	"Heu d'especificar el nom.";
#elif L==2	// de
	"Sie m&uml;ssen den Namen angeben.";
#elif L==3	// en
	"You must specify the name.";
#elif L==4	// es
	"Debe especificar el nombre.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le nom.";
#elif L==6	// gn
	"Debe especificar el nombre.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome.";
#elif L==8	// pl
	"Musisz poda&cacute; nazw&eogon;.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome.";
#elif L==10	// tr
	"You must specify the name.";	// eviri lazim!
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name =
#if   L==1	// ca
	"Heu d'especificar el nom curt i el nom complet.";
#elif L==2	// de
	"Sie m&uml;ssen den Kurznamen und den vollst&auml;ndigen Namen angeben.";
#elif L==3	// en
	"You must specify the short name and the full name.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le nom abr&eacute;g&eacute; et le nom complet.";
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo.";
#elif L==8	// pl
	"Musisz poda&cacute; kr&oacute;tk&aogon; nazw&eogon; i pe&lstrok;n&aogon; nazw&eogon;.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo.";
#elif L==10	// tr
	"You must specify the short name and the full name.";		// eviri lazim!
#endif

const char *Txt_You_must_specify_the_title =
#if   L==1	// ca
	"Heu d'especificar el t&iacute;tol.";
#elif L==2	// de
	"Sie m&uuml;ssen den Titel angeben.";
#elif L==3	// en
	"You must specify the title.";
#elif L==4	// es
	"Debe especificar el t&iacute;tulo.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le titre.";
#elif L==6	// gn
	"Debe especificar el t&iacute;tulo.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; necessario specificare il titolo.";
#elif L==8	// pl
	"Musisz poda&cacute; tytu&lstrok;.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o t&iacute;tulo.";
#elif L==10	// tr
	"You must specify the title.";		// eviri lazim!
#endif

const char *Txt_You_must_specify_the_web_address =
#if   L==1	// ca
	"Heu d'especificar l'adre&ccedil;a web.";
#elif L==2	// de
	"Sie m&uuml;ssen die Webadresse angeben.";
#elif L==3	// en
	"You must specify the web address.";
#elif L==4	// es
	"Debe especificar la direcci&oacute;n web.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier l'adresse Web.";
#elif L==6	// gn
	"Debe especificar la direcci&oacute;n web.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'indirizzo web.";
#elif L==8	// pl
	"Musisz poda&cacute; adres internetowy.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o endere&ccedil;o web.";
#elif L==10	// tr
	"You must specify the web address.";		// eviri lazim!
#endif

const char *Txt_You_must_type_at_least_one_tag_for_the_question =
#if   L==1	// ca
	"Heu d'escriure almenys un descriptor per a la pregunta.";
#elif L==2	// de
	"Sie m&uuml;ssen mindestens ein Tag f&uuml;r die Frage eingeben.";
#elif L==3	// en
	"You must type at least one tag for the question.";
#elif L==4	// es
	"Debe escribir al menos un descriptor para la pregunta.";
#elif L==5	// fr
	"Vous devez saisir au moins une balise pour la question.";
#elif L==6	// gn
	"Debe escribir al menos un descriptor para la pregunta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere almeno un'etichetta per la domanda.";
#elif L==8	// pl
	"Musisz wpisa&cacute; co najmniej jeden tag dla pytania.";
#elif L==9	// pt
	"Voc&ecirc; deve digitar pelo menos um descritor para a pergunta.";
#elif L==10	// tr
	"You must type at least one tag for the question.";		// eviri lazim!
#endif

const char *Txt_You_must_type_at_least_the_first_answer =
#if   L==1	// ca
	"Heu d'escriure almenys la primera resposta.";
#elif L==2	// de
	"Sie m&uuml;ssen mindestens die erste Antwort eingeben.";
#elif L==3	// en
	"You must type at least the first answer.";
#elif L==4	// es
	"Debe escribir al menos la primera respuesta.";
#elif L==5	// fr
	"Vous devez taper au moins la premi&egrave;re r&eacute;ponse.";
#elif L==6	// gn
	"Debe escribir al menos la primera respuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere almeno la prima risposta.";
#elif L==8	// pl
	"Musisz wpisa&cacute; przynajmniej pierwsz&aogon; odpowied&zacute;.";
#elif L==9	// pt
	"Voc&ecirc; deve digitar pelo menos a primeira resposta.";
#elif L==10	// tr
	"You must type at least the first answer.";	// eviri lazim!
#endif

const char *Txt_You_must_type_at_least_the_first_two_answers =
#if   L==1	// ca
	"Heu d'escriure almenys les dues primeres respostes.";
#elif L==2	// de
	"Sie m&uuml;ssen mindestens die ersten beiden Antworten eingeben.";
#elif L==3	// en
	"You must type at least the first two answers.";
#elif L==4	// es
	"Debe escribir al menos las dos primeras respuestas.";
#elif L==5	// fr
	"Vous devez taper au moins les deux premi&egrave;res r&eacute;ponses.";
#elif L==6	// gn
	"Debe escribir al menos las dos primeras respuestas.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere almeno le prime due risposte.";
#elif L==8	// pl
	"Musisz wpisa&cacute; co najmniej dwie pierwsze odpowiedzi.";
#elif L==9	// pt
	"Voc&ecirc; deve digitar pelo menos as duas primeiras respostas.";
#elif L==10	// tr
	"You must type at least the first two answers.";	// eviri lazim!
#endif

const char *Txt_You_must_type_the_question_stem =
#if   L==1	// ca
	"Heu d'escriure l'enunciat de la pregunta.";
#elif L==2	// de
	"Sie m&uuml;ssen den Fragenstamm eingeben.";
#elif L==3	// en
	"You must type the question stem.";
#elif L==4	// es
	"Debe escribir el enunciado de la pregunta.";
#elif L==5	// fr
	"Vous devez r&eacute;diger l'&eacute;nonc&eacute; de la question.";
#elif L==6	// gn
	"Debe escribir el enunciado de la pregunta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere l'enunciato della domanda.";
#elif L==8	// pl
	"Musisz wpisa&cacute; temat pytania.";
#elif L==9	// pt
	"Voc&ecirc; deve digitar a declara&ccedil;&atilde;o da pergunta.";
#elif L==10	// tr
	"You must type the question stem.";		// eviri lazim!
#endif

const char *Txt_You_need_an_XML_file_containing_a_list_of_questions =
#if   L==1	// ca
	"Necessites un arxiu XML"
	" que contingui una llista de preguntes.";
#elif L==2	// de
	"Sie ben&ouml;tigen eine XML-Datei"
	" mit einem Fragenkatalog.";
#elif L==3	// en
	"You need an XML file"
	" containing a list of questions.";
#elif L==4	// es
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";
#elif L==5	// fr
	"Vous avez besoin d'un fichier XML"
	" contenant une liste de questions.";
#elif L==6	// gn
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; necessario un file XML"
	" contenente un elenco di domande.";
#elif L==8	// pl
	"Potrzebujesz pliku XML"
	" zawieraj&aogon;cego list&eogon; pyta&nacute;.";
#elif L==9	// pt
	"Voc&ecirc; precisa de um arquivo XML"
	" contendo uma lista de perguntas.";
#elif L==10	// tr
	"You need an XML file"
	" containing a list of questions.";		// eviri lazim!
#endif

const char *Txt_You_should_read_the_following_information =
#if   L==1	// ca
	"Vost&egrave; ha de llegir la seg%uuml;ent informaci&oacute;:";
#elif L==2	// de
	"Sie sollten die folgenden Informationen lesen:";
#elif L==3	// en
	"You should read the following information:";
#elif L==4	// es
	"Debe leer la siguiente informaci&oacute;n:";
#elif L==5	// fr
	"Vous devez lire les informations suivantes:";
#elif L==6	// gn
	"Debe leer la siguiente informaci&oacute;n:";	// Okoteve traduccin
#elif L==7	// it
	"Dovresti leggere le seguenti informazioni:";
#elif L==8	// pl
	"Nale&zdot;y przeczyta&cacute; nast&eogon;puj&aogon;ce informacje:";
#elif L==9	// pt
	"Voc&ecirc; deve ler as seguintes informa&ccedil;&otilde;es:";
#elif L==10	// tr
	"You should read the following information:";	// eviri lazim!
#endif

const char *Txt_You_were_already_enroled_as_X_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Vost&egrave; ja estava inscrit/a com a <strong>%s</strong>"
	" a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"Sie waren bereits als <strong>%s</strong>"
	" im Studiengang <strong>%s</strong> eingeschrieben.";
#elif L==3	// en
	"You were already enroled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Vous &eacute;tiez d&eacute;j&agrave; inscrit comme <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Eri gi&agrave; iscritto come <strong>%s</strong>"
	" al corso <strong>%s</strong>.";
#elif L==8	// pl
	"By&lstrok;e&sacute; ju&zdot; zapisany jako <strong>%s</strong>"
	" na kurs <strong>%s</strong>.";
#elif L==9	// pt
	"Voc&ecirc; j&aacute; estava inscrito/a como <strong>%s</strong>"
	" na disciplina <strong>%s</strong>.";
#elif L==10	// tr
	"You were already enroled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// eviri lazim!
#endif

const char *Txt_Your_comment_has_been_updated =
#if   L==1	// ca
	"El seu comentari ha estat actualitzat.";
#elif L==2	// de
	"Ihr Kommentar wurde aktualisiert.";
#elif L==3	// en
	"Your comment has been updated.";
#elif L==4	// es
	"Su comentario ha sido actualizado.";
#elif L==5	// fr
	"Votre commentaire a &eacute;t&eacute; mis &agrave; jour.";
#elif L==6	// gn
	"Su comentario ha sido actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tuo commento &egrave; stato aggiornato.";
#elif L==8	// pl
	"Komentarz zosta&lstrok; zaktualizowany.";
#elif L==9	// pt
	"O seu coment&aacute;rio foi atualizado.";
#elif L==10	// tr
	"Your comment has been updated.";	// eviri lazim!
#endif

const char *Txt_Your_password_is_not_secure_enough =
#if   L==1	// ca
	"La vostra contrasenya no &eacute;s prou segura.";
#elif L==2	// de
	"Ihr Passwort ist nicht sicher genug.";
#elif L==3	// en
	"Your password is not secure enough.";
#elif L==4	// es
	"Su contrase&ntilde;a no es suficientemente segura.";
#elif L==5	// fr
	"Votre mot de passe n'est pas suffisamment s&eacute;curis&eacute;.";
#elif L==6	// gn
	"Su contrase&ntilde;a no es suficientemente segura.";	// Okoteve traduccin
#elif L==7	// it
	"La tua password non &egrave; sicura abbastanza.";
#elif L==8	// pl
	"Twoje has&lstrok;o nie jest wystarczaj&aogon;co bezpieczne.";
#elif L==9	// pt
	"Sua senha n&atilde;o &eacute; segura o suficiente.";
#elif L==10	// tr
	"Your password is not secure enough.";			// eviri lazim!
#endif

const char *Txt_Your_password_must_be_at_least_X_characters_and_can_not_contain_spaces_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La vostra contrasenya ha de tenir almenys %u car&agrave;cters"
	" i no pot contenir espais."
	" &Eacute;s molt important no utilitzar nom&eacute;s"
	" dgits num&egrave;rics, ni un nom o cognom,"
	" ni una paraula coneguda.";
#elif L==2	// de
	"Ihr Passwort muss mindestens %u Zeichen lang sein"
	" und darf keine Leerzeichen enthalten."
	" Es ist sehr wichtig, nicht nur Ziffern oder"
	" einen Vor- oder Nachnamen oder ein bekanntes Wort"
	" zu verwenden.";
#elif L==3	// en
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname,"
	" or a known word.";
#elif L==4	// es
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido,"
	" ni una palabra conocida.";
#elif L==5	// fr
	"Votre mot de passe doit comporter au moins %u caract&egrave;res"
	" et ne peut pas contenir d'espaces."
	" Il est tr&egrave;s important de ne pas utiliser uniquement"
	" des chiffres, ni un nom ou un pr&eacute;nom,"
	" ni un mot connu.";
#elif L==6	// gn
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido,"
	" ni una palabra conocida.";			// Okoteve traduccin
#elif L==7	// it
	"La tua password deve essere almeno di %u caratteri"
	" e non pu&ograve; contenere spazi."
	" E' molto importante non usare solo"
	" tasti numerici, o un nome o cognome,"
	" o una parola conosciuta.";
#elif L==8	// pl
	"Twoje has&lstrok;o musi mie&cacute; co najmniej %u znak&oacute;w"
	" i nie mo&zdot;e zawiera&cacute; spacji. "
	"Bardzo wa&zdot;ne jest, aby nie u&zdot;ywa&cacute;"
	" tylko cyfr, imienia, nazwiska"
	" lub znanego s&lstrok;owa.";
#elif L==9	// pt
	"Sua senha deve ter pelo menos %u caracteres"
	" e n&atilde;o pode conter espa&ccedil;os."
	" &Eacute; muito importante n&atilde;o usar apenas"
	" dgitos num&eacute;ricos, nem um nome ou apelido,"
	" nem uma palavra conhecida.";
#elif L==10	// tr
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname,"
	" or a known word.";				// eviri lazim!
#endif

const char *Txt_Your_personal_data_have_been_updated =
#if   L==1	// ca
	"Les vostres dades personals s'han actualitzat.";
#elif L==2	// de
	"Ihre pers&ouml;nlichen Daten wurden aktualisiert.";
#elif L==3	// en
	"Your personal data have been updated.";
#elif L==4	// es
	"Sus datos personales se han actualizado.";
#elif L==5	// fr
	"Vos donn&eacute;es personnelles ont &eacute;t&eacute; mises &agrave; jour.";
#elif L==6	// gn
	"Sus datos personales se han actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"I tuoi dati personali sono stati aggiornati.";
#elif L==8	// pl
	"Twoje dane osobowe zosta&lstrok;y zaktualizowane.";
#elif L==9	// pt
	"Seus dados pessoais foram atualizados.";
#elif L==10	// tr
	"Your personal data have been updated.";	// eviri lazim!
#endif

const char *Txt_Your_settings_about_notifications_have_changed =
#if   L==1	// ca
	"La seva configuraci&oacute; de les notificacions ha canviat.";
#elif L==2	// de
	"Ihre Einstellungen zu Benachrichtigungen haben sich ge&auml;ndert.";
#elif L==3	// en
	"Your settings about notifications have changed.";
#elif L==4	// es
	"Sus ajustes sobre las notificaciones han cambiado.";
#elif L==5	// fr
	"Vos param&egrave;tres concernant les notifications ont chang&eacute;.";
#elif L==6	// gn
	"Sus ajustes sobre las notificaciones han cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Le tue impostazioni sulle notifiche sono cambiate.";
#elif L==8	// pl
	"Twoje ustawienia dotycz&aogon;ce powiadomie&nacute; uleg&lstrok;y zmianie.";
#elif L==9	// pt
	"Suas configura&ccedil;&otilde;es sobre notifica&ccedil;&otilde;es foram alteradas.";
#elif L==10	// tr
	"Your settings about notifications have changed.";	// eviri lazim!
#endif

const char *Txt_Your_record_card_in_this_course_has_been_updated =
#if   L==1	// ca
	"La seva fitxa a l'assignatura s'ha actualitzat:";
#elif L==2	// de
	"Ihre Karteikarte in diesem Kurs wurde aktualisiert:";
#elif L==3	// en
	"Your record card in this course has been updated:";
#elif L==4	// es
	"Su ficha en la asignatura se ha actualizado:";
#elif L==5	// fr
	"Votre fiche dans ce cours a &eacute;t&eacute; mise &agrave; jour:";
#elif L==6	// gn
	"Su ficha en la asignatura se ha actualizado:";		// Okoteve traduccin
#elif L==7	// it
	"La tua scheda in questo corso &egrave; stata aggiornata:";
#elif L==8	// pl
	"Twoja karta wynik&oacute;w na tym kursie zosta&lstrok;a zaktualizowana:";
#elif L==9	// pt
	"Seu cart&atilde;o nesta disciplina foi atualizado:";
#elif L==10	// tr
	"Your record card in this course has been updated:";	// eviri lazim!
#endif

const char *Txt_Your_request_for_enrolment_as_X_in_the_course_Y_has_been_accepted_for_processing =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La seva petici&oacute; d'inscripci&oacute; com <strong>%s</strong>"
	" en l'assignatura <strong>%s</strong> ha estat realitzada."
	" La petici&oacute; ser&agrave; revisada per un professor o un administrador."
	" Vost&egrave; rebr&agrave; una notificaci&oacute; quan s'accepti la seva inscripci&oacute;.";
#elif L==2	// de
	"Sein Antrag auf Registrierung als <strong>%s</strong>"
	" in Kurs <strong>%s</strong> gemacht wurde."
	" Ihre Anfrage wird durch einen Lehrer oder Administrator &uuml;berpr&uuml;ft werden."
	" Sie werden benachrichtigt, sobald die Anmeldung akzeptiert wird.";
#elif L==3	// en
	"Your request for enrolment as <strong>%s</strong>"
	" in the course <strong>%s</strong> has been accepted for processing."
	" Your request will be reviewed by a teacher or administrator."
	" You will be notified when registration is accepted.";
#elif L==4	// es
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong> ha sido realizada."
	" La petici&oacute;n ser&aacute; revisada por un/a profesor/a o un/a administrador/a."
	" Usted recibir&aacute; una notificaci&oacute;n cuando se acepte su inscripci&oacute;n.";
#elif L==5	// fr
	"Votre demande d'enregistrement en tant que <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong> a &eacute;t&eacute; fait."
	" Votre demande sera examin&eacute;e par un enseignant ou un administrateur."
	" Vous serez avis&eacute; lorsque l'enregistrement est accept&eacute;e.";
#elif L==6	// gn
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong> ha sido realizada."
	" La petici&oacute;n ser&aacute; revisada por un/a profesor/a o un/a administrador/a."
	" Usted recibir&aacute; una notificaci&oacute;n cuando se acepte su inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; stato fatto la sua domanda di registrazione come <strong>%s</strong>"
	" nel corso <strong>%s</strong>."
	" La richiesta verr&agrave; esaminata da un insegnante o un amministratore."
	" Riceverai una notifica quando la registrazione &egrave; stata accettata.";
#elif L==8	// pl
	"Jego wniosek o rejestracj&eogon; jako <strong>%s</strong>"
	" w <strong>%s</strong> przedmiotu zosta&lstrok;a dokonana."
	" Twoja pro&sacute;ba zostanie poddane przegl&aogon;dowi przez nauczyciela lub administratora."
	" Zostaniesz powiadomiony, gdy rejestracja zostanie zaakceptowana.";
#elif L==9	// pt
	"Seu pedido de registo como <strong>%s</strong>"
	" na disciplina <strong>%s</strong> tem sido feito."
	" O pedido ser&aacute; analisado por um professor ou administrador."
	" Voc&ecirc; ser&aacute; notificado quando o registro &eacute; aceito.";
#elif L==10	// tr
	"Your request for enrolment as <strong>%s</strong>"
	" in the course <strong>%s</strong> has been accepted for processing."
	" Your request will be reviewed by a teacher or administrator."
	" You will be notified when registration is accepted.";						// eviri lazim!
#endif

const char *Txt_ZIP_file =
#if   L==1	// ca
	"Fitxer ZIP";
#elif L==2	// de
	"ZIP-Datei";
#elif L==3	// en
	"ZIP file";
#elif L==4	// es
	"Archivo ZIP";
#elif L==5	// fr
	"Fichier ZIP";
#elif L==6	// gn
	"Archivo ZIP";	// Okoteve traduccin
#elif L==7	// it
	"File ZIP";
#elif L==8	// pl
	"Plik ZIP";
#elif L==9	// pt
	"Arquivo ZIP";
#elif L==10	// tr
	"ZIP file";	// eviri lazim!
#endif
